// -*- mode:C++ ; compile-command: "g++ -I.. -g -c alg_ext.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include <cmath>
#include <cstdlib>
#include <stdexcept>
#include <errno.h>
#include "gen.h"
#include "gausspol.h"
#include "identificateur.h"
#include "poly.h"
#include "usual.h"
#include "sym2poly.h"
#include "vecteur.h"
#include "modpoly.h"
#include "alg_ext.h"
#include "vecteur.h"
#include "solve.h"
#include "subst.h"
#include "plot.h"
#include "derive.h"
#include "ezgcd.h"
#include <map>

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  bool islesscomplex(const gen & a,const gen & b){
    if (a==b)
      return false;
    return a.islesscomplexthan(b);
  }
  typedef map<gen,gen,const std::pointer_to_binary_function < const gen &, const gen &, bool> > rootmap;
  rootmap symbolic_rootof_list(ptr_fun(islesscomplex));

  gen algebraic_EXTension(const gen & a,const gen & v){
    if (is_zero(a) )
      return a;
      if (a.type==_VECT){
          if (a._VECTptr->empty())
              return zero;
          if (a._VECTptr->size()==1)
              return a._VECTptr->front();
      }
    gen res;
    res.type=_EXT;
    res.ref_count = new int(1);
    res._EXTptr = new gen[2];
    *(res._EXTptr+1) = v;
    if (a.type==_FRAC){
        *res._EXTptr = a._FRACptr->num;
        return fraction(res,a._FRACptr->den);
    }    
    *res._EXTptr = a;
    return res;
  }

  gen select_root(const vecteur & v){
    vecteur a=proot(v);
    if (a.empty())
      return undef;
    gen current(a.front());
    double max_re(re(current,0).evalf()._DOUBLE_val),max_im(im(current,0).evalf()._DOUBLE_val);
    const_iterateur it=a.begin(),itend=a.end();
    for (;it!=itend;++it){
      double cur_re(re(*it,0).evalf()._DOUBLE_val),cur_im(im(*it,0).evalf()._DOUBLE_val);
      if (cur_re > max_re ){
	current=*it;
	max_re=cur_re;
	max_im=cur_im;
      }
      else { // same argument
	if ( (cur_re == max_re) && (cur_im>max_im) ){
	  current=*it;
	  max_im=cur_im;
	}
      }
    }
    return current;
  }

  gen alg_evalf(const gen & a,const gen &b){
    if (a.type==_FRAC)
      return rdiv(alg_evalf(a._FRACptr->num,b),alg_evalf(a._FRACptr->den,b));
    if (a.type!=_VECT)
      return a.evalf();
    if (b.type!=_VECT)
      return algebraic_EXTension(a,b);
    gen r(select_root(*b._VECTptr));
    if (is_undef(r))
      return algebraic_EXTension(a,b);
    return horner(*a._VECTptr,select_root(*b._VECTptr));
  }

  gen ext_reduce(const gen & a, const gen & v){
    if (a.type==_FRAC)
      return fraction(ext_reduce(a._FRACptr->num,v),ext_reduce(a._FRACptr->den,v));
    if (a.type!=_VECT)
      return a;// algebraic_EXTension(a,v);
    if (a._VECTptr->empty())
      return zero;
    if (a._VECTptr->size()==1)
      return a._VECTptr->front();
    if (v.type==_VECT)
      return algebraic_EXTension((*a._VECTptr) % (*v._VECTptr),v);
    if (v.type==_FRAC)
        return horner(*a._VECTptr,*v._FRACptr);
    if (v.type!=_EXT)
      settypeerr("ext_reduce");
    gen va=*v._EXTptr,vb=*(v._EXTptr+1);
    if (va.type==_FRAC)
        return ext_reduce(horner(*a._VECTptr,*va._FRACptr),vb);
    if (va.type!=_VECT){
      assert(vb.type==_VECT);
      return algebraic_EXTension( (*a._VECTptr) % (*vb._VECTptr),v);
    }
    return ext_reduce(horner(*a._VECTptr,*va._VECTptr),vb);
  }

  gen ext_reduce(const gen & e){
#ifdef DEBUG_SUPPORT
    assert(e.type==_EXT);
#endif
    
    if ( (e._EXTptr->type==_VECT) && ((e._EXTptr+1)->type==_VECT) && 
	 (e._EXTptr->_VECTptr->size()<(e._EXTptr+1)->_VECTptr->size()) )
      return e;
    
    return ext_reduce(*(e._EXTptr),*(e._EXTptr+1));
  }

  void polynome2vecteur(const polynome & p,int na,int nb,vecteur & v){
    v=vecteur(na*nb,zero);
    int i,j;
    assert(p.dim==2);
    vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
    for (;it!=itend;++it){
      i=it->index.iptr->front();
      j=it->index.iptr->back();
      v[nb*(na-i-1)+nb-j-1]=it->value;
    }
  }

  bool is_known_rootof(const vecteur & v,gen & symroot){
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type!=_INT_)
	return false;
    }
    rootmap::iterator ritend=symbolic_rootof_list.end(),rit=symbolic_rootof_list.find(v);
    if (rit!=ritend){
      symroot=rit->second;
      return true;
    }
    if (v.size()==3){
      vecteur w;
      identificateur x(" x");
      in_solve(symb_horner(v,x),x,w,0,0); // FIXME GIAC_CONTEXT
      if (w.empty())
	return false;
      symroot=w.front();
      return true;
    }
    return false;
  }

  // given theta1 and theta2 with minimal poly va and vb
  // find k / Q[theta1+ k*theta2 ] contains theta1 and theta2
  // return the minimal poly of theta=theta1+k*theta2
  // and return in a and b theta1 and theta2 as ext (in terms of theta)
  vecteur common_minimal_POLY(const vecteur & va,const vecteur & vb, gen & a,gen & b,int & k){
    int na=va.size()-1,nb=vb.size()-1;
    if (nb==1){
      k=0;
      vecteur un(2,zero);
      un[0]=plus_one;
      a=algebraic_EXTension(un,va);
      gen tmp=-vb[1];
      if (tmp.type!=_POLY)
	b=tmp;
      else {
	if (tmp._POLYptr->coord.empty())
	  b=zero;
	else
	  b=tmp._POLYptr->coord.front().value;
      }
      return va;
    }
    // create minimal polynomial of theta1/theta2 as 2-d polynomials
    // with main variable respectively a and b
    // (since pb is used for reduction after var reordering of p)
    polynome pa(2),pb(2);
    const_iterateur it=va.begin(),itend=va.end();
    for (int d=na;it!=itend;++it,--d){
      if (!is_zero(*it))
	pa.coord.push_back(monomial<gen>(*it,d,1,2)); // deg=d, var=1, dim=2
    }
    it=vb.begin(),itend=vb.end();
    for (int d=nb;it!=itend;++it,--d){
        if (!is_zero(*it)){
                // if *it is an ext, it must be the same ext as a
            if (it->type==_EXT){
                pb=pb+poly12polynome(*(it->_EXTptr->_VECTptr),1,1).untrunc1(d);
            }
            else
                pb.coord.push_back(monomial<gen>(*it,d,1,2));
        }
    }
    polynome p(2);
    p.coord.push_back(monomial<gen>(1,2));
    // identificateur theta("X");
    matrice m;
    m.reserve(na*nb);
    for (k=1;;++k){
      polynome q(2);
      q.coord.push_back(monomial<gen>(1,1,1,2)); // a: deg=1, var=1, dim=2
      q.coord.push_back(monomial<gen>(k,1,2,2)); // k*b: deg=1, var=2
      // create the matrix
      // lines are 1, a+k*b, ..., (a+k*b)^(na*nb)
      // in terms of (columns)
      // a^(na-1)*b^(nb-1) ... a^(na-1) ... ab^(nb-1) ... ab a b^(nb-1) ... b 1
      m.clear();
      vecteur ligne;
      for (int j=0;j<=na*nb;++j){
	polynome2vecteur(p,na,nb,ligne);
	// ligne.push_back(pow(theta,j));
	m.push_back(ligne);
	p=p*q;
	// permutation of indices order before making division by pb
	p.reorder(transposition(0,1,2));
	p=p%pb;
	p.reorder(transposition(0,1,2));
            // division by a after because b might depend on a
	p=p%pa;
      }
      // Add the line corresponding to b (i.e. theta2)
      ligne=vecteur(na*nb);
      ligne[na*nb-2]=plus_one;
      m.push_back(ligne);
      ligne=vecteur(na*nb);
      ligne[na*nb-nb-1]=plus_one;
      m.push_back(ligne);
      // Transpose matrix
      // then we have the na*nb+2 columns 1, theta, ..., theta^(na*nb), b
      // in terms of a basis (with na*nb coordinates)
      m=mtran(m);
      // reduce the matrix m to echelon form and test rank=na*nb
      // if ok break, else try another value of k
      matrice m_red;
      vecteur pivots;
      gen det;
      mrref(m,m_red,pivots,det,0,na*nb,0,na*nb+3,true);
      m=m_red;
      // the reduced matrix m should have the form
      // * 0      ... 0 * * *
      // 0 *      ... 0 * * *
      //          ...
      // 0 0      ... 0 * * *
      // 0 0      ... 0 * * *
      // 0 0      ... ? * * *
      // with ? != 0, we check ?, if it is zero we try another value k
      vecteur v(m[na*nb-1]._VECTptr->begin(),m[na*nb-1]._VECTptr->end()-1);
      if (!is_zero__VECT(v))
	break;
    }
    mdividebypivot(m);
    // add a -1 at the end of column na*nb (C index) to get the min poly
    vecteur v(na*nb+1);
    for (int i=0;i<na*nb;++i)
      v[i]=-m[i][na*nb];
    v[na*nb]=plus_one;
    reverse(v.begin(),v.end());
    // remove denominators
    gen e;
    lcmdeno(v,e);
    // use column na*nb+1 to find theta2 in terms of theta
    vecteur w(na*nb);
    for (int i=0;i<na*nb;++i)
      w[i]=m[i][na*nb+1];
    reverse(w.begin(),w.end());
    w=trim(w,0);
    lcmdeno(w,e);
    b=fraction(w,e);
    // to get a use column na*nb+2
    for (int i=0;i<na*nb;++i)
      w[i]=m[i][na*nb+2];
    reverse(w.begin(),w.end());
    w=trim(w,0);
    lcmdeno(w,e);
    a=fraction(w,e);    
    // convert to algebraic extensions
    b=algebraic_EXTension(b,v);
    a=algebraic_EXTension(a,v);
    // add v to the rootof_list
    rootmap::iterator ritend=symbolic_rootof_list.end(),rit=symbolic_rootof_list.find(v);
    if (rit==ritend){
      // should first check that va/vb are solvable poly
      gen gaa,gbb;
      if (is_known_rootof(va,gaa) && is_known_rootof(vb,gbb))
	symbolic_rootof_list[v]=gaa +k*gbb;
    }
    return v;
  }

      // assuming a is the extptr+1 of an ext, return the min pol of
      // theta generating the algebraic extension
  vecteur min_pol(gen & a){
      if (a.type==_VECT)
          return *a._VECTptr;
      else {    
          assert( (a.type==_EXT) && ((a._EXTptr+1)->type==_VECT) );
          return *((a._EXTptr+1)->_VECTptr);
      }
  }

  // Find an evaluation point for p at b where pb=p[b] is squarefree
  bool find_good_eval(const polynome & F,polynome & Fb,vecteur & b){
    int Fdeg=F.lexsorted_degree(),nvars=b.size();
    gen Fg;
    int essai=0;
    for (;;++essai){
      Fb=peval_1(F,b,0);
      if (Fb.lexsorted_degree()==Fdeg && gcd(Fb,Fb.derivative()).lexsorted_degree()==0 ){
	return true;
      }
      b=vranm(nvars,0,0); // find another random point
    }
  }

  // a and b are supposed to be *(_EXTptr+1) of some algebraic extension
  // common_EXT will return a new algebraic extension 
  // (suitable to be an extptr+1)
  // and will modify a and b to be ext of the returned common_EXT
  gen common_EXT(gen & a,gen & b){
    if (a==b)
      return a;
    // extract minimal polynomials
    gen a_orig(a),b_orig(b);
    vecteur a__VECT,b__VECT;
    if (a.type==_VECT)
      a__VECT=*a._VECTptr;
    else {
      assert( (a.type==_EXT) && ((a._EXTptr+1)->type==_VECT) );
      a__VECT=*((a._EXTptr+1)->_VECTptr);
    }
    if (b.type==_VECT)
      b__VECT=*b._VECTptr;
    else {
      assert( (b.type==_EXT) && ((b._EXTptr+1)->type==_VECT) );
      b__VECT=*((b._EXTptr+1)->_VECTptr);
    }
    if (b__VECT.size()>a__VECT.size())
      return common_EXT(b,a);
    // reduce extension degree by factorizing b__VECT over Q[a]
    polynome p(poly12polynome(b__VECT));
    polynome p_content(p.dim);
    factorization f;
    gen an;
    ext_factor(p,algebraic_EXTension(a__VECT,a__VECT),an,p_content,f);
    // now choose in the factorization which factor is relevant for b
    // this is done by approximation if possible
    // or by choosing the factor of lowest degree
    // this way we update b__VECT
    int min_deg=b__VECT.size();
    factorization::const_iterator f_it=f.begin(),f_itend=f.end();
    bool trouve=false;
    if (f_itend-f_it==1)
      trouve=true;
    vecteur racines;
    vector<double> real_racines;
    int innerdim=0;
    const_iterateur b_it=b__VECT.begin(),b_itend=b__VECT.end();
    for (;b_it!=b_itend;++b_it){
      if (b_it->type==_POLY)
	innerdim=b_it->_POLYptr->dim;
    }
    vecteur vb(innerdim);
    if (!trouve){
      // Change for multivariate polynomials p, added evaluation
      if (innerdim){
	polynome pb(1),px(unsplitmultivarpoly(p,innerdim));
	find_good_eval(px,pb,vb);
	racines=proot(polynome2poly1(pb));
      }
      else
	racines=proot(b__VECT); 

      // racines= list of approx roots if b__VECT is numeric
      // empty if not numeric
      // remove complex roots
      const_iterateur rit=racines.begin(),ritend=racines.end();
      for (;rit!=ritend;++rit){
	if (rit->type==_DOUBLE_)
	  real_racines.push_back(rit->_DOUBLE_val);
      }
    }
    if (!trouve && !real_racines.empty()){ // choose largest real root by approximation
      double racine_max=*real_racines.begin();
      vector<double>::const_iterator dit=real_racines.begin(),ditend=real_racines.end();
      for (;dit!=ditend;++dit){
	if (*dit>racine_max)
	  racine_max=*dit;
      }
      // now eval each factor over racine_max and choose the one with
      // minimal absolute value
      double min_abs;
      for (;f_it!=f_itend;++f_it){
	vecteur vtmp(polynome2poly1(f_it->fact));
	gen tmp;
	if (innerdim)
	  tmp=r2sym(vtmp,vecteur(1,vb));
	else
	  tmp=r2sym(vtmp,vecteur(1,vecteur(0)));
	tmp=evalf(tmp);
	gen f_racine_max(horner(tmp,racine_max));
	if (f_racine_max.type!=_DOUBLE_)
	  continue;
	double current_evaluation=fabs(f_racine_max._DOUBLE_val);
	if (!trouve){
	  trouve=true;
	  min_abs=current_evaluation;
	  p=f_it->fact;
	}
	else {
	  if (min_abs>current_evaluation){
	    min_abs=current_evaluation;
	    p=f_it->fact;
	  }
	}
      }
    }
    if (!trouve) {
      for (;f_it!=f_itend;++f_it){
	if ( (b.type==_EXT) && is_zero(horner(polynome2poly1(f_it->fact,1),*b._EXTptr)) ){
	  p=f_it->fact;
	  break;
	}
	int d=f_it->fact.lexsorted_degree();
	if (d && (d<=min_deg)){
	  p=f_it->fact;
	  min_deg=d;
	}
      }
    } // end choose by degree
    b__VECT=polynome2poly1(p/p.coord.front().value); // p must be monic (?)
    // _VECTute new minimal polynomial
    int k;    
    vecteur res1=common_minimal_POLY(a__VECT,b__VECT,a,b,k);
    if ((a_orig.type==_EXT) && (b_orig.type==_EXT))
        return algebraic_EXTension(a_orig+gen(k)*b_orig,res1);
    else
        return res1;
  }

  gen ext_add(const gen & aa,const gen & bb){
      gen a(ext_reduce(aa)),b(ext_reduce(bb));
      if ( (a.type!=_EXT) || (b.type!=_EXT) )
	return a+b;
    if (*(a._EXTptr+1)==*(b._EXTptr+1)){
      if ( (a._EXTptr->type==_VECT) && (b._EXTptr->type==_VECT))
	return ext_reduce(*(a._EXTptr->_VECTptr)+ *(b._EXTptr->_VECTptr),*(a._EXTptr+1));
      else
	return ext_reduce(*a._EXTptr+*b._EXTptr,*(a._EXTptr+1));
    }
    gen c=common_EXT(*(a._EXTptr+1),*(b._EXTptr+1));
    // if c.type==_INT_/_ZINT, call ichinrem on a.extptr,b.extptr,...
    return ext_reduce(a)+ext_reduce(b);
  }

  gen ext_sub(const gen & a,const gen & b){
    if (*(a._EXTptr+1)==*(b._EXTptr+1)){
      if ( (a._EXTptr->type==_VECT) && (b._EXTptr->type==_VECT))
	return ext_reduce(*(a._EXTptr->_VECTptr)- *(b._EXTptr->_VECTptr),*(a._EXTptr+1));
      else
	return ext_reduce(*a._EXTptr-*b._EXTptr,*(a._EXTptr+1));
    }
    ext_add(a,-b);
  }

  gen ext_mul(const gen & aa,const gen & bb){
      gen a(ext_reduce(aa)),b(ext_reduce(bb));
      if ( (a.type!=_EXT) || (b.type!=_EXT) )
	return a*b;
    if (*(a._EXTptr+1)==*(b._EXTptr+1)){
      if ((a._EXTptr->type==_VECT) && (b._EXTptr->type==_VECT))
	return ext_reduce( *(a._EXTptr->_VECTptr) * *(b._EXTptr->_VECTptr),*(a._EXTptr+1));
      else
	return ext_reduce((*a._EXTptr)*(*b._EXTptr),*(a._EXTptr+1));
    }
    gen c=common_EXT(*(a._EXTptr+1),*(b._EXTptr+1));
    // if c.type==_INT_/_ZINT, call ichinrem on a._EXTptr,b._EXTptr,...
    return ext_reduce(a)*ext_reduce(b);
  }

  gen inv_EXT(const gen & aa){
    gen a(ext_reduce(aa));
    if (a.type==_FRAC){
      return a._FRACptr->den*inv_EXT(a._FRACptr->num);
    }
    if (a.type!=_EXT)
      return inv(a);
    if (a._EXTptr->type==_VECT){
      vecteur u,v,d;
      egcd(*(a._EXTptr->_VECTptr),*((a._EXTptr+1)->_VECTptr),0,u,v,d);
      if (d.size()!=1)
	setsizeerr();
      gen de=d.front(),du=u;
      simplify(du,de);
      return fraction(algebraic_EXTension(du,*(a._EXTptr+1)),de);
    }
    settypeerr("inv_EXT");
  }

  // rootof has 2 args: P(theta) and Pmin(theta)
  gen symb_rootof(const gen & p,const gen &pmin ){
    if (p.type!=_VECT)
      return p;
    // first check that pmin is in the list of known rootof
    rootmap::iterator it=symbolic_rootof_list.find(pmin),itend=symbolic_rootof_list.end();
    if (it==itend)
      return symbolic(at_rootof,makevecteur(p,pmin));
    return symb_horner(*p._VECTptr,it->second);
  }
  gen rootof(const gen & e){
    if (e.type!=_VECT)
      settypeerr("rootof");
    if (e._VECTptr->size()!=2)
      setdimerr("rootof");
    // should call factor before returning unevaluated rootof
    return symbolic(at_rootof,e);
  }
  gen approx_rootof(const gen & e){
    if ( (e.type!=_VECT) || (e._VECTptr->size()!=2) )
      setsizeerr();
    gen a=e._VECTptr->front(),b=e._VECTptr->back();
    return alg_evalf(a,b);
  }
  gen d1_rootof(const gen & args){
      settypeerr();
      return zero;
  }
  gen d2_rootof(const gen & args){
      settypeerr();
      return zero;
  }
  unary_function_ptr D1_rootof(unary_function_unary(&d1_rootof,""));
  unary_function_ptr D2_rootof(unary_function_unary(&d2_rootof,""));
  unary_function_ptr d_rootof(int i){
    if (i==1)
      return D1_rootof;
    if (i==2)
      return D2_rootof;
    setsizeerr();
  }
  partial_derivative_multiargs D_rootof(&d_rootof);
  const string _rootof_s("rootof");
  unary_function_unary __rootof(&giac::rootof,&D_rootof,_rootof_s);
  unary_function_ptr at_rootof (&__rootof);

  vecteur sturm(const gen & g){
    if (g.type!=_POLY)
      return vecteur(1,g);
    polynome p(*g._POLYptr);
    polynome pl(lgcd(p));
    polynome pp=p/pl;
    polynome cont(p.dim);
    factorization f(sqff(pp));
    factorization::const_iterator it=f.begin(),itend=f.end();
    vecteur v(1,pl);
    for (;it!=itend;++it){
      if (it->mult %2)
	v.push_back(sturm_seq(it->fact,cont));
    }
    return v;
  }
  vecteur sturm(const gen &g,const gen & x){
    if (g.type==_VECT)
      setsizeerr();
    vecteur l;
    if (!is_zero(x))
      l.push_back(x);
    lvar(g,l);
    fraction fa(e2r(g,l));
    gen n,d;
    fxnd(fa,n,d);
    vecteur v=mergevecteur(sturm(n),sturm(d));
    vecteur res,tmp,ll=cdr_VECT(l);
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type==_VECT){
	const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
	vecteur tmpres;
	tmpres.reserve(jtend-jt);
	for (;jt!=jtend;++jt){
	  if (jt->type==_POLY){
	    tmp=polynome2poly1(*(jt->_POLYptr),1);
	    tmpres.push_back(r2e(tmp,ll));
	  }
	  else
	    tmpres.push_back(*jt);
	}
	res.push_back(tmpres);
      }
      else { // it->type != _VECT but we must convert anyway the cst coeff!
	if (it->type==_POLY){
	  gen tmpg=polynome2poly1(*(it->_POLYptr),1).front();
	  res.push_back(r2e(tmpg,ll));
	}
	else
	  res.push_back(*it);
      }
    }
    return res;
  }
  gen _sturm(const gen & g_orig){
    if ((g_orig.type==_VECT) && (g_orig._VECTptr->size()==2) )
      return sturm(g_orig._VECTptr->front(),g_orig._VECTptr->back());
    else
      return sturm(g_orig,zero);
  }
  const string _sturm_s("sturm");
  unary_function_unary __sturm(&giac::_sturm,_sturm_s);
  unary_function_ptr at_sturm (&__sturm);

  // v is a sequence of dense polynomials
  // each poly is evaluated at a, then we count # of sign changes
  // ignoring zeros
  // The function modifies a sign variable according to the sign first
  // non-zero element of v
  int number_of_sign_changes(const vecteur & v,const gen & a,int & global_sign,GIAC_CONTEXT){
    gen w=normal(apply1st(v,a,horner));
    int previous_sign=0,current_sign,res=0;
    const_iterateur it=w._VECTptr->begin(),itend=w._VECTptr->end();
    for (;it!=itend;++it){
      if (is_zero(*it))
	continue;
      if (ck_is_strictly_positive(*it,contextptr))
	current_sign=1;
      else
	current_sign=-1;
      if (!previous_sign) {// assign first non-zero sign
	previous_sign=current_sign;
	global_sign = global_sign *current_sign;
      }
      if (previous_sign==current_sign)
	continue;
      ++res;
      previous_sign=current_sign;
    }
    return res;
  }
  int sturmab(const gen & g,const gen & x,const gen & a,const gen & b,GIAC_CONTEXT){
    if (g.type==_VECT)
      setsizeerr();
    if (ck_is_strictly_greater(a,b,contextptr))
      return sturmab(g,x,b,a,contextptr);
    int res=0,dontcare,global_sign=1;
    vecteur v=sturm(g,x);
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type==_VECT)
	res += number_of_sign_changes(*it->_VECTptr,a,global_sign,contextptr)-number_of_sign_changes(*it->_VECTptr,b,dontcare,contextptr);
      else {
	if (!ck_is_positive(*it,contextptr))
	  global_sign = -global_sign;
      }
    }
    if (res)
      return res;
    return (global_sign-1)/2;
  }
  gen _sturmab(const gen & g_orig,GIAC_CONTEXT){
    if ((g_orig.type!=_VECT) || (g_orig._VECTptr->size()!=4) )
      return symbolic(at_sturmab,g_orig);
    vecteur v(*g_orig._VECTptr);
    return sturmab(v[0],v[1],v[2],v[3],contextptr);
  }
  const string _sturmab_s("sturmab");
  unary_function_eval __sturmab(&giac::_sturmab,_sturmab_s);
  unary_function_ptr at_sturmab (&__sturmab);

  void recompute_minmax(const vecteur & w,const vecteur & range,const gen & expr,const gen & var,gen & resmin,gen & resmax,vecteur & xmin,vecteur & xmax,int direction,GIAC_CONTEXT){
    const_iterateur it=w.begin(),itend=w.end();
    for (;it!=itend;++it){
      if (ck_is_strictly_greater(*it,range[1],contextptr) || ck_is_strictly_greater(range[0],*it,contextptr))
	continue;
      gen tmp=limit(expr,*var._IDNTptr,*it,direction,contextptr);
      if (is_undef(tmp))
	continue;
      if (tmp==resmax)
	xmax.push_back(*it);
      else {
	if (ck_is_strictly_greater(tmp,resmax,contextptr)){
	  resmax=tmp;
	  xmax=vecteur(1,*it);
	}
      }
      if (tmp==resmin)
	xmin.push_back(*it);
      else {
	if (ck_is_strictly_greater(resmin,tmp,contextptr)){
	  resmin=tmp;
	  xmin=vecteur(1,*it);
	}
      }
    }
  }

  // minmax=0 both 1 min, 2 max, /3 =1 if return x instead of f(x)
  gen fminmax(const gen & g,int minmax,GIAC_CONTEXT){
    gen expr,var;
    vecteur v(gen2vecteur(g));
    if (v.size()==1)
      v.push_back(vx_var);
    if (v.size()!=2)
      setsizeerr();
    expr=v[0];
    var=v[1];
    if (expr.type==_SYMB){
      unary_function_ptr & u=expr._SYMBptr->sommet;
      if (u==at_exp || u==at_ln || u==at_atan){
	gen tmp=fminmax(makevecteur(expr._SYMBptr->feuille,var),minmax,contextptr);
	if (u==at_ln && tmp.type==_VECT && tmp._VECTptr->size()==2 && is_positive(-tmp._VECTptr->front(),contextptr) )
	  tmp._VECTptr->front()=zero;
	if (minmax/3)
	  return tmp;
	else
	  return u(tmp,contextptr); 
      }
    }
    if (var.type!=_IDNT)
      setsizeerr();
    gen df(derive(v[0],var));
    vecteur w=solve(df,var,0,contextptr);
    if (w.empty() && debug_infolevel)
      cerr << "Warning: " << df << "=0: no solution found" << endl;
    vecteur range;
    find_range(var,range,contextptr);
    if (range.size()!=1 || range.front().type!=_VECT)
      setsizeerr("Or condition not implemented");
    range=*range.front()._VECTptr;
    gen resmin=plus_inf;
    gen resmax=minus_inf;
    vecteur xmin,xmax;
    // Extrema
    recompute_minmax(w,range,expr,var,resmin,resmax,xmin,xmax,0,contextptr);
    // Limits at begin and end of range
    recompute_minmax(vecteur(1,range[0]),range,expr,var,resmin,resmax,xmin,xmax,1,contextptr);
    recompute_minmax(vecteur(1,range[1]),range,expr,var,resmin,resmax,xmin,xmax,-1,contextptr);
    // Singularities
    w=find_singularities(v[0],*var._IDNTptr,contextptr);
    recompute_minmax(w,range,expr,var,resmin,resmax,xmin,xmax,1,contextptr);
    recompute_minmax(w,range,expr,var,resmin,resmax,xmin,xmax,-1,contextptr);
    if (minmax/3){
      if (minmax %3 ==1)
	return xmin;
      if (minmax %3 ==2)
	return xmax;
      return gen(makevecteur(xmin,xmax),_LINE__VECT);      
    }
    else {
      if (minmax %3 ==1)
	return resmin;
      if (minmax %3 ==2)
	return resmax;
      return gen(makevecteur(resmin,resmax),_LINE__VECT);
    }
  }
  // find extremals values of g
  // should be improved (currently return -1..1 for sin and cos
  void find_range(const gen & g,vecteur & a,GIAC_CONTEXT){
    if (g.type==_IDNT){
      gen g2=g._IDNTptr->eval();
      if ((g2.type==_VECT) && (g2.subtype==_ASSUME__VECT)){
	vecteur v=*g2._VECTptr;
	if ( (v.size()==3) && (v.front()==_DOUBLE_) && (v[1].type==_VECT)){
	  a=*v[1]._VECTptr;
	  return;
	}
      }
    }
    if (g.type==_SYMB){
      try {
	vecteur lv(lvar(g._SYMBptr->feuille));
	if (!lv.empty()){
	  gen res=fminmax(makevecteur(g,lv[0]),0,contextptr);
	  a=vecteur(1,res);
	  return;
	}
      }
      catch (std::runtime_error & e){
      }
      unary_function_ptr s(g._SYMBptr->sommet);
      if ( (s==at_sin) || (s==at_cos) ){
	a=vecteur(1,gen(makevecteur(minus_one,plus_one),_LINE__VECT));
	return;
      }
    }
    a=vecteur(1,gen(makevecteur(minus_inf,plus_inf),_LINE__VECT));
  }

  int sturmsign(const gen & g,GIAC_CONTEXT){
    vecteur v(lvar(g));
    int s=v.size();
    if (!s)
      return 0;
    gen v0(v[0]);
    for (int i=0;i<s;++i){ // replace by first idnt with an assumption
      if (v[i].type==_IDNT && v[i]._IDNTptr->eval().type!=_IDNT){
	v0=v[i];
	break;
      }
    }
    vecteur a;
    find_range(v0,a,contextptr);
    int previous_sign=2,current_sign;
    try {
      const_iterateur ita=a.begin(),itaend=a.end();
      for (;ita!=itaend;++ita){
	if ( (ita->type!=_VECT) || (ita->subtype!=_LINE__VECT) || (ita->_VECTptr->size()!=2) )
	  return 0;
	gen last(ita->_VECTptr->back());
	gen gg(g);
	identificateur idnttmp("t");
	gen testg(subst(g,v0,idnttmp));
	if (is_zero(limit(testg,idnttmp,last,-1,contextptr))){
	  gen tmp=_e2r(makevecteur(gg,v0));
	  gen den=plus_one;
	  if (tmp.type==_FRAC){
	    den=_r2e(makevecteur(tmp._FRACptr->den,v0));
	    if (is_inf(last) && tmp._FRACptr->den.type==_VECT)
	      den=den/pow(v0,2*int(tmp._FRACptr->den._VECTptr->size()/2));
	    tmp=tmp._FRACptr->num;
	    tmp=gen2vecteur(tmp);
	  }
	  if (tmp.type==_VECT){
	    modpoly p(*tmp._VECTptr),q;
	    if (!is_inf(last)) {
	      while (is_zero(horner(p,last,0,q)))
		p=-q;
	    }
	    gg=_r2e(makevecteur(p,v0))/den;
	  }
	}
	current_sign=sturmab(gg,v0,ita->_VECTptr->front(),last,contextptr);
	if (current_sign>0)
	  return 0;
	if (previous_sign==2)
	  previous_sign=current_sign;
	if (previous_sign!=current_sign)
	  return 0;
      }
    }
    catch (std::runtime_error & error){
      return 0;
    }
    return 2*current_sign+1;
  }

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
