// -*- mode:C++ ; compile-command: "g++ -I.. -I../include -g -c cas.cc" -*-
#include "first.h"

/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include "global.h"
#ifdef HAVE_LIBREADLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif // HAVE_LIBREADLINE
using namespace std;
#include <string>
#include <stdexcept>
#include <fstream>
#include <iomanip>
#include <time.h>
#include <sys/time.h>
//#include <unistd.h> // For reading arguments from file
#include <fcntl.h>
#include <cstdlib>
#include "gen.h"
#include "index.h"
#include "sym2poly.h"
#include "derive.h"
#include "intg.h"
#include "tex.h"
#include "lin.h"
#include "solve.h"
#include "modpoly.h"
#include "usual.h"
#include "sym2poly.h"
#include "moyal.h"
#include "ifactor.h"
#include "gauss.h"
#include "isom.h"
#include "plot.h"
#include "prog.h"
#include "rpn.h"
#include "pari.h"
#include "help.h"
#include "plot.h"
#include "input_lexer.h"


// using namespace giac;
#ifdef HAVE_LIBREADLINE
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.  Returns NULL on EOF. */
char *
rl_gets (int count)
{
  /* If the buffer has already been allocated, return the memory
     to the free pool. */
  if (line_read)
    {
      free (line_read);
      line_read = (char *)NULL;
    }
  
  /* Get a line from the user. */
  string prompt(giac::print_INT_(count)+">> ");
  line_read = readline ((char *)prompt.c_str());
  
  /* If the line has any text in it, save it on the history. */
  if (line_read && *line_read)
    add_history (line_read);
  
  return (line_read);
}

#endif // HAVE_LIBREADLINE

#ifdef HAVE_EQASCII
extern "C" {
typedef struct
  {
    int x;			/* width */
    int y;			/* heigth */
    int baseline;		/* default line for single characters
				   counting from down */
  }
Tdim;

struct Tgraph;
struct Tgraph		/* the order of fields is important--see Tarray */
  {
    struct Tgraph **down;	/* downnodes for sequential chilren */
    Tdim dim;			/* dimensions of this field */
    int children;		/* number of children */
    struct Tgraph *up;		/* upnode */
    char *txt;			/* the text. #1 points for a child */
  };

Tdim dim (char *txt, struct Tgraph *graph);
char **draw (struct Tgraph *graph);
char *preparse (char *txt);
void dealloc(struct Tgraph *graph); 
}
#endif // HAVE_EQASCII

#ifdef HAVE_LIBFLTK
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/Enumerations.H>
#include <FL/Fl_Value_Input.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Output.H>
int run_plot_setup=0;
Fl_Window * Xcas = (Fl_Window *) 0;

Fl_Group *Plot_setup_w=(Fl_Group *)0;

Fl_Value_Input *Xmin=(Fl_Value_Input *)0;

Fl_Value_Input *Xmax=(Fl_Value_Input *)0;

Fl_Value_Input *Ymin=(Fl_Value_Input *)0;

Fl_Value_Input *Ymax=(Fl_Value_Input *)0;

Fl_Value_Input *Zmin=(Fl_Value_Input *)0;

Fl_Value_Input *Zmax=(Fl_Value_Input *)0;

Fl_Value_Input *Tmin=(Fl_Value_Input *)0;

Fl_Value_Input *Tmax=(Fl_Value_Input *)0;

Fl_Value_Input *Window_xmin=(Fl_Value_Input *)0;

Fl_Value_Input *Window_xmax=(Fl_Value_Input *)0;

Fl_Value_Input *Window_ymin=(Fl_Value_Input *)0;

Fl_Value_Input *Window_ymax=(Fl_Value_Input *)0;

Fl_Return_Button *Plot_setup_OK=(Fl_Return_Button *)0;

Fl_Button *Plot_setup_cancel=(Fl_Button *)0;

Fl_Check_Button *Axes=(Fl_Check_Button *)0;

Fl_Button *Adjust_window=(Fl_Button *)0;

Fl_Button *Adjust_gnuplot=(Fl_Button *)0;

Fl_Value_Input *Rot_x=(Fl_Value_Input *)0;

Fl_Value_Input *Rot_z=(Fl_Value_Input *)0;

Fl_Button *Rot_x_plus=(Fl_Button *)0;

Fl_Button *Rot_x_minus=(Fl_Button *)0;

Fl_Button *Rot_z_plus=(Fl_Button *)0;

Fl_Button *Rot_z_minus=(Fl_Button *)0;

Fl_Value_Input *X_scale=(Fl_Value_Input *)0;

Fl_Value_Input *Z_scale=(Fl_Value_Input *)0;

Fl_Button *Plot_setup_apply=(Fl_Button *)0;

void gnuplot_setview(){
#ifdef WITH_GNUPLOT
  FILE *stream;
  int r;
  stream = fdopen (giac::run_gnuplot(r), "w");
  fprintf(stream,"%s %f,%f,%f,%f\n%s\n","set view ",Rot_x->value(),Rot_z->value(),X_scale->value(),Z_scale->value(),"replot");
  fflush(stream);
#endif
}

static void cb_Plot_setup_OK(Fl_Return_Button*, void*) {
  run_plot_setup=0;
  giac::gnuplot_xmin=Xmin->value();
  giac::gnuplot_xmax=Xmax->value();
  giac::gnuplot_ymin=Ymin->value();
  giac::gnuplot_ymax=Ymax->value();
  giac::gnuplot_zmin=Zmin->value();
  giac::gnuplot_zmax=Zmax->value();
  giac::gnuplot_tmin=Tmin->value();
  giac::gnuplot_tmax=Tmax->value();
  giac::global_window_xmin=Window_xmin->value();
  giac::global_window_xmax=Window_xmax->value();
  giac::global_window_ymin=Window_ymin->value();
  giac::global_window_ymax=Window_ymax->value();
  Plot_setup_w->hide();
}

static void cb_Plot_setup_cancel(Fl_Button*, void*) {
  run_plot_setup=0;
  Plot_setup_w->hide();
}

static void cb_Axes(Fl_Check_Button*, void*) {

}

static void cb_Adjust_window(Fl_Button*, void*) {
  Window_xmin->value(Xmin->value());
  Window_xmax->value(Xmax->value());
  Window_ymin->value(Ymin->value());
  Window_ymax->value(Ymax->value());
}

static void cb_Adjust_gnuplot(Fl_Button*, void*) {
  Xmin->value(Window_xmin->value());
  Xmax->value(Window_xmax->value());
  Ymin->value(Window_ymin->value());
  Ymax->value(Window_ymax->value());
}

static void cb_Rot_x_plus(Fl_Button*, void*) {
  Rot_x->value(Rot_x->value()+5);gnuplot_setview();
}

static void cb_Rot_x_minus(Fl_Button*, void*) {
  Rot_x->value(Rot_x->value()-5);
  gnuplot_setview();
}

static void cb_Rot_z_plus(Fl_Button*, void*) {
  Rot_z->value(Rot_z->value()+5);
  gnuplot_setview();
}

static void cb_Rot_z_minus(Fl_Button*, void*) {
  Rot_z->value(Rot_z->value()-5);
  gnuplot_setview();
}

static void cb_Plot_setup_apply(Fl_Button*, void*) {
  gnuplot_setview();
}

void set_view_point(int ARGC, char *ARGV[]){
  Window_xmin->value(giac::global_window_xmin);
  Window_xmax->value(giac::global_window_xmax);
  Window_ymin->value(giac::global_window_ymin);
  Window_ymax->value(giac::global_window_ymax);
  Xmin->value(giac::gnuplot_xmin);
  Xmax->value(giac::gnuplot_xmax);
  Ymin->value(giac::gnuplot_ymin);
  Ymax->value(giac::gnuplot_ymax);
  Zmin->value(giac::gnuplot_zmin);
  Zmax->value(giac::gnuplot_zmax);
  Tmin->value(giac::gnuplot_tmin);
  Tmax->value(giac::gnuplot_tmax);
  // cout << "Running controls" << endl;
  Plot_setup_w->show();
  Xcas->show();
  Fl::run();
}  

void redraw_callback(void *) {
  if (!Plot_setup_w->visible())
    Xcas->hide();
}
	  
#endif // HAVE_LIBFLTK

/* TeXmacs interface 
   See /usr/share/TeXmacs examples/plugins and progs for ex. of scheme commands
   Run texmacs -x '(make-session "giac" "default")' to start automatically
   giac at texmacs beginning
 */

#define TEXMACS_DATA_BEGIN   ((char) 2)
#define TEXMACS_DATA_END     ((char) 5)
#define TEXMACS_DATA_ESCAPE  ((char) 27)
#define TEXMACS_DATA_COMMAND ((char) 16)

#define EMACS_ESCAPE   ((char) 8)
#define EMACS_DATA_BEGIN   ((char) 6)
#define EMACS_DATA_END   ((char) 7)
#define EMACS_QUESTION ((char) 1)
#define EMACS_RESULT ((char) 2)
#define EMACS_ERROR ((char) 9)
#define EMACS_INLINE_HELP ((char) 8)
#define EMACS_PROMPT ((char) 13)
#define EMACS_ASK_COMPLETION ((char) 31)
#define EMACS_COMPLETION ((char) 32)
#define EMACS_END_COMPLETION ((char) 33)

static int texmacs_counter= 0;

void format_plugin () {
  // The configuration of a plugin can be completed at startup time.
  // This is for instance interesting if you add tab-completion a posteriori.
  cout << TEXMACS_DATA_BEGIN << "command:";
  cout << "(plugin-configure complete (:tab-completion #t))";
  cout << TEXMACS_DATA_END;
}

void flush_stdout(){
  usleep(2000);
  /*
#ifdef __APPLE__
  usleep(2000);
#else
#ifdef GNUWINCE
  usleep(2000);
#else
  timespec t,tr;
  t.tv_sec=0;
  t.tv_nsec=20000000;
  nanosleep(&t,&tr);
#endif
#endif
  */
  fflush (stdout);
}

void flush_stderr(){
  usleep(2000);
  /*
#ifdef __APPLE__
  usleep(2000);
#else
#ifdef GNUWINCE
  usleep(2000);
#else
  timespec t,tr;
  t.tv_sec=0;
  t.tv_nsec=20000000;
  nanosleep(&t,&tr);
#endif
#endif
  */
  fflush (stderr);
}

void texmacs_next_input () {
  putchar(TEXMACS_DATA_BEGIN);
  printf("prompt#");
  // printf("quest(%i) ",texmacs_counter);
  printf("> ");
  putchar(TEXMACS_DATA_END);
  flush_stdout();
}

void ifstream_output(istream & tmpif){
  flush_stdout();
  // putchar(TEXMACS_DATA_BEGIN);
  // printf("output#");
  putchar(TEXMACS_DATA_BEGIN);
  printf("ps:");
  char c;
  for (int j=1;!tmpif.eof();++j){
    tmpif.get(c);
    putchar(c);
    if (!(j%1024))
      flush_stdout();
  }
  putchar('\n');
  putchar(TEXMACS_DATA_END);
  // putchar(TEXMACS_DATA_END);
  flush_stdout();
}


bool has_graph_output(const giac::gen & g){
  if (g.type==giac::_VECT && !g._VECTptr->empty())
    return has_graph_output(g._VECTptr->back());
  if (g.type!=giac::_SYMB)
    return false;
  if (g._SYMBptr->sommet==giac::at_couleur && g._SYMBptr->feuille.type==giac::_VECT && g._SYMBptr->feuille._VECTptr->size()==2)
    return has_graph_output(g._SYMBptr->feuille._VECTptr->front());
  return g._SYMBptr->sommet==giac::at_graph2tex || equalposcomp(giac::implicittex_plot_sommets,g._SYMBptr->sommet);
}

void texmacs_output(const giac::gen & g,const giac::gen & gg,bool reading_file,int no){
  giac::history_in.push_back(g);
  giac::history_out.push_back(gg);
  bool graph_output=has_graph_output(g);
  if (reading_file){
    /*
    putchar(TEXMACS_DATA_BEGIN);
    printf("prompt#");    
    putchar(TEXMACS_DATA_BEGIN);
    printf("verbatim:quest(%i) ",no);
    putchar(TEXMACS_DATA_END);
    putchar(TEXMACS_DATA_END);

    putchar(TEXMACS_DATA_BEGIN);
    printf("output#");
    putchar(TEXMACS_DATA_BEGIN);
    printf("command:(session-insert-input-below)");
    putchar(TEXMACS_DATA_END);
    putchar(TEXMACS_DATA_END);
    */

    // putchar(TEXMACS_DATA_BEGIN);
    // printf("output#");
    // printf("input#");
    putchar(TEXMACS_DATA_BEGIN);
    printf("verbatim: %s\n",g.print().c_str());
    putchar(TEXMACS_DATA_END);
    // putchar(TEXMACS_DATA_END);
    flush_stdout();
  }
  if (graph_output && (!gg.is_symb_of_sommet(giac::at_pnt) || gg.subtype<0)){
    double horiz_unit=giac::horiz_latex/(giac::gnuplot_xmax-giac::gnuplot_xmin);
    double vert_unit=giac::vert_latex/(giac::gnuplot_ymax-giac::gnuplot_ymin);
    double unit=horiz_unit;
    if (horiz_unit>vert_unit)
      unit=vert_unit;
    int i=giac::history_out.size();
    giac::graph2tex("casgraph.tex",giac::vecteur(giac::history_out.begin()+giac::erase_pos(i),giac::history_out.begin()+i),giac::gnuplot_xmin,giac::gnuplot_xmax,giac::gnuplot_ymin,giac::gnuplot_ymax,unit);
    // putchar(TEXMACS_DATA_BEGIN);
    // printf("output#");
    putchar(TEXMACS_DATA_BEGIN);
    printf("verbatim:");
    /* By default horiz_latex and vert_latex = 12 
       double horiz_unit=horiz_latex/(gnuplot_xmax-gnuplot_xmin);
       double vert_unit=vert_latex/(gnuplot_ymax-gnuplot_ymin);
       double unit=horiz_unit;
       if (horiz_unit>vert_unit)
       unit=vert_unit;
    */
    string dims("18cm,"+giac::gen(18*(giac::gnuplot_ymax-giac::gnuplot_ymin)/(giac::gnuplot_xmax-giac::gnuplot_xmin)).print()+"cm");
    system(("latex casgraph.tex >& casgraph.___ && dvips casgraph -o casgraph.ps -T "+dims+" -O -2cm,-4cm >& casgraph.___ && rm casgraph.___").c_str()) ;
    ifstream tmpif("casgraph.ps");
    ifstream_output(tmpif);
    putchar(TEXMACS_DATA_END);
    // putchar(TEXMACS_DATA_END);
    flush_stdout();
    ofstream log("log");
    log << g << endl;
    return;
  }
  if ( graph_output && giac::has_gnuplot && gg.is_symb_of_sommet(giac::at_pnt) && gg.subtype>=0 ){
    string filename;
    if (reading_file)
      filename="casgr"+giac::print_INT_(gg.val);
    else
      filename="casgraph";
    ofstream of((filename+".tex").c_str());
    of << giac::tex_preamble;
    if (reading_file)
      of << "\\input graph3d"<<giac::print_INT_(gg.val) << ".tex" << endl;
    else
      of << "\\input casgraph.3d" << endl;
    of << giac::tex_end ;
    of.close();
    if (!reading_file)
      giac::_graph3d2tex(giac::string2gen("casgraph.3d",false));
    // putchar(TEXMACS_DATA_BEGIN);
    // printf("output#");
    putchar(TEXMACS_DATA_BEGIN);
    printf("verbatim:");
    system(("latex "+filename+" >& /dev/null && dvips "+filename+" -o "+filename+".ps -O -2cm,-3cm -T 17cm,15cm >& /dev/null").c_str());
    ifstream tmpif((filename+".ps").c_str());
    ifstream_output(tmpif);
    putchar(TEXMACS_DATA_END);
    // putchar(TEXMACS_DATA_END);
    flush_stdout();
    return;
  }
  if (reading_file && gg.is_symb_of_sommet(giac::at_program))
     return; 
  if (g.is_symb_of_sommet(giac::at_nodisp))
    return;
  // putchar(TEXMACS_DATA_BEGIN);
  // printf("output#");
  putchar(TEXMACS_DATA_BEGIN);
  if (gg.is_symb_of_sommet(giac::at_program))
    printf("verbatim: %s\n",gg.print().c_str());
  else {
    if (gg.type==giac::_STRNG)
      printf("verbatim: %s\n",gg._STRNGptr->c_str());
    else 
      printf("latex:$$ %s $$",giac::gen2tex(gg).c_str());
  }
  putchar(TEXMACS_DATA_END);
  // putchar(TEXMACS_DATA_END);
  flush_stdout();
}

void xdvi_view_2d_plot(){
  double horiz_unit=giac::horiz_latex/(giac::gnuplot_xmax-giac::gnuplot_xmin);
  double vert_unit=giac::vert_latex/(giac::gnuplot_ymax-giac::gnuplot_ymin);
  double unit=horiz_unit;
  if (horiz_unit>vert_unit)
    unit=vert_unit;
  int i=giac::history_out.size();
  giac::graph2tex("casgraph.tex",giac::vecteur(giac::history_out.begin()+giac::erase_pos(i),giac::history_out.begin()+i),giac::gnuplot_xmin,giac::gnuplot_xmax,giac::gnuplot_ymin,giac::gnuplot_ymax,unit);
  system("latex casgraph.tex >& casgraph.___ && xdvi casgraph &") ;
}
 
void check_browser_help(const giac::gen & g){
  if (g.is_symb_of_sommet(giac::at_findhelp)){
    giac::gen f=g._SYMBptr->feuille;
    string s;
    if (f.type==giac::_SYMB)
      f=f._SYMBptr->sommet;
    if (f.type==giac::_FUNC)
      s=f._FUNCptr->ptr->s;
    giac::html_vtt=giac::html_help(giac::html_mtt,s);
    if (!giac::html_vtt.empty())
      system(giac::browser_command(giac::html_vtt.front()).c_str());
  }
}

int main(int ARGC, char *ARGV[]){    
  //cerr << giac::remove_filename(ARGV[0]) << endl;
#ifdef HAVE_LIBGSL
    gsl_set_error_handler_off();
#endif
  giac::secure_run=false;
  giac::gnuplot_ymin=-2.4;
  giac::gnuplot_ymax=2.1;
#ifdef GNUWINCE
  if (ARGC<2 || ARGC==3){
    signal(SIGINT,giac::ctrl_c_signal_handler);
    giac::child_id=1;
    // chdir("My Documents");
    if (ARGC==3){
      chdir(ARGV[1]);
      giac::language=atoi(ARGV[2]);
    }
    else
      chdir(giac::remove_filename(ARGV[0]).c_str());
    FILE * stream = fopen("gnuplot.txt","w");
    fputc(' ',stream);
    fflush(stream);
    fclose(stream);
    ifstream in("Cas.txt");
    ofstream out("Out.txt");
    giac::outptr=&out;
    giac::vecteur args,l;
    giac::readargs_from_stream(in,args,l);
    clock_t start, end;  
    int s=args.size();
    for (int i=0;i<s;++i){
      out << "// " << args[i] << ' ' << char(10) << char(13) ;
      start = clock();
      giac::gen tmp=giac::_simplifier(giac::protecteval(args[i],giac::DEFAULT_EVAL_LEVEL,0));
      end = clock();
      out << tmp << ' ' << char(10) << char(13) ;
      // out << "// Time " << double(end-start)/CLOCKS_PER_SEC << char(10) << char(13)  << char(10) << char(13) ;
    }
    return 0;
  }
#endif
  if (ARGC==2 && (string(ARGV[1])=="-v" || string(ARGV[1])=="--version" ) ){
    cout << VERSION << endl;
#ifndef GNUWINCE
    return 0;
#endif
  }
#ifdef HAVE_LIBFLTK
  Fl_Window *w = Xcas = new Fl_Window(422, 381, ("xcas"));
  {
    Fl_Group* o = Plot_setup_w = new Fl_Group(0, 20, 315, 255, ("Plot setup"));
    o->box(FL_SHADOW_BOX);
    o->color(7);
    o->labeltype(FL_NO_LABEL);
    o->labelsize(10);
    { Fl_Value_Input* o = Xmin = new Fl_Value_Input(50, 28, 45, 22, ("X-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-6.5);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Xmax = new Fl_Value_Input(150, 28, 45, 22, ("X+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(6.5);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Ymin = new Fl_Value_Input(50, 55, 45, 20, ("Y-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-3.1);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Ymax = new Fl_Value_Input(150, 55, 45, 20, ("Y+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(3.1);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Zmin = new Fl_Value_Input(50, 80, 45, 20, ("Z-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-10);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Zmax = new Fl_Value_Input(150, 80, 45, 20, ("Z+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(10);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Tmin = new Fl_Value_Input(50, 105, 45, 20, ("t-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-10);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Tmax = new Fl_Value_Input(150, 105, 45, 20, ("t+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-10);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Window_xmin = new Fl_Value_Input(50, 130, 45, 20, ("WX-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-6.5);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Window_xmax = new Fl_Value_Input(150, 130, 45, 20, ("WX+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(6.5);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Window_ymin = new Fl_Value_Input(50, 155, 45, 20, ("WY-"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(-3.1);
    o->textsize(10);
    }
    { Fl_Value_Input* o = Window_ymax = new Fl_Value_Input(150, 155, 45, 20, ("WY+"));
    o->labelsize(10);
    o->minimum(-1e+99);
    o->maximum(1e+99);
    o->value(3.1);
    o->textsize(10);
    }
    { Fl_Return_Button* o = Plot_setup_OK = new Fl_Return_Button(225, 175, 80, 25, ("OK"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Plot_setup_OK);
    }
    { Fl_Button* o = Plot_setup_cancel = new Fl_Button(225, 210, 80, 25, ("Cancel"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Plot_setup_cancel);
    }
    { Fl_Check_Button* o = Axes = new Fl_Check_Button(215, 30, 70, 20, ("axes"));
    o->down_box(FL_DIAMOND_DOWN_BOX);
    o->value(1);
    o->selection_color(2);
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Axes);
    }
    { Fl_Button* o = Adjust_window = new Fl_Button(215, 105, 70, 20, ("->W"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Adjust_window);
    }
    { Fl_Button* o = Adjust_gnuplot = new Fl_Button(215, 130, 70, 20, ("W->"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Adjust_gnuplot);
    }
    { Fl_Value_Input* o = Rot_x = new Fl_Value_Input(55, 180, 45, 20, ("x_rot"));
    o->labelsize(10);
    o->maximum(180);
    o->value(60);
    o->textsize(10);
    o->when(FL_WHEN_ENTER_KEY);
    }
    { Fl_Value_Input* o = Rot_z = new Fl_Value_Input(55, 203, 45, 22, ("z_rot"));
    o->labelsize(10);
    o->maximum(360);
    o->value(30);
    o->textsize(10);
    o->when(FL_WHEN_ENTER_KEY);
    }
    { Fl_Button* o = Rot_x_plus = new Fl_Button(165, 180, 25, 20, ("+"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Rot_x_plus);
    }
    { Fl_Button* o = Rot_x_minus = new Fl_Button(125, 180, 25, 20, ("-"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Rot_x_minus);
    }
    { Fl_Button* o = Rot_z_plus = new Fl_Button(165, 205, 25, 20, ("+"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Rot_z_plus);
    }
    { Fl_Button* o = Rot_z_minus = new Fl_Button(125, 205, 25, 20, ("-"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Rot_z_minus);
    }
    { Fl_Value_Input* o = X_scale = new Fl_Value_Input(55, 230, 45, 20, ("x_scale"));
    o->labelsize(10);
    o->minimum(1e-99);
    o->maximum(1e+99);
    o->value(1);
    o->textsize(10);
    o->when(FL_WHEN_ENTER_KEY);
    }
    { Fl_Value_Input* o = Z_scale = new Fl_Value_Input(55, 255, 45, 20, ("z_scale"));
    o->labelsize(10);
    o->minimum(1e-99);
    o->maximum(1e+99);
    o->value(1);
    o->textsize(10);
    o->when(FL_WHEN_ENTER_KEY);
    }
    { Fl_Button* o = Plot_setup_apply = new Fl_Button(125, 240, 65, 25, ("apply"));
    o->labelsize(10);
    o->callback((Fl_Callback*)cb_Plot_setup_apply);
    }
    o->end();
  }
  w->end();
  Fl::add_idle(redraw_callback,0);
#endif
  signal(SIGINT,giac::ctrl_c_signal_handler);
  //signal(SIGUSR1,data_signal_handler);
  //signal(SIGUSR2,plot_signal_handler);
  giac::child_id=1;
  bool intexmacs=((ARGC>=2) && std::string(ARGV[1])=="--texmacs");
  bool inemacs=((ARGC>=2) && std::string(ARGV[1])=="--emacs");
  if (inemacs){
    putchar(EMACS_DATA_BEGIN);
    putchar(EMACS_ERROR);
  }
  // Config
  bool show_time=false,show_tex=false;
  giac::read_env();
  giac::protected_read_config();
  // Help and completion
  int helpitems;
  vector<giac::aide> aide_v=giac::readhelp("aide_cas",helpitems,false);
  if (!helpitems){
    if (getenv("XCAS_HELP"))
      aide_v=giac::readhelp(getenv("XCAS_HELP"),helpitems);
    else
      aide_v=giac::readhelp((giac::giac_aide_dir()+"aide_cas").c_str(),helpitems);
  }
  if (giac::debug_infolevel){
    if (!helpitems)
      cerr << "Unable to open help file aide_cas" << endl;
    else
      cerr << "Registered " << helpitems << " commands" << endl;
  }
  vector<string> completion_tab;
  for (int k=0;k<aide_v.size();++k){
    if (giac::debug_infolevel)
      cerr << aide_v[k].cmd_name << endl;
    completion_tab.push_back(aide_v[k].cmd_name);
  }

  /* *******
   * EMACS *
   *********/
  // #define EMACS_DEBUG 1
  if (inemacs){
    giac::html_help_init(ARGV[0]);
    int out_handle;
    giac::run_gnuplot(out_handle);
    putchar(EMACS_DATA_END);
    putchar(EMACS_DATA_BEGIN);
    putchar(EMACS_RESULT);
    printf(" Giac CAS for mupacs, released under the GPL license 2.0\n");
    printf("| (c) 2003 B. Parisse & al (giac), F.Maltey & al (mupacs) |\n");
    putchar(EMACS_DATA_END);
    bool prompt=true;
    for (int k=0;;++k) {
      if (prompt){
	putchar(EMACS_DATA_BEGIN);
	putchar(EMACS_PROMPT);
	printf(">> ");
	putchar(EMACS_DATA_END);
      }
      else
	prompt=true;
      string buffer;
      char car,nxt;
#ifdef EMACS_DEBUG
      ofstream logfile(("log"+giac::print_INT_(k)).c_str());
#endif
      for (;;){
	int i=getchar();
	car=i;
#ifdef EMACS_DEBUG
	if (i<32)
	  logfile << "Ctrl-" << i << endl;
	else
	  logfile << car << endl;
#endif
	if (i==EOF)
	  break;
	if (i==EMACS_DATA_END){
	  buffer = buffer + '\n';
	  break;
	}
	buffer=buffer+car;
      }
      // end read buffer
      int s=buffer.size();
      if (s<=2)
	continue;
      if (buffer[0]==10){
	--s;
	buffer=buffer.substr(1,s);
	if (s<=2)
	  continue;
      }
      if (buffer[0]==EMACS_DATA_BEGIN){
	char cmd=buffer[1];
	s -= 2;
	buffer=buffer.substr(2,s);
	if (s && buffer[s-1]=='\n'){
	  --s;
	  buffer=buffer.substr(0,s);
	}
#ifdef EMACS_DEBUG
	logfile << buffer << " " << s << " " << int(buffer[buffer.size()-1]) <<endl;
#endif
	if (cmd==EMACS_ASK_COMPLETION){
	  // reading possible completions from aide_cas
	  vector<string> vres;
	  for (int k=0;k<completion_tab.size();++k){
	    if (completion_tab[k].substr(0,s)==buffer){
	      vres.push_back(completion_tab[k]);
	    }
	  }
	  // add global names
	  giac::gen gv(giac::_VARS(giac::zero,0));
	  if (gv.type==giac::_VECT){
	    giac::vecteur & vv =*gv._VECTptr;
	    for (int k=0;k<vv.size();++k){
	      if (vv[k].print().substr(0,s)==buffer){
		vres.push_back(vv[k].print());
	      }
	    }
	  }
	  string common;
	  int l=s;
	  int vs=vres.size();
#ifdef EMACS_DEBUG
	  logfile << vs << " completions" << endl;
#endif
	  for (int k=0;k<vs;k++){
	    if (common.empty()){
	      common=vres[k];
	      l=common.size();
	    }
	    else {
	      int maxl=min(common.size(),vres[k].size());
#ifdef EMACS_DEBUG
	      logfile << maxl << " " << common << " " << vres[k] << endl;
#endif
	      for (l=s;l<maxl;++l){
		if (common[l]!=vres[k][l])
		  break;
	      }
	      common=common.substr(0,l);
	    }
	  }
	  putchar(EMACS_DATA_BEGIN);
	  putchar(EMACS_COMPLETION);
	  if (vs!=1){
	    if (vs==0){
	      putchar(EMACS_ESCAPE);
	      putchar(EMACS_DATA_END);
	    }
	    for (int k=0;k<vs;k++){
	      printf("%s",vres[k].c_str());
	      if (k!=vs-1)
		printf(", ");
	    }
	  }
	  putchar(EMACS_DATA_END);
	  putchar(EMACS_DATA_BEGIN);
	  putchar(EMACS_END_COMPLETION);
	  // common completion
#ifdef EMACS_DEBUG
	  logfile << common << " " << l << " " << s;
#endif
	  if (l>s)
	    printf(common.substr(s,l-s).c_str());
	  putchar(EMACS_DATA_END);
	  prompt=false;
	}
	if (cmd==EMACS_QUESTION){
	  if (buffer=="quit")
	    break;
	  if (buffer=="?")
	    buffer="?giac";
	  giac::messages_to_print="";
	  giac::gen g(buffer),gg;
	  
	  if (giac::messages_to_print.size()>1){
	    putchar(EMACS_DATA_BEGIN);
	    putchar(EMACS_ERROR);
	    printf("%s\n",giac::messages_to_print.c_str());
	    putchar(EMACS_DATA_END);
	  }
	  check_browser_help(g);
	  putchar(EMACS_DATA_BEGIN);
	  putchar(EMACS_RESULT);
	  // printf("%s\n",giac::messages_to_print.c_str());

	  // GEO SETUP?
	  if ( (g.type==giac::_SYMB) && (g._SYMBptr->sommet==giac::at_xyztrange) && (g._SYMBptr->feuille.type==giac::_VECT) && (g._SYMBptr->feuille._VECTptr->size()<12) ){
#ifdef HAVE_LIBFLTK
	    set_view_point(0,0);
#endif
	    giac::history_in.push_back(g);
	    giac::history_out.push_back(gg);
	    printf("Done\n");
	    putchar(EMACS_DATA_END); // end answer
	    continue;
	  }

	  try {
	    bool reading_file=g.is_symb_of_sommet(giac::at_read);
	    if (reading_file)
	      g=quote_read(g._SYMBptr->feuille);
	    if (giac::approx_mode(0))
	      gg=g.evalf();
	    else
	      gg=g.eval();
	  }
	  catch(std::runtime_error & e){
	    giac::protection_level=0;
	    gg=giac::string2gen(e.what(),false);
	  }
	  giac::history_in.push_back(g);
	  giac::history_out.push_back(gg);
	  // 3-d plot?
#ifdef HAVE_LIBFLTK
	  if (g.type==giac::_SYMB && gg.is_symb_of_sommet(giac::at_pnt) && gg.subtype>=0 && equalposcomp(giac::implicittex_plot_sommets,g._SYMBptr->sommet))
	    set_view_point(0,0);
#endif
	  // 2-d plot?
	  bool graph_output=has_graph_output(g);
	  if (graph_output){
	    xdvi_view_2d_plot();
	    printf("Done\n");
	  }
	  else
	    printf("%s\n",gg.print().c_str());
	  putchar(EMACS_DATA_END);
	}
      } // end normal command
    } // end while(1)
    return 0;
  }
  /* END EMACS */

  /* *********************************************************
     *                       BEGIN TEXMACS                  *
     ********************************************************* */
  if ( intexmacs){
    giac::html_help_init(ARGV[0]);
    int out_handle;
    giac::run_gnuplot(out_handle);
    putchar(TEXMACS_DATA_BEGIN);
    printf("verbatim:");
    format_plugin();
    printf("----------------------------------------------------------------\n");
    printf("|  Giac CAS for TeXmacs, released under the GPL license (2.0)  |\n");
    printf("| (c) 2003 B. Parisse & al (giac), J. van der Hoeven (TeXmacs) |\n");
    printf("----------------------------------------------------------------\n");
    switch (giac::maple_mode){
    case 0:
      printf("C compatibility mode\n");
      break;
    case 1:
      printf("Maple compatibility mode\n");
      break;
    case 2:
      printf("MuPAD compatibility mode\n");
      break;
    case 3:
      printf("TI89/92 compatibility mode\n");
    } 
    printf("Type ? for documentation or ?commandname for help on commandname\n");
    printf("Type tabulation key to complete a partial command\n");
    putchar(TEXMACS_DATA_END);
    texmacs_next_input();
    while (1) {      
      string buffer;
      char car,nxt;
      for (;;){
	int i=getchar();
	// cerr << i << endl;
	if (i==EOF)
	  break;
	car=i;
	if (car=='\n'){
	  giac::set_nonblock_flag(STDIN_FILENO,1); // set non blocking mode on stdin
	  usleep(5000);
	  // ssize_t s=read(STDIN_FILENO,&nxt,1);
	  i=getchar();
	  giac::set_nonblock_flag(STDIN_FILENO,0); // set blocking mode on stdin
	  // cerr << "read "  << s << endl;
	  if (i==EOF)
	    break;
	  buffer += car;
	  car = i;
	  buffer += car;
	}
	else
	  buffer +=car;
      }
      // end read buffer
      if (buffer[0]==TEXMACS_DATA_COMMAND){
	int bs=buffer.size();
	--bs;
	buffer=buffer.substr(1,bs);
	int pos=buffer.find(' ');
	if (pos>0 && pos<bs-1){
	  bs = bs-pos-1;
	  buffer=buffer.substr(pos+1,bs);
	  pos=buffer.find(' ');
	  string complete_string=buffer.substr(1,pos-2);
	  // search non ascii char starting from the end
	  int ss=complete_string.size(),cs=ss;
	  string partial_string;
	  for (int i=ss-1;i>=0;--i){
	    if (isalpha(complete_string[i]) || complete_string[i]=='_' || (complete_string[i]>='0' && complete_string[i]<='9') )
	      ;
	    else
	      break;
	    partial_string=complete_string[i]+partial_string;
	  }
	  ss=partial_string.size();
	  // remove beginning part
	  buffer=buffer.substr(pos+1,bs-pos-2);
	  int n=atoi(buffer.c_str());
	  string cmd("(tuple ");
	  cmd += '"'+complete_string.substr(0,n)+'"' ;
	  n=n-(cs-ss);
	  if (n>=0 && n<=cs){
	    // reading possible completions from aide_cas
	    vector<string> vres;
	    string res=partial_string.substr(0,n);
	    for (int k=0;k<completion_tab.size();++k){
	      if (completion_tab[k].substr(0,n)==res){
		vres.push_back(completion_tab[k]);
	      }
	    }
	    int vs=vres.size();
	    for (int k=0;k<vs;k++){
	      string & tmp=vres[k];
	      cmd += " \""+tmp.substr(n,tmp.size()-n)+'"';
	    }
	    putchar(TEXMACS_DATA_BEGIN);
	    printf("scheme:%s)",cmd.c_str());
	    putchar(TEXMACS_DATA_END);
	    flush_stdout();
	  } 
	  else {
	    putchar(TEXMACS_DATA_BEGIN);
	    printf("scheme:(tuple \"%s\" \"\")",complete_string.c_str());
	    putchar(TEXMACS_DATA_END);
	    flush_stdout();
	  }
	}
	else {
	  // should not happen
	}
	buffer="";
	continue;
      }
      if ( buffer=="quit") 
	break; // end of session
      // Begin answer
      putchar( TEXMACS_DATA_BEGIN); 
      printf("verbatim:");
      // ONLINE HELP
      if (buffer=="?"){
	system(giac::browser_command("doc/index.html").c_str());
	buffer="?giac";
      }
      giac::messages_to_print="";
      giac::gen g(buffer),gg;
      printf("%s\n",giac::messages_to_print.c_str());
      check_browser_help(g);
      // END ONLINE HELP
      // GEO SETUP
      if ( (g.type==giac::_SYMB) && (g._SYMBptr->sommet==giac::at_xyztrange) && (g._SYMBptr->feuille.type==giac::_VECT) && (g._SYMBptr->feuille._VECTptr->size()<12) ){
#ifdef HAVE_LIBFLTK
	set_view_point(0,0);
#endif
	giac::history_in.push_back(g);
	giac::history_out.push_back(gg);
	printf("Done\n");
	putchar(TEXMACS_DATA_BEGIN); 
	printf("latex:");
	printf("x_-=%.4f,x_+=%.4f,y_-=%.4f,y_+=%.4f",giac::gnuplot_xmin,giac::gnuplot_xmax,giac::gnuplot_ymin,giac::gnuplot_ymax);
	putchar(TEXMACS_DATA_END); 
	texmacs_counter++;
	putchar(TEXMACS_DATA_END); // end answer
	texmacs_next_input();
	continue;
      }
      // END GEO SETUP
      bool reading_file=false;
      try {
	reading_file=g.is_symb_of_sommet(giac::at_read);
	if (reading_file)
	  g=quote_read(g._SYMBptr->feuille);
	if (giac::approx_mode(0))
	  gg=g.evalf();
	else
	  gg=g.eval();
      }
      catch(std::runtime_error & e){
	giac::protection_level=0;
	gg=giac::string2gen(e.what(),false);
      }
      if (reading_file && g.type==giac::_VECT && gg.type==giac::_VECT){
	// output each component of the vector gg 1 by 1
	giac::vecteur & gv=*g._VECTptr;
	giac::vecteur & ggv=*gg._VECTptr;
	int s=min(gv.size(),ggv.size());
	for (int j=0;j<s;++j){
	  texmacs_output(gv[j],ggv[j],reading_file,texmacs_counter);
	  texmacs_counter++;
	}
      }
      else {
#ifdef HAVE_LIBFLTK
	if (g.type==giac::_SYMB && gg.is_symb_of_sommet(giac::at_pnt) && gg.subtype>=0 && equalposcomp(giac::implicittex_plot_sommets,g._SYMBptr->sommet))
	  set_view_point(0,0);
#endif
	texmacs_output(g,gg,reading_file,0);
	texmacs_counter++;
      }
      putchar(TEXMACS_DATA_END); // end answer
      texmacs_next_input();
    }
    if (giac::has_gnuplot)
      giac::kill_gnuplot();
    return 0;
  }
  /* *********************************************************
     *                       END OF TEXMACS                  *
     ********************************************************* */
  if (getenv("GIAC_TIME"))
    show_time=true;
  if (getenv("GIAC_TEX")){
    cerr << "// Setting tex log" << endl;
    show_tex=true;
  }
#ifdef HAVE_LIBREADLINE
  if (ARGC==1){
    clock_t start, end;  
    using_history();
    cout << "Welcome to giac readline interface" << endl;
    cout << "(c) 2001,2003 B. Parisse & others" << endl;
    cout << "Homepage http://www-fourier.ujf-grenoble.fr/~parisse/giac.html" << endl;
    cout << "Released under the GPL license 2.0 or above" << endl;
    cout << "See http://www.gnu.org for license details" << endl;
    cout << "-------------------------------------------------" << endl;
    cout << "Press CTRL and D simultaneously to finish session" << endl;
    cout << "Type ?commandname for help" << endl;
    for (int count=0;;++count) {
      char * res=rl_gets(count);
      if (!res)
	break;
      string s(res);
      s += '\n';
      giac::messages_to_print="";
      giac::gen gq(s),ge;
      giac::ctrl_c=false;
      try {
	start = clock();
	if (giac::approx_mode(0))
	  ge=gq.evalf();
	else
	  ge=gq.eval();
	end = clock();
      }
      catch (std::runtime_error & error ){
	cerr << error.what() << endl ;
	giac::protection_level=0;
      }
      giac::history_in.push_back(gq);
      giac::history_out.push_back(ge);
      // 2-d plot?
      bool graph_output=has_graph_output(gq);
      if (graph_output && ge.type!=giac::_INT_){
	xdvi_view_2d_plot();
	cout << "Done" << endl;
      }
      else
	cout << ge << endl;
      cerr << giac::messages_to_print << endl;
      cerr << "// Time " << double(end-start)/CLOCKS_PER_SEC << endl;
#ifdef HAVE_EQASCII
      struct Tgraph *graph=(Tgraph *)malloc(sizeof(struct Tgraph));
      char **screen;
      int i, j;
      graph->up = 0;
      graph->down = 0;
      graph->children = 0;
      string ges(ge.print());
      dim (preparse ((char *)ges.c_str()), graph);
      screen = draw (graph);
      for (i = 0; i < graph->dim.y; i++)
	{
	  for (j = 0; j < graph->dim.x; j++)
	    printf ("%c", screen[i][j]);
	  printf ("\n");
	}
      dealloc(graph); 
#endif // HAVE_EQASCII
    }
    if (giac::has_gnuplot)
      giac::kill_gnuplot();
    return 0;
  }
#endif // HAVE_LIBREADLINE
  int command=0;
  bool showcommand=false;
  if (getenv("GIAC_SHOWCOMMAND"))
    showcommand=true;
  string s(ARGV[0]);
  // keep only what remains after the last / of s
  string t;
  giac::gen u;
  for (int i=s.size()-1;i>=0;--i){
    if (s[i]!='/')
      t=s[i]+t;
    else
      break;
  }
  if (t=="cas2tex"){
    command=-1;
    cout << giac::tex_preamble ;
  }
  else {
    if ( t=="cas" || t=="maplec" || t=="mupadc" || t=="giac" ){
      command=0;
      if (t=="maplec")
	giac::maple_mode=1;
      if (t=="mupadc")
	giac::maple_mode=2;
    }
    else {
      command=1;
      t = "'"+t+"'";
      u=giac::gen(t);
      if (u.type!=giac::_FUNC)
	command=0;
    }
  }
  ofstream texlog("session.tex",ios::app);
  timeval tt; // get time info for label initialization
  gettimeofday(&tt,0);
  giac::gen et((int) tt.tv_sec),e;
  string st(et.print());
  giac::vecteur v;
  giac::vecteur l;
  clock_t start, end,f1,f2;  
  start = clock();
  f2=start;
  //maple_mode=1;
  //rpn_mode=true;
  giac::messages_to_print="";
  if (ARGC==2)
    giac::parse_filename=ARGV[1];
  giac::readargs(ARGC,ARGV,l,v);
  cerr << giac::messages_to_print << endl;
  bool resultat=(giac::messages_to_print=="\n");
  giac::vecteur::const_iterator it=v.begin(),itend=v.end();
  for (int i=0;it!=itend;++it,++i){
    f1=f2;
    giac::history_in.push_back(*it);
    try {
      if (command==-1){
	cout << "\\begin{equation} \\label{eq:d_" << st << "_" << i << "}" << endl;
	cout << giac::gen2tex(*it)  ;
	if (giac::approx_mode(0))
	  e=it->evalf(giac::DEFAULT_EVAL_LEVEL);
	else
	  e = it->eval(giac::DEFAULT_EVAL_LEVEL);
	if ((it->type==giac::_SYMB) && (*it!=e))
	  cout << " = " << giac::gen2tex(e) ;
	cout << " \\end{equation} " << endl;
      }
      else {
	if (command>0){
	  if (giac::approx_mode(0))
	    e = u(it->evalf(),0);
	  else
	    e = u(it->eval(),0);
	}
	else {
	  if (giac::approx_mode(0))
	    e = it->evalf(giac::DEFAULT_EVAL_LEVEL);
	  else
	    e = it->eval(giac::DEFAULT_EVAL_LEVEL);
	}
	// maple_mode=1;
	// 2-d plot?
	bool graph_output=has_graph_output(*it);
	if (showcommand)
	  cout << "// " << *it << endl;
	if (graph_output && e.type!=giac::_INT_){
	  xdvi_view_2d_plot();
	  cout << "Done" << endl;
	}
	else
	  cout << e ;
	if (it+1!=itend){
	  if ( (it->type!=giac::_SYMB) || (it->_SYMBptr->sommet!=giac::at_comment)) 
	    cout << "," <<endl ;
	}
	else
	  cout << endl;
	f2=clock();
	if (show_time)
	  cerr << "// Time " << double(f2-f1)/CLOCKS_PER_SEC << endl;
	if (show_tex) { // append to session.tex
	  texlog << "\\begin{equation} \\label{eq:d_" << st << "_" << i << "}" << endl;
	  texlog << giac::gen2tex(*it)  ;
	  if ((it->type==giac::_SYMB) && (*it!=e))
	    texlog << " = " << giac::gen2tex(e) ;
	  texlog << " \\end{equation} " << endl;
	}
      } // end if (command==-1) else
      /* if (giac::env->moduloon)
	 cerr << "Moduloon" << endl; */
      giac::history_out.push_back(e);      
    }
    catch (std::runtime_error & error ){
      cerr << error.what() << endl ;
      giac::protection_level=0;
    }
  }
  // cerr << messages_to_print << endl;
  // ofstream ans(("ans"+giac::cas_suffixe).c_str());
  // ans << e << endl;
  end = clock();
  if (command==-1){
    cout << giac::tex_end << endl;
    cout << "% Generated by cas2tex in " << double(end-start)/CLOCKS_PER_SEC << "s" << endl;
  }
  else {
    if (show_time)
      cerr << "// Total time" << double(end-start)/CLOCKS_PER_SEC << endl;
  }
  giac::kill_gnuplot();
  return resultat;
}
