/* -*- mode:C++ ; compile-command: "g++ -I.. -g -c desolve.cc" -*- */
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include "desolve.h"
#include "derive.h"
#include "intg.h"
#include "subst.h"
#include "usual.h"
#include "symbolic.h"
#include "unary.h"
#include "poly.h"
#include "sym2poly.h" // for equalposcomp
#include "tex.h"
#include "modpoly.h"
#include "series.h"
#include "solve.h"
#include "ifactor.h"
#include "prog.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  identificateur laplace_var(" s");

  gen integrate_without_lnabs(const gen & e,const gen & x,GIAC_CONTEXT){
    context * newcontextptr=0;
    if (contextptr)
      newcontextptr=new context(*contextptr);
    bool save_do_lnabs=do_lnabs;
    if (newcontextptr && newcontextptr->globalptr)
      newcontextptr->globalptr->do_lnabs=false;
    else
      do_lnabs=false;
    gen res=integrate(e,x,newcontextptr);
    if (newcontextptr)
      delete newcontextptr;
    else
      do_lnabs=save_do_lnabs;
    return res;
  }

  gen laplace(const gen & f,const gen & x,const gen & s,GIAC_CONTEXT){
    if (x.type!=_IDNT)
      setsizeerr();
    gen t(s);
    if (s==x)
      t=identificateur(" t");
    gen remains,res=integrate(f*exp(-t*x),*x._IDNTptr,remains,contextptr);
    res=subst(-res,x,zero);
    if (s==x)
      res=subst(res,t,x);
    if (!is_zero(remains))
      res = res +symbolic(at_integrate,gen(makevecteur(remains,x,0,plus_inf),_SEQ__VECT));
    return res;
  }
    
  // "unary" version
  gen _laplace(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return laplace(args,vx_var,vx_var,contextptr);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s==2)
      return laplace( v[0],v[1],v[1],contextptr);
    if (s!=3)
      setsizeerr();
    return laplace( v[0],v[1],v[2],contextptr);    
  }
  const string _laplace_s("laplace");
  unary_function_eval __laplace(&_laplace,_laplace_s);
  unary_function_ptr at_laplace (&__laplace,0,true);

  gen pf_ilaplace(const gen & e,const identificateur & x, gen & remains,GIAC_CONTEXT){
    vecteur varx(lvarx(e,x));
    int varxs=varx.size();
    if (!varxs){ // should be delta function
      remains=e;
      return zero;
    }
    if ( (varxs>1) || (varx.front()!=x) ) {
      remains = e;
      return zero;
    }
    vecteur l;
    l.push_back(x); // insure x is the main var
    l.push_back(laplace_var); // s var as second var
    l=vecteur(1,l);
    alg_lvar(e,l);
    gen glap=e2r(laplace_var,l);
    assert(glap.type==_POLY);
    int s=l.front()._VECTptr->size();
    if (!s){
      l.erase(l.begin());
      s=l.front()._VECTptr->size();
    }
    gen r=e2r(e,l);
    gen r_num,r_den;
    fxnd(r,r_num,r_den);
    if (r_num.type==_EXT){
      remains=e;
      return zero;
    }
    if (r_den.type!=_POLY){ // should be delat function
      remains=e;
      return zero;
    }
    polynome den(*r_den._POLYptr),num(s);
    if (r_num.type==_POLY)
      num=*r_num._POLYptr;
    else
      num=polynome(r_num,s);
    polynome p_content(lgcd(den));
    factorization vden(sqff(den/p_content)); // first square-free factorization
    vector< pf<gen> > pfde_VECT;
    polynome ipnum(s),ipden(s),temp(s),tmp(s);
    partfrac(num,den,vden,pfde_VECT,ipnum,ipden);
    vector< pf<gen> >::iterator it=pfde_VECT.begin();
    vector< pf<gen> >::const_iterator itend=pfde_VECT.end();
    vector< pf<gen> > rest,finalde_VECT;
    for (;it!=itend;++it){
      pf<gen> single(Tlaplace_reduce_pf<gen>(*it,*glap._POLYptr));
      factor(single.den,p_content,vden,false,withsqrt(contextptr),complex_mode(contextptr));
      partfrac(single.num,single.den,vden,finalde_VECT,temp,tmp);
    }
    it=finalde_VECT.begin();
    itend=finalde_VECT.end();
    gen lnpart(0),deuxaxplusb,sqrtdelta,exppart;
    polynome a(s),b(s),c(s);
    polynome d(s),E(s),lnpartden(s);
    polynome delta(s),atannum(s),alpha(s);
    vecteur lprime(l);
    assert(lprime.front().type==_VECT);
    lprime.front()=cdr_VECT(*(lprime.front()._VECTptr));
    bool uselog;
    for (;it!=itend;++it){
      int deg=it->fact.lexsorted_degree();
      switch (deg) {
      case 1: // 1st order
	findde(it->den,a,b);	
	lnpart=lnpart+rdiv(r2e(it->num,l),r2e(firstcoeff(a),lprime))*exp(r2e(rdiv(-b,a),lprime)*laplace_var);
	break; 
      case 2: // 2nd order
	findabcdelta(it->fact,a,b,c,delta);
	exppart=exp(r2e(rdiv(-b,2*a),lprime)*laplace_var);
	uselog=is_positive(delta);
	alpha=(it->den/it->fact).trunc1()*a;
	findde(it->num,d,E);
	atannum=a*E*gen(2)-b*d;
	// cos part d/alpha*ln(fact)
	lnpartden=alpha;
	simplify(d,lnpartden);
	if (uselog){
	  sqrtdelta=ratnormal(sqrt(r2e(delta,lprime)));
	  gen racine=ratnormal(sqrtdelta/gen(2)/r2e(a,lprime));
	  lnpart=lnpart+rdiv(r2e(d,lprime),r2e(lnpartden,lprime))*cosh(racine*laplace_var)*exppart;
	  gen aa=ratnormal(r2e(atannum,lprime)/r2e(alpha,lprime)/sqrtdelta);
	  lnpart=lnpart+aa*sinh(racine*laplace_var)*exppart;
	}
	else {
	  sqrtdelta=ratnormal(sqrt(r2e(-delta,lprime)));
	  gen racine=ratnormal(sqrtdelta/gen(2)/r2e(a,lprime));
	  lnpart=lnpart+rdiv(r2e(d,lprime),r2e(lnpartden,lprime))*cos(racine*laplace_var,contextptr)*exppart;
	  gen aa=ratnormal(r2e(atannum,lprime)/r2e(alpha,lprime)/sqrtdelta);
	  lnpart=lnpart+aa*sin(racine*laplace_var,contextptr)*exppart;
	}
	break; 
      default:
	rest.push_back(pf<gen>(it->num,it->den,it->fact,1));
	break ;
      }
    }
    remains=rdiv(r2e(ipnum,l),r2e(ipden,l))+r2sym(rest,l);
    return lnpart;
  }
  
  gen ilaplace(const gen & f,const gen & x,const gen & s,GIAC_CONTEXT){
    if (x.type!=_IDNT)
      setsizeerr();
    gen remains,res=linear_apply(f,*x._IDNTptr,remains,contextptr,pf_ilaplace);
    res=subst(res,laplace_var,s);
    if (!is_zero(remains))
      res=res+symbolic(at_ilaplace,makevecteur(remains,x,s));
    return res;
  }
  // "unary" version
  gen _ilaplace(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return ilaplace(args,vx_var,vx_var,contextptr);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s==2)
      return ilaplace( v[0],v[1],v[1],contextptr);
    if (s!=3)
      setsizeerr();
    return ilaplace( v[0],v[1],v[2],contextptr);
  }
  const string _ilaplace_s("ilaplace");
  unary_function_eval __ilaplace(&_ilaplace,_ilaplace_s);
  unary_function_ptr at_ilaplace (&__ilaplace,0,true);

  void unable_to_solve_diffeq(){
    setsizeerr("Unable to solve diff eq");
  }

  gen diffeq_constante(int i){
    string s("c_"+print_INT_(i));
    return gen(s);
  }

  // return -1 if f does not depend on y
  int diffeq_order(const gen & f,const gen & y){
    vecteur ydepend(rlvarx(f,*y._IDNTptr));
    const_iterateur it=ydepend.begin(),itend=ydepend.end();
    // since we did a recursive lvar we dismiss all variables except
    // if they begin with derive
    int n=-1;
    for (;it!=itend;++it){
      if (*it==y)
	n=max(n,0);
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_derive) ){
	gen & g=it->_SYMBptr->feuille;
	int m=-1,nder=1;
	if ( (g.type==_VECT) && (!g._VECTptr->empty()) ){
	  m=diffeq_order(g._VECTptr->front(),y);
	  if (g._VECTptr->size()==3){
	    gen & gg=g._VECTptr->back();
	    if (gg.type==_INT_)
	      nder=gg.val;
	  }
	}
	else 
	  m=diffeq_order(g,y);
	if (m>=0)
	  n=max(n,m+nder);
      }
    }
    return n;
  }

  // true if f is a linear differential equation
  // & returns the coefficient in v in descending order
  // v has size order+2 with last term=cst coeff of the diff equation
  bool is_linear_diffeq(const gen & f_orig,const gen & x,const gen & y,int order,vecteur & v,GIAC_CONTEXT){
    v.clear();
    gen f(f_orig),a,b,cur_y(y);
    identificateur t(" t");
    for (int i=0;i<=order;++i){
      gen ftmp(quotesubst(f,cur_y,t));
      if (!is_linear_wrt(eval(ftmp,contextptr),t,a,b))
	return false;
      if (!rlvarx(a,*y._IDNTptr).empty())
	return false;
      if (!i)
	v.push_back(b);
      v.push_back(a);
      cur_y=symb_derive(y,x,i+1);
    }
    reverse(v.begin(),v.end());
    return true;
  }

  void find_n_derivatives_function(const gen & f,const gen & x,int & nder,gen & fonction){
    if ( (f.type!=_SYMB) || (f._SYMBptr->sommet!=at_derive) ){
      nder=0;
      fonction=f;
      return ;
    }
    if (f._SYMBptr->feuille.type!=_VECT){
      find_n_derivatives_function(f._SYMBptr->feuille,x,nder,fonction);
      ++nder;
      return;
    }
    vecteur & v=*f._SYMBptr->feuille._VECTptr;
    if ( (v.size()>1) && (v[1]!=x) )
      setsizeerr();
    find_n_derivatives_function(v[0],x,nder,fonction);
    if ( (v.size()==3) && (v[2].type==_INT_) )
      nder += v[2].val;
    else
      nder += 1;
  }

  gen function_of(const gen & y_orig,const gen & x_orig){
    if ( (y_orig.type!=_SYMB) || (y_orig._SYMBptr->sommet!=at_of) )
      setsizeerr();
    vecteur & v =*y_orig._SYMBptr->feuille._VECTptr;
    if ( (v[1]!=x_orig) || (v[0].type!=_IDNT) )
      setsizeerr();
    return v[0];
  }

  gen in_desolve_with_conditions(const vecteur & v,const gen & x,const gen & y,const gen & solution_generale,const vecteur & parameters,GIAC_CONTEXT){
    gen yy(y);
    if (yy.type!=_IDNT)
      yy=function_of(y,x);
    // solve boundary conditions
    const_iterateur it=v.begin()+1,itend=v.end();
    vecteur conditions(remove_equal(it,itend));
    if (conditions.empty())
      return solution_generale;
    // conditions must be in terms of y(value) or derivatives
    vecteur condvar(rlvarx(conditions,*yy._IDNTptr));
    vecteur yvar; // will contain triplet (var,n,x) n=nth derivative, x point
    it=condvar.begin(),itend=condvar.end();
    int maxnder=0;
    for (;it!=itend;++it){
      if ( (it->type!=_SYMB) || (it->_SYMBptr->sommet!=at_of) )
	continue;
      vecteur & w=*it->_SYMBptr->feuille._VECTptr;
      int nder;
      gen fonction;
      find_n_derivatives_function(w[0],x,nder,fonction);
      if (fonction==y){
	if ( (w[1].type==_VECT) && (!w[1]._VECTptr->empty()))
	  yvar.push_back(makevecteur(*it,nder,w[1]._VECTptr->front()));
	else
	  yvar.push_back(makevecteur(*it,nder,w[1]));
      }
      if (nder>maxnder)
	maxnder=nder;
    }
    // compute all derivatives of the general solution
    vecteur derivatives(1,solution_generale);
    gen current=solution_generale;
    for (int i=1;i<=maxnder;++i){
      current=derive(current,x);
      derivatives.push_back(current);
    }
    // evaluate at points of yvar making substition vectors
    it=yvar.begin(),itend=yvar.end();
    vecteur substin,substout;
    for (;it!=itend;++it){
      vecteur & w=*it->_VECTptr;
      substin.push_back(w[0]);
      substout.push_back(subst(derivatives[w[1].val],x,w[2]));
    }
    // replace in conditions
    conditions=*eval(subst(conditions,substin,substout),contextptr)._VECTptr;
    // solve system over _c0..._cn-1
    bool save_maple_mode=maple_mode;
    maple_mode=0;
    vecteur parameters_solutions=*_solve(gen(makevecteur(conditions,parameters),_SEQ__VECT),contextptr)._VECTptr;
    maple_mode=save_maple_mode;
    // replace _c0..._cn-1 in solution_generale
    it=parameters_solutions.begin(),itend=parameters_solutions.end();
    vecteur res;
    for (;it!=itend;++it){
      res.push_back(eval(subst(solution_generale,parameters,*it),contextptr));
    }
    return res;
  }
  gen desolve_with_conditions(const vecteur & v,const gen & x,const gen & y,GIAC_CONTEXT){
    if (v.empty())
      setsizeerr();
    int ordre;
    vecteur parameters;
    gen solution_generale(desolve(v.front(),x,y,ordre,parameters,contextptr));
    if (solution_generale.type!=_VECT) 
      return in_desolve_with_conditions(v,x,y,solution_generale,parameters,contextptr);
    const_iterateur it=solution_generale._VECTptr->begin(),itend=solution_generale._VECTptr->end();
    vecteur res;
    res.reserve(itend-it);
    for (;it!=itend;++it){
      if (it->type==_VECT)
	cerr << "Boundary conditions for parametric curve not implemented" << endl;
      res.push_back(in_desolve_with_conditions(v,x,y,*it,parameters,contextptr));
    }
    return res;
  }

  // f must be a vector obtained using factors
  // x, y are 2 idnt
  // xfact and yfact should be initialized to 1
  // return true if f=xfact*yfact where xfact depends on x and yfact on y only
  bool separate_variables(const gen & f,const gen & x,const gen & y,gen & xfact,gen & yfact){
    const_iterateur jt=f._VECTptr->begin(),jtend=f._VECTptr->end();
    for (;jt!=jtend;jt+=2){
      vecteur tmp(*_lname(*jt)._VECTptr);
      if (equalposcomp(tmp,y)){
	if (equalposcomp(tmp,x))
	  return false;
	yfact=yfact*pow(*jt,*(jt+1));
      }
      else
	xfact=xfact*pow(*jt,*(jt+1));
    }
    return true;
  }

  gen desolve(const gen & f_orig,const gen & x_orig,const gen & y_orig,int & ordre,vecteur & parameters,GIAC_CONTEXT){
    gen x(x_orig);
    if ( (x_orig.type==_VECT) && (x_orig._VECTptr->size()==1) )
      x=x_orig._VECTptr->front();
    if (x.type!=_IDNT)
      setsizeerr();
    if (y_orig.type==_VECT) // FIXME: differential system
      setsizeerr();
    gen f(remove_and(f_orig,at_and));
    if (f.type==_VECT)
      return desolve_with_conditions(*f._VECTptr,x,y_orig,contextptr);
    parameters.clear();
    gen y(y_orig),yof(y_orig);
    if (y_orig.type==_IDNT){
      yof=symb_of(y,gen(vecteur(1,x),_SEQ__VECT));
      f=quotesubst(f,yof,y);
      f=quotesubst(f,y,yof);
    }
    else 
      y=function_of(y_orig,x);
    f=quotesubst(remove_equal(eval(f,contextptr)),yof,y);
    // Here f= f(derive(y,x),y) for a 1st order equation
    int n=diffeq_order(f,y);
    if (n<=0)
      setsizeerr();
    vecteur v;
    identificateur t(" t");
    if (is_linear_diffeq(f,x,y,n,v,contextptr)){
      // 1st order
      if (n==1){ // a(x)*y'+b(x)*y+c(x)=0
	// y'/y=-b/a -> y=C(x)exp(-int(b/a)) and a(x)*C'*exp()+c(x)=0
	gen & a=v[0];
	gen & b=v[1];
	gen & c=v[2];
	gen i=simplify(exp(integrate_without_lnabs(rdiv(b,a),x,contextptr)));
	gen C=integrate_without_lnabs(rdiv(-c,a)*i,x,contextptr);
	parameters.push_back(diffeq_constante(0));
	return (C+parameters.front())/i;
      }
      // cst coeff?
      gen cst=v.back();
      v.pop_back();
      if (derive(v,x)==vecteur(n+1,zero)){
	// Yes!
	gen laplace_cst=laplace(-cst,x,t,contextptr);
	gen arbitrary,tmp;
	for (int i=n-1;i>=0;--i){
	  parameters.push_back(diffeq_constante(n-1-i));
	  tmp=tmp*t+parameters.back();
	  arbitrary=arbitrary+v[i]*tmp;
	}
	arbitrary=(laplace_cst+arbitrary)/symb_horner(v,t);
	arbitrary=ilaplace(arbitrary,t,x,contextptr);
	return arbitrary;
      }
    }
    vecteur substin(n);
    vecteur substout(n);
    for (int i=0;i<n;++i){
      substin[i]=symb_derive(y,x,i+1);
      substout[i]=identificateur(" y"+print_INT_(i));
    }
    gen ff=quotesubst(f,substin,substout);
    if (is_zero(derive(ff,y))){ // y incomplete
      for (int i=0;i<n;++i){
	substout[i]=symb_derive(y,x,i);
      }
      f=quotesubst(f,substin,substout);
      int tmp;
      gen sol=desolve(f,x,y,tmp,parameters,contextptr);
      parameters.push_back(diffeq_constante(int(parameters.size())));
      gen p(parameters.back());
      if (sol.type==_VECT)
	p=vecteur(sol._VECTptr->size(),p);
      sol=integrate_without_lnabs(sol,x,contextptr)+p;
      return sol;
    }
    if (n==1) { // 1st order 
      f=quotesubst(f,symb_derive(y,x),t);
      vecteur v(solve(f,t,3,contextptr)); // now solve y'=v[i](y)
      const_iterateur it=v.begin(),itend=v.end();
      vecteur sol;
      parameters.push_back(diffeq_constante(0));
      for (;it!=itend;++it){
	if (is_zero(derive(*it,x))){ // x incomplete
	  if (debug_infolevel)
	    cerr << "Incomplete" << endl;
	  gen pr=integrate_without_lnabs(inv(*it),y,contextptr)+parameters.back();
	  sol=mergevecteur(sol,solve(pr-x,*y._IDNTptr,3,contextptr));
	  continue;
	}
	// Separable variables?
	f=factors(*it,x); // Factor then split factors
        gen xfact(plus_one),yfact(plus_one);
	if (separate_variables(f,x,y,xfact,yfact)){ // y'/yfact=xfact
	  gen pr=parameters.back()+integrate_without_lnabs(inv(yfact),y,contextptr);
	  sol=mergevecteur(sol,solve(pr-integrate_without_lnabs(xfact,x,contextptr),*y._IDNTptr,3,contextptr));
	  continue;
	}
	// homogeneous?
	f=quotesubst(*it,makevecteur(x,y),makevecteur(t*x,t*y));
	if (is_zero(normal(f-*it))){
	  cerr << "Homogeneous" << endl;
	  f=normal(quotesubst(*it,y,t*x)-t);
	  // y=tx -> t'x=f
	  // Singular solutions f(t)=0
	  vecteur singuliere(multvecteur(x,solve(f,t,3,contextptr)));
	  sol=mergevecteur(sol,singuliere);
	  // Non singular: t'/f(t)=1/x
	  gen pr=parameters.back()*_simplify(exp(integrate_without_lnabs(inv(f),t,contextptr)));
	  sol.push_back(gen(makevecteur(pr,t*pr),_CURVE__VECT));
	  continue;
	}
	// exact? y'=*it=f(x,y) -> N dy + M dx=0 where -M/N=y'
	gen M,N;
	f=_fxnd(*it);
	M=-f[0];
	N=f[1];
	// find an integrating factor P such that d_x(P*N)=d_y(P*M)
	// If P depends on x then N*d_x(P)+Pd_x(N)=Pd_y(M) -> 
	// d_x(P)/P=(d_y(M)-d_x(N))/N should depend on x only
	// If P depends on y then P d_x(N)=Pd_y(M)+Md_y(P)
	// d_y(P)/P=(d_x(N)-d_y(M))/M
	// Then solve P*Ndy+P*Mdx=dF
	f=normal((derive(M,y)-derive(N,x))/N);
	if (is_zero(derive(f,y))){
	  gen P=simplify(exp(integrate_without_lnabs(f,x,contextptr)));
	  // D_y(F)=P*N
	  gen F=P*integrate_without_lnabs(N,y,contextptr);
	  // D_x(F)=P*M
	  F=F+integrate_without_lnabs(normal(P*M-derive(F,x)),x,contextptr);
	  sol=mergevecteur(sol,solve(F,*y._IDNTptr,3,contextptr));
	  continue;
	}
	f=normal((derive(N,x)-derive(M,y))/M);
	if (is_zero(derive(f,x))){
	  gen P=simplify(exp(integrate_without_lnabs(f,y,contextptr)));
	  gen F=P*integrate_without_lnabs(M,x,contextptr);
	  // D_y(F)=P*N
	  F=F+integrate_without_lnabs(normal(P*N-derive(F,y)),y,contextptr);
	  sol=mergevecteur(sol,solve(F,*y._IDNTptr,3,contextptr));
	  continue;
	}
	// Bernoulli?
	// y'=a(x)*y+b(x)*y^k
	// Let z=y^(1-k)
	// z'=(1-k)*y^(-k)*y'=(1-k)*[a(x)*z+b(x)]
	// Solve for z then for y
	f=subst(*it,y,2*y);
	f=factors(f-2*(*it)); // should be (2^k-2)*b(x)*y^k
	xfact=plus_one;
	yfact=plus_one;
	if (separate_variables(f,x,y,xfact,yfact)){
	  // xfact should be (2^k-2)*b(x) and yfact=y^k
	  if ( (yfact.type==_SYMB) && (yfact._SYMBptr->sommet==at_pow) &&
	       (yfact._SYMBptr->feuille._VECTptr->front()==y) ){
	    gen k=yfact._SYMBptr->feuille._VECTptr->back();
	    gen B=normal(xfact/(pow(plus_two,k)-plus_two));
	    gen A=normal((*it-B*pow(y,k))/y);
	    gen b=(k-1)*A;
	    gen c=(k-1)*B;
	    gen i=simplify(integrate_without_lnabs(b,x,contextptr));
	    gen C=integrate_without_lnabs(-c*exp(i),x,contextptr);
	    f= (C+parameters.front())*exp(-i);
	    sol.push_back(pow(f,inv(1-k)));
	  }
	}
      }
      return sol;
    }
    unable_to_solve_diffeq();
  }
  // "unary" version
  gen _desolve(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return _desolve(makevecteur(args,y__IDNT_e),contextptr);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    int ordre;
    vecteur parameters;
    if (s==2){
      if ( (v[1].type==_SYMB) || (v[1]._SYMBptr->sommet==at_of) )
	return desolve(v[0],(*v[1]._SYMBptr->feuille._VECTptr)[1],v[1],ordre,parameters,contextptr);
      return desolve( v[0],vx_var,v[1],ordre,parameters,contextptr);
    }
    if (s!=3)
      setsizeerr();
    return desolve( v[0],v[1],v[2],ordre,parameters,contextptr);    
  }
  const string _desolve_s("desolve");
  unary_function_eval __desolve(&_desolve,_desolve_s);
  unary_function_ptr at_desolve (&__desolve,1,true);

  const string _dsolve_s("dsolve");
  unary_function_eval __dsolve(&_desolve,_dsolve_s);
  unary_function_ptr at_dsolve (&__dsolve,_QUOTE_ARGUMENTS,true);


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
