// -*- mode:C++ ; compile-command: "g++ -I.. -g -c gauss.cc " -*- 
#include "first.h"
/*
 *  Copyright (C) 2001 R. De Graeve, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include <fstream>
#include "gauss.h"
#include "vecteur.h"
#include "derive.h"
#include "subst.h"
#include "usual.h"
#include "sym2poly.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  vecteur quad(int &b,const gen & q, const vecteur & x){
    //x=vecteur des variables, q=la fonction a tester,n=la dimension de x
    //b=2 si q est quadratique,=0,1 ou 3 si il y des termes d'ordre 0,1 ou 3
    //renvoie (la jacobienne de q)/2
    gen qs;
    gen dq;
    gen dqs;
    gen qdd;
    int n=x.size();
    
    vecteur A;
    //creation d'une matrice carree A d'ordre n
    for (int i=0;i<n;i++){
      vecteur li(n);
      A.push_back(li);
    }
    //A est un vecteur de vecteur=une matrice!
    //on met ds A :(la jacobienne de q)/2 
    for (int i=0;i<n;i++){
      for (int j=i;j<n;j++){
	qdd=derive(derive(q,x[i]),x[j]);
	qdd=recursive_normal(qdd,0); // FIXME GIAC_CONTEXT
	//cout<<i<<","<<j<<qdd<<endl;
	if (i==j){
	  (*A[i]._VECTptr)[i]=rdiv(qdd,2);
	} 
	else {
	  (*A[i]._VECTptr)[j]=rdiv(qdd,2);
	  (*A[j]._VECTptr)[i]=rdiv(qdd,2);
	}
      }
    }
    //2*A=jacobienne de q
    //on calcule qs=q en zero
    //cout<<A<<endl;  
    qs=q;
    for (int i=0;i<n;i++){
      qs=subst(qs,x[i],0);           
    }
    //qs=la valeur de q en 0
    if (qs !=0){
      b=0;
      return(A);
    }
    //on regarde si il y des termes lineaires
    for (int j=0;j<n;j++){
      dq=derive(q,x[j]);
      dqs=dq;
      for (int i=0;i<n;i++){
	dqs=subst(dqs,x[i],0);
      }
      //dqs=la diff de q en zero
      if (dqs!=0){
	b=1;
	return(A);
      }
    }
    for (int i=0;i<n;i++){
      for (int j=i;j<n;j++){
	for (int k=0;k<n;k++){
	  if (derive(A[i][j],x[k])!=0){
	    b=3;
	    return(A);
	  }
	}
      }
    }
    b=2;
    //(*(A[1]._VECTptr))[0]=21;
    return(A);
  } 
  
  vecteur qxa(const gen &q,const vecteur & x){
    //transforme une forme quadratique en une matrice symetrique A
    //(les variables sont dans x)
    int d;
    //d nbre de variables
    d=x.size();
    int da;
    //il faut verifier que q est quadratique
    vecteur A;
    int b;
    A=quad(b,q,x);  
    if (b==2) {
      return(A);
    }
    else {
      setsizeerr("q is not quadratic");
    }
  }

  gen symb_q2a(const gen & args){
    return symbolic(at_q2a,args);
  }
  gen _q2a(const gen & args){
    if (args.type!=_VECT)
      return symb_q2a(args);
    int s=args._VECTptr->size();
    if (s!=2)
      setdimerr();
    if (args._VECTptr->back().type==_VECT)
      return qxa(args._VECTptr->front(),*args._VECTptr->back()._VECTptr);
    return symb_q2a(args);
  }
  const string _q2a_s("q2a");
  unary_function_unary __q2a(&_q2a,_q2a_s);
  unary_function_ptr at_q2a (&__q2a);

  vecteur gauss(const gen & q, const vecteur & x, vecteur & D, vecteur & U, vecteur & P){
    int n=x.size();
    int b;
    gen u1;
    gen u2;
    gen q1; 
    gen l1;
    gen l2;
    vecteur R(1);
    vecteur A;
 
    vecteur I;
    vecteur L;
    for (int i=0;i<n;i++){
      vecteur li(n);
      A.push_back(li);
    }
    vecteur PR;
    for (int i=0;i<n-1;i++){
      vecteur li(n-1);
      PR.push_back(li);
    }
    vecteur PP;
    for (int i=0;i<n;i++){
      vecteur li(n);
      PP.push_back(li);
    } 
    for (int i=0;i<n;i++){
      vecteur li(n);
      li[i]=gen(1);
      I.push_back(li);
    }
    //n=x.size();
    //if (n==0){
    //R[0]=q; 
    //vecteur vide;
    //D=vide;
    //U=vide;
    //P=vide;
    //return(R);
    //}
  
    //si q n'est pas quadratique b<>2 et on retourne q
    A=quad(b,q,x);
    if (b!=2){
      R[0]=q;
      vecteur vide;
      D=vide;
      U=vide;
      return(R);
    }
    //la forme q est quadratique de matrice A
    if (q==0) { 
      //R[0]=q;    
      vecteur vide(n);
      D=vide; 
      U=vide;
    
      P=I;
      return(vide);
    }
    if (n==1){
      gen q0=_factor(q,0);  // FIXME GIAC CONTEXT
      R[0]=q0;
      vecteur un(1);
      un[0]=A[0][0];
      D=un;
      U=x;
      P=I;
      return(R);
    }
  
    int r;
    r=n;
    for (int i=n-1 ;i>=0;i--){
      if (A[i][i]!=0) {
	r=i;
      }
    }
    if (r!=n) {
      //il y a des termes carres
      u1=recursive_normal(rdiv(derive(q,x[r]),plus_two),0); // FIXME GIAC_CONTEXT
      q1=recursive_normal(q-rdiv(u1*u1,A[r][r]),0); // FIXME GIAC_CONTEXT
      
     
      
      vecteur y;
      //y contient les variables qui restent (on enleve x[r])	   
      for (int j=0;j<n;j++){
	if (j!=r){
	  y.push_back(x[j]);
	}
      }
      L=gauss(q1,y,D,U,PR);
      //on rajoute 1/a_r_r sur la diagonale D
      R[0]=rdiv(1,A[r][r]);
      D=mergevecteur(R,D);
      //on rajoute u1 aux vecteurs constitue des formes lineaires
      //q= 1/a_r_r*(u1)^2+... 
      R[0]=u1; 
      U=mergevecteur(R,U);
      
      //on _VECTlete la matrice PR de dim n-1 en la matrice PP de dim n
      //1iere ligne les coeff de u1 et rieme colonne doit avoir des 0
      for (int i=0;i<n;i++){
	(*PP[0]._VECTptr)[i]=recursive_normal(derive(u1,x[i]),0); // FIXME GIAC_CONTEXT
      }
	
      for (int i=1;i<n;i++){
	
	for (int j=0;j<r;j++){
	  (*PP[i]._VECTptr)[j]=PR[i-1][j];
	}
	for (int j=r+1;j<n;j++){
	  (*PP[i]._VECTptr)[j]=PR[i-1][j-1];
	}
      }
	
	
      P=PP;
	
      R[0]=rdiv(pow(u1,2),A[r][r]);
     
      return(mergevecteur(R,L));
    }
    
    //il n'y a pas de carres
    int r1;
    int r2;
    r1=0;
    r2=0;
    for (int i=n-2;i>=0;i--){
      for (int j=i+1;j>=1;j--){
	if (A[i][j]!=0) {
	  r1=i;
	  r2=j;
	}
      }
    }
    l1=rdiv(derive(q,x[r1]),2);
    l2=rdiv(derive(q,x[r2]),2);
    u1=recursive_normal(l1+l2,0);
    u2=recursive_normal(l1-l2,0);
    q1=recursive_normal(q-rdiv(plus_two*l1*l2,A[r1][r2]),0);
   
    vecteur y;
    for (int j=0;j<n;j++){
      if ((j!=r1) && (j!=r2)) {
	y.push_back(x[j]);
      }
    }
    
    L=gauss(q1,y,D,U,PR);
    //on rajoute 1/a_r1_r2 et -1/a_r1_r2 sur la diagonale D
    R[0]=rdiv(1,plus_two*A[r1][r2]);
    R.push_back(rdiv(-1,plus_two*A[r1][r2]));
    D=mergevecteur(R,D); 
    //on rajoute u1 et u2 au vecteur U constitue des formes lineaires
    //q= 1/a_r1_r2*(u1)^2 - 1/a_r1_r2*(u2)^2 + ... 
    R[0]=u1;
    R[1]=u2;
    U=mergevecteur(R,U);
     
      
    //on _VECTlete la matrice PR de dim n-2 en la matrice PP de dim n
    //1iere et 2ieme ligne les coeff de u1 et de u2 
    //r1ieme et r2ieme colonne doit avoir des 0
    for (int i=0;i<n;i++){
      (*PP[0]._VECTptr)[i]=recursive_normal(derive(u1,x[i]),0);
      (*PP[1]._VECTptr)[i]=recursive_normal(derive(u2,x[i]),0);
    }
	
    for (int i=2;i<n;i++){
	
      for (int j=0;j<r1;j++){
	(*PP[i]._VECTptr)[j]=PR[i-2][j];
      }
      for (int j=r1+1;j<r2;j++){
	(*PP[i]._VECTptr)[j]=PR[i-2][j-1];
      }
      for (int j=r2+1;j<n;j++){
	(*PP[i]._VECTptr)[j]=PR[i-2][j-2];
      }
    }
	
	
    P=PP;
    R[0]=rdiv(pow(u1,2),plus_two*A[r1][r2]);
    R[1]=rdiv(-pow(u2,2),plus_two*A[r1][r2]);
   
    
    return(mergevecteur(R,L)); 
  } 

  vecteur gauss(const gen & q,const vecteur & x){
    vecteur D,U,P;
    return gauss(q,x,D,U,P);
  }

  gen symb_gauss(const gen & q,const gen & x){
    return symbolic(at_gauss,makevecteur(q,x));
  }
  gen symb_gauss(const gen & args){
    return symbolic(at_gauss,args);
  }
  gen _gauss(const gen & args){
    if (args.type!=_VECT)
      return symb_gauss(args);
    int s=args._VECTptr->size();
    if (s!=2)
      setdimerr();
    if (args._VECTptr->back().type==_VECT)
      return _plus(gauss(args._VECTptr->front(),*(args._VECTptr->back()._VECTptr)));
    return symb_gauss(args);
  }
  const string _gauss_s("gauss");
  unary_function_unary __gauss(&_gauss,_gauss_s);
  unary_function_ptr at_gauss (&__gauss);

  gen axq(const vecteur &A,const vecteur & x){
    //transforme une matrice carree (symetrique) en la forme quadratique q
    //(les variables sont dans x)
    int d;
    //d nbre de variables
    d=x.size();
    int da;
    //il faut verifier que A est carree
    //A n'est pas forcement symetrique  
    da=A.size();
    if (!(is_squarematrix(A)) || (da!=d) ){
      setsizeerr("Invalid dimension");
    } 
    vecteur XL(1);
    XL=makevecteur(x);
    cout<<XL<<endl;
    vecteur XC;
    for (int i=0;i<d;i++) {
      vecteur elem;
      elem=makevecteur(x[i]);
      XC.push_back(elem);
    }
    vecteur QI(d);
    vecteur Q(1);
    QI=mmult(A,XC);
    Q=mmult(XL,QI);
    return(normal(Q[0][0]));
  }
  
  gen symb_a2q(const gen & args){
    return symbolic(at_a2q,args);
  }
  gen _a2q(const gen & args){
    if (args.type!=_VECT)
      return symb_a2q(args);
    int s=args._VECTptr->size();
    if (s!=2)
      setdimerr();
    if ((args._VECTptr->front().type==_VECT) && (args._VECTptr->back().type==_VECT))
      return axq(*args._VECTptr->front()._VECTptr,*args._VECTptr->back()._VECTptr);
    return symb_a2q(args);
  }
  const string _a2q_s("a2q");
  unary_function_unary __a2q(&_a2q,_a2q_s);
  unary_function_ptr at_a2q (&__a2q);

  vecteur qxac(const gen &q,const vecteur & x){
    //transforme une forme quadratique en une matrice symetrique A
    //(les variables sont dans x)
    int d;
    //d nbre de variables
    d=x.size();
    int da;
    //il faut verifier que q est quadratique
    vecteur A;
    int b;
    A=quad(b,q,x);  
    if (b==2) {
      return(A);
    }
    else {
      setsizeerr("q is not quadratic");
    }
  }

  void conique_reduite(const gen & equation_conique,const vecteur & nom_des_variables,gen & x0, gen & y0, vecteur & V1, vecteur &V2, gen & propre,gen & equation_reduite){
    gen q(equation_conique);
    vecteur x(nom_des_variables);
    if (x.size()!=2)
      setsizeerr();
    x.push_back(identificateur(" Tt"));
    //n est le nombre de variables en geo. projective   
    int n=3; 
    vecteur X(2);
    //nom des nouvelles variables
    X[0]=gen(string(" Xx"));
    X[1]=gen(string(" Yy"));
  
    vecteur A;
    gen qp;
    qp=q;
    for (int i=0;i<n-1;i++){
      qp=subst(qp,x[i],rdiv(x[i],x[2]));           
    }
    qp=recursive_normal(x[2]*x[2]*qp,0);
    //qp est l'equation en projective qp est quadratique
    A=qxac(qp,x);
    //q=ax^2+2bxy+cy^2+2dx+2ey+f
    gen a=A[0][0];
    gen b=A[0][1]; 
    gen c=A[1][1];
    gen d=A[0][2];
    gen e=A[1][2];
    gen f=A[2][2];
    //propre=(a*c-b*b)*f+d*(b*e-d*c)-e*(a*e-b*d);
    if ((a*c-b*b)*f+d*(b*e-d*c)-e*(a*e-b*d)!=0) {
      propre=1;
    } else {
      propre=0;
    };
    // propre=(a*c-b*b)*f+d*(b*e-d*c)-e*(a*e-b*d);
    gen X0;
    gen Y0;
    gen vp1;
    gen vp2;
    V1=vecteur(2);
    V2=vecteur(2);
    gen norme;
    norme=sqrt(recursive_normal(a*a+b*b,0)); 
    if (a*c-b*b==0) {
      //on a une parabole ou 2dr//
      //vecteur propre V1 (vp1 =0) V2 (vp2=a+c)
      if (a!=0){ 
	//on calcule le nouveau repere origine=(x0,y0) base (V1,V2)
	//(X0,Y0) sont les coord du centre ds la base (V1,V2)
	vp1=0;
	vp2=a+c;
	V1[0]=rdiv(b,norme);
	V1[1]=rdiv(-a,norme);
	V2[0]=rdiv(a,norme);
	V2[1]=rdiv(b,norme);
	Y0=-rdiv(d*a+e*b,norme*vp2);
	//cout<<"Y0="<<Y0<<endl;	
	if (d*b-e*a!=0){
	  X0=rdiv((d*a+e*b)*(d*a+e*b)-f*(a*a+b*b)*vp2,gen(2)*vp2*(d*b-e*a)*norme);
	  equation_reduite=recursive_normal(a+c,0)*pow(x[1],2)+recursive_normal(gen(2)*rdiv((d*b-e*a),norme),0)*x[0];
	} else { 
	  //si d*b-e*a==0 alors X0=0
	  equation_reduite=recursive_normal(a+c,0)*pow(x[1],2)+recursive_normal(f-recursive_normal(rdiv((d*a+e*b)*(d*a+e*b),(a+c)*(a*a+b*b)),0),0);
	  X0=0;
	}
      } else {
	//si a==0 alors b=0 (puisque a*c-b*b==0) et c!=0
	V1[0]=1;
	V1[1]=0;
	V2[0]=0;
	V2[1]=1;
	Y0=-rdiv(e,c);
	if (d!=0){
	  X0=rdiv(e*e-f*c,gen(2)*d*c);
	  equation_reduite=c*pow(x[1],2)+recursive_normal(gen(2)*d,0)*x[0];	
	} else {
	  X0=0;
	  equation_reduite=recursive_normal(c*c,0)*pow(x[1],2)+recursive_normal(f*c-e*e,0);
	}
      } 
      x0=recursive_normal(V1[0]*X0+V2[0]*Y0,0);
      y0=recursive_normal(V1[1]*X0+V2[1]*Y0,0);
    
    } 
    else {
      //si a*c-b*b!=0 on a une conique a centre ou 2 dr concourantes
    
      if (b==0){
	vp1=a;
	vp2=c;
	V1[0]=1;
	V1[1]=0;
	V2[0]=0;
	V2[1]=1;
      } else {
	//si b!=0
	gen delta;
	delta=(a-c)*(a-c)+b*b+b*b+b*b+b*b;
	vp1=recursive_normal(rdiv(a+c+sqrt(delta),2),0);
	vp2=recursive_normal(rdiv(a+c-sqrt(delta),2),0);
      
	V1[0]=rdiv(b,sqrt(recursive_normal(b*b+(vp1-a)*(vp1-a),0)));
	V1[1]=rdiv(recursive_normal((vp1-a),0),sqrt(recursive_normal(b*b+(vp1-a)*(vp1-a),0)));
	V2[0]=rdiv(b,sqrt(recursive_normal(b*b+(vp2-a)*(vp2-a),0)));
	V2[1]=rdiv(recursive_normal((vp2-a),0),sqrt(recursive_normal(b*b+(vp2-a)*(vp2-a),0)));
	if (is_positive(b,0)) { // FIXME GIAC_CONTEXT
	  //pour avoir V1,V2 base directe
	  V1[0]=-V1[0];V1[1]=-V1[1];
	} 
      }
    
      //coord du centre
      x0=rdiv((-d*c+b*e),(a*c-b*b));
      y0=rdiv((-a*e+d*b),(a*c-b*b));
    
      equation_reduite=vp1*pow(x[0],2)+vp2*pow(x[1],2)+recursive_normal(d*x0+e*y0+f,0);  
    }
  }  

  gen symb_conique_reduite(const gen & args){
    return symbolic(at_conique_reduite,args);
  }
  gen _conique_reduite(const gen & args){
    if (args.type!=_VECT)
      return symb_conique_reduite(args);
    int s=args._VECTptr->size();
    if (s!=2)
      setdimerr();
    if ((args._VECTptr->front().type==_SYMB) && (args._VECTptr->back().type==_VECT)){
      gen x0,y0,propre,eq_reduite;
      vecteur V1,V2;
      conique_reduite(args._VECTptr->front(),*args._VECTptr->back()._VECTptr,x0,y0,V1,V2,propre,eq_reduite);
      return makevecteur(makevecteur(x0,y0),mtran(makevecteur(V1,V2)),propre,eq_reduite);
    }
    return symb_conique_reduite(args);
  }
  const string _conique_reduite_s("conique_reduite");
  unary_function_unary __conique_reduite(&_conique_reduite,_conique_reduite_s);
  unary_function_ptr at_conique_reduite (&__conique_reduite);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

