/* -*- compile-command: "g++ -I.. -g -c global.cc" -*- */
#include "first.h"
/*  
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include "global.h"
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <stdexcept>
#include "gen.h"
#include "identificateur.h"
#include "symbolic.h"
#include "sym2poly.h"
#include "plot.h"
#include "rpn.h"
#include "prog.h"
#include "usual.h"
#include "tex.h"
#include "path.h"
#include <time.h>
#include "input_lexer.h"
#ifdef _HAS_LIMITS
#include <limits>
#endif
#ifdef WIN32
#ifndef GNUWINCE
#include <sys/cygwin.h>
#endif
#endif

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  // Global variable when context is 0
  bool do_lnabs=true;
  void (*fl_widget_delete_function)(void *) =0;
  ostream & (*fl_widget_archive_function)(ostream &,void *)=0;
  gen (*fl_widget_unarchive_function)(istream &,const vecteur & v)=0;
  gen (*fl_widget_updatepict_function)(const gen & g)=0;
  std::string (*fl_widget_texprint_function)(void * ptr)=0;
  int _default_color_=0;
  int default_color(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_default_color_;
    else
      return _default_color_;
  }
  void default_color(int c,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr)
      contextptr->globalptr->_default_color_=c;
    else
      _default_color_=c;
  }

  int _spread_Row_=0;
  int spread_Row(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_spread_Row_;
    else
      return _spread_Row_;
  }
  void spread_Row(int c,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr)
      contextptr->globalptr->_spread_Row_=c;
    else
      _spread_Row_=c;
  }

  int _spread_Col_=0;
  int spread_Col(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_spread_Col_;
    else
      return _spread_Col_;
  }
  void spread_Col(int c,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr)
      contextptr->globalptr->_spread_Col_=c;
    else
      _spread_Col_=c;
  }

#ifdef __SGI_CPP_LIMITS
  double _epsilon_=100*numeric_limits<double>::epsilon();
#else
  double _epsilon_=1e-10;
#endif
  double epsilon(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_epsilon_;
    else
      return _epsilon_;
  }
  void epsilon(double c,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr)
      contextptr->globalptr->_epsilon_=c;
    else
      _epsilon_=c;
  }

  bool _expand_re_im_=true; 
  bool expand_re_im(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_expand_re_im_;
    else
      return _expand_re_im_;
  }
  void expand_re_im(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_expand_re_im_=b;
    else
      _expand_re_im_=b;
  }

  int _scientific_format_=0; 
  int scientific_format(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_scientific_format_;
    else
      return _scientific_format_;
  }
  void scientific_format(int b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_scientific_format_=b;
    else
      _scientific_format_=b;
  }

  bool _complex_mode_=false; 
  bool complex_mode(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_complex_mode_;
    else
      return _complex_mode_;
  }

  void complex_mode(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_complex_mode_=b;
    else
      _complex_mode_=b;
  }

  bool _all_trig_sol_=false; 
  bool all_trig_sol(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_all_trig_sol_;
    else
      return _all_trig_sol_;
  }

  void all_trig_sol(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_all_trig_sol_=b;
    else
      _all_trig_sol_=b;
  }

  bool _complex_variables_=false; 
  bool complex_variables(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_complex_variables_;
    else
      return _complex_variables_;
  }

  void complex_variables(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_complex_variables_=b;
    else
      _complex_variables_=b;
  }

  bool _increasing_power_=false;
  bool increasing_power(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_increasing_power_;
    else
      return _increasing_power_;
  }

  void increasing_power(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_increasing_power_=b;
    else
      _increasing_power_=b;
  }

  bool _approx_mode_=false;
  bool approx_mode(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_approx_mode_;
    else
      return _approx_mode_;
  }

  void approx_mode(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_approx_mode_=b;
    else
      _approx_mode_=b;
  }

  bool _angle_radian_=true;
  bool angle_radian(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_angle_radian_;
    else
      return _angle_radian_;
  }

  void angle_radian(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_angle_radian_=b;
    else
      _angle_radian_=b;
  }

  bool _show_point_=true;
  bool show_point(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_show_point_;
    else
      return _show_point_;
  }

  void show_point(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_show_point_=b;
    else
      _show_point_=b;
  }

  bool _io_graph_=true;
  bool io_graph(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_io_graph_;
    else
      return _io_graph_;
  }

  void io_graph(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_io_graph_=b;
    else
      _io_graph_=b;
  }

  bool _variables_are_files_=false;
  bool variables_are_files(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_variables_are_files_;
    else
      return _variables_are_files_;
  }

  void variables_are_files(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_variables_are_files_=b;
    else
      _variables_are_files_=b;
  }

  bool _inside_limit_=true;
  bool inside_limit(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_inside_limit_;
    else
      return _inside_limit_;
  }

  void inside_limit(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_inside_limit_=b;
    else
      _inside_limit_=b;
  }

  bool _withsqrt_=false;
  bool withsqrt(GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      return contextptr->globalptr->_withsqrt_;
    else
      return _withsqrt_;
  }

  void withsqrt(bool b,GIAC_CONTEXT){
    if (contextptr && contextptr->globalptr )
      contextptr->globalptr->_withsqrt_=b;
    else
      _withsqrt_=b;
  }

  // Other global variables
  bool secure_run=true;
  bool center_history=false;
  bool in_texmacs=false;
  bool block_signal=false;
  int history_begin_level=0; 
  int first_error_line=-1;
  string error_token_name;
  // variable used to avoid copying the whole history between processes 
#ifdef WIN32 // Temporary
  int debug_infolevel=0;
#else
  int debug_infolevel=0;
#endif
  int threads=1;
  string cas_suffixe(".cas");
  int TEST_PROBAB_PRIME=25;
  int GCDHEU_MAXTRY=5;
  int GCDHEU_DEGREE=100;
  int DEFAULT_EVAL_LEVEL=100;
  int MODFACTOR_PRIMES =5;
  int NTL_MODGCD=50;
  int HENSEL_QUADRATIC_POWER=25;
  int KARAMUL_SIZE=17;
  int INT_KARAMUL_SIZE=300;
  int FFTMUL_SIZE=1500; 
  int MAX_ALG_EXT_ORDER_SIZE = 5;
  int TRY_FU_UPRIME=5;
  int SOLVER_MAX_ITERATE=25;
  const int BUFFER_SIZE=16384;
  // at the beginning the parent process calls make_child
  // this forks, the child process then waits for a SIGUSR1 from
  // the parent that indicate data to evaluate is ready in #cas_entree#
  // When finished the child kills the parent with SIGUSR1
  // The answer is in #cas_sortie#
  // Parent: child_busy is set to true during evaluation
  // and reset to false by SIGUSR1
  // Child uses signal_child for signal trapping and parent uses
  // data_ready
  // SIGUSR2 is used by the child process for intermediate data
  // [Typically a plot instruction]
  // The child kills the parent with SIGUSR2,
  // child_busy remains true but data ready is set
  // Then the child waits for a parent SIGUSR2 signal
  // signal_plot_parent is true for intermediate data, false otherwise
  pid_t parent_id=getpid();
  pid_t child_id=0; // child process (to replace by a vector of childs?)
  volatile bool signal_child=true; // true if busy
  volatile bool signal_plot_child=false; // true if child can continue
  volatile bool signal_plot_parent=false; 
  volatile bool child_busy=false,data_ready=false;
  volatile bool ctrl_c=false,interrupted=false;
  void_function readqueue=0;
  char buf[BUFFER_SIZE];
  bool running_file=false;
  string xcasroot; // used by WIN32 for the path to the xcas directory
#ifdef WIN32
  string xcasrc("xcas.rc");
#else
  string xcasrc(".xcasrc");
#endif

  void ctrl_c_signal_handler(int signum){
    ctrl_c=true;
#ifndef WIN32
    if (child_id)
      kill(child_id,SIGINT);
#endif
    cerr << getpid() << "Ctrl-C pressed" << endl;
  }
  // child sends a SIGUSR1
  void data_signal_handler(int signum){
          // cerr << "Parent called" << endl;
    signal_plot_parent=false;
    child_busy=false;
    data_ready=true;
  }

  void control_c(){
    if (readqueue)
      readqueue();
    if (ctrl_c){
      ctrl_c=false;
      interrupted=true;
      cerr << "Throwing exception" << endl;
      throw(std::runtime_error("Stopped by Ctrl-C"));
    }
  }

  // child sends a SIGUSR2 (intermediate data)
  void plot_signal_handler(int signum){
          // cerr << "Plot_signal_handler Parent called" << endl;
    signal_plot_parent=true;
    child_busy=false;
    data_ready=true;
  }

  void child_signal_handler(int signum){
    // cerr << "Child called" << endl;
    signal_child=true;
  }
  
  void child_plot_done(int signum){
    signal_plot_child=true;
  }
  
  bool synchronize_history=true;
  gen catch_err(const std::runtime_error & error){
    cerr << error.what() << endl;
    sst_at_stack.clear();
    current_instruction_stack.clear();
    args_stack.clear();
    protection_level=0;
    debug_mode=false;
    return string2gen(string(error.what()),false);
  }

  void kill_and_wait_sigusr2(){
    sigset_t mask, oldmask;
    sigemptyset (&mask);
    sigaddset (&mask, SIGUSR2);
    /* Wait for a signal to arrive. */
    sigprocmask (SIG_BLOCK, &mask, &oldmask);
    signal_plot_child=false;
#ifndef WIN32
    kill(parent_id,SIGUSR2);
#endif
    // cerr << "Child ready" << endl;
#ifndef WIN32
    while (!signal_plot_child)
      sigsuspend (&oldmask);
    sigprocmask (SIG_UNBLOCK, &mask, NULL);
#endif
  }

  gen wait_parent(){
    kill_and_wait_sigusr2();
    ifstream child_in(cas_entree_name().c_str());    
    gen res;
    try {
      res=unarchive(child_in,list_one_letter__IDNT);
    }
    catch (std::runtime_error & e){
      res=string2gen(e.what(),false);
    }
    child_in.close();
    return res;
  }

  vecteur subvect(const vecteur & v,int i){
    int s=v.size();
    if (i<0)
      i=-i;
    vecteur res(v);
    for (;s<i;++s)
      res.push_back(undef);
    return vecteur(res.begin(),res.begin()+i);
  }

  pid_t make_child(){ // forks and return child id
    running_file=false;
    child_busy=false;
    ctrl_c=false;
    signal(SIGINT,ctrl_c_signal_handler);
    signal(SIGUSR1,data_signal_handler);
    signal(SIGUSR2,plot_signal_handler);
    if (child_id>(pid_t) 0)
      return child_id; // exists
    child_id=fork();
    if (child_id<(pid_t) 0)
      throw(std::runtime_error("Make_child error: Unable to fork"));
    if (!child_id){ // child process, redirect input/output
      sigset_t mask, oldmask;
      sigemptyset (&mask);
      sigaddset (&mask, SIGUSR1);
      signal(SIGUSR1,child_signal_handler);
      signal(SIGUSR2,child_plot_done);
      signal_child=false;
      gen args;
      /* Wait for a signal to arrive. */
      sigprocmask (SIG_BLOCK, &mask, &oldmask);
      signal_child=false;
      for (;;){
	// cerr << "Child ready" << endl;
#ifndef WIN32
	while (!signal_child)
	  sigsuspend (&oldmask);
	sigprocmask (SIG_UNBLOCK, &mask, NULL);
#endif
	// read and evaluate input
	clock_t start, end;
	double elapsed;
	start = clock();
        messages_to_print="";
	ifstream child_in(cas_entree_name().c_str());
	// Unarchive step
	try {
	  child_in >> rpn_mode >> global_window_ymin >> history_begin_level ;
	  // cerr << args << endl;
	  if (history_begin_level<0){
	    child_in >> synchronize_history;
	    args=unarchive(child_in,list_one_letter__IDNT);
	    if (!synchronize_history){
	      // cerr << "No sync " << endl;
	      history_in[-history_begin_level-1]=args;
	      history_out=subvect(history_out,-history_begin_level-1);
	    }
	    else {
	      // cerr << " Sync " << endl;
	      history_in=*args._VECTptr;
	      args=unarchive(child_in,list_one_letter__IDNT);
	      history_out=*args._VECTptr;
	    }
	  }
	  else {
	    args=unarchive(child_in,list_one_letter__IDNT);
	    // cerr << "Lu1 " << args << endl;
	    if (history_begin_level>history_in.size())
	      history_begin_level=history_in.size();
	    history_in=mergevecteur(subvect(history_in,history_begin_level),*args._VECTptr);
	    args=unarchive(child_in,list_one_letter__IDNT);
	    // cerr << "Lu2 " << args << endl;
	    history_out=mergevecteur(subvect(history_out,history_begin_level),*args._VECTptr);
	    args=unarchive(child_in,list_one_letter__IDNT);
	    // cerr << "Lu3 " << args << endl;
	    history_in.push_back(args);
	  }
	}
	catch (std::runtime_error & error ){
	  args = string2gen("Child unarchive error:"+string(error.what()),false);
	}
	child_in.close();
	// cerr << args << endl;
	// output result of evaluation to child_out
	gen args_evaled;
	{ // BEGIN of old try block
	  if (history_begin_level<0){
	    history_begin_level=-history_begin_level-1;
	    int s=history_in.size();
	    block_signal=true;
	    for (int k=history_begin_level;k<s;++k){
	      try {
                if (history_in[k].is_symb_of_sommet(at_signal) || history_in[k].is_symb_of_sommet(at_debug))
		  history_out.push_back(eval(history_in[k]._SYMBptr->feuille));
                else
		  history_out.push_back(eval(history_in[k]));
	      }
	      catch (std::runtime_error & error){
		history_out.push_back(catch_err(error));
	      }
	    }
	    args=vecteur(history_in.begin()+history_begin_level,history_in.end());
	    args_evaled=vecteur(history_out.begin()+history_begin_level,history_out.end());
	  }
	  else {
	    if ( (args.type!=_VECT) || (args.subtype!=_RUNFILE__VECT) ){
	      if (debug_infolevel>10)
		cerr << "Child eval " << args << endl;
	      try {
		args_evaled=args.eval();
	      }
	      catch (std::runtime_error & error){
		args_evaled=catch_err(error);
	      }
	      history_out.push_back(args_evaled);
	      if (debug_infolevel>10)
		cerr << "Child result " << args_evaled << endl;
	    }
	    else {
	      vecteur v;
	      history_in.pop_back();
	      const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
	      for (;it!=itend;++it){
		if (it->is_symb_of_sommet(at_signal) ||it->is_symb_of_sommet(at_debug) )
		  continue;
		history_in.push_back(*it);
		try {
		  if (it->is_symb_of_sommet(at_debug))
		    args_evaled=it->_SYMBptr->feuille.eval();
		  else
		    args_evaled=it->eval();
		}
		catch (std::runtime_error & error){
		  args_evaled=catch_err(error);
		}
		// cerr << args_evaled << endl;
		history_out.push_back(args_evaled);
		v.push_back(args_evaled);
		ofstream child_out(cas_sortie_name().c_str());
		archive(child_out,*it);
		archive(child_out,args_evaled);
		child_out << messages_to_print << '' ;
		child_out.close();
		// cerr << "Signal reads " << res << endl;
		kill_and_wait_sigusr2();
	      }
	      // args_evaled=gen(v,args.subtype);
	      args=0;
	      args_evaled=0;
	    }
	  }
	} // END of old try/catch block
	block_signal=false;
	end = clock();
	elapsed = ((double) (end - start)) / CLOCKS_PER_SEC;
	ofstream child_out(cas_sortie_name().c_str());
	archive(child_out,args) ;
	archive(child_out,args_evaled) ;
	child_out << messages_to_print ;
	int mm=messages_to_print.size();
	if (mm && (messages_to_print[mm-1]!='\n'))
	  child_out << endl;
	child_out << "Time: " << elapsed << '' ;
	child_out.close();
	// cerr << "Child sending signal to " << parent_id << endl;
	/* Wait for a signal to arrive. */
	sigprocmask (SIG_BLOCK, &mask, &oldmask);
	signal_child=false;
#ifndef WIN32
	kill(parent_id,SIGUSR1);
#endif
      }
    }
    // cerr << "Forking " << parent_id << " " << child_id << endl;
    return child_id;
  }

  void archive_write_error(){
    cerr << "Archive error on " << cas_entree_name() << endl;
  }
  
  // return true if entree has been sent to evalation by child process
  bool child_eval(const string & entree,bool numeric,bool is_run_file){
    if (is_run_file || rpn_mode)
      history_begin_level=0;
    // added signal re-mapping because PARI seems to mess signal on the ipaq
    signal(SIGUSR1,data_signal_handler);
    signal(SIGUSR2,plot_signal_handler);
    if (!child_id)
      child_id=make_child();
    if (child_busy || data_ready)
      return false;
    gen entr;
    clock_t start, end;
    start = clock();
    try {
      ofstream parent_out(cas_entree_name().c_str());
      if (!signal_plot_parent){
	parent_out << rpn_mode << " " << global_window_ymin << " " << history_begin_level << endl;
	archive(parent_out,vecteur(history_in.begin()+history_begin_level,history_in.end()));
	archive(parent_out,vecteur(history_out.begin()+history_begin_level,history_out.end()));
      }
      if (is_run_file){
	ifstream infile(entree.c_str());
	char c;
	string s;
	while (!infile.eof()){
	  infile.get(c);
	  s += c;
	}
	entr = gen(s,list_one_letter__IDNT);
	if (entr.type!=_VECT)
	  entr=gen(makevecteur(entr),_RUNFILE__VECT);
	else
	  entr.subtype=_RUNFILE__VECT;
      }
      else {
	entr = gen(entree,list_one_letter__IDNT);
	if (numeric)
	  entr = symbolic(at_evalf,entree);
      }
      archive(parent_out,entr);
      if (!parent_out)
	setsizeerr();
      parent_out.close();
      if (!parent_out)
	setsizeerr();
    } catch (std::runtime_error & e){
      archive_write_error();
      return false;
    }
    child_busy=true;
    if (signal_plot_parent){
      // cerr << "child_eval: Sending SIGUSR2 to child" << endl;
      signal_plot_parent=false;
#ifndef WIN32
      kill(child_id,SIGUSR2);
#endif
      return true;
    }
    // cerr << "Sending SIGUSR1 to " << child_id << endl;
#ifndef WIN32
    kill(child_id,SIGUSR1);
#endif
    running_file=is_run_file;
    end = clock();
    // cerr << "# Save time" << double(end-start)/CLOCKS_PER_SEC << endl;
    return true;
  }

  bool child_reeval(int history_begin_level){
    signal(SIGUSR1,data_signal_handler);
    signal(SIGUSR2,plot_signal_handler);
    if (!child_id)
      child_id=make_child();
    if (child_busy || data_ready)
      return false;
    messages_to_print="";
    try {
      ofstream parent_out(cas_entree_name().c_str());
      parent_out << rpn_mode << " " << global_window_ymin << " " << -1-history_begin_level << " " << synchronize_history << endl;
      if (synchronize_history){
	archive(parent_out,history_in);
	archive(parent_out,vecteur(history_out.begin(),history_out.begin()+history_begin_level));
      }
      else
	archive(parent_out,history_in[history_begin_level]);
      if (!parent_out)
	setsizeerr();
      parent_out.close();
      if (!parent_out)
	setsizeerr();
    } catch (std::runtime_error & e){
      archive_write_error();
      return false;
    }
    child_busy=true;
    running_file=true;
    // erase the part of the history that we are computing again
    if (run_modif_pos<history_in.size())
      history_in.erase(history_in.begin()+run_modif_pos,history_in.end());
    if (run_modif_pos<history_out.size())
      history_out.erase(history_out.begin()+run_modif_pos,history_out.end());
#ifndef WIN32
    kill(child_id,SIGUSR1);
#endif
    return true;
  }

  void updatePICT(const vecteur & args){
    redraw_mouse_position_only=false;
    const_iterateur it=args.begin(),itend=args.end();
    gen sortie;
    for (;it!=itend;++it){
      sortie=*it;
      if (sortie.type==_POINTER_ && sortie.subtype==_FL_WIDGET_POINTER && fl_widget_updatepict_function){
	sortie = fl_widget_updatepict_function(sortie);
	// cerr << "updatepict" << sortie << endl;
      }
      if ( (sortie.type==_SYMB) && equalposcomp(plot_sommets,sortie._SYMBptr->sommet)){
	plot_instructions.push_back(sortie);
	if ((sortie._SYMBptr->feuille.type==_VECT) && (sortie._SYMBptr->feuille._VECTptr->size()==3) && (sortie._SYMBptr->feuille._VECTptr->back().type==_STRNG) && ( ((*sortie._SYMBptr->feuille._VECTptr)[1].type==_VECT) || is_zero((*sortie._SYMBptr->feuille._VECTptr)[1])) ){
	  string lab=*(sortie._SYMBptr->feuille._VECTptr->back()._STRNGptr);
	  if (lab.size() && (lab>=PICTautoname)){
	    PICTautoname=lab;
	    PICTautoname_plus_plus();
	  }
	}
      }
      else {
	if ( ((sortie.type==_SYMB) && (sortie._SYMBptr->sommet==at_erase)) ||
             ((sortie.type==_FUNC) && (*sortie._FUNCptr==at_erase)) ){
	  plot_instructions.clear();
	}
	else {
	  if ( (sortie.type==_VECT) && (sortie._VECTptr->size()) && (sortie._VECTptr->back().type==_SYMB) && (equalposcomp(plot_sommets,sortie._VECTptr->back()._SYMBptr->sommet))){
	    plot_instructions.push_back(sortie);
	    sortie=sortie._VECTptr->back();
	    if ((sortie._SYMBptr->feuille.type==_VECT) && (sortie._SYMBptr->feuille._VECTptr->size()==3) && (sortie._SYMBptr->feuille._VECTptr->back().type==_STRNG)  ){
	      string lab=*(sortie._SYMBptr->feuille._VECTptr->back()._STRNGptr);
	      if (lab.size() && (lab>=PICTautoname)){
		PICTautoname=lab;
		PICTautoname_plus_plus();
	      }
	    } 
	  }
	  else
	    plot_instructions.push_back(zero);
	}
      }
    } // end for (;it!=itend;++it)
  } 

  void signal_child_ok(){
    child_busy=true;
    data_ready=false;
    signal_plot_parent=false;
#ifndef WIN32
    kill(child_id,SIGUSR2);
#endif // WIN32
  }

  unary_function_ptr parent_evalonly_sommets[]={at_widget_size,at_keyboard,at_current_sheet,at_Row,at_Col,0};
  bool update_data(gen & entree,gen & sortie,GIAC_CONTEXT){
    // if (entree.type==_IDNT)
    //   entree=symbolic(at_sto,makevecteur(sortie,entree));
    // discarded sto autoadd otherwise files with many definitions
    // are overwritten
    debug_mode=false;
    if (signal_plot_parent){
      // cerr << "Child signaled " << entree << " " << sortie << endl;
      if ( entree.type==_SYMB ){
	if ( (entree._SYMBptr->sommet==at_click && entree._SYMBptr->feuille.type==_VECT && entree._SYMBptr->feuille._VECTptr->empty() ) 
	     || (entree._SYMBptr->sommet==at_debug) 
	     ) {
	  debug_mode=(entree._SYMBptr->sommet==at_debug);
	  // cerr << "Child waiting" << endl;
	  data_ready=false;
	  debug_info=entree._SYMBptr->feuille;
	  debug_refresh=true;
	  return true;
	}
	if ( entree._SYMBptr->sommet==at_click || entree._SYMBptr->sommet==at_inputform || entree._SYMBptr->sommet==at_interactive ){
	  // cerr << entree << endl;
	  gen res=entree.eval();
	  // cerr << res << endl;
	  ofstream parent_out(cas_entree_name().c_str());
	  archive(parent_out,res);
	  parent_out.close();
	  signal_child_ok();
	  return true;
	}
	// cerr << "Child signaled " << entree << " " << sortie << endl;
      }
      if (sortie.type==_SYMB){
	if (sortie._SYMBptr->sommet==at_SetFold){
	  current_folder_name=sortie._SYMBptr->feuille;
	  signal_child_ok();
	  return false;
	}
	if (sortie._SYMBptr->sommet==at_sto && sortie._SYMBptr->feuille.type==_VECT){
	  vecteur & v=*sortie._SYMBptr->feuille._VECTptr;
	  // cerr << v << endl;
	  if ((v.size()==2) && (v[1].type==_IDNT)){
	    if (v[1]._IDNTptr->value)
	      delete v[1]._IDNTptr->value;
	    v[1]._IDNTptr->value = new gen(v[0]);
	  }
	  signal_child_ok();
	  return false;	  
	}
	if (sortie._SYMBptr->sommet==at_purge){
	  gen & g=sortie._SYMBptr->feuille;
	  if ((g.type==_IDNT) && (g._IDNTptr->value) ){
	    delete g._IDNTptr->value;
	    g._IDNTptr->value=0;
	  }
	  signal_child_ok();
	  return false;
	}
	if ((sortie._SYMBptr->sommet==at_cd) && (sortie._SYMBptr->feuille.type==_STRNG)){
	  chdir(sortie._SYMBptr->feuille._STRNGptr->c_str());
	  signal_child_ok();
	  return false;
	}
	if ( sortie._SYMBptr->sommet==at_insmod || sortie._SYMBptr->sommet==at_rmmod || sortie._SYMBptr->sommet==at_user_operator ){
	  protecteval(sortie,DEFAULT_EVAL_LEVEL,contextptr);
	  signal_child_ok();
	  return false;
	}
	if (sortie._SYMBptr->sommet==at_xyztrange){
	  gen f=sortie._SYMBptr->feuille;
	  if ( (f.type==_VECT) && (f._VECTptr->size()>=12)){
	    protecteval(sortie,2,contextptr);
            signal_child_ok();
	    return false;
	  }
	}
	if (sortie._SYMBptr->sommet==at_cas_setup){
	  gen f=sortie._SYMBptr->feuille;
	  if ( (f.type==_VECT) && (f._VECTptr->size()>=7)){
	    vecteur v=*f._VECTptr;
	    cas_setup(v,contextptr); 
            signal_child_ok();
	    return false;
	  }
	}
      }
      if (entree.type==_SYMB && entree._SYMBptr->sommet==at_signal && sortie.type==_SYMB && equalposcomp(parent_evalonly_sommets,sortie._SYMBptr->sommet) ) {
	gen res=sortie.eval();
	ofstream parent_out(cas_entree_name().c_str());
	archive(parent_out,res);
	parent_out.close();
	signal_child_ok();	
	return false;
      }
    } // end signal_plot_parent
    // cerr << "# Parse time" << double(end-start)/CLOCKS_PER_SEC << endl;
    // see if it's a PICT update
    vecteur args;
    // update history
    if (rpn_mode) {
      if ((sortie.type==_VECT)&& (sortie.subtype==_RPN_STACK__VECT)){
	history_out=*sortie._VECTptr;
	history_in=vecteur(history_out.size(),undef);
	int i=erase_pos();
	args=vecteur(history_out.begin()+i,history_out.end());
	plot_instructions.clear();
      }
      else {
	if (entree.type==_FUNC){
	  int s=min(max(entree.subtype,0),history_out.size());
	  vecteur v(s);
	  for (int k=s-1;k>=0;--k){
	    v[k]=history_out.back();
	    history_out.pop_back();
	    history_in.pop_back();
	  }
	  entree=symbolic(*entree._FUNCptr,v);
	}
	history_in.push_back(entree);
	history_out.push_back(sortie);
	int i=erase_pos();
	args=vecteur(history_out.begin()+i,history_out.end());
	plot_instructions.clear();
      }
    }  
    else {
      bool fait=false;
      if (running_file) {
	if (entree.type==_VECT && sortie.type==_VECT) {
	  history_in=mergevecteur(history_in,*entree._VECTptr);
	  history_out=mergevecteur(history_out,*sortie._VECTptr);
	  fait=true;
	}
	if (is_zero(entree) && is_zero(sortie))
	  fait=true;
      }
      if (!fait){
	if (in_texmacs){
	  cout << GIAC_DATA_BEGIN << "verbatim:";
	  cout << "ans(" << history_out.size() << ") " << sortie << "\n";
  
	  cout << GIAC_DATA_BEGIN << "latex:$$ " << gen2tex(entree) << "\\quad = \\quad " << gen2tex(sortie) << "$$" << GIAC_DATA_END;
	  cout << "\n";
	  cout << GIAC_DATA_BEGIN << "channel:prompt" << GIAC_DATA_END;
          cout << "quest(" << history_out.size()+1 << ") ";
          cout << GIAC_DATA_END;
          fflush (stdout);
	}
	history_in.push_back(entree);
	history_out.push_back(sortie);
	// for PICT update
	args=vecteur(1,sortie);
      }
      if (running_file){
	// for PICT update
	int i=erase_pos();
	args=vecteur(history_out.begin()+i,history_out.end());
	// cerr << "PICT clear" << endl;
	plot_instructions.clear();
        //running_file=false;
      }
      // now do the update
    }
    updatePICT(args);
    data_ready=false;
    if (signal_plot_parent)
        signal_child_ok();
    return true;
  }

  void archive_read_error(){
    cerr << "Read error on " << cas_sortie_name() << endl;
    data_ready=false;
#ifndef WIN32
    if (child_id)
      kill(child_id,SIGKILL);
#endif
    child_id=0;
  }

  bool read_data(gen & entree,gen & sortie,string & message,GIAC_CONTEXT){
    if (!data_ready)
      return false;
    message="";
    try {
      ifstream parent_in(cas_sortie_name().c_str());
      if (!parent_in)
	setsizeerr();
      clock_t start, end;  
      start = clock();
      entree=unarchive(parent_in,list_one_letter__IDNT);
      sortie=unarchive(parent_in,list_one_letter__IDNT);
      end = clock();
      parent_in.getline(buf,BUFFER_SIZE,'');
      if (buf[0]=='\n')
	message += (buf+1);
      else
	message += buf;
      if (!parent_in)
	setsizeerr();
    } catch (std::runtime_error & e){
      archive_read_error();
      return false;
    }
    return update_data(entree,sortie,contextptr);
  }

  string home_directory(){
    if (access("/etc/passwd",R_OK))
      return "";
    uid_t u=getuid();
    passwd * p=getpwuid(u);
    string s(p->pw_dir);
    return s+"/";
  }

  string cas_entree_name(){
    if (getenv("XCAS_TMP"))
      return getenv("XCAS_TMP")+("/#cas_entree#"+print_INT_(parent_id));
#ifdef IPAQ
    return "/tmp/#cas_entree#"+print_INT_(parent_id);
#else
    return home_directory()+"#cas_entree#"+print_INT_(parent_id);
#endif
  }

  string cas_sortie_name(){
    if (getenv("XCAS_TMP"))
      return getenv("XCAS_TMP")+("/#cas_sortie#"+print_INT_(parent_id));
#ifdef IPAQ
    return "/tmp/#cas_sortie#"+print_INT_(parent_id);
#else
    return home_directory()+"#cas_sortie#"+print_INT_(parent_id);
#endif
  }

  void read_config(const string & name){
    if (access(name.c_str(),R_OK)) {
      cerr << "// Unable to find config file " << name << endl;
      return;
    }
    ifstream inf(name.c_str());
    if (!inf)
      return;
    vecteur args,l(list_one_letter__IDNT);
    cerr << "// Reading config file " << name << endl;
    readargs_from_stream(inf,args,l);
    gen g(args);
    if (debug_infolevel)    
      cerr << g << endl;
    g.eval();
    cerr << "// User configuration done" << endl;
  }

  // Unix: configuration is read from xcas.rc in the giac_aide_location dir
  // then from the user ~/.xcasrc
  // Win: configuration from $XCAS_ROOT/xcas.rc then from home_dir()+xcasrc
  // or if not available from current dir xcasrc
  void protected_read_config(){
    try {
      string s;
#ifdef WIN32
      if (getenv("XCAS_ROOT")){
	s=getenv("XCAS_ROOT");
	if (debug_infolevel)
	  cerr << "Found XCAS_ROOT " << s << endl;
      }
#else
      s=giac_aide_location;
      s=s.substr(0,s.size()-8);
#endif
      if (s.size())
	giac::read_config(s+"/xcas.rc");
      s=giac::home_directory();
      if (s.size()<2)
	s="";
      giac::read_config(s+xcasrc);
    }
    catch (std::runtime_error & e){
      cerr << "Error in config file " << xcasrc << " " << e.what() << endl;
    }
  }

  string giac_aide_dir(){
    if (!access((xcasroot+"aide_cas").c_str(),R_OK)){
      return xcasroot;
    }
    if (xcasroot.size()>4 && xcasroot.substr(xcasroot.size()-4,4)=="bin/"){
      string s(xcasroot.substr(0,xcasroot.size()-4));
      s+="share/giac/";
      if (!access((s+"aide_cas").c_str(),R_OK)){
	return s;
      }
    }
#ifdef WIN32
    return "/cygdrive/c/xcas/";
#endif
    string s(giac_aide_location); // ".../aide_cas"
    // test if aide_cas is there, if not test at xcasroot return ""
    if (!access(s.c_str(),R_OK)){
      return s.substr(0,s.size()-8);
    }
    return "";
  }

  std::string absolute_path(const std::string & orig_file){
#ifndef WIN32
    if (orig_file[0]=='/')
      return orig_file;
    else
      return giac_aide_dir()+orig_file;
#else
#ifndef GNUWINCE
     string res=orig_file;
     const char *_epath;
     _epath = orig_file.c_str()  ;
     /* If we have a POSIX path list, convert to win32 path list */
     if (_epath != NULL && *_epath != 0
         && cygwin_posix_path_list_p (_epath)){
       char * _win32path = (char *) malloc
	 (cygwin_posix_to_win32_path_list_buf_size (_epath));
       cygwin_posix_to_win32_path_list (_epath, _win32path);
       free(_win32path);
       int s=strlen(_win32path);
       res.clear();
       for (int i=0;i<s;++i){
	 if (_win32path[i]!='\\')
	   res += _win32path[i];
	 else
	   res += "\\\\";
       }
     }
     return res;
#else
    string file=orig_file,s;
    if (file[0]!='/'){
      file=giac_aide_dir()+file;
    }
    if (file.substr(0,10)=="/cygdrive/" || (file[0]!='/' && file[1]!=':')){
      string s1=xcasroot;
      if (file.substr(0,10)=="/cygdrive/")
	s1=file[10]+(":"+file.substr(11,file.size()-11));
      else {
	// remove /cygdrive/
	if (s1.substr(0,10)=="/cygdrive/")
	  s1=s1[10]+(":"+s1.substr(11,s1.size()-11));
	else
	  s1="c:/xcas/";
	s1 += file;
      }
      string s2;
      int t=s1.size();
      for (int i=0;i<t;++i){
	if (s1[i]=='/')
	  s2+="\\\\";
	else
	  s2+=s1[i];
      }
      return s2;
    }
#endif // GNUWINCE
#endif // WIN32
    return orig_file;
  }

  string browser_command(const string & orig_file){
    string file=orig_file;
    string s;
    bool url=false;
    if (file.substr(0,4)=="http"){
      url=true;
      s=file;
    }
    else {
      if (file[0]!='/'){
#ifdef WIN32
	file=giac_aide_dir()+file;
#else    
	s=giac_aide_dir();
#endif
      }
      s="file:"+s+file;
    }
    if (debug_infolevel)
      cerr << s << endl;
#ifdef WIN32
    if (!url && (file.substr(0,10)=="/cygdrive/" || (file[0]!='/' && file[1]!=':')) ){
      string s1=xcasroot;
      if (file.substr(0,10)=="/cygdrive/")
	s1=file[10]+(":"+file.substr(11,file.size()-11));
      else {
	// remove /cygdrive/
	if (s1.substr(0,10)=="/cygdrive/")
	  s1=s1[10]+(":"+s1.substr(11,s1.size()-11));
	else
	  s1="c:/xcas/";
	s1 += s.substr(5,s.size()-5);
      }
      string s2;
      int t=s1.size();
      for (int i=0;i<t;++i){
	if (s1[i]=='/')
	  s2+="\\\\";
	else
	  s2+=s1[i];
      }
      s="file:"+s2;
      // s = s.substr(0,5)+"C:\\\\xcas\\\\"+s2;
    }
    if (getenv("BROWSER"))
      s=getenv("BROWSER")+(" "+s+" &");
    else
      s="'/cygdrive/c/Program Files/Internet Explorer/IEXPLORE.EXE' "+s+" &";
#else
    string browser;
    if (getenv("BROWSER"))
      browser=getenv("BROWSER");
    else {
#ifdef __APPLE__
      browser="/Applications/Safari.app/Contents/MacOS/Safari";
      // Remove file: that seems not supported by Safari
      if (!url)
	s = s.substr(5,s.size()-5);
      // Remove # trailing part of URL
      int ss=s.size();
      for (--ss;ss>0;--ss){
	if (s[ss]=='#' || s[ss]=='.' || s[ss]=='/' )
	  break;
      }
      if (ss && s[ss]!='.')
	s=s.substr(0,ss);
#else
      browser="mozilla";
#endif
    }
    // find binary name
    int bs=browser.size(),i;
    for (i=bs-1;i>=0;--i){
      if (browser[i]=='/')
	break;
    }
    ++i;
    if (browser.substr(i,bs-i)=="mozilla" || browser.substr(i,bs-i)=="mozilla-bin"){
      s="if ! "+browser+" -remote \"openurl("+s+")\" ; then "+browser+" "+s+" & fi";
    }
    else
      s=browser+" "+s+" &";
#endif
    if (debug_infolevel)
      cerr << "// Running command:"+ s<<endl;
    return s;
  }

  void read_env(){
    if (getenv("XCAS_RPN")){
      cerr << "// Setting RPN mode" << endl;
      giac::rpn_mode=true;
    }
    if (getenv("GIAC_MAPLE_MODE")){
      giac::maple_mode=atoi(getenv("GIAC_MAPLE_MODE"));;
      cerr << "// Setting maple mode " << giac::maple_mode << endl;
    }
    if (getenv("GIAC_C")){
      giac::maple_mode=0;
      cerr << "// Setting giac C mode" << endl;
    }
    if (getenv("GIAC_MAPLE")){
      giac::maple_mode=1;
      cerr << "// Setting giac maple mode" << endl;
    }
    if (getenv("GIAC_MUPAD")){
      giac::maple_mode=2;
      cerr << "// Setting giac mupad mode" << endl;
    }
    if (getenv("GIAC_TI")){
      giac::maple_mode=3;
      cerr << "// Setting giac TI mode" << endl;
    }
    if (getenv("GIAC_DEBUG")){
      giac::debug_infolevel=atoi(getenv("GIAC_DEBUG"));
      cerr << "// Setting debug_infolevel to " << giac::debug_infolevel << endl;
    }
    if (getenv("LANG")){ 
      string s(getenv("LANG"));
      s=s.substr(0,2);
      if (s=="fr"){
	language=1;
	return;
      }
      if (s=="sp" || s=="es"){
	language=3;
	return;
      } 
      language=2;
    }
  }

  string cas_setup_string(GIAC_CONTEXT){
    string s("cas_setup(");
    if (approx_mode(contextptr))
      s+="1,";
    else
      s+="0,";
    if (complex_variables(contextptr))
      s+="1,";
    else
      s+="0,";
    if (complex_mode(contextptr)) 
      s+="1,";
    else
      s+="0,";
    if (angle_radian(contextptr))
      s+="1,";
    else
      s+="0,";
    if (variables_are_files(contextptr))
      s+="1,";
    else
      s+="0,";
    s += gen(epsilon(contextptr)).print();
    s += ",";
    s += print_INT_(decimal_digits);
    s += ",";
    s += print_INT_(threads);
    s += ",";
    if (increasing_power(contextptr))
      s += "1,";
    else
      s += "0,";
    if (withsqrt(contextptr))
      s += "1,";
    else
      s += "0,";
    if (all_trig_sol(contextptr))
      s += "1";
    else
      s += "0";
    s += "),";
    s += "maple_mode(";
    s += print_INT_(maple_mode);
    s += ")";
    return s;
  }

  string geo_setup_string(){
    return xyztrange(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,global_window_xmin,global_window_xmax,global_window_ymin,global_window_ymax,show_axes,class_minimum,class_size,gnuplot_hidden3d+2*gnuplot_pm3d).print();
  }

  string add_extension(const string & s,const string & ext,const string & def){
    if (s.empty())
      return def+"."+ext;
    int i=s.size();
    for (--i;i>0;--i){
      if (s[i]=='.')
	break;
    }
    if (i<=0)
      return s+"."+ext;
    return s.substr(0,i)+"."+ext;
  }

  void in_mws_translate(istream & inf,ostream & of){
    char c,oldc=0;
    // now read char by char, 
    for (;;){
      inf.get(c);
      if (c=='"')
	break;
    }
    for (;;){
      inf.get(c);
      if (c=='_'){
	of << '~';
	continue;
      }
      if (c==')' && oldc=='%') // %) -> % )
	of << " ";
      if (c=='"'){
	break;
      }
      if (c=='\n' || c==13)
	continue;
      if (c=='\\'){
	inf.get(c);
	switch (c) {
	case 'n':
	  of << '\n';
	  break;
	case '+':
	  break;
	case '"':
	  of << "\"\"";
	  break;
	default:
	  of << c;
	}
	continue;
      }
      of << c;
      oldc=c;
    }
  }

  // Maple worksheet translate
  void mws_translate(istream & inf,ostream & of){
    string thet;
    while (!inf.eof()){
      inf >> thet;
      int n1,n2,n3;
      n1=thet.size();
      if (n1>7 && thet.substr(n1-7,7)=="MPLTEXT"){
        inf >> n1 >> n2 >> n3;
	in_mws_translate(inf,of);	
	of << "\n";
      }
      else {
	if ( (n1>4 && thet.substr(n1-4,4)=="TEXT") || (n1>7 && thet.substr(n1-7,7)=="XPPEDIT") ){
	  inf >> n1 >> n2;
	  of << '"';
	  in_mws_translate(inf,of);
	  of << '"' << ";\n";
	}
      }
    }
  }

  // TI89/92 function/program translate
  void ti_translate(istream & inf,ostream & of){
    char thebuf[BUFFER_SIZE];
    inf.getline(thebuf,BUFFER_SIZE,'\n');
    inf.getline(thebuf,BUFFER_SIZE,'\n');
    string lu=thebuf;
    lu=lu.substr(6,lu.size()-7);
    cerr << "Function name: " << lu << endl;
    of << ":" << lu;
    inf.getline(thebuf,BUFFER_SIZE,'\n');  
    inf.getline(thebuf,BUFFER_SIZE,'\n');  
    of << thebuf << endl;
    for (;inf;){
      inf.getline(thebuf,BUFFER_SIZE,'\n');
      lu=thebuf;
      if (lu=="\r")
        continue;
      if (lu=="\\STOP92\\\r"){
        break;
      }
      lu = giac::tiasc_translate(lu);
      if (lu.size())
        of << ":" << lu << endl;
    }
  }

  context::context() { 
    tabptr=new sym_tab; 
    globalcontextptr=0; previous=0; globalptr=new global; 
    quoted_global_vars=new vecteur;
  }

  string remove_filename(const string & s){
    int l=s.size();
    for (;l;--l){
      if (s[l-1]=='/')
	break;
    }
    return s.substr(0,l);
  }

#ifdef __APPLE__
  bool my_isnan(double d){
    return __isnand(d);
  }

  bool my_isinf(double d){
    return __isinfd(d);
  }

#else // __APPLE__
  bool my_isnan(double d){
    return isnan(d);
  }

  bool my_isinf(double d){
    return isinf(d);
  }

#endif // __APPLE__

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
