/* -*- mode:C++;compile-command: "g++ -I.. -g -c global.cc" -*- */

/* Global definition and constants (see also dispatch.h)
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _GIAC_GLOBAL_H
#define _GIAC_GLOBAL_H
#define GIAC_CONTEXT const context * contextptr
#define GIAC_CONTEXT0 const context * contextptr
#include "first.h"
// #ifndef __APPLE__
#ifndef WIN32
#include <math.h>
//#define isnan __isnan
//#define isinf __isinf
#endif
// #endif
#ifdef __VISUALC__ // Visual C++?
typedef long pid_t;
typedef __int64 longlong ;
typedef unsigned __int64 ulonglong ;
#else
typedef long long longlong;
typedef unsigned long long ulonglong;
#endif

#ifdef GNUWINCE
#include "softmath.h"
#else
#include <cmath>
#endif

#ifdef _SOFTMATH_H
inline double giac_floor(double d){
  return std::giac_gnuwince_floor(d);
}
inline double giac_ceil(double d){
  return std::giac_gnuwince_ceil(d);
}
inline double giac_log(double d){
  return std::giac_gnuwince_log(d);
}
#else
inline double giac_floor(double d){
  return std::floor(d);
}
inline double giac_ceil(double d){
  return std::ceil(d);
}
inline double giac_log(double d){
  return std::log(d);
}
#endif



#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <signal.h>
#include <stdexcept>
#include "help.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

#define GIAC_DATA_BEGIN   ((char) 2)
#define GIAC_DATA_END     ((char) 5)
#define GIAC_DATA_ESCAPE  ((char) 27)
  extern void (*fl_widget_delete_function)(void *);
  extern std::ostream & (*fl_widget_archive_function)(std::ostream &,void *);
  extern bool secure_run; // true if used in a non-trusted environment
  extern bool center_history,in_texmacs,block_signal,synchronize_history;
  enum { 
    smallint=256, // max small int to make modular oply operations with int
    max_order=64 
  };
  extern std::string cas_suffixe;
  extern const int BUFFER_SIZE;
  extern int history_begin_level;
  extern int debug_infolevel;
  extern int threads;
  extern int DEFAULT_EVAL_LEVEL;

  extern int TEST_PROBAB_PRIME; // probabilistic primality tests
  extern int GCDHEU_MAXTRY; // maximal number of retry for heuristic algorithms
  extern int GCDHEU_DEGREE; // max degree allowed inside gcdheu
  extern int MODFACTOR_PRIMES; // number of primes used for factorization
  extern int NTL_MODGCD; // lowest degree for NTL univariate modular GCD 
  extern int HENSEL_QUADRATIC_POWER; // above #steps do quadratic Hensel lift
  extern int KARAMUL_SIZE; // Use Karatsuba multiplication if degree is >
  extern int INT_KARAMUL_SIZE; // Use Karatsuba multiplication if degree is >
  extern int FFTMUL_SIZE; // Currently adapted to (x+1)^n
  // Should be lower for larger coeff
  extern int MAX_ALG_EXT_ORDER_SIZE; // x^1/d extension not algebraic if d>
  extern int TRY_FU_UPRIME;
  extern int SOLVER_MAX_ITERATE;
  extern char buf[];
  extern volatile bool signal_plot_parent; 
  extern volatile bool ctrl_c,interrupted;
  void ctrl_c_signal_handler(int signum);
  void control_c();
  typedef void ( * void_function )();
  extern void_function readqueue; 
  // set to non-0 if you want to hook a function call inside control_c()

  template <class T> class dbgprint_vector: public std::vector<T> {
  public:
    // inherited constructors
    dbgprint_vector() : std::vector<T>::vector() { };
    dbgprint_vector(const typename std::vector<T>::const_iterator & b,const typename std::vector<T>::const_iterator & e) : std::vector<T>::vector(b,e) { };
    dbgprint_vector(int i) : std::vector<T>::vector(i) { };
    dbgprint_vector(int i,const T & t) : std::vector<T>::vector(i,t) { };
    // ~dbgprint_vector() { };
    // inherited destructors
    void dbgprint() { std::cout << *this << std::endl; }
  };
  
  struct user_function {
    std::string s;
    int parser_token;
    user_function(const std::string & mys,int i):s(mys),parser_token(i){};
  };

  class gen;
  // vecteurs and dense 1-d polynomilas
#ifdef DEBUG_SUPPORT
  typedef dbgprint_vector<gen> vecteur; // debugging support
#else
  typedef std::vector<gen> vecteur; // no debug. support
#endif
  extern gen (*fl_widget_unarchive_function)(std::istream &,const vecteur & v);
  extern std::string (*fl_widget_texprint_function)(void * ptr);
  extern gen (*fl_widget_updatepict_function)(const gen & g);
  // name -> gen table
  typedef std::map<std::string, gen> sym_tab;
  // a structure that should contain all global variables
  struct global {
    int decimal_digits;
    int _scientific_format_;
    bool do_lnabs;
    bool _expand_re_im_;
    bool _complex_mode_;
    bool _complex_variables_;
    bool _increasing_power_;
    bool _approx_mode_;
    bool _angle_radian_;
    bool _variables_are_files_;
    bool _inside_limit_;
    int _default_color_;
    double _epsilon_;
    bool _withsqrt_;
    bool _show_point_; // show 3-d point 
    bool _io_graph_; // show 2-d point in io
    int _spread_Row_,_spread_Col_;
    bool _all_trig_sol_;
    std::ostream * logptr;
    global() { decimal_digits=12; _scientific_format_=0; do_lnabs=true; _expand_re_im_=true; _complex_mode_=false; _complex_variables_=false; _default_color_=0; _increasing_power_=false; _approx_mode_=false; _angle_radian_=true; _variables_are_files_=false; _inside_limit_=false; _epsilon_=1e-10; _withsqrt_=true; _show_point_=true; _io_graph_=true; _spread_Row_ = _spread_Col_ = -1; _all_trig_sol_=false; logptr=0;};
  };

  // Context type to be used for evaluation without global variables
  // tabptr is the current evaluation context, 
  // it is a global context if globalcontextptr=0, local otherwise
  // previous is the upper local or global evaluation context
  // globalcontextptr points to the current global evaluation context
  // If globalcontextptr=0 and previous=0, we are at the top folder
  // The top level local context should have previous=globalcontextptr
  struct context {
    sym_tab * tabptr ;
    context * globalcontextptr ;
    context * previous ;
    global * globalptr; 
    vecteur * quoted_global_vars;
    context();
  };

  int scientific_format(GIAC_CONTEXT0);
  void scientific_format(int b,GIAC_CONTEXT0);

  bool expand_re_im(GIAC_CONTEXT0);
  void expand_re_im(bool b,GIAC_CONTEXT0);

  extern bool do_lnabs;
  bool complex_mode(GIAC_CONTEXT0);
  void complex_mode(bool b,GIAC_CONTEXT0);

  bool complex_variables(GIAC_CONTEXT0);
  void complex_variables(bool b,GIAC_CONTEXT0);

  bool increasing_power(GIAC_CONTEXT0);
  void increasing_power(bool b,GIAC_CONTEXT0);

  bool approx_mode(GIAC_CONTEXT0);
  void approx_mode(bool b,GIAC_CONTEXT0);

  // True if we factor 2nd order polynomials using sqrt
  bool withsqrt(GIAC_CONTEXT0);
  void withsqrt(bool b,GIAC_CONTEXT0);

  bool all_trig_sol(GIAC_CONTEXT0);
  void all_trig_sol(bool b,GIAC_CONTEXT0);

  bool angle_radian(GIAC_CONTEXT0);
  void angle_radian(bool b,GIAC_CONTEXT0);

  bool show_point(GIAC_CONTEXT0);
  void show_point(bool b,GIAC_CONTEXT0);

  bool io_graph(GIAC_CONTEXT0);
  void io_graph(bool b,GIAC_CONTEXT0);

  bool variables_are_files(GIAC_CONTEXT0);
  void variables_are_files(bool b,GIAC_CONTEXT0);

  bool inside_limit(GIAC_CONTEXT0);
  void inside_limit(bool b,GIAC_CONTEXT0);

  int default_color(GIAC_CONTEXT0);
  void default_color(int c,GIAC_CONTEXT0);

  int spread_Row(GIAC_CONTEXT0);
  void spread_Row(int c,GIAC_CONTEXT0);

  int spread_Col(GIAC_CONTEXT0);
  void spread_Col(int c,GIAC_CONTEXT0);

  double epsilon(GIAC_CONTEXT0);
  void epsilon(double c,GIAC_CONTEXT0);

  // gen_op is the type of all functions taking 1 or more gen args 
  // and returning 1 arg of type gen
  // gen argument has atomic type for an unary op, 
  // and vecteur type (compttr) otherwise
  typedef gen ( * gen_op ) (const gen & arg);
  typedef gen ( * gen_op_context ) (const gen & arg,const context * context_ptr);

  // Polynomials
  struct monome;

  // sparse polynomials: uncomment one of the 2 next lines
#ifdef DEBUG_SUPPORT
  typedef dbgprint_vector<monome> sparse_poly1; // debugging support
#else
  typedef std::vector<monome> sparse_poly1; // no debug. support
#endif

  void kill_and_wait_sigusr2(); // called by child for interm. data
  // subprocess evaluation
  // return true if entree has been sent to evalation by child process
  bool child_eval(const std::string & entree,bool numeric,bool is_run_file);
  bool child_reeval(int history_begin_level);
  bool update_data(gen & entree,gen & sortie,GIAC_CONTEXT0);
  void updatePICT(const vecteur & args);

  bool read_data(gen & entree,gen & sortie,std::string & message,GIAC_CONTEXT0);
  extern pid_t child_id;
  gen wait_parent(); // wait a SIGUSR2 from parent process, return a gen
  pid_t make_child(); // forks and return child id
  extern pid_t parent_id;
  extern bool running_file;
  extern volatile bool child_busy;
  extern volatile bool data_ready;
  extern volatile bool signal_plot_child;
  extern int first_error_line;
  extern std::string error_token_name;
  void read_config(const std::string & name);
  void protected_read_config();
  void read_env();
  std::string home_directory();
  std::string cas_entree_name();
  std::string cas_sortie_name();
  std::string cas_setup_string(GIAC_CONTEXT0);
  std::string geo_setup_string();
  std::string giac_aide_dir(); // PATH to the directory of aide_cas
  std::string browser_command(const std::string & file);
  // convert doc name to an absolute path name
  std::string absolute_path(const std::string & orig_file);
  extern std::string xcasrc,xcasroot;
  std::string add_extension(const std::string & s,const std::string & ext,const std::string & def);
  // Maple worksheet translate
  void mws_translate(std::istream & inf,std::ostream & of);
  // TI function/program ASC file translate
  void ti_translate(std::istream & inf,std::ostream & of);
  std::string remove_filename(const std::string & s);
  bool my_isnan(double d);
  bool my_isinf(double d);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC


#endif // _GIAC_GLOBAL_H
