// -*- mode:C++ ; compile-command: "g++ -I.. -g -c help.cc" -*-
#include "first.h"
#include "path.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <algorithm>
#include "help.h"
#include <iostream>
#include <dirent.h>

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  multimap<string,string> html_mtt,html_mall;
  std::vector<std::string> html_vtt,html_vall;
  int language=1;

  const char default_helpfile[]=giac_aide_location; // help filename
  const int HELP_MAXLENSIZE = 1600; // less than 20 lines of 80 chars
  const int BUFFER_SIZE=16383;

  string printint(int i){
    if (!i)
      return string("0");
    if (i<0)
      return string("-")+printint(-i);      
    int length = (int) std::floor(std::log10((double) i));
    char s[length+2];
    s[length+1]=0;
    for (;length>-1;--length,i/=10)
      s[length]=i%10+'0';
    return s;
  }

  inline int mon_max(int a,int b){
    if (a>b)
      return a;
    else
      return b;
  }

  inline int max(int a,int b,int c){
    if (a>=b){
      if (a>=c)
	return a;
      else
	return c;
    }
    if (b>=c)
      return b;
    else
      return c;
  }

  int score(const string & s,const string & t){
    int ls=s.size(),lt=t.size();
    vector<int> cur_l, new_l(lt+1,0);
    for (int j=0;j<=lt;++j)
      cur_l.push_back(-j);
    vector<int>::iterator newbeg=new_l.begin(),newend=new_l.end(),newit;
    vector<int>::iterator curbeg=cur_l.begin(),curend=cur_l.end(),curit;
    for (int i=0;i<ls;++i){
      newit=newbeg;
      curit=curbeg;
      int oldres=-i,res;
      for (int j=0;j<lt;++curit,++j){
	*newit=oldres;
	if (s[i]==t[j])
	  res=max(oldres-1,*(curit+1)-1,*curit+2);
	else {
          if (abs(s[i]-t[i])==32)
	    res=max(oldres-1,*(curit+1)-1,*curit+1);
          else
	    res=max(oldres,*(curit+1),*curit)-1;
	}
	++newit;
	oldres=res;
      }
      *newit=oldres;
      copy(newbeg,newend,curbeg);
    }
    return *newit;
  }

  bool alpha_order(const aide & a1,const aide & a2){
    return a1.cmd_name<= a2.cmd_name;
  }

  vector<aide> readhelp(const char * f_name,int & count,bool warn){
    vector<aide> v(1);
    count=0;
    if (access(f_name,R_OK)){
      if (warn)
	std::cerr << "Help file " << f_name << " not found" << endl;
      return v;
    }
    ifstream f(f_name);
    char fs[HELP_MAXLENSIZE+1];
    vector<localized_string> current_blabla;
    vector<indexed_string> current_related;
    vector<string> current_examples;
    aide current_aide;
    while (f){
      f.getline(fs,HELP_MAXLENSIZE,'\n');
      if (!fs[0])
	continue;
      string current=fs;
      if (fs[0]=='#'){
	current_aide.blabla=current_blabla;
	current_aide.examples=current_examples;
	current_aide.related=current_related;
	if (!current_aide.cmd_name.empty()){
	  vector<string> current_synonymes;
	  // parse curren_aide.cmd_name for synonyms
	  string s=current_aide.cmd_name;
	  int i;
	  for (;;){
	    // cout << s << endl;
	    i=s.find(' ');
	    if (i<=0){
	      if (!s.empty())
		current_synonymes.push_back(s);
	      break;
	    }
	    current_synonymes.push_back(s.substr(0,i));
	    s=s.substr(i+1,s.size()-i-1);
	  }
	  current_aide.synonymes=current_synonymes;
	  vector<string>::const_iterator it=current_synonymes.begin(),itend=current_synonymes.end();
	  for (;it!=itend;++it){
	    current_aide.cmd_name=*it;
	    v.push_back(current_aide);
	    ++count;
	  }
	}
	current_blabla.clear();
	current_examples.clear();
	current_related.clear();
	current_aide.cmd_name=current.substr(2,current.size()-2);
	continue;
      }
      // look for space
      int l=current.find_first_of(' ');
      if ( (l==1) && (current[0]=='0') ){
	int cs=current.size();
	while (l<cs && current[l]==' '){ ++l; }
        current_aide.syntax=current.substr(l,cs-l);
        continue;
      }
      int n=0;
      bool positif=true;
      int i=0;
      if (current[i]=='-'){
	positif=false;
	++i;
      }
      for (;i<l;++i){
	if ((current[i]<'0') || (current[i]>'9')){
	  n=0;
	  break;
	}
	else
	  n=10*n+(current[i]-int('0'));
      }
      if (!positif)
	n=-n;
      if (n>0)
	current_blabla.push_back(localized_string(n,current.substr(l+1,current.size()-l)));
      else {
	if (n<0)
	  current_related.push_back(indexed_string(-n,current.substr(l+1,current.size()-l)));
	else
	  current_examples.push_back(current);
      }
    } // end reading help from file
    current_aide.synonymes=vector<string>(1,current_aide.cmd_name);
    current_aide.blabla=current_blabla;
    current_aide.examples=current_examples;
    current_aide.related=current_related;
    v.push_back(current_aide);
    count++;
    sort(v.begin(),v.end(),alpha_order);
    return v;
  }

  aide add_synonyme_name_to_examples(const aide & a){
    aide res(a);
    std::vector<std::string>::iterator it=res.examples.begin(),itend=res.examples.end();
    for (;it!=itend;++it){
      if (!it->empty() && (*it)[0]==' ')
	continue;
      // look for a (
      unsigned i=it->find('(');
      if (i>0 && i<it->size()){ // check whether the beginning of the string is in synonyms
	string cmd=it->substr(0,i);
	std::vector<std::string>::const_iterator jt=res.synonymes.begin(),jtend=res.synonymes.end();
	for (;jt!=jtend;++jt){
	  if (*jt==cmd)
	    break;
	}
	if (jt!=jtend) // Yes, replace it
	  *it=res.cmd_name+it->substr(i,it->size()-i);
	else
	  *it=res.cmd_name+'('+*it+')';
      }
      else
	*it=res.cmd_name+'('+*it+')';
    }
    return res;
  }

  aide helpon(const string & demande,const vector<aide> & v,int language,int count,bool with_op){
    aide result;
    string current(demande);
    if (with_op)
      result.syntax = "No help available for " +current +"\n";
    else
      result.syntax="NULL";
    if (!count){
      return result;
    }
    for (int i=1;;++i){
      if (i==count){
	if (!with_op)
	  return result;
	// Find closest string
	int best_score=0,cur_score;
	vector<int> best_j;
	for (int j=1;j<count;++j){
	  cur_score=score(current,v[j].cmd_name);
	  if (cur_score>best_score){
	    best_j.clear();
	    best_j.push_back(j);
	    best_score=cur_score;
	    continue;
	  }
	  if (cur_score>=mon_max(best_score-4,0))
	    best_j.push_back(j);
	}
	if (best_score>0){
	  vector<int>::iterator it=best_j.begin(),itend=best_j.end();
	  for (int k=1;(k<10) && (it!=itend);++k,++it)
	    result.related.push_back(indexed_string(k, v[*it].cmd_name));
	}
	result.syntax += "Best match has score " + printint(best_score) + "\n";
	result.cmd_name = current;
	return result;
      }
      if (current==v[i].cmd_name){
	result=v[i];
	if (!with_op)
	  return add_synonyme_name_to_examples(result);
	result.syntax= current + "(" +result.syntax +")\n";
	return add_synonyme_name_to_examples(result);
      }
    } // end for i
  }

  string writehelp(const aide & cur_aide,int language){
    string result=cur_aide.syntax;
    vector<localized_string>::const_iterator it=cur_aide.blabla.begin(),itend=cur_aide.blabla.end();
    for (;it!=itend;++it){
      if (it->language==language){
	result += it->chaine +'\n' ;
	break;
      }
    }
    vector<indexed_string>::const_iterator iti=cur_aide.related.begin(),itiend=cur_aide.related.end();
    if (itiend!=iti){
      result +=  "See also: ";
      for (;iti!=itiend;++iti){
	result += printint(iti->index) + "/ " + iti->chaine + " ";
      }
      result += '\n' ;
    }
    vector<string>::const_iterator its=cur_aide.examples.begin(),itsend=cur_aide.examples.end();
    for (int i=1;its!=itsend;++its,++i){
      string current = "Ex" + printint(i)+':'+*its ;
      result += current +'\n' ;
      // system(current.c_str());
    }
    return result;
  }

  // find index nodes in file file
  void find_index(const std::string & current_dir,const std::string & file,multimap<std::string,std::string>&mtt,multimap<std::string,std::string>&mall,bool is_index=false,bool warn=false){
    if (access(file.c_str(),R_OK))
      return;
    ifstream i(file.c_str());
    // Skip navigation panel
    char buf[BUFFER_SIZE+1];
    for (;!i.eof();){
      i.getline(buf,BUFFER_SIZE,'\n');
      string s(buf);
      if (s=="<!--End of Navigation Panel-->")
	break;
      int t=s.size();
      if (t>14 && s.substr(t-14,14)=="Index</A></B> "){
	// look in the corresponding index file instead
	int t1=s.find("HREF")+6;
	if (t1>=0 && t1<t-16){
	  s=s.substr(t1,t-16-t1);
	  if (warn)
	    cerr << "Using index " << s << endl;
	  find_index(current_dir,current_dir+s,mtt,mall,true,warn);
	  return;
	}
      }
    }
    string tmp;
    char c;
    for (;!i.eof();){
      // read i search for a <A word
      i >> tmp;
      int l=tmp.size();
      string tts;
      if (is_index){
	if (l<13 || tmp.substr(0,12)!="<DT><STRONG>")
	  continue;
	int l1=tmp.find("</STRONG>");
	if (l1<13 || l1>=l)
	  continue;
	tts=tmp.substr(12,l1-12);
      } 
      else {
	if (l<2 || tmp.substr(l-2,2)!="<A")
	  continue;
      }
      // read the link
      tmp="";
      int s=0;
      for (;!i.eof();++s){
	i.get(c);
	tmp += c;
	if (s>4 && tmp.substr(s-4,4)=="</A>")
	  break;
      }
      // cerr << tmp << endl;
      // analysis, search for HREF
      int href=tmp.find("HREF=\"");
      if (href<0 || href+6>=s)
	continue;
      string hrefs(current_dir);
      int hrefend=0;
      for (int j=href+6;j<s;++j){
	if (tmp[j]=='"'){ // remove HREF=
	  hrefend=j+1;
	  break;
	}
	hrefs += tmp[j];
      }
      if (!hrefend)
	continue;
      if (is_index){
	mtt.insert(pair<string,string>(tts,hrefs));
	mall.insert(pair<string,string>(tts,hrefs));
      }
      else {
	// search for TT
	int tt=tmp.find("<TT>"),ttend=tt;
	if (tt>=0 && tt+6<s){
	  for (ttend+=4;ttend<s;++ttend){
	    if (tmp[ttend]=='<'){
	      ttend +=5;
	      break;
	    }
	    tts += tmp[ttend];
	  }
	  mtt.insert(pair<string,string>(tts,hrefs));
	  mall.insert(pair<string,string>(tts,hrefs));
	  tmp=tmp.substr(0,tt)+tmp.substr(ttend,tmp.size()-ttend);
	}
	// add href for all normal words
	s=tmp.size();
	int j=hrefend+1;
	for (;j<s;){
	  // read word
	  int pos=tmp.find(' ',j);
	  if (pos>j && pos<s){
	    // add it
	    string tmpins(tmp.substr(j,pos-j));
	    mall.insert(pair<string,string>(tmpins,hrefs));
	  }
	  if (pos==-1){
	    string tmpins(tmp.substr(j,s-j));
	    mall.insert(pair<string,string>(tmpins,hrefs));
	    break;
	  }
	  j=pos+1;
	}
      } // end else is_index
    }
  }

  string subdir_strings[]={"casflan","casgeo","casrouge","cassim","castor","tutoriel"};
  int subdir_taille=6;
  int equalposcomp(string * tab,const string & s){
    int i=s.size()-1;
    for (;i>=0;--i){
      if (s[i]=='/')
	break;
    }
    ++i;
    string t=s.substr(i,s.size()-i);
    for (i=0;i<subdir_taille;++i){
      // cerr << *(tab+i) << " " << t << endl;
      if (*(tab+i)==t)
	return i+1;
    }
    return 0;
  }
#ifdef WIN32
  static int dir_select (const struct dirent *d){
    string s(d->d_name);
    // cerr << s << endl;
    int t=s.size();    
    if (s[t-1]=='\\'){
      return s!="." && s!="..";
    }
    if (t<9)
      return 0;
    if (s[t-1]=='l'){
      s=s.substr(0,t-1);
      --t;
    }
    if (t>9)
      s=s.substr(t-9,9);
    return  s=="index.htm";
  }
#else
  static int dir_select (const struct dirent *d){
    string s(d->d_name);
    if (d->d_type==DT_DIR || equalposcomp(subdir_strings,s)){
      return s!="." && s!="..";
    }
    int t=s.size();
    if (t<9)
      return 0;
    if (s[t-1]=='l'){
      s=s.substr(0,t-1);
      --t;
    }
    if (t>9)
      s=s.substr(t-9,9);
    return  s=="index.htm";
  }
#endif

  void find_all_index(const std::string & subdir,multimap<std::string,std::string> & mtt,multimap<std::string,std::string> & mall){
#ifdef GNUWINCE
    return;
#else
    // cerr << "HTML help Scanning " << subdir << endl;
    DIR *dp;
    struct dirent *ep;
    
    dp = opendir (subdir.c_str());
    if (dp != NULL){
      string s;
      int t;
      while (ep = readdir (dp)){
	s=ep->d_name;
	t=s.size();
	if (t>5 && s.substr(t-4,4)=="html")
	  html_vall.push_back(subdir+s);
      }
      closedir (dp);
    }

    struct dirent **eps;
    int n;
    n = scandir (subdir.c_str(), &eps, dir_select, alphasort);
    if (n >= 0){
      int cnt;
      for (cnt = 0; cnt < n; ++cnt){
	string s(eps[cnt]->d_name);
	s= subdir+s;
#ifdef WIN32
        int t=s.size();
        if (s[t-1]=='\\')
	  find_all_index(s+"/",mtt,mall);
        else 
	  find_index(subdir,s,mtt,mall);
#else
	unsigned char type=eps[cnt]->d_type;
	if (type==DT_DIR || equalposcomp(subdir_strings,s))
	  find_all_index(s+"/",mtt,mall);
	else
	  find_index(subdir,s,mtt,mall);
#endif
      }
    }
#endif // GNUWINCE
  }

  // Return all HTML nodes refered to s in mtt
  std::vector<std::string> html_help(multimap<std::string,std::string> & mtt,const std::string & s){
    vector<string> v;
    multimap<string,string>::const_iterator it=mtt.lower_bound(s),itend=mtt.upper_bound(s);
    for (;it!=itend;++it){
      v.push_back(it->second);
    }
    return v;
  }

  string xcasroot_dir(char * arg){
    string xcasroot;
    if (getenv("XCAS_ROOT")){
      xcasroot=string(getenv("XCAS_ROOT"));
      if (xcasroot[xcasroot.size()-1]!='/')
	xcasroot+='/';
    }
    else {
      xcasroot=arg;
      int xcasroot_size=xcasroot.size()-1;
      for (;xcasroot_size>=0;--xcasroot_size){
	if (xcasroot[xcasroot_size]=='/')
	  break;
      }
      if (xcasroot_size>0)
	xcasroot=xcasroot.substr(0,xcasroot_size)+"/";
      else {
	if (access("/usr/bin/xcas",R_OK)==0)
	  xcasroot="/usr/bin/";
	else {
#ifdef __APPLE__
	if (access("/Volume/xcas/xcas",R_OK)==0)
	  xcasroot="/Volume/xcas/";
#else
	  if (access("/usr/local/bin/xcas",R_OK)==0)
	    xcasroot="/usr/local/bin/";
#endif
	  else
	    xcasroot="./";
	}
      }
    }
    // ofstream of("/tmp/xcasroot");
    // of << xcasroot << endl;
    return xcasroot;
  }

  // extern int debug_infolevel;
  bool get_index_from_cache(const char * filename, multimap<string,string> & multi){
    char buf[BUFFER_SIZE];
    ifstream if_mtt(filename);
    int n=0;
    while (if_mtt && !if_mtt.eof()){
      if_mtt.getline(buf,BUFFER_SIZE,'');
      if (!if_mtt || if_mtt.eof()){
	cerr << "// Read " << n << " entries from cache " << filename << endl;
	return true;
      }
      string first(buf);
      if_mtt.getline(buf,BUFFER_SIZE,'');
      if (!if_mtt || if_mtt.eof())
	return false;
      multi.insert(pair<string,string>(first,buf));
      ++n;
      if_mtt.getline(buf,BUFFER_SIZE,'\n');
    }
    cerr << "// Read " << n << " entries from cache " << filename ;
    return true;
  }

  bool get_index_from_cache(const char * filename, vector<string> & multi){
    char buf[BUFFER_SIZE];
    ifstream if_mtt(filename);
    int n=0;
    while (if_mtt && !if_mtt.eof()){
      if_mtt.getline(buf,BUFFER_SIZE,'');
      if (!if_mtt || if_mtt.eof()){
	cerr << "// Read " << n << " entries from cache " << filename << endl;
	return true;
      }
      multi.push_back(buf);
      ++n;
      if_mtt.getline(buf,BUFFER_SIZE,'\n');
    }
    cerr << "// Read " << n << " entries from cache " << filename ;
    return true;
  }

  string html_help_init(char * arg){
    string xcasroot=xcasroot_dir(arg);
    // HTML online help
    string html_help_dir=xcasroot+"doc/";
    if (access(html_help_dir.c_str(),R_OK)){
      if (!access("/usr/bin/xcas",R_OK))
	html_help_dir="/usr/share/giac/doc/";
      else {
	if (!access("/usr/local/bin/xcas",R_OK))
	  html_help_dir="/usr/local/share/giac/doc/";
      }
    }
    if (access(html_help_dir.c_str(),R_OK) && xcasroot.size()>4 && xcasroot.substr(xcasroot.size()-4,4)=="bin/")
      html_help_dir=xcasroot.substr(0,xcasroot.size()-4)+"share/giac/doc/";
    if (access(html_help_dir.c_str(),R_OK))      
      cerr << "Unable to open HTML doc directory " << html_help_dir << endl;
    switch (language){
    case 1:
      html_help_dir +="fr/";
#ifdef WIN32
	html_help_dir +="casflan/"; // temporary workaround
#endif
	break;
    case 2:
      html_help_dir +="en/";
      break;
    case 3:
      html_help_dir +="es/";
      break;
    }
    // Get indices from file cache if it exists
    if (!access((html_help_dir+"html_mtt").c_str(),R_OK) && !access((html_help_dir+"html_mall").c_str(),R_OK) && !access((html_help_dir+"html_vall").c_str(),R_OK)){
      if (get_index_from_cache((html_help_dir+"html_mtt").c_str(),html_mtt)&&
	  get_index_from_cache((html_help_dir+"html_mall").c_str(),html_mall)&&
	  get_index_from_cache((html_help_dir+"html_vall").c_str(),html_vall) )
	return html_help_dir;
    }
    find_all_index(html_help_dir,html_mtt,html_mall);
    // Write all indices in a file cache
    ofstream of_mtt((html_help_dir+"html_mtt").c_str());
    multimap<string,string>::const_iterator it=html_mtt.begin(),itend=html_mtt.end();
    for (;it!=itend;++it)
      of_mtt << it->first << '' << it->second << '' << endl;
    of_mtt.close();
    ofstream of_mall((html_help_dir+"html_mall").c_str());
    it=html_mall.begin();itend=html_mall.end();
    for (;it!=itend;++it)
      of_mall << it->first << '' << it->second << '' << endl;
    of_mall.close();
    ofstream of_vall((html_help_dir+"html_vall").c_str());
    vector<string>::const_iterator st=html_vall.begin(),stend=html_vall.end();
    for (;st!=stend;++st)
      of_vall << *st << '' << endl;
    of_vall.close();
    /*
    if (debug_infolevel){
      vector<string>::const_iterator it=html_vall.begin(),itend=html_vall.end();
      for (;it!=itend;++it)
	cerr << *it << endl;
    }
    */
    return html_help_dir;
  }

  bool grep(FILE * f,const string & s){
    int l=s.size();
    int pos=0;
    if (!f || !l)
      return false;
    char c0=tolower(s[0]),c;
    for (;;){
      if (feof(f) || ferror(f)){
	return false;
      }
      c=tolower(fgetc(f));
      if (c==tolower(s[pos])){
	++pos;
	if (pos==l){
	  return true;
	}
      }
      else {
	if (c==c0)
	  pos=1;
	else
	  pos=0;
      }
    }
  }

  bool grep(const string & filename,const string & s){
    FILE * f=fopen(filename.c_str(),"r");
    bool res=grep(f,s);
    fclose(f);
    return res;
  }

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
