 /* -*- mode: C++; compile-command: "flex -Pgiac_yy -olex.yy.c input_lexer.ll && mv -f lex.yy.c input_lexer.cc && g++ -g -I.. -c input_lexer.cc" -*- */
/** @file input_lexer.h
 *
 *  Lexical analyzer definition for reading expressions.
 *  input_lexer.ll must be processed with flex. */

/*
 *  Original version by GiNaC 
 *  Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
 *  Modified for Giac (c) 2001, Bernard Parisse, Institut Fourier
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GIAC_INPUT_LEXER_H__
#define __GIAC_INPUT_LEXER_H__
#include "first.h"

extern "C" {
#include <stdio.h>
}

#include "global.h"
#include <string>
#include <map>

// yacc stack type
#ifndef YYSTYPE
#define YYSTYPE gen
#endif

// lex functions/variables
extern int giac_yyerror(const char *s);
extern int giac_yylex(void);
extern void giac_yyrestart(FILE *f);

extern char *giac_yytext;

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
  extern std::map<std::string,std::vector<std::string> > lexer_translator;
  extern std::map<std::string,std::vector<std::string> > library_functions;
  extern sym_tab lexer_functions;
  extern sym_tab syms;
  // The lexer recognize first static declared symbols as declared in
  // input_lexer.ll, then it does 2 steps:
  // step 1: look in lexer_translator if the name is a recognized
  // function name, if so translate to the real giac function name
  // step 2: look in syms for an identifier, if no exists with this
  // name make one
  // lexer_functions is the table of all used symbols with real giac names. 
  // The subtype of the gen is used
  // to keep the parser token returned by the lexer

  // Used to keep track of functions inserted during an insmod
  extern std::vector<user_function> registered_lexer_functions;

  // Other global variables
  extern std::string parse_filename; // filename used for error reporting
  extern int lexer_line_number;
  // Result returned by the parser (defined in input_parser.yy, used in gen.cc)
  extern gen parsed_gen;
  /* true if in a spreadsheet formula i.e. beginning with = */
  extern bool spread_formula;
  
  class unary_function_ptr;
  // Return true if s is associated to a function with non prefix syntax
  bool has_special_syntax(const std::string & s);
  bool lexer_functions_register(const unary_function_ptr & u,const std::string & s,int parser_token);
  bool lexer_function_remove(const std::vector<user_function> & v);

  // return the token associated to the string, T_SYMBOL if not found
  int find_or_make_symbol(const std::string & s,gen & res);
  
  /** Set the input string to be parsed by giac_yyparse() (used internally). */
  void set_lexer_string(const std::string &s);
  std::string the_lexer_string();
  int the_lexer_pos();
  
  /** Add to the list of predefined symbols for the lexer. */
  void set_lexer_symbols(const vecteur & l);
  
  /** The expression parser function (used internally). */
  int giac_yyparse();
  
  /** Get error message from the parser. */
  std::string get_parser_error(void);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

#endif // ndef __GIAC_INPUT_LEXER_H__
