// -*- mode:C++ ; compile-command: "g++ -I.. -g -c intg.cc" -*-
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _GIAC_INTG_H
#define _GIAC_INTG_H
#include "first.h"
#include <string>

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
  class gen;
  class identificateur;
  class unary_function_ptr;

  gen firstcoefftrunc(const gen & e);
  // special version of lvarx that does not remove cst powers
  vecteur lvarxpow(const gen &e,const identificateur & x);

  bool is_rewritable_as_f_of(const gen & fu,const gen & u,gen & fx,const identificateur & x,GIAC_CONTEXT0);
  bool in_is_rewritable_as_f_of(const gen & fu,const gen & u,gen & fx,const identificateur & x,GIAC_CONTEXT0);
  bool is_constant_wrt(const gen & e,const identificateur & x);
  bool is_linear_wrt(const gen & e,const identificateur &x,gen & a,gen & b);
  bool is_quadratic_wrt(const gen & e,const identificateur &x,gen & a,gen & b,gen & c);
  gen linear_apply(const gen & e,const identificateur & x,gen & remains, GIAC_CONTEXT, gen (* f)(const gen &,const identificateur &,gen &,const context *));

  // integr. of rat. fcns.
  gen integrate(const gen & e, const identificateur & x, gen & remains_to_integrate,GIAC_CONTEXT0);
  gen linear_integrate(const gen & e,const identificateur & x,gen & remains_to_integrate,GIAC_CONTEXT0);
  gen integrate(const gen & e,const identificateur & x,GIAC_CONTEXT0);
  gen integrate(const gen & e,const gen & f,GIAC_CONTEXT0);
  gen _integrate(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_integrate ;
  double rombergo(const gen & f,const gen & x, const gen & a, const gen & b, int n);
  double rombergt(const gen & f,const gen & x, const gen & a, const gen & b, int n);
  gen symb_romberg(const gen & a,const gen & b);
  extern const std::string _romberg_s;
  gen _romberg(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_romberg;

  // remove quote inside a maple-like sum/product argument vector
  // returns true if maple syntax was used
  bool maple_sum_product_unquote(vecteur & v,GIAC_CONTEXT);
  // replace x=a..b by x,a,b in the second vector arg
  void adjust_int_sum_arg(vecteur & v,int & s);
  gen prodsum(const gen & g,bool isprod);
  // type=0 for seq, 1 for prod, 2 for sum
  gen seqprod(const gen & g,int type,GIAC_CONTEXT0);
  extern const std::string _sum_s;
  gen sum(const gen & e,const gen & x,const gen & a,const gen &b,GIAC_CONTEXT0);
  gen _sum(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_sum;
  gen sum_loop(const gen & e,const gen & x,int i,int j,GIAC_CONTEXT0);

  gen bernoulli(const gen & x);
  extern const std::string _bernoulli_s;
  gen _bernoulli(const gen & args);
  extern unary_function_ptr at_bernoulli;

  // solve dy/dt=f(t,y) with initial value y(t0)=y0 to final value t1
  // returns by default y[t1] or a std::vector of [t,y[t]]
  // if return_curve is true stop as soon as y is outside ymin,ymax
  // f is eitheir a prog (t,y) -> f(t,y) or a comp [f(t,y) t y]
  gen odesolve(const gen & t0orig,const gen & t1orig,const gen & f,const gen & y0orig,bool return_curve=false,double * ymin=NULL,double * ymax=NULL,GIAC_CONTEXT = 0);
  extern const std::string _odesolve_s;
  gen _odesolve(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_odesolve;

  extern const std::string _ibpdv_s;
  gen _ibpdv(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_ibpdv;

  gen _fourier_an(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_fourier_an ;

  gen _fourier_bn(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_fourier_bn ;
   
  gen _fourier_cn(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_fourier_cn ;


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC


#endif // _GIAC_INTG_H
