// -*- mode:C++ ; compile-command: "g++ -I.. -I../include -g -c maple.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include <cstdlib>
#include <stdio.h>
#include <fcntl.h>
#include "sym2poly.h"
#include "usual.h"
#include "moyal.h"
#include "solve.h"
#include "intg.h"
#include "prog.h"
#include "misc.h"
#include "maple.h"
#include "plot.h"
#include "ifactor.h"
#include "subst.h"
#include "input_parser.h"
#include "sym2poly.h"
#include "modpoly.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  gen _zip(const gen & g,const context * contextptr){
    if (g.type!=_VECT || g._VECTptr->size()<3)
      return symbolic(at_zip,g);
    vecteur & v=*g._VECTptr;
    int s=v.size();
    gen & f=v[0];
    if (v[1].type!=_VECT || v[2].type!=_VECT)
      return f(gen(makevecteur(v[1],v[2]),_SEQ__VECT),contextptr);
    vecteur & w1=*v[1]._VECTptr;
    vecteur & w2=*v[2]._VECTptr;
    int s1=w1.size(),s2=w2.size();
    vecteur res;
    int ss=min(s1,s2),i=0;
    res.reserve(ss);
    for (;i<ss;++i)
      res.push_back(_zip(gen(makevecteur(f,w1[i],w2[i]),_SEQ__VECT),contextptr));
    if (s==3)
      return res;
    gen & ff=v[3];
    for (;i<s1;++i)
      res.push_back(_zip(gen(makevecteur(f,w1[i],ff),_SEQ__VECT),contextptr));
    for (;i<s2;++i)
      res.push_back(_zip(gen(makevecteur(f,ff,w2[i]),_SEQ__VECT),contextptr));
    return res;
  }
  const string _zip_s("zip");
  unary_function_eval __zip(&_zip,_zip_s);
  unary_function_ptr at_zip (&__zip,0,true);

  gen _accumulate_head_tail(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=3)
      return symbolic(at_accumulate_head_tail,g);
    vecteur & v=*g._VECTptr;
    if (v[0].type!=_VECT || v[1].type!=_INT_ || v[2].type!=_INT_)
      setsizeerr();
    vecteur & w=*v[0]._VECTptr;
    int tail=v[1].val;
    int head=v[2].val;
    int s=w.size();
    if (tail<1 || head<1 || tail+head>s)
      setsizeerr();
    gen tmp;
    int i=0;
    for (;i<tail;++i)
      tmp=tmp+w[i];
    vecteur res;
    res.push_back(tmp);
    int t=s-head;
    for (;i<t;++i)
      res.push_back(w[i]);
    tmp=zero;
    for (;i<s;++i)
      tmp=tmp+w[i];
    res.push_back(tmp);
    return gen(res,v[0].subtype);
  }
  const string _accumulate_head_tail_s("accumulate_head_tail");
  unary_function_unary __accumulate_head_tail(&_accumulate_head_tail,_accumulate_head_tail_s);
  unary_function_ptr at_accumulate_head_tail (&__accumulate_head_tail,0,true);

  gen _divide(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    int s=v.size();
    if (s==2)
      return _quorem(g);
    gen q("Quo"),r("Rem"),arg;
    gen mode=v.back();
    if (s==4)
      arg=gen(makevecteur(v[0],v[1],v[2]),_SEQ__VECT);
    else
      arg=gen(makevecteur(v[0],v[1]),_SEQ__VECT);
    if (mode==q)
      return _quo(arg);
    if (mode==r)
      return _rem(arg);
    if (s==4)
      return _quorem(arg);
    if (s==3 && mode.type==_IDNT)
      return _quorem(g);
    setsizeerr();
  }
  const string _divide_s("divide");
  unary_function_unary __divide(&_divide,_divide_s);
  unary_function_ptr at_divide (&__divide,0,true);

  const string _ratnormal_s("ratnormal");
  unary_function_unary __ratnormal(&ratnormal,_ratnormal_s);
  unary_function_ptr at_ratnormal (&__ratnormal,0,true);

  gen _about(const gen & g){
    if (g.type==_VECT)
      return apply(g,_about);
    if (g.type==_IDNT)
      return g._IDNTptr->eval();
    return g;
  }
  const string _about_s("about");
  unary_function_unary __about(&_about,_about_s);
  unary_function_ptr at_about (&__about,0,true);

  gen _inverse(const gen & a_orig){
    matrice a;
    bool rationalize,use_bareiss,minor_det,keep_pivot;
    int last_col;
    if (!read_reduction_options(a_orig,a,rationalize,use_bareiss,minor_det,keep_pivot,last_col))
      return inv(a_orig);
    if (keep_pivot)
      setsizeerr("Option keep_pivot not applicable");
    if (minor_det){ // not really minors, use Leverrier algorithm
      vecteur b;
      vecteur p(mpcar(a,b,true));
      gen res=b.back()/p.back();
      // if (a.size()%2==0)
	res=-res;
      return res;
    }
    matrice res;
    minv(a,res,rationalize,use_bareiss);
    return res;
  }
  const string _inverse_s("inverse");
  unary_function_unary __inverse(&_inverse,_inverse_s);
  unary_function_ptr at_inverse (&__inverse,0,true);

  gen _Inverse(const gen & g){
    return symbolic(at_inverse,g);
  }
  const string _Inverse_s("Inverse");
  unary_function_unary __Inverse(&_Inverse,_Inverse_s);
  unary_function_ptr at_Inverse (&__Inverse,0,true);

  gen maple_gcdigcd(const gen & a_orig,unary_function_ptr & u,GIAC_CONTEXT){
    if (a_orig.type!=_VECT || a_orig._VECTptr->size()<2)
      setsizeerr();
    vecteur & v=*a_orig._VECTptr;
    int s=v.size();
    if (s<4)
      return u(a_orig,contextptr);
    gen res;
    if (s==4)
      res=u(makevecteur(v[0],v[1]),contextptr);
    else
      res=u(makevecteur(v[0],v[1],v[2]),contextptr);
    if (res.type==_VECT && res._VECTptr->size()==3 && v[s-2].type==_IDNT && v[s-1].type==_IDNT){
      sto(res[0],v[s-2],contextptr);
      sto(res[1],v[s-1],contextptr);
      return res[2];
    }
    else
      return res;
    setsizeerr();
  }
  gen _igcdex(const gen & a_orig,GIAC_CONTEXT){
    return maple_gcdigcd(a_orig,at_iegcd,contextptr);
  }
  const string _igcdex_s("igcdex");
  unary_function_eval __igcdex(&_igcdex,_igcdex_s);
  unary_function_ptr at_igcdex (&__igcdex,0,true);

  const string _igcd_s("igcd");
  unary_function_unary __igcd(&_gcd,_igcd_s);
  unary_function_ptr at_igcd (&__igcd,0,true);

  gen _gcdex(const gen & a_orig,GIAC_CONTEXT){
    return maple_gcdigcd(a_orig,at_egcd,contextptr);
  }
  const string _gcdex_s("gcdex");
  unary_function_eval __gcdex(&_gcdex,_gcdex_s);
  unary_function_ptr at_gcdex (&__gcdex,0,true);

  const string _indets_s("indets");
  unary_function_unary __indets(&_lname,_indets_s);
  unary_function_ptr at_indets (&__indets,0,true);

  gen _revlist(const gen & a){
    if (a.type==_VECT){
      vecteur v=*a._VECTptr;
      reverse(v.begin(),v.end());
      return gen(v,a.subtype);
    }
    return a;
  }
  const string _revlist_s("revlist");
  unary_function_unary __revlist(&_revlist,_revlist_s);
  unary_function_ptr at_revlist (&__revlist,0,true);

  const string _restart_s("restart");
  unary_function_eval __restart(&_rm_all_vars,_restart_s);
  unary_function_ptr at_restart (&__restart,0,T_RETURN);

  gen _time(const gen & a,GIAC_CONTEXT){
    if (a.type==_VECT && a.subtype==_SEQ__VECT)
      return double(clock())/CLOCKS_PER_SEC;
    double delta;
    int ntimes=1,i=0;
    clock_t b=clock(),e;
    for (;;){ // do it 10 times more
      for (;i<ntimes;++i){
	eval(a,contextptr);
      }
      e=clock();
      delta=double(e-b)/CLOCKS_PER_SEC;
      if (delta>1)
	break;
      if (delta>0.5) // max wait time will be 2 seconds
	ntimes *= 2;
      else {
	if (delta>0.2) 
	  ntimes *= 5;
	else
	  ntimes *= 10;
      }
    }
    return (delta/ntimes);
  }
  const string _time_s("time");
  unary_function_eval __time(&_time,_time_s);
  unary_function_ptr at_time (&__time,_QUOTE_ARGUMENTS,true);

  const string _phi_s("phi");
  unary_function_unary __phi(&_euler,_phi_s);
  unary_function_ptr at_phi (&__phi,0,true);

  const string _powermod_s("powermod");
  unary_function_unary __powermod(&_powmod,_powermod_s);
  unary_function_ptr at_powermod (&__powermod,0,true);

  const string _length_s("length");
  unary_function_unary __length(&_size,_length_s);
  unary_function_ptr at_length (&__length,0,true);

  const string _nops_s("nops");
  unary_function_unary __nops(&_size,_nops_s);
  unary_function_ptr at_nops (&__nops,0,true);

  gen _cat(const gen & a_orig){
    vecteur v(gen2vecteur(a_orig));
    string s;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type==_STRNG)
	s = s + *it->_STRNGptr;
      else
	s = s+it->print();
    }
    return string2gen(s,false);
  }
  const string _cat_s("cat");
  unary_function_unary __cat(&_cat,_cat_s);
  unary_function_ptr at_cat (&__cat,0,true);

  gen _pivot(const gen & a_orig,GIAC_CONTEXT){
    vecteur v(gen2vecteur(a_orig));
    int s=v.size();
    if (s!=3 && s!=4)
      setsizeerr();
    if (!ckmatrix(v.front()) || v[1].type!=_INT_ || v[2].type!=_INT_)
      settypeerr();
    matrice m=*v.front()._VECTptr;
    int ml,mc;
    mdims(m,ml,mc);
    int l=v[1].val,c=v[2].val;
    if (l<0 || l>=ml || c<0 || c>=mc)
      setsizeerr();
    gen p=m[l][c];
    int l1=0,l2=ml-1;
    if (s==4 && v[3].type==_INT_){
      l2=max(0,min(v[3].val,ml-1));
      l1=l2;
    }
    for (;l1<=l2;++l1){
      if (l1!=l)
	linear_combination(p,*m[l1]._VECTptr,-m[l1][c],*m[l]._VECTptr,plus_one,*m[l1]._VECTptr,epsilon(contextptr));
    }
    return m;
  }
  const string _pivot_s("pivot");
  unary_function_eval __pivot(&_pivot,_pivot_s);
  unary_function_ptr at_pivot (&__pivot,0,true);

  gen rowcolspace(const gen & g,bool transpose,GIAC_CONTEXT){
    if (g.type==_VECT && g._VECTptr->size()==2 && g._VECTptr->back().type!=_VECT){
      gen tmp=rowcolspace(g._VECTptr->front(),transpose,contextptr);
      if (transpose)
	sto(tmp._VECTptr->front()._VECTptr->size(),g._VECTptr->back(),contextptr);
      else
	sto(tmp._VECTptr->size(),g._VECTptr->back(),contextptr);
      return tmp;
    }
    if (!ckmatrix(g))
      setsizeerr();
    vecteur v=*g._VECTptr;
    if (transpose)
      v=mtran(v);
    v=mrref(v);
    vecteur newv;
    int s=v.size();
    vecteur cmp(v.front()._VECTptr->size());
    for (int i=0;i<s;++i){
      if (v[i]!=cmp)
	newv.push_back(v[i]);
    }
    if (transpose)
      newv=mtran(newv);
    return newv; // gen(newv,_SET__VECT);
  }
  gen _rowspace(const gen & g,GIAC_CONTEXT){
    return rowcolspace(g,false,contextptr);
  }
  const string _rowspace_s("rowspace");
  unary_function_eval __rowspace(&_rowspace,_rowspace_s);
  unary_function_ptr at_rowspace (&__rowspace,0,true);

  gen _colspace(const gen & g,GIAC_CONTEXT){
    return rowcolspace(g,true,contextptr);
  }
  const string _colspace_s("colspace");
  unary_function_eval __colspace(&_colspace,_colspace_s);
  unary_function_ptr at_colspace (&__colspace,0,true);

  const string _nullspace_s("nullspace");
  unary_function_unary __nullspace(&_ker,_nullspace_s);
  unary_function_ptr at_nullspace (&__nullspace,0,true);

  // Maple assignement does not make a distinct copy of array/table
  gen _copy(const gen & g){
    return g;
  }
  const string _copy_s("copy");
  unary_function_unary ___copy(&_copy,_copy_s);
  unary_function_ptr at_copy (&___copy,0,true);

  extern unary_function_ptr at_count;
  gen count(const gen & f,const gen & v,const context * contextptr){
    if (v.type!=_VECT)
      return f(v,contextptr);
    const_iterateur it=v._VECTptr->begin(),itend=v._VECTptr->end();
    gen res;
    for (;it!=itend;++it){
      res=res+count(f,*it,contextptr);
    }
    return res;
  }
  gen _count(const gen & args,const context * contextptr){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_count,args);
    gen v(args._VECTptr->back());
    gen f(args._VECTptr->front());
    return count(f,v,contextptr);
  }
  const string _count_s("count");
  unary_function_eval ___count(&_count,_count_s);
  unary_function_ptr at_count (&___count,0,true);

  gen count_eq(const gen & f,const gen & v,GIAC_CONTEXT,int type){
    if (v.type!=_VECT){
      switch (type){
      case 0:
	return v==f;
      case 1:
	return is_strictly_greater(v,f,contextptr);
      case 2:
	return is_strictly_greater(f,v,contextptr);
      default:
	settypeerr();
      }
    }
    const_iterateur it=v._VECTptr->begin(),itend=v._VECTptr->end();
    gen res;
    for (;it!=itend;++it){
      res=res+count_eq(f,*it,contextptr,type);
    }
    return res;
  }

  gen _count_eq(const gen & args,const context * contextptr){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_count,args);
    gen v(args._VECTptr->back());
    gen f(args._VECTptr->front());
    return count_eq(f,v,contextptr,0);
  }
  const string _count_eq_s("count_eq");
  unary_function_eval ___count_eq(&_count_eq,_count_eq_s);
  unary_function_ptr at_count_eq (&___count_eq,0,true);

  gen _count_sup(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_count,args);
    gen v(args._VECTptr->back());
    gen f(args._VECTptr->front());
    return count_eq(f,v,contextptr,1);
  }
  const string _count_sup_s("count_sup");
  unary_function_eval ___count_sup(&_count_sup,_count_sup_s);
  unary_function_ptr at_count_sup (&___count_sup,0,true);

  gen _count_inf(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_count,args);
    gen v(args._VECTptr->back());
    gen f(args._VECTptr->front());
    return count_eq(f,v,contextptr,2);
  }
  const string _count_inf_s("count_inf");
  unary_function_eval ___count_inf(&_count_inf,_count_inf_s);
  unary_function_ptr at_count_inf (&___count_inf,0,true);

  gen _trunc(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT) 
      return apply(args,contextptr,_trunc);
    if (is_strictly_positive(-args,contextptr))
      return -_floor(-args,contextptr);
    return _floor(args,contextptr);
  }
  const string _trunc_s("trunc");
  unary_function_eval ___trunc(&_trunc,_trunc_s);
  unary_function_ptr at_trunc (&___trunc,0,true);

  const string _giac_s("giac");
  unary_function_unary __giac(&_version,_giac_s);
  unary_function_ptr at_giac (&__giac,0,true);

  const string _div_s("div");
  unary_function_unary __div(&_iquo,_div_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_div (&__div,0,T_MOD);

  gen _evalc(const gen & g,GIAC_CONTEXT){
    vecteur l(lop(g,at_arg));
    gen tmp(g);
    if (!l.empty()){
      vecteur lp=*apply(l,gen_feuille)._VECTptr;
      lp=*apply(lp,contextptr,arg_CPLX)._VECTptr;
      tmp=subst(tmp,l,lp);
    }
    tmp=recursive_normal(tmp,contextptr);
    return re(tmp,contextptr)+cst_i*im(tmp,contextptr);
  }
  const string _evalc_s("evalc");
  unary_function_eval __evalc(&_evalc,_evalc_s);
  unary_function_ptr at_evalc (&__evalc,0,true);

  const string _evala_s("evala");
  unary_function_eval __evala(&recursive_normal,_evala_s);
  unary_function_ptr at_evala (&__evala,0,true);

  void find_conjugates(const gen & g,vecteur & v_in,vecteur & v_out){
    v_in.clear();
    v_out.clear();
    vecteur v1(lop(g,at_pow));
    int n=v1.size();
    for (int i=0;i<n;++i){
      gen & tmp =v1[i]._SYMBptr->feuille;
      if (tmp.type!=_VECT)
	continue;
      vecteur & v=*tmp._VECTptr;
      if (v.size()==2 && v.back().type==_FRAC && v.back()._FRACptr->den==plus_two){
	v_in.push_back(v1[i]);
	v_out.push_back(-v1[i]);
	break ; // change only 1 sqrt
      }
    }
  }

  gen _mult_conjugate(const gen & g){
    gen n,d;
    vecteur v_in,v_out;
    prod2frac(g,v_in,v_out);
    n=_prod(v_in);
    d=_prod(v_out);
    find_conjugates(d,v_in,v_out);
    // search sqrt in d
    if (v_in.empty()){
      find_conjugates(n,v_in,v_out);
      gen mult=subst(n,v_in,v_out);
      n=n*mult; // n=simplify(n*mult);
      d=d*mult;
    }
    else {
      gen mult=subst(d,v_in,v_out);
      d=d*mult; // d=simplify(d*mult);
      n=n*mult;
    }
    return n/d;
  }
  const string _mult_conjugate_s("mult_conjugate");
  unary_function_unary __mult_conjugate(&_mult_conjugate,_mult_conjugate_s);
  unary_function_ptr at_mult_conjugate (&__mult_conjugate,0,true);

  gen _mult_c_conjugate(const gen & g,GIAC_CONTEXT){
    // cerr << g << endl;
    gen n,d;
    vecteur v_in,v_out;
    prod2frac(g,v_in,v_out);
    n=_prod(v_in);
    d=_prod(v_out);
    if (!is_zero(im(d,contextptr))){
      gen mult=conj(d,contextptr);
      n=symbolic(at_prod,gen(makevecteur(n,mult),_SEQ__VECT));
      d=symbolic(at_prod,gen(makevecteur(d,mult),_SEQ__VECT));
    }
    else {
      if (!is_zero(im(n,contextptr))){
	gen mult=conj(n,contextptr);
	n=symbolic(at_prod,gen(makevecteur(n,mult),_SEQ__VECT));
	d=symbolic(at_prod,gen(makevecteur(d,mult),_SEQ__VECT));
      }
    }
    return n/d;
  }
  const string _mult_c_conjugate_s("mult_c_conjugate");
  unary_function_eval __mult_c_conjugate(&_mult_c_conjugate,_mult_c_conjugate_s);
  unary_function_ptr at_mult_c_conjugate (&__mult_c_conjugate,_QUOTE_ARGUMENTS,true);

  // open a file, returns a FD
  gen _open(const gen & g){
    check_secure();
    gen filename(g);
    if (filename.type!=_STRNG)
      setsizeerr();
    int access=O_RDWR | O_CREAT | O_TRUNC | O_APPEND ;
    int res=open(filename._STRNGptr->c_str(),access);
    if (res==-1)
      setsizeerr("Could not open file");
    gen r(res);
    r.subtype=_INT_FD;
    return r;
  }
  const string _open_s("open");
  unary_function_unary __open(&_open,_open_s);
  unary_function_ptr at_open (&__open,0,true);

  // open a file, returns a FILE *
  gen _fopen(const gen & g){
    check_secure();
    gen filename(g);
    if (filename.type!=_STRNG)
      setsizeerr();
    FILE * f=fopen(filename._STRNGptr->c_str(),"w+");
    return gen((void *) f,_FILE_POINTER);
  }
  const string _fopen_s("fopen");
  unary_function_unary __fopen(&_fopen,_fopen_s);
  unary_function_ptr at_fopen (&__fopen,0,true);

  // fprint first arg=FD, rest is printed
  gen _fprint(const gen & g){
    check_secure();
    if (g.type!=_VECT || g._VECTptr->size()<1)
      setsizeerr("1st arg=open result, then other args");
    vecteur & v=*g._VECTptr;
    int s=v.size();
    FILE * f=0;
    if (v[0].type==_INT_ && v[0].subtype==_INT_FD)
      f= fdopen(v[0].val,"a");
    if (v[0].type==_POINTER_ && v[0].subtype==_FILE_POINTER)
      f=(FILE *) v[0]._POINTER_val;
    if (f){
      if (s>1 && v[1]==gen("Unquoted")){
	for (int i=2;i<s;++i)
	  fprintf(f,"%s",unquote(v[i].print()).c_str());
      }
      else {
	for (int i=1;i<s;++i)
	  fprintf(f,"%s",v[i].print().c_str());
      }
      // fclose(f);
      return plus_one;
    }
    return zero;
  }
  const string _fprint_s("fprint");
  unary_function_unary __fprint(&_fprint,_fprint_s);
  unary_function_ptr at_fprint (&__fprint,0,true);

  gen _close(const gen & g){
    if (g.type==_INT_ && g.subtype==_INT_FD){
      close(g.val);
      return plus_one;
    }
    if (g.type==_POINTER_){
      fclose((FILE *)g._POINTER_val);
      return plus_one;
    }
    return zero;
  }
  const string _close_s("close");
  unary_function_unary __close(&_close,_close_s);
  unary_function_ptr at_close (&__close,0,true);

  const string _fclose_s("fclose");
  unary_function_unary __fclose(&_close,_fclose_s);
  unary_function_ptr at_fclose (&__fclose,0,true);

  gen _blockmatrix(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=3)
      settypeerr();
    vecteur & v = *g._VECTptr;
    if (v[0].type!=_INT_ || v[1].type!=_INT_ || v[2].type!=_VECT)
      setsizeerr();
    int n=v[0].val,m=v[1].val;
    vecteur l=*v[2]._VECTptr;
    int s=l.size();
    if (n<=0 || m<=0 || n*m!=s)
      setdimerr();
    for (int k=0;k<s;++k){
      if (!ckmatrix(l[k])){
	l[k]=mtran(vecteur(1,l[k]));
	if (!ckmatrix(l[k]))
	  settypeerr();
      }
    }
    matrice res;
    int pos=0;
    for (int i=0;i<n;++i){
      // glue m matrices from l vertically, these matrices must have same #rows
      pos=i*m;
      int nrows=l[pos]._VECTptr->size();
      for (int j=1;j<m;++j){
	if (l[pos+j]._VECTptr->size()!=nrows)
	  setdimerr();
      }
      for (int k=0;k<nrows;++k){
	// glue row k of l[pos+j] for j=0..m
	vecteur tmp;
	for (int j=0;j<m;++j){
	  gen cur_row = (*l[pos+j]._VECTptr)[k];
	  const_iterateur it=cur_row._VECTptr->begin(),itend=cur_row._VECTptr->end();
	  for (;it!=itend;++it)
	    tmp.push_back(*it);
	}
	res.push_back(tmp);
      }
    }
    return res;
  }
  const string _blockmatrix_s("blockmatrix");
  unary_function_unary __blockmatrix(&_blockmatrix,_blockmatrix_s);
  unary_function_ptr at_blockmatrix (&__blockmatrix,0,true);

  gen delrowscols(const gen & g,bool isrow){
    gen gm,interval,f,fa,fb;
    if (g.type!=_VECT || g._VECTptr->size()!=2 || !ckmatrix(gm=g._VECTptr->front()) || !(interval=g._VECTptr->back()).is_symb_of_sommet(at_interval) || (f=interval._SYMBptr->feuille).type!=_VECT || f._VECTptr->size()!=2 || (fa=f._VECTptr->front()).type!=_INT_ || (fb=f._VECTptr->back()).type!=_INT_)
      settypeerr();
    int a=fa.val-(maple_mode!=0),b=fb.val-(maple_mode!=0),s;
    matrice m=*gm._VECTptr;
    if (!isrow)
      m=mtran(m);
    s=m.size();
    if (a>=s || b>=s || a<0 || b<0 || a>b)
      setdimerr();
    m.erase(m.begin()+a,m.begin()+b+1);
    if (!isrow)
      m=mtran(m);
    return m;
  }

  gen _delrows(const gen & g){
    return delrowscols(g,true);
  }
  const string _delrows_s("delrows");
  unary_function_unary __delrows(&_delrows,_delrows_s);
  unary_function_ptr at_delrows (&__delrows,0,true);

  gen _delcols(const gen & g){
    return delrowscols(g,false);
  }
  const string _delcols_s("delcols");
  unary_function_unary __delcols(&_delcols,_delcols_s);
  unary_function_ptr at_delcols (&__delcols,0,true);

  const string _gaussjord_s("gaussjord");
  unary_function_unary __gaussjord(&_rref,_gaussjord_s);
  unary_function_ptr at_gaussjord (&__gaussjord,0,true);

  gen _row(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    gen res=g._VECTptr->front()[g._VECTptr->back()-(maple_mode!=0)];
    if (ckmatrix(res))
      return gen(*res._VECTptr,_SEQ__VECT);
    else
      return res;
  }
  const string _row_s("row");
  unary_function_unary __row(&_row,_row_s);
  unary_function_ptr at_row (&__row,0,true);

  gen _col(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    gen res=_tran(g._VECTptr->front())[g._VECTptr->back()-(maple_mode!=0)];
    if (ckmatrix(res))
      return gen(*res._VECTptr,_SEQ__VECT);
    else
      return res;
  }
  const string _col_s("col");
  unary_function_unary __col(&_col,_col_s);
  unary_function_ptr at_col (&__col,0,true);

  gen _JordanBlock(const gen & g){
    int s;
    if (g.type!=_VECT || g._VECTptr->size()!=2 || g._VECTptr->back().type!=_INT_ )
      settypeerr();
    s=g._VECTptr->back().val;
    if (s<=0)
      setdimerr();
    --s;
    gen x=g._VECTptr->front();
    matrice m;
    m.reserve(s+1);
    for (int i=0;i<=s;++i){
      vecteur v(s+1);
      v[i]=x;
      if (i<s)
	v[i+1]=plus_one;
      m.push_back(v);
    }
    return m;
  }
  const string _JordanBlock_s("JordanBlock");
  unary_function_unary __JordanBlock(&_JordanBlock,_JordanBlock_s);
  unary_function_ptr at_JordanBlock (&__JordanBlock,0,true);

  gen _companion(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      settypeerr();
    gen P=g._VECTptr->front();
    gen x=g._VECTptr->back();
    gen Px=_e2r(makevecteur(P,x));
    if (Px.type!=_VECT)
      setsizeerr();
    vecteur & v=*Px._VECTptr;
    int s=v.size()-1;
    if (s<=0)
      setdimerr();
    matrice m;
    m.reserve(s);
    for (int i=0;i<s;++i){
      vecteur w(s);
      w[s-1]=-v[s-i];
      if (i>0)
	w[i-1]=plus_one;
      m.push_back(w);
    }
    return m;
  }
  const string _companion_s("companion");
  unary_function_unary __companion(&_companion,_companion_s);
  unary_function_ptr at_companion (&__companion,0,true);

  gen _border(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2 || !ckmatrix(g._VECTptr->front()) || g._VECTptr->back().type!=_VECT)
      setsizeerr();
    matrice m=*g._VECTptr->front()._VECTptr,v=*g._VECTptr->back()._VECTptr;
    if (m.size()!=v.size())
      setdimerr();
    m=mtran(m);
    m.push_back(v);
    return mtran(m);
  }
  const string _border_s("border");
  unary_function_unary __border(&_border,_border_s);
  unary_function_ptr at_border (&__border,0,true);

  // pade(f(x),x,n,p) or pade(f(x),x,N(x),p)
  // Find P/Q such that P/Q=f(x) mod N(x) [1st case N(x)=x^n], deg(P)<p
  // Assume deg(f)<n
  gen _pade(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=4)
      setsizeerr();
    vecteur v =*g._VECTptr;
    if (v[1].type!=_IDNT || v[3].type!=_INT_)
      setsizeerr();
    int n,p=v[3].val,ns;
    gen f=v[0],x=v[1],N=v[2];
    if (v[2].type==_INT_){
      n=v[2].val+1;
      N=pow(v[1],n);
      ns=n;
    }
    else {
      n=p;
      ns=_degree(makevecteur(N,x)).val;
    }
    f=_series(makevecteur(f,x,zero,ns-1));
    vecteur l1(lop(f,at_order_size));
    vecteur l2(l1.size(),zero);
    f=subst(f,l1,l2);
    // Convert f and N to poly1
    vecteur l(1,x);
    lvar(f,l);
    lvar(N,l);
    int ls=l.size();
    gen ff(sym2r(f,l));
    gen fn,fd;
    fxnd(ff,fn,fd);
    gen Nf(sym2r(N,l));
    gen Nn,Nd;
    fxnd(Nf,Nn,Nd); // Note nd has no meaning
    vecteur fp;
    if (fn.type==_POLY)
      fp=polynome2poly1(*fn._POLYptr,1);
    else
      setsizeerr();
    vecteur Np;
    if (Nn.type==_POLY)
      Np=polynome2poly1(*Nn._POLYptr,1);
    else
      setsizeerr();
    n=Np.size()-1;
    // Check that fp is a poly of degree less than n
    if (n<1 || p>n || fp.size()>n)
      setdimerr();
    vecteur a,b;
    egcd_pade(Np,fp,p,a,b,0);
    gen res=poly12polynome(a,1,ls);
    res=res/(fd*poly12polynome(b,1,ls));
    res=r2sym(res,l);
    return res;
  }
  const string _pade_s("pade");
  unary_function_unary __pade(&_pade,_pade_s);
  unary_function_ptr at_pade (&__pade,0,true);

  const string _interp_s("interp");
  unary_function_unary __interp(&_lagrange,_interp_s);
  unary_function_ptr at_interp (&__interp,0,true);

  gen lhsrhs(const gen & g,int i){
    if (!g.is_symb_of_sommet(at_equal) && !g.is_symb_of_sommet(at_interval))
      setsizeerr();
    gen & f=g._SYMBptr->feuille;
    if (f.type!=_VECT || f._VECTptr->size()!=2)
      setsizeerr();
    return (*f._VECTptr)[i];
  }
  gen _lhs(const gen & g){
    return lhsrhs(g,0);
  }
  const string _lhs_s("lhs");
  unary_function_unary __lhs(&_lhs,_lhs_s);
  unary_function_ptr at_lhs (&__lhs,0,true);

  gen _rhs(const gen & g){
    return lhsrhs(g,1);
  }
  const string _rhs_s("rhs");
  unary_function_unary __rhs(&_rhs,_rhs_s);
  unary_function_ptr at_rhs (&__rhs,0,true);

  // Given [v_0 ... v_(2n-1)] (begin of the recurrence sequence) 
  // return [b_n...b_0] such that b_n*v_{n+k}+...+b_0*v_k=0
  // Example [1,-1,3,3] -> [1,-3,-6]
  // -> the recurrence relation is v_{n+2}=3v_{n+1}+6v_n
  gen _reverse_rsolve(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur v=reverse_rsolve(*g._VECTptr);
    return v/v.front();
  }
  const string _reverse_rsolve_s("reverse_rsolve");
  unary_function_unary __reverse_rsolve(&_reverse_rsolve,_reverse_rsolve_s);
  unary_function_ptr at_reverse_rsolve (&__reverse_rsolve,0,true);

  // Approx fft
  gen fft(const gen & g_orig,int direct){
    gen g=evalf_double(g_orig);
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v =*g._VECTptr;
    int n=v.size();
    unsigned m=gen(n).bindigits()-1;
    if (n!=1<<m)
      setsizeerr("Size is not a power of 2 "+print_INT_(n));
    vecteur res;
    double theta;
    vecteur w(n);
    iterateur it=w.begin(),itend=w.end();
    for (int i=0;it!=itend;++it,++i){
      theta = (2*M_PI*i)/n;
      if (direct==1)
	*it = gen(std::cos(theta),-std::sin(theta));
      else
	*it = gen(std::cos(theta),std::sin(theta));
    }
    fft(v,w,res,0);
    if (direct==1)
      return res;
    else
      return gen(res)/n;
  }

  gen _fft(const gen & g){
    return fft(g,1);
  }
  const string _fft_s("fft");
  unary_function_unary __fft(&_fft,_fft_s);
  unary_function_ptr at_fft (&__fft,0,true);

  gen _ifft(const gen & g){
    return fft(g,0);
  }
  const string _ifft_s("ifft");
  unary_function_unary __ifft(&_ifft,_ifft_s);
  unary_function_ptr at_ifft (&__ifft,0,true);

  gen _Resultant(const gen & g){
    return symbolic(at_resultant,g);
  }
  const string _Resultant_s("Resultant");
  unary_function_unary __Resultant(&_Resultant,_Resultant_s);
  unary_function_ptr at_Resultant (&__Resultant,0,true);

  gen _Nullspace(const gen & g){
    return symbolic(at_nullspace,g);
  }
  const string _Nullspace_s("Nullspace");
  unary_function_unary __Nullspace(&_Nullspace,_Nullspace_s);
  unary_function_ptr at_Nullspace (&__Nullspace,0,true);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
