// -*- mode:C++ ; compile-command: "g++ -I.. -g -c mathml.cc" -*-

#include "mathml.h"
#include "gen.h"
#include "symbolic.h"
#include "usual.h"
#include "intg.h"
#include "derive.h"
#include "series.h"
#include "plot.h"
#include "tex.h"

#include <fcntl.h>
#include <unistd.h>
#include <cstdlib>
#include <cstdio>
#include <iomanip>
#include <math.h>

#ifdef HAVE_SSTREAM
#include <sstream>
#else
#include <strstream>
#endif
using namespace std;

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

#ifndef NO_NAMESPACE_GIAC
  using namespace giac;
#endif // ndef NO_NAMESPACE_GIAC

  // a supprimer srement
  double horiz_mathml=12.;
  double vert_mathml=12.;

  string mathml_preamble("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN\"\n\"http://www.w3.org/TR/MathML2/dtd/xhtml-math11-f.dtd\" [\n<!ENTITY mathml \"http://www.w3.org/1998/Math/MathML\">\n]>\n<html xmlns=\"http://www.w3.org/1999/xhtml\">\n<body>\n");
  string mathml_end("\n</body>\n</html>");
  string math_begin("<math mode=\"display\" xmlns=\"http://www.w3.org/1998/Math/MathML\">");
  string math_end("</math>");
  string provisoire_mbox_begin("<mi>");
  string provisoire_mbox_end("</mi>");
  string svg_legend("");
  string svg_end("</svg>");

  // 2 fonctions de service
  void elimine_inf_sup(string &s){
    string t="";
    while (t!=s){
      t=s;
      int i=s.find("<"), j=s.find(">");
      if (i!=-1)
	s.replace(s.find("<"),1,"&lt;");
      if (j!=-1)
	s.replace(s.find(">"),1,"&gt;");
    }
  }
 
  void delete_cr(string & s){
    for (int i=0 ; i<s.length(); i++)
      if (s[i]=='\n') 
	s[i]=' ';
  }


  string spread2mathml(const matrice & m,int formule){
    int l=m.size();
    if (!l)
      return "<p>empty_spread</p>";
    int c=m.front()._VECTptr->size();
    string percent="", percent_first="";
    if (c<=50){
      percent="width=\""+print_INT_(200/(2*c+1))+"%\"";
      percent_first="width=\""+print_INT_(100/(2*c+1))+"%\""; 
    }
    string s="<table border=\"1\"><tbody>";
    string tmp;
    for (int k=0;;){ // write first line
      int i=k;
      for(int j=0;;++j){
	tmp+="<td bgcolor=\"lightGray\" "+percent+">";
	tmp+=char('A'+i%26-(j!=0));
	tmp+="</td>";
	//	tmp="<td>"+char('A'+i%26-(j!=0))+"</td>"+tmp;
	i=i/26;
	if (!i)
	  break;
      } // end for j
      ++k;
      if (k==c)
	break;
    } // end for k
    s += "<tr align=\"center\"><td bgcolor=\"lightGray\" "+percent_first+"> &nbsp; &nbsp; &nbsp;</td>"
      +tmp+"</tr>";
    for (int i=0;i<l;++i){
      s +="<tr align=\"center\"><td  bgcolor=\"lightGray\">"+ print_INT_(i)+"</td>";
      for (int j=0;j<c;++j){ 
	if (formule)
	  s += "<td>"+math_begin+gen2mathml(m[i][j][formule])+math_end+"</td>" ;
	else {
	  int save_r=printcell_current_row,save_c=printcell_current_col;
	  printcell_current_row=i,printcell_current_col=j;
	  string t(m[i][j][0].print()),tt("<td>");
	  int ll=t.size();
	  for (int l=0;l<ll;++l){
	    if (t[l]=='<')
	      tt +=  "&lt;";
	    else  if (t[l]=='>')
	      tt += "&gt;";
	    else  if (t[l]=='&')
	      tt += "&amp;";
	    else
	      tt += t[l];
	  } // end for l
	  s += tt+"</td>";
	  printcell_current_row=save_r;printcell_current_col=save_c;
	} // end if (formule)
      } // end for j
      s +="</tr>";
    } // end for i
    s += "</tbody></table>";
    return s;
  }   

  string matrix2mathml(const matrice & m){
    int l=m.size();
    if (!l)
      return string("()");
    int c=m.front()._VECTptr->size();
    string s("<mrow><mo>(</mo><mtable>");
    for (int i=0;i<l;++i){
      s+="<mtr>"; 
      for (int j=0;j<c;++j){
	s += "<mtd>"+gen2mathml(m[i][j])+"</mtd>" ;
      }
      s+= "</mtr>"; 
    }
    s += "</mtable><mo>)</mo></mrow>";
    return s;
  }

  string _VECT2mathml(const vecteur & v, string &svg){ 
    string s("<mo>[</mo>");
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;){
      s += gen2mathml(*it,svg);  
      ++it;
      if (it!=itend)
	s += "<mo>,</mo>";
    }
    s +="<mo>]</mo>";
    return s;
  }

  bool needs_times(gen g){
    if (eval(g).is_constant())
      return true;
    else if (g.type==_SYMB) {
      symbolic symb = *(g._SYMBptr);
      if ( symb.sommet.ptr->print()=="/" )
	return true;
    }
    else
      return false;
  }

  string prod_vect2mathml(const vecteur & v){
    if (v.empty())
      return "<mn>1</mn>";
    vecteur::const_iterator it=v.begin(),itend=v.end();
    string s;
    for (;;){
      if ( (it->type==_SYMB) && ( (it->_SYMBptr->sommet==at_plus) || (it->_SYMBptr->sommet==at_neg))) 
	s += string(" <mo>(</mo>")+gen2mathml(*it)+string(" <mo>)</mo>");
      else if (it->type==_CPLX){
	if  (!is_zero(re(*it,0)) && !is_zero(im(*it,0)))
	  s += "<mo>(</mo>"+gen2mathml(*it)+"<mo>)</mo>";
	else
	  s += gen2mathml(*it);
      }
      else 
	s += gen2mathml(*it);
      ++it;
      if (it==itend)
	return s;
      else if (needs_times(*it))
	s += "<mo>*</mo>";  
    }
  }


  string prod_vect2mathml_no_bra(const vecteur & v){
    if (v.empty())
      return "<mn>1</mn>";
    vecteur::const_iterator it=v.begin(),itend=v.end();
    if (v.size()==1)
      return gen2mathml(*it);
    string s;
    for (;;){
      if ( (it->type==_SYMB) && ( (it->_SYMBptr->sommet==at_plus) || (it->_SYMBptr->sommet==at_neg)))
	s += string(" <mo>(</mo>")+gen2mathml(*it)+string(" <mo>)</mo>");
      else if (it->type==_CPLX){
	if  (!is_zero(re(*it,0)) && !is_zero(im(*it,0)))
	  s += "<mo>(</mo>"+gen2mathml(*it)+"<mo>)</mo>";
	else
	  s += gen2mathml(*it);
      }
      else 
	s += gen2mathml(*it);
      ++it;
      if (it==itend)
	return s;
      else if (needs_times(*it))
	s += "<mo>*</mo>";  
    }
  }





  // // --------------------------------- recopi de unary.cc ---------------------------------------

  string mathml_printsommetasoperator(const gen & feuille, const string & sommetstr){
    if (feuille.type!=_VECT)
      return "<mi>"+feuille.print()+"</mi>"; 
    vecteur::const_iterator itb=feuille._VECTptr->begin(),itend=feuille._VECTptr->end();
    if (itb==itend)
      return "";
    string s;
    if (itb->type==_FRAC)
      s=gen2mathml(*itb); 
    else {
      if ( (itb->type==_IDNT) || ((itb->type!=_SYMB) && is_positive(*itb,0)) || sommetstr=="<mo>=</mo>")
	s=gen2mathml(*itb);
      else
	s="<mo>(</mo>"+gen2mathml(*itb)+"<mo>)</mo>";
    }
    ++itb;
    for (;;){
      if (itb==itend)
	return s;
      if ((itb->type!=_SYMB) || sommetstr=="<mo>=</mo>")
	s += sommetstr + gen2mathml(*itb);
      else
	s += sommetstr + "<mo>(</mo>"+gen2mathml(*itb)+"<mo>)</mo>";
      ++itb;
    }
  }
  // ------------------------------- fin provenant de unary.cc -------------------------------------



  // // ------------------------------- provenant de usual.cc ----------------------------------
  string mathml_printasexp(const gen & g){
    return "<msup><mi>e</mi><mrow>"+gen2mathml(g)+"</mrow></msup>";
  }

  string mathml_printassqrt(const gen & g){
    return   "<msqrt>"+gen2mathml(g)+"</msqrt>";
  }

  //-------------- inutile ? -------------------------
  string mathml_printassqr(const gen & g){
    return "<msup><mrow>"+gen2mathml(g)+"</mrow><mn>2</mn></msup>";
  }

  string mathml_printasre(const gen & g){
    return "<mi>Re</mi><mrow><mo>(</mo>"+gen2mathml(g)+"<mo>)</mo></mrow>";   
  }

  string mathml_printasim(const gen & g){
    return "<mi>Im</mi><mrow><mo>(</mo>"+gen2mathml(g)+"<mo>)</mo></mrow>";   
  }

  // overline en mathml ????????????
  // string texprintasconj(const gen & g,const string & s){
  //   return "\\overline{"+gen2tex(g)+"}";
  // }



  string mathml_printassto(const gen & g,const string & sommetstr){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return sommetstr+"<mo>(</mo>"+gen2mathml(g)+"<mo>)</mo>";
    string s(gen2mathml(g._VECTptr->back())+"<mi>:=</mi>");
    if (g._VECTptr->front().type==_SEQ__VECT)
      return s+"<mo>(</mo>"+gen2mathml(g._VECTptr->front())+"<mo>)</mo>";
    else
      return s+gen2mathml(g._VECTptr->front());
  }



  // --------------------------- fin de provenant de usual.cc ----------------------------------

  // n'existe pas avec tex :
  string mathml_equal2arrow(gen &g){
    if (g._SYMBptr->sommet!=at_equal)
      return gen2mathml(g);
    vecteur v=*g._SYMBptr->feuille._VECTptr;
    return "<mrow>"+gen2mathml(v[0])+"<mo>&rarr;</mo>"+gen2mathml(v[1])+"</mrow>";
  
  }

  // ----------------------------- provenant de series.cc ----------------------------------
  string mathml_printaslimit(const gen & g){
 
    string s("<mo>lim</mo>");
    if (g.type!=_VECT)
      return s+gen2mathml(g);
    vecteur v(*g._VECTptr);
    int l(v.size());
    if (!l)
      return s;
    if (l==1)
      return s+gen2mathml(v[0]);
    if (l==2)
      return "<munder>"+s+mathml_equal2arrow(v[1])+"</munder>"+gen2mathml(eval(v[0]));
    // directional limit
    if (l==3){
      if (is_one(v[2]))
	return "<munder>"+s+"<mrow><msup>"+mathml_equal2arrow(v[1])+"<mo>+</mo></msup></mrow></munder>"
	  +gen2mathml(eval(v[0]));
      if (is_zero(eval(v[2]+1))) 
	return "<munder>"+s+"<mrow><msup>"+mathml_equal2arrow(v[1])+"<mo>-</mo></msup></mrow></munder>"
	  +gen2mathml(eval(v[0]));
      else  
	return "<munder>"+s+mathml_equal2arrow(v[1])+"</munder>"+gen2mathml(eval(v[0]));
    }
    return s;
  }
  // ----------------------------- fin provenant de series.cc ----------------------------------

  // ----------------------------- provenant de intg.cc ----------------------
  string mathml_printasintegrate(const gen g){
    string s("<mo>&int;</mo>");
    if (g.type!=_VECT)
      return s+gen2mathml(g);
    vecteur v(*g._VECTptr);
    int l(v.size());
    if (!l)
      return s;
    if (l==1)
      return s+gen2mathml(v.front());
    if (l==2)
      return s+gen2mathml(v.front())+"<mi>d</mi>"+gen2mathml(v.back());
    if (l==4)
      return "<msubsup>"+s+"<mi>"+v[2].print()+"</mi>"
	+"<mi>"+v[3].print()+"</mi></msubsup>"+gen2mathml(eval(v.front()))
	+"<mi>d</mi><mi>"+v[1].print()+"</mi>";
    return s;
  } 
  // ---------------------------- fin provenant de intg.cc ----------------------



  // --------------------------- provenant de derive ----------------------------

  string mathml_printasderive(const gen & feuille){
    if (feuille.type!=_VECT)
      return "<msup><mrow><mo>(</mo>"+gen2mathml(feuille)+"<mo>)</mo></mrow><mi>'</mi></msup>";
    return "<mfrac><mrow><mo>&part;</mo><mo>(</mo>"+gen2mathml(feuille._VECTptr->front())
      +"<mo>)</mo></mrow><mrow><mo>&part;</mo>"+gen2mathml(feuille._VECTptr->back())+"</mrow></mfrac>";
  }
  // ----------------------- fin provenant de derive ---------------------------

  string mathml_print(const symbolic & mys){
    if ("\'"+mys.sommet.ptr->s+"\'"==at_equal)
      return mathml_printsommetasoperator(mys.feuille,"<mo>=</mo>");  
    if ("\'"+mys.sommet.ptr->s+"\'"==at_inferieur_egal)
      return mathml_printsommetasoperator(mys.feuille,"<mo>&le;</mo>");
    if ("\'"+mys.sommet.ptr->s+"\'"==at_superieur_egal)
      return mathml_printsommetasoperator(mys.feuille,"<mo>&ge;</mo>");  
    if ("\'"+mys.sommet.ptr->s+"\'"==at_inferieur_strict)
      return mathml_printsommetasoperator(mys.feuille,"<mo>&lt;</mo>");  
    if ("\'"+mys.sommet.ptr->s+"\'"==at_superieur_strict)
      return mathml_printsommetasoperator(mys.feuille,"<mo>&gt;</mo>");
    if ("\'"+mys.sommet.ptr->s+"\'"==at_re)
      return mathml_printasre(mys.feuille);
    if ("\'"+mys.sommet.ptr->s+"\'"==at_im)
      return mathml_printasim(mys.feuille);
    if ("\'"+mys.sommet.ptr->s+"\'"==at_exp)
      return  mathml_printasexp(mys.feuille);
    if ("\'"+mys.sommet.ptr->s+"\'"==at_sqrt)
      return mathml_printassqrt(mys.feuille);
    if  (mys.sommet.ptr->s==at_integrate)
      return mathml_printasintegrate(mys.feuille);
    if  (mys.sommet.ptr->s==at_derive) 
      return mathml_printasderive(mys.feuille);
    if  (mys.sommet.ptr->s==at_sto) 
      return mathml_printassto(mys.feuille, "<mi>:=</mi>");
    if  (mys.sommet.ptr->s==at_limit) 
      return mathml_printaslimit(mys.feuille);
    string s=mys.print();
    elimine_inf_sup(s);
    delete_cr(s);
    return "<mtext>"+s+"</mtext>";
  }
     
  string mathml_printassum(const gen e){
    vecteur v = *e._SYMBptr->feuille._VECTptr;
    if (v.size()!=4)
      return "<mtext>"+e.print()+"</mtext>";
    return "<munderover><mi>&Sigma;</mi><mrow>"+gen2mathml(v[1])+"<mo>=</mo>"+gen2mathml(v[2])+"</mrow><mrow>"+gen2mathml(v[3])+"</mrow></munderover>"+gen2mathml(eval(v[0]));
  }

  string mathml_printasabs(const gen e){
    return  "<mo>&VerticalBar;</mo>"+gen2mathml(e._SYMBptr->feuille)+ "<mo>&VerticalBar;</mo>";
  }



  //---------------- Zone SVG  ---------------

void svg_dx_dy(double svg_width, double svg_height, int * dx, int *dy){
 // calibrage des graduations
  int p_h=(int) std::log10(svg_height);
  int p_w=(int) std::log10(svg_width);
  *dx=(int) std::pow((double)10,p_w-1);
  *dy=(int) std::pow((double)10,p_h-1);
  if (*dx==0)
    *dx=1;
 if (*dy==0)
    *dy=1;
 if (svg_width/(*dx)>25)
   *dx=5*(*dx);
 if (svg_height/(*dy)>25)
   *dy=5*(*dy);
 }


  string svg_preamble(double svg_width_cm, double svg_height_cm){
    double svg_width=gnuplot_xmax-gnuplot_xmin;
    double svg_height=gnuplot_ymax-gnuplot_ymin;
    double x_scale=(gnuplot_xmax-gnuplot_xmin)/10;
    double y_scale=(gnuplot_ymax-gnuplot_ymin)/10;
    int i;
    string grid_color="blue";
    string grid_color2="navy";
#ifdef HAVE_SSTREAM
    ostringstream sortie;
#else
    ostrstream sortie;
#endif
    if (svg_width<svg_height){
      svg_width_cm=svg_width_cm*svg_width/svg_height+1; 
      svg_height_cm=svg_height_cm+1;
    }  else {
      svg_height_cm=svg_height_cm*svg_height/svg_width+1;
      svg_width_cm=svg_width_cm+1;
    }
    sortie<<"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>"<<endl;
    sortie<<"<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.0\" preserveAspectRatio=\"xMidyMin meet\" width=\""<<svg_width_cm+2<<"cm\" height=\""<<svg_height_cm+1<<"cm\" viewBox=\""<<gnuplot_xmin-x_scale<<" "<<gnuplot_ymin-y_scale<<" "<<svg_width+3*x_scale<<" "<<svg_height+2*x_scale<<"\" >\n<g transform=\"translate(0,"<<svg_height+2*gnuplot_ymin<<") scale(1,-1)\">\n";


    // calibrage des graduations
    int  dx, dy;
    svg_dx_dy(svg_width, svg_height, &dx, &dy);
    double i_min_x= gnuplot_xmin/dx;
    double i_min_y= gnuplot_ymin/dy;
  
    //grille  
    double x,y;
    //gen thickness((gnuplot_xmax-gnuplot_xmin)/1000);
    double thickness((gnuplot_xmax+gnuplot_ymax-gnuplot_xmin-gnuplot_ymin)/2000);
    
    for (i=(int) i_min_x; i*dx<gnuplot_xmax; i++){
      x=i*dx;
      sortie<<"<line x1=\""<<x <<"\" y1=\""<<gnuplot_ymin <<"\" x2=\""<<x <<"\" y2=\""<<gnuplot_ymax;
      if(i%5==0)
	sortie<<"\" stroke=\""<<grid_color2<<"\"  stroke-width=\""<<2*thickness<<"\" />"<<endl;
      else   
	sortie<<"\" stroke=\""<<grid_color<<"\"  stroke-dasharray=\""
	      <<10*thickness<<","<<10*thickness
	      <<"\" stroke-width=\""<<thickness<<"\" />"<<endl;
    }

    for (i=(int) i_min_y; i*dy<gnuplot_ymax; i++){
      y=i*dy;
      sortie<<"<line x1=\""<<gnuplot_xmin <<"\" y1=\""<<y <<"\" x2=\""<<gnuplot_xmax <<"\" y2=\""<<y;
      if (i%5==0)
	sortie<<"\" stroke=\""<<grid_color2<<"\"  stroke-width=\""<<2*thickness<<"\" />"<<endl;
      else
	sortie<<"\" stroke=\""<<grid_color<<"\"  stroke-dasharray=\""
	      <<10*thickness<<","<<10*thickness
	      <<"\" stroke-width=\""<<thickness<<"\" />"<<endl;
    }

    //cadre
    sortie<<"<rect stroke=\""<<"black"<<"\"  stroke-width=\""<<2*thickness<<"\" fill=\"none\" x=\""<<gnuplot_xmin<<"\" y=\""<<gnuplot_ymin<<"\" width=\""<<svg_width<<"\" height=\""<<svg_height<<"\" />"<<endl;  
    return sortie.str();
  }

  string svg_grid(){
    double svg_width=gnuplot_xmax-gnuplot_xmin;
    double svg_height=gnuplot_ymax-gnuplot_ymin;
#ifdef HAVE_SSTREAM
    ostringstream sortie;
#else
    ostrstream sortie;
#endif
    string s;

    // 4 rectangles to mask the borders
    double x_scale=(gnuplot_xmax-gnuplot_xmin)/10;
    double y_scale=(gnuplot_ymax-gnuplot_ymin)/10;
    sortie<<"<rect x=\""<<gnuplot_xmin-x_scale<<"\" y=\""<<gnuplot_ymax<<"\" height=\""<<y_scale<<"\" width=\""<<svg_width+2*x_scale<<
      "\" stroke=\"none\" fill=\"white\"/>"<<endl;
    sortie<<"<rect x=\""<<gnuplot_xmin-x_scale<<"\" y=\""<<gnuplot_ymin<<"\" width=\""<<x_scale <<"\" height=\""<<svg_height<<
      "\" stroke=\"none\" fill=\"white\"/>"<<endl;
    sortie<<"<rect x=\""<<gnuplot_xmax<<"\" y=\""<<gnuplot_ymin<<"\" width=\""<<x_scale*2 <<"\" height=\""<<svg_height<<
      "\" stroke=\"none\" fill=\"white\"/>"<<endl;
    sortie<<"<rect x=\""<<gnuplot_xmin-x_scale<<"\" y=\""<<gnuplot_ymin-y_scale<<"\" width=\""<<svg_width+2*x_scale <<
      "\" height=\""<<y_scale <<"\" stroke=\"none\" fill=\"white\"/>\n</g>"<<endl;

    // calibrage des graduations
    int dx,dy;
    svg_dx_dy(svg_width, svg_height, & dx, &dy);
    double x,y;
    int i;
    double i_min_x= gnuplot_xmin/dx;
    double i_min_y= gnuplot_ymin/dy;
    int di_x=1;
    int di_y=1;
    if ((gnuplot_xmax-gnuplot_xmin)/dx>9)
      di_x=5;
    if ((gnuplot_ymax-gnuplot_ymin)/dx>9)
      di_y=5;
    // index des graduations 
    for (i=(int) i_min_x;  i*dx<=gnuplot_xmax ; i=i+di_x){
      x=i*dx;
      sortie<<setprecision(5)<<"<text x=\""<<x<<"\" y=\""<<gnuplot_ymax+0.6*y_scale<<"\" style=\"font-size:"<<0.3*x_scale<<"pt; text-anchor:middle;\">"<<(float)x<<"</text>"<<endl;
    } 
    for (i=(int) i_min_y;  i*dy<=gnuplot_ymax ; i=i+di_y){
      y=i*dy;
      sortie<<setprecision(5)<<"<text x=\""<<gnuplot_xmax+0.2*x_scale<<"\" y=\""<<gnuplot_ymax+gnuplot_ymin-y+0.1*y_scale<<"\" style=\"font-size:"<<0.3*x_scale<<"pt\">"<<(float)y<<"</text>"<<endl;
    }
    return string(sortie.str())+"\n";
  }


  string color_string(int color){
    switch (color) {
    case 0:
      return "black";
    case 1:
      return "red";
    case 2:
      return "green";
    case 3:
      return "yellow";
    case 4:
      return "blue";
    }
    return "black";
  }

  int color_int(string color){
    if (color=="black" || color=="noir")
      return 0;
    if (color=="red" || color=="rouge")
      return 1;
    if (color=="green" || color=="vert")
      return 2;
    if (color=="yellow" || color=="jaune")
      return 3;
    if (color=="blue" || color=="bleu")
      return 4;
    return 0;
  }

 
  void svg_text(gen A, string legende, int color){
    static gen right_space((gnuplot_xmax-gnuplot_xmin)/50);
    static gen text_size(0.3*(gnuplot_ymax-gnuplot_ymin)/10);
    if (legende=="")
      return;
    // remove " from legende
    int end=legende.size()-1;
    if (legende[0]=='\"')
      legende=legende.substr(1,end-1);
    gen x=re(A,0)-right_space;
    gen y(gnuplot_ymax+gnuplot_ymin-im(A,0));
    if (x==gnuplot_xmax){
      // A traiter, la lgende sort du cadre
    }
    svg_legend=svg_legend+ "<text  fill=\"" +color_string(color)+"\"  x=\""+x.print()+"\" y=\""+y.print()
      +"\" style=\"font-size:"+text_size.print()+"pt; text-anchor:end;\">"
      +legende
      +"</text>\n";                                      
  }

  string svg_circle(gen diameter0, gen diameter1, int color, string legende){
    string s="";
    gen center=evalf((diameter0+diameter1)/2);
    gen radius=evalf(abs(diameter1-diameter0)/2);
    gen thickness((gnuplot_xmax+gnuplot_ymax-gnuplot_xmin-gnuplot_ymin)/500);
    s=  "<circle stroke=\""+color_string(color)+"\"  stroke-width=\""+thickness.print()+"\" fill=\"none\" cx=\""
      +re(center,0).print()+"\" cy=\""
      +im(center,0).print()+"\" r=\""
      +radius.print()
      +"\" />\n";
    if (legende!="")
      svg_text(evalf(center+radius),legende,color);
    return s;
  }


  string svg_segment(gen A, gen B, int  color, string legende){
    string s;
    gen thickness((gnuplot_xmax+gnuplot_ymax-gnuplot_xmin-gnuplot_ymin)/500);
    A=evalf(A); B=evalf(B);
    s= "<line  stroke-width=\""+thickness.print()+"\" stroke=\""
      +color_string(color)+"\" x1=\""
      +re(A,0).print()+"\" y1=\""
      +im(A,0).print()+"\" x2=\""
      +re(B,0).print()+"\" y2=\""
      +im(B,0).print()+"\"/>\n";
    svg_text(B,legende,color);
    return s;
  }

  string svg_point(gen center, int color, string legende){
    double svg_width=gnuplot_xmax-gnuplot_xmin;
    double svg_height=gnuplot_ymax-gnuplot_ymin;
    gen i("i");
    gen dx(svg_width/100);
    gen dy(svg_width/100);
    svg_text(center,legende,color);
    return svg_segment(center-dx-i*dy,center+dx+i*dy,color,"")+
      svg_segment(center-dx+i*dy,center+dx-i*dy,color,"");
  }

  string svg_line(gen A, gen B, int color, string legende){
    gen i("i"),xmin(gnuplot_xmin),ymin(gnuplot_ymin),xmax(gnuplot_xmax),ymax(gnuplot_ymax), C,D;
    A=evalf(A); B=evalf(B);
    // recherche de l'quation de la droite
    if (is_zero(eval(re(A,0)-re(B,0)))){
      gen x=re(A,0);
      C=x+i*ymin; D=x+i*ymax;
    } else {
      gen a=eval((im(A,0)-im(B,0))/(re(A,0)-re(B,0)));
      gen b=eval(im(A,0)-a*re(A,0));
      C=xmin+i*(a*xmin+b);
      D=xmax+i*(a*xmax+b);
    }
    return svg_segment(C,D,color,legende);
  }

  string svg_half_line(gen A, gen B, int color, string legende){
    gen i("i"),xmin(gnuplot_xmin),ymin(gnuplot_ymin),xmax(gnuplot_xmax),ymax(gnuplot_ymax), C,D;
    A=evalf(A); B=evalf(B);
    // recherche de l'quation de la droite
    if (is_zero(eval(re(A,0)-re(B,0)))){
      gen x=re(A,0);
      if (is_positive(eval(im(B,0)-im(A,0)),0))
	C=x+i*ymax;
      else
	C=x+i*ymin;
    } 
    else {
      gen a=eval((im(A,0)-im(B,0))/(re(A,0)-re(B,0)));
      gen b=eval(im(A,0)-a*re(A,0));
      if (is_positive(eval(im(B,0)-im(A,0)),0))
	C=xmax+i*(a*xmax+b);
      else
	C=xmin+i*(a*xmin+b); 
    }
    return svg_segment(A,C,color,legende);
  }


  string svg_polyline(gen g,int color, string name){
    string x="x", y="y", s;
    int i;
    gen thickness((gnuplot_xmax+gnuplot_ymax-gnuplot_xmin-gnuplot_ymin)/500);
    g=eval(g);
    vecteur v=*(g._VECTptr);
    if (v[0]==v[v.size()-1])
      s= "<polygon  stroke-width=\""+thickness.print()+"\" stroke=\""+color_string(color)+"\" fill=\"none\" points=\"";
    else
      s= "<polyline  stroke-width=\""+thickness.print()+"\" stroke=\""+color_string(color)+"\" fill=\"none\" points=\"";
    for (i=0 ; i<v.size()-1 ; i++){
      s=s+re(evalf(v[i]),0).print()+" "+im(evalf(v[i]),0).print()+", ";
    }
    s=s+re(evalf(v[i]),0).print()+" "+im(evalf(v[i]),0).print()+"\" /> ";
    svg_text(v[i],name,color);
    return s;
  }

  // Trac de courbes
  // un point sur 2 sert de point de contrle
  // on peut sophistiquer
  string svg_bezier_curve(gen g, int color, string legende){ 
    string x="x", y="y";
    int i;
    gen thickness((gnuplot_xmax-gnuplot_xmin)/250);
    string s= "<path  stroke-width=\""+thickness.print()+"\" stroke=\""+color_string(color)+"\" fill=\"none\" d=\"M";
    g=evalf(g);
    vecteur v=*(g._VECTptr);
    for (i=0 ; i<v.size() ; i++){
      s=s+re(v[i],0).print()+" "+im(v[i],0).print()+" ";
      if (i%2==0)
	s=s+"Q ";
      else
	s=s+", ";
    }
    i--;
    if (i%2==0)
      s=s+re(v[i],0).print()+" "+im(v[i],0).print()+" ,  "+re(v[i],0).print()+" "+im(v[i],0).print();
    if (i%2==1)
      s=s+re(v[i],0).print()+" "+im(v[i],0).print()+" ";
    s=s+"\" />\n ";
    // on crit la lgende
    //recherche d'un point dans le cadre pour ancrer la lgende
    for (i=0 ; i<v.size() ; i++){
      if (is_positive(re(v[i],0)-gnuplot_xmin,0) 
	  && is_positive(im(v[i],0)-gnuplot_ymin,0)
	  && is_positive(gnuplot_xmax-re(v[i],0),0) 
	  && is_positive(gnuplot_ymax-im(v[i],0),0) ){
	svg_text(v[i],legende,color);
	break;
      }
    }
    return s;
  }


  string symbolic2svg(const symbolic & mys);

  //fonction appele ssi v est un vecteur
  string vect2svg(gen v, int color, string name){
    if (v.type != _VECT)
      return "error";
    if (v.subtype==_SYMB){
      if (v[0].type==_VECT){
	return vect2svg(v[0], color, name);
      }
      if (v[0].type==_SYMB)
	return symbolic2svg(*v[0]._SYMBptr);
    }
    if (v.subtype==_GROUP__VECT)
      return svg_polyline(v, color, name);
    if (v.subtype==_LINE__VECT)
      return svg_line(v[0],v[1], color, name);
    if (v.subtype==_HALFLINE__VECT)
      return svg_half_line(v[0],v[1], color, name);
  }


  string symbolic2svg(const symbolic & mys){ 
   int color=default_color(0);
    string name="";
    if (mys.sommet==at_pnt){ 
      vecteur v=*(mys.feuille._VECTptr);
      if(v.size()==3)
	name=v[2].print();
      if (v[0].type==_VECT){
	return vect2svg(v[0], color, name);
      }                     
      if (v[0].type==4)           //indispensable, mais je ne sais pas pourquoi
	v[0]=gen(v[0].print());
      if (v[0].type==_SYMB){ 
	symbolic figure=*v[0]._SYMBptr; 
	if (figure.sommet == at_curve){
	  gen curve=figure.feuille;
	  return svg_bezier_curve(curve[1],color,name);
	}
	if (figure.sommet == at_pnt)
	  return symbolic2svg(figure);
	if (figure.sommet==at_segment){
	  gen segment=figure.feuille;
	  return svg_segment(segment[0],segment[1], color, name); 
	}   
	if (figure.sommet==at_droite ){
	  gen segment=figure.feuille;
	  return svg_line(segment[0],segment[1], color, name);
	}
	if (figure.sommet==at_demi_droite ){
	  gen segment=figure.feuille;
	  return svg_half_line(segment[0],segment[1], color, name);
	}
	if (figure.sommet==at_cercle ){
	  gen diametre;
	  if (figure.feuille[0].type==_VECT)
	    diametre=figure.feuille[0];
	  else
	    diametre=figure.feuille;
	  return svg_circle(diametre[0],diametre[1], color, name);
	}
	return svg_point(v[0], color, name); 
      }
      return svg_point(v[0], color, name); 
    }
    return "undef";
  }


  string gen2svg(const gen &e){  
    if (e.type== _SYMB)
      return symbolic2svg(*e._SYMBptr);
    if (e.type==_VECT){
      string s;
      vecteur v=*e._VECTptr;
      for (int i=0; i<v.size(); i++){
	if (v[i].type==_SYMB){
	  symbolic sym=*v[i]._SYMBptr; 
	  if (sym.sommet==at_pnt)
	    s=s+symbolic2svg(sym);
	}
      }
      return s;
    }
    return "error";
  }

  // --------------------- End SVG --------------------


  string symbolic2mathml(const symbolic & mys, string &svg){
    string opstring(mys.sommet.ptr->print());
    if (opstring!="/" && mys.sommet.ptr->texprint)  
      return mathml_print(mys);
    if (mys.sommet==at_pnt) { 
      svg=svg+symbolic2svg(mys);
      return "<mtext>"+mys.print()+"</mtext>";
    }
    if ( (mys.feuille.type==_VECT) && (mys.feuille._VECTptr->empty()) )
      return provisoire_mbox_begin+mys.sommet.ptr->print()+string("()")+provisoire_mbox_end;
    if ( (mys.feuille.type!=_VECT) || (mys.feuille._VECTptr->front().type==_VECT)){
      if ((mys.sommet==at_neg) || (mys.sommet==at_plus)){
	if (mys.feuille.type!=_SYMB) 
	  return "<mo>"+mys.sommet.ptr->print()+"</mo>"+gen2mathml(mys.feuille) ;	    
	if (mys.feuille._SYMBptr->sommet==at_inv || mys.feuille._SYMBptr->sommet==at_pow )
	  return "<mo>"+mys.sommet.ptr->print()+"</mo>"+gen2mathml(mys.feuille) ;
	return "<mo>"+mys.sommet.ptr->print()+"</mo>"+string("<mrow><mo>(</mo>") 
	  + gen2mathml(mys.feuille) +string("<mo>)</mo></mrow>");
      }
      if (mys.sommet==at_inv){
	return string("<mfrac><mrow><mn>1</mn></mrow><mrow>") + gen2mathml(mys.feuille) 
	  + string("</mrow></mfrac>");
      }
      return provisoire_mbox_begin +mys.sommet.ptr->print()+ provisoire_mbox_end 
	+ "<mrow><mo>(</mo>" + gen2mathml(mys.feuille) +"<mo>)</mo></mrow>" ;
    }
    string s;
    int l=mys.feuille._VECTptr->size();
    if ( mys.sommet==at_plus ){
      for (int i=0;i<l;++i){
	gen e((*(mys.feuille._VECTptr))[i]);
	if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg)){
	  if ( (e._SYMBptr->feuille).type==_SYMB && (e._SYMBptr->feuille)._SYMBptr->sommet==at_plus)
	    s += string("<mo>-</mo><mrow><mo>(</mo>")
	      + gen2mathml(e._SYMBptr->feuille)+
	      string("<mo>)</mo></mrow>");
	  else
	    s += string("<mo>-</mo><mrow>") + gen2mathml(e._SYMBptr->feuille)+string("</mrow>");
	}
	else {
	  if ( ( (e.type==_INT_) || (e.type==_ZINT) ) && (!is_positive(e,0)) )
	    s += "<mo>-</mo><mn>"+(-e).print()+"</mn>";
	  else {
	    if (i)
	      s += "<mo>"+opstring+"</mo>";
	    s += gen2mathml(e);
	  }
	}
      } // end_for
      return s;
    }
    if (mys.sommet==at_prod) {
      vecteur num;
      vecteur den;
      for (int i=0;i<l;++i){
	gen e((*(mys.feuille._VECTptr))[i]);
	if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_inv) )
	  den.push_back(e._SYMBptr->feuille);
	else {
	  if (!den.empty()){
	    s += "<mfrac><mrow>"+prod_vect2mathml_no_bra(num)+"</mrow><mrow>"+prod_vect2mathml_no_bra(den)
	      +"</mrow></mfrac><mo>*</mo>"; // A revoir ?  //"} \\* ";
	    num.clear();
	    den.clear();
	  }
	  num.push_back(e);
	}
      }
      if (den.empty())
	return s+prod_vect2mathml(num);
      return s+"<mfrac><mrow>"+prod_vect2mathml_no_bra(num)+"</mrow><mrow>"
	+prod_vect2mathml_no_bra(den)+"</mrow></mfrac>";
    } // end if sommet_is_prod

    if (mys.sommet==at_pow){
      if ( (mys.feuille._VECTptr->back()==plus_one_half)  )
	return "<msqrt>"+gen2mathml(mys.feuille._VECTptr->front())+"</msqrt>";
      if ( (mys.feuille._VECTptr->back()==minus_one_half ) || 
	   (mys.feuille._VECTptr->back()==fraction(minus_one,plus_two) ) )
	return "<mfrac><mn>1</mn><msqrt>"+gen2mathml(mys.feuille._VECTptr->front())+"</msqrt></mfrac>";
      string s_bra="<msup><mrow><mo>(</mo>"+gen2mathml((*(mys.feuille._VECTptr))[0])
	+"<mo>)</mo></mrow><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[1])
	+"</mrow></msup>";
      string s_no_bra= "<msup><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[0])        
	+"</mrow><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[1])+"</mrow></msup>";
      if (mys.feuille._VECTptr->front().type==_SYMB){
	symbolic mantisse(*mys.feuille._VECTptr->front()._SYMBptr);
	if ( (mantisse.feuille.type==_VECT) && (mantisse.feuille._VECTptr->empty()) )
	  return s_bra;
	if ( (mantisse.feuille.type!=_VECT) || (mantisse.feuille._VECTptr->front().type==_VECT))
	  return s_bra;
	if (mantisse.feuille._VECTptr->size()>1)
	  return  s_bra;
	else
	  return  s_no_bra;
      }
      else if (mys.feuille._VECTptr->front().type==_FRAC)
	return s_bra;
      else
	return s_no_bra;
    }

    if (opstring=="/") { //at_division non reconnu
      return "<mfrac><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[0])
	+"</mrow><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[1])+"</mrow></mfrac>";
    } 

    s = provisoire_mbox_begin+opstring + provisoire_mbox_end +"<mrow><mo>(</mo>";
    for (int i=0;;++i){
      s += gen2mathml((*(mys.feuille._VECTptr))[i]);
      if (i==l-1)
	return s+"<mo>)</mo></mrow>";
      s += ',';
    }
  }




  // assume math mode enabled
  string gen2mathml(const gen &e){
    string svg="";
    return gen2mathml(e, svg);
  }

  string gen2mathml(const gen &e, string &svg){
    string part_re="", part_im="<mi>i</mi>";
    if (e.type==_SYMB && e._SYMBptr->sommet==at_sum)
      return mathml_printassum(e);
    else if (e.type==_SYMB && e._SYMBptr->sommet==at_abs)
      return mathml_printasabs(e);
    else
      switch (e.type){
      case _INT_: case _ZINT:                        
	return "<mn>"+e.print()+"</mn>";
      case _DOUBLE_:                        
	if (fabs(e._DOUBLE_val)<1.1e-5)
	  return "<mn>0.0</mn>";
	else
	  return "<mn>"+e.print()+"</mn>"; 
      case _CPLX:                        
	if (!is_zero(re(e,0)))
	    part_re="<mn>"+re(e,0).print()+"</mn>";
	if (!is_zero(im(e,0))){
	  if (is_positive(im(e,0),0)){
	    if (!is_zero(re(e,0)))
	      part_re+="<mo>+</mo>";
	  }
	  else
	    part_re+="<mo>-</mo>";
	}
	if (is_zero(im(e,0)))
	  part_im="";
	if (!is_one(-im(e,0)) && ! is_one(im(e,0)))
	  part_im="<mn>"+abs(im(e,0),0).print()+"</mn>"+part_im;	
      	return part_re+part_im;
      case _IDNT:                        
	if (e==unsigned_inf)
	  return "<mn>&infin;</mn>";
	if (e==cst_pi)
	  return "<mi>&pi;</mi>";
	if (e==undef)
	  return "<mo>,</mo><mi>undef</mi>";
	return  "<mi>"+e.print()+"</mi>";
      case _SYMB:                        
	return symbolic2mathml(*e._SYMBptr, svg);
      case _VECT:                        
	if (e.subtype==_SPREAD__VECT)
	  return spread2mathml(*e._VECTptr,1); //----------------vrifier le 2me paramtre
	if (ckmatrix(*e._VECTptr))
	  return matrix2mathml(*e._VECTptr);
	else
	  return _VECT2mathml(*e._VECTptr, svg);
      case _POLY:
	return string("<mi>polynome</mi>");
      case _FRAC:                        
	return string("<mfrac><mrow>")+gen2mathml(e._FRACptr->num)+"</mrow><mrow>"
	  +gen2mathml(e._FRACptr->den)+"</mrow></mfrac>";
      case _EXT: case _ROOT:
	return "";
      case _STRNG:
	return "<mi>"+(*e._STRNGptr)+"</mi>";
      case _FUNC:
	return "<mi>"+e.print()+"</mi>";
      case _USER:
	return "<mi>"+e._USERptr->texprint()+"</mi>"; // <--------------------------- A traduire ?
      case _MOD:
	return gen2mathml(*e._MODptr)+"<mo>%</mo>"+gen2mathml(*(e._MODptr+1));
      default:
	settypeerr("MathMl convert "+e.print());
      }
  }



  string gen2mathmlfull(const gen & g){
    return mathml_preamble+"\n<math mode=\"display\" xmlns=\"http://www.w3.org/1998/Math/MathML\">\n\n"+gen2mathml(g)+"\n\n</math><br/>\n"+mathml_end+'\n';
  }
  gen _mathml(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>1 && (*g._VECTptr)[1].type==_STRNG){
      ofstream of((*g._VECTptr)[1]._STRNGptr->c_str());
      of << gen2mathmlfull(g._VECTptr->front()) << endl;
      return plus_one;
    }
    return string2gen(gen2mathmlfull(g),false);
  }
  const string _mathml_s("mathml");
  unary_function_unary __mathml(&_mathml,_mathml_s);
  unary_function_ptr at_mathml (&__mathml,0,true);

  string spread2mathmlfull(const matrice m, int formule){
    return mathml_preamble+spread2mathml(m, formule)+mathml_end+'\n';
  }

  gen _spread2mathml(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      return string2gen(" syntax error ");
    vecteur &v=*g._VECTptr; 
    if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>2 && (*g._VECTptr)[2].type==_STRNG){
      ofstream of((*g._VECTptr)[2]._STRNGptr->c_str());
      of << spread2mathmlfull(*(v[0]._VECTptr), is_one(v[1])) << endl;
      return plus_one;
    }
    return string2gen(spread2mathmlfull(*(v[0]._VECTptr) , is_one(v[1])));
  }
  const string _spread2mathml_s("spread2mathml");
  unary_function_unary __spread2mathml(&_spread2mathml,_spread2mathml_s);
  unary_function_ptr at_spread2mathml (&__spread2mathml,0,true);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
