/* -*- mode:C++ ; compile-command: "g++ -I.. -g -c misc.cc" -*- */
#include "first.h"
/*
 *  Copyright (C) 2001 R. De Graeve, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include <fstream>
#include <unistd.h> // For reading arguments from file
#include <time.h>
#include <string>
#include "misc.h"
#include "usual.h"
#include "sym2poly.h"
#include "rpn.h"
#include "prog.h"
#include "derive.h"
#include "subst.h"
#include "intg.h"
#include "vecteur.h"
#include "ifactor.h"
#include "solve.h"
#include "modpoly.h"
#include "permu.h"
#include "sym2poly.h"
#include "plot.h"
#include "lin.h"
#include "modpoly.h"
#include "desolve.h"
#include "alg_ext.h"
#include "input_parser.h"
#include "input_lexer.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  gen _scalar_product(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    return scalar_product(v[0],v[1],contextptr);
  }    
  const string _scalar_product_s("scalar_product");
  unary_function_eval __scalar_product(&_scalar_product,_scalar_product_s);
  unary_function_ptr at_scalar_product (&__scalar_product,0,true);

  gen _compare(const gen & args){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    return v[0].islesscomplexthan(v[1]);
  }    
  const string _compare_s("compare");
  unary_function_unary __compare(&_compare,_compare_s);
  unary_function_ptr at_compare (&__compare,0,true);

  gen _preval(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_preval,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<3)
      toofewargs("");
    gen f(v[0]),x,a,b;
    a=v[1];
    b=v[2];
    if (s==3)
      x=vx_var;
    else 
      x=v[3];
    if (x.type!=_IDNT)
      settypeerr();
    gen res;
    if (is_greater(b,a,contextptr))
      res=limit(f,*x._IDNTptr,b,-1,contextptr)-limit(f,*x._IDNTptr,a,1,contextptr);
    else {
      if (is_greater(a,b,contextptr))
	res=limit(f,*x._IDNTptr,b,1,contextptr)-limit(f,*x._IDNTptr,a,-1,contextptr) ;
      else
	res=limit(f,*x._IDNTptr,b,0,contextptr)-limit(f,*x._IDNTptr,a,0,contextptr);
    }
    return res;
  }    
  const string _preval_s("preval");
  unary_function_eval __preval(&_preval,_preval_s);
  unary_function_ptr at_preval (&__preval,0,true);

  vecteur divided_differences(const vecteur & x,const vecteur & y){
    vecteur res(y);
    int s=x.size();
    for (int k=1;k<s;++k){
      for (int j=s-1;j>=k;--j){
	res[j]=(res[j]-res[j-1])/(x[j]-x[j-k]);
      }
    }
    return res;
  }
  gen _lagrange(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_lagrange,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      toofewargs("");
    gen v0(v[0]),v1(v[1]),x;
    if (s>=3)
      x=v[2];
    else
      x=vx_var;
    if ( (v0.type!=_VECT) || (v1.type!=_VECT) )
      setsizeerr();
    vecteur & vx =*v0._VECTptr;
    vecteur & vy=*v1._VECTptr;
    s=vx.size();
    if (vy.size()!=s)
      setdimerr();
    // Using divided difference instead of the theoretical formula
    vecteur w=divided_differences(vx,vy);
    gen pi(1),res;
    for (int i=0;i<s;++i){
      res=res+w[i]*pi;
      pi=pi*(x-vx[i]);
    }
    return res;
    /*
    gen res(zero);
    for (int i=0;i<s;++i){
      gen pix(plus_one),pix0(plus_one),x0(vx[i]);
      for (int j=0;j<s;++j){
	if (j==i)
	  continue;
	pix=pix*(x-vx[j]);
	pix0=pix0*(x0-vx[j]);
      }
      res=res+vy[i]*rdiv(pix,pix0);
    }
    return res;
    */
  }    
  const string _lagrange_s("lagrange");
  unary_function_unary __lagrange(&_lagrange,_lagrange_s);
  unary_function_ptr at_lagrange (&__lagrange,0,true);

  gen _reorder(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_reorder,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      toofewargs("");
    gen e(v[0]),l(v[1]);
    if (l.type!=_VECT)
      setsizeerr();
    vecteur w(*l._VECTptr);
    lvar(e,w);
    e=e2r(e,w);
    return r2e(e,w);
  }    
  const string _reorder_s("reorder");
  unary_function_unary __reorder(&_reorder,_reorder_s);
  unary_function_ptr at_reorder (&__reorder,0,true);

  gen _adjoint_matrix(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_adjoint_matrix,args);
    matrice mr(*args._VECTptr);
    if (!is_squarematrix(mr))
      setsizeerr();
    matrice m_adj;
    vecteur p_car;
    p_car=mpcar(mr,m_adj,true);
    return makevecteur(p_car,m_adj);
  }    
  const string _adjoint_matrix_s("adjoint_matrix");
  unary_function_unary __adjoint_matrix(&_adjoint_matrix,_adjoint_matrix_s);
  unary_function_ptr at_adjoint_matrix (&__adjoint_matrix,0,true);

  gen _equal2diff(const gen & args){
    return apply(args,equal2diff);
  }    
  const string _equal2diff_s("equal2diff");
  unary_function_unary __equal2diff(&_equal2diff,_equal2diff_s);
  unary_function_ptr at_equal2diff (&__equal2diff,0,true);

  gen equal2list(const gen & arg){
    if ( (arg.type!=_SYMB) || (arg._SYMBptr->sommet!=at_equal))
      return makevecteur(arg,zero);
    return arg._SYMBptr->feuille;
  }
  gen _equal2list(const gen & args){
    return apply(args,equal2list);
  }    
  const string _equal2list_s("equal2list");
  unary_function_unary __equal2list(&_equal2list,_equal2list_s);
  unary_function_ptr at_equal2list (&__equal2list,0,true);

  gen _rank(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_adjoint_matrix,args);
    matrice mr(*args._VECTptr);
    if (!ckmatrix(mr))
      setsizeerr();
    mr=mrref(mr);
    int r=mr.size();
    for (;r;--r){
      if (!is_zero(mr[r-1]))
	break;
    }
    return r;
  }    
  const string _rank_s("rank");
  unary_function_unary __rank(&_rank,_rank_s);
  unary_function_ptr at_rank (&__rank,0,true);

  gen _sec(const gen & args,GIAC_CONTEXT){
    return inv(cos(args,contextptr));
  }    
  const string _sec_s("sec");
  unary_function_eval __sec(&_sec,_sec_s);
  unary_function_ptr at_sec (&__sec,0,true);

  gen _csc(const gen & args,GIAC_CONTEXT){
    return inv(sin(args,contextptr));
  }    
  const string _csc_s("csc");
  unary_function_eval __csc(&_csc,_csc_s);
  unary_function_ptr at_csc (&__csc,0,true);

  gen _cot(const gen & args,GIAC_CONTEXT){
    return rdiv(cos(args,contextptr),sin(args,contextptr));
  }    
  const string _cot_s("cot");
  unary_function_eval __cot(&_cot,_cot_s);
  unary_function_ptr at_cot (&__cot,0,true);

  gen _asec(const gen & args,GIAC_CONTEXT){
    return acos(inv(args),contextptr);
  }    
  const string _asec_s("asec");
  unary_function_eval __asec(&_asec,_asec_s);
  unary_function_ptr at_asec (&__asec,0,true);

  gen _acsc(const gen & args,GIAC_CONTEXT){
    return asin(inv(args),contextptr);
  }    
  const string _acsc_s("acsc");
  unary_function_eval __acsc(&_acsc,_acsc_s);
  unary_function_ptr at_acsc (&__acsc,0,true);

  gen _acot(const gen & args,GIAC_CONTEXT){
    return atan(inv(args),contextptr);
  }    
  const string _acot_s("acot");
  unary_function_eval __acot(&_acot,_acot_s);
  unary_function_ptr at_acot (&__acot,0,true);

  // args=[u'*v,v] or [[F,u'*v],v] -> [F+u*v,-u*v']
  // a third argument would be the integration var
  // if v=0 returns F+integrate(u'*v,x)
  gen _ibpu(const gen & args,GIAC_CONTEXT) {
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return symbolic(at_ibpu,args);
    vecteur & w=*args._VECTptr;
    gen x(vx_var);
    if (w.size()>=3)
      x=w[2];
    gen u,v(w[1]),uprimev,F;
    if (w.front().type==_VECT){
      vecteur & ww=*w.front()._VECTptr;
      if (ww.size()!=2)
	setsizeerr();
      F=ww.front();
      uprimev=ww.back();
    }
    else 
      uprimev=w.front();
    if (is_zero(v) || is_one(v))
      return integrate(uprimev,x,contextptr)+F;
    gen uprime(normal(rdiv(uprimev,v)));
    u=integrate(uprime,x,contextptr);
    return makevecteur(F+u*v,normal(-u*derive(v,x)));
  }
  const string _ibpu_s("ibpu");
  unary_function_eval __ibpu(&_ibpu,_ibpu_s);
  unary_function_ptr at_ibpu (&__ibpu,0,true);

  gen _changebase(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_changebase,args);
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      settypeerr();
    gen a=v.front(),p=v.back();
    if (!is_squarematrix(p))
      setsizeerr();
    return minv(*p._VECTptr)*a*p;
  }    
  const string _changebase_s("changebase");
  unary_function_unary __changebase(&_changebase,_changebase_s);
  unary_function_ptr at_changebase (&__changebase,0,true);

  gen epsilon2zero(const gen & g,GIAC_CONTEXT){
    switch (g.type){
    case _DOUBLE_:
      if (fabs(g._DOUBLE_val)<epsilon(contextptr))
	return zero;
      else
	return g;
    case _CPLX:
      return epsilon2zero(re(g,contextptr),contextptr)+cst_i*epsilon2zero(im(g,contextptr),contextptr);
    case _SYMB:
      return symbolic(g._SYMBptr->sommet,epsilon2zero(g._SYMBptr->feuille,contextptr));
    case _VECT:
      return apply(g,epsilon2zero,contextptr);
    default:
      return g;
    }
  }
  gen _epsilon2zero(const gen & args,GIAC_CONTEXT){
    return epsilon2zero(args,contextptr);
  }    
  const string _epsilon2zero_s("epsilon2zero");
  unary_function_eval __epsilon2zero(&_epsilon2zero,_epsilon2zero_s);
  unary_function_ptr at_epsilon2zero (&__epsilon2zero,_QUOTE_ARGUMENTS,true);

  gen _suppress(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_suppress,args);
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      settypeerr();
    gen l=v.front(),i=v.back();
    int ii;
    if (i.type==_INT_ )
      ii=i.val-(maple_mode!=0);
    if (l.type==_STRNG){
      string res;
      string & s=*l._STRNGptr;
      int n=s.size();
      if (i.type==_INT_ && ii>=0 && ii<n)
	res=s.substr(0,ii)+s.substr(ii+1,n-ii-1);
      if (i.type==_STRNG){
	string & remove=*i._STRNGptr;
	int removen=remove.size();
	for (int j=0;j<n;++j){
	  int k=remove.find(s[j]);
	  if (k<0 || k>=removen)
	    res += s[j];
	}
      }
      return string2gen(res,false);
    }
    if ( (l.type!=_VECT) || (i.type!=_INT_) )
      setsizeerr();
    const_iterateur it=l._VECTptr->begin(),itend=l._VECTptr->end();
    vecteur res;
    res.reserve(itend-it);
    for (int j=0;it!=itend;++it,++j){
      if (j!=ii)
	res.push_back(*it);
    }
    return gen(res,l.subtype);
  }    
  const string _suppress_s("suppress");
  unary_function_unary __suppress(&_suppress,_suppress_s);
  unary_function_ptr at_suppress (&__suppress,0,true);

  int valuation(const polynome & p){
    if (p.coord.empty())
      return -1;
    return p.coord.back().index.iptr->front();
  }
  gen _valuation(const gen & args){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if (!s)
	return minus_inf;
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) ){
	int j=s;
	for (;j;--j){
	  if (!is_zero(v[j-1]))
	    break;
	}
	return s-j;
      }
      x=v.back();
      p=v.front();
    }
    vecteur lv(1,x);
    lvar(p,lv);
    gen aa=e2r(p,lv),aan,aad;
    if (is_zero(aa))
      return minus_inf;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (aan.type!=_POLY)
      return zero;
    int res=valuation(*aan._POLYptr);
    if (res==-1)
      return minus_inf;
    else
      return res;
  }    
  const string _valuation_s("valuation");
  unary_function_unary __valuation(&_valuation,_valuation_s);
  unary_function_ptr at_valuation (&__valuation,0,true);

  const string _ldegree_s("ldegree");
  unary_function_unary __ldegree(&_valuation,_ldegree_s);
  unary_function_ptr at_ldegree (&__ldegree,0,true);

  gen _degree(const gen & args){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) )
	return s-1;
      x=v.back();
      p=v.front();
    }
    vecteur lv(1,x);
    lvar(p,lv);
    gen aa=e2r(p,lv),aan,aad;
    if (is_zero(aa))
      return zero;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (aan.type!=_POLY)
      return zero;
    return aan._POLYptr->lexsorted_degree();
  }    
  const string _degree_s("degree");
  unary_function_unary __degree(&_degree,_degree_s);
  unary_function_ptr at_degree (&__degree,0,true);

  gen _lcoeff(const gen & args){
    gen x,p;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) )
	return v.front();
      x=v[1];
      p=v[0];
    }
    gen g=_e2r(makevecteur(p,x));
    if (g.type!=_VECT)
      return zero;
    return g._VECTptr->front();
  }
  const string _lcoeff_s("lcoeff");
  unary_function_unary __lcoeff(&_lcoeff,_lcoeff_s);
  unary_function_ptr at_lcoeff (&__lcoeff,0,true);

  gen tcoeff(const vecteur & v){
    int s=v.size();
    gen g;
    for (;s;--s){
      g=v[s-1];
      if (!is_zero(g))
	return g;
    }
    return zero;
  }
  gen _tcoeff(const gen & args){
    gen x,p;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur& v=*args._VECTptr;
      int s=v.size();
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) )
	return tcoeff(v);
      x=v[1];
      p=v[0];
    }
    gen g=_e2r(makevecteur(p,x));
    if (g.type!=_VECT)
      return g;
    return tcoeff(*g._VECTptr);
  }
  const string _tcoeff_s("tcoeff");
  unary_function_unary __tcoeff(&_tcoeff,_tcoeff_s);
  unary_function_ptr at_tcoeff (&__tcoeff,0,true);

  gen sqrfree(const gen & g,const vecteur & l){
    if (g.type!=_POLY)
      return r2sym(g,l);
    factorization f(sqff(*g._POLYptr));
    factorization::const_iterator it=f.begin(),itend=f.end();
    gen res(plus_one);
    for (;it!=itend;++it)
      res=res*pow(r2e(it->fact,l),it->mult);
    return res;
  }
  gen _sqrfree(const gen & args){
    if (args.type==_VECT)
      return apply(args,_sqrfree);
    if (args.type!=_SYMB)
      return args;
    vecteur l(alg_lvar(args));
    fraction f(e2r(args,l));
    return sqrfree(f.num,l)/sqrfree(f.den,l);
  }
  const string _sqrfree_s("sqrfree");
  unary_function_unary __sqrfree(&_sqrfree,_sqrfree_s);
  unary_function_ptr at_sqrfree (&__sqrfree,0,true);

  gen _truncate(const gen & args){
    gen e(args);
    int n,s=1;
    vecteur w(1,vx_var);
    gen gn(5);
    if (args.type==_VECT){
      vecteur & v=*args._VECTptr;
      s=v.size();
      if (s==0)
	setsizeerr();
      e=v[0];
      if (s==3){
	w=gen2vecteur(v[1]);
	gn=v[2];
      }
      else {
	if (s==2)
	  gn=v[1];
      }
    }
    if (gn.type!=_INT_)
      setsizeerr();
    n=gn.val;
    int nvar=w.size(); // number of var w.r.t. which we truncate
    vecteur l(lop(e,at_order_size));
    vecteur lp(l.size(),zero);
    e=subst(e,l,lp);
    // FIXME if l not empty, adjust order of truncation using arg of order_size
    lvar(e,w);
    e=e2r(e,w);
    gen num,den;
    fxnd(e,num,den);
    if ( (den.type==_POLY) && (den._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (num.type==_POLY){
      vector< monomial<gen> >::const_iterator it=num._POLYptr->coord.begin(),itend=num._POLYptr->coord.end();
      vector< monomial<gen> > res;
      for (;it!=itend;++it){
	index_t & i=*it->index.iptr;
	int deg=0;
	for (int j=0;j<nvar;++j)
	  deg=deg+i[j];
	if (deg<=n)
	  res.push_back(*it);
      }
      num._POLYptr->coord=res;
    }
    return r2e(rdiv(num,den),w);
  }    
  const string _truncate_s("truncate");
  unary_function_unary __truncate(&_truncate,_truncate_s);
  unary_function_ptr at_truncate (&__truncate,0,true);

  gen _canonical_form(const gen & args){
    gen p,x,a,b,c;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      if (v.size()!=2)
	settypeerr();
      p=v.front();
      x=v.back();
    }
    if (x.type!=_IDNT)
      settypeerr();
    if (!is_quadratic_wrt(p,*x._IDNTptr,a,b,c))
      setsizeerr();
    if (is_zero(a))
      return b*x+c;
    // a*x^2+b*x+c -> a*(x+b/(2*a))^2+(b^2-4*a*c)/(4*a)
    return a*pow(x+b/(2*a),2)+(4*a*c-pow(b,2))/(4*a);
  }    
  const string _canonical_form_s("canonical_form");
  unary_function_unary __canonical_form(&_canonical_form,_canonical_form_s);
  unary_function_ptr at_canonical_form (&__canonical_form,0,true);

  gen _fcoeff(const gen & args){
    gen x;
    vecteur p;
    if (args.type!=_VECT)
      return symbolic(at_fcoeff,args);
    vecteur & v=*args._VECTptr;
    if ( (v.size()!=2) || (v.front().type!=_VECT) ){
      p=v;
      x=vx_var;
    }
    else {
      p=*v.front()._VECTptr;
      x=v.back();
    }
    if (x.type!=_IDNT)
      settypeerr();    
    const_iterateur it=p.begin(),itend=p.end();
    if ( (itend-it)%2 )
      setsizeerr();
    gen res(plus_one);
    for (;it!=itend;it+=2){
      res=res*pow(x-*it,*(it+1));
    }
    return res;
  }    
  const string _fcoeff_s("fcoeff");
  unary_function_unary __fcoeff(&_fcoeff,_fcoeff_s);
  unary_function_ptr at_fcoeff (&__fcoeff,0,true);

  void addfactors(const gen & p,const gen & x,int mult,vecteur & res,GIAC_CONTEXT){
    vecteur v=factors(p,x);
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;){
      vecteur w=solve(*it,x,1,contextptr);
      ++it;
      int n=it->val;
      ++it;
      const_iterateur jt=w.begin(),jtend=w.end();
      for (;jt!=jtend;++jt){
	res.push_back(*jt);
	res.push_back(n*mult);
      }
    }
  }    

  gen _froot(const gen & args,GIAC_CONTEXT){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      if (v.size()!=2)
	setsizeerr();
      x=v.back();
      if (x.type!=_IDNT)
	setsizeerr("2nd arg");
      p=v.front();
    }
    vecteur lv(lvar(p));
    gen aa=e2r(p,lv),aan,aad;
    fxnd(aa,aan,aad);
    vecteur res;
    addfactors(r2e(aan,lv),x,1,res,contextptr);
    addfactors(r2e(aad,lv),x,-1,res,contextptr);
    return res;
  }

  const string _froot_s("froot");
  unary_function_eval __froot(&_froot,_froot_s);
  unary_function_ptr at_froot (&__froot,0,true);

  gen _roots(const gen & g,GIAC_CONTEXT){
    gen r=_froot(g,contextptr);
    if (r.type!=_VECT || (r._VECTptr->size() % 2) )
      setsizeerr();
    vecteur & v = *r._VECTptr;
    vecteur res;
    int s=v.size()/2;
    for (int i=0;i<s;++i){
      if (v[2*i+1].val>0)
	res.push_back(makevecteur(v[2*i],v[2*i+1]));
    }
    return res;
  }
  const string _roots_s("roots");
  unary_function_eval __roots(&_roots,_roots_s);
  unary_function_ptr at_roots (&__roots,0,true);

  gen _divpc(const gen & args){
    gen p,q,x;
    if (args.type!=_VECT)
      return symbolic(at_divpc,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<3)
      setsizeerr();
    p=v.front();
    q=v[1];
    if (v[2].type!=_INT_)
      setsizeerr();
    if (s==3)
      x=vx_var;
    else 
      x=v.back();
    vecteur lv(1,x);
    lvar(p,lv);
    lvar(q,lv);
    gen aa=e2r(p,lv),aan,aad;
    fxnd(aa,aan,aad);
    gen ba=e2r(q,lv),ban,bad;
    fxnd(ba,ban,bad);
    if ( ( (aad.type==_POLY)&&(aad._POLYptr->lexsorted_degree()) )
	 || (bad.type==_POLY)&&(bad._POLYptr->lexsorted_degree())
	 )
      setsizeerr();
    if (ban.type!=_POLY)
      return r2e(rdiv(aan*bad,ban*aad),lv);
    vecteur a;
    if (aan.type==_POLY)
      a=polynome2poly1(*aan._POLYptr,1);
    else
      a=vecteur(1,aan);
    vecteur b=polynome2poly1(*ban._POLYptr,1);
    if (is_zero(b.back()))
      divisionby0err(q);
    reverse(a.begin(),a.end());
    reverse(b.begin(),b.end());
    int n=b.size()-a.size()+v[2].val;
    for (int i=0;i<n;++i)
      a.push_back(zero);
    vecteur quo,rem;
    environment * env=new environment;
    DivRem(a,b,env,quo,rem);
    delete env;
    reverse(quo.begin(),quo.end());
    gen res(vecteur2polynome(quo,lv.size()));
    res=rdiv(res*bad,aad);
    return r2e(res,lv);
  }

  const string _divpc_s("divpc");
  unary_function_unary __divpc(&_divpc,_divpc_s);
  unary_function_ptr at_divpc (&__divpc,0,true);

  gen _ptayl(const gen & args){
    gen p,q,x;
    if (args.type!=_VECT)
      return symbolic(at_ptayl,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      setsizeerr();
    p=v.front();
    q=v[1];
    if (p.type==_VECT){
      vecteur a=*p._VECTptr;
      int n=a.size();
      vecteur b,res;
      for (int i=0;i<n;++i){
	res.push_back(horner(a,q,0,b));
	a=b;
      }
      reverse(res.begin(),res.end());
      return res;
    }
    if (s==2)
      x=vx_var;
    else 
      x=v.back();
    if (!is_zero(derive(q,x)))
      setsizeerr();
    vecteur lv(1,x);
    lvar(p,lv);
    lvar(q,lv);
    gen aa=e2r(p,lv),aan,aad;
    fxnd(aa,aan,aad);
    if ( ( (aad.type==_POLY)&&(aad._POLYptr->lexsorted_degree()) )
	 )
      setsizeerr();
    if (aan.type!=_POLY)
      return p;
    gen ba=e2r(q,vecteur(lv.begin()+1,lv.end()));
    vecteur a(polynome2poly1(*aan._POLYptr,1));
    int n=a.size();
    vecteur b,res;
    for (int i=0;i<n;++i){
      res.push_back(horner(a,ba,0,b));
      a=b;
    }
    reverse(res.begin(),res.end());
    return r2e(vecteur2polynome(res,lv.size()),lv);
  }

  const string _ptayl_s("ptayl");
  unary_function_unary __ptayl(&_ptayl,_ptayl_s);
  unary_function_ptr at_ptayl (&__ptayl,0,true);
  
  vecteur gen2continued_fraction(const gen & g,int n){
    // Compute a vector of size n+1 with last element=remainder
    vecteur res,remain;
    gen tmp(g),f;
    int i=0,j;
    for (;i<n;++i){
      if (j=equalposcomp(remain,tmp)){
	int s=remain.size();
	res.push_back(vecteur(res.begin()+j-1,res.end()));
	return res;
      }
      else
	remain.push_back(tmp);
      f=_floor(tmp,0);
      res.push_back(f);
      if (is_zero(tmp-f))
	return res;
      tmp=normal(inv(tmp-f));
    }
    res.push_back(tmp);
    return res;
  }
  gen _dfc(const gen & g_orig,GIAC_CONTEXT){
    gen g=g_orig;
    double eps=epsilon(contextptr);
    if (g.type==_VECT && g._VECTptr->size()==2){
      if (g._VECTptr->back().type==_DOUBLE_){
	eps=g._VECTptr->back()._DOUBLE_val;
	g=evalf_double(g._VECTptr->front());
      }
      else {
	if (g._VECTptr->back().type==_INT_)
	  return gen2continued_fraction(g._VECTptr->front(),g._VECTptr->back().val);
      }
    }
    g=evalf_double(g);
    if (g.type!=_DOUBLE_)
      setsizeerr();
    return vector_int_2_vecteur(float2continued_frac(g._DOUBLE_val,eps));
  }
  const string _dfc_s("dfc");
  unary_function_eval __dfc(&_dfc,_dfc_s);
  unary_function_ptr at_dfc (&__dfc,0,true);

  gen _dfc2f(const gen & g){
    if (g.type!=_VECT || g._VECTptr->empty())
      setsizeerr();
    vecteur & v (*g._VECTptr);
    gen res(v.back());
    if (v.back().type==_VECT){
      // represent a quadratic x=[... x], find equation
      identificateur tmp(" x");
      gen eq(tmp);
      const_iterateur it=v.back()._VECTptr->end()-1,itend=v.back()._VECTptr->begin()-1;
      for (;it!=itend;--it)
	eq=inv(eq)+(*it);
      vecteur w=solve(eq-tmp,tmp);
      gen ws=_sort(w,0);
      if (ws.type!=_VECT || ws._VECTptr->empty())
	setsizeerr();
      res=ws._VECTptr->back();
    }
    for (;;){
      v.pop_back();
      if (v.empty())
	return res;
      res=inv(res);
      res=res+v.back();
    }
    // return continued_frac2gen(vecteur_2_vector_int(*g._VECTptr),nan(),epsilon);
  }
  const string _dfc2f_s("dfc2f");
  unary_function_unary __dfc2f(&_dfc2f,_dfc2f_s);
  unary_function_ptr at_dfc2f (&__dfc2f,0,true);

  gen float2rational(double d_orig,double eps){
    double d=d_orig;
    if (d<0)
      return -float2rational(-d,eps);
    if (d>RAND_MAX)
      return d;    // reconstruct
    vector<int> v(float2continued_frac(d,eps));
    return continued_frac2gen(v,d_orig,eps);
  }
  gen _float2rational(const gen & g,GIAC_CONTEXT){
    switch (g.type){
    case _DOUBLE_:
      return float2rational(g._DOUBLE_val,epsilon(contextptr));
    case _CPLX:
      return _float2rational(re(g,contextptr),contextptr)+cst_i*_float2rational(im(g,contextptr),contextptr);
    case _SYMB:
      return symbolic(g._SYMBptr->sommet,_float2rational(g._SYMBptr->feuille,contextptr));
    case _VECT:
      return apply(g,_float2rational,contextptr);
    default:
      return g;
    }
  }
  const string _float2rational_s("float2rational");
  unary_function_eval __float2rational(&_float2rational,_float2rational_s);
  unary_function_ptr at_float2rational (&__float2rational,0,true);

  gen _gramschmidt(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return symbolic(at_gramschmidt,g);
    vecteur & v(*g._VECTptr);
    if (is_squarematrix(v))
      return gramschmidt(v);
    if (v.size()==2){
      gen lvect=v[0];
      gen scalaire=v[1];
      if (lvect.type!=_VECT)
	setsizeerr();
      vecteur lv=*lvect._VECTptr;
      int s=lv.size();
      if (!s)
	return lv;
      vecteur sc(1,scalaire(gen(makevecteur(lv[0],lv[0]),_SEQ__VECT),contextptr));
      for (int i=1;i<s;++i){
	gen cl;
	for (int j=0;j<i;++j)
	  cl=cl+rdiv(scalaire(gen(makevecteur(lv[i],lv[j]),_SEQ__VECT),contextptr),sc[j])*lv[j];
	lv[i]=lv[i]-cl;
	sc.push_back(scalaire(gen(makevecteur(lv[i],lv[i]),_SEQ__VECT),contextptr));
      }
      for (int i=0;i<s;++i)
	lv[i]=rdiv(lv[i],sqrt(sc[i]));
      return lv;
    }
    setsizeerr();
  }
  const string _gramschmidt_s("gramschmidt");
  unary_function_eval __gramschmidt(&_gramschmidt,_gramschmidt_s);
  unary_function_ptr at_gramschmidt (&__gramschmidt,0,true);

  void aplatir(const matrice & m,vecteur & v){
    int s=m.size();
    v=vecteur(s*s);
    iterateur kt=v.begin(),ktend=v.end();
    const_iterateur it=m.begin(),itend=m.end(),jt,jtend;
    for (;it!=itend;++it){
      jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
      for (;jt!=jtend;++jt,++kt)
	*kt=*jt;
    }
  }

  gen pmin(const matrice & m){
    int s=m.size();
    matrice mpow(midn(s));
    matrice res;
    vecteur v;
    for (int i=0;i<=s;++i){
      aplatir(mpow,v);
      v.push_back(pow(vx_var,i));
      res.push_back(v);
      mpow=mmult(mpow,m);
    }
    matrice r;
    gen det;
    mrref(res,r,v,det,0,s+1,0,s*s,false,1);
    // find 1st line with zeros (except in the last col)
    const_iterateur it=r.begin(),itend=r.end();
    for (;it!=itend;++it){
      if (is_zero(vecteur(it->_VECTptr->begin(),it->_VECTptr->end()-1)))
	break;
    }
    if (it==itend)
      setsizeerr();
    gen t= _e2r(makevecteur(it->_VECTptr->back(),vx_var));
    if (t.type==_VECT)
      return t/lgcd(*t._VECTptr);
    else
      return t;
  }
  gen _pmin(const gen & g){
    if (is_squarematrix(g)){
      matrice &m =*g._VECTptr;
      vecteur w;
      if (probabilistic_pmin(m,w,true))
	return w;
      return pmin(m);
    }
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      return symbolic(at_pmin,g);
    vecteur & v(*g._VECTptr);
    if (!is_squarematrix(v.front()))
      setsizeerr();
    matrice &m=*v.front()._VECTptr;
    // probabilistic minimal polynomial
    vecteur w;
    if (probabilistic_pmin(m,w,true))
      return symb_horner(w,v.back());
    else
      return _r2e(makevecteur(pmin(m),v.back()));
  }
  const string _pmin_s("pmin");
  unary_function_unary __pmin(&_pmin,_pmin_s);
  unary_function_ptr at_pmin (&__pmin,0,true);

  // a faire: vpotential, signtab
  gen _potential(const gen & g,GIAC_CONTEXT){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return symbolic(at_potential,g);
    vecteur v(plotpreprocess(g,contextptr));
    gen f=v[0];
    gen x=v[1];
    if ( (f.type!=_VECT) || (x.type!=_VECT) )
      setsizeerr();
    vecteur & fv=*f._VECTptr;
    vecteur & xv=*x._VECTptr;
    int s=fv.size();
    if (s!=xv.size())
      setdimerr();
    for (int i=0;i<s;++i){
      for (int j=i+1;j<s;++j){
	if (!is_zero(simplify(derive(fv[i],xv[j])-derive(fv[j],xv[i]))))
	  setsizeerr("Not a potential");
      }
    }
    gen res;
    for (int i=0;i<s;++i){
      res=res+integrate(simplify(fv[i]-derive(res,xv[i])),xv[i],contextptr);
    }
    return res;
  }
  const string _potential_s("potential");
  unary_function_eval __potential(&_potential,_potential_s);
  unary_function_ptr at_potential (&__potential,_QUOTE_ARGUMENTS,true);

  gen _vpotential(const gen & g,GIAC_CONTEXT){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return symbolic(at_vpotential,g);
    vecteur v(plotpreprocess(g,contextptr));
    gen f=v[0];
    gen x=v[1];
    if ( (f.type!=_VECT) || (x.type!=_VECT) )
      setsizeerr();
    vecteur & fv=*f._VECTptr;
    vecteur & xv=*x._VECTptr;
    int s=fv.size();
    if ( (s!=3) || (s!=xv.size()) )
      setdimerr();
    if (!is_zero(simplify(_divergence(g,contextptr))))
      setsizeerr("Not a vector potential");
    vecteur res(3);
    /* return A0=0, A1=int[B_2,x0], A2=-int[B_1,x0]+F(x1,x2)
     * where F=int[B0+d_2[int[B_2,x0]]+d_1[int[B_1,x0]],x1]
     * F does not depend on x0 since divergence[B]=0 */
    res[1]=integrate(fv[2],xv[0],contextptr);
    res[2]=integrate(fv[1],xv[0],contextptr);
    gen F=simplify(fv[0]+derive(res[1],xv[2])+derive(res[2],xv[1]));
    F=integrate(F,xv[1],contextptr);
    res[2]=F-res[2];
    return res;
  }
  const string _vpotential_s("vpotential");
  unary_function_eval __vpotential(&_vpotential,_vpotential_s);
  unary_function_ptr at_vpotential (&__vpotential,_QUOTE_ARGUMENTS,true);

  gen _poly2symb(const gen & g){
    return _r2e(g);
  }
  const string _poly2symb_s("poly2symb");
  unary_function_unary __poly2symb(&_poly2symb,_poly2symb_s);
  unary_function_ptr at_poly2symb (&__poly2symb,0,true);

  gen _symb2poly(const gen & g){
    return _e2r(g);
  }
  const string _symb2poly_s("symb2poly");
  unary_function_unary __symb2poly(&_symb2poly,_symb2poly_s);
  unary_function_ptr at_symb2poly (&__symb2poly,0,true);

  gen _exp2trig(const gen & g,GIAC_CONTEXT){
    return _sincos(g,contextptr);
  }
  const string _exp2trig_s("exp2trig");
  unary_function_eval __exp2trig(&_exp2trig,_exp2trig_s);
  unary_function_ptr at_exp2trig (&__exp2trig,0,true);

  gen _nrows(const gen & g){
    if (!ckmatrix(g))
      setsizeerr();
    return int(g._VECTptr->size());
  }
  const string _nrows_s("nrows");
  unary_function_unary __nrows(&_nrows,_nrows_s);
  unary_function_ptr at_nrows (&__nrows,0,true);

  gen _ncols(const gen & g){
    if (!ckmatrix(g))
      setsizeerr();
    if (g._VECTptr->empty())
      return zero;
    return int(g._VECTptr->front()._VECTptr->size());
  }
  const string _ncols_s("ncols");
  unary_function_unary __ncols(&_ncols,_ncols_s);
  unary_function_ptr at_ncols (&__ncols,0,true);

  gen _l2norm(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return abs(g,contextptr);
    vecteur v;
    if (ckmatrix(g))
      aplatir(*g._VECTptr,v);
    else
      v=*g._VECTptr;
    return l2norm(v,contextptr);
  }
  const string _l2norm_s("l2norm");
  unary_function_eval __l2norm(&_l2norm,_l2norm_s);
  unary_function_ptr at_l2norm (&__l2norm,0,true);

  const string _norm_s("norm");
  unary_function_eval __norm(&_l2norm,_norm_s);
  unary_function_ptr at_norm (&__norm,0,true);

  gen _normalize(const gen & a,GIAC_CONTEXT){
    return a/_l2norm(a,contextptr);
  }
  const string _normalize_s("normalize");
  unary_function_eval __normalize(&_normalize,_normalize_s);
  unary_function_ptr at_normalize (&__normalize,0,true);

  const string _randmatrix_s("randmatrix");
  unary_function_eval __randmatrix(&_ranm,_randmatrix_s);
  unary_function_ptr at_randmatrix (&__randmatrix,0,true);

  extern unary_function_ptr at_lgcd;
  gen _lgcd(const gen & g){
    if (g.type!=_VECT)
      return symbolic(at_lgcd,g);
    return lgcd(*g._VECTptr);
  }
  const string _lgcd_s("lgcd");
  unary_function_unary __lgcd(&_lgcd,_lgcd_s);
  unary_function_ptr at_lgcd (&__lgcd,0,true);

  // synonyms
  gen _float(const gen & g){
    return evalf(g);
  }
  const string _float_s("float");
  unary_function_unary __float(&_float,_float_s);
  unary_function_ptr at_float (&__float,0,true);

  gen _hold(const gen & g){
    return g;
  }
  const string _hold_s("hold");
  unary_function_unary __hold(&_hold,_hold_s);
  unary_function_ptr at_hold (&__hold,_QUOTE_ARGUMENTS,true);

  gen _eigenvals(const gen & g,GIAC_CONTEXT){
    return gen(*_diag(_egvl(g,contextptr))._VECTptr,_SEQ__VECT);
  }
  const string _eigenvals_s("eigenvals");
  unary_function_eval __eigenvals(&_eigenvals,_eigenvals_s);
  unary_function_ptr at_eigenvals (&__eigenvals,0,true);

  const string _giackernel_s("kernel");
  unary_function_unary __giackernel(&_ker,_giackernel_s);
  unary_function_ptr at_kernel (&__giackernel,0,true);

  const string _eigenvects_s("eigenvects");
  unary_function_eval __eigenvects(&_egv,_eigenvects_s);
  unary_function_ptr at_eigenvects (&__eigenvects,0,true);

  const string _eigenvalues_s("eigenvalues");
  unary_function_eval __eigenvalues(&_egvl,_eigenvalues_s);
  unary_function_ptr at_eigenvalues (&__eigenvalues,0,true);

  const string _charpoly_s("charpoly");
  unary_function_eval __charpoly(&giac::_pcar,_charpoly_s);
  unary_function_ptr at_charpoly (&__charpoly,0,true);

  const string _eigenvectors_s("eigenvectors");
  unary_function_eval __eigenvectors(&_egv,_eigenvectors_s);
  unary_function_ptr at_eigenvectors (&__eigenvectors,0,true);

  const string _rowdim_s("rowdim");
  unary_function_unary __rowdim(&_nrows,_rowdim_s);
  unary_function_ptr at_rowdim (&__rowdim,0,true);

  const string _coldim_s("coldim");
  unary_function_unary __coldim(&_ncols,_coldim_s);
  unary_function_ptr at_coldim (&__coldim,0,true);

  const string _multiply_s("multiply");
  unary_function_unary __multiply(&_prod,_multiply_s);
  unary_function_ptr at_multiply (&__multiply,0,true);

  /* Maple inert forms */
  gen _Gcd(const gen & g){
    return symbolic(at_gcd,g);
  }
  const string _Gcd_s("Gcd");
  unary_function_unary __Gcd(&_Gcd,_Gcd_s);
  unary_function_ptr at_Gcd (&__Gcd,0,true);

  gen _Factor(const gen & g){
    return symbolic(at_factor,g);
  }
  const string _Factor_s("Factor");
  unary_function_unary __Factor(&_Factor,_Factor_s);
  unary_function_ptr at_Factor (&__Factor,0,true);

  gen _Rref(const gen & g){
    return symbolic(at_rref,g);
  }
  const string _Rref_s("Rref");
  unary_function_unary __Rref(&_Rref,_Rref_s);
  unary_function_ptr at_Rref (&__Rref,0,true);

  gen _Det(const gen & g){
    return symbolic(at_det,g);
  }
  const string _Det_s("Det");
  unary_function_unary __Det(&_Det,_Det_s);
  unary_function_ptr at_Det (&__Det,0,true);

  gen _Quo(const gen & g){
    return symbolic(at_quo,g);
  }
  const string _Quo_s("Quo");
  unary_function_unary __Quo(&_Quo,_Quo_s);
  unary_function_ptr at_Quo (&__Quo,0,true);

  gen _Rem(const gen & g){
    return symbolic(at_rem,g);
  }
  const string _Rem_s("Rem");
  unary_function_unary __Rem(&_Rem,_Rem_s);
  unary_function_ptr at_Rem (&__Rem,0,true);

  gen _Int(const gen & g){
    return symbolic(at_integrate,g);
  }
  const string _Int_s("Int");
  unary_function_unary __Int(&_Int,_Int_s);
  unary_function_ptr at_Int (&__Int,0,true);

  gen _divisors(const gen & g){
    return gen(*_idivis(g)._VECTptr,_SET__VECT);
  }
  const string _divisors_s("divisors");
  unary_function_unary __divisors(&_divisors,_divisors_s);
  unary_function_ptr at_divisors (&__divisors,0,true);

  const string _maxnorm_s("maxnorm");
  unary_function_eval __maxnorm(&linfnorm,_maxnorm_s);
  unary_function_ptr at_maxnorm (&__maxnorm,0,true);

  gen _l1norm(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return linfnorm(g,contextptr);
    gen res;
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    for (;it!=itend;++it)
      res=res+linfnorm(g,contextptr);
    return res;
  }
  const string _l1norm_s("l1norm");
  unary_function_eval __l1norm(&_l1norm,_l1norm_s);
  unary_function_ptr at_l1norm (&__l1norm,0,true);

  gen _dotprod(const gen & g){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2))
      settypeerr();
    vecteur & v=*g._VECTptr;
    if ( (v[0].type!=_VECT) || (v[1].type!=_VECT) )
      settypeerr();
    vecteur &v0=*v[0]._VECTptr;
    vecteur &v1=*v[1]._VECTptr;
    return dotvecteur(v0,v1);
  }
  const string _dotprod_s("dotprod");
  unary_function_unary __dotprod(&_dotprod,_dotprod_s);
  unary_function_ptr at_dotprod (&__dotprod,0,true);

  const string _crossproduct_s("crossproduct");
  unary_function_unary __crossproduct(&_cross,_crossproduct_s);
  unary_function_ptr at_crossproduct (&__crossproduct,0,true);

  gen _diag(const gen & g){
    if (g.type!=_VECT || g._VECTptr->empty())
      setsizeerr();
    vecteur v=*g._VECTptr;
    int l=v.size();
    if (is_squarematrix(v)){
      vecteur res(l);
      for (int i=0;i<l;++i)
	res[i]=v[i][i];
      return res;
    }
    if (ckmatrix(v)){
      if (l==1 && v[0].type==_VECT){
	v=*v[0]._VECTptr;
      }
      else
	v=*mtran(v)[0]._VECTptr;
    }
    l=v.size();
    matrice res;
    if (l && ckmatrix(v.front()) ){
      int s=0,r=0;
      for (int i=0;i<l;++i){
	if (!is_squarematrix(v[i]))
	  settypeerr();
	s += v[i]._VECTptr->size();
      }
      for (int i=0;i<l;++i){
	vecteur & current=*v[i]._VECTptr;
	int c=current.size();
	for (int j=0;j<c;++j){
	  vecteur tmp(r);
	  vecteur & currentj=*current[j]._VECTptr;
	  for (int k=0;k<c;++k){
	    tmp.push_back(currentj[k]);
	  }
	  for (int k=c+r;k<s;++k)
	    tmp.push_back(zero);
	  res.push_back(tmp);
	}
	r += c;
      }
      return res;
    }
    for (int i=0;i<l;++i){
      vecteur tmp(i);
      tmp.push_back(v[i]);
      res.push_back(mergevecteur(tmp,vecteur(l-1-i)));
    }
    return res;
  }
  const string _diag_s("diag");
  unary_function_unary __diag(&_diag,_diag_s);
  unary_function_ptr at_diag (&__diag,0,true);

  const string _BlockDiagonal_s("BlockDiagonal");
  unary_function_unary __BlockDiagonal(&_diag,_BlockDiagonal_s);
  unary_function_ptr at_BlockDiagonal (&__BlockDiagonal,0,true);

  gen _input(const gen & args){
    return _input(args,false);
  }
  const string _input_s("input");
  unary_function_unary __input(&_input,_input_s);
  unary_function_ptr at_input (&__input,_QUOTE_ARGUMENTS,true);

  gen _textinput(const gen & args){
    return _input(args,true);
  }
  const string _textinput_s("textinput");
  unary_function_unary __textinput(&_textinput,_textinput_s);
  unary_function_ptr at_textinput (&__textinput,_QUOTE_ARGUMENTS,true);

  const string _f2nd_s("f2nd");
  unary_function_unary __f2nd(&_fxnd,_f2nd_s);
  unary_function_ptr at_f2nd (&__f2nd,0,true);

  // service=-3 for content -2 for primpart -1 for coeff, degree for coeff
  gen primpartcontent(const gen& g,int service){
    vecteur v(gen2vecteur(g));
    int s=v.size();
    if (s==2 && v[1].is_symb_of_sommet(at_pow)){
      gen & f = v[1]._SYMBptr->feuille;
      if (f.type==_VECT && f._VECTptr->size()==2 && f._VECTptr->back().type==_INT_){
	v[1]=f._VECTptr->front();
	service=f._VECTptr->back().val;
      }
    }
    if (s>=2 && v[1].type==_VECT){
      vecteur l(*v[1]._VECTptr);
      int outerdim=l.size();
      lvar(v[0],l);
      int innerdim=l.size()-outerdim;
      fraction f(e2r(v[0],l));
      vecteur ll(l.begin()+outerdim,l.end());
      if (f.num.type!=_POLY){
	if (service==-1){
	  gen res=r2e(v[0],l);
	  if (s==3){
	    if (is_zero(v[2]))
	      return res;
	    else
	      return zero;
	  }
	  return makevecteur(res);
	}
	if (service==-2)
	  return r2e(inv(f.den),l);
	if (service==-3)
	  return r2e(f.num,l);
	setsizeerr();
      }
      polynome & p_aplati=*f.num._POLYptr;
      polynome p=splitmultivarpoly(p_aplati,innerdim); 
      vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
      vecteur coeffs;
      coeffs.reserve(itend-it);
      for (;it!=itend;++it)
	coeffs.push_back(it->value);
      if (service==-1){
	gen gden=r2e(f.den,l);
	if (s==3 && v[2].type==_VECT){
	  index_t ind;
	  if (!vecteur2index(*v[2]._VECTptr,ind))
	    return zero;
	  index_m i(ind);
	  it=p.coord.begin();
	  for (;it!=itend;++it){
	    if (it->index==i)
	      return r2e(it->value,ll)/gden;
	  }
	  return zero;
	}
	return r2e(coeffs,ll)/gden;
      }
      if (service==-2){
	p=p/_lgcd(coeffs);
	p=unsplitmultivarpoly(p,innerdim);
	return r2e(p/f.den,l);
      }
      if (service==-3)
	return r2e(_lgcd(coeffs),ll);
      setsizeerr();
    }
    if (s!=1 && s!=2)
      setsizeerr();
    gen x(vx_var);
    if (s==2)
      x=v[1];
    gen f(_e2r(makevecteur(v[0],x)));
    if (f.type!=_VECT){
      switch(service){
      case -1:
	return makevecteur(f);
      case -2:
	return plus_one;
      case -3:
	return f;
      default:
	if (service>0)
	  return zero;
	else
	  return f;
      }
    }
    switch (service){
    case -1:
      return f;
    case -2:
      return symb_horner(*f._VECTptr/_lgcd(f),x);
    case -3:
      return _lgcd(f);
    }
    vecteur & w=*f._VECTptr;
    int ss=w.size();
    if (service>=ss)
      return zero;
    return w[ss-service-1];
  }
  gen _primpart(const gen & g){
    return primpartcontent(g,-2);
  }
  const string _primpart_s("primpart");
  unary_function_unary __primpart(&_primpart,_primpart_s);
  unary_function_ptr at_primpart (&__primpart,0,true);

  gen _content(const gen & g){
    return primpartcontent(g,-3);
  }
  const string _content_s("content");
  unary_function_unary __content(&_content,_content_s);
  unary_function_ptr at_content (&__content,0,true);

  const string _icontent_s("icontent");
  unary_function_unary __icontent(&_content,_icontent_s);
  unary_function_ptr at_icontent (&__icontent,0,true);

  gen _coeff(const gen & g){
    if (g.type==_VECT && !g._VECTptr->empty() && g._VECTptr->back().type==_INT_){
      vecteur v=*g._VECTptr;
      int n=absint(v.back().val);
      v.pop_back();
      return primpartcontent(v,n);
    }
    if (maple_mode==1 && g.type==_VECT && g._VECTptr->size()==2 && g._VECTptr->back().type==_IDNT){
      return primpartcontent(g,1);
    }
    return primpartcontent(g,-1);
  }
  const string _coeff_s("coeff");
  unary_function_unary __coeff(&_coeff,_coeff_s);
  unary_function_ptr at_coeff (&__coeff,0,true);

  const string _coeffs_s("coeffs");
  unary_function_unary __coeffs(&_coeff,_coeffs_s);
  unary_function_ptr at_coeffs (&__coeffs,0,true);

  const string _ichrem_s("ichrem");
  unary_function_unary __ichrem(&_ichinrem,_ichrem_s);
  unary_function_ptr at_ichrem (&__ichrem,0,true);

  gen _chrem(const gen & g){
    if (!ckmatrix(g) || g._VECTptr->size()!=2)
      setsizeerr();
    matrice m=mtran(*g._VECTptr);
    const_iterateur it=m.begin(),itend=m.end();
    if (it==itend)
      setsizeerr();
    gen res=*it;
    for (++it;it!=itend;++it){
      res=_ichinrem(makevecteur(res,*it));
    }
    return res;
  }
  const string _chrem_s("chrem");
  unary_function_unary __chrem(&_chrem,_chrem_s);
  unary_function_ptr at_chrem (&__chrem,0,true);

  gen _genpoly(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=3)
      settypeerr();
    vecteur & v=*g._VECTptr;
    gen n=v[0],b=v[1],x=v[2];
    if (b.type!=_INT_ && b.type!=_ZINT)
      settypeerr();
    b=abs(b,0);
    if (is_zero(b)||is_one(b))
      setsizeerr();
    vecteur l(lvar(n));
    fraction f(e2r(n,l));
    if (is_integer(f.num))
      f.num=pzadic(polynome(f.num,0),b);
    else {
      if (f.num.type==_POLY)
	f.num=pzadic(*f.num._POLYptr,b);
    }
    if (is_integer(f.den))
      f.den=pzadic(polynome(f.den,0),b);
    else {
      if (f.den.type==_POLY)
	f.den=pzadic(*f.den._POLYptr,b);
    }
    l.insert(l.begin(),x);
    return r2e(f,l);
  }
  const string _genpoly_s("genpoly");
  unary_function_unary __genpoly(&_genpoly,_genpoly_s);
  unary_function_ptr at_genpoly (&__genpoly,0,true);

  gen freq_quantile(const matrice & v,double d){
    if (!ckmatrix(v))
      return undef;
    matrice w;
    if (v.size()!=2)
      w=mtran(v);
    else
      w=v;
    if (w.size()!=2)
      return undef;
    // Row Sort (using row 1)
    w=ascsort(w,false);
    // w[0]=data, w[1]=frequencies
    vecteur data=*w[0]._VECTptr;
    vecteur freq=*w[1]._VECTptr;
    gen sigma=d*prodsum(freq,false);
    int s=freq.size();
    gen partial_sum;
    for (int i=0;i<s;++i){
      partial_sum=partial_sum+freq[i];
      if (is_greater(partial_sum,sigma,0))
	return data[i];
    }
    return undef;
  }
  gen _median(const gen & g){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.5);
    if (!ckmatrix(v))
      return mtran(ascsort(mtran(vecteur(1,v)),true))[(v.size()-1)/2][0];
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[(v.size()-1)/2];
  }
  const string _median_s("median");
  unary_function_unary unary_median(&_median,_median_s);
  unary_function_ptr at_median (&unary_median,0,true);

  gen _quartile1(const gen & g){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.25);
    if (!ckmatrix(v))
      return mtran(ascsort(mtran(vecteur(1,v)),true))[(v.size()-1)/4][0];
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[(v.size()-1)/4];
  }
  const string _quartile1_s("quartile1");
  unary_function_unary unary_quartile1(&_quartile1,_quartile1_s);
  unary_function_ptr at_quartile1 (&unary_quartile1,0,true);

  gen _quartile3(const gen & g){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.75);
    if (!ckmatrix(v))
      return mtran(ascsort(mtran(vecteur(1,v)),true))[(3*(v.size()-1))/4][0];
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[(3*(v.size()-1))/4];
  }
  const string _quartile3_s("quartile3");
  unary_function_unary unary_quartile3(&_quartile3,_quartile3_s);
  unary_function_ptr at_quartile3 (&unary_quartile3,0,true);

  gen _quantile(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()<2 || v.front().type!=_VECT || v.back().type!=_DOUBLE_)
      setsizeerr();
    double d=v.back()._DOUBLE_val;
    if (d<0 || d>=1)
      setdimerr("");
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==3)
      return freq_quantile(makevecteur(v[0],v[1]),d);
    v=*v.front()._VECTptr;
    if (!ckmatrix(v))
      v=ascsort(mtran(vecteur(1,v)),true);
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[int(d*(v.size()-1))];
  }
  const string _quantile_s("quantile");
  unary_function_unary unary_quantile(&_quantile,_quantile_s);
  unary_function_ptr at_quantile (&unary_quantile,0,true);

  gen _quartiles(const gen & g){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return makevecteur(freq_quantile(v,0.0),freq_quantile(v,0.25),freq_quantile(v,0.5),freq_quantile(v,0.75),freq_quantile(v,1.0));
    if (!ckmatrix(v))
      v=ascsort(mtran(vecteur(1,v)),true);
    else
      v=ascsort(v,true);
    v=mtran(v);
    int s=v.size()-1;
    if (s<0)
      setsizeerr();
    return makevecteur(v[0],v[s/4],v[s/2],v[(3*s)/4],v[s]);
  }
  const string _quartiles_s("quartiles");
  unary_function_unary unary_quartiles(&_quartiles,_quartiles_s);
  unary_function_ptr at_quartiles (&unary_quartiles,0,true);

  gen _moustache(const gen & g,GIAC_CONTEXT){
    vecteur v0(*_quartiles(g)._VECTptr),v;
    if (!ckmatrix(v0))
      v=vecteur(1,v0);
    else
      v=mtran(v0);
    _Pictsize(0);
    int s=v.size();
    vecteur res;
    int vertical_pixels=plot_instructionsh;;
    double y_scale=(global_window_ymax-global_window_ymin)/vertical_pixels;
    double xmin=gnuplot_xmin,xmax=gnuplot_xmax;
    gen tmpx=_min(v0[0],contextptr),tmpxx=_max(v0[4],contextptr);
    if (tmpx.type==_DOUBLE_)
      xmin=tmpx._DOUBLE_val;
    if (tmpxx.type==_DOUBLE_)
      xmax=tmpxx._DOUBLE_val;
    for (int i=0;i<s;++i){
      double y_middle=global_window_ymax-(40*i+40)*y_scale;
      double y_up=global_window_ymax-(40*i+30)*y_scale;
      double y_down=global_window_ymax-(40*i+50)*y_scale;
      vecteur current=gen2vecteur(v[i]);
      if (current.size()!=5)
	continue;
      // trait min -> 1er quartile
      res.push_back(_segment(makevecteur(current[0]+y_middle*cst_i,current[1]+y_middle*cst_i),contextptr));
      // traits verticaux
      for (int j=1;j<=3;++j){
	res.push_back(_segment(makevecteur(current[j]+y_down*cst_i,current[j]+y_up*cst_i),contextptr));
      }
      // trait horizontaux quartiles 1->3
      res.push_back(_segment(makevecteur(current[1]+y_up*cst_i,current[3]+y_up*cst_i),contextptr));
      res.push_back(_segment(makevecteur(current[1]+y_down*cst_i,current[3]+y_down*cst_i),contextptr));
      // trait 3eme quartile -> fin
      res.push_back(_segment(makevecteur(current[3]+y_middle*cst_i,current[4]+y_middle*cst_i),contextptr));
    }
    eval(xyztrange(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,xmin,xmax,global_window_ymin,global_window_ymax,show_axes,class_minimum,class_size,gnuplot_hidden3d));
    return gen(res,_SEQ__VECT);
  }
  const string _moustache_s("moustache");
  unary_function_eval unary_moustache(&_moustache,_moustache_s);
  unary_function_ptr at_moustache (&unary_moustache,0,true);


  gen stddevmean(const vecteur & v,int withstddev,int xcol,int freqcol){
    int sv=v.size();
    if (xcol>=sv || freqcol>=sv)
      setdimerr();
    if (v[xcol].type!=_VECT || v[freqcol].type!=_VECT)
      setsizeerr();
    vecteur v1(*v[xcol]._VECTptr),v2(*v[freqcol]._VECTptr);
    // if v1 is made of intervals replace by the center of these intervals
    iterateur it=v1.begin(),itend=v1.end();
    for (;it!=itend;++it){
      if (it->is_symb_of_sommet(at_interval)){
	gen & f=it->_SYMBptr->feuille;
	if (f.type==_VECT && f._VECTptr->size()==2)
	  *it=(f._VECTptr->front()+f._VECTptr->back())/2;
      }
    }
    if (ckmatrix(v1) ^ ckmatrix(v2))
      setsizeerr();
    int n=v1.size();
    if (n!=v2.size())
      setsizeerr();
    gen m,m2,s;
    for (int i=0;i<n;++i){
      s = s + v2[i];
      m = m + apply(v2[i],v1[i],prod);
      if (withstddev)
	m2 = m2 + apply(v2[i],apply(v1[i],v1[i],prod),prod);
    }
    m = apply(m,s,rdiv);
    if (withstddev){
      m2=m2-apply(s,apply(m,m,prod),prod);
      m2=apply(m2,s-(withstddev==2),rdiv);
      if (withstddev==3)
	return m2;
      return apply(m2,sqrt);
    }
    else
      return m;
  }
  // withstddev=0 (mean), 1 (stddev divided by n), 2 (by n-1), 3 (variance)
  gen stddevmean(const gen & g,int withstddev){
    vecteur & v=*g._VECTptr;
    int s=v.size();
    if (s<2)
      setsizeerr();
    if (v[1].type!=_INT_)
      return stddevmean(v,withstddev,0,1);
    if (v[0].type!=_VECT)
      setsizeerr();
    int xcol=v[1].val;
    int freqcol=xcol+1;
    if (s>2 && v[2].type==_INT_)
      freqcol=v[2].val;
    return stddevmean(mtran(*v[0]._VECTptr),withstddev,xcol,freqcol);
  }

  gen _mean(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,0);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return mean(mtran(vecteur(1,v)),true)[0];
    else
      v=mean(v,true);
    return v;
  }
  const string _mean_s("mean");
  unary_function_unary __mean(&_mean,_mean_s);
  unary_function_ptr at_mean (&__mean,0,true);

  gen _stdDev(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,2);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,2)[0];
    else
      v=stddev(v,true,2);
    return v;
  }
  const string _stdDev_s("stdDev");
  unary_function_unary __stdDev(&_stdDev,_stdDev_s);
  unary_function_ptr at_stdDev (&__stdDev,0,true);

  const string _stddevp_s("stddevp");
  unary_function_unary __stddevp(&_stdDev,_stddevp_s);
  unary_function_ptr at_stddevp (&__stddevp,0,true);

  gen _stddev(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,1);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,1)[0];
    else
      v=stddev(v,true,1);
    return v;
  }
  const string _stddev_s("stddev");
  unary_function_unary __stddev(&_stddev,_stddev_s);
  unary_function_ptr at_stddev (&__stddev,0,true);

  gen _variance(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,3);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,3)[0];
    else
      v=stddev(v,true,3);
    return v;
  }
  const string _variance_s("variance");
  unary_function_unary __variance(&_variance,_variance_s);
  unary_function_ptr at_variance (&__variance,0,true);

  vecteur covariance_correlation(const gen & g,const gen & u1,const gen & u2,int xcol,int ycol,int freqcol,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v) || v.empty() || v.front()._VECTptr->size()<2)
      return makevecteur(undef,undef);
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpy,n,freq;
    if (freqcol<-1){
      // g is interpreted as a double-entry table with 1st col = x-values
      // 1st line=y-values, line/col is a frequency for this value of x/y
      int r,c;
      mdims(v,r,c);
      if (r<2 || c<2)
	setdimerr();
      vecteur & vy=*v[0]._VECTptr;
      for (int i=1;i<r;++i){
	vecteur & w=*v[i]._VECTptr;
	gen & currentx=w[0];
	for (int j=1;j<c;++j){
	  gen & currenty=vy[j];
	  freq=w[j];
	  n=n+freq;
	  sigmax=sigmax+currentx*freq;
	  sigmax2=sigmax2+currentx*currentx*freq;
	  sigmay=sigmay+currenty*freq;
	  sigmay2=sigmay2+currenty*currenty*freq;
	  sigmaxy=sigmaxy+currentx*currenty*freq;
	}
      }
    }
    else {
      const_iterateur it=v.begin(),itend=v.end();
      int s=it->_VECTptr->size();
      if (xcol>=s || ycol>=s || freqcol >=s)
	setdimerr();
      for (;it!=itend;++it){
	vecteur & w=*it->_VECTptr;
	if (u1.type==_FUNC)
	  tmpx=u1(w[xcol],contextptr);
	else
	  tmpx=w[xcol];
	if (u2.type==_FUNC)
	  tmpy=u2(w[ycol],contextptr);
	else
	  tmpy=w[ycol];
	if (freqcol>=0)
	  freq=w[freqcol];
	else
	  freq=plus_one;
	n = n+freq;
	sigmax = sigmax + tmpx*freq;
	sigmax2 = sigmax2 + tmpx*tmpx*freq;
	sigmay = sigmay + tmpy*freq;
	sigmay2 = sigmay2 + tmpy*tmpy*freq;
	sigmaxy = sigmaxy + tmpx*tmpy*freq;
      }
    }
    gen covariance=(n*sigmaxy-sigmax*sigmay)/(n*n);
    gen correlation=(n*sigmaxy-sigmax*sigmay)/sqrt((n*sigmax2-sigmax*sigmax)*(n*sigmay2-sigmay*sigmay));
    return makevecteur(covariance,correlation);
  }
  
  void find_xyfreq(const gen & g,gen & gv,int & xcol,int & ycol,int &freqcol){
    xcol=0;
    ycol=1;
    freqcol=-1;
    if (g.type==_VECT && g.subtype==_SEQ__VECT && !g._VECTptr->empty()){
      vecteur v=*g._VECTptr;
      if (v[0].type!=_VECT)
	setsizeerr();
      int s=v.size();
      if (s==3 && v[1].type==_VECT){
	if (!ckmatrix(v[2]))
	  v[2]=_diag(v[2]);
	int n,c;
	mdims(*v[2]._VECTptr,n,c);
	if (n==v[0]._VECTptr->size() && c==v[1]._VECTptr->size()){
	  vecteur v0(*v[1]._VECTptr);
	  v0.insert(v0.begin(),zero);
	  matrice m(mtran(*v[2]._VECTptr));
	  m.insert(m.begin(),v[0]);
	  m=mtran(m);
	  m.insert(m.begin(),v0);
	  gv=m;
	  freqcol=-2;
	  return;
	}
      }
      if (s>1) {
	if (v[1].type==_INT_){
	  xcol=v[1].val;
	  if (xcol<0)
	    freqcol=-2;
	}
	else {
	  if (!ckmatrix(v))
	    setsizeerr();
	  gv=mtran(v);
	  return;
	}
      }
      if (s>2 && v[2].type==_INT_)
	ycol=v[2].val;
      if (s>3 && v[3].type==_INT_)
	freqcol=v[3].val;
      gv=v[0];
    }
    else {
      if (!ckmatrix(g) || g._VECTptr->empty())
	setsizeerr();
      gv=g;
      if (g._VECTptr->front()._VECTptr->size()>2)
	freqcol=2;
      if (g._VECTptr->front()._VECTptr->front().type==_STRNG)
	freqcol=-2;
    }
  }
  gen _covariance_correlation(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr);      
  }
  const string _covariance_correlation_s("covariance_correlation");
  unary_function_eval __covariance_correlation(&_covariance_correlation,_covariance_correlation_s);
  unary_function_ptr at_covariance_correlation (&__covariance_correlation,0,true);

  gen _covariance(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr)[0];
  }
  const string _covariance_s("covariance");
  unary_function_eval __covariance(&_covariance,_covariance_s);
  unary_function_ptr at_covariance (&__covariance,0,true);

  gen _correlation(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr)[1];
  }
  const string _correlation_s("correlation");
  unary_function_eval __correlation(&_correlation,_correlation_s);
  unary_function_ptr at_correlation (&__correlation,0,true);

  gen _interval2center(const gen & g){
    if (g.type==_VECT)
      return apply(g,_interval2center);
    if (g.is_symb_of_sommet(at_interval)){
      gen & tmp=g._SYMBptr->feuille;
      if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	setsizeerr();
      vecteur & v=*tmp._VECTptr;
      return (v.front()+v.back())/2;
    }
    return g;
  }
  const string _interval2center_s("interval2center");
  unary_function_unary __interval2center(&_interval2center,_interval2center_s);
  unary_function_ptr at_interval2center (&__interval2center,0,true);

  void function_regression(const gen & g,const gen & u1,const gen & u2,gen & a,gen &b,GIAC_CONTEXT){
    gen gv,freq;
    int xcol,ycol,freqcol;
    find_xyfreq(g,gv,xcol,ycol,freqcol);
    if (!ckmatrix(gv))
      setsizeerr();
    vecteur & v = *gv._VECTptr;
    gen n;
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpy;
    if (freqcol<-1){
      int r,c;
      mdims(v,r,c);
      if (r<2 || c<2)
	setdimerr();
      vecteur & vy=*v[0]._VECTptr;
      gen currentx,currenty;
      for (int i=1;i<r;++i){
	vecteur & w=*v[i]._VECTptr;
	if (u1.type==_FUNC)
	  currentx=u1(w[0],contextptr);
	else
	  currentx=w[0];
	for (int j=1;j<c;++j){
	  if (u2.type==_FUNC)
	    currenty=u2(vy[j],contextptr);
	  else
	    currenty=vy[j];
	  currenty=_interval2center(currenty);
	  freq=w[j];
	  n=n+freq;
	  sigmax=sigmax+currentx*freq;
	  sigmax2=sigmax2+currentx*currentx*freq;
	  sigmay=sigmay+currenty*freq;
	  // sigmay2=sigmay2+currenty*currenty*freq;
	  sigmaxy=sigmaxy+currentx*currenty*freq;
	}
      }
    }
    else {
      const_iterateur it=v.begin(),itend=v.end();
      for (;it!=itend;++it){
	vecteur & w=*it->_VECTptr;
	if (u1.type==_FUNC)
	  tmpx=u1(w[xcol],contextptr);
	else
	  tmpx=w[xcol];
	tmpx=_interval2center(tmpx);
	if (u2.type==_FUNC)
	  tmpy=u2(w[ycol],contextptr);
	else
	  tmpy=w[ycol];
	tmpy=_interval2center(tmpy);
	if (freqcol<0)
	  freq=plus_one;
	else
	  freq=w[freqcol];
	sigmax = sigmax + freq*tmpx;
	sigmax2 = sigmax2 + freq*tmpx*tmpx;
	sigmay = sigmay + freq*tmpy;
	// sigmay2 = sigmay2 + w.back()*w.back();
	sigmaxy = sigmaxy + freq*tmpx*tmpy;
	n = n + freq;
      }
    }
    a=(n*sigmaxy-sigmax*sigmay)/(n*sigmax2-sigmax*sigmax);
    b=(sigmay-a*sigmax)/n;
  }

  gen function_regression(const gen & g,const gen & u1,const gen & u2,GIAC_CONTEXT){
    gen a,b;
    function_regression(g,u1,u2,a,b,contextptr);
    return gen(makevecteur(a,b),_SEQ__VECT);
  }

  gen _linear_regression(const gen & g){
    return function_regression(g,zero,zero,0);
  }
  const string _linear_regression_s("linear_regression");
  unary_function_unary __linear_regression(&_linear_regression,_linear_regression_s);
  unary_function_ptr at_linear_regression (&__linear_regression,0,true);


  gen _exponential_regression(const gen & g,GIAC_CONTEXT){
    return exp(function_regression(g,zero,at_ln,contextptr));
  }
  const string _exponential_regression_s("exponential_regression");
  unary_function_eval __exponential_regression(&_exponential_regression,_exponential_regression_s);
  unary_function_ptr at_exponential_regression (&__exponential_regression,0,true);

  gen _logarithmic_regression(const gen & g,GIAC_CONTEXT){
    return function_regression(g,at_ln,zero,contextptr);
  }
  const string _logarithmic_regression_s("logarithmic_regression");
  unary_function_eval __logarithmic_regression(&_logarithmic_regression,_logarithmic_regression_s);
  unary_function_ptr at_logarithmic_regression (&__logarithmic_regression,0,true);

  gen _power_regression(const gen & g,GIAC_CONTEXT){
    gen res= function_regression(g,at_ln,at_ln,contextptr);
    if (res.type==_VECT && res._VECTptr->size()==2){
      vecteur v(*res._VECTptr);
      v[1]=exp(v[1]);
      return gen(v,_SEQ__VECT);
    }
    return res;
  }
  const string _power_regression_s("power_regression");
  unary_function_eval __power_regression(&_power_regression,_power_regression_s);
  unary_function_ptr at_power_regression (&__power_regression,0,true);

  gen _linear_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b;
    function_regression(g,zero,zero,a,b,0);
    return _droite(makevecteur(b*cst_i,1+(b+a)*cst_i),contextptr);
  }
  const string _linear_regression_plot_s("linear_regression_plot");
  unary_function_eval __linear_regression_plot(&_linear_regression_plot,_linear_regression_plot_s);
  unary_function_ptr at_linear_regression_plot (&__linear_regression_plot,0,true);

  gen _exponential_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b;
    function_regression(g,zero,at_ln,a,b,contextptr);
    return _plotfunc(evalf(exp(b))*exp(a*vx_var),contextptr);
  }
  const string _exponential_regression_plot_s("exponential_regression_plot");
  unary_function_eval __exponential_regression_plot(&_exponential_regression_plot,_exponential_regression_plot_s);
  unary_function_ptr at_exponential_regression_plot (&__exponential_regression_plot,0,true);

  gen _logarithmic_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b;
    function_regression(g,at_ln,zero,a,b,contextptr);
    return _plotfunc(a*ln(vx_var)+b,contextptr);    
  }
  const string _logarithmic_regression_plot_s("logarithmic_regression_plot");
  unary_function_eval __logarithmic_regression_plot(&_logarithmic_regression_plot,_logarithmic_regression_plot_s);
  unary_function_ptr at_logarithmic_regression_plot (&__logarithmic_regression_plot,0,true);

  gen _power_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b;
    function_regression(g,at_ln,at_ln,a,b,contextptr);
    return _plotfunc(exp(b)*pow(vx_var,a),contextptr);
  }
  const string _power_regression_plot_s("power_regression_plot");
  unary_function_eval __power_regression_plot(&_power_regression_plot,_power_regression_plot_s);
  unary_function_ptr at_power_regression_plot (&__power_regression_plot,0,true);

  gen polynomial_regression(const gen & g,int d,const gen & u1, const gen & u2,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v) || v.empty() || v.front()._VECTptr->size()<2)
      return undef;
    // use first and second column
    const_iterateur it=v.begin(),itend=v.end();
    int n(itend-it);
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpy;
    vecteur xmoment(2*d+1),xymoment(d+1);
    for (;it!=itend;++it){
      vecteur & w=*it->_VECTptr;
      if (u1.type==_FUNC)
	tmpx=u1(w.front(),contextptr);
      else
	tmpx=w.front();
      if (u2.type==_FUNC)
	tmpy=u2(w.back(),contextptr);
      else
	tmpy=w.back();
      xmoment[0]=xmoment[0]+1;
      xymoment[0]=xymoment[0]+tmpy;
      for (int i=1;i<=2*d;++i)
	xmoment[i]=xmoment[i]+pow(tmpx,i);
      for (int i=1;i<=d;++i)
	xymoment[i]=xymoment[i]+pow(tmpx,i)*tmpy;      
    }
    // make linear system
    matrice mat;
    for (int i=0;i<=d;++i){
      vecteur tmp;
      for (int j=d;j>=0;--j){
	tmp.push_back(xmoment[i+j]);
      }
      mat.push_back(tmp);
    }
    return linsolve(mat,xymoment);
  }
  gen _polynomial_regression(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g._VECTptr->size()==3){
      vecteur & v=*g._VECTptr;
      if (v[0].type==_VECT && v[1].type==_VECT && v[0]._VECTptr->size()==v[1]._VECTptr->size())
	return _polynomial_regression(makevecteur(mtran(makevecteur(v[0],v[1])),v[2]),contextptr);
    }
    if (g.type!=_VECT || g._VECTptr->size()!=2 || g._VECTptr->back().type!=_INT_)
      setsizeerr();
    return polynomial_regression(g._VECTptr->front(),absint(g._VECTptr->back().val),zero,zero,contextptr);
  }
  const string _polynomial_regression_s("polynomial_regression");
  unary_function_eval __polynomial_regression(&_polynomial_regression,_polynomial_regression_s);
  unary_function_ptr at_polynomial_regression (&__polynomial_regression,0,true);

  vector<double> prepare_effectifs(const vecteur & v){
    if (v.empty())
      setdimerr();
    vecteur w;
    if (ckmatrix(v)){
      int s=v.front()._VECTptr->size();
      if (s==1)
	w=*evalf_double(mtran(v)[0])._VECTptr;
      else
	setdimerr();
    }
    else
      w=*evalf_double(v)._VECTptr;
    // vector will be sorted keeping only DOUBLE data
    int s=w.size();
    vector<double> w1;
    w1.reserve(s);
    for (int i=0;i<s;++i){
      if (w[i].type==_DOUBLE_)
	w1.push_back(w[i]._DOUBLE_val);
    }
    sort(w1.begin(),w1.end());
    s=w1.size();
    if (!s)
      setdimerr();
    return w1;
  }
  matrice effectifs(const vecteur & data,double class_minimum,double class_size){
    if (data.empty())
      return data;
    vector<double>  w1;
    if (ckmatrix(data)){
      if (!data.empty() && data.front()._VECTptr->size()>1){
	matrice tmp=mtran(ascsort(data,false));
	vecteur tmpval=*evalf_double(tmp[0])._VECTptr;
	vecteur tmpeff=*tmp[1]._VECTptr;
	if (tmpval.front().type!=_DOUBLE_ || tmpval.back().type!=_DOUBLE_)
	  setsizeerr();
	double kbegin=floor((tmpval.front()._DOUBLE_val-class_minimum)/class_size);
	double kend=floor((tmpval.back()._DOUBLE_val-class_minimum)/class_size);
	int s=tmpval.size(),i=0;
	vecteur res;
	for (;kbegin<=kend;++kbegin){
	  // count in this class
	  double min_class=kbegin*class_size+class_minimum;
	  double max_class=min_class+class_size;
	  gen effectif;
	  for (;i<s;effectif=effectif+tmpeff[i],++i){
	    if (tmpval[i].type!=_DOUBLE_)
	      setsizeerr();
	    if (tmpval[i]._DOUBLE_val>=max_class)
	      break;
	  }
	  res.push_back(makevecteur(symbolic(at_interval,makevecteur(min_class,max_class)),effectif));
	}
	return res;
      }
      w1=prepare_effectifs(*mtran(data)[0]._VECTptr);
    }
    else
      w1=prepare_effectifs(data);
    // class_min + k*class_size <= mini hence k
    double kbegin=floor((w1.front()-class_minimum)/class_size);
    double kend=floor((w1.back()-class_minimum)/class_size);
    vector<double>::const_iterator it=w1.begin(),itend=w1.end();
    vecteur res;
    for (;kbegin<=kend;++kbegin){
      // count in this class
      double min_class=kbegin*class_size+class_minimum;
      double max_class=min_class+class_size;
      int effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=max_class)
	  break;
      }
      res.push_back(makevecteur(symbolic(at_interval,makevecteur(min_class,max_class)),effectif));
    }
    return res;
  }

  matrice effectifs(const vecteur & data,const vecteur & intervalles){
    int s=intervalles.size();
    matrice res(s);
    vector<double> sorted_data;
    if (ckmatrix(data))
      sorted_data=prepare_effectifs(*mtran(data)[0]._VECTptr);
    else
      sorted_data=prepare_effectifs(data);
    vector<double>::const_iterator it=sorted_data.begin(),itend=sorted_data.end();
    for (int i=0;i<s;++i){
      gen cur_intervalle=intervalles[i];
      double debut,fin;
      if (!chk_double_interval(cur_intervalle,debut,fin))
	setsizeerr();
      for (;it!=itend;++it){
	if (*it>=debut)
	  break;
      }
      int effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=fin)
	  break;
      }
      res[i]=makevecteur(cur_intervalle,effectif);
    }
    return res;
  }

  vecteur centres2intervalles(const vecteur & centres,double class_min,bool with_class_min){
    if (centres.size()<2)
      setsizeerr();
    double d0=evalf_double(centres[0])._DOUBLE_val,d1=evalf_double(centres[1])._DOUBLE_val;
    double debut=class_min;
    if (!with_class_min)
      debut=d0+(d0-d1)/2;
    vecteur res;
    const_iterateur it=centres.begin(),itend=centres.end();
    res.reserve(itend-it);
    for (;it!=itend;++it){
      gen g=evalf_double(*it);
      if (g.type!=_DOUBLE_)
	setsizeerr();
      double milieu=g._DOUBLE_val;
      double fin=milieu+(milieu-debut);
      if (fin<=debut)
	setsizeerr();
      res.push_back(symb_interval(debut,fin));
      debut=fin;
    }
    return res;
  }

  gen _center2interval(const gen & g){
    if (g.type!=_VECT)
      settypeerr();
    if (g.subtype==_SEQ__VECT && g._VECTptr->size()==2){
      vecteur & v=*g._VECTptr;
      gen c=evalf_double(v[1]);
      if (v[0].type!=_VECT || c.type!=_DOUBLE_)
	settypeerr();
      return gen(centres2intervalles(*v[0]._VECTptr,c._DOUBLE_val,true),_SEQ__VECT);
    }
    return gen(centres2intervalles(*g._VECTptr,0.0,false),_SEQ__VECT);
  }
  const string _center2interval_s("center2interval");
  unary_function_unary __center2interval(&_center2interval,_center2interval_s);
  unary_function_ptr at_center2interval (&__center2interval,0,true);


  gen histogram(const vecteur & v,double class_minimum,double class_size,GIAC_CONTEXT){
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    if (ckmatrix(v) && !v.empty() && v.front()._VECTptr->size()==2){
      // matrix format is 2 columns 1st column=interval, 2nd column=frequency
      // OR value/frequencies
      // get total of population
      const_iterateur it=v.begin(),itend=v.end();
      double n=0;
      for (;it!=itend;++it)
	n += evalf_double(it->_VECTptr->back())._DOUBLE_val;
      // get surface
      gen g=v.front()._VECTptr->front();
      if (g.is_symb_of_sommet(at_interval)){
	g=g._SYMBptr->feuille;
	if (g.type!=_VECT || g._VECTptr->size()!=2)
	  settypeerr();
	g=evalf_double(g._VECTptr->front());
      }
      else
	g=g-class_size/2;
      gen h=(itend-1)->_VECTptr->front();
      if (h.is_symb_of_sommet(at_interval)){
	h=h._SYMBptr->feuille;
	if (h.type!=_VECT || h._VECTptr->size()!=2)
	  settypeerr();
	h=evalf_double(h._VECTptr->back());
      }
      else
	h=h+class_size/2;
      if (g.type!=_DOUBLE_ || h.type!=_DOUBLE_ || g._DOUBLE_val>=h._DOUBLE_val)
	setsizeerr();
      double delta=h._DOUBLE_val-g._DOUBLE_val,inf,sup;
      it=v.begin();
      int nclass=itend-it;
      vecteur res;
      for (;it!=itend;++it){
	gen current=it->_VECTptr->front();
	if (current.is_symb_of_sommet(at_interval)){
	  if (!chk_double_interval(current,inf,sup))
	    settypeerr();
	}
	else {
	  gen tmp=evalf_double(current);
	  if (tmp.type!=_DOUBLE_)
	    settypeerr();
	  inf = tmp._DOUBLE_val -class_size/2;
	  sup = tmp._DOUBLE_val + class_size/2;
	}
	double height=1/(sup-inf);
	height=height*evalf_double(it->_VECTptr->back())._DOUBLE_val/n;
	gen mini(inf,height),maxi(sup,height);
	res.push_back(_segment(makevecteur(inf,mini),contextptr));
	res.push_back(_segment(makevecteur(mini,maxi),contextptr));
	res.push_back(_segment(makevecteur(maxi,sup),contextptr));	    
      }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
      return res;
    }
    vector<double>  w1=prepare_effectifs(v);
    int s=w1.size();
    // class_min + k*class_size <= mini hence k
    double kbegin=floor((w1.front()-class_minimum)/class_size);
    double kend=floor((w1.back()-class_minimum)/class_size);
    vector<double>::const_iterator it=w1.begin(),itend=w1.end();
    vecteur res;
    for (;kbegin<=kend;++kbegin){
      // count in this class
      double min_class=kbegin*class_size+class_minimum;
      double max_class=min_class+class_size;
      double effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=max_class)
	  break;
      }
      effectif /= s*class_size; // height of the class
      gen ming=min_class+gen(0.0,effectif);
      gen maxg=max_class+gen(0.0,effectif);
      res.push_back(_segment(makevecteur(min_class,ming),contextptr));
      res.push_back(_segment(makevecteur(ming,maxg),contextptr));
      res.push_back(_segment(makevecteur(maxg,max_class),contextptr));
    }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
    return gen(res,_SEQ__VECT);
  }
  gen _histogram(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      vecteur & args=*g._VECTptr;
      int s=args.size();
      if (s<2)
	setsizeerr();
      if (args[0].type!=_VECT)
	setsizeerr();
      vecteur data=*args[0]._VECTptr;
      gen arg1=evalf_double(args[1]);
      if (ckmatrix(data)&&arg1.type==_DOUBLE_){ // [ [center, effectif] ... ], min
	data=mtran(data); // 1st line = all centers
	if (data.size()!=2)
	  setsizeerr();
	data[0]=centres2intervalles(*data[0]._VECTptr,arg1._DOUBLE_val,true);
	data=mtran(data);
	return histogram(data,0.0,0.0,contextptr);
      }
      if (s==3){
	gen arg2=evalf_double(args[2]);
	if (arg1.type==_DOUBLE_ && arg2.type==_DOUBLE_)
	  return histogram(data,arg1._DOUBLE_val,arg2._DOUBLE_val,contextptr);
      }
      if (s==2 && args[1].type==_VECT)
	return _histogram(gen(makevecteur(mtran(args),class_minimum),_SEQ__VECT),contextptr);
      setsizeerr();
    }
    vecteur v(gen2vecteur(g));
    return histogram(v,class_minimum,class_size,contextptr);
  }
  const string _histogram_s("histogram");
  unary_function_eval __histogram(&_histogram,_histogram_s);
  unary_function_ptr at_histogram (&__histogram,0,true);

  struct xeff {
    double x;
    double eff;
    xeff(double x0,double eff0): x(x0),eff(eff0) {}
  };

  bool operator <(const xeff & a,const xeff & b){
    return a.x<b.x;
  }

  gen _cumulated_frequencies(const gen & g,GIAC_CONTEXT){
    gen g0(g);
    double class_min=class_minimum,class_s=class_size;
    if (g0.type==_VECT && g0.subtype==_SEQ__VECT && g0._VECTptr->size()==2){
      vecteur v = *g._VECTptr;
      gen tmp=evalf_double(v[1]);
      if (tmp.type!=_DOUBLE_) {
	if (ckmatrix(g)){
	  if (!v[0]._VECTptr->front().is_symb_of_sommet(at_interval))
	    v[0]=centres2intervalles(*v[0]._VECTptr,class_min,true);
	  g0=mtran(v);
	}
      }
      else {
	g0=v[0];
	class_min=tmp._DOUBLE_val;
      }
    }
    if (!ckmatrix(g0))
      setsizeerr();
    // 1st column = values (or classes), 2nd column = effectif
    matrice m= *g0._VECTptr ;
    if (m.empty() || m[0]._VECTptr->size()<2)
      setsizeerr();
    int s=m[0]._VECTptr->size();
    vecteur ans;
    for (int k=1;k<s;++k){
      // compute total eff
      iterateur it=m.begin(),itend=m.end();
      vector<xeff> veff;
      double n=0,x=0;
      if (it !=itend && it->_VECTptr->front().is_symb_of_sommet(at_interval)){
	gen tmp=it->_VECTptr->front()._SYMBptr->feuille;
	if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	  setsizeerr();
	else
	  tmp=tmp._VECTptr->front();
	tmp=evalf_double(tmp);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	veff.push_back(xeff(tmp._DOUBLE_val,0));
      }
      else
	veff.push_back(xeff(class_min,0));
      for (;it!=itend;++it){
	vecteur & v = *it->_VECTptr;
	gen tmp=evalf_double(v[k]);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	// class_s = tmp._DOUBLE_val - x;
	n = n + (x=tmp._DOUBLE_val) ;
	if (v.front().is_symb_of_sommet(at_interval)){
	  tmp=v.front()._SYMBptr->feuille;
	  if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	    setsizeerr();
	  else
	    tmp=tmp._VECTptr->back();
	}
	else
	  tmp=v.front()+class_s/2;
	tmp=evalf_double(tmp);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	veff.push_back(xeff(tmp._DOUBLE_val,x));
      }
      sort(veff.begin(),veff.end());
      vecteur res;
      vector<xeff>::const_iterator jt=veff.begin(),jtend=veff.end();
      double cumul=0;
      for (;jt!=jtend;++jt){
	cumul += jt->eff/n ;
	res.push_back(gen(jt->x)+cst_i*gen(cumul));
      }
      ans.push_back(symb_pnt(gen(res,_GROUP__VECT),k,contextptr));
    }
    return gen(ans,_SEQ__VECT);
  }
  const string _cumulated_frequencies_s("cumulated_frequencies");
  unary_function_eval __cumulated_frequencies(&_cumulated_frequencies,_cumulated_frequencies_s);
  unary_function_ptr at_cumulated_frequencies (&__cumulated_frequencies,0,true);

  // classes(vector or column matrix,begin of class, class size)
  // "      ( "                     ,list of intervals)
  // "      ( "                     ,list of centers,begin of 1st class)
  gen _classes(const gen & g){
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      vecteur & args=*g._VECTptr;
      int s=args.size();
      if (s<2)
	setsizeerr();
      if (args[0].type!=_VECT)
	setsizeerr();
      vecteur data=*args[0]._VECTptr;
      if (s==2 && args[1].type==_VECT){ // 2nd arg=list of intervals
	return effectifs(data,*args[1]._VECTptr);
      }
      if (s==3){
	gen arg2=evalf_double(args[2]);
	gen arg1=evalf_double(args[1]);
	if (args[2].type==_VECT && arg1.type==_DOUBLE_){
	  return effectifs(data,centres2intervalles(*args[2]._VECTptr,0.0,false));
	}
	if (arg1.type==_DOUBLE_ && arg2.type==_DOUBLE_)
	  return effectifs(data,arg1._DOUBLE_val,arg2._DOUBLE_val);
      }
      setsizeerr();
    }
    vecteur v(gen2vecteur(g));
    return effectifs(v,class_minimum,class_size);
  }
  const string _classes_s("classes");
  unary_function_unary __classes(&_classes,_classes_s);
  unary_function_ptr at_classes (&__classes,0,true);

  // [[x1 y1] [x2 y2] ...]
  gen scatterplot(const vecteur & v_orig,bool polygone,GIAC_CONTEXT){
    int ncol;
    if (!ckmatrix(v_orig)||v_orig.empty() || (ncol=v_orig.front()._VECTptr->size())<2)
      setsizeerr();
    vecteur v(v_orig);
    const_iterateur it=v.begin(),itend=v.end();
    sort(v.begin(),v.end(),first_ascend_sort);
    vecteur res;
    if (polygone){
      vecteur vres;
      for (int j=1;j<ncol;++j){
	res.clear();
	for (it=v.begin();it!=itend;++it){
	  res.push_back(it->_VECTptr->front()+cst_i*(*it->_VECTptr)[j]);
	}
	vres.push_back(symb_pnt(res,j,contextptr));
      }
      if (ncol==2)
	return vres.front();
      return gen(vres,_SEQ__VECT);
    }
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    for (;it!=itend;++it){
      vecteur & w=*it->_VECTptr;
      gen x=w.front();
      for (int j=1;j<ncol;++j){
	res.push_back(symb_pnt(x+cst_i*w[j],j,contextptr));
      }
    }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
    return gen(res,_SEQ__VECT);
  }
  gen _scatterplot(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      if (g._VECTptr->size()!=2 || !ckmatrix(g))
	setsizeerr(); // FIXME implement options
      return _scatterplot(_tran(g),contextptr);
    }
    vecteur v(gen2vecteur(g));
    return scatterplot(v,false,contextptr);
  }
  const string _scatterplot_s("scatterplot");
  unary_function_eval __scatterplot(&_scatterplot,_scatterplot_s);
  unary_function_ptr at_scatterplot (&__scatterplot,0,true);

  gen _polygonplot(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      if (g._VECTptr->size()!=2 || !ckmatrix(g))
	setsizeerr(); // FIXME implement options
      return _polygonplot(_tran(g),contextptr);
    }
    vecteur v(gen2vecteur(g));
    return scatterplot(v,true,contextptr);
  }
  const string _polygonplot_s("polygonplot");
  unary_function_eval __polygonplot(&_polygonplot,_polygonplot_s);
  unary_function_ptr at_polygonplot (&__polygonplot,0,true);


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
