/* -*- mode:C++ ; compile-command: "g++ -I.. -g -c pari.cc" -*- */
#include "first.h"
/*  PARI interface
 *  Copyright (C) 2001 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include "config.h"
#ifdef HAVE_LIBPARI
#include "gen.h" 
#include "identificateur.h"
#include "sym2poly.h"

long int abs(long int & l){
  if (l<0)
    return -l;
  else
    return l;
}
#include "pari.h"
extern "C" {
#include <pari/pari.h>
}
#include <cstdlib>

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  struct giac_pari_init {
    giac_pari_init(long maxprime) { 
      long pari_mem_size=10000000;
      if (getenv("PARI_SIZE")){
	string pari_size_s(getenv("PARI_SIZE"));
	pari_mem_size= atoi(pari_size_s.c_str());
      }
      pari_init(pari_mem_size,maxprime);
    }
  };
  long pari_maxprime=100000;
  giac_pari_init bidon(pari_maxprime);

  GEN gen2GEN(const gen & e){
    string s=pariprint(e);
    return flisexpr((char *) s.c_str());
  }

  // WARNING: If g is a matrix this print the transpose of the matrix
  string GEN2string(const GEN & g){
    // cerr << typ(g) << " " << t_MAT << endl;
    char * ch;
    string s;
    if ((typ(g)==t_MAT) || (typ(g)==t_COL)){
      int taille=lg(g);
      s +="[";
      for (int i=1;i<taille;++i){
	s += GEN2string((long *)g[i]);
	if (i==taille-1)
	  s+="]";
	else
	  s+=",";
      }
      return s;
    }
    ch=GENtostr(g);
    s=ch;
#ifndef HAVE_LIBGC
    free(ch);
#endif
    return s;
  }

  string pari_isprime(const gen & e){
    long av=avma;
    string s=GEN2string(gisprime(gen2GEN(e),2));
    avma=av;
    return s;
  }

  string pari_ifactor(const gen & e){
    long av=avma;
    GEN g=gen2GEN(e);
    GEN gf=factorint(g,0);
    string s=GEN2string(gf);
    avma=av;
    return s;
  }

#ifndef HAVE_LIBNTL
  // for factorization over Z when many modular factors arise
  // This is a call to PARI combine_factors
  // WARNING: You must remove static from the declaration of combine_factors
  // in pari/src/basemath/polarith2.c
  // GEN combine_factors(GEN a, GEN famod, GEN p, long klim, long hint);
  void pari_lift_combine(const vecteur & a,const vector<vecteur> & factmod,gen & modulo,vector<vecteur> & res){
    long av=avma;
    GEN pari_a=gen2GEN(r2e(a,x__IDNT_e));
    string s("[");
    vector<vecteur>::const_iterator it=factmod.begin(),itend=factmod.end();
    for (;it!=itend;){
      s += r2e(*it,x__IDNT_e).print();
      ++it;
      if (it==itend)
	break;
      s+=",";
    }
    s+="]";
    // cerr << s << endl;
    GEN pari_factmod=flisexpr((char *) s.c_str());
    GEN pari_modulo=gen2GEN(modulo);
    GEN pari_res=combine_factors(pari_a,pari_factmod,pari_modulo,0,1);
    // back conversion
    string res_s=GENtostr(pari_res);
    gen res_v(res_s.substr(0,res_s.size()-1));
    if (res_v.type!=_VECT)
      setsizeerr();
    const_iterateur jt=res_v._VECTptr->begin(),jtend=res_v._VECTptr->end();
    for (;jt!=jtend;++jt){
      res.push_back(*e2r(*jt,x__IDNT_e)._VECTptr);
    }
    avma=av;
  }
#endif // HAVE_LIBNTL

#ifndef NO_NAMESPACE_GIAC
}
#endif // ndef NO_NAMESPACE_GIAC

#endif // HAVE_LIBPARI
