// -*- mode:C++ ; compile-command: "g++ -I.. -g -c series.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include "derive.h"
#include "subst.h"
#include "series.h"
#include "symbolic.h"
#include "unary.h"
#include "usual.h"
#include "poly.h"
#include "sym2poly.h" 
#include "tex.h"
#include "prog.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  bool taylor(const gen & f_x,const gen & x,const gen & lim_point,int ordre,vecteur & v,GIAC_CONTEXT){
    gen current_derf(f_x),value,factorielle(1);
    for (int i=0;;++i){
      value=subst(current_derf,x,lim_point);
      if (is_undef(value))
	return false;
      v.push_back(normal(rdiv(value,factorielle)));
      if (i==ordre){
	v.push_back(undef);
	return true;
      }
      factorielle = factorielle * gen(i+1);
      current_derf=normal(derive(current_derf,x));
    }
#ifdef DEBUG_SUPPORT
    v.dbgprint();
#endif
  }

  // this will contain var of the type sin(x) as x->inf
  // the variables inside this vecteur are assumed to be in [-1..1]
  vecteur sincosinf; 
  // direction is always ignored for taylor, but might not 
  // for generic series_expansion
  // shift coeff =0 for taylor
  gen taylor(const gen & lim_point,int ordre,const unary_function_ptr & f,int direction,gen & shift_coeff,GIAC_CONTEXT){
    // Special handling for sin/cos expansion inside limit
    if ( (inside_limit(contextptr)) && is_inf(lim_point) && ( (f==at_cos)  || (f==at_sin) ) ){
      int i=sincosinf.size();
      sincosinf.push_back(gen(" sincosinf"+print_INT_(i)));
      gen g=sincosinf.back();
      if (!g._IDNTptr->value){
	vecteur minusone_one(2);
	minusone_one[0]=minus_one;
	minusone_one[1]=plus_one;
	gen v(vecteur(1,gen(minusone_one,_LINE__VECT)));
	gen d(_DOUBLE_);
	d.subtype=_INT_TYPE;
	gen aa(makevecteur(d,v,vecteur(0)),_ASSUME__VECT);
	g._IDNTptr->value=new gen(aa);
      }
      return vecteur(1,g);
    }
    // if preprocessing is needed for f, series_expansion for ordre==-1 should 
    // push back in a global vector f and it's substitution
    if (ordre<0) 
      return 0;
    shift_coeff=0;
    if (is_undef(lim_point) || is_inf(lim_point))
      invalidserieserr("non tractable function "+f.ptr->print()+" at "+lim_point.print());
    identificateur x(" ");
    vecteur v;
    if (taylor(f(x,contextptr),x,lim_point,ordre,v,contextptr)) // FIXME GIAC_CONTEXT
      return v;
    else
      return undef;
  }

  gen default_ordre=20;

  gen porder(const sparse_poly1 & a){
    if (a.empty())
      return plus_inf;
    sparse_poly1::const_iterator a_end=a.end()-1;
    if (is_undef(a_end->coeff))
      return a_end->exponent;
    else
      return plus_inf;
  }

  void vecteur2sparse_poly1(const vecteur & v,sparse_poly1 & p){
    p.clear();
    vecteur::const_iterator it=v.begin(),itend=v.end();
    p.reserve(itend-it);
    for (int i=0;it!=itend;++i,++it){
      if (!is_zero(*it))
	p.push_back(monome(*it,i));
    }
  }

  sparse_poly1 vecteur2sparse_poly1(const vecteur & v){
    sparse_poly1 p;
    vecteur2sparse_poly1(v,p);
    return p;
  }

  gen sparse_poly12gen(const sparse_poly1 & p,const gen & x){
    gen res;
    sparse_poly1::const_iterator it=p.begin(),itend=p.end();
    for (;it!=itend;++it){
      if (is_undef(it->coeff))
	return res+pow(x,it->exponent)*order_size(x);
      res = res + it->coeff * pow(x,it->exponent);
    }
    return res;
  }

  void padd(const sparse_poly1 & a,const sparse_poly1 &b, sparse_poly1 & res){
    // Series addition
    context * contextptr=new context();
    if (a.empty()){
      if (&b!=&res)
	res=b;
      return;
    }
    if (b.empty()){
      if (&a!=&res)
	res=a;
      return;
    }
    sparse_poly1::const_iterator a_cur,a_end,b_cur,b_end;
    sparse_poly1 temp_a,temp_b;
    if (&res==&a){ // must make a copy of a
      temp_a=a;
      a_cur=temp_a.begin();
      a_end=temp_a.end();
    }
    else {
      a_cur=a.begin();
      a_end=a.end();
    }
    if (&res==&b){ // must make a copy of b
      temp_b=b;
      b_cur=temp_b.begin();
      b_end=temp_b.end();
    }
    else {
      b_cur=b.begin();
      b_end=b.end();
    }
    res.clear();
    res.reserve(a_end-a_cur+b_end-b_cur);
    for (;(a_cur!=a_end) && (b_cur!=b_end) ;) {
      gen a_pow=a_cur->exponent;
      gen b_pow=b_cur->exponent;
      // a and b are non-empty, compare powers
      if (ck_is_strictly_greater(b_pow,a_pow,0)) {
	// get coefficient from a
	res.push_back(*a_cur);
	if (is_undef(a_cur->coeff))
	  return;
	a_cur++;
	continue;
      }
      if (ck_is_strictly_greater(a_pow,b_pow,0)) {
	// get coefficient from b
	res.push_back(*b_cur);
	if (is_undef(b_cur->coeff))
	  return;
	b_cur++;
	continue;
      }
      // Add coefficient of a and b
      gen sum=recursive_normal(a_cur->coeff+b_cur->coeff,contextptr);
      // gen sum=(a_cur->coeff+b_cur->coeff);
      if (!is_zero(sum))
	res.push_back(monome(sum,a_pow));
      if (is_undef(sum))
	return;
      a_cur++;
      b_cur++;
    }
    for (;a_cur!=a_end;++a_cur)
      res.push_back(*a_cur);
    for (;b_cur!=b_end;++b_cur)
      res.push_back(*b_cur);
  }

  sparse_poly1 spadd(const sparse_poly1 & a,const sparse_poly1 &b){
      sparse_poly1 res;
      padd(a,b,res);
      return res;
  }

  sparse_poly1 spsub(const sparse_poly1 & a,const sparse_poly1 &b){
      sparse_poly1 res(b);
      pneg(b,res);
      padd(a,res,res);
      return res;
  }


  void pmul(const sparse_poly1 & a,const gen & b_orig, sparse_poly1 & res){
    gen b(b_orig);
    if (&a==&res){
      sparse_poly1::iterator it=res.begin(),itend=res.end();
      for (;it!=itend;++it)
	it->coeff = it->coeff * b ;
      return;
    }
    sparse_poly1::const_iterator it=a.begin(),itend=a.end();
    res.clear();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(monome(it->coeff * b, it->exponent));
  }

  void pmul(const gen & b, const sparse_poly1 & a,sparse_poly1 & res){
    pmul(a,b,res);
  }

  sparse_poly1 spmul(const sparse_poly1 & a,const gen &b){
      sparse_poly1 res;
      pmul(a,b,res);
      return res;
  }

  sparse_poly1 spmul(const gen & a,const sparse_poly1 &b){
      sparse_poly1 res;
      pmul(a,b,res);
      return res;
  }

  bool monome_less(const monome & a,const monome & b){
    return ck_is_strictly_greater(b.exponent,a.exponent,0);
  }

  void pmul(const sparse_poly1 & celuici,const sparse_poly1 &other, sparse_poly1 & final_seq){
    context * contextptr=new context();
    int asize=celuici.size();
    int bsize=other.size();
    if ( (!asize) || (!bsize) ) {
      final_seq.clear();
      return ;
    }
    if (asize==1){
      gen temp(celuici.front().coeff);
      pshift(other,celuici.front().exponent,final_seq);
      // cout << other << "Shifted" << final_seq << endl;
      pmul(final_seq,temp,final_seq);
      // cout << other << "Multiplied" << final_seq << endl;
      return ;
    }
    if (bsize==1){
      gen temp(other.front().coeff);
      pshift(celuici,other.front().exponent,final_seq);
      pmul(final_seq,temp,final_seq);
      return ;
    }
    sparse_poly1 new_seq;
    new_seq.reserve(asize*bsize);
    // General sparse series multiplication: complexity is N*M*ln(N*M)
    // Storage capacity 2*N*M expair
    // That's much more than O(N+M) for dense poly *but*
    // it works for non integer powers
    // cout << celuici << "pmul" << other << endl;
    // First find the order product
    gen a_max = porder(celuici);
    gen b_max = porder(other);
    gen a_min = celuici.front().exponent;
    gen b_min = other.front().exponent;
    gen c_min = normal(a_min + b_min);
    gen c_max = min(normal(a_min + b_max),normal(b_min + a_max));
    if (c_max.type==_SYMB)
      assert(c_max._SYMBptr->sommet!=at_max);
    // compute all products term by term, with optimization for dense poly
    // (coeff are sorted for dense poly)
    sparse_poly1::const_iterator itb = other.begin(),itbend = other.end();
    sparse_poly1::const_iterator ita = celuici.begin(),ita_end=celuici.end();
    sparse_poly1::const_iterator itabegin = ita;
    gen old_pow=normal(ita->exponent+itb->exponent);
    gen res(0);
    for ( ; ita!=ita_end; ++ita ){
      sparse_poly1::const_iterator itacur=ita;
      sparse_poly1::const_iterator itbcur=itb;
      for (;;) {
	gen cur_pow=normal(itacur->exponent+itbcur->exponent);
	if (ck_is_greater(c_max,cur_pow,0)){
	  if (cur_pow!=old_pow){
	    new_seq.push_back( monome(res,old_pow ));
	    res=itacur->coeff * itbcur->coeff;
	    old_pow=cur_pow;
	  }
	  else
	    res=res+ itacur->coeff  * itbcur->coeff;
	}
	if (itacur==itabegin)
	  break;
	--itacur;
	++itbcur;
	if (itbcur==itbend)
	  break;
      }
    }
    --ita;
    ++itb;
    for ( ; itb!=itbend;++itb){
      sparse_poly1::const_iterator itacur=ita;
      sparse_poly1::const_iterator itbcur=itb;
      for (;;) {
	gen cur_pow=normal(itacur->exponent + itbcur->exponent);
	if (ck_is_greater(c_max,cur_pow,0)){
	  if (cur_pow!=old_pow){
	    new_seq.push_back( monome(res ,old_pow ));
	    res= itacur->coeff  * itbcur->coeff ;
	    old_pow=cur_pow;
	  }
	  else
	    res=res+ itacur->coeff  * itbcur->coeff ;
	}
	if (itacur==itabegin)
	  break;
	--itacur;
	++itbcur;
	if (itbcur==itbend)
	  break;
      }
    }
    new_seq.push_back( monome(res ,old_pow ));
    // cout << new_seq << endl;
    // sort by asc. power
    sort( new_seq.begin(),new_seq.end(),monome_less);
    // cout << "Sorted" << new_seq << endl;
    // add terms with same power
    sparse_poly1::const_iterator it=new_seq.begin();
    sparse_poly1::const_iterator itend=new_seq.end();
    final_seq.clear();
    final_seq.reserve(itend-it);
    while (it!=itend){
      gen res=it->coeff;
      gen pow=it->exponent;
      if (is_undef(res)){
	final_seq.push_back(*it);
	return ;
      }
      ++it;
      while ( (it!=itend) && (it->exponent==pow)){
	if (is_undef(res)){
	  final_seq.push_back(*it);
	  return;
	}
	res=res+it->coeff;
	++it;
      }
      res=recursive_normal(res,contextptr);
      if (!is_zero(res))
	final_seq.push_back(monome(res, pow));
    }
    if (c_max!=plus_inf)
      final_seq.push_back(monome(undef, c_max));
    return ;
    cout << final_seq.back().coeff << endl;
  }

  sparse_poly1 spmul(const sparse_poly1 & a,const sparse_poly1 &b){
      sparse_poly1 res;
      pmul(a,b,res);
      return res;
  }

  void pneg(const sparse_poly1 & a,sparse_poly1 & res){
    if (&a==&res){
      sparse_poly1::iterator it=res.begin(),itend=res.end();
      for (;it!=itend;++it)
	it->coeff=-it->coeff;
      return;
    }
    sparse_poly1::const_iterator it=a.begin(),itend=a.end();
    res.clear();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(monome(-it->coeff, it->exponent));
  }

  sparse_poly1 spneg(const sparse_poly1 & a){
      sparse_poly1 res;
      pneg(a,res);
      return res;
  }
  
  void pshift(const sparse_poly1 & a,const gen & b_orig, sparse_poly1 & res){
    if (is_zero(b_orig)){
      if (&a!=&res)
	res=a;
      return;
    }
    gen b(b_orig);
    if (&a==&res){
      sparse_poly1::iterator it=res.begin(),itend=res.end();
      for (;it!=itend;++it)
	it->exponent = normal(it->exponent + b);
      return;
    }
    sparse_poly1::const_iterator it=a.begin(),itend=a.end();
    res.clear();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(monome(it->coeff , normal(it->exponent +b)));
  }

  // ascending order division
  void pdiv(const sparse_poly1 & a,const sparse_poly1 &b, sparse_poly1 & res,gen ordre=default_ordre){
    if (b.empty())
      divisionby0err(a);
    gen b0=b.front().coeff;
    if (is_undef(b0)){
      if (&b!=&res)
	res=b;
      return;
    }
    if (b.size()==1){
      pshift(a,-b.front().exponent,res);
      pdiv(res,b0,res);
      return ;
    }
    // cout << a << "/" << b << endl;
    assert(&res!=&b);
    gen e0=b.front().exponent;
    ordre=min(min(porder(a),porder(b)),ordre);
    if (ordre==plus_inf)
      ordre=default_ordre;
    // cout << ordre << endl;
    if (ordre.type==_SYMB)
      assert(ordre._SYMBptr->sommet!=at_max);
    sparse_poly1 rem(a);
    res.clear();
    sparse_poly1 bshift;
    gen q_cur,e_cur; // current quotient, current exponent
    for (;;){
      if (is_undef(rem.front().coeff)){
	res.push_back(monome(undef,rem.front().exponent-e0));
	// cout << "=" << res << endl;
	return;
      }
      q_cur=rdiv(rem.front().coeff,b0);
      e_cur=rem.front().exponent-e0;
      res.push_back(monome(q_cur,e_cur));
      pshift(b,e_cur,bshift);
      pmul(-q_cur,bshift,bshift);
      padd(rem,bshift,rem);
      // cout << rem.front().exponent << " " << e0+ordre << endl;
      if (ck_is_strictly_greater(rem.front().exponent,e0+ordre,0)){
	res.push_back(monome(undef,ordre+1));
	return;
      }
    }
  }

  sparse_poly1 spdiv(const sparse_poly1 & a,const sparse_poly1 &b){
      sparse_poly1 res;
      pdiv(a,b,res,default_ordre);
      return res;
  }

  void pdiv(const sparse_poly1 & a,const gen & b_orig, sparse_poly1 & res){
    if (is_zero(b_orig))
      divisionby0err(a);
    if (is_one(b_orig)){
      if (&a!=&res)
	res=a;
      return;
    }
    gen b(b_orig);
    if (&a==&res){
      sparse_poly1::iterator it=res.begin(),itend=res.end();
      for (;it!=itend;++it)
	it->coeff=normal(rdiv(it->coeff, b));
      // it->coeff=rdiv(it->coeff, b);
      return;
    }
    sparse_poly1::const_iterator it=a.begin(),itend=a.end();
    res.clear();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(monome(normal(rdiv(it->coeff,b)) , it->exponent));
    // res.push_back(monome(rdiv(it->coeff,b) , it->exponent));
  }

  sparse_poly1 spdiv(const sparse_poly1 & a,const gen &b){
      sparse_poly1 res;
      pdiv(a,b,res);
      return res;
  }
  
  // v is replaced by e*v where e*v has no denominator 
  void lcmdeno(vecteur &v,gen & e){
    if (v.empty()){
      e=1;
      return;
    }
    if (is_undef(v.front())){
      v.erase(v.begin());
      lcmdeno(v,e);
      v.insert(v.begin(),undef);
      return;
    }
    vecteur l;
    lvar(v,l);
    int l_size(l.size());
    vecteur w;
    w.reserve(2*l_size);
    gen common=1,f,num,den;
    // compute lcm of denominators in common
    vecteur::iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      f=e2r(*it,l);
      fxnd(f,num,den);
      w.push_back(num);
      w.push_back(den);
      // replace common by lcm of common and den
      common = lcm(common,den);
    }
    // compute e and recompute v
    e=r2sym(common,l);
    it=v.begin();
    for (int i=0;it!=itend;++it,i=i+2){
      *it=r2sym(w[i]*rdiv(common,w[i+1]),l);
    }
  }

  void lcmdeno(sparse_poly1 &v,gen & e){
    if (v.empty()){
      e=1;
      return;
    }
    if (is_undef(v.back().coeff)){
      monome last=v.back();
      v.pop_back();
      lcmdeno(v,e);
      v.push_back(last);
      return;
    }
    vecteur l;
    lvar(v,l);
    int l_size(l.size());
    vector<gen> w;
    w.reserve(2*l_size);
    gen common=1,num,den,f;
    // compute lcm of denominators in common
    sparse_poly1::iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      f=e2r(it->coeff,l);
      fxnd(f,num,den);
      w.push_back(num);
      w.push_back(den);
      common=lcm(common,den);
    }
    // compute e and recompute v
    e=r2sym(common,l);
    it=v.begin();
    for (int i=0;it!=itend;++it,i=i+2){
      it->coeff=r2sym(w[i]*rdiv(common,w[i+1]),l);
    }
  }

  void pcompose(const vecteur & v,const sparse_poly1 & p, sparse_poly1 & res){
    if (v.empty() || p.empty() ){
      res.clear();
      return;
    }
    // Conversion of p and v to "internal" polynomial form
    vecteur l; // will contain the list of variables common to v and p
    lvar(v,l);
    lvar(p,l);
    int l_size(l.size());
    gen plcm=plus_one,vlcm=plus_one,f,num,den;
    // compute lcm of denominators of p in plcm
    sparse_poly1::const_iterator its=p.begin(),itsend=p.end();
    vecteur ptemp;
    ptemp.reserve(2*(itsend-its));
    for (;its!=itsend;++its){
      f=e2r(its->coeff,l);
      fxnd(f,num,den);
      ptemp.push_back(num);
      ptemp.push_back(den);
      plcm=lcm(den,plcm);
    }
    // compute pcopy such that pcopy/plcm=p
    its=p.begin();
    sparse_poly1 pcopy;
    pcopy.reserve(itsend-its);
    for (int i=0;its!=itsend;++its,i=i+2){
      num=ptemp[i]*rdiv(plcm,ptemp[i+1]);
      pcopy.push_back(monome(num,its->exponent));
    }
    // do the same thing on v
    vecteur w;
    // compute lcm of denominators in common
    vecteur::const_iterator it=v.begin(),itend=v.end();
    w.reserve(2*(itend-it));
    for (;it!=itend;++it){
      f=e2r(*it,l);
      fxnd(f,num,den);
      w.push_back(num);
      w.push_back(den);
      vlcm=lcm(vlcm,den);
    }
    // compute vcopy
    it=v.begin();
    vecteur vcopy;
    vcopy.reserve(itend-it);
    for (int i=0;it!=itend;++it,i=i+2)
      vcopy.push_back(w[i]*rdiv(vlcm,w[i+1]));
    reverse(vcopy.begin(),vcopy.end());
    if (vcopy.empty()  ){
      res=sparse_poly1(1,monome(undef,minus_inf));
      return;
    }
    // cout << "compose " << vcopy << " with " << pcopy << endl;
    it=vcopy.begin(),itend=vcopy.end();
    int n=itend-it-1;
    if (is_undef(*it)){
      ++it;
      // add undef order term
      gen cur_ordre=porder(p);
      gen n_valuation=gen(n)*p.front().exponent;
      // compare cur_ordre with n*valuation(pcopy)
      if ( (cur_ordre==plus_inf) || (ck_is_strictly_greater(cur_ordre,n_valuation,0)) ){
	// remove greater order terms from pcopy
	for (;!pcopy.empty();){
	  if (ck_is_strictly_greater(pcopy.back().exponent,n_valuation,0))
	    pcopy.pop_back();
	  else
	    break;
	}
	// insert undef
	if (pcopy.empty() || (!is_undef(pcopy.back().coeff)) )
	  pcopy.push_back(monome(undef,n_valuation));
      }
    }
    if (it==itend){
      res=sparse_poly1(1,monome(undef));
      return;
    }
    res=sparse_poly1(1,monome(*it));
    // cout << res << endl;
    ++it;
    gen plcmn=plus_one;
    for (;it!=itend;++it){
      plcmn=plcmn*plcm;
      // cout << res << "*" << pcopy << endl ;
      pmul(res,pcopy,res);
      // cout << res << endl;
      if (!is_zero(*it))
	padd(res,sparse_poly1(1,monome(*it*plcmn)),res);
      // cout << res << endl;
    }
    den=vlcm*plcmn;
    // back conversion from res to symbolic form
    sparse_poly1::iterator sit=res.begin(),sitend=res.end();
    for (;sit!=sitend;++sit){
      num=den;
      sit->coeff=r2sym(fraction(sit->coeff,num).normal(),l);
    }
  }

  void ppow(const sparse_poly1 & base,int m,sparse_poly1 & res){
    if (m==0){
      res.clear();
      return;
    }
    if (m==1){
      if (&base!=&res)
	res=base;
      return;
    }
    sparse_poly1 temp;
    pmul(base,base,temp);
    if (m%2){
      ppow(temp,m/2,temp);
      pmul(temp,base,res);
    }
    else
      ppow(temp,m/2,res);
  }
  
  // constant power, otherwise use exp(ln)
  void ppow(const sparse_poly1 & base,const gen & e,int ordre,int direction,sparse_poly1 & res){
    if (base.size()==1){
      if (&base==&res){
	res.front().coeff=pow(res.front().coeff,e);
	res.front().exponent=res.front().exponent*e;	
      }
      else 
	res=sparse_poly1(1,monome(pow(base.front().coeff,e),base.front().exponent*e));
      return;
    }
    gen n=porder(base);
    if ((n==plus_inf) && (e.type==_INT_) && (e.val>=0) ){ // exact power
      int m=e.val;
      ppow(base,m,res);
      return;
    }
    if (base.empty()){
      if (ck_is_positive(e,0))
	res.clear();
      else
	divisionby0err(base);
      return;
    }
    // series expansion to a constant power
    monome first(base.front());
    // If first.exponent!=0 and direction==0 we can not find 
    // first.exponent^e consistently around 0
    if (!direction && !is_integer(e) && !is_zero(first.exponent) )
      setsizeerr("Vanishing non integral power expansion");
    sparse_poly1 basecopy(base);
    basecopy.erase(basecopy.begin());
    pshift(basecopy,-first.exponent,basecopy);
    pdiv(basecopy,first.coeff,basecopy);
    if (n==plus_inf && !basecopy.empty()){ // add an O() error term
      monome last(undef,ordre+1);
      basecopy.push_back(last);
    }
    // answer=first.coeff^e*x^(first.exponent*e)*(1+base)^e
    // first (1+base)^e -> compose( [1,e,e(e-1)/2,...], base)
    vecteur v(1,plus_one);
    gen produit(e),factorielle(1);
    for (int i=1;i<=ordre;++i){
      v.push_back(rdiv(produit,factorielle));
      produit=produit*(e-gen(i));
      factorielle=factorielle*gen(i+1);
    }
    // cout << v << endl;
    pcompose(v,basecopy,res);
    // cout << res << endl;
    // final multiplication ans shift
    pshift(res,first.exponent*e,res);
    pmul(res,pow(first.coeff,e),res);
  }

  void in_series__SPOL1(const gen & e,const identificateur & x,const vecteur & lvx, const vecteur & lvx_s,int ordre,int direction,sparse_poly1 & s){
    s.clear();
    int pos=equalposcomp(lvx,e);
    if (pos){
      gen f=lvx_s[pos-1]; // since vectors begin at position 0
      if (f.type==_SPOL1){
	s=*(f._SPOL1ptr);
	return;
      }
      if (f.type!=_VECT)
	settypeerr();
      vecteur2sparse_poly1(*f._VECTptr,s);
      return;
    }
    if ( (e.type!=_SYMB) || !contains(e,x) ){
      s.push_back(monome(e,0));
      return;
    }
    // do rational operations
    if (e._SYMBptr->sommet==at_plus){
      if (e._SYMBptr->feuille.type!=_VECT){
	in_series__SPOL1(e._SYMBptr->feuille,x,lvx,lvx_s,ordre,direction,s);
	return;
      }
      const_iterateur it=e._SYMBptr->feuille._VECTptr->begin(),itend=e._SYMBptr->feuille._VECTptr->end();
      sparse_poly1 temp;
      for (;it!=itend;++it){
	in_series__SPOL1(*it,x,lvx,lvx_s,ordre,direction,temp);
	padd(s,temp,s);
      }
      return;
    }
    if (e._SYMBptr->sommet==at_neg){
      if (e._SYMBptr->feuille.type!=_VECT){
	in_series__SPOL1(e._SYMBptr->feuille,x,lvx,lvx_s,ordre,direction,s);
	pneg(s,s);
	return;
      }
      const_iterateur it=e._SYMBptr->feuille._VECTptr->begin(),itend=e._SYMBptr->feuille._VECTptr->end();
      sparse_poly1 temp;
      for (;it!=itend;++it){
	in_series__SPOL1(*it,x,lvx,lvx_s,ordre,direction,temp);
	pneg(temp,temp);
	padd(s,temp,s);
      }
      return;
    }
    if (e._SYMBptr->sommet==at_prod){
      if (e._SYMBptr->feuille.type!=_VECT){
	in_series__SPOL1(e._SYMBptr->feuille,x,lvx,lvx_s,ordre,direction,s);
	return;
      }
      const_iterateur it=e._SYMBptr->feuille._VECTptr->begin(),itend=e._SYMBptr->feuille._VECTptr->end();
      sparse_poly1 temp;
      s=sparse_poly1(1,monome(1,0));
      for (;it!=itend;++it){
	in_series__SPOL1(*it,x,lvx,lvx_s,ordre,direction,temp);
	pmul(s,temp,s);
      }
      return;
    }
    if (e._SYMBptr->sommet==at_inv){
      assert(e._SYMBptr->feuille.type!=_VECT);
      sparse_poly1 temp;
      in_series__SPOL1(e._SYMBptr->feuille,x,lvx,lvx_s,ordre,direction,temp);
      pdiv(sparse_poly1(1,monome(1,0)),temp,s,ordre);
      return;
    }
    if (e._SYMBptr->sommet==at_pow){
      // the power is independent on x
      gen base=(*(e._SYMBptr->feuille._VECTptr))[0];
      gen exponent=(*(e._SYMBptr->feuille._VECTptr))[1];
      in_series__SPOL1(base,x,lvx,lvx_s,ordre,direction,s);
      ppow(s,exponent,ordre,direction,s);
      return;
    }
    // unknown rational operator
    invalidserieserr("unknown rational operator");
  }

  int find_direction(const sparse_poly1 & s,int direction){
    int image_of_direction=0;
    if (!s.empty() && fastsign(s.front().coeff,0)){
      if (direction)
	image_of_direction=1;
      else {
	if (s.front().exponent.type==_INT_) {
	  if (s.front().exponent.val %2)
	    image_of_direction=direction;
	  else
	    image_of_direction=1;
	}
      }
      image_of_direction=image_of_direction*fastsign(s.front().coeff,0);
      return image_of_direction;
    }
    return 0;
  }

  int ck_is_greater(const sparse_poly1 & s1, sparse_poly1 & s2,int direction){
    sparse_poly1 s(s2);
    pneg(s,s);
    padd(s1,s,s);
    int image_of_direction=find_direction(s,direction);
    if (!image_of_direction)
      cksignerr(s);
    return image_of_direction==1;
  }

  void series__SPOL1(const gen & e,const identificateur & x,const gen & lim_point,int ordre,int direction,sparse_poly1 & s){
    // fast check first
    if (!contains(e,x)){
      s.push_back(monome(e,0));
      return ;
    }
    if (e.type==_IDNT){
      if (!is_zero(lim_point))
	s.push_back(monome(lim_point));
      if (ordre)
	s.push_back(monome(1,1));
      else
	s.push_back(monome(undef,1));
      return;
    }
    if (e.type!=_SYMB)
      settypeerr(); // comp not allowed

    // find list of vars depending on x
    vecteur lvx(rlvarx(e,x));
    iterateur lvx_it=lvx.begin(),lvx_end=lvx.end();
    // find asymptotic series expansion of vars in lvx
    vecteur lvx_s;
    lvx_s.reserve(lvx_end-lvx_it);
    for (;lvx_it!=lvx_end;++lvx_it){
      if (lvx_it->type==_IDNT){
	sparse_poly1 tmp;
	if (!is_zero(lim_point))
	  tmp.push_back(monome(lim_point));
	if (ordre)
	  tmp.push_back(monome(1,1));
	else
	  tmp.push_back(monome(undef,1));
	lvx_s.push_back(tmp);
	continue;
      }
      if (lvx_it->type!=_SYMB) // just in case...
	settypeerr();
      // test for a^b
      symbolic temp__SYMB=*lvx_it->_SYMBptr;
      if ( (temp__SYMB.sommet==at_pow) && (!contains((*temp__SYMB.feuille._VECTptr)[1],x) ) ){
	in_series__SPOL1((*temp__SYMB.feuille._VECTptr)[0],x,lvx,lvx_s,ordre,direction,s); 
	ppow(s,(*temp__SYMB.feuille._VECTptr)[1],ordre,direction,s);
	lvx_s.push_back(s);
	continue;
      }
      if (temp__SYMB.sommet==at_pow)
	temp__SYMB=symbolic(at_exp,(*temp__SYMB.feuille._VECTptr)[1]*ln((*temp__SYMB.feuille._VECTptr)[0]));
      // Check here for logarithms if image_of_lim_point=+/-inf
      // In such case we must factor x^s.begin().exponent and add
      // it to the exponent 0 term of the series expansion of the ln
      if (temp__SYMB.sommet==at_ln){
	// recursive call, works since lvx is sorted by increasing size
	in_series__SPOL1(temp__SYMB.feuille,x,lvx,lvx_s,ordre,direction,s); 
	gen exponent=s.front().exponent;
	gen c=s.front().coeff;
	s.erase(s.begin());
	if (!s.empty()){
	  pshift(s,-exponent,s);
	  pdiv(s,c,s);
	  vecteur expansion(1,zero);
	  expansion.reserve(ordre);
	  for (int i=1;i<=ordre;i++){
	    if (i%2)
	      expansion.push_back(inv(gen(i)));
	    else
	      expansion.push_back(-inv(gen(i)));
	  }
	  expansion.push_back(undef);
	  pcompose(expansion,s,s);
	}
	if (!is_zero(exponent) || !is_one(c))
	  s.insert(s.begin(),monome(exponent*ln(x)+ln(c)));
	lvx_s.push_back(s);
	continue;
	cout << s.back() << endl;
      }
      // test for the special case var=f(x)
      if ((temp__SYMB.feuille.type==_IDNT) && (temp__SYMB.sommet!=at_abs)){ 
	// Since e contains x feuille of e must be x
	if (!temp__SYMB.sommet.ptr->series_expansion)
	  invalidserieserr("no taylor method for "+temp__SYMB.sommet.ptr->print());
	gen shift_coeff;
	gen res=temp__SYMB.sommet.ptr->series_expansion(lim_point,ordre,temp__SYMB.sommet,direction,shift_coeff,0); // FIXME GIAC_CONTEXT
	if (res.type==_SPOL1)
	  lvx_s.push_back(*res._SPOL1ptr);
	else {// res must be a vecteur
	  sparse_poly1 temp(vecteur2sparse_poly1(*res._VECTptr));
	  if (!is_zero(shift_coeff)){
	    pshift(temp,shift_coeff,temp);
	    if (is_positive(shift_coeff,0))
	      temp.insert(temp.begin(),monome(temp__SYMB.sommet(lim_point,0))); // FIXME GIAC_CONTEXT
	  }
	  lvx_s.push_back(temp);
	}
	continue;
      }
      // Taylor not successfull: find series_expansion of arg, 
      // compose with sommet expansion
      // fixme: multiargs disabled, should return a vecteur of series_exp
      if (temp__SYMB.sommet != at_of && temp__SYMB.feuille.type==_VECT){
	const_iterateur fit=temp__SYMB.feuille._VECTptr->begin(),fitend=temp__SYMB.feuille._VECTptr->end();
	dbgprint_vector <sparse_poly1> vs;
	vs.reserve(fitend-fit);
	for (;fit!=fitend;++fit){
	  in_series__SPOL1(*fit,x,lvx,lvx_s,ordre,direction,s); 
	  vs.push_back(s);
	}
	if (temp__SYMB.sommet==at_max){
	  if (ck_is_greater(vs.front(),vs.back(),direction))
	    s=vs.front();
	  else
	    s=vs.back();
	}
	else {
	  if (temp__SYMB.sommet==at_min){
	    if (ck_is_greater(vs.front(),vs.back(),direction))
	      s=vs.back();
	    else
	      s=vs.front();
	  }
	  else
	    invalidserieserr(" multiargs not implemented");
	}
	lvx_s.push_back(s);
	continue;
      }
      if (temp__SYMB.sommet==at_of){
	gen & tf=temp__SYMB.feuille;
	if (tf.type==_VECT && tf._VECTptr->size()==2){
	  gen tff=tf._VECTptr->front();
	  gen tfx=tf._VECTptr->back();
	  in_series__SPOL1(tfx,x,lvx,lvx_s,ordre,direction,s); // s<-arg
	}
      }
      else
	in_series__SPOL1(temp__SYMB.feuille,x,lvx,lvx_s,ordre,direction,s); // s<-arg
      gen image_of_lim_point; 
      int image_of_direction=0;
      if (!s.empty()){
	if (s.begin()->exponent==0){
	  image_of_lim_point=s.begin()->coeff;
	  // ?? FIXME ???
	  if (temp__SYMB.sommet!=at_abs)
	    s.erase(s.begin()); // remove cst coeff from s
	}
	else {
	  if (ck_is_strictly_positive(s.begin()->exponent,0))
	    image_of_lim_point=0;
	  else {
	    image_of_lim_point=unsigned_inf;
	    if ( (s.begin()->exponent.type==_INT_) && !(s.begin()->exponent.val%2) ){ // odd negative exponent
	      if (is_strictly_positive(s.begin()->coeff,0))
		image_of_lim_point=plus_inf;
	      if (is_strictly_positive(-s.begin()->coeff,0))
		image_of_lim_point=minus_inf;
	    }
	    else { // other negative exponent
	      if (direction){
		if (is_strictly_positive(s.begin()->coeff,0))
		  image_of_lim_point=plus_inf;
		if (is_strictly_positive(-s.begin()->coeff,0))
		  image_of_lim_point=minus_inf;
		if (direction<0){
		  if (s.begin()->exponent.type==_INT_)
		    image_of_lim_point=-image_of_lim_point;
		  else
		    image_of_lim_point=unsigned_inf;
		}
	      }
	    }
	  } // end negative leading exponent
	} // end non-zero leading exponent
      } // end non-empty series
      image_of_direction = find_direction(s,direction);
      // Symbolic series expansion f(x), f is assumed to be analytic
      if (temp__SYMB.sommet==at_of){
	gen & tf=temp__SYMB.feuille;
	if (tf.type==_VECT && tf._VECTptr->size()==2){
	  gen tff=tf._VECTptr->front();
	  gen tfx=tf._VECTptr->back();
	  // Symbolic Taylor expansion of tff
	  vecteur expansion(1,symbolic(at_of,makevecteur(tff,image_of_lim_point))); 
	  expansion.reserve(ordre);
	  for (int i=1;i<=ordre;i++){
	    gen fn;
	    if (i==1)
	      fn=symbolic(at_of,makevecteur(symbolic(at_function_diff,tff),image_of_lim_point));
	    else
	      fn=symbolic(at_of,makevecteur(symbolic(at_of,makevecteur(symbolic(at_composepow,makevecteur(at_function_diff,i)),tff)),image_of_lim_point));
	    expansion.push_back(fn/factorial(i));
	  }
	  expansion.push_back(undef);
	  pcompose(expansion,s,s);
	  lvx_s.push_back(s);
	  continue;
	}
      }
      if (temp__SYMB.sommet==at_abs){
	if (!image_of_direction)
	  cksignerr(s);
	if (image_of_direction==-1)
	  pneg(s,s);
	lvx_s.push_back(s);
	continue;
      }
      if (is_inf(image_of_lim_point)){ // invert series expansion
	sparse_poly1 stmp;
	pdiv(sparse_poly1(1,monome(1,0)),s,stmp,ordre);
	s=stmp;
      }
      gen shift_coeff;
      gen expansion(temp__SYMB.sommet.ptr->series_expansion(image_of_lim_point,ordre,temp__SYMB.sommet,image_of_direction,shift_coeff,0)); // FIXME GIAC_CONTEXT
      if (expansion.type==_VECT){
	if (is_zero(shift_coeff))
	  pcompose(*expansion._VECTptr,s,s);
	else {
	  sparse_poly1 temp;
	  ppow(s,shift_coeff,ordre,direction,temp);
	  pcompose(*expansion._VECTptr,s,s);
	  pmul(s,temp,s);
	  if (is_positive(shift_coeff,0))
	    s.insert(s.begin(),monome(temp__SYMB.sommet(image_of_lim_point,0))); // FIXME GIAC_CONTEXT
	}
      }
      else {
	s.clear();
	s.push_back(monome(undef,minus_inf));
	return;
      }
      lvx_s.push_back(s);
      // fixme: add support for sparse_poly1 composition
    } // end loop lvx_it!=lvx_end
    in_series__SPOL1(e,x,lvx,lvx_s,ordre,direction,s);
  }

  sparse_poly1 series__SPOL1(const gen & e,const identificateur & x,const gen & lim_point,int ordre,int direction){
    sparse_poly1 s;
    series__SPOL1(e,x,lim_point,ordre,direction,s);
    return s;
#ifdef DEBUG_SUPPORT
    s.dbgprint();
#endif
  }

  // ***********************
  // LIMITS
  // ***********************

  bool contains(const gen & e,const gen & elem){
    if (e==elem)
      return true;
    if (e.type==_VECT){
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	if (contains(*it,elem))
	  return true;
      return false;
    }
    if (e.type==_SYMB){
      return contains(e._SYMBptr->feuille,elem);
    }
    if (e.type==_FRAC)
      return contains(e._FRACptr->num,elem) || contains(e._FRACptr->den,elem);
    return false;
  }
  
  vecteur lvarx(const gen &e,const identificateur & x){
    vecteur v(lvar(e));
    vecteur res;
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      // remove ^ if exponent does not depend on x
      if ( (it->type==_SYMB) 
	   && (it->_SYMBptr->sommet==at_pow) 
	   && !contains((*(it->_SYMBptr->feuille._VECTptr))[1],x)
	   ){
	vecteur tmp(lvarx((*(it->_SYMBptr->feuille._VECTptr))[0],x));
	const_iterateur it=tmp.begin(),itend=tmp.end();
	for (;it!=itend;++it){
	  if (!equalposcomp(res,*it))
	    res.push_back(*it);
	}
      }
      else {
	if (contains(*it,x))
	  res.push_back(*it);
      }
    }
    return res;
  }

  void rlvarx(const gen &e,const identificateur & x,vecteur & res){
    vecteur v(lvar(e));
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (!contains(*it,x) || equalposcomp(res,*it))
	continue;
      // recursive call
      res.push_back(*it);
      if (it->type==_SYMB) {
	rlvarx(it->_SYMBptr->feuille,x,res);
	if ( (it->_SYMBptr->sommet==at_pow) 
	     && contains((*(it->_SYMBptr->feuille._VECTptr))[1],x) )
	  rlvarx(ln((*(it->_SYMBptr->feuille._VECTptr))[0]),x,res);
      }
    }
  }

  vecteur rlvarx(const gen &e,const identificateur & x){
    vecteur res;
    rlvarx(e,x,res);
    sort(res.begin(),res.end(),symb_size_less);
    return res;
  }

  void upscale(gen & e,const identificateur & x){
    vecteur a_remplacer,remplacer_par;
    a_remplacer.push_back(ln(x));
    remplacer_par.push_back(x);
    a_remplacer.push_back(x);
    remplacer_par.push_back(exp(x));
    e=subst(e,a_remplacer,remplacer_par);
  }

  void downscale(gen & e,const identificateur & x){
    vecteur a_remplacer,remplacer_par;
    a_remplacer.push_back(exp(x));
    remplacer_par.push_back(x);
    a_remplacer.push_back(x);
    remplacer_par.push_back(ln(x));
    e=subst(e,a_remplacer,remplacer_par);
  }
  
  /*
  gen pow2exp(const gen & e,const identificateur & x){
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      vecteur v;
      v.reserve(itend-it);
      for (;it!=itend;++it)
	v.push_back(pow2exp(*it,x));
      return v;
    }
    if (e.type!=_SYMB)
      return e;
    if ( e._SYMBptr->sommet==at_pow && contains((*(e._SYMBptr->feuille._VECTptr))[1],x))
      return exp(pow2exp((*(e._SYMBptr->feuille._VECTptr))[1],x)*pow2exp(ln((*(e._SYMBptr->feuille._VECTptr))[0]),x));
    if ( e._SYMBptr->sommet==at_tan && contains (e._SYMBptr->feuille,x))
      return symbolic(at_sin,pow2exp(e._SYMBptr->feuille,x))/symbolic(at_cos,pow2exp(e._SYMBptr->feuille,x));
    return e._SYMBptr->sommet(pow2exp(e._SYMBptr->feuille,x),0); // FIXME GIAC_CONTEXT
  }
  */

  void mrv_lead_term(const gen & e,const identificateur & x,gen & coeff, gen & mrv_var, gen & exponent,sparse_poly1 & q,int begin_ordre=3);

  gen in_limit(const gen & e,const identificateur & x,const gen & lim_point,int direction=0){
    // Adjust direction for +/- inf limits
    if (lim_point==plus_inf)
      direction=1;
    if (lim_point==minus_inf)
      direction=-1;    
    // First try substitution
    gen first_try= quotesubst(e,x,lim_point);
    first_try = normal(eval(first_try));
    if (!is_undef(first_try)){
      if (!direction) 
	return first_try;
      if (first_try!=unsigned_inf)
	return first_try;
    }
    checkanglemode(0); // FIXME GIAC_CONTEXT
    if (e.type!=_SYMB)
      return e;
    if (e._SYMBptr->sommet==at_exp)
      return exp(in_limit(e._SYMBptr->feuille,x,lim_point,direction));
    if (e._SYMBptr->sommet==at_ln)
      return ln(in_limit(e._SYMBptr->feuille,x,lim_point,direction));
    gen e_copy;
    // Rewrite non rational ^ and tan 
    e_copy=_pow2exp(tan2sincos(e));
    if (!direction) { // supposed to be analytic, series expansion only
      sparse_poly1 p;
      p.push_back(monome(undef,0));
      int ordre=5;
      for ( ; !p.empty() && is_undef(p.front().coeff) && (ordre<max_order);ordre=2*ordre) 
	p=series__SPOL1(e_copy,x,lim_point,ordre,0);
      // cout << p << endl;
      if (ordre>=max_order)
	maxordererr();
      if (p.empty() || ck_is_strictly_positive(p.front().exponent,0))
	return 0;
      if (ck_is_strictly_positive(-p.front().exponent,0))
	return unsigned_inf;
      if (is_zero(p.front().exponent))
	return p.front().coeff;
      invalidserieserr("should not occur");
    }
    // Unidirectional limit, rewrite first (if needed)
    if (is_inf(lim_point)){
      if (lim_point==minus_inf)
	e_copy=subst(e_copy,x,-x);
    }
    else {
      if (direction>0)
	e_copy=subst(e_copy,x,lim_point+inv(x));
      else
	e_copy=subst(e_copy,x,lim_point-inv(x));
    }
    gen coeff,mrv_var,exponent;
    sparse_poly1 p;
    mrv_lead_term(e_copy,x,coeff,mrv_var,exponent,p);
    if (ck_is_strictly_positive(exponent,0))
      return 0;
    if (is_zero(exponent))
      return coeff;
    // check sign of coeff, if coeff depends on x first find equivalent
    gen essai=subst(coeff,x,plus_inf);
    if (is_undef(essai) || is_zero(essai) || (essai==unsigned_inf)){
      while (contains(coeff,x)){
	e_copy=coeff;
	mrv_lead_term(e_copy,x,coeff,mrv_var,exponent,p);
      }
      essai=coeff;
    }
    gen s=sign(essai,0); // FIXME GIAC_CONTEXT
    if (s==plus_one)
      return plus_inf;
    if (s==minus_one)
      return minus_inf;
    essai=eval(subst(essai,sincosinf,vecteur(sincosinf.size(),undef)));
    if (is_undef(essai))
      return undef;
    else
      return unsigned_inf;
  }
  
  // return plus_inf if a > b (at x=+infinity), !0 if a#b, 0 if a < b
  gen mrv_compare(const gen & a,const gen & b,const identificateur & x){
    if ((a.type!=_SYMB) && (b.type!=_SYMB))
      return 1;
    gen lna,lnb,l;
    if ((a.type==_SYMB) && (a._SYMBptr->sommet==at_exp))
      lna=a._SYMBptr->feuille;
    else
      lna=ln(a);
    if ((b.type==_SYMB) && (b._SYMBptr->sommet==at_exp))
      lnb=b._SYMBptr->feuille;
    else
      lnb=ln(b);
    gen coeff,mrv_var,exponent;
    sparse_poly1 p;
    mrv_lead_term(rdiv(lna,lnb),x,coeff,mrv_var,exponent,p);
    if (ck_is_strictly_positive(exponent,0))
      return 0;
    if (is_zero(exponent))
      return coeff;
    return plus_inf;
  }

  void mrv_max(const vecteur & a_faster_var, const vecteur & a_coeff_ln, const vecteur & a_slower_var, const vecteur & b_faster_var,const vecteur & b_coeff_ln, const vecteur & b_slower_var,const identificateur & x, vecteur & faster_var, vecteur & coeff_ln, vecteur & slower_var){
    int pos_a,pos_b;
    gen s;
    if (intersect(a_faster_var,b_faster_var,pos_a,pos_b))
      s=normal(rdiv(a_faster_var[pos_a],b_faster_var[pos_b]));
    else {
      if (a_faster_var.empty() || intersect(a_faster_var,b_slower_var,pos_a,pos_b))
	s=0;
      else {
	if (b_faster_var.empty() || intersect(b_faster_var,a_slower_var,pos_a,pos_b) )
	  s=plus_inf;
	else
	  s=mrv_compare(a_faster_var.front(),b_faster_var.front(),x);
      }
      if (s==plus_inf){
	slower_var=mergevecteur(a_slower_var,b_slower_var);
	slower_var=mergevecteur(b_faster_var,slower_var);      
	faster_var=a_faster_var;
	coeff_ln=a_coeff_ln;
	return ;
      }
      if (is_zero(s)){
	slower_var=mergevecteur(a_slower_var,b_slower_var);
	slower_var=mergevecteur(a_faster_var,slower_var);
	faster_var=b_faster_var;
	coeff_ln=b_coeff_ln;
	return;
      }
    }
    // s!=0 && s!=plus_inf
    // size test used to get w or inv(w) at the front()
    if (a_faster_var.front().symb_size()>b_faster_var.front().symb_size()){
      coeff_ln=mergevecteur(b_coeff_ln,multvecteur(s,a_coeff_ln));
      faster_var=mergevecteur(b_faster_var,a_faster_var);
      slower_var=mergevecteur(b_slower_var,a_slower_var);
    }
    else {
      coeff_ln=mergevecteur(a_coeff_ln,multvecteur(inv(s),b_coeff_ln));
      faster_var=mergevecteur(a_faster_var,b_faster_var);
      slower_var=mergevecteur(a_slower_var,b_slower_var);
    }
  }
  
  // Find most rapidly varying subexpression of e and res
  void mrv(const gen & e,const identificateur & x,vecteur & faster_var,vecteur & coeff_ln, vecteur & slower_var){
    // Find all var of e depending on x
    vecteur v0(lvarx(e,x));
    // Find mrv of these vars
    vecteur::const_iterator it=v0.begin(),itend=v0.end();
    for (;it!=itend;++it){
      if (equalposcomp(faster_var,*it) || equalposcomp(slower_var,*it))
	continue;
      if (it->type!=_SYMB){
	mrv_max(faster_var,coeff_ln,slower_var,
		vecteur(1,*it),vecteur(1,plus_one),vecteur(0),
		x,faster_var,coeff_ln,slower_var);
	continue;
      }
      gen temp=*it;
      if (temp._SYMBptr->sommet==at_ln){
	mrv(temp._SYMBptr->feuille,x,faster_var,coeff_ln,slower_var);
	continue;
      }
      if (temp._SYMBptr->sommet==at_pow)
	temp=symbolic(at_exp,(*(temp._SYMBptr->feuille._VECTptr))[1]*ln((*(temp._SYMBptr->feuille._VECTptr))[0]));
      if (temp._SYMBptr->feuille.type==_VECT)
	setsizeerr("Unable to handle "+temp.print());
      gen l=in_limit(temp._SYMBptr->feuille,x,plus_inf);
      if (is_undef(l) || l==unsigned_inf)
	setsizeerr("Undef/Unsigned Inf encountered in limit");
      if (!is_inf(l)){
	mrv(temp._SYMBptr->feuille,x,faster_var,coeff_ln,slower_var);
	continue;
      }
      if (temp._SYMBptr->sommet==at_exp){
	mrv(temp._SYMBptr->feuille,x,faster_var,coeff_ln,slower_var);
	mrv_max(faster_var,coeff_ln,slower_var,
		vecteur(1,temp),vecteur(1,plus_one),vecteur(0),
		x,faster_var,coeff_ln,slower_var);
	continue;
      }
      // (semi-)tractable functions?
      gen shift_coeff;
      if (!temp._SYMBptr->sommet.ptr->series_expansion)
	invalidserieserr("no taylor method for "+temp._SYMBptr->sommet.ptr->print());
      gen test=temp._SYMBptr->sommet.ptr->series_expansion(l,0,temp._SYMBptr->sommet,0,shift_coeff,0); // fixme: 0 should be image_of_direction FIXME GIAC_CONTEXT
      if (!is_undef(test)){
	mrv(temp._SYMBptr->feuille,x,faster_var,coeff_ln,slower_var);
	continue;
      }
      // fixme: add support for integrals
      assert(temp._SYMBptr->sommet==at_exp);
    }
  }

  // find asymptotic equivalent of e in terms of the mrv var of e
  void mrv_lead_term(const gen & e,const identificateur & x,gen & coeff, gen & mrv_var, gen & exponent,sparse_poly1 & q,int begin_ordre){
    if (!contains(e,x)){
      coeff=e;
      mrv_var=x;
      exponent=0;
      q.clear();
      q.push_back(monome(e,0));
      return ;
    }
    vecteur faster_var,coeff_ln,slower_var;
    gen ecopy(e);
    mrv(ecopy,x,faster_var,coeff_ln,slower_var);
    int rescaling=0; // number of ln(x) -> x substitution
    // if the mrv contains x, we must change scale ln(x) -> x and x -> e^x
    for (;equalposcomp(faster_var,x);++rescaling){
      upscale(ecopy,x);
      // next test needed to avoid infinite recursion
      // if there is an e^x inside the new mrv is m rescaled
      if (contains(ecopy,exp(x))){
	gen temp(faster_var);
	upscale(temp,x);
	faster_var=*temp._VECTptr;
      }
      else {
	faster_var.clear();
	slower_var.clear();
	coeff_ln.clear();
	mrv(ecopy,x,faster_var,coeff_ln,slower_var);
      }
    }
    // now find w, the mrv element -> 0 and express other elements of the mrv
    // algo: sort faster_var by symb_size
    // w is the shortest one, set g=ln(w)
    // replace the next shortest at position pos
    // by w^coeff_ln[pos]* exp(ln(faster_var[pos])-coeff_ln[pos]*g)
    // then replace faster_var[0] by w above
    // go on, the replace operation should be done with 
    // previous ordered_faster by their expression in terms of w
    // At the end replace w by 1/w if w -> plus_inf
    bool dont_invert=is_zero(in_limit(faster_var.front(),x,plus_inf));
    vecteur faster_var_tmp(faster_var);
    stable_sort(faster_var.begin(),faster_var.end(),symb_size_less);
    identificateur w(" w");
    vecteur faster_var_subst(1,w);
    gen g=faster_var.front()._SYMBptr->feuille;
    iterateur it=faster_var.begin()+1,itend=faster_var.end();
    gen c,f;
    for (;it!=itend;++it){
      int p=equalposcomp(faster_var_tmp,*it);
      // assert(p);
      c=coeff_ln[p-1];
      f=subst(it->_SYMBptr->feuille,faster_var,faster_var_subst);
      faster_var_subst.push_back(pow(w,c)*exp(normal(f-c*g)));
    }
    // subst in original expression and make the asymptotic expansion
    int ordre=begin_ordre;
    f=subst(ecopy,faster_var,faster_var_subst);
    if (!dont_invert)
      f=subst(f,w,inv(w));
    sparse_poly1 p;
    p.push_back(monome(undef,0));
    for (; !p.empty() && 
	   (p.size()==1) && is_undef(p.front().coeff) ;ordre *= 2){
      p=series__SPOL1(f,w,0,ordre,1);
      // replace ln( w) in coeff by g or -g
      if (dont_invert)
	p=subst(p,ln(w),g);
      else
	p=subst(p,ln(w),-g);
    }
    q=p;
    if (!p.empty()){
      // check for exponent 0 at front()
      if (is_zero(p.front().exponent) && contains(p.front().coeff,x))
	mrv_lead_term(p.front().coeff,x,coeff,mrv_var,exponent,p);
      else {
	if (dont_invert)
	  mrv_var=faster_var.front();
	else
	  mrv_var=inv(faster_var.front());
	coeff = p.front().coeff;
	exponent = p.front().exponent;
      }
    }
    // mrv_var is w as function of x, rescaled using the int variable rescaling
    // coeff must be rescaled as well
    for (;rescaling;--rescaling){
      downscale(mrv_var,x);
      downscale(coeff,x);
    }
  }

  bool intersect(const vecteur & a,const vecteur &b,int & pos_a,int & pos_b){
    vecteur res;
    if (a.empty() || b.empty())
      return false;
    vecteur::const_iterator it=a.begin(),itend=a.end();
    for (;it!=itend;++it)
      pos_b=equalposcomp(b,*it);
      if (pos_b){
	--pos_b;
	pos_a=it-a.begin();
	return true;
      }
    return false;
  }

  int convert_to_direction(const gen & l){
    if (is_one(l))
      return 1;
    if (is_minus_one(l))
      return -1;
    if (is_zero(l))
      return 0;
    setsizeerr();
  }

  // Main limit entry point
  gen limit(const gen & e,const identificateur & x,const gen & lim_point,int direction,GIAC_CONTEXT){
    // Insert here code for cleaning limit remember
    bool save_inside_limit=inside_limit(contextptr);
    inside_limit(true,contextptr);
    sincosinf.clear();
    gen l=in_limit(e,x,lim_point,direction);
    inside_limit(save_inside_limit,contextptr);
    vecteur sincosinfsub(sincosinf.size(),undef);
    l=eval(subst(l,sincosinf,sincosinfsub));
    return l;
  }

  // "unary" version
  const string _limit_s("limit");
  gen _limit(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return limit(args,*vx_var._IDNTptr,0,0,contextptr);
    int s=args._VECTptr->size();
    if (!s)
      toofewargs(_limit_s);
    if (s==1)
      return limit( (*(args._VECTptr))[0],*vx_var._IDNTptr,0,0,contextptr);
    gen e=(*(args._VECTptr))[1];
    if (s==2){
      if (e.type==_IDNT)
	return limit( (*(args._VECTptr))[0],*e._IDNTptr,0,0,contextptr);
      if (e.type!=_SYMB)
	settypeerr();
      if (e._SYMBptr->sommet!=at_equal)
	setsizeerr();
      gen x=(*(e._SYMBptr->feuille._VECTptr))[0];
      if (x.type!=_IDNT)
	setsizeerr();
      return limit( (*(args._VECTptr))[0],*x._IDNTptr,(*(e._SYMBptr->feuille._VECTptr))[1],0,contextptr);
    }
    if (s==3){
      if (e.type==_IDNT)
	return limit( (*(args._VECTptr))[0],*e._IDNTptr,(*(args._VECTptr))[2],0,contextptr);
      if (e.type!=_SYMB)
	settypeerr();
      if (e._SYMBptr->sommet!=at_equal)
	setsizeerr();
      gen x=(*(e._SYMBptr->feuille._VECTptr))[0];
      if (x.type!=_IDNT)
	setsizeerr();
      return limit( (*(args._VECTptr))[0],*x._IDNTptr,(*(e._SYMBptr->feuille._VECTptr))[1],convert_to_direction((*(args._VECTptr))[2]),contextptr);
    }
    if (s>4)
      toomanyargs(_limit_s);
    if (e.type!=_IDNT)
      settypeerr();
    return limit( (*(args._VECTptr))[0],*e._IDNTptr,(*(args._VECTptr))[2],convert_to_direction((*(args._VECTptr))[3]),contextptr);
  }
  string texprintaslimit(const gen & g,const string & orig_s){
    string s("\\lim ");
    if (g.type!=_VECT)
      return s+gen2tex(g);
    vecteur v(*g._VECTptr);
    int l(v.size());
    if (!l)
      return s;
    if (l==1)
      return s+gen2tex(v[0]);
    if (l==2)
      return s+"_{"+gen2tex(v[1])+"}"+gen2tex(v[0]);
    // directional limit
    if (l==3){
      if (is_one(v[2]))
	return s+"_{"+gen2tex(v[1])+"^+}"+gen2tex(v[0]);
      if (is_minus_one(v[2]))
	return s+"_{"+gen2tex(v[1])+"^-}"+gen2tex(v[0]);
      else return s+"_{"+gen2tex(v[1])+"}"+gen2tex(v[0]);
    }
    return s;
  }
  unary_function_eval __limit(&_limit,_limit_s,0,&texprintaslimit);
  unary_function_ptr at_limit (&__limit);

  gen in_series(const gen & e,const identificateur & x,const gen & lim_point=0,int ordre=5,int direction=0){
    checkanglemode(0); // FIXME GIAC_CONTEXT
    if (lim_point==plus_inf){
      gen coeff,mrv_var,exponent;
      sparse_poly1 s;
      mrv_lead_term(e,x,coeff,mrv_var,exponent,s,ordre);
      return sparse_poly12gen(s,mrv_var);
    }
    if (lim_point==minus_inf){
      gen coeff,mrv_var,exponent;
      sparse_poly1 s;
      mrv_lead_term(subst(e,x,-x),x,coeff,mrv_var,exponent,s,ordre);
      return subst(sparse_poly12gen(s,mrv_var),x,-x);
    }
    if (direction==1){
      gen ecopy=subst(e,x,lim_point+inv(x));
      gen coeff,mrv_var,exponent;
      sparse_poly1 s;
      mrv_lead_term(ecopy,x,coeff,mrv_var,exponent,s,ordre);
      return subst(sparse_poly12gen(s,mrv_var),x,inv(x-lim_point));
    }
    if (direction==-1){
      gen ecopy=subst(e,x,lim_point-inv(x));
      gen coeff,mrv_var,exponent;
      sparse_poly1 s;
      mrv_lead_term(ecopy,x,coeff,mrv_var,exponent,s,ordre);
      return subst(sparse_poly12gen(s,mrv_var),x,inv(lim_point-x));
    }
    switch (e.type){
    case _INT_: case _ZINT: case _DOUBLE_: case _CPLX:
      return e;
    case _IDNT:
      if ( !ordre && (*e._IDNTptr==x) )
	return lim_point+(x-lim_point)*order_size(x-lim_point);
      else
	return e;
    case _SYMB:
      return sparse_poly12gen(series__SPOL1(e,x,lim_point,ordre,direction),x-lim_point);
    default:
      return symbolic(at_series,makevecteur(e,x,lim_point,ordre));
    }
  }

  // Main series entry point
  gen series(const gen & e,const identificateur & x,const gen & lim_point,int ordre,int direction){
    gen res=in_series(_pow2exp(tan2sincos(e)),x,lim_point,ordre,direction);
    return res;
  }

  gen series(const gen & e,const gen & vars,const gen & lim_point,int ordre,int direction){
    gen x,l;
    if (vars.is_symb_of_sommet(at_equal)){ 
      // vars= x==lim_point, overwrites lim_point definition
      // direction is given by lim_point (for interactive input)
      x = (*(vars._SYMBptr->feuille._VECTptr)) [0];
      l = (*(vars._SYMBptr->feuille._VECTptr)) [1];
      if (lim_point.type!=_INT_)
	setsizeerr();
      if (absint(lim_point.val)>0)
	ordre=absint(lim_point.val);
      else
	direction = lim_point.val;
    }
    else {
      x=vars;
      l=lim_point;
    }
    if (x.type==_VECT && l.type==_VECT){
      vecteur &v=*x._VECTptr;
      gen h(identificateur(" h"));
      vecteur w=multvecteur(h,subvecteur(v,*l._VECTptr));
      gen newe=subst(e,v,w);
      sparse_poly1 res=series__SPOL1(newe,*h._IDNTptr,zero,ordre,direction);
      if (!res.empty() && is_undef(res.back().coeff))
	res.pop_back();
      // remove order term 
      return sparse_poly12gen(res,1);
    }
    if (x.type!=_IDNT){
      identificateur xx("x");
      gen res=series(subst(e,x,xx),xx,l,ordre,direction);
      return subst(res,xx,x);
    }
    return series(e,*x._IDNTptr,l,ordre,direction);
  }

  gen series(const gen & e,const gen & vars,const gen & lim_point,const gen & ordre){
    if (ordre.type!=_INT_)
      setsizeerr();
    if (vars.is_symb_of_sommet(at_equal))
      return series(e,vars,lim_point,0,ordre.val); // it's the direction
    return series(e,vars,lim_point,ordre.val);
  }

  // "unary" version
  const string _series_s("series");
  gen _series(const gen & args){
    if (args.type!=_VECT)
      return series(args,vx_var);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (!s)
      toofewargs(_series_s);
    if (s==1)
      return series( v[0],vx_var);
    if (s==2)
      return series( v[0],v[1]);
    if (s==3){
      if ( (v[1].type==_VECT && v[2].type==_VECT) ||
	   ( v[1].type==_IDNT || ( v[1].type==_SYMB && (v[1]._SYMBptr->sommet==at_equal || v[1]._SYMBptr->sommet==at_at ) ) )
	   )
	return series( v[0],v[1],v[2]);
      return series( v[0],symbolic(at_equal,makevecteur(vx_var,v[1])),v[2]);
    }
    if (s==4)
      return series( v[0],v[1],v[2],v[3]);    
    if (s>5 || v[3].type!=_INT_ || v[4].type!=_INT_)
      toomanyargs(_series_s);
    return series(v[0],v[1],v[2],v[3].val,v[4].val);
  }
  unary_function_unary __series(&_series,_series_s);
  unary_function_ptr at_series (&__series);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

