// -*- mode:C++ ; compile-command: "g++ -I.. -g -c solve.cc" -*-
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _GIAC_SOLVE_H
#define _GIAC_SOLVE_H
#include "first.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
  extern int intvar_counter;
  extern int realvar_counter;
  gen equal2diff(const gen & g); // rewrite = as -
  vecteur find_singularities(const gen & e,const identificateur & x,GIAC_CONTEXT);
  // isolate_mode & 1 is complex_mode, isolate_mode & 2 is 0 for principal sol
  vecteur solve(const gen & e,const identificateur & x,int isolate_mode=0,GIAC_CONTEXT=0);
  vecteur solve(const gen & e,const gen & x,int isolate_mode,GIAC_CONTEXT);
  vecteur solve(const vecteur & v,bool complex_mode,GIAC_CONTEXT); // v is a 1-d dense polynomial
  void solve(const gen & e,const identificateur & x,vecteur &v,int isolate_mode,GIAC_CONTEXT0);
  void in_solve(const gen & e,const identificateur & x,vecteur &v,int isolate_mode,GIAC_CONTEXT0);
  gen solvepostprocess(const gen & g,const gen & x);
  // convert solutions to an expression
  gen _solve(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_solve ;
  gen _fsolve(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_fsolve ;
  vecteur sxa(const vecteur & sl,const vecteur & x);
  vecteur linsolve(const vecteur & sl,const vecteur & x);
  gen symb_linsolve(const gen & syst,const gen & vars);
  gen _linsolve(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_linsolve ;
  
  /*
  gen newtona(const gen & f, const gen & x, const gen & arg,int niter1, int niter2, double eps1,double eps2,double prefact1,double prefact2, int & b);
  gen newton(const gen & f, const gen & x,const gen & guess,int niter1=5,int niter2=50,double eps1=1e-3,double eps2=1e-12,double prefact1=0.5,double prefact2=1.0);
  */
  
  gen newton(const gen & f, const gen & x,const gen & guess,int niter=20,double eps1 = 1e-5, double eps2=1e-12);
  gen _newton(const gen & args,GIAC_CONTEXT0);
  extern unary_function_ptr at_newton ;

  bool has_num_coeff(const vecteur & v);
  bool has_num_coeff(const polynome & p);
  bool has_num_coeff(const gen & e);

  polynome spoly(const polynome & p,const polynome & q);
  polynome reduce(const polynome & p,std::vector<polynome>::const_iterator it,std::vector<polynome>::const_iterator itend);
  polynome reduce(const polynome & p,const std::vector<polynome> & v);
  void reduce(std::vector<polynome> & res);
  std::vector<polynome> gbasis(const std::vector<polynome> & v);
  gen remove_equal(const gen & f);
  vecteur remove_equal(const_iterateur it,const_iterateur itend);
  vecteur gsolve(const vecteur & eq_orig,const vecteur & var,bool complexmode,GIAC_CONTEXT0);

  gen _greduce(const gen & args);
  extern unary_function_ptr at_greduce ;

  gen _gbasis(const gen & args);
  extern unary_function_ptr at_gbasis ;

  double nan();
  gen remove_and(const gen & g,const unary_function_ptr & u);
  vecteur solvepreprocess(const gen & args,GIAC_CONTEXT0);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

#endif // _GIAC_SOLVE_H
