// -*- mode:C++ ; compile-command: "g++ -I.. -g -c symbolic.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include "symbolic.h"
#include "identificateur.h"
#include "usual.h"
#include "prog.h"
#include "rpn.h"
#include "plot.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  // unary_function_ptr quoted_op[]={at_of,at_for,at_bloc,at_local,at_program,at_rpn_prog,at_ifte,at_try_catch,at_print,at_signal,at_as_function_of,at_lieu,at_legende,at_debug,at_sst,at_sst_in,at_cont,at_kill,at_halt,at_watch,at_rmwatch,at_breakpoint,at_maple2mupad,at_mupad2maple,at_maple2xcas,at_mupad2xcas,at_purge,0};
  unary_function_ptr archive_function_tab[]={at_plus,at_neg,at_binary_minus,at_prod,at_division,at_inv,at_pow,at_exp,at_ln,at_abs,at_arg,at_pnt,at_point,at_segment,at_sto,at_sin,at_cos,at_tan,at_asin,at_acos,at_atan,at_sinh,at_cosh,at_tanh,at_asinh,at_acos,at_atanh,at_interval,at_union,at_minus,at_intersect,at_not,at_and,at_ou,at_inferieur_strict,at_inferieur_egal,at_superieur_strict,at_superieur_egal,at_different,at_equal,at_rpn_prog,at_local,at_return,at_Dialog,at_double_deux_points,at_pointprod,at_pointdivision,at_pointpow,at_hash,at_pourcent,at_tilocal,at_break,at_continue,at_ampersand_times,at_maple_lib,at_unit,at_plot_style,at_xor,at_check_type,at_quote_pow,at_case,at_dollar,at_IFTE,at_RPN_CASE,at_RPN_LOCAL,at_RPN_FOR,at_RPN_WHILE,at_NOP,at_unit,0};
  vector<string> last_evaled_function_name;

  int equalposcomp(unary_function_ptr tab[],const unary_function_ptr & f){
    for (int i=1;*tab!=0;++tab,++i){
      if (*tab==f)
	return i;
    }
    return 0;
  }

  symbolic::symbolic(const symbolic & mys,const gen & e): sommet(mys.sommet){
    vecteur tmp;
    if (mys.feuille.type==_VECT){
      tmp = *mys.feuille._VECTptr;
      tmp.push_back(e);
    }
    else {
      tmp.push_back(mys.feuille);
      tmp.push_back(e);
    }
    feuille = tmp;
  };
  
  symbolic::symbolic(const gen & a,const unary_function_ptr & o,const gen & b):sommet(o) {
    if (b.type==_VECT)
      feuille= mergevecteur(vecteur(1,a),*b._VECTptr);
    else
      feuille=makevecteur(a,b);
  };
  
  int symbolic::size() const {
    if (feuille.type==_SYMB)
      return 1+feuille._SYMBptr->size();
    if (feuille.type!=_VECT)
      return 2;
    int s=1;
    iterateur it=feuille._VECTptr->begin(),itend=feuille._VECTptr->end();
    for (;it!=itend;++it){
      if (it->type==_SYMB)
	s += it->_SYMBptr->size();
      else
	++s;
    }
    return s;
  }

  string symbolic::print() const
  {
    // first look if operator has it's own printing method
    if ( (sommet.ptr->printsommet) && (sommet!=at_plus) && (sommet!=at_prod) )
      return sommet.ptr->printsommet(feuille,sommet.ptr->s);
    // default
    if ( (feuille.type==_VECT) && (feuille._VECTptr->empty()) )
        return sommet.ptr->print()+"(NULL)";
    if ( (feuille.type!=_VECT) || ( (sommet!=at_prod) && (feuille._VECTptr->front().type==_VECT) ) ){
      if (sommet==at_neg){
	if (feuille.type!=_CPLX){
	  if (feuille.type!=_SYMB)
	    return sommet.ptr->print()+feuille.print() ;
	  if (feuille._SYMBptr->sommet==at_inv)
	    return sommet.ptr->print()+feuille.print() ;
	}
	return sommet.ptr->print()+string("(") + feuille.print() +string(")");
      }
      if (sommet==at_inv){
	if ((feuille.type!=_SYMB) && (feuille.type!=_CPLX) && (feuille.type!=_MOD))
	  return "1/"+feuille.print();
	else
	  return "1/("+feuille.print()+")";
      }
      return sommet.ptr->print()+string("(") + feuille.print() +string(")");
    }
    string s;
    string opstring( sommet.ptr->print());
    int l=feuille._VECTptr->size();
    if ( sommet==at_plus ){
      for (int i=0;i<l;++i){
	gen e((*(feuille._VECTptr))[i]);
	if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg)){
	  s += string("-") + e._SYMBptr->feuille.print();
	}
	else {
	  if ( ( (e.type==_INT_) || (e.type==_ZINT) ) && !is_positive(e,0) )
	    s += e.print();
	  else {
	    if (i)
	      s += opstring;
	    s += e.print();
	  }
	}
      } // end_for
      return s;
    }
    if (sommet==at_prod) {
      for (int i=0;i<l;++i){
	gen e((*(feuille._VECTptr))[i]);
	if (e.type!=_SYMB){
	  if (i)
	    s += opstring;
	  if ( (e.type==_CPLX) || (e.type==_MOD) )
	    s += "("+e.print()+")";
	  else
	    s +=e.print();
	}
	else {
	  if (e._SYMBptr->sommet==at_inv){
	    gen f(e._SYMBptr->feuille);
	    if (i){
	      if ( (f.type==_CPLX) || (f.type==_MOD) ||
		   ((f.type==_SYMB) && 
		    ( (f._SYMBptr->sommet==at_plus) || (f._SYMBptr->sommet==at_prod) || (f._SYMBptr->sommet==at_neg) ))
		   )
		s += string("/(")+f.print() + string(")");
	      else
		s += string("/")+f.print();
	    }
	    else
	      s += e.print();
	  } // end if e._SYMBptr->sommet==at_inv
	  else {
	    if (i)
	      s += opstring;
	    if ( (e._SYMBptr->sommet==at_plus) || (e._SYMBptr->sommet==at_neg) )
	      s += string("(")+e.print()+string(")");
	    else
	      s += e.print();
	  }
	}
      } // end_for
      return s;
    } // end if sommet_is_prod
    s = opstring + '(';
    if (feuille.subtype)
      s += begin_VECT_string(feuille.subtype);
    for (int i=0;;++i){
      s += (*(feuille._VECTptr))[i].print();
      if (i==l-1){
	break;
      }
      s += ',';
    }
    if (feuille.subtype)
      s += end_VECT_string(feuille.subtype);
    return s+')';
  }

  gen symbolic::eval(int level,const context * contextptr) const {
    last_evaled_function_name.push_back(sommet.ptr->print()+' ');
    gen ans;
    if (sommet==at_sto && feuille.type==_VECT){ // autoname function
      if ( (feuille._VECTptr->back().type==_SYMB) && (feuille._VECTptr->back()._SYMBptr->sommet==at_unquote || feuille._VECTptr->back()._SYMBptr->sommet==at_hash) ){
	ans=_sto(feuille.eval(level-1,contextptr),contextptr);
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return ans;
      }
      bool b=show_point(contextptr);
      show_point(false,contextptr);
      gen e=feuille._VECTptr->front().eval(level-1,contextptr);
      show_point(b,contextptr);
      if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt) && (e._SYMBptr->feuille.type==_VECT) && (e._SYMBptr->feuille._VECTptr->size()==2)){
	e=symbolic(at_pnt,gen(makevecteur(e._SYMBptr->feuille._VECTptr->front(),e._SYMBptr->feuille._VECTptr->back(),string2gen(feuille._VECTptr->back().print(),false)),_PNT__VECT));
	e.subtype=gnuplot_show_pnt(*e._SYMBptr,contextptr);
      }
      if ( (e.type==_VECT) && (e._VECTptr->size()) && (e._VECTptr->back().type==_SYMB) && (e._VECTptr->back()._SYMBptr->sommet==at_pnt)){
	vecteur v=*e._VECTptr;
	iterateur it=v.begin(),itend=v.end();
	for (;it!=itend;++it){
	  if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_pnt) && (it->_SYMBptr->feuille._VECTptr->size()==2)){
	    *it=symbolic(at_pnt,gen(makevecteur(it->_SYMBptr->feuille._VECTptr->front(),it->_SYMBptr->feuille._VECTptr->back().val,string2gen(feuille._VECTptr->back().print(),false)),_PNT__VECT));
	    it->subtype=gnuplot_show_pnt(*it->_SYMBptr,contextptr);
	  }
	}
	e=gen(v,e.subtype);
      }
      ans=sto(e,feuille._VECTptr->back(),contextptr);
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return ans;
    } // end sommet==at_sto
    if (sommet.quoted){ 
      if (feuille.type==_SYMB){ 
	unary_function_ptr & u=feuille._SYMBptr->sommet;
	if (u==at_unquote){
	  ans=sommet(feuille.eval(level-1,contextptr),contextptr);
	  if (!last_evaled_function_name.empty())
	    last_evaled_function_name.pop_back();
	  return ans;
	}
	if (u==at_hash){
	  ans=sommet(gen(*feuille._SYMBptr->feuille._STRNGptr),contextptr);
	  if (!last_evaled_function_name.empty())
	    last_evaled_function_name.pop_back();
	  return ans;
	}
      }
      ans=sommet(feuille,contextptr);
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return ans;
    }
    else {
      if ((sommet==at_neg) && (feuille.type==_IDNT) && (*feuille._IDNTptr==_IDNT_infinity)){
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return minus_inf;
      }
      if (sommet==at_quote){
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return feuille;
      }
      ans=sommet(feuille.eval(level-1,contextptr),contextptr);
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return ans;
    }
  }
    
  gen symbolic::evalf(int level,const context * contextptr) const {
    last_evaled_function_name.push_back(sommet.ptr->print()+' ');
    if (sommet==at_sto){ // autoname function
      gen e=feuille._VECTptr->front().evalf(level,contextptr);
      if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt) && (e._SYMBptr->feuille.type==_VECT) && (e._SYMBptr->feuille._VECTptr->size()==2))
	e=symbolic(at_pnt,gen(makevecteur(e._SYMBptr->feuille._VECTptr->front(),e._SYMBptr->feuille._VECTptr->back(),string2gen(feuille._VECTptr->back().print(),false)),_PNT__VECT));
      if ( (e.type==_VECT) && (e._VECTptr->size()) && (e._VECTptr->back().type==_SYMB) && (e._VECTptr->back()._SYMBptr->sommet==at_pnt)){
	vecteur v=*e._VECTptr;
	iterateur it=v.begin(),itend=v.end();
	for (;it!=itend;++it){
	  if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_pnt) && (it->_SYMBptr->feuille._VECTptr->size()==2))
	    *it=symbolic(at_pnt,gen(makevecteur(it->_SYMBptr->feuille._VECTptr->front(),it->_SYMBptr->feuille._VECTptr->back().val,string2gen(feuille._VECTptr->back().print(),false)),_PNT__VECT));
	}
	e=v;
      }
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return sto(e,feuille._VECTptr->back(),contextptr);
    }
    gen ans;
    if (sommet.quoted && !equalposcomp(plot_sommets,sommet) ){ 
      ans=sommet(feuille,contextptr);
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return ans;
    }
    else {
      if ((sommet==at_neg) && (feuille.type==_IDNT) && (*feuille._IDNTptr==_IDNT_infinity)){
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return minus_inf;
      }
      if (sommet==at_quote){
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return feuille;
      }
      if (equalposcomp(plot_sommets,sommet)){
	// bool save_is_inevalf=is_inevalf;
	// is_inevalf=true;
	ans=symbolic(sommet,feuille.evalf());
	// is_inevalf=save_is_inevalf;
	if (!last_evaled_function_name.empty())
	  last_evaled_function_name.pop_back();
	return ans;
      }
      ans=sommet(feuille.evalf(level,contextptr),contextptr);
      if (!last_evaled_function_name.empty())
	last_evaled_function_name.pop_back();
      return ans;
    }
  }
    
  ostream & operator << (ostream & os,const symbolic & s) { return os << s.print(); }

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
