// -*- mode:C++ ; compile-command: "g++ -I.. -g -c ti89.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include <cstdlib>
// #include <numeric_limits>
#include "misc.h"
#include "usual.h"
#include "sym2poly.h"
#include "rpn.h"
#include "prog.h"
#include "derive.h"
#include "subst.h"
#include "intg.h"
#include "vecteur.h"
#include "ifactor.h"
#include "solve.h"
#include "modpoly.h"
#include "permu.h"
#include "sym2poly.h"
#include "plot.h"
#include "lin.h"
#include "modpoly.h"
#include "desolve.h"
#include "alg_ext.h"
#include "moyal.h"
#include "ti89.h"
#include "input_parser.h"
#include "input_lexer.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  /*
  string printasti_not_implemented(const gen & feuille,const string & sommetstr){
    if (feuille.type!=_VECT || feuille._VECTptr->size()!=2)
      return sommetstr+"("+feuille.print()+")";
    vecteur & v=*feuille._VECTptr;
    if (v[0].type==_STRNG)
      return *v[0]._STRNGptr+"("+v[1].print()+")";
    else
      return v[0].print()+"("+v[1].print()+")";
  }
  */
  const string _ti_not_implemented_s("ti_not_implemented");
  unary_function_unary __ti_not_implemented(&_nop,_ti_not_implemented_s); // never evaled
  unary_function_ptr at_ti_not_implemented (&__ti_not_implemented);

  const string _ti_endtag_s("ti_endtag");
  unary_function_unary __ti_endtag(&_nop,_ti_endtag_s); // never evaled
  unary_function_ptr at_ti_endtag (&__ti_endtag);

  gen _seq(const gen & g,GIAC_CONTEXT){
    return seqprod(g,0,contextptr);
  }
  const string _seq_s("seq");
  unary_function_eval __seq(&_seq,_seq_s);
  unary_function_ptr at_seq (&__seq,_QUOTE_ARGUMENTS,true);

  gen _logb(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    return ln(g._VECTptr->front())/ln(g._VECTptr->back());
  }
  const string _logb_s("logb");
  unary_function_unary __logb(&_logb,_logb_s);
  unary_function_ptr at_logb (&__logb,0,true);

  string getType(const gen & g){
    switch (g.type){
    case _INT_: case _REAL: case _DOUBLE_:
      return "NUM";
    case _VECT:
      if (ckmatrix(g))
	return "MAT";
      else
	return "LIST";
    case _IDNT:
      return "VAR";
    case _SYMB:
      if (g.is_symb_of_sommet(at_program))
	return "FUNC";
      else
	return "EXPR";
    case _CPLX:
      return "EXPR";
    case _STRNG:
      return "STR";
    default:
      return "OTHER";
    }
  }
  gen _getType(const gen & g){
    return string2gen(getType(g),false);
  }
  const string _getType_s("getType");
  unary_function_unary __getType(&_getType,_getType_s);
  unary_function_ptr at_getType (&__getType,0,true);

  gen _Define(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    gen g1=v.front(),g2=v.back();
    if (!g1.is_symb_of_sommet(at_of))
      setsizeerr();
    gen &g11=g1._SYMBptr->feuille;
    if (g11.type!=_VECT || g11._VECTptr->size()!=2)
      setsizeerr();
    gen gname=g11._VECTptr->front(),garg=g11._VECTptr->back();
    return symb_sto(symb_program(garg,garg*zero,symb_bloc(g2)),gname);
  }
  const string _colDim_s("colDim");
  unary_function_unary __colDim(&_ncols,_colDim_s);
  unary_function_ptr at_colDim (&__colDim,0,true);

  const string _rowDim_s("rowDim");
  unary_function_unary __rowDim(&_nrows,_rowDim_s);
  unary_function_ptr at_rowDim (&__rowDim,0,true);

  const string _randMat_s("randMat");
  unary_function_eval __randMat(&_ranm,_randMat_s);
  unary_function_ptr at_randMat (&__randMat,0,true);

  const string _eigVc_s("eigVc");
  unary_function_eval __eigVc(&_egv,_eigVc_s);
  unary_function_ptr at_eigVc (&__eigVc,0,true);

  const string _eigVl_s("eigVl");
  unary_function_eval __eigVl(&_egvl,_eigVl_s);
  unary_function_ptr at_eigVl (&__eigVl,0,true);

  const string _transpose_s("transpose");
  unary_function_unary __transpose(&_tran,_transpose_s);
  unary_function_ptr at_transpose (&__transpose,0,true);

  const string _identity_s("identity");
  unary_function_unary __identity(&_idn,_identity_s);
  unary_function_ptr at_identity (&__identity,0,true);

  gen _isprime(const gen & args){
    gen g=_is_prime(args);
    if (g==0){
      g.subtype=_INT_BOOLEAN;
      return g;
    }
    g=plus_one;
    g.subtype=_INT_BOOLEAN;
    return g;
  }
  const string _isprime_s("isprime");
  unary_function_unary __isprime(&_isprime,_isprime_s);
  unary_function_ptr at_isprime (&__isprime,0,true);

  /*
  gen _Input(const gen & g){
    vecteur v(gen2vecteur(g));
    int s=v.size();
    gen res;
    if (s==1)
      res=_inputform(symbolic(at_click,makevecteur(v[0],0,v[0])));
    else {
      if (s==2)
	res= _inputform(symbolic(at_click,makevecteur(v[0],0,v[1])));
      else
	res= __click.op(g);
    }
    __interactive.op(symbolic(at_print,makevecteur(g,res)));
    return res;
  }
  */
  gen _Input(const gen & args){
    return _input(args,false);
  }
  const string _Input_s("Input");
  unary_function_unary __Input(&_Input,_Input_s,&printastifunction);
  unary_function_ptr at_Input (&__Input,_QUOTE_ARGUMENTS,T_RETURN);

  const string _lis_s("lis");
  unary_function_unary __lis(&_Input,_lis_s,&printastifunction);
  unary_function_ptr at_lis (&__lis,_QUOTE_ARGUMENTS,T_RETURN);

  /*
  gen _InputStr(const gen & g){
    vecteur v(gen2vecteur(g));
    int s=v.size();
    gen res;
    if (s==1)
      res= _inputform(symbolic(at_click,makevecteur(v[0],0,v[0],0)));
    else {
      if (s==2)
	res= _inputform(symbolic(at_click,makevecteur(v[0],0,v[1],0)));
      else
	res= _inputform(symbolic(at_click,g));
    }
    __interactive.op(symbolic(at_print,makevecteur(g,res)));
    return res;
  }
  */
  gen _InputStr(const gen & args){
    return _input(args,true);
  }
  const string _InputStr_s("InputStr");
  unary_function_unary __InputStr(&_InputStr,_InputStr_s,&printastifunction);
  unary_function_ptr at_InputStr (&__InputStr,_QUOTE_ARGUMENTS,T_RETURN);

  const string _lis_phrase_s("lis_phrase");
  unary_function_unary __lis_phrase(&_InputStr,_lis_phrase_s,&printastifunction);
  unary_function_ptr at_lis_phrase (&__lis_phrase,_QUOTE_ARGUMENTS,T_RETURN);

  gen _Prompt(const gen & g){
    gen res= _inputform(symbolic(at_click,makevecteur(g,g,g)));
    __interactive.op(symbolic(at_print,makevecteur(g,res)));
    return res;
  }
  const string _Prompt_s("Prompt");
  unary_function_unary __Prompt(&_Prompt,_Prompt_s,&printastifunction);
  unary_function_ptr at_Prompt (&__Prompt,_QUOTE_ARGUMENTS,T_RETURN);

  const string _PopUp_s("PopUp");
  unary_function_unary __PopUp(&_choosebox,_PopUp_s,&printastifunction);
  unary_function_ptr at_PopUp (&__PopUp,_QUOTE_ARGUMENTS,T_RETURN);

  gen _cSolve(const gen & g,GIAC_CONTEXT){
    bool old_complex_mode=complex_mode(contextptr);
    complex_mode(true,contextptr);
    gen res=_solve(g,contextptr);
    complex_mode(old_complex_mode,contextptr);
    return res;
  }
  const string _cSolve_s("cSolve");
  unary_function_eval __cSolve(&_cSolve,_cSolve_s);
  unary_function_ptr at_cSolve (&__cSolve,_QUOTE_ARGUMENTS,true);

  gen _cFactor(const gen & g,GIAC_CONTEXT){
    bool old_complex_mode=complex_mode(contextptr);
    complex_mode(true,contextptr);
    gen res=_factor(g,contextptr);
    complex_mode(old_complex_mode,contextptr);
    return res;
  }
  const string _cFactor_s("cFactor");
  unary_function_eval __cFactor(&_cFactor,_cFactor_s);
  unary_function_ptr at_cFactor (&__cFactor,0,true);

  gen _nSolve(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=2)
      setsizeerr();
    gen var=v[1],guess;
    if (var.type==_SYMB && (var._SYMBptr->sommet==at_equal || var._SYMBptr->sommet==at_same)){
      guess=var._SYMBptr->feuille._VECTptr->back();
      var=var._SYMBptr->feuille._VECTptr->front();
      return newton(remove_equal(v[0]),var,guess);
    }
    else
      return newton(remove_equal(v[0]),var,undef);
  }
  const string _nSolve_s("nSolve");
  unary_function_unary __nSolve(&_nSolve,_nSolve_s);
  unary_function_ptr at_nSolve (&__nSolve,0,true);

  gen zeros(const gen &g,bool complexmode,GIAC_CONTEXT){
    vecteur v(solvepreprocess(g,contextptr));
    int s=v.size();
    if (s>2)
      toomanyargs("solve");
    return solve(v.front(),v.back(),complexmode,contextptr);
  }
  gen _zeros(const gen & g,GIAC_CONTEXT){
    return zeros(g,false,contextptr);
  }
  const string _zeros_s("zeros");
  unary_function_eval __zeros(&_zeros,_zeros_s);
  unary_function_ptr at_zeros (&__zeros,_QUOTE_ARGUMENTS,true);

  gen _cZeros(const gen & g,GIAC_CONTEXT){
    return zeros(g,true,contextptr);
  }
  const string _cZeros_s("cZeros");
  unary_function_eval __cZeros(&_cZeros,_cZeros_s);
  unary_function_ptr at_cZeros (&__cZeros,_QUOTE_ARGUMENTS,true);

  gen _getDenom(const gen & g){
    gen res=_fxnd(g);
    return res._VECTptr->back();
  }
  const string _getDenom_s("getDenom");
  unary_function_unary __getDenom(&_getDenom,_getDenom_s);
  unary_function_ptr at_getDenom (&__getDenom,0,true);

  const string _denom_s("denom");
  unary_function_unary __denom(&_getDenom,_denom_s);
  unary_function_ptr at_denom (&__denom,0,true);

  gen _getNum(const gen & g){
    gen res=_fxnd(g);
    return res._VECTptr->front();
  }
  const string _getNum_s("getNum");
  unary_function_unary __getNum(&_getNum,_getNum_s);
  unary_function_ptr at_getNum (&__getNum,0,true);

  const string _numer_s("numer");
  unary_function_unary __numer(&_getNum,_numer_s);
  unary_function_ptr at_numer (&__numer,0,true);

  const string _propFrac_s("propFrac");
  unary_function_unary __propFrac(&_propfrac,_propFrac_s);
  unary_function_ptr at_propFrac (&__propFrac,0,true);

  const string _tCollect_s("tCollect");
  unary_function_eval __tCollect(&_tcollect,_tCollect_s);
  unary_function_ptr at_tCollect (&__tCollect,0,true);

  gen _tExpand(const gen & g,GIAC_CONTEXT){
    return _simplify(_texpand(g,contextptr));
  }
  const string _tExpand_s("tExpand");
  unary_function_eval __tExpand(&_tExpand,_tExpand_s);
  unary_function_ptr at_tExpand (&__tExpand,0,true);

  gen _comDenom(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()<2 )
      return normal(g);
    vecteur & v(*g._VECTptr);
    return _reorder(makevecteur(v.front(),vecteur(v.begin()+1,v.end())));
  }
  const string _comDenom_s("comDenom");
  unary_function_unary __comDenom(&_comDenom,_comDenom_s);
  unary_function_ptr at_comDenom (&__comDenom,0,true);

  gen _randPoly(const gen & g){
    vecteur v(gen2vecteur(g));
    int s=v.size();
    gen x=vx_var;
    if (!s){
      s=100;
    } 
    else {
      if (s==1){
	if (v[0].type==_INT_)
	  s=v[0].val;
	else {
	  x=v[0];
	  s=100;
	}
      }
      else {
	if (v[0].type==_INT_){
	  s=v[0].val;
	  x=v[1];
	}
	else {
	  s=v[1].val;
	  x=v[0];
	}
      }
    }
    vecteur w;
    for (;;){
      w=vranm(absint(s)+1,0,0);
      if (!is_zero(w.front()))
	break;
    }
    return symb_horner(w,x);
  }
  const string _randPoly_s("randPoly");
  unary_function_unary __randPoly(&_randPoly,_randPoly_s);
  unary_function_ptr at_randPoly (&__randPoly,0,true);

  const string _randpoly_s("randpoly");
  unary_function_unary __randpoly(&_randPoly,_randpoly_s);
  unary_function_ptr at_randpoly (&__randpoly,0,true);

  gen _nInt(const gen & g){
    if (g.type!=_VECT && g._VECTptr->size()!=4)
      setsizeerr();
    return evalf(symbolic(at_integrate,g));
  }
  const string _nInt_s("nInt");
  unary_function_unary __nInt(&_nInt,_nInt_s);
  unary_function_ptr at_nInt (&__nInt,0,true);

  gen _nDeriv(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()<2)
      setsizeerr();
    gen step(0.001);
    if (v.size()>2)
      step=v[2];
    return evalf(rdiv(subst(v[0],v[1],v[1]+step)-subst(v[0],v[1],v[1]-step),2*step));
  }
  const string _nDeriv_s("nDeriv");
  unary_function_unary __nDeriv(&_nDeriv,_nDeriv_s);
  unary_function_ptr at_nDeriv (&__nDeriv,0,true);

  gen _avgRC(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()<2)
      setsizeerr();
    gen step(0.001);
    if (v.size()>2)
      step=v[2];
    return evalf(rdiv(subst(v[0],v[1],v[1]+step)-subst(v[0],v[1],v[1]),step));
  }
  const string _avgRC_s("avgRC");
  unary_function_unary __avgRC(&_avgRC,_avgRC_s);
  unary_function_ptr at_avgRC (&__avgRC,0,true);

  gen _fMin(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()==1)
      v.push_back(vx_var);
    gen w(fminmax(v,4,contextptr));
    return solvepostprocess(w,v[1]);
  }
  const string _fMin_s("fMin");
  unary_function_eval __fMin(&_fMin,_fMin_s);
  unary_function_ptr at_fMin (&__fMin,0,true);

  gen _fMax(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()==1)
      v.push_back(vx_var);
    gen w(fminmax(v,5,contextptr));
    return solvepostprocess(w,v[1]);
  }
  const string _fMax_s("fMax");
  unary_function_eval __fMax(&_fMax,_fMax_s);
  unary_function_ptr at_fMax (&__fMax,0,true);

  gen _taylor(const gen & g){
    /* if (maple_mode==0)
       return _series(g); */
    vecteur v(gen2vecteur(g));
    if (v.empty())
      toofewargs("Taylor needs 3 args");    
    if (v.size()<2)
      v.push_back(vx_var);
    if (v.size()<3)
      v.push_back(5);
    gen x0;
    if (v.size()==4)
      x0=v[3];
    if (v[1].is_symb_of_sommet(at_equal))
      return _series(makevecteur(v[0],v[1],v[2]));
    return _series(makevecteur(v[0],symbolic(at_equal,makevecteur(v[1],x0)),v[2]));
  }
  const string _taylor_s("taylor");
  unary_function_unary __taylor(&_taylor,_taylor_s);
  unary_function_ptr at_taylor (&__taylor,0,true);

  gen _arcLen(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()!=4 || v[1].type!=_IDNT)
      toofewargs("");
    gen fprime=derive(v[0],v[1]);
    if (fprime.type==_VECT)
      fprime=_l2norm(fprime,contextptr);
    else
      fprime=sqrt(normal(sq(fprime)+1));
    return _integrate(gen(makevecteur(fprime,v[1],v[2],v[3]),_SEQ__VECT),contextptr);
  }
  const string _arcLen_s("arcLen");
  unary_function_eval __arcLen(&_arcLen,_arcLen_s);
  unary_function_ptr at_arcLen (&__arcLen,0,true);

  gen _dim(const gen & g){
    if (!ckmatrix(g))
      return _size(g);
    vecteur res(2);
    if (!g._VECTptr->empty()){
      res[0]=g._VECTptr->size();
      res[1]=g._VECTptr->front()._VECTptr->size();
    }
    return res;
  }
  const string _dim_s("dim");
  unary_function_unary __dim(&_dim,_dim_s);
  unary_function_ptr at_dim (&__dim,0,true);

  // FIXME: the print() should have an additionnal format argument
  // that would cover normal, tex, C, and formatted output
  string format(const gen & g,const string & forme,GIAC_CONTEXT){
    gen tmp=evalf_double(g,DEFAULT_EVAL_LEVEL,contextptr);
    if (tmp.type!=_DOUBLE_)
      setsizeerr();
    double d=tmp._DOUBLE_val;
    char ch=forme[0];
    if (tolower(ch)!='g' || tolower(ch)!='a' || tolower(ch)!='f' || tolower(ch)!='e')
      ch='g';
    if (forme.size()<2)
      setsizeerr();
    if (my_isnan(d))
      return "undef";
    if (my_isinf(d))
      return "infinity";
    char s[256];
    sprintf(s,("%."+forme.substr(1,forme.size()-1)+ch).c_str(),d);
    return s;
  }
  gen _format(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()!=2 || v[1].type!=_STRNG)
      setsizeerr();
    return string2gen(format(v.front(),*v[1]._STRNGptr,contextptr),false);
  }
  const string _format_s("format");
  unary_function_eval __format(&_format,_format_s);
  unary_function_ptr at_format (&__format,0,true);

  gen _inString(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()<2 || v[0].type!=_STRNG || v[1].type!=_STRNG)
      setsizeerr();
    int debut=0;
    if (v.size()>2 && v[2].type==_INT_)
      debut=v[2].val;
    return v[0]._STRNGptr->find(*v[1]._STRNGptr,debut-(maple_mode!=0))+(maple_mode!=0);
  }
  const string _inString_s("inString");
  unary_function_unary __inString(&_inString,_inString_s);
  unary_function_ptr at_inString (&__inString,0,true);

  gen _left(const gen & g){
    if (g.type==_SYMB && g._SYMBptr->feuille.type==_VECT && !g._SYMBptr->feuille._VECTptr->empty())
      return g._SYMBptr->feuille._VECTptr->front();
    vecteur v(1,g);
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      v=*g._VECTptr;
    if (v.size()<2 || v[1].type!=_INT_)
      return g;
    if (v[0].type==_STRNG)
      return string2gen(v[0]._STRNGptr->substr(0,v[1].val),false);
    if (v[0].type==_VECT){
      const_iterateur it=v[0]._VECTptr->begin(),itend=v[0]._VECTptr->end();
      int length=max(0,min(itend-it,v[1].val));
      return gen(vecteur(it,it+length),v[0].subtype);
    }
    return g;
  }
  const string _left_s("left");
  unary_function_unary __left(&_left,_left_s);
  unary_function_ptr at_left (&__left,0,true);

  gen _right(const gen & g){
    if (g.type==_SYMB && g._SYMBptr->feuille.type==_VECT && !g._SYMBptr->feuille._VECTptr->empty())
      return g._SYMBptr->feuille._VECTptr->back();
    vecteur v(1,g);
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      v=*g._VECTptr;
    if (v.size()<2 || v[1].type!=_INT_)
      return g;
    if (v[0].type==_STRNG){
      string & s=*v[0]._STRNGptr;
      int l=s.size();
      int m=min(max(v[1].val,0),l);
      return string2gen(s.substr(l-m,m),false);
    }
    if (v[0].type==_VECT){
      const_iterateur it=v[0]._VECTptr->begin(),itend=v[0]._VECTptr->end();
      int length=max(0,min(itend-it,v[1].val));
      return gen(vecteur(itend-length,itend),v[0].subtype);
    }
    return g;
  }
  const string _right_s("right");
  unary_function_unary __right(&_right,_right_s);
  unary_function_ptr at_right (&__right,0,true);

  gen _mid(const gen & g){
    vecteur v(1,g);
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      v=*g._VECTptr;
    if (v.size()<2 || v[1].type!=_INT_)
      return g;
    int debut=v[1].val-(maple_mode!=0);
    int nbre=RAND_MAX;
    if (v.size()>2 && v[2].type==_INT_)
      nbre=v[2].val;
    if (v[0].type==_STRNG){
      string & s=*v[0]._STRNGptr;
      if (debut>=s.size())
	return string2gen("",false);
      int m=min(max(nbre,0),s.size());
      return string2gen(s.substr(debut,m),false);
    }
    if (v[0].type==_VECT){
      const_iterateur it=v[0]._VECTptr->begin(),itend=v[0]._VECTptr->end();
      if (debut>=itend-it)
	return gen(vecteur(0),v[0].subtype);
      int length=max(0,min(itend-it-debut,nbre));
      return gen(vecteur(it+debut,it+debut+length),v[0].subtype);
    }
    return g;
  }
  const string _mid_s("mid");
  unary_function_unary __mid(&_mid,_mid_s);
  unary_function_ptr at_mid (&__mid,0,true);

  gen _ord(const gen & g){
    if (g.type==_VECT)
      return apply(g,_ord);
    if (g.type!=_STRNG || !g._STRNGptr->size())
      setsizeerr();
    return int((*g._STRNGptr)[0]);
  }
  const string _ord_s("ord");
  unary_function_unary __ord(&_ord,_ord_s);
  unary_function_ptr at_ord (&__ord,0,true);

  gen shiftrotate(const gen & g,bool right){
    bool shift=right;
    vecteur v(1,g);
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      v=*g._VECTptr;
    int nbre=-1;
    /* if (shift)
       nbre=1; */
    if (v.size()>1 && v[1].type==_INT_)
      nbre=v[1].val;
    if (nbre<0){
      nbre=-nbre;
      right=!right;
    }
    gen & a=v[0];
    if (a.type==_INT_){
      if (right)
	return a.val >> nbre;
      else
	return a.val << nbre;
    }
    if (a.type==_VECT){
      const_iterateur it=a._VECTptr->begin(),itend=a._VECTptr->end();
      nbre=min(nbre,itend-it);
      if (shift){
	if (right)
	  return gen(mergevecteur(vecteur(it+nbre,itend),vecteur(nbre,undef)),a.subtype);
	return gen(mergevecteur(vecteur(nbre,undef),vecteur(it,itend-nbre)),a.subtype);
      }
      if (right)
	return gen(mergevecteur(vecteur(itend-nbre,itend),vecteur(it,itend-nbre)),a.subtype);
      return gen(mergevecteur(vecteur(it+nbre,itend),vecteur(it,it+nbre)),a.subtype);
    }
    if (a.type==_STRNG){
      string & s=*a._STRNGptr;
      int l=s.size();
      nbre=min(nbre,l);
      if (shift){
	if (right)
	  return string2gen(s.substr(nbre,l-nbre)+string(nbre,' '),false);
	return string2gen(string(l-nbre,' ')+s.substr(0,nbre),false);
      }
      if (right)
	return string2gen(s.substr(l-nbre,nbre)+s.substr(0,l-nbre),false);
      return string2gen(s.substr(nbre,l-nbre)+s.substr(0,nbre),false);
    }
    return a;
  }
  gen _rotate(const gen & g){
    return shiftrotate(g,false);
  }
  const string _rotate_s("rotate");
  unary_function_unary __rotate(&_rotate,_rotate_s);
  unary_function_ptr at_rotate (&__rotate,0,true);

  gen _shift(const gen & g){
    return shiftrotate(g,true);
  }
  const string _shift_s("shift");
  unary_function_unary __shift(&_shift,_shift_s);
  unary_function_ptr at_shift (&__shift,0,true);

  gen _augment(const gen & g){
    return concat(g,false);
  }
  const string _augment_s("augment");
  unary_function_unary __augment(&_augment,_augment_s);
  unary_function_ptr at_augment (&__augment,0,true);

  gen _semi_augment(const gen & g){
    return concat(g,true);
  }
  const string _semi_augment_s("semi_augment");
  unary_function_unary __semi_augment(&_semi_augment,_semi_augment_s);
  unary_function_ptr at_semi_augment (&__semi_augment,0,true);

  const string _crossP_s("crossP");
  unary_function_unary __crossP(&_cross,_crossP_s);
  unary_function_ptr at_crossP (&__crossP,0,true);

  const string _scalarProduct_s("scalarProduct");
  unary_function_unary __scalarProduct(&_dotprod,_scalarProduct_s);
  unary_function_ptr at_scalarProduct (&__scalarProduct,0,true);

  const string _dotP_s("dotP");
  unary_function_unary __dotP(&_dotprod,_dotP_s);
  unary_function_ptr at_dotP (&__dotP,0,true);

  gen _cumSum(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    const_iterateur it=v.begin(),itend=v.end();
    if (it==itend)
      return zero;
    vecteur res;
    res.reserve(itend-it);
    gen somme(zero*(*it));
    for (;it!=itend;++it){
      res.push_back(somme=*it+somme);
    }
    return gen(res,g.subtype);
  }
  const string _cumSum_s("cumSum");
  unary_function_unary __cumSum(&_cumSum,_cumSum_s);
  unary_function_ptr at_cumSum (&__cumSum,0,true);

  gen _rightapply(const gen & g){
    if (g.type!=_VECT)
      return _right(g);
    return apply(g,_rightapply);
  }
  gen _exp2list(const gen & g){
    gen g1(g);
    if (!g1.is_symb_of_sommet(at_ou))
      g1=eval(g);
    g1=remove_and(g1,at_ou);
    return _rightapply(g1);
  }
  const string _exp2list_s("exp2list");
  unary_function_unary __exp2list(&_exp2list,_exp2list_s);
  unary_function_ptr at_exp2list (&__exp2list,_QUOTE_ARGUMENTS,true);

  gen _list2mat(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    if (g.subtype!=_SEQ__VECT || v.size()!=2 || v[0].type!=_VECT || v[1].type!=_INT_)
      return vecteur(1,v);
    vecteur res;
    int nbre=max(1,v[1].val);
    const_iterateur it=v[0]._VECTptr->begin(),itend=v[0]._VECTptr->end();
    for (;it!=itend;it+=nbre){
      if (itend-it<nbre){
	res.push_back(mergevecteur(vecteur(it,itend),vecteur(nbre-(itend-it))));
	break;
      }
      res.push_back(vecteur(it,it+nbre));
    }
    return res;
  }
  const string _list2mat_s("list2mat");
  unary_function_unary __list2mat(&_list2mat,_list2mat_s);
  unary_function_ptr at_list2mat (&__list2mat,0,true);

  gen _deltalist(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    const_iterateur it=v.begin(),itend=v.end();
    if (itend-it<2)
      setdimerr();
    vecteur res;
    res.reserve(itend-it-1);
    gen prec=*it;
    ++it;
    for (;it!=itend;++it){
      res.push_back(*it-prec);
      prec=*it;
    }
    return gen(res,g.subtype);
  }
  const string _deltalist_s("deltalist");
  unary_function_unary __deltalist(&_deltalist,_deltalist_s);
  unary_function_ptr at_deltalist (&__deltalist,0,true);

  gen _mat2list(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    if (!ckmatrix(v))
      return g;
    vecteur res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      res=mergevecteur(res,*it->_VECTptr);
    }
    return res;
  }
  const string _mat2list_s("mat2list");
  unary_function_unary __mat2list(&_mat2list,_mat2list_s);
  unary_function_ptr at_mat2list (&__mat2list,0,true);

  gen _newList(const gen & g){
    if (g.type!=_INT_)
      setsizeerr();
    return vecteur(absint(g.val));
  }
  const string _newList_s("newList");
  unary_function_unary __newList(&_newList,_newList_s);
  unary_function_ptr at_newList (&__newList,0,true);

  gen polyEval(const gen & p,const gen & x){
    if (x.type==_VECT)
      return apply2nd(p,x,polyEval);
    return horner(p,x);
  }
  gen _polyEval(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    return polyEval(v[0],v[1]);
  }
  const string _polyEval_s("polyEval");
  unary_function_unary __polyEval(&_polyEval,_polyEval_s);
  unary_function_ptr at_polyEval (&__polyEval,0,true);

  gen _product(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT){
      if (g.subtype!=_SEQ__VECT)
	return prodsum(g.eval(DEFAULT_EVAL_LEVEL,contextptr),true);
      vecteur v=*g._VECTptr;
      maple_sum_product_unquote(v,contextptr);
      int s=v.size();
      adjust_int_sum_arg(v,s);
      return seqprod(gen(v,_SEQ__VECT),1,contextptr);
    }
    gen tmp=g.eval(DEFAULT_EVAL_LEVEL,contextptr);
    if (tmp.type==_VECT)
      return _product(tmp,contextptr);
    return seqprod(g,1,contextptr);
  }
  const string _product_s("product");
  unary_function_eval __product(&_product,_product_s);
  unary_function_ptr at_product (&__product,_QUOTE_ARGUMENTS,true);

  gen sortad(const vecteur & v,bool ascend){
    vecteur valeur=*eval(v)._VECTptr;
    bool ismat=ckmatrix(valeur);
    if (!ismat)
      valeur=vecteur(1,valeur);
    valeur=mtran(valeur);
    if (ascend)
      sort(valeur.begin(),valeur.end(),first_ascend_sort);
    else
      sort(valeur.begin(),valeur.end(),first_descend_sort);
    valeur=mtran(valeur);
    if (!ismat)
      return valeur.front();
    return valeur;
  }
  gen _SortA(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return sortad(*g._VECTptr,true);
    if (g.type!=_IDNT)
      setsizeerr();
    gen valeur=eval(g,contextptr);
    if (valeur.type!=_VECT)
      setsizeerr();
    return sto(_sort(valeur,contextptr),g,contextptr);
  }
  const string _SortA_s("SortA");
  unary_function_eval __SortA(&_SortA,_SortA_s);
  unary_function_ptr at_SortA (&__SortA,_QUOTE_ARGUMENTS,T_RETURN);

  gen _SortD(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return sortad(*g._VECTptr,false);
    if (g.type!=_IDNT)
      setsizeerr();
    gen valeur=eval(g,contextptr);
    if (valeur.type!=_VECT)
      setsizeerr();
    return sto(_sort(makevecteur(valeur,at_superieur_egal),contextptr),g,contextptr);
  }
  const string _SortD_s("SortD");
  unary_function_eval __SortD(&_SortD,_SortD_s);
  unary_function_ptr at_SortD (&__SortD,_QUOTE_ARGUMENTS,T_RETURN);

  const string _approx_s("approx");
  unary_function_eval __approx(&_evalf,_approx_s);
  unary_function_ptr at_approx (&__approx,0,true);

  const string _ceiling_s("ceiling");
  unary_function_eval __ceiling(&_ceil,_ceiling_s);
  unary_function_ptr at_ceiling (&__ceiling,0,true);

  const string _imag_s("imag");
  unary_function_eval unary__imag(&im,_imag_s);
  unary_function_ptr at_imag (&unary__imag,0,true);

  const string _real_s("real");
  unary_function_eval unary__real(&re,_real_s);
  unary_function_ptr at_real (&unary__real,0,true);

  const string _intDiv_s("intDiv");
  unary_function_unary __intDiv(&_iquo,_intDiv_s);
  unary_function_ptr at_intDiv (&__intDiv,0,true);

  const string _remain_s("remain");
  unary_function_eval __remain(&_irem,_remain_s);
  unary_function_ptr at_remain (&__remain,0,true);

  gen _int(const gen & g,GIAC_CONTEXT){
    if (maple_mode==3)
      return _floor(g,contextptr);
    else
      return _integrate(g,contextptr);
  }
  const string _int_s("int");
  unary_function_eval __int(&_int,_int_s);
  unary_function_ptr at_int (&__int,0,true);

  const string _isPrime_s("isPrime");
  unary_function_unary __isPrime(&_isprime,_isPrime_s);
  unary_function_ptr at_isPrime (&__isPrime,0,true);

  const string _nCr_s("nCr");
  unary_function_eval __nCr(&_comb,_nCr_s);
  unary_function_ptr at_nCr (&__nCr,0,true);

  const string _nPr_s("nPr");
  unary_function_unary __nPr(&_perm,_nPr_s);
  unary_function_ptr at_nPr (&__nPr,0,true);

  gen _iPart(const gen & g,GIAC_CONTEXT){
    return evalf(_floor(g,contextptr));
  }
  const string _iPart_s("iPart");
  unary_function_eval __iPart(&_iPart,_iPart_s);
  unary_function_ptr at_iPart (&__iPart,0,true);

  gen _Fill(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()!=2 || v[1].type!=_IDNT)
      setsizeerr();
    gen value(eval(v[1]));
    if (value.type!=_VECT)
      return sto(v[0],v[1],contextptr);
    gen dim=_dim(value);
    if (dim.type==_INT_)
      return sto(vecteur(dim.val,eval(v[0])),v[1],contextptr);
    if (dim.type==_VECT){
      vecteur & w=*dim._VECTptr;
      if (w.size()!=2 || w.front().type!=_INT_ || w.back().type!=_INT_)
	setsizeerr();
      return sto(vecteur(w[0].val,vecteur(w[1].val,eval(v[0]))),v[1],contextptr);
    }
    setsizeerr();
  }
  const string _Fill_s("Fill");
  unary_function_eval __Fill(&_Fill,_Fill_s);
  unary_function_ptr at_Fill (&__Fill,_QUOTE_ARGUMENTS,T_RETURN);

  gen _mRow(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=3 || !ckmatrix(v[1]) || v[2].type!=_INT_ )
      settypeerr();
    int s=v[1]._VECTptr->size();
    int l=v[2].val-(maple_mode!=0);
    if (l<0 || l>=s)
      setdimerr();
    vecteur w=*v[1]._VECTptr;
    w[l]=v[0]*w[l];
    return w;
  }
  const string _mRow_s("mRow");
  unary_function_unary __mRow(&_mRow,_mRow_s);
  unary_function_ptr at_mRow (&__mRow,0,true);

  gen _mRowAdd(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=4 || !ckmatrix(v[1]) || v[2].type!=_INT_ || v[3].type!=_INT_)
      settypeerr();
    int s=v[1]._VECTptr->size();
    int l1=v[2].val-(maple_mode!=0),l2=v[3].val-(maple_mode!=0);
    if (l1<0 || l1>=s || l2<0 || l2>=s)
      setdimerr();
    vecteur w=*v[1]._VECTptr;
    w[l2]=v[0]*w[l1]+w[l2];
    return w;
  }
  const string _mRowAdd_s("mRowAdd");
  unary_function_unary __mRowAdd(&_mRowAdd,_mRowAdd_s);
  unary_function_ptr at_mRowAdd (&__mRowAdd,0,true);

  gen _rowAdd(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=3 || !ckmatrix(v[0]) || v[1].type!=_INT_ || v[2].type!=_INT_)
      settypeerr();
    int s=v[0]._VECTptr->size();
    int l1=v[1].val,l2=v[2].val;
    if (l1<1 || l1>s || l2<1 || l2>s)
      setdimerr();
    vecteur w=*v[0]._VECTptr;
    w[l2-(maple_mode!=0)]=w[l1-(maple_mode!=0)]+w[l2-(maple_mode!=0)];
    return w;
  }
  const string _rowAdd_s("rowAdd");
  unary_function_unary __rowAdd(&_rowAdd,_rowAdd_s);
  unary_function_ptr at_rowAdd (&__rowAdd,0,true);

  gen _rowSwap(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=3 || !ckmatrix(v[0]) || v[1].type!=_INT_ || v[2].type!=_INT_)
      settypeerr();
    int s=v[0]._VECTptr->size();
    int l1=v[1].val,l2=v[2].val;
    if (l1<1 || l1>s || l2<1 || l2>s)
      setdimerr();
    vecteur w=*v[0]._VECTptr;
    std::swap<gen>(w[l2-(maple_mode!=0)],w[l1-(maple_mode!=0)]);
    return w;
  }
  const string _rowSwap_s("rowSwap");
  unary_function_unary __rowSwap(&_rowSwap,_rowSwap_s);
  unary_function_ptr at_rowSwap (&__rowSwap,0,true);

  gen _LU(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    gen res;
    if (v.size()==5)
      v.pop_back();
    if (v.size()!=4 || !is_squarematrix(res=eval(v[0])) || v[1].type!=_IDNT || v[2].type!=_IDNT || v[3].type!=_IDNT)
      settypeerr();
    res=lu(res); // P,L,U
    if (res.type!=_VECT || res.subtype!=_SEQ__VECT || res._VECTptr->size()!=3)
      return res;
    sto(res[1],v[1],contextptr);
    sto(res[2],v[2],contextptr);
    return sto(_permu2mat(res[0]),v[3],contextptr);    
  }
  const string _LU_s("LU");
  unary_function_eval __LU(&_LU,_LU_s);
  unary_function_ptr at_LU (&__LU,_QUOTE_ARGUMENTS,true);

  gen _QR(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()==4)
      v.pop_back();
    gen res;
    if (v.size()!=3 || !is_squarematrix(res=eval(v[0])) || v[1].type!=_IDNT || v[2].type!=_IDNT )
      settypeerr();
    gen res1=qr(res);
    if (!ckmatrix(res1))
      return res;
    sto(res*inv(res1),v[1],contextptr);
    return sto(res1,v[2],contextptr);
  }
  const string _QR_s("QR");
  unary_function_eval __QR(&_QR,_QR_s);
  unary_function_ptr at_QR (&__QR,_QUOTE_ARGUMENTS,true);

  gen _newMat(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      settypeerr();
    vecteur & v=*g._VECTptr;
    if (v[0].type!=_INT_ || v[1].type!=_INT_ )
      setsizeerr();
    int l=absint(v[0].val),c=absint(v[1].val);
    return vecteur(l,vecteur(c));
  }
  const string _newMat_s("newMat");
  unary_function_unary __newMat(&_newMat,_newMat_s);
  unary_function_ptr at_newMat (&__newMat,0,true);

  gen _ref(const gen & a) {
    if (!ckmatrix(a))
      setsizeerr();
    gen det;
    vecteur pivots;
    matrice res;
    mrref(*a._VECTptr,res,pivots,det,0,a._VECTptr->size(),0,a._VECTptr->front()._VECTptr->size(),false);
    mdividebypivot(res);
    return res;
  }
  const string _ref_s("ref");
  unary_function_unary __ref(&giac::_ref,_ref_s);
  unary_function_ptr at_ref (&__ref,0,true);

  vecteur gen2vecteur(const gen & g,int exclude){
    if (g.type!=_VECT || g.subtype==exclude)
      return vecteur(1,g);
    return *g._VECTptr;
  }
  gen _subMat(const gen & g) {
    vecteur v(gen2vecteur(g,0));
    if (v.empty() || !ckmatrix(v[0]) )
      settypeerr();
    int lignedeb=1,colonnedeb=1,lignefin=1,colonnefin=1;
    mdims(*v[0]._VECTptr,lignefin,colonnefin);
    if (v.size()>1 && v[1].type==_INT_)
      lignedeb=max(1,v[1].val+(maple_mode==0));
    if (v.size()>2 && v[2].type==_INT_)
      colonnedeb=max(1,v[2].val+(maple_mode==0));
    if (v.size()>3 && v[3].type==_INT_)
      lignefin=max(min(lignefin,v[3].val+(maple_mode==0)),lignedeb);
    if (v.size()>4 && v[4].type==_INT_)
      colonnefin=max(colonnedeb,min(colonnefin,v[4].val+(maple_mode==0)));
    return matrice_extract(*v[0]._VECTptr,lignedeb-1,colonnedeb-1,lignefin-lignedeb+1,colonnefin-colonnedeb+1);
  }
  const string _subMat_s("subMat");
  unary_function_unary __subMat(&giac::_subMat,_subMat_s);
  unary_function_ptr at_subMat (&__subMat,0,true);

  gen _unitV(const gen & g,GIAC_CONTEXT) {
    return rdiv(g,_l2norm(g,contextptr));
  }
  const string _unitV_s("unitV");
  unary_function_eval __unitV(&giac::_unitV,_unitV_s);
  unary_function_ptr at_unitV (&__unitV,0,true);

  gen L1norm(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return abs(g,contextptr);
    vecteur & v=*g._VECTptr;
    gen res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it)
      res=res+abs(*it,contextptr);
    return res;
  }

  gen _rowNorm(const gen & g,GIAC_CONTEXT) {
    if (!ckmatrix(g))
      settypeerr();
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    gen res;
    for (;it!=itend;++it){
      res=max(res,L1norm(*it,contextptr));
    }
    return res;
  }
  const string _rowNorm_s("rowNorm");
  unary_function_eval __rowNorm(&giac::_rowNorm,_rowNorm_s);
  unary_function_ptr at_rowNorm (&__rowNorm,0,true);

  const string _rownorm_s("rownorm");
  unary_function_eval __rownorm(&giac::_rowNorm,_rownorm_s);
  unary_function_ptr at_rownorm (&__rownorm,0,true);

  gen _colNorm(const gen & g,GIAC_CONTEXT) {
    if (!ckmatrix(g))
      settypeerr();
    return _rowNorm(mtran(*g._VECTptr),contextptr);
  }
  const string _colNorm_s("colNorm");
  unary_function_eval __colNorm(&giac::_colNorm,_colNorm_s);
  unary_function_ptr at_colNorm (&__colNorm,0,true);

  const string _colnorm_s("colnorm");
  unary_function_eval __colnorm(&giac::_colNorm,_colnorm_s);
  unary_function_ptr at_colnorm (&__colnorm,0,true);

  // FIXME SECURITY
  // archive is made of couples name/value
  sym_tab read_ti_archive(const string & s){
    ifstream inf(s.c_str());
    vecteur v,l;
    readargs_from_stream(inf,v,l);
    sym_tab res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type!=_STRNG)
	continue;
      string name=*it->_STRNGptr;
      ++it;
      if (it==itend)
	break;
      gen value=*it;
      res[name]=value;
    }
    return res;
  }
  
  void print_ti_archive(const string & s,const sym_tab & m){
    check_secure();
    ofstream of(s.c_str());
    sym_tab::const_iterator it=m.begin(),itend=m.end();
    if (it==itend)
      of << "[ ]" << endl;
    of << "[" << string2gen(it->first,false) ;
    of << "," << it->second ;
    ++it;
    for (;it!=itend;++it){
      of << "," << endl ;
      of << string2gen(it->first,false) ;
      of << "," << it->second ;
    }
    of << "]" << endl;
  }
  gen _Archive(const gen & g){
    sym_tab arc(read_ti_archive("archive"));
    if (g.type==_IDNT)
      arc[g.print()]=eval(g);
    else {
      if (g.type!=_VECT)
	setsizeerr();
      const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
      for (;it!=itend;++it){
	if (it->type==_IDNT)
	  arc[it->print()]=eval(*it);
      }
    }
    print_ti_archive("archive",arc);
    return 1;
  }
  const string _Archive_s("Archive");
  unary_function_unary __Archive(&_Archive,_Archive_s,&printastifunction);
  unary_function_ptr at_Archive (&__Archive,_QUOTE_ARGUMENTS,T_RETURN);

  gen _Unarchiv(const gen & g,GIAC_CONTEXT){
    sym_tab arc(read_ti_archive("archive"));
    if (g.type==_IDNT)
      return sto(arc[g.print()],g,contextptr);
    else {
      if (g.type!=_VECT)
	setsizeerr();
      const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
      for (;it!=itend;++it){
	if (it->type==_IDNT)
	  sto(arc[it->print()],*it,contextptr);
      }
    }
    return 1;
  }
  const string _Unarchiv_s("Unarchiv");
  unary_function_eval __Unarchiv(&_Unarchiv,_Unarchiv_s,&printastifunction);
  unary_function_ptr at_Unarchiv (&__Unarchiv,_QUOTE_ARGUMENTS,T_RETURN);

  gen _ClrIO(const gen & g){
    return __interactive.op(symbolic(at_ClrIO,0));
  }
  const string _ClrIO_s("ClrIO");
  unary_function_unary __ClrIO(&_ClrIO,_ClrIO_s,&printastifunction);
  unary_function_ptr at_ClrIO (&__ClrIO,0,T_EXPRESSION);

  gen _CopyVar(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()!=2 || v[0].type!=_IDNT || v[1].type!=_IDNT )
      settypeerr();
    return sto(v[0].eval(2),v[1],contextptr);
  }
  const string _CopyVar_s("CopyVar");
  unary_function_eval __CopyVar(&_CopyVar,_CopyVar_s,&printastifunction);
  unary_function_ptr at_CopyVar (&__CopyVar,_QUOTE_ARGUMENTS,T_RETURN);

  gen _Output(const gen & g){
    vecteur v(gen2vecteur(g));
    if (v.size()!=3 || v[0].type!=_INT_ || v[1].type!=_INT_ )
      settypeerr();
    return __interactive.op(g);
  }
  const string _Output_s("Output");
  unary_function_unary __Output(&_Output,_Output_s,&printastifunction);
  unary_function_ptr at_Output (&__Output,0,T_RETURN);

  gen _getKey(const gen & g){
    char ch;
    cerr << "Waiting for a keystroke in konsole screen" << endl;
    cin >> ch;
    return int(ch);
  }
  const string _getKey_s("getKey");
  unary_function_unary __getKey(&_getKey,_getKey_s);
  unary_function_ptr at_getKey (&__getKey,0,true);

  gen _DelFold(const gen & g,GIAC_CONTEXT){
    gen res=_SetFold(0);
    try {
      _purge(g,contextptr);
    } catch(std::runtime_error & e){
      _SetFold(res);
      throw(e);
    }
    return res;
  }
  const string _DelFold_s("DelFold");
  unary_function_eval __DelFold(&_DelFold,_DelFold_s,&printastifunction);
  unary_function_ptr at_DelFold (&__DelFold,_QUOTE_ARGUMENTS,T_RETURN);

  gen _DispG(const gen & g){
    return __interactive.op(symbolic(at_DispG,0));
  }
  const string _DispG_s("DispG");
  unary_function_unary __DispG(&_DispG,_DispG_s,&printastifunction);
  unary_function_ptr at_DispG (&__DispG,0,T_EXPRESSION);

  gen _DispHome(const gen & g){
    return __interactive.op(symbolic(at_DispHome,0));
  }
  const string _DispHome_s("DispHome");
  unary_function_unary __DispHome(&_DispHome,_DispHome_s,&printastifunction);
  unary_function_ptr at_DispHome (&__DispHome,0,T_EXPRESSION);

  const string _entry_s("entry");
  unary_function_unary __entry(&_quest,_entry_s);
  unary_function_ptr at_entry (&__entry,0,true);

  gen _Exec(const gen & g){
    return string2gen("TI instruction not supported",false);
  }
  const string _Exec_s("Exec");
  unary_function_unary __Exec(&_Exec,_Exec_s,&printastifunction);
  unary_function_ptr at_Exec (&__Exec,0,T_RETURN);

  const string _Get_s("Get");
  unary_function_unary __Get(&_Exec,_Get_s,&printastifunction);
  unary_function_ptr at_Get (&__Get,_QUOTE_ARGUMENTS,T_RETURN);

  const string _GetCalc_s("GetCalc");
  unary_function_unary __GetCalc(&_Exec,_GetCalc_s,&printastifunction);
  unary_function_ptr at_GetCalc (&__GetCalc,_QUOTE_ARGUMENTS,T_RETURN);

  gen _NewFold(const gen & g,GIAC_CONTEXT){ 
    if (g.type!=_IDNT)
      setsizeerr();
    sto(gen(vecteur(1,vecteur(0)),_FOLDER__VECT),g,contextptr);
    return _SetFold(g);
  }
  const string _NewFold_s("NewFold");
  unary_function_eval __NewFold(&_NewFold,_NewFold_s,&printastifunction);
  unary_function_ptr at_NewFold (&__NewFold,_QUOTE_ARGUMENTS,T_RETURN);

  gen _GetFold(const gen & g){
    return getfold(current_folder_name);
  }
  const string _GetFold_s("GetFold");
  unary_function_unary __GetFold(&_GetFold,_GetFold_s,&printastifunction);
  unary_function_ptr at_GetFold (&__GetFold,0,T_EXPRESSION);

  gen _StoPic(const gen & g,GIAC_CONTEXT){
    if (g.type!=_IDNT)
      setsizeerr();
    return sto(__interactive.op(0),g,contextptr);
  }
  const string _StoPic_s("StoPic");
  unary_function_eval __StoPic(&_StoPic,_StoPic_s,&printastifunction);
  unary_function_ptr at_StoPic (&__StoPic,_QUOTE_ARGUMENTS,T_RETURN);

  gen _RclPic(const gen & g){
    if (g.type!=_IDNT)
      setsizeerr();
    gen tmp=eval(g);
    if (tmp.type!=_VECT)
      setsizeerr();
    return __interactive.op(symbolic(at_RclPic,tmp));
  }
  const string _RclPic_s("RclPic");
  unary_function_unary __RclPic(&_RclPic,_RclPic_s,&printastifunction);
  unary_function_ptr at_RclPic (&__RclPic,_QUOTE_ARGUMENTS,T_RETURN);

  gen _RplcPic(const gen & g){
    if (g.type!=_IDNT)
      setsizeerr();
    gen tmp=eval(g);
    if (tmp.type!=_VECT)
      setsizeerr();
    return __interactive.op(symbolic(at_RplcPic,tmp));
  }
  const string _RplcPic_s("RplcPic");
  unary_function_unary __RplcPic(&_RplcPic,_RplcPic_s,&printastifunction);
  unary_function_ptr at_RplcPic (&__RplcPic,_QUOTE_ARGUMENTS,T_RETURN);

  gen _ClrGraph(const gen & g){
    return __interactive.op(symbolic(at_erase,0));
  }
  const string _ClrGraph_s("ClrGraph");
  unary_function_unary __ClrGraph(&_ClrGraph,_ClrGraph_s,&printastifunction);
  unary_function_ptr at_ClrGraph (&__ClrGraph,0,T_EXPRESSION);

  gen _ClrDraw(const gen & g){
    return __interactive.op(symbolic(at_erase,0));
  }
  const string _ClrDraw_s("ClrDraw");
  unary_function_unary __ClrDraw(&_ClrDraw,_ClrDraw_s,&printastifunction);
  unary_function_ptr at_ClrDraw (&__ClrDraw,0,T_EXPRESSION);

  gen _PtOn(const gen & g,GIAC_CONTEXT){
    return _point(g,contextptr);
  }
  const string _PtOn_s("PtOn");
  unary_function_eval __PtOn(&_PtOn,_PtOn_s,&printastifunction);
  unary_function_ptr at_PtOn (&__PtOn,0,T_RETURN);

  gen _PtOff(const gen & g,GIAC_CONTEXT){
    gen tmp=_point(g,contextptr);
    if (tmp.type==_SYMB && tmp._SYMBptr->sommet==at_pnt)
      return symb_pnt(tmp[0],FL_WHITE,contextptr);
    else
      return tmp;
  }
  const string _PtOff_s("PtOff");
  unary_function_eval __PtOff(&_PtOff,_PtOff_s,&printastifunction);
  unary_function_ptr at_PtOff (&__PtOff,0,T_RETURN);

  const string _PxlOn_s("PxlOn");
  unary_function_eval __PxlOn(&_pixon,_PxlOn_s,&printastifunction);
  unary_function_ptr at_PxlOn (&__PxlOn,0,T_RETURN);

  const string _PxlOff_s("PxlOff");
  unary_function_eval __PxlOff(&_pixoff,_PxlOff_s,&printastifunction);
  unary_function_ptr at_PxlOff (&__PxlOff,0,T_RETURN);

  gen _Line(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<4)
      setsizeerr();
    int couleur=FL_BLACK;
    if (v.size()==5 && v[4].val==0)
      couleur=FL_WHITE;
    return _couleur(makevecteur(_segment(makevecteur(v[0]+cst_i*v[1],v[2]+cst_i*v[3]),contextptr),couleur),contextptr);
  }
  const string _Line_s("Line");
  unary_function_eval __Line(&_Line,_Line_s,&printastifunction);
  unary_function_ptr at_Line (&__Line,0,T_RETURN);

  gen _LineHorz(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<1)
      setsizeerr();
    int couleur=FL_BLACK;
    if (v.size()==2 && v[1].val==0)
      couleur=FL_WHITE;
    return _couleur(makevecteur(_droite(makevecteur(cst_i*v[0],1+cst_i*v[0]),contextptr),couleur),contextptr);
  }
  const string _LineHorz_s("LineHorz");
  unary_function_eval __LineHorz(&_LineHorz,_LineHorz_s,&printastifunction);
  unary_function_ptr at_LineHorz (&__LineHorz,0,T_RETURN);

  gen _LineVert(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<1)
      setsizeerr();
    int couleur=FL_BLACK;
    if (v.size()==2 && v[1].val==0)
      couleur=FL_WHITE;
    return _couleur(makevecteur(_droite(makevecteur(v[0],cst_i+v[0]),contextptr),couleur),contextptr);
  }
  const string _LineVert_s("LineVert");
  unary_function_eval __LineVert(&_LineVert,_LineVert_s,&printastifunction);
  unary_function_ptr at_LineVert (&__LineVert,0,T_RETURN);

  gen _DrawSlp(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<3)
      setsizeerr();
    gen pt(v[0]+cst_i*v[1]);
    return _droite(makevecteur(pt,pt+1+cst_i*v[2]),contextptr);
  }
  const string _DrawSlp_s("DrawSlp");
  unary_function_eval __DrawSlp(&_DrawSlp,_DrawSlp_s,&printastifunction);
  unary_function_ptr at_DrawSlp (&__DrawSlp,0,T_RETURN);

  gen _Circle(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<3)
      setsizeerr();
    int couleur=FL_BLACK;
    if (v.size()==4 && v[3].val==0)
      couleur=FL_WHITE;
    gen centre(v[0]+cst_i*v[1]);
    return _couleur(makevecteur(_cercle(makevecteur(centre,v[2]),contextptr),couleur),contextptr);
  }
  const string _Circle_s("Circle");
  unary_function_eval __Circle(&_Circle,_Circle_s,&printastifunction);
  unary_function_ptr at_Circle (&__Circle,0,T_RETURN);

  gen _PtText(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<3)
      setsizeerr();
    gen tmp(v[1]+cst_i*v[2]);
    return _legende(makevecteur(tmp,v[0]),contextptr);
  }
  const string _PtText_s("PtText");
  unary_function_eval __PtText(&_PtText,_PtText_s,&printastifunction);
  unary_function_ptr at_PtText (&__PtText,0,T_RETURN);

  gen _NewPic(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<2|| !ckmatrix(v[0]) || v[1].type!=_IDNT)
      setsizeerr();
    vecteur w=*v[0]._VECTptr;
    iterateur it=w.begin(),itend=w.end();
    for (;it!=itend;++it){
      if (it->_VECTptr->size()!=2)
	setsizeerr();
      *it=symbolic(at_pnt,makevecteur(it->_VECTptr->front()+cst_i*it->_VECTptr->back(),FL_BLACK),_PNT__VECT);
    }
    return sto(gen(w,_SEQ__VECT),v[1],contextptr);
  }
  const string _NewPic_s("NewPic");
  unary_function_eval __NewPic(&_NewPic,_NewPic_s,&printastifunction);
  unary_function_ptr at_NewPic (&__NewPic,0,T_RETURN);

  vecteur zoom_save;
  gen _ZoomSto(const gen & g){
    vecteur v;
    v.push_back(gnuplot_xmin);
    v.push_back(gnuplot_xmax);
    v.push_back(gnuplot_ymin);
    v.push_back(gnuplot_ymax);
    v.push_back(gnuplot_zmin);
    v.push_back(gnuplot_zmax);
    v.push_back(gnuplot_tmin);
    v.push_back(gnuplot_tmax);
    v.push_back(global_window_xmin);
    v.push_back(global_window_xmax);
    v.push_back(global_window_ymin);
    v.push_back(global_window_ymax);
    v.push_back(show_axes);
    zoom_save=v;
    return v;
  }
  const string _ZoomSto_s("ZoomSto");
  unary_function_unary __ZoomSto(&_ZoomSto,_ZoomSto_s,&printastifunction);
  unary_function_ptr at_ZoomSto (&__ZoomSto,0,T_RETURN);

  gen _ZoomRcl(const gen & g){
    vecteur v;
    if (g.type!=_VECT || g._VECTptr->size()<13)
      v=zoom_save;
    else
      v=*g._VECTptr;
    return _xyztrange(v);
  }
  const string _ZoomRcl_s("ZoomRcl");
  unary_function_unary __ZoomRcl(&_ZoomRcl,_ZoomRcl_s,&printastifunction);
  unary_function_ptr at_ZoomRcl (&__ZoomRcl,0,T_RETURN);

  gen _deSolve(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.empty())
      setsizeerr();
    if (v[0].is_symb_of_sommet(at_and))
      v[0]=remove_and(v[0],at_and);
    return _desolve(gen(v,_SEQ__VECT),contextptr);
  }
  const string _deSolve_s("deSolve");
  unary_function_eval __deSolve(&_deSolve,_deSolve_s,&printastifunction);
  unary_function_ptr at_deSolve (&__deSolve,_QUOTE_ARGUMENTS,T_RETURN);

  gen _LineTan(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()!=2)
      setsizeerr();
    gen f(v[0]);
    gen x0(v[1]);
    gen fprime(derive(f,vx_var));
    gen M0(x0+cst_i*subst(f,vx_var,x0));
    gen direction(1+cst_i*subst(fprime,vx_var,x0));
    return _droite(makevecteur(M0,M0+direction),contextptr);
  }
  const string _LineTan_s("LineTan");
  unary_function_eval __LineTan(&_LineTan,_LineTan_s,&printastifunction);
  unary_function_ptr at_LineTan (&__LineTan,_QUOTE_ARGUMENTS,T_RETURN);

  gen _CyclePic(const gen & g){
    vecteur v(gen2vecteur(g));
    int s=v.size();
    if (s<2 || v[0].type!=_STRNG || v[1].type!=_INT_)
      setsizeerr();
    int n=max(absint(v[1].val),1);
    double delay=1.0;
    if (s>2 && v[2].type==_DOUBLE_)
      delay=v[2]._DOUBLE_val;
    delay=std::abs(delay*1e3);
    int d=int(delay);
    int ds=d/1000,ns=(d%1000)*1000000;
    timespec t,tr;
    t.tv_sec=ds;
    t.tv_nsec=ns;
    int repete=1;
    if (s>3 && v[3].type==_INT_)
      repete=max(absint(v[3].val),1);
    int direction=1;
    if (s>4 && is_minus_one(v[4]))
      direction=-1;
    string & orig_name = *v[0]._STRNGptr;
    for (int i=0;i<repete;++i){
      if (direction==1){
	for (int j=1;j<=n;++j){
	  string name=orig_name+print_INT_(j);
	  gen g(name);
	  if (g.type==_IDNT)
	    _RplcPic(g);
	  usleep(d*1000);
	  /*
#ifdef WIN32
	  sleep(ds);
#else
#ifdef __APPLE__
  usleep(2000);
#else
	  nanosleep(&t,&tr);
#endif
#endif
	  */
	}
      }
      else {
	for (int j=n;j>=1;--j){
	  string name=orig_name+print_INT_(j);
	  gen g(name);
	  if (g.type==_IDNT)
	    _RplcPic(g);
	  usleep(d*1000);
	  /*
#ifdef WIN32
	  sleep(ds);
#else
#ifdef __APPLE__
  usleep(2000);
#else
	  nanosleep(&t,&tr);
#endif
#endif
	  */
	}
      }
    }
    return zero;
  }
  const string _CyclePic_s("CyclePic");
  unary_function_unary __CyclePic(&_CyclePic,_CyclePic_s,&printastifunction);
  unary_function_ptr at_CyclePic (&__CyclePic,0,T_RETURN);

  gen _RandSeed(const gen & g){
#ifndef GNUWINCE
    srandom(g.val);
#endif
    return g;
  }
  const string _RandSeed_s("RandSeed");
  unary_function_unary __RandSeed(&_RandSeed,_RandSeed_s,&printastifunction);
  unary_function_ptr at_RandSeed (&__RandSeed,0,T_RETURN);

  gen _Store(const gen & g,const context * contextptr){
    return _sto(g,contextptr);
  }
  const string _Store_s("Store");
  unary_function_eval __Store(&_Store,_Store_s);
  unary_function_ptr at_Store (&__Store,_QUOTE_ARGUMENTS,true);

  gen exact_double(double d,double eps){
    if (d<0)
      return -exact_double(-d,eps);
    vector<int> res;
    for (;;){
      res.push_back(int(d+eps));
      d=d-int(d+eps);
      if (d<eps)
	break;
      d=1/d;
      eps=eps*d;
    }
    reverse(res.begin(),res.end());
    vector<int>::const_iterator it=res.begin(),itend=res.end();
    gen x(*it);
    for (++it;it!=itend;++it){
      x=*it+inv(x);
    }
    return x;
  }
  gen exact(const gen & g,GIAC_CONTEXT){
    switch (g.type){
    case _DOUBLE_:
      return exact_double(g._DOUBLE_val,epsilon(contextptr));
    case _CPLX:
      return exact(re(g,contextptr),contextptr)+cst_i*exact(im(g,contextptr),contextptr);
    case _SYMB:
      return symbolic(g._SYMBptr->sommet,exact(g._SYMBptr->feuille,contextptr));
    case _VECT:
      return apply(g,exact,contextptr);
    default:
      return g;
    }
  }
  const string _exact_s("exact");
  unary_function_eval __exact(&exact,_exact_s);
  unary_function_ptr at_exact (&__exact,0,true);

  gen fPart(const gen & g,GIAC_CONTEXT){
    return g-_iPart(g,contextptr);
  }
  const string _fPart_s("fPart");
  unary_function_eval __fPart(&fPart,_fPart_s);
  unary_function_ptr at_fPart (&__fPart,0,true);

  const string _frac_s("frac");
  unary_function_eval __frac(&fPart,_frac_s);
  unary_function_ptr at_frac (&__frac,0,true);

  gen simult(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    gen m1(v[0]),m2(v[1]);
    if (!is_squarematrix(m1)|| !ckmatrix(m2) )
      setsizeerr();
    matrice m=mtran(mergevecteur(mtran(*m1._VECTptr),mtran(*m2._VECTptr)));
    m=mrref(m);
    mdividebypivot(m);
    int n,c;
    mdims(m,n,c);
    return matrice_extract(m,0,n,n,c-n);
  }
  const string _simult_s("simult");
  unary_function_unary __simult(&simult,_simult_s);
  unary_function_ptr at_simult (&__simult,0,true);

  /* TI89 compatibility Notes
     Use maple_mode(3) in your config file (~/.xcasrc or xcas.rc) or
     begin your session with cas configuration (or type maple_mode(3))

     1/ Instructions implemented
     Algebra: all
     Calculus: all except product
     Strings: all
     Graphics: pixel instructions are not implemented except PxlOn/PxlOff
       And/Xor/Pixel tests are not implemented
       Zoom instructions not implemented except ZoomSto/ZoomRcl
       BldData/RclGDB/StoGDB not implemented
       Graph currently points to DrawFunc
     Lists: all
     Maths: all except conversions and units
     Matrices: all except statistics. 
       LU/QR work with numeric matrices and tolerance argument is ignored
     Program: all except 
       mode handling, units, 
       getKeys, the Custom and Toolbar instructions
       Lock/Unlock
     Stats: implemented !, rand, RandSeed, nCr, nPr, median, mean, stddev

     2/ Be sure to use correct case. archive (native) is not like Archive (TI)
     Conversion of special char (ASCII code >128)
     conversion sign translated to to
     Store sign: =>
     Different: != or <>, Greater or equal: >=, Lower or equal: <=
     i=sqrt(-1): use the i button of xcas
     transpose sign not translated (use transpose())
   */

  gen ti_decode_unsigned(octet * & ptr){
    short l=(*ptr);
    --ptr;
    // Horner
    gen res;
    for (short i=0;i<l;++i,--ptr)
      res=256*res+int(*ptr);
    return res;
  }

  double ti_decode_double(octet * & ptr){
    ptr -= 8;
    octet c1=*ptr,c2=*(ptr+1);
    unsigned int c=c1*256+c2;
    int mantissa=c-4*16*16*16-13;
    octet * q=ptr+2;
    --ptr;
    double d=0;
    for (int i=0;i<7;++i,++q){
      // now read quartet by quartet in BCD
      c1=(*q) & 0xF0;
      c1=c1 >> 4;
      d=d*10+c1;
      c2=(*q) & 0x0F;
      d=d*10+c2;      
    }
    // multiply by 10^-mantissa
    d=d*std::pow(10.0,double(mantissa));
    return d;
  }

  gen ti_decode_fraction(octet * & ptr){
    --ptr;
    gen num=ti_decode_unsigned(ptr);
    gen den=ti_decode_unsigned(ptr);
    return fraction(num,den);
  }

  gen make_symbol(const string & s){
    gen res;
    if (find_or_make_symbol(s,res)==T_SYMBOL)
      return res;
    else
      return make_symbol("_"+s);
  }

  gen ti_decode_identificateur(octet * &ptr){
    string s;
    octet c;
    for (;(c=*ptr)!=VAR_NAME_TAG;--ptr){
      if (c>0x7f){
	s="Z"+print_INT_(-char(c))+s;
      }
      else
	s = (char) c+s;
    }
    --ptr;
    int pos=s.find('\\');
    int l=s.size();
    gen res;
    if (pos>0 && pos+1<l){
      string s1=s.substr(0,pos);
      string s2=s.substr(pos+1,l-pos-1);
      res=symb_double_deux_points(makevecteur(make_symbol(s1),make_symbol(s2)));
    }
    else
      res=make_symbol(s);
    return res;
  }

  // Also used for variable arguments functions
  vecteur ti_decode_list(octet * & ptr,octet end){
    vecteur res;
    for (;(*ptr)!=end;)
      res.push_back(ti_decode_tag(ptr));
    --ptr;
    return res;
  }

  // true if it's a newline or nextexp
  bool ti_decode_newline(octet * & ptr){
    if (*ptr==NEWLINE_TAG || *ptr==NEXTEXPR_TAG){
      --ptr;
      if (!*ptr)
	--ptr;
      ti_decode_newline(ptr);
      return true;
    }
    return false;
  }
  vecteur ti_decode_list(octet * & ptr,octet end1,octet end2){
    vecteur res;
    for (;;){
      ti_decode_newline(ptr);
      if (*(ptr-1)==end1 && *ptr==end2)
	break;
      res.push_back(ti_decode_tag(ptr));
    }
    ptr -=2 ;
    return res;
  }

  string ti_decode_string(octet * & ptr){
    string s;
    char c;
    for (;c=(*ptr);--ptr){
      switch (c){
      case '\\':
	s="::"+s;
      default:
	s=c+s;
      }
    }
    --ptr;
    return s;
  }

  gen ti_decode_function(octet * & ptr){
    octet flag1=*ptr;
    if (!(flag1 &0x08)){
      ptr -=3;
      vecteur param;
      // decode parameter list
      for (;*ptr!=END_TAG;){ // END_TAG==0xE5
	param.push_back(ti_decode_tag(ptr));
      }
      --ptr;
      // now either it's a function like x->sin(x)
      // or a bloc function/program with local var
      // in the second case we will have FUNC_ITAG 0x17 or PRGM_ITAG 0x19 0xE4
      // if local var are there we will in addition have LOCAL_ITAG 0xE4 etc.
      vecteur prog;
      for (;*ptr!=END_OF_SEGMENT;)
	prog.push_back(ti_decode_tag(ptr));
      --ptr;
      if (prog.size()==1)
	return symb_program(gen(param,_SEQ__VECT),param*zero,prog.front());
      else
	return symb_program(gen(param,_SEQ__VECT),param*zero,prog);
    }
    setsizeerr("Can't find length of non tokenized programs");
  }
  gen ti_decode_sysvar_tag(octet * & ptr){
    gen res;
    switch (*ptr){
    case X_BAR_TAG:
      res=make_symbol("xmean");
    break; case Y_BAR_TAG:
      res=make_symbol("ymean");
    break; case SIGMA_X_TAG:
      res=make_symbol("Sx");
    break; case SIGMA_X2_TAG:
      res=make_symbol("Sx2");
    break; case SIGMA_Y_TAG:
      res=make_symbol("Sy");
    break; case SIGMA_Y2_TAG:
      res=make_symbol("Sy2");
    break; case SIGMA_XY_TAG:
      res=make_symbol("Sxy");
    break; case SX_TAG:
      res=make_symbol("Sx");
    break; case SY_TAG:
      res=make_symbol("Sy");
    break; case SMLSIGMA_X_TAG:
      res=make_symbol("sx");
    break; case SMLSIGMA_Y_TAG:
      res=make_symbol("sy");
    break; case NSTAT_TAG:
      res=make_symbol("nStat");
    break; case MINX_TAG:
      res=make_symbol("minX");
    break; case MINY_TAG:
      res=make_symbol("minY");
    break; case Q1_TAG:
      res=make_symbol("q1");
    break; case MEDSTAT_TAG:
      res=make_symbol("medStat");
    break; case Q3_TAG:
      res=make_symbol("q3");
    break; case MAXX_TAG:
      res=make_symbol("maxX");
    break; case MAXY_TAG:
      res=make_symbol("maxY");
    break; case CORR_TAG:
      res=make_symbol("corr");
    break; case R2_TAG:
      res=make_symbol("R2");
    break; case MEDX1_TAG:
      res=make_symbol("mdex1");
    break; case MEDX2_TAG:
      res=make_symbol("medx2");
    break; case MEDX3_TAG:
      res=make_symbol("medx3");
    break; case MEDY1_TAG:
      res=make_symbol("medy1");
    break; case MEDY2_TAG:
      res=make_symbol("medy2");
    break; case MEDY3_TAG:
      res=make_symbol("medy3");
    break; case XC_TAG:
      res=make_symbol("xc");
    break; case YC_TAG:
      res=make_symbol("yc");
    break; case ZC_TAG:
      res=make_symbol("zc");
    break; case TC_TAG:
      res=make_symbol("tc");
    break; case RC_TAG:
      res=make_symbol("rc");
    break; case THETA_C_TAG:
      res=make_symbol("qc");
    break; case NC_TAG:
      res=make_symbol("nc");
    break; case XFACT_TAG:
      res=make_symbol("xfact");
    break; case YFACT_TAG:
      res=make_symbol("yfact");
    break; case ZFACT_TAG:
      res=make_symbol("zfact");
    break; case XMIN_TAG:
      res=make_symbol("xmin");
    break; case XMAX_TAG:
      res=make_symbol("xmax");
    break; case XSCL_TAG:
      res=make_symbol("xscl");
    break; case YMIN_TAG:
      res=make_symbol("ymin");
    break; case YMAX_TAG:
      res=make_symbol("ymax");
    break; case YSCL_TAG:
      res=make_symbol("yscl");
    break; case DELTA_X_TAG:
      res=make_symbol("Dx");
    break; case DELTA_Y_TAG:
      res=make_symbol("Dy");
    break; case XRES_TAG:
      res=make_symbol("xres");
    break; case XGRID_TAG:
      res=make_symbol("xgrid");
    break; case YGRID_TAG:
      res=make_symbol("ygrid");
    break; case ZMIN_TAG:
      res=make_symbol("zmin");
    break; case ZMAX_TAG:
      res=make_symbol("zmax");
    break; case ZSCL_TAG:
      res=make_symbol("zscl");
    break; case EYE_THETA_TAG:
      res=make_symbol("eyeq");
    break; case EYE_PHI_TAG:
      res=make_symbol("eyeF");
    break; case THETA_MIN_TAG:
      res=make_symbol("qmin");
    break; case THETA_MAX_TAG:
      res=make_symbol("qmax");
    break; case THETA_STEP_TAG:
      res=make_symbol("qstep");
    break; case TMIN_TAG:
      res=make_symbol("tmin");
    break; case TMAX_TAG:
      res=make_symbol("tmax");
    break; case TSTEP_TAG:
      res=make_symbol("tstep");
    break; case NMIN_TAG:
      res=make_symbol("nmin");
    break; case NMAX_TAG:
      res=make_symbol("nmax");
    break; case PLOTSTRT_TAG:
      res=make_symbol("plotStrt");
    break; case PLOTSTEP_TAG:
      res=make_symbol("plotStep");
    break; case ZXMIN_TAG:
      res=make_symbol("zxmin");
    break; case ZXMAX_TAG:
      res=make_symbol("zxmax");
    break; case ZXSCL_TAG:
      res=make_symbol("zxscl");
    break; case ZYMIN_TAG:
      res=make_symbol("zymin");
    break; case ZYMAX_TAG:
      res=make_symbol("zymax");
    break; case ZYSCL_TAG:
      res=make_symbol("zyscl");
    break; case ZXRES_TAG:
      res=make_symbol("zxres");
    break; case Z_THETA_MIN_TAG:
      res=make_symbol("zqmin");
    break; case Z_THETA_MAX_TAG:
      res=make_symbol("zqmax");
    break; case Z_THETA_STEP_TAG:
      res=make_symbol("zqstep");
    break; case ZTMIN_TAG:
      res=make_symbol("ztmin");
    break; case ZTMAX_TAG:
      res=make_symbol("ztmax");
    break; case ZTSTEP_TAG:
      res=make_symbol("ztsep");
    break; case ZXGRID_TAG:
      res=make_symbol("zxgrid");
    break; case ZYGRID_TAG:
      res=make_symbol("zygrid");
    break; case ZZMIN_TAG:
      res=make_symbol("zzmin");
    break; case ZZMAX_TAG:
      res=make_symbol("zzmax");
    break; case ZZSCL_TAG:
      res=make_symbol("zzscl");
    break; case ZEYE_THETA_TAG:
      res=make_symbol("zeyeq");
    break; case ZEYE_PHI_TAG:
      res=make_symbol("zeyeF");
    break; case ZNMIN_TAG:
      res=make_symbol("znmin");
    break; case ZNMAX_TAG:
      res=make_symbol("znmax");
    break; case ZPLTSTEP_TAG:
      res=make_symbol("zpltstep");
    break; case ZPLTSTRT_TAG:
      res=make_symbol("zpltstrt");
    break; case SEED1_TAG:
      res=make_symbol("seed1");
    break; case SEED2_TAG:
      res=make_symbol("seed2");
    break; case OK_TAG:
      res=make_symbol("ok");
    break; case ERRORNUM_TAG:
      res=make_symbol("errornum");
    break; case SYSMATH_TAG:
      res=make_symbol("sysMath");
    break; case SYSDATA_TAG:
      res=make_symbol("sysData");
    break; case REGEQ_TAG:
      res=make_symbol("regEq");
    break; case REGCOEF_TAG:
      res=make_symbol("regCoef");
    break; case TBLINPUT_TAG:
      res=make_symbol("tblInput");
    break; case TBLSTART_TAG:
      res=make_symbol("tblStart");
    break; case DELTA_TBL_TAG:
      res=make_symbol("Dtbl");
    break; case FLDPIC_TAG:
      res=make_symbol("fldpic");
    break; case EYE_PSI_TAG:
      res=make_symbol("eyeY");
    break; case TPLOT_TAG:
      res=make_symbol("tplot");
    break; case DIFTOL_TAG:
      res=make_symbol("diftol");
    break; case ZEYE_PSI_TAG:
      res=make_symbol("zeyeY");
    break; case T0_TAG:
      res=make_symbol("t0");
    break; case DTIME_TAG:
      res=make_symbol("dtime");
    break; case NCURVES_TAG:
      res=make_symbol("ncurves");
    break; case FLDRES_TAG:
      res=make_symbol("fldres");
    break; case ESTEP_TAG:
      res=make_symbol("Estep");
    break; case ZT0DE_TAG:
      res=make_symbol("zt0de");
    break; case ZTMAXDE_TAG:
      res=make_symbol("ztmaxde");
    break; case ZTSTEPDE_TAG:
      res=make_symbol("ztstepde");
    break; case ZTPLOTDE_TAG:
      res=make_symbol("ztplotde");
    break; case NCONTOUR_TAG:
      res=make_symbol("ncontout");
    default:
      setsizeerr("Unknown sysvar tag"+print_INT_(*ptr));
    }
    --ptr;
    return res;
  }

  gen ti_decode_unary(octet * & ptr,unary_function_ptr & u){
    --ptr;
    gen g=ti_decode_tag(ptr);
    return symbolic(u,g);
  }

  gen ti_decode_binary(octet * & ptr,unary_function_ptr & u,bool normal_order=true){
    --ptr;
    gen g1=ti_decode_tag(ptr);
    gen g2=ti_decode_tag(ptr);
    if (normal_order)
      return symbolic(u,makevecteur(g1,g2));
    else
      return symbolic(u,makevecteur(g2,g1));
  }

  gen ti_decode_nary(octet * & ptr,unary_function_ptr & u,int n){
    --ptr;
    vecteur arg;
    for (;*ptr!=END_TAG;)
      arg.push_back(ti_decode_tag(ptr));
    --ptr;
    return symbolic(u,gen(arg,_SEQ__VECT));
  }

  gen ti_decode_arb_real(octet * & ptr,bool real){
    unsigned int i=*ptr;
    --ptr;
    string s;
    gen res;
    if (real)
      s="at_";
    else
      s="at_n";
    s += print_INT_(i);
    find_or_make_symbol(s,res);
    return res;
  }

  gen ti_decode_not_implemented(octet * & ptr,const string & s,int i=1){
    gen gs(string2gen(s,false));
    gen g;
    if (i==1)
      g=ti_decode_unary(ptr,at_ti_not_implemented);
    else {
      if (i==2)
	g=ti_decode_binary(ptr,at_ti_not_implemented);
      else
	g=ti_decode_nary(ptr,at_ti_not_implemented,-i);
    }
    gen gf=g._SYMBptr->feuille;
    if (gf.type==_VECT)
      gf.subtype=_SEQ__VECT;
    return symbolic(at_ti_not_implemented,makevecteur(gs,gf));
  }

  // secondary or extension tag
  gen ti_secondary_tag(octet * & ptr){
    switch (*ptr){
    case INDIR_TAG:
      return ti_decode_unary(ptr,at_hash);
    case GETKEY_TAG:
      return ti_decode_not_implemented(ptr,"getKey",-0);
      // --ptr;
      // return symbolic(at_getKey,vecteur(0));
    case GETFOLD_TAG:
      return ti_decode_not_implemented(ptr,"getFold",-0);
      // --ptr;
      // return symbolic(at_GetFold,vecteur(0));
    case SWITCH_TAG:
      return ti_decode_nary(ptr,at_window_switch,0);
    case UNITCONV_TAG:
      return ti_decode_not_implemented(ptr,"UnitConv",2);
    case ORD_TAG:
      return ti_decode_unary(ptr,at_ord);
    case EXPR_TAG:
      return ti_decode_unary(ptr,at_expr);
    case CHAR_TAG:
      return ti_decode_unary(ptr,at_char);
    case STRING_TAG:
      return ti_decode_unary(ptr,at_string);
    case SETFOLD_TAG:
      return ti_decode_unary(ptr,at_SetFold);
    case GETTYPE_TAG:
      return ti_decode_unary(ptr,at_getType);
    case GETMODE_TAG:
      return ti_decode_not_implemented(ptr,"getMode");
    case PTTEST_TAG:
      return ti_decode_not_implemented(ptr,"ptTest",2);
    case PXLTEST_TAG:
      return ti_decode_not_implemented(ptr,"PxlTest",2);
    case SETGRAPH_TAG:
      return ti_decode_not_implemented(ptr,"setGraph",2);
    case SETTABLE_TAG:
      return ti_decode_not_implemented(ptr,"setTable",2);
    case SETMODE_TAG:
      return ti_decode_not_implemented(ptr,"setMode",-1);
    case FORMAT_TAG:
      return ti_decode_nary(ptr,at_format,1);
    case INSTRING_TAG:
      return ti_decode_nary(ptr,at_inString,2);
    case APPEND_TAG:
      return ti_decode_binary(ptr,at_append,false);
    case DD_TAG:
      return ti_decode_not_implemented(ptr,"DD");
    case EXPR2DMS_TAG:
      return ti_decode_not_implemented(ptr,"->DMS");
    case VEC2RECT_TAG:
      return ti_decode_not_implemented(ptr,">Rect");
    case VEC2POLAR_TAG:
      return ti_decode_not_implemented(ptr,">Polar");
    case VEC2CYLIND_TAG:
      return ti_decode_not_implemented(ptr,">Cylind");
    case VEC2SPHERE_TAG:
      return ti_decode_not_implemented(ptr,">Sphere");
    case PARENTH_START_TAG:
    case PARENTH_END_TAG:
    case MAT_START_TAG:
    case MAT_END_TAG:
    case LIST_START_TAG:
    case LIST_END_TAG:
    case COMMA_TAG:
    case SEMICOLON_TAG:
    case COMPLEX_ANGLE_TAG:
    case SINGLE_QUOTE_TAG:
    case QUOTE_TAG:
      setsizeerr("Parser internal token");
    case POLCPLX_TAG: // FIXME not clear if 2 or (2 or more args)
      return ti_decode_not_implemented(ptr,"polCplx",-2);
      // return ti_decode_not_implemented(ptr,"polCplx",2);
    case TMPCNV_TAG:
      return ti_decode_not_implemented(ptr,"tmpCnv",2);
    case DELTA_TMPCNV_TAG:
      return ti_decode_not_implemented(ptr,"DtmpCnv",2);
    case GETUNITS_TAG:
      return ti_decode_not_implemented(ptr,"getUnits",-0);
      // --ptr;
      // return string2gen("getUnits",false);
    case SETUNITS_TAG:
      return ti_decode_not_implemented(ptr,"setUnits");
    case BIN_TAG:
      return ti_decode_not_implemented(ptr,"0b");
    case HEX_TAG:
      return ti_decode_not_implemented(ptr,"0h");
    case INT2BIN_TAG:
      return ti_decode_not_implemented(ptr,">Bin");
    case INT2DEC_TAG:
      return ti_decode_not_implemented(ptr,">Dec");
    case INT2HEX_TAG:
      return ti_decode_not_implemented(ptr,">Hex");
    case DET_TOL_TAG:
      return ti_decode_binary(ptr,at_det);
    case REF_TOL_TAG:
      return ti_decode_binary(ptr,at_ref);
    case RREF_TOL_TAG:
      return ti_decode_binary(ptr,at_rref);
    case SIMULT_TOL_TAG:
      return ti_decode_nary(ptr,at_simult,3);
    case GETCONFG_TAG:
      return ti_decode_not_implemented(ptr,"getConfg",-0);
      // --ptr;
      // return string2gen("getConfg",false);
    case V_AUGMENT_TAG:
      return ti_decode_binary(ptr,at_semi_augment);
    case VARIANCE_TWOARG_TAG:
      return ti_decode_binary(ptr,at_variance);
    default:
      setsizeerr("Unknown secondary tag"+print_INT_(*ptr));
   }
  }

  const string _ti_then_s("ti_then");
  unary_function_unary __ti_then(0,_ti_then_s); // never evaled
  unary_function_ptr at_ti_then (&__ti_then);

  const string _ti_elseif_s("ti_elseif");
  unary_function_unary __ti_elseif(0,_ti_elseif_s); // never evaled
  unary_function_ptr at_ti_elseif (&__ti_elseif);

  const string _ti_else_try_s("ti_else_try");
  unary_function_unary __ti_else_try(0,_ti_else_try_s); // never evaled
  unary_function_ptr at_ti_else_try (&__ti_else_try);

  const string _ti_else_s("ti_else");
  unary_function_unary __ti_else(0,_ti_else_s); // never evaled
  unary_function_ptr at_ti_else (&__ti_else);

  gen ti_stop(const gen & g){
    setsizeerr("TI stop");
    return zero;
  }
  const string _ti_stop_s("ti_stop");
  unary_function_unary __ti_stop(&ti_stop,_ti_stop_s); // never evaled
  unary_function_ptr at_ti_stop (&__ti_stop);

  gen ti_decode_ifthen(const vecteur & arg0){
    vecteur arg(arg0);
    gen gif,gthen,gelse;
    int i=equalposcomp(arg,at_ti_then);
    if (!i){
      gif=arg.front();
      arg=vecteur(arg.begin()+1,arg.end());
    }
    else {
      gif=symb_bloc(vecteur(arg.begin(),arg.begin()+i-1));
      arg=vecteur(arg.begin()+i,arg.end());
    }
    // elseif?
    i=equalposcomp(arg,at_ti_elseif);
    if (i){
      gthen=symb_bloc(vecteur(arg.begin(),arg.begin()+i-1));
      arg=vecteur(arg.begin()+i,arg.end());
      return symb_ifte(gif,gthen,ti_decode_ifthen(arg));
    }
    // no more elseif found, search for else
    i=equalposcomp(arg,at_ti_else);
    if (!i)
      gelse=undef;
    else {
      gelse=symb_bloc(vecteur(arg.begin()+i,arg.end()));
      arg=vecteur(arg.begin(),arg.begin()+i-1);
    }
    gthen=symb_bloc(arg);
    return symb_ifte(gif,gthen,gelse);
  }

  gen ti_decode_ifthen(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDIF_ITAG,COMMAND_TAG));
    return ti_decode_ifthen(arg);
  }

  gen ti_decode_if(octet * & ptr){
    gen test(ti_decode_tag(ptr));
    ti_decode_newline(ptr);
    gen ifclause(ti_decode_tag(ptr));
    ti_decode_newline(ptr);
    return symb_ifte(test,ifclause,undef);
  }

  gen ti_decode_loop(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDLOOP_ITAG,COMMAND_TAG));
    ptr -= 2; // skip size
    return symb_for(zero,plus_one,zero,symb_bloc(arg));
  }

  gen ti_decode_for(octet * & ptr){
    gen gcompteur(ti_decode_tag(ptr));
    gen gdebut(ti_decode_tag(ptr));
    gen gfin(ti_decode_tag(ptr));
    gen gstep(plus_one);
    if (*ptr==END_TAG)
      --ptr;
    else {
      if (!ti_decode_newline(ptr)){
	gstep=ti_decode_tag(ptr);
	if (*ptr==END_TAG)
	  --ptr;
      }
    }
    ti_decode_newline(ptr);    
    vecteur arg(ti_decode_list(ptr,ENDFOR_ITAG,COMMAND_TAG));
    ptr -= 2; // skip size
    if (ck_is_strictly_positive(gstep,0))
      return symb_for(symb_sto(gdebut,gcompteur),symb_inferieur_egal(gcompteur,gfin),symb_sto(gcompteur+gstep,gcompteur),symb_bloc(arg));
    else
      return symb_for(symb_sto(gdebut,gcompteur),symb_superieur_egal(gcompteur,gfin),symb_sto(gcompteur+gstep,gcompteur),symb_bloc(arg));
  }

  gen ti_decode_while(octet * & ptr){
    gen test(ti_decode_tag(ptr));
    ti_decode_newline(ptr);
    vecteur arg(ti_decode_list(ptr,ENDWHILE_ITAG,COMMAND_TAG));
    ptr -= 2; // skip size
    return symb_for(zero,test,zero,symb_bloc(arg));
  }

  gen ti_decode_try(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDTRY_ITAG,COMMAND_TAG));
    ptr -= 2; // skip size
    int i=equalposcomp(arg,at_ti_else_try);
    gen gtry=symb_bloc(vecteur(arg.begin()+i,arg.end()));
    gen gelse=symb_bloc(vecteur(arg.begin(),arg.begin()+i-1));
    return symb_try_catch(makevecteur(gtry,_IDNT_break,gelse));
  }

  gen ti_decode_func(octet * & ptr,octet end){
    vecteur localvar;
    ti_decode_newline(ptr);
    if (*ptr==COMMENT_TAG){ // skip comment
      --ptr;
      while (!*ptr) --ptr;
      ti_decode_string(ptr);
      ti_decode_newline(ptr);
    }
    if (*(ptr-1)==LOCAL_ITAG && *ptr==COMMAND_TAG){ // check local
      ptr -=2;
      localvar=ti_decode_list(ptr,END_TAG);
    }
    vecteur arg(ti_decode_list(ptr,end,COMMAND_TAG));
    if (localvar.empty())
      return symb_bloc(arg);
    else
      return symb_local(localvar,arg);
  }

  gen ti_decode_dialog(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDDLOG_ITAG,COMMAND_TAG));
    return symbolic(at_Dialog,arg);
  }

  gen ti_decode_custom(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDCUSTM_ITAG,COMMAND_TAG));
    return symbolic(at_ti_not_implemented,arg);
  }

  gen ti_decode_toolbar(octet * & ptr){
    vecteur arg(ti_decode_list(ptr,ENDTBAR_ITAG,COMMAND_TAG));
    return symbolic(at_ti_not_implemented,arg);
  }

  gen ti_decode_local(octet * & ptr){
    // decode list until ENDFUNC or ENDPRGM COMMAND_TAG
    vecteur res;
    for (;;){
      ti_decode_newline(ptr);
      if ( (*(ptr-1)==ENDPRGM_ITAG || *(ptr-1)==ENDFUNC_ITAG) && 
	   *ptr==COMMAND_TAG)
	break;
      res.push_back(ti_decode_tag(ptr));
    }
    // do not decrement ptr, keep the end tags for func_itag or prgm_itag
    // split res using first end_tag
    int i=equalposcomp(res,at_ti_endtag);
    if (!i)
      setsizeerr("empty local declaration");
    gen gloc=vecteur(res.begin(),res.begin()+i-1);
    gen gbloc=vecteur(res.begin()+i,res.end());
    return symb_local(gloc,gbloc);
  }

  // command or instruction tag
  gen ti_command_tag(octet * & ptr){
    switch (*ptr){
    case CLRDRAW_ITAG:
      --ptr;
      return symbolic(at_ClrDraw,vecteur(0));
    case CLRGRAPH_ITAG:
      --ptr;
      return symbolic(at_ClrGraph,vecteur(0));
    case CLRIO_ITAG:
      --ptr;
      return symbolic(at_ClrIO,vecteur(0));
    case CLRHOME_ITAG: // FIXME
      --ptr;
      return symbolic(at_ti_not_implemented,string2gen("ClrHome",false));
    case CLRTABLE_ITAG:
      --ptr;
      return symbolic(at_ti_not_implemented,string2gen("ClrTable",false));
    case DISPG_ITAG:
      --ptr;
      return symbolic(at_DispG,vecteur(0));
    case DISPTBL_ITAG:
      --ptr;
      return symbolic(at_ti_not_implemented,string2gen("DispTbl",false));
    case CYCLE_ITAG:
      --ptr; // maybe ptr -= 3 to skip offset
      return symbolic(at_continue,zero);
    case CUSTOM_ITAG:
      --ptr;
      return ti_decode_custom(ptr);
    case DIALOG_ITAG:
      --ptr;
      return ti_decode_dialog(ptr);
    case TOOLBAR_ITAG:
      --ptr;
      return ti_decode_toolbar(ptr);
    case ELSE_ITAG:
      --ptr;
      return at_ti_else;
    case ENDCUSTM_ITAG:
    case ENDDLOG_ITAG: 
    case ENDFOR_ITAG:
    case ENDFUNC_ITAG:
    case ENDIF_ITAG:
    case ENDLOOP_ITAG:
    case ENDPRGM_ITAG:
    case ENDTBAR_ITAG:
    case ENDTRY_ITAG:
    case ENDWHILE_ITAG:
      setsizeerr("End structure "+print_INT_(*ptr)+" "+print_INT_(*(ptr-2))+" "+print_INT_(*(ptr-1)));
    case EXIT_ITAG:
      ptr -=3 ; // skip offset
      return symbolic(at_break,zero);;
    case FUNC_ITAG:
      --ptr;
      return ti_decode_func(ptr,ENDFUNC_ITAG);
    case PRGM_ITAG:
      --ptr;
      return ti_decode_func(ptr,ENDPRGM_ITAG);
    case LOOP_ITAG:
      --ptr;
      return ti_decode_loop(ptr);
    case STOP_ITAG:
      --ptr; // maybe ptr -=3 to skip itag
      return at_ti_stop;
    case THEN_ITAG:
      --ptr;
      return at_ti_then;
    case TRY_ITAG:
      --ptr;
      return ti_decode_try(ptr);
    case SHOWSTAT_ITAG:
      --ptr;
      return string2gen("showStat",false);
    case TRACE_ITAG:
      --ptr;
      return string2gen("Trace",false);
    case ZOOMBOX_ITAG:
      --ptr;
      return string2gen("ZoomBox",false);
    case ZOOMDATA_ITAG:
      --ptr;
      return string2gen("ZoomData",false);
    case ZOOMDEC_ITAG:
      --ptr;
      return string2gen("ZoomDec",false);
    case ZOOMFIT_ITAG:
      --ptr;
      return string2gen("ZoomFit",false);
    case ZOOMIN_ITAG:
      --ptr;
      return string2gen("ZoomIn",false);
    case ZOOMINT_ITAG:
      --ptr;
      return string2gen("ZoomInt",false);
    case ZOOMOUT_ITAG:
      --ptr;
      return string2gen("ZoomOut",false);
    case ZOOMPREV_ITAG:
      --ptr;
      return string2gen("ZoomPrev",false);
    case ZOOMRCL_ITAG:
      --ptr;
      return string2gen("ZoomRcl",false);
    case ZOOMSQR_ITAG:
      --ptr;
      return string2gen("ZoomSqr",false);
    case ZOOMSTD_ITAG:
      --ptr;
      return string2gen("ZoomStd",false);
    case ZOOMSTO_ITAG:
      --ptr;
      return string2gen("ZoomSto",false);
    case ZOOMTRIG_ITAG: // FIXME
      --ptr;
      return string2gen("ZoomTrig",false);
    case DRAWFUNC_ITAG:
      return ti_decode_unary(ptr,at_plotfunc);
    case DRAWINV_ITAG: // FIXME
      return ti_decode_unary(ptr,at_DrawInv);
      // return ti_decode_unary(ptr,at_ti_drawinv);
    case GOTO_ITAG:
      return ti_decode_unary(ptr,at_goto);
    case LBL_ITAG:
      return ti_decode_unary(ptr,at_label);
    case GET_ITAG: 
      return ti_decode_unary(ptr,at_Get);
    case SEND_ITAG:
      return ti_decode_not_implemented(ptr,"Send");
    case GETCALC_ITAG:
      return ti_decode_unary(ptr,at_GetCalc);
    case SENDCALC_ITAG:
      return ti_decode_not_implemented(ptr,"SendCalc");
    case NEWFOLD_ITAG:
      return ti_decode_unary(ptr,at_NewFold);
    case PRINTOBJ_ITAG: //FIXME
      return ti_decode_not_implemented(ptr,"printObj");
    case RCLGDB_ITAG:
      return ti_decode_not_implemented(ptr,"RclGDB");
    case STOGDB_ITAG:
      return ti_decode_not_implemented(ptr,"StoGDB");
    case ELSEIF_ITAG:
      --ptr;
      return at_ti_elseif;
    case IF_ITAG:
      --ptr;
      return ti_decode_if(ptr);
    case IFTHEN_ITAG:
      --ptr;
      return ti_decode_ifthen(ptr);
    case WHILE_ITAG:
      --ptr;
      return ti_decode_while(ptr);
    case RANDSEED_ITAG:
      return ti_decode_unary(ptr,at_RandSeed);
    case COPYVAR_ITAG:
      return ti_decode_binary(ptr,at_CopyVar);
    case RENAME_ITAG:
      return ti_decode_not_implemented(ptr,"Rename",2);
    case STYLE_ITAG:
      return ti_decode_not_implemented(ptr,"Style",2);
    case LINETAN_ITAG: 
      return ti_decode_binary(ptr,at_LineTan);
    case FILL_ITAG:
      return ti_decode_not_implemented(ptr,"Fill",2);
    case REQUEST_ITAG:
      return ti_decode_binary(ptr,at_Request);
    case POPUP_ITAG:
      return ti_decode_binary(ptr,at_PopUp);
    case PTCHG_ITAG:
      return ti_decode_not_implemented(ptr,"PtChg",2);
    case PTOFF_ITAG:
      return ti_decode_binary(ptr,at_PtOff);
    case PTON_ITAG:
      return ti_decode_binary(ptr,at_PtOn);
    case PXLCHG_ITAG:
      return ti_decode_not_implemented(ptr,"PxlChg",2);
    case PXLOFF_ITAG:
      return ti_decode_binary(ptr,at_PxlOff);
    case PXLON_ITAG:
      return ti_decode_binary(ptr,at_PxlOn);
    case MOVEVAR_ITAG:
      return ti_decode_not_implemented(ptr,"MoveVar",-3);
    case DROPDOWN_ITAG:
      return ti_decode_nary(ptr,at_DropDown,3);
    case OUTPUT_ITAG:
      return ti_decode_nary(ptr,at_Output,3);
    case PTTEXT_ITAG:
      return ti_decode_nary(ptr,at_PtText,3);
    case PXLTEXT_ITAG:
      return ti_decode_not_implemented(ptr,"PxlText",-3);
    case DRAWSLP_ITAG:
      return ti_decode_nary(ptr,at_DrawSlp,3);
    case PAUSE_ITAG:
      return ti_decode_nary(ptr,at_Pause,0);
    case RETURN_ITAG:
      return ti_decode_nary(ptr,at_return,0);
    case INPUT_ITAG:
      return ti_decode_nary(ptr,at_Input,2);
    case PLOTSOFF_ITAG:
      return ti_decode_not_implemented(ptr,"PlotsOff",-0);
    case PLOTSON_ITAG:
      return ti_decode_not_implemented(ptr,"PlotsOn",-0);
    case ONEVAR_ITAG:
      return ti_decode_not_implemented(ptr,"OneVar",-1);
    case TITLE_ITAG:
      return ti_decode_nary(ptr,at_Title,1);
    case ITEM_ITAG:
      return ti_decode_not_implemented(ptr,"Item",-1); 
    case INPUTSTR_ITAG:
      return ti_decode_nary(ptr,at_InputStr,3);
    case LINEHORZ_ITAG:
      return ti_decode_nary(ptr,at_LineHorz,4);
    case LINEVERT_ITAG:
      return ti_decode_nary(ptr,at_LineVert,4);
    case PXLHORZ_ITAG:
      return ti_decode_not_implemented(ptr,"PxlHorz",-1);
    case PXLVERT_ITAG:
      return ti_decode_not_implemented(ptr,"PxlVert",-1);
    case ANDPIC_ITAG:
      return ti_decode_not_implemented(ptr,"AndPic",-1);
    case XORPIC_ITAG:
      return ti_decode_not_implemented(ptr,"XorPic",-1);
    case DRAWPOL_ITAG:
      return ti_decode_not_implemented(ptr,"DrawPol",-1);
    case TABLE_ITAG:
      return ti_decode_not_implemented(ptr,"Table",-1);
    case RCLPIC_ITAG:
      return ti_decode_nary(ptr,at_RclPic,1);
    case RPLCPIC_ITAG:
      return ti_decode_nary(ptr,at_RplcPic,1);
    case TEXT_ITAG:
      return ti_decode_nary(ptr,at_Text,1);
    case STOPIC_ITAG:
      return ti_decode_nary(ptr,at_StoPic,1);
    case GRAPH_ITAG:
      return ti_decode_nary(ptr,at_Graph,1);
    case NEWPIC_ITAG:
      return ti_decode_nary(ptr,at_NewPic,1);
    case DRAWPARM_ITAG:
      return ti_decode_nary(ptr,at_DrawParm,2);
    case CYCLEPIC_ITAG:
      return ti_decode_nary(ptr,at_CyclePic,2);
    case CUBICREG_ITAG:
      return ti_decode_not_implemented(ptr,"CubicReg",-2);
    case EXPREG_ITAG:
      return ti_decode_not_implemented(ptr,"ExpReg",-2);
    case LINREG_ITAG:
      return ti_decode_not_implemented(ptr,"LinReg",-2);
    case LNREG_ITAG:
      return ti_decode_not_implemented(ptr,"LnReg",-2);
    case MEDMED_ITAG:
      return ti_decode_not_implemented(ptr,"MedMed",-2);
    case POWERREG_ITAG:
      return ti_decode_not_implemented(ptr,"PowerReg",-2);
    case QUADREG_ITAG:
      return ti_decode_not_implemented(ptr,"QuadReg",-2);
    case QUARTREG_ITAG:
      return ti_decode_not_implemented(ptr,"QuartReg",-2);
    case SINREG_ITAG:
      return ti_decode_not_implemented(ptr,"SinReg",-2);
    case LOGISTIC_ITAG:
      return ti_decode_not_implemented(ptr,"Logistic",-2);
    case TWOVAR_ITAG:
      return ti_decode_not_implemented(ptr,"TwoVar",-2);
    case SHADE_ITAG:
      return ti_decode_not_implemented(ptr,"Shade",-2);
    case FOR_ITAG:
      --ptr;
      return ti_decode_for(ptr);
    case CIRCLE_ITAG:
      return ti_decode_nary(ptr,at_Circle,2);
    case LINE_ITAG:
      return ti_decode_nary(ptr,at_Line,2);
    case DISP_ITAG:
      return ti_decode_nary(ptr,at_print,1);
    case PXLCRCL_ITAG:
      return ti_decode_not_implemented(ptr,"PxlCrcl",-3);
    case NEWPLOT_ITAG:
      return ti_decode_not_implemented(ptr,"NewPlot",-0);
    case PXLLINE_ITAG:
      return ti_decode_not_implemented(ptr,"PxlLine",-4);
    case FNOFF_ITAG:
      return ti_decode_not_implemented(ptr,"FnOff",-0);
    case FNON_ITAG:
      return ti_decode_not_implemented(ptr,"FnOn",-0);
    case LOCAL_ITAG:
      --ptr;
      return ti_decode_local(ptr);
    case DELFOLD_ITAG: // FIXME might be nary or unary!!!
      return ti_decode_unary(ptr,at_DelFold);
    case DELVAR_ITAG:
      return ti_decode_nary(ptr,at_DelVar,0);
    case PROMPT_ITAG:
      return ti_decode_nary(ptr,at_Prompt,0);
    case SORTA_ITAG:
      return ti_decode_nary(ptr,at_SortA,1);
    case SORTD_ITAG:
      return ti_decode_nary(ptr,at_SortD,1);
    case LOCK_ITAG:
      return ti_decode_not_implemented(ptr,"Lock",-1);
    case UNLOCK_ITAG: // FIXME maybe unary op
      return ti_decode_not_implemented(ptr,"Unlock",-1);
    case NEWDATA_ITAG:
      return ti_decode_not_implemented(ptr,"NewData",-2);
    case DEFINE_ITAG:
      return _Define(ti_decode_binary(ptr,at_nop)._SYMBptr->feuille);
    case ELSE_TRY_ITAG:
      --ptr;
      return at_ti_else_try;
    case CLRERR_ITAG:
      --ptr;
      return string2gen("ClrErr",false);
    case PASSERR_ITAG:
      --ptr;
      return string2gen("PassErr",false);
    case DISPHOME_ITAG:
      --ptr;
      return symbolic(at_DispHome,vecteur(0));
    case EXEC_ITAG:
      return ti_decode_nary(ptr,at_Exec,0);      
    case ARCHIVE_ITAG:
      return ti_decode_nary(ptr,at_Archive,1);      
    case UNARCHIV_ITAG:
      return ti_decode_nary(ptr,at_Unarchiv,1);      
    case LU_ITAG:
      return ti_decode_nary(ptr,at_LU,4);      
    case QR_ITAG:
      return ti_decode_nary(ptr,at_QR,4);   
    case BLDDATA_ITAG:
      return ti_decode_not_implemented(ptr,"BldData",-1);
    case DRWCTOUR_ITAG:
      return ti_decode_unary(ptr,at_DrwCtour);
    case NEWPROB_ITAG:
      --ptr;
      return string2gen("NewProb",false);
    case CUSTMON_ITAG:
      --ptr;
      return string2gen("CustmOn",false);
    case CUSTMOFF_ITAG:
      --ptr;
      return string2gen("CustmOff",false);
    case SENDCHAT_ITAG:
      return ti_decode_not_implemented(ptr,"SendChat");      
    default:
      setsizeerr("Unknown instruction tag:"+print_INT_(*ptr));
    }
  }

  // convert a TI9x object to a gen
  // ptr points to the tag of the structure (at the end)
  // after the call ptr points to the address previous beginning of object
  gen ti_decode_tag(octet * & ptr){
    unsigned short l;
    gen res;
    switch (*ptr){
    case COMPLEX_TAG: // FIXME
      res=cst_i*ti_decode_tag(ptr);
      return ti_decode_tag(ptr)+res;
      // return ti_decode_not_implemented(ptr,"ComplexTag",2);
    case NONNEGATIVE_INTEGER_TAG:
      --ptr;
      return ti_decode_unsigned(ptr);
    case NEGATIVE_INTEGER_TAG:
      --ptr;
      return -ti_decode_unsigned(ptr);
    case POSITIVE_FRACTION_TAG:
      --ptr;
      return ti_decode_fraction(ptr);
    case NEGATIVE_FRACTION_TAG:
      --ptr;
      return -ti_decode_fraction(ptr);
    case FLOAT_TAG:
      --ptr;
      return ti_decode_double(ptr);
    case STR_DATA_TAG:
      ptr-=2;
      return string2gen(ti_decode_string(ptr),false);
    case LIST_TAG: 
      --ptr;
      return ti_decode_list(ptr,END_TAG);
    case MATRIX_TAG:
      --ptr;
      res=ti_decode_list(ptr,END_TAG);
      res.subtype=_MATRIX__VECT;
      return res;
    case USER_DEF_TAG:
      --ptr;
      return ti_decode_function(ptr);
    case DATA_VAR_TAG:
      setsizeerr("Data_var_tag not implemented");
    case GDB_VAR_TAG:
      setsizeerr("Gdb_var_tag not implemented");
    case PIC_VAR_TAG:
      setsizeerr("Unable to find pic_var_tag length");      
    case TEXT_VAR_TAG:
      setsizeerr("Unable to find text_var_tag length");
    case COMMAND_TAG: // or INSTRUCTION_TAG
      --ptr;
      return ti_command_tag(ptr);
    case EXT_TAG: // or SECONDARY_TAG
      --ptr;
      return ti_secondary_tag(ptr);
    case END_TAG:
      --ptr;
      return at_ti_endtag;
    case END_OF_SEGMENT:
      setsizeerr("End_tag/End_of_segment");
    case ASM_PRGM_TAG:
      setsizeerr("Asm programs not implemented");
    case GEN_DATA_TEG:
      setsizeerr("3rd party data");
    case VAR_NAME_TAG:
      --ptr;
      return ti_decode_identificateur(ptr);
    case VAR_A_TAG:
      --ptr;
      return a__IDNT_e;
    case VAR_B_TAG:
      --ptr;
      return b__IDNT_e;
    case VAR_C_TAG:
      --ptr;
      return c__IDNT_e;
    case VAR_D_TAG:
      --ptr;
      return d__IDNT_e;
    case VAR_E_TAG:
      --ptr;
      return e__IDNT_e;
    case VAR_F_TAG:
      --ptr;
      return f__IDNT_e;
    case VAR_G_TAG:
      --ptr;
      return g__IDNT_e;
    case VAR_H_TAG:
      --ptr;
      return h__IDNT_e;
    case VAR_I_TAG:
      --ptr;
      return i__IDNT_e; 
    case VAR_J_TAG:
      --ptr;
      return j__IDNT_e;
    case VAR_K_TAG:
      --ptr;
      return k__IDNT_e;
    case VAR_L_TAG:
      --ptr;
      return l__IDNT_e;
    case VAR_M_TAG:
      --ptr;
      return m__IDNT_e;
    case VAR_N_TAG:
      --ptr;
      return n__IDNT_e;
    case VAR_O_TAG:
      --ptr;
      return o__IDNT_e;
    case VAR_P_TAG:
      --ptr;
      return p__IDNT_e;
    case _VAR_Q_TAG:
      cerr << "_var_q_tag" << endl;
    case VAR_Q_TAG:
      --ptr;
      return q__IDNT_e;
    case VAR_R_TAG:
      --ptr;
      return r__IDNT_e;
    case VAR_S_TAG:
      --ptr;
      return s__IDNT_e;
    case VAR_T_TAG:
      --ptr;
      return t__IDNT_e;
    case VAR_U_TAG:
      --ptr;
      return u__IDNT_e;
    case VAR_V_TAG:
      --ptr;
      return v__IDNT_e;
    case VAR_W_TAG:
      --ptr;
      return w__IDNT_e;
    case VAR_X_TAG:
      --ptr;
      return x__IDNT_e;
    case VAR_Y_TAG:
      --ptr;
      return y__IDNT_e;
    case VAR_Z_TAG:
      --ptr;
      return z__IDNT_e;
    case EXT_SYSTEM_TAG:
      --ptr;
      return ti_decode_sysvar_tag(ptr);
    case ARB_REAL_TAG:
      --ptr;
      return ti_decode_arb_real(ptr,true);
    case ARB_INT_TAG:
      --ptr;
      return ti_decode_arb_real(ptr,false);
    case PI_TAG:
      --ptr;
      return cst_pi;
    case EXP_TAG:
      --ptr;
      return e__IDNT_e;
    case IM_TAG:
      --ptr;
      return cst_i;
    case NEGINFINITY_TAG:
      --ptr;
      return minus_inf;
    case INFINITY_TAG:
      --ptr;
      return plus_inf;       
    case PN_INFINITY_TAG:
      --ptr;
      return unsigned_inf;
    case UNDEF_TAG:
      --ptr;
      return undef;
    case FALSE_TAG:
      --ptr;
      return zero;
    case TRUE_TAG:
      --ptr;
      return plus_one;
    case NOTHING_TAG:
      return ti_decode_unary(ptr,at_nop);
    case ACOSH_TAG:
      return ti_decode_unary(ptr,at_acosh);
    case ASINH_TAG:
      return ti_decode_unary(ptr,at_asinh);
    case ATANH_TAG:
      return ti_decode_unary(ptr,at_atanh);
    case COSH_TAG:
      return ti_decode_unary(ptr,at_acosh);
    case SINH_TAG:
      return ti_decode_unary(ptr,at_asinh);
    case TANH_TAG:
      return ti_decode_unary(ptr,at_atanh);
    case ACOS_TAG:
      return ti_decode_unary(ptr,at_acos);
    case ASIN_TAG:
      return ti_decode_unary(ptr,at_asin);
    case ATAN_TAG:
      return ti_decode_unary(ptr,at_atan);
    case RACOS_TAG:
      return ti_decode_unary(ptr,at_acos);
    case RASIN_TAG:
      return ti_decode_unary(ptr,at_asin);
    case RATAN_TAG:
      return ti_decode_unary(ptr,at_atan);
    case COS_TAG:
      return ti_decode_unary(ptr,at_cos);
    case SIN_TAG:
      return ti_decode_unary(ptr,at_sin);
    case TAN_TAG:
      return ti_decode_unary(ptr,at_tan);
    case 0x47: // FIXME!! where is 10^x?
      return ti_decode_unary(ptr,at_nop);
    case ITAN_TAG:
      return ti_decode_unary(ptr,at_tan);
    case ABS_TAG:
      return ti_decode_unary(ptr,at_abs);
    case ANGLE_TAG:
      return ti_decode_unary(ptr,at_arg);
    case CEILING_TAG:
      return ti_decode_unary(ptr,at_ceil);
    case FLOOR_TAG:
      return ti_decode_unary(ptr,at_floor);
    case INT_TAG:
      return ti_decode_unary(ptr,at_floor);
    case SIGN_TAG:
      return ti_decode_unary(ptr,at_sign);
    case SQRT_TAG:
      return ti_decode_unary(ptr,at_sqrt);
    case EXPF_TAG:
      return ti_decode_unary(ptr,at_exp);
    case LN_TAG:
      return ti_decode_unary(ptr,at_ln);
    case LOG_TAG:
      return ti_decode_unary(ptr,at_log10);
    case FPART_TAG:
      return ti_decode_unary(ptr,at_fPart);
    case IPART_TAG:
      return ti_decode_unary(ptr,at_iPart);
    case CONJ_TAG:
      return ti_decode_unary(ptr,at_conj);
    case IMAG_TAG:
      return ti_decode_unary(ptr,at_im);
    case REAL_TAG:
      return ti_decode_unary(ptr,at_re);
    case APPROX_TAG:
      return ti_decode_unary(ptr,at_evalf);
    case TEXPAND_TAG:
      return ti_decode_unary(ptr,at_texpand);
    case TCOLLECT_TAG:
      return ti_decode_unary(ptr,at_tcollect);
    case GETDENOM_TAG:
      return ti_decode_unary(ptr,at_getDenom);
    case GETNUM_TAG:
      return ti_decode_unary(ptr,at_getNum);
    case CUMSUM_TAG:
      return ti_decode_unary(ptr,at_cumSum);
    case DET_TAG:
      return ti_decode_unary(ptr,at_det);
    case COLNORM_TAG:
      return ti_decode_unary(ptr,at_colNorm);
    case ROWNORM_TAG:
      return ti_decode_unary(ptr,at_rowNorm);
    case NORM_TAG:
      return ti_decode_unary(ptr,at_l2norm);
    case MEAN_TAG:
      return ti_decode_unary(ptr,at_mean);
    case MEDIAN_TAG:
      return ti_decode_unary(ptr,at_median);
    case PRODUCT_TAG:
      return ti_decode_unary(ptr,at_product);
    case STDDEV_TAG:
      return ti_decode_unary(ptr,at_stddev);
    case SUM_TAG:
      return ti_decode_unary(ptr,at_sum);
    case VARIANCE_TAG:
      return ti_decode_unary(ptr,at_variance);
    case UNITV_TAG:
      return ti_decode_unary(ptr,at_unitV);
    case DIM_TAG:
      return ti_decode_unary(ptr,at_dim);
    case MAT2LIST_TAG: 
      return ti_decode_unary(ptr,at_mat2list);
    case NEWLIST_TAG:
      return ti_decode_unary(ptr,at_newList);
    case RREF_TAG:
      return ti_decode_unary(ptr,at_rref);
    case REF_TAG:
      return ti_decode_unary(ptr,at_ref);
    case IDENTITY_TAG:
      return ti_decode_unary(ptr,at_identity);
    case DIAG_TAG:
      return ti_decode_unary(ptr,at_diag);
    case COLDIM_TAG:
      return ti_decode_unary(ptr,at_colDim);
    case ROWDIM_TAG:
      return ti_decode_unary(ptr,at_rowDim);
    case TRANSPOSE_TAG:
      return ti_decode_unary(ptr,at_transpose);
    case FACTORIAL_TAG:
      return ti_decode_unary(ptr,at_factorial);
    case PERCENT_TAG: 
      return ti_decode_not_implemented(ptr,"percent");
    case RADIANS_TAG: 
      return ti_decode_not_implemented(ptr,"radians");
    case NOT_TAG:
      return ti_decode_unary(ptr,at_not);
    case MINUS_TAG:
      return ti_decode_unary(ptr,at_neg);
    case VEC_POLAR_TAG:
      return ti_decode_not_implemented(ptr,">Polar");
    case VEC_CYLIND_TAG:
      return ti_decode_not_implemented(ptr,">Cylind");
    case VEC_SPHERE_TAG:
      return ti_decode_not_implemented(ptr,">Sphere");
    case START_TAG: // internal tag
      setsizeerr("start tag");
    case ISTORE_TAG:
      return ti_decode_binary(ptr,at_sto,false);
    case STORE_TAG:
      return ti_decode_binary(ptr,at_sto,false);
    case WITH_TAG: 
      return ti_decode_binary(ptr,at_tilocal);
    case XOR_TAG:
      return ti_decode_binary(ptr,at_xor);
    case OR_TAG:
      return ti_decode_binary(ptr,at_ou);
    case AND_TAG:
      return ti_decode_binary(ptr,at_and);
    case LT_TAG:
      return ti_decode_binary(ptr,at_inferieur_strict);
    case LE_TAG:
      return ti_decode_binary(ptr,at_inferieur_egal);
    case EQ_TAG:
      return ti_decode_binary(ptr,at_equal);
    case GE_TAG:
      return ti_decode_binary(ptr,at_superieur_strict);
    case GT_TAG:
      return ti_decode_binary(ptr,at_superieur_egal);
    case NE_TAG:
      return ti_decode_binary(ptr,at_different);
    case ADD_TAG:
      return ti_decode_binary(ptr,at_plus,false);
    case ADDELT_TAG:
      return ti_decode_binary(ptr,at_plus,false);
    case SUB_TAG:
      res=ti_decode_binary(ptr,at_minus,false);
      res=*res._SYMBptr->feuille._VECTptr;
      res._VECTptr->back()=-res._VECTptr->back();
      return symbolic(at_plus,res);
    case SUBELT_TAG:
      return ti_decode_binary(ptr,at_minus,false);
    case MUL_TAG:
      return ti_decode_binary(ptr,at_prod,false);
    case MULELT_TAG:
      return ti_decode_binary(ptr,at_pointprod,false);
    case DIV_TAG:
      return ti_decode_binary(ptr,at_division,false);
    case DIVELT_TAG:
      return ti_decode_binary(ptr,at_pointdivision,false);
    case POW_TAG:
      return ti_decode_binary(ptr,at_pow);
    case POWELT_TAG:
      return ti_decode_binary(ptr,at_pointpow);
    case SINCOS_TAG:
      setsizeerr("Internal sincos token");
    case SOLVE_TAG:
      return ti_decode_binary(ptr,at_solve);
    case CSOLVE_TAG:
      return ti_decode_binary(ptr,at_cSolve);
    case NSOLVE_TAG:
      return ti_decode_binary(ptr,at_nSolve);
    case ZEROS_TAG:
      return ti_decode_binary(ptr,at_zeros);
    case CZEROS_TAG:
      return ti_decode_binary(ptr,at_cZeros);
    case FMIN_TAG:
      return ti_decode_binary(ptr,at_fMin);
    case FMAX_TAG:
      return ti_decode_binary(ptr,at_fMax);
    case POLYEVAL_TAG:
      return ti_decode_binary(ptr,at_polyEval);
    case RANDPOLY_TAG:
      return ti_decode_binary(ptr,at_randPoly);
    case CROSSP_TAG:
      return ti_decode_binary(ptr,at_crossP);
    case DOTP_TAG:
      return ti_decode_binary(ptr,at_dotP);
    case GCD_TAG:
      return ti_decode_binary(ptr,at_gcd);
    case LCM_TAG:
      return ti_decode_binary(ptr,at_lcm);
    case MOD_TAG:
      return ti_decode_binary(ptr,at_irem);
    case INTDIV_TAG:
      return ti_decode_binary(ptr,at_intDiv);
    case REMAIN_TAG:
      return ti_decode_binary(ptr,at_remain);
    case NCR_TAG:
      return ti_decode_binary(ptr,at_nCr);
    case NPR_TAG:
      return ti_decode_binary(ptr,at_nPr);
    case P2RX_TAG:
      return ti_decode_not_implemented(ptr,"P->Rx",2);
    case P2RY_TAG:
      return ti_decode_not_implemented(ptr,"P->Ry",2);
    case P2PTHETA_TAG:
      return ti_decode_not_implemented(ptr,"R->Pq",2);
    case P2PR_TAG:
      return ti_decode_not_implemented(ptr,"R->Pr",2);
    case AUGMENT_TAG:
      return ti_decode_binary(ptr,at_augment);
    case NEWMAT_TAG:
      return ti_decode_binary(ptr,at_newMat);
    case RANDMAT_TAG:
      return ti_decode_binary(ptr,at_randMat);
    case SIMULT_TAG:
      return ti_decode_binary(ptr,at_simult);
    case PART_TAG:
      return ti_decode_nary(ptr,at_part,1);
    case EXP2LIST_TAG:
      return ti_decode_binary(ptr,at_exp2list);
    case RANDNORM_TAG:
      return ti_decode_binary(ptr,at_randNorm);
    case MROW_TAG:
      return ti_decode_nary(ptr,at_mRow,3);
    case ROWADD_TAG:
      return ti_decode_nary(ptr,at_rowAdd,4);
    case ROWSWAP_TAG:
      return ti_decode_nary(ptr,at_rowSwap,3);
    case ARCLEN_TAG:
      return ti_decode_nary(ptr,at_arcLen,4);
    case NINT_TAG:
      return ti_decode_nary(ptr,at_nInt,4);
    case PI_PRODUCT_TAG:
      return ti_decode_nary(ptr,at_product,1);
    case SIGMA_SUM_TAG:
      return ti_decode_nary(ptr,at_sum,4);
    case MROWADD_TAG:
      return ti_decode_nary(ptr,at_mRowAdd,4);
    case ANS_TAG:
      return ti_decode_nary(ptr,at_ans,0);
    case ENTRY_TAG:
      return ti_decode_nary(ptr,at_entry,0);
    case EXACT_TAG:
      return ti_decode_nary(ptr,at_exact,1);
    case LOGB_TAG:
      return ti_decode_binary(ptr,at_logb);
    case COMDENOM_TAG:
      return ti_decode_nary(ptr,at_comDenom,1);
    case EXPAND_TAG:
      return ti_decode_nary(ptr,at_expand,1);
    case FACTOR_TAG:
      return ti_decode_nary(ptr,at_factor,1);
    case CFACTOR_TAG:
      return ti_decode_nary(ptr,at_cFactor,1);
    case INTEGRATE_TAG:
      return ti_decode_nary(ptr,at_integrate,2);
    case DIFFERENTIATE_TAG:
      return ti_decode_nary(ptr,at_derive,2);
    case AVGRC_TAG:
      return ti_decode_nary(ptr,at_avgRC,2);
    case NDERIV_TAG:
      return ti_decode_nary(ptr,at_nDeriv,2);
    case TAYLOR_TAG:
      return ti_decode_nary(ptr,at_taylor,3);
    case LIMIT_TAG:
      return ti_decode_nary(ptr,at_limit,3);
    case PROPFRAC_TAG:
      return ti_decode_nary(ptr,at_propFrac,1);
    case WHEN_TAG:
      return ti_decode_nary(ptr,at_when,3);
    case ROUND_TAG:
      return ti_decode_nary(ptr,at_round,1);
    case DMS_TAG: // FIXME not clear
      return ti_decode_not_implemented(ptr,"DMS");
      // return ti_decode_not_implemented(ptr,"DMS",-1);
    case LEFT_TAG:
      return ti_decode_nary(ptr,at_left,1);
    case RIGHT_TAG:
      return ti_decode_nary(ptr,at_right,1);
    case MID_TAG:
      return ti_decode_nary(ptr,at_mid,2);
    case SHIFT_TAG:
      return ti_decode_nary(ptr,at_shift,1);
    case SEQ_TAG:
      return ti_decode_nary(ptr,at_seq,4);
    case LIST2MAT_TAG:
      return ti_decode_nary(ptr,at_list2mat,1);
    case SUBMAT_TAG:
      return ti_decode_nary(ptr,at_subMat,1);
    case SUBSCRIPT_TAG:
      res=ti_decode_nary(ptr,at_at,2);
      if (res.type!=_SYMB || res._SYMBptr->feuille.type!=_VECT)
	setsizeerr();
      res=res._SYMBptr->feuille;
      if (res._VECTptr->size()==2)
	return symbolic(at_at,makevecteur(res._VECTptr->front(),(res._VECTptr->back()-plus_one)));
      else
	return symbolic(at_at,makevecteur(res._VECTptr->front(),gen(vecteur(res._VECTptr->begin()+1,res._VECTptr->end()),_SEQ__VECT)-gen(vecteur(res._VECTptr->size()-1,plus_one),_SEQ__VECT)));
    case RAND_TAG:
      return ti_decode_nary(ptr,at_rand,0);
    case MIN_TAG:
      return ti_decode_nary(ptr,at_min,1);
    case MAX_TAG:
      return ti_decode_nary(ptr,at_max,1);
    case USERFUNC_TAG:
      res=ti_decode_nary(ptr,at_of,0);
      if (res.type!=_SYMB || res._SYMBptr->feuille.type!=_VECT)
	setsizeerr();
      res=res._SYMBptr->feuille;
      return symbolic(at_of,makevecteur(res._VECTptr->front(),gen(vecteur(res._VECTptr->begin()+1,res._VECTptr->end()),_SEQ__VECT)));
    case FIG_TAG:
      setsizeerr("fig_tag");
    case MAC_TAG:
      setsizeerr("mac_tag");
    case COMMENT_TAG:
      ptr -=2;
      while (!*ptr) --ptr;
      return string2gen(ti_decode_string(ptr),false);
    case NEXTEXPR_TAG: case NEWLINE_TAG: // ":" or "\n" inside a prog
      ti_decode_newline(ptr);
      return at_nop;
    case PN1_TAG:
      return ti_decode_unary(ptr,at_neg);
    case PN2_TAG: 
      cerr << "pn2_tag" << endl;
      return ti_decode_binary(ptr,at_neg);
    case ERROR_MSG_TAG:
      return ti_decode_not_implemented(ptr,"ErrorMsg");
    case EIGVC_TAG:
      return ti_decode_unary(ptr,at_eigVc);
    case EIGVL_TAG:
      return ti_decode_unary(ptr,at_eigVl);
    case DASH_TAG:
      return ti_decode_unary(ptr,at_derive);
    case LOCALVAR_TAG:
      --ptr;
      return ti_decode_tag(ptr);
    case DESOLVE_TAG:
      return ti_decode_nary(ptr,at_deSolve,3);
    case FDASH_TAG:
      return ti_decode_binary(ptr,at_derive);
    case ISPRIME_TAG:
      return ti_decode_unary(ptr,at_isprime);
    case ROTATE_TAG:
      return ti_decode_nary(ptr,at_rotate,1);
    default:
      setsizeerr("Unknown tag "+print_INT_(*ptr));
    }
    --ptr;
    return res;
  }

  // convert a TI9x object to a gen
  // ptr points to the beginning of the structure
  gen ti2gen(octet * ptr){
    /*
    if (numeric_limits<unsigned char>::Digits!=8){
      setsizeerr("Must recompile with a typedef for octet as 8-bit data");
    }
    */
    int offset= ptr[0]*256+ptr[1];
    octet * tag = ptr + (offset +1);
    // Check here for text, non tokenized progs, 3rd party data 
    // since length must be known
    if (*tag==TEXT_VAR_TAG){
      char * ptrs=(char *)ptr+4;
      return string2gen(tiasc_translate(ptrs),false);
    }
    if (*tag==USER_DEF_TAG && (*(tag-1) & 0x08)){
      ptr +=2; // point to argument list
      string s(":tmpfunc");
      char c;
      tag -= 8;
      for (;ptr!=tag;++ptr){
	c=*ptr;
	if (*ptr>0x7F){
	  if (-c==87){ // must skip until end of line
	    for (;ptr!=tag;++ptr){
	      if (*ptr=='\r')
		break;
	    }
	  }
	  else
	    s += "Z"+print_INT_(-c);
	}
	else {
	  switch(c){
	  case 0:
	    continue;
	  case '\r':
	    s = (s+'\n')+':';
	    break;
	  case '\026':
	    s += "=>";
	    break;
	  default:
	    s += c;
	  }
	}
      }
      s=tiasc_translate(s);
      cerr << s << endl;
      int save_maple_mode=maple_mode;
      maple_mode=3;
      gen res(s);
      maple_mode=save_maple_mode;
      return res;
    }
    return ti_decode_tag(tag);
  }

  gen decode_name(octet * buf){
    string lu;
    char c;
    for (int i=0;i<8 && (c=buf[i]);++i)
      lu += c;
    gen gname(lu);
    if (gname.type!=_IDNT)
      gname=gen("_"+lu);
    return gname;
  }

  // FIXME SECURITY
  // Format [length 2 bytes] 0xE9 prgm 0x19 or 0x13 0xE5 ... 0x0 0x0 0x0 0xDC
  // length is at offset 0x56 in a program file, name at 0x40
  gen _unarchive_ti(const gen & g){
    if (g.type!=_STRNG)
      setsizeerr();
    if (access(g._STRNGptr->c_str(),R_OK))
      setsizeerr("Unable to open "+g.print());
    ifstream is(g._STRNGptr->c_str());
    string lu;
    char c;
    for (;!is.eof();){
      is.get(c);
      lu += c;
    }
    unsigned int s=lu.size();
    if (s<0x60)
      setsizeerr("Too short for a TI archive");
    octet buf[s];
    memcpy(buf,lu.c_str(),s);
    if (lu[6]=='P'){ // package/group file
      // Group file format
      // First name is folder at 0x40 offset 0x4c (dir name)
      // List of names 0x50, 0x60, ... offsets 0x5c 0x6c etc.
      // offset + 4 = begin of func/prog, ex. offset=0x1322, length at 0x1326
      gen gfoldername(decode_name(&buf[0x40]));
      vecteur res;
      if (gfoldername.print()!="main"){
	cerr << "Degrouping in folder " << gfoldername << endl;
	res.push_back(symbolic(at_NewFold,gfoldername));
	res.push_back(symbolic(at_SetFold,gfoldername));
      }
      unsigned int t=buf[0x3e]*65536+buf[0x3d]*256+buf[0x3c];
      if (s<t)
	setsizeerr("Not long enough");
      unsigned int nprogs=(t-0x52)/0x10;
      // decode each prog
      for (unsigned int i=0;i<nprogs;++i){
	gen gname(decode_name(&buf[0x50+0x10*i]));
	cerr << "Fonction " << gname << endl;
	parse_filename=gname.print();
	unsigned int tt=buf[0x4e +0x10*i]*65536+buf[0x4d+0x10*i]*256+buf[0x4c+0x10*i]+4;
	if (tt>s)
	  return res;
	unsigned int ttt=buf[tt]*256+buf[tt+1];
	if (s<ttt+tt+2)
	  return res;
	gen gval;
	try {
	  gval=ti2gen(&buf[tt]);
	}
	catch (std::runtime_error & e){
	  cerr << gname << ":" << e.what();
	  gval=string2gen(e.what(),false);
	}
	if (gval.is_symb_of_sommet(at_sto))
	  res.push_back(symb_sto(gval._SYMBptr->feuille[0],gname));
	res.push_back(symb_sto(gval,gname));
      }
      return res;
    }
    unsigned int t=buf[0x56]*256+buf[0x57];
    if (s<t+0x56+2)
      setsizeerr("Not a TI89/92 program/function/group file");
    gen gname(decode_name(&buf[0x40]));
    gen gval(ti2gen(&buf[0x56]));
    if (gval.is_symb_of_sommet(at_sto))
      return symb_sto(gval._SYMBptr->feuille[0],gname);
    return symb_sto(gval,gname);
  }
  const string _unarchive_ti_s("unarchive_ti");
  unary_function_unary __unarchive_ti(&_unarchive_ti,_unarchive_ti_s);
  unary_function_ptr at_unarchive_ti (&__unarchive_ti,0,true);

  // French structure keywords
  string printassialorssinon(const gen & feuille,const string & sommetstr){
    if ( (feuille.type!=_VECT) || (feuille._VECTptr->size()!=3) )
      return "sialorssinon("+feuille.print()+')';
    const_iterateur it=feuille._VECTptr->begin(),itend=feuille._VECTptr->end();
    string res("si ");
    res += sametoequal(*it).print();
    ++it;
    res += " alors ";
    indent_spaces +=2;
    if ((it->type==_SYMB) && (it->_SYMBptr->sommet==at_bloc))
      res += printasinnerbloc(it->_SYMBptr->feuille);
    else
      res += it->print() ;
    indent_spaces -=2;
    res+= " sinon ";
    ++it;
    indent_spaces +=2;
    if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_bloc))
      res += printasinnerbloc(it->_SYMBptr->feuille);
    else
      res += it->print() ;
    indent_spaces -=2;
    res += indent()+ "fsi;";
    return res;
  }

  gen _sialorssinon(const gen & g,GIAC_CONTEXT){
    return _ifte(g,contextptr);
  }
  const string _sialorssinon_s("sialorssinon");
  unary_function_eval __sialorssinon(&_sialorssinon,_sialorssinon_s,&printassialorssinon);
  unary_function_ptr at_sialorssinon (&__sialorssinon,_QUOTE_ARGUMENTS,T_IFTE);

  const string _si_s("si");
  unary_function_eval __si(&_sialorssinon,_si_s,&printassialorssinon);
  unary_function_ptr at_si (&__si,_QUOTE_ARGUMENTS,T_MUPMAP_IF);

  const string _alors_s("alors");
  unary_function_eval __alors(&_ifte,_alors_s);
  unary_function_ptr at_alors (&__alors,_QUOTE_ARGUMENTS,T_THEN);
  
  const string _sinon_s("sinon");
  unary_function_eval __sinon(&_ifte,_sinon_s);
  unary_function_ptr at_sinon (&__sinon,_QUOTE_ARGUMENTS,T_ELSE);
  
  const string _fsi_s("fsi");
  unary_function_eval __fsi(&_ifte,_fsi_s);
  unary_function_ptr at_fsi (&__fsi,_QUOTE_ARGUMENTS,T_BLOC_END);

  string printaspour(const gen & feuille,const string & sommetstr){
    if ( (feuille.type!=_VECT) || (feuille._VECTptr->size()!=4) )
      return sommetstr+'('+feuille.print()+')';
    const_iterateur it=feuille._VECTptr->begin(),itend=feuille._VECTptr->end();
    string res;
    if (is_zero(*it) && is_zero(*(it+2))){
      ++it;
      res ="tantque " + sametoequal(*it).print() + " faire ";
      ++it;
      ++it;
      indent_spaces += 2;
      if ((it->type==_SYMB) && (it->_SYMBptr->sommet==at_bloc))
          res += printasinnerbloc(it->_SYMBptr->feuille);
      else
          res += it->print() ;
      indent_spaces -= 2;
      return res+indent()+" ftantque;";
    }
    else {  
      if ( (it->type!=_SYMB) || (it->_SYMBptr->sommet!=at_sto) || ((it+2)->type!=_SYMB) || ((it+2)->_SYMBptr->sommet!=at_sto) || (it->_SYMBptr->feuille._VECTptr->back()!=(it+2)->_SYMBptr->feuille._VECTptr->back()) ){
	res="pour (";
	res += it->print() + ';';
	++it;
	res += it->print() + ';';
	++it;
	res += it->print() + ") ";
	++it;
	indent_spaces += 2;
	res += it->print() ;
	indent_spaces -= 2;
      }
      else {
	gen var_name=it->_SYMBptr->feuille._VECTptr->back();
	gen step=normal((it+2)->_SYMBptr->feuille._VECTptr->front()-var_name);
	gen condition=*(it+1),limite;
	bool simple_loop=false,strict=true,ascending=true;
	if (condition.type==_SYMB){
	  unary_function_ptr op=condition._SYMBptr->sommet;
	  if (condition._SYMBptr->feuille.type==_VECT){
	    if (op==at_inferieur_strict)
	      simple_loop=true;
	    if (op==at_inferieur_egal){
	      strict=false;
	      simple_loop=true;
	    }
	    if (op==at_superieur_strict){
	      simple_loop=(maple_mode==2);
	      ascending=false;
	    }
	    if (op==at_superieur_egal){
	      simple_loop=(maple_mode==2);
	      ascending=false;
	      strict=false;
	    }
	  }
	  if (simple_loop){
	    simple_loop=(condition._SYMBptr->feuille._VECTptr->front()==var_name);
	    limite=condition._SYMBptr->feuille._VECTptr->back();
	  }
	}
	res ="pour ";
	res += var_name.print();
	res += " de ";
	res += it->_SYMBptr->feuille._VECTptr->front().print();
	if (simple_loop){
	  step = abs(step);
	  if (ascending)
	    res += " jusque ";
	  else
	    res += " downto ";
	  res += limite.print();
	  if (strict){
	    if (ascending)
	      res +="+";
	    else
	      res += "-";
	    res += step.print();
	    res += "/2";
	  }
	}
	if (!is_one(step)){
	  res += " by ";
	  res += step.print();
	}
	if (!simple_loop){
	  res += " tantque ";
	  res += (it+1)->print();
	}
	res += " faire ";
	it += 3;
	if ((it->type==_SYMB) && (it->_SYMBptr->sommet==at_bloc))
	  res += printasinnerbloc(it->_SYMBptr->feuille);
	else
	  res += it->print() ;
	return res + indent()+" fpour;";
      }
    }
    if (res[res.size()-1]!='}')
      res += "; ";
    return res;
  }
  gen _pour(const gen & g,GIAC_CONTEXT){
    return _for(g,contextptr);
  }
  const string _pour_s("pour");
  unary_function_eval __pour(&_pour,_pour_s,&printaspour);
  unary_function_ptr at_pour (&__pour,_QUOTE_ARGUMENTS,T_FOR);

  const string _fpour_s("fpour");
  unary_function_eval __fpour(&_pour,_fpour_s);
  unary_function_ptr at_fpour (&__fpour,_QUOTE_ARGUMENTS,T_BLOC_END);
  
  const string _ftantque_s("ftantque");
  unary_function_eval __ftantque(&_pour,_ftantque_s);
  unary_function_ptr at_ftantque (&__ftantque,_QUOTE_ARGUMENTS,T_BLOC_END);
  
  const string _de_s("de");
  unary_function_eval __de(&_pour,_de_s);
  unary_function_ptr at_de (&__de,_QUOTE_ARGUMENTS,T_FROM);

  const string _faire_s("faire");
  unary_function_eval __faire(&_pour,_faire_s);
  unary_function_ptr at_faire (&__faire,_QUOTE_ARGUMENTS,T_DO);

  const string _pas_s("pas");
  unary_function_eval __pas(&_pour,_pas_s);
  unary_function_ptr at_pas (&__pas,_QUOTE_ARGUMENTS,T_BY);

  const string _jusque_s("jusque");
  unary_function_eval __jusque(&_pour,_jusque_s);
  unary_function_ptr at_jusque (&__jusque,_QUOTE_ARGUMENTS,T_TO);

  const string _tantque_s("tantque");
  unary_function_eval __tantque(&_pour,_tantque_s);
  unary_function_ptr at_tantque (&__tantque,_QUOTE_ARGUMENTS,T_MUPMAP_WHILE);

  const string _et_s("et");
  unary_function_eval __et(&_and,_et_s);
  unary_function_ptr at_et (&__et,_QUOTE_ARGUMENTS,T_AND_OP);

  const string _oufr_s("ou");
  unary_function_eval __oufr(&_ou,_oufr_s);
  unary_function_ptr at_oufr (&__oufr,_QUOTE_ARGUMENTS,T_OR_OP);

  const string _non_s("non");
  unary_function_unary __non(&_not,_non_s);
  unary_function_ptr at_non (&__non,_QUOTE_ARGUMENTS,T_NOT);

  const string _resultat_s("resultat");
  unary_function_unary __resultat(&_nop,_resultat_s);
  unary_function_ptr at_resultat (&__resultat,_QUOTE_ARGUMENTS,T_RETURN);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
