// -*- mode:C++ ; compile-command: "g++ -I.. -I../include -g -c usual.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include <cstdlib>
#include "gen.h"
#include "identificateur.h"
#include "symbolic.h"
#include "poly.h"
#include "usual.h"
#include "series.h"
#include "modpoly.h"
#include "sym2poly.h"
#include "moyal.h"
#include "subst.h"
#include "gausspol.h"
#include "identificateur.h"
#include "ifactor.h"
#include <unistd.h>
#include "prog.h"
#include "rpn.h"
#include "plot.h"
#include "pari.h"
#include "tex.h"
#include "unary.h"
#include "intg.h"
#include "ti89.h"
#ifdef HAVE_LIBGSL
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_sf_psi.h>
#include <gsl/gsl_sf_zeta.h>
#include <gsl/gsl_errno.h>
#endif

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  string messages_to_print ;

  // utilities for trig functions
  enum { trig_deno=24 };
  bool is_multiple_of_pi_over_12(const gen & a,int & l,bool angle_unit=true){
    if (is_zero(a)){
      l=0;
      return true;
    }
    gen k;
    if (angle_unit){
      if (!contains(a,_IDNT_pi))
	return false;
      k=normal(rdiv(a*gen(trig_deno/2),cst_pi));      
    }
    else 
      k=rdiv(a,15);
    if (!k.is_integer()){
      return false;
    }
    k=smod(k,trig_deno);
    if (k.type!=_INT_)
      return false;
    l=k.val+trig_deno/2;
    return (l%6!=1) && (l%6!=5);
  }

  // 0 = -pi, 12=0, 24=pi
  gen * table_cos[trig_deno+1]={
    &minus_one,&undef,&minus_sqrt3_2,&minus_sqrt2_2,&minus_one_half,&undef,
    &zero,&undef,&plus_one_half,&plus_sqrt2_2,&plus_sqrt3_2,&undef,
    &plus_one,&undef,&plus_sqrt3_2,&plus_sqrt2_2,&plus_one_half,&undef,
    &zero,&undef,&minus_one_half,&minus_sqrt2_2,&minus_sqrt3_2,&undef,
    &minus_one
  };
  gen * table_tan[trig_deno/2+1]={
    &zero,&undef,&plus_sqrt3_3,&plus_one,&plus_sqrt3,&undef,
    &unsigned_inf,&undef,&minus_sqrt3,&minus_one,&minus_sqrt3_3,&undef,
    &zero
  };


  bool is_rational(const gen & a,int &n,int &d){
    gen num,den;
    fxnd(a,num,den);
    if (num.type!=_INT_ || den.type!=_INT_)
      return false;
    n=num.val;
    d=den.val;
    return true;
  }
  // checking utility
  void check_2d_vecteur(const gen & args) {
    if (args.type!=_VECT)
      settypeerr("check_2d_vecteur");
    if (args._VECTptr->size()!=2)
      setsizeerr("check_2d_vecteur");
  }

  // zero arg
  unary_function_constant __1(1);
  unary_function_ptr at_one (&__1);
  unary_function_constant __0(0);
  unary_function_ptr at_zero (&__0);
  
  gen _rm_a_z(const gen & args,GIAC_CONTEXT){
    if (variables_are_files(contextptr)){
      char a_effacer[]="a.cas";
      for (;a_effacer[0]<='z';++a_effacer[0]){
	unlink(a_effacer);
      }
    }
    _purge(a__IDNT_e,contextptr);
    _purge(b__IDNT_e,contextptr);
    _purge(c__IDNT_e,contextptr);
    _purge(d__IDNT_e,contextptr);
    _purge(f__IDNT_e,contextptr);
    _purge(g__IDNT_e,contextptr);
    _purge(h__IDNT_e,contextptr);
    _purge(i__IDNT_e,contextptr);
    _purge(j__IDNT_e,contextptr);
    _purge(k__IDNT_e,contextptr);
    _purge(l__IDNT_e,contextptr);
    _purge(m__IDNT_e,contextptr);
    _purge(n__IDNT_e,contextptr);
    _purge(o__IDNT_e,contextptr);
    _purge(p__IDNT_e,contextptr);
    _purge(q__IDNT_e,contextptr);
    _purge(r__IDNT_e,contextptr);
    _purge(s__IDNT_e,contextptr);
    _purge(t__IDNT_e,contextptr);
    _purge(u__IDNT_e,contextptr);
    _purge(v__IDNT_e,contextptr);
    _purge(w__IDNT_e,contextptr);
    _purge(x__IDNT_e,contextptr);
    _purge(y__IDNT_e,contextptr);
    _purge(z__IDNT_e,contextptr);
    return args;
  }
  const string _rm_a_z_s("rm_a_z");
  unary_function_eval __rm_a_z(&_rm_a_z,_rm_a_z_s);
  unary_function_ptr at_rm_a_z (&__rm_a_z);

  gen _rm_all_vars(const gen & args,const context * contextptr){
    gen g=_VARS(zero,contextptr);
    if (g.type!=_VECT)
      return g;
    vecteur & v=*g._VECTptr;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type==_IDNT && (*it!=cst_pi) && (*it!=e__IDNT_e) )
	_purge(*it,contextptr);
    }
    return g;
  }
  const string _rm_all_vars_s("rm_all_vars");
  unary_function_eval __rm_all_vars(&_rm_all_vars,_rm_all_vars_s);
  unary_function_ptr at_rm_all_vars (&__rm_all_vars);

  bool is_equal(const gen & g){
    return (g.type==_SYMB) && (g._SYMBptr->sommet==at_equal);
  }

  gen apply_to_equal(const gen & g,const gen_op & f){
    if (g.type!=_SYMB || g._SYMBptr->sommet!=at_equal || g._SYMBptr->feuille.type!=_VECT)
      return f(g);
    vecteur & v=*g._SYMBptr->feuille._VECTptr;
    if (v.empty())
      setsizeerr();
    return symbolic(at_equal,makevecteur(f(v.front()),f(v.back())));
  }

  gen apply_to_equal(const gen & g,gen (* f) (const gen &, GIAC_CONTEXT),GIAC_CONTEXT){
    if (g.type!=_SYMB || g._SYMBptr->sommet!=at_equal || g._SYMBptr->feuille.type!=_VECT)
      return f(g,contextptr);
    vecteur & v=*g._SYMBptr->feuille._VECTptr;
    if (v.empty())
      setsizeerr();
    return symbolic(at_equal,makevecteur(f(v.front(),contextptr),f(v.back(),contextptr)));
  }

  // one arg
  gen _id(const gen & args){
    return args;
  }
  extern partial_derivative_onearg D_at_id;
  const string _id_s("id");
  unary_function_unary __id(&_id,&D_at_id,_id_s);
  unary_function_ptr at_id (&__id);
  partial_derivative_onearg D_at_id(at_one);


  symbolic symb_not(const gen & args){
    return symbolic(at_not,args);
  }
  gen _not(const gen & args){
    return !args;
  }
  const string _not_s("not");
  unary_function_unary __not(&_not,_not_s);
  unary_function_ptr at_not (&__not);

  symbolic symb_neg(const gen & args){
    return symbolic(at_neg,args);
  }
  gen _neg(const gen & args){
    return -args;
  }
  extern partial_derivative_onearg D_at_neg;
  const string _neg_s("-");
  unary_function_unary __neg(&_neg,&D_at_neg,_neg_s);
  unary_function_ptr at_neg (&__neg);
  partial_derivative_onearg D_at_neg(at_neg);

  symbolic symb_inv(const gen & a){
    return symbolic(at_inv,a);
  }
  gen _inv(const gen & args){
    if ((args.type!=_VECT) || is_squarematrix(args))
        return inv(args);
    iterateur it=args._VECTptr->begin(), itend=args._VECTptr->end();
    gen prod(1);
    for (;it!=itend;++it)
      prod = prod * (*it);
    return inv(prod);
  }
  const string _inv_s("inv");
  unary_function_unary __inv(&_inv,_inv_s);
  unary_function_ptr at_inv (&__inv);

  symbolic symb_ln(const gen & e){
    return symbolic(at_ln,e);
  }

  gen ln(const gen & e){
    if (e.type==_DOUBLE_){
      if (e._DOUBLE_val==0)
	return minus_inf;
      if (e._DOUBLE_val>0)
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_log(e._DOUBLE_val);
#else
	return std::log(e._DOUBLE_val);
#endif
      else
#ifdef _SOFTMATH_H
	return M_PI*cst_i+std::giac_gnuwince_log(-e._DOUBLE_val);
#else
	return M_PI*cst_i+std::log(-e._DOUBLE_val);
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->log();
    if (e.type==_CPLX){ 
      if (e.subtype)
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_log(gen2complex_d(e));
#else
	return std::log(gen2complex_d(e));
#endif
      if (e._CPLXptr->type==_REAL)
	return ln(abs(e,0))+cst_i*arg(e,0);
      if (is_zero(*e._CPLXptr)){
	if (is_one(*(e._CPLXptr+1)))
	  return cst_i*cst_pi_over_2;
	if (is_minus_one(*(e._CPLXptr+1)))
	  return -cst_i*cst_pi_over_2;
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_ln,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::ln);
    if (is_zero(e))
      return minus_inf;
    if (is_one(e))
      return 0;
    if (is_minus_one(e))
      return cst_i*cst_pi;
    if ( (e==undef) || (e==unsigned_inf) || (e==plus_inf))
      return e;
    if (e==e__IDNT_e)
      return plus_one;
    if (is_equal(e))
      return apply_to_equal(e,ln);
    if (e.type==_SYMB && e._SYMBptr->sommet==at_exp){ 
      context tmp;
      if (is_real(e._SYMBptr->feuille,&tmp) ) 
	return e._SYMBptr->feuille;
    }
    return symb_ln(e);
  }
  gen log(const gen & e){
    return ln(e);
  }
  const string _ln_s("log"); // Using C notation, log for natural
  gen d_ln(const gen & args){
    return inv(args);
  }
  partial_derivative_onearg D_at_ln(&d_ln);
  unary_function_unary __ln(&giac::ln,&D_at_ln,_ln_s);
  unary_function_ptr at_ln (&__ln);

  gen log10(const gen & e){
    if (e.type==_DOUBLE_ && e._DOUBLE_val>0 ){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_log10(e._DOUBLE_val);
#else
      return std::log10(e._DOUBLE_val);
#endif
    }
    if ( e.type==_DOUBLE_ || (e.type==_CPLX && e.subtype)){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_log(gen2complex_d(e))/M_LN10;
#else
      return std::log(gen2complex_d(e))/M_LN10;
#endif
    }
    return rdiv(ln(e),ln(10));
  }
  const string _log10_s("log10"); // Using C notation, log for natural
  gen d_log10(const gen & args){
    return inv(args*ln(10));
  }
  partial_derivative_onearg D_at_log10(&d_log10);
  unary_function_unary __log10(&giac::log10,&D_at_log10,_log10_s);
  unary_function_ptr at_log10 (&__log10);

  gen alog10(const gen & e){
    if (is_squarematrix(e))
      return analytic_apply(at_alog10,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::alog10);
    if (is_equal(e))
      return apply_to_equal(e,alog10);
    return pow(gen(10),e);
  }
  const string _alog10_s("alog10"); 
  unary_function_unary __alog10(&giac::alog10,_alog10_s);
  unary_function_ptr at_alog10 (&__alog10);

  symbolic symb_atan(const gen & e){
    return symbolic(at_atan,e);
  }
  gen atanasln(const gen & e){
    return plus_one_half*cst_i*ln(rdiv(cst_i+e,cst_i-e));
  }
  gen atan(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      double res=std::giac_gnuwince_atan(e._DOUBLE_val);
#else
      double res=std::atan(e._DOUBLE_val);
#endif
      if (angle_radian(contextptr)) 
	return res;
      else
	return res*rad2deg_d;
    }
    if (e.type==_REAL)
      return e._REALptr->atan();
    if ( (e.type==_CPLX) && (e.subtype || e._CPLXptr->type==_REAL)){
      if (angle_radian(contextptr)) 
	return no_context_evalf(atanasln(e));
      else
	return no_context_evalf(atanasln(e))*gen(rad2deg_d);
    }
    if (is_squarematrix(e))
      return analytic_apply(at_atan,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,giac::atan,contextptr);
    if (is_zero(e))
      return e;
    if (is_one(e)){
      if (angle_radian(contextptr)) 
	return rdiv(cst_pi,4);
      return 45;
    }
    if (is_minus_one(e)){
      if (angle_radian(contextptr)) 
	return rdiv(-cst_pi,4);
      return -45;
    }
    if (e==plus_sqrt3_3){
      if (angle_radian(contextptr)) 
	return rdiv(cst_pi,6);
      return 30;
    }
    if (e==plus_sqrt3){
      if (angle_radian(contextptr)) 
	return rdiv(cst_pi,3);
      return 60;
    }
    if (e==plus_inf){
      if (angle_radian(contextptr)) 
	return cst_pi_over_2;
      return 90;
    }
    if (e==minus_inf){
      if (angle_radian(contextptr)) 
	return -cst_pi_over_2;
      return -90;
    }
    if ((e==undef)||(e==unsigned_inf))
      return undef;
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg))
      return -atan(e._SYMBptr->feuille,contextptr);
    if ( (e.type==_INT_) && (e.val<0) )
      return -atan(-e,contextptr);
    if (is_equal(e))
      return apply_to_equal(e,atan,contextptr);
    if (e.is_symb_of_sommet(at_tan)){
      gen tmp=cst_pi;
      if (!angle_radian(contextptr))
	tmp=180;
      return e._SYMBptr->feuille-_floor(e._SYMBptr->feuille/tmp+plus_one_half,contextptr)*tmp;
    }
    return symb_atan(e);
  }
  gen d_atan(const gen & args,GIAC_CONTEXT){
    gen g=inv(1+pow(args,2));
    if (angle_radian(contextptr))
      return g;
    return g*deg2rad_e;
  }
  partial_derivative_onearg D_at_atan(&d_atan);
  gen taylor_atan (const gen & lim_point,const int ordre,const unary_function_ptr & f, int direction,gen & shift_coeff,GIAC_CONTEXT){
    if (ordre<0)
      return 0; // no symbolic preprocessing
    shift_coeff=0;
    if (!is_inf(lim_point))
      return taylor(lim_point,ordre,f,0,shift_coeff,contextptr);
    vecteur v;
    identificateur x(" ");
    taylor(atan(x,contextptr),x,0,ordre,v,contextptr);
    v=negvecteur(v);
    v.front()=atan(lim_point,contextptr);
    return v;
  }
  const string _atan_s("atan");
  unary_function_eval __atan(&giac::atan,&D_at_atan,&taylor_atan,_atan_s);
  unary_function_ptr at_atan (&__atan);

  symbolic symb_exp(const gen & e){
    return symbolic(at_exp,e);
  }
  gen numeric_matrix_exp(const gen & e,double eps){
    gen res=midn(e._VECTptr->size());
    gen eee(e);
    for (double j=2;linfnorm(eee,0)._DOUBLE_val>eps;++j){
      res = res + eee;
      eee = gen(1/j) * eee * e ; 
    }
    return res;
  }

  gen exp(const gen & e){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_exp(e._DOUBLE_val);
#else
      return std::exp(e._DOUBLE_val);
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->exp();
    if (e.type==_CPLX){ 
      if (e.subtype){
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_exp(gen2complex_d(e));
#else
	return std::exp(gen2complex_d(e));
#endif
      }
      if (e._CPLXptr->type==_REAL){
	context * contextptr=new context();
	return exp(*e._CPLXptr)*(cos(*(e._CPLXptr+1),contextptr)+cst_i*sin(*(e._CPLXptr+1),contextptr));
	delete contextptr;
      }
    }
    if (e.type==_VECT){
      if (is_squarematrix(e)){ 
	// check for numeric entries -> numeric exp
	if (is_fully_numeric(e))
	  return numeric_matrix_exp(e,epsilon(0)); // GIAC_CONTEXT
	return analytic_apply(at_exp,*e._VECTptr,0);
      }
      return apply(e,giac::exp);
    }
    if (is_zero(e))
      return 1;
    if ((e==undef) || (e==plus_inf))
      return e;
    if (e==unsigned_inf)
      return undef;
    if (e==minus_inf)
      return 0;
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_ln))
      return e._SYMBptr->feuille;
    int k;
    if (contains(e,_IDNT_pi)){ // if (!approx_mode(0)) // FIXME GIAC_CONTEXT 
      if (is_multiple_of_pi_over_12(e*cst_i,k) )
	return (*table_cos[k])+cst_i*(*table_cos[(k+6)%24]);
      else {
	gen kk;
	kk=normal(rdiv(e*cst_i,cst_pi));
	if (is_assumed_integer(kk)){ 
	  if (is_assumed_integer(normal(rdiv(kk,plus_two))))
	    return plus_one;
	  else
	    return pow(minus_one,kk);
	}
	int n,d,q,r;
	if (is_rational(kk,n,d)){
	  q=-n/d;
	  r=-n%d;
	  if (q%2)
	    q=-1;
	  else
	    q=1;
	  return q*symb_exp(r*(cst_pi*cst_i/d));
	}
      }
    }
    if (is_equal(e))
      return apply_to_equal(e,exp);
    return symb_exp(e);
  }
  partial_derivative_onearg D_at_exp(&giac::exp);
  gen taylor_exp (const gen & lim_point,const int ordre,const unary_function_ptr & f,int direction,gen & shift_coeff,GIAC_CONTEXT){
    if (ordre<0)
      return 0; // no symbolic preprocessing
    shift_coeff=0;
    gen image=f(lim_point,contextptr); 
    vecteur v(1,image);
    if (is_undef(image))
      return v;
    gen factorielle(1);
    for (int i=1;i<=ordre;++i,factorielle=factorielle*gen(i))
      v.push_back(rdiv(image,factorielle));
    v.push_back(undef);
    return v;
  }
  const string _exp_s("exp");
  string texprintasexp(const gen & g,const string & s){
    return "e^{"+gen2tex(g)+"}";
  }
  unary_function_unary __exp(&giac::exp,&D_at_exp,&taylor_exp,_exp_s,0,&texprintasexp);
  unary_function_ptr at_exp (&__exp);

  symbolic symb_sqrt(const gen & e){
    return symbolic(at_sqrt,e);
  }
  gen sqrt(const gen & e){
    if (e.type==_DOUBLE_){
      if (e._DOUBLE_val>=0){
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_sqrt(e._DOUBLE_val);
#else
	return std::sqrt(e._DOUBLE_val);
#endif
      }
      else
#ifdef _SOFTMATH_H
	return gen(0.0,std::giac_gnuwince_sqrt(-e._DOUBLE_val));
#else
	return gen(0.0,std::sqrt(-e._DOUBLE_val));
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->sqrt();
    if (e.type==_CPLX){
      if (e.subtype){
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_sqrt(gen2complex_d(e));
#else
	return std::sqrt(gen2complex_d(e));
#endif
      }
      // sqrt of an exact complex number
      gen x(re(e,0)),y(im(e,0));
      if (is_zero(y))
	return sqrt(x);
      gen rho=sqrt(x*x+y*y);
      gen realpart=sqrt(rdiv(x+rho,plus_two));
      return realpart+cst_i*rdiv(y,plus_two*realpart);
    }
    if (e.type==_VECT)
      return apply(e,giac::sqrt);
    if ((e==undef) || (e==plus_inf) || (e==unsigned_inf))
      return e;
    if (is_perfect_square(e))
      return isqrt(e);
    if (e.type==_INT_ || e.type==_ZINT){ 
      // factorization 
      gen e_copy;
      bool pos=ck_is_positive(e,0);
      if (!pos)
	e_copy=-e;
      else
        e_copy=e;
      vecteur u(pfacprem(e_copy));
      gen simpl(e_copy),doubl(1),f;
      int m,k;
      const_iterateur it=u.begin(),itend=u.end();
      for (;it!=itend;++it){
	f=it->val;
	++it;
	m=it->val;
	if (m%2)
	  simpl = simpl*f;
	for (k=0;k<m/2;++k)
	  doubl = doubl*f;
      }
      if (!pos)
	return cst_i*doubl*pow(simpl,plus_one_half);
      else
	return doubl*pow(simpl,plus_one_half);
    }
    if (e.is_symb_of_sommet(at_inv))
      return inv(sqrt(e._SYMBptr->feuille));
    return pow(e,plus_one_half);
  }
  gen d_sqrt(const gen & e){
    return inv(gen(2)*sqrt(e));
  }
  partial_derivative_onearg D_at_sqrt(&d_sqrt);
  const string _sqrt_s("sqrt");
  string texprintassqrt(const gen & g,const string & s){
    return "\\sqrt{"+gen2tex(g)+"}";
  }
  unary_function_unary __sqrt(&giac::sqrt,&D_at_sqrt,_sqrt_s,0,&texprintassqrt);
  unary_function_ptr at_sqrt (&__sqrt);

  gen _sq(const gen & e){
    return pow(e,2);
  }
  gen d_sq(const gen & e){
    return gen(2)*e;
  }
  partial_derivative_onearg D_at_sq(&d_sq);
  const string _sq_s("sq");
  string texprintassq(const gen & g,const string & s){
    return gen2tex(g)+"^2";
  }
  unary_function_unary __sq(&giac::_sq,&D_at_sq,_sq_s);
  unary_function_ptr at_sq (&__sq);

  symbolic symb_cos(const gen & e){
    return symbolic(at_cos,e);
  }
  gen cos(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
      double d;
      if (angle_radian(contextptr)) 
	d=e._DOUBLE_val;
      else
	d=e._DOUBLE_val*deg2rad_d;
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_cos(d);
#else
      return std::cos(d);
#endif
    }	
    if (e.type==_REAL)
      return e._REALptr->cos(); // FIXME GIAC_CONTEXT
    if (e.type==_CPLX){ 
      if (e.subtype){
	complex<double> d;
	if (angle_radian(contextptr)) 
	  d=gen2complex_d(e);
	else
	  d=gen2complex_d(e)*deg2rad_d;
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_cos(d);
#else
	return std::cos(d);
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen e1=e;
	if (!angle_radian(contextptr)) 
	  e1=e*deg2rad_g;
	gen e2=im(e1,contextptr);
	e1=re(e1,contextptr);
	bool tmp=angle_radian(contextptr);
	angle_radian(true,contextptr);
	e1= cos(e1,contextptr)*cosh(e2)-cst_i*sinh(e2)*sin(e1,contextptr);
	angle_radian(tmp,contextptr);
	return e1;
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_cos,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,giac::cos,contextptr);
    if (is_zero(e))
      return 1;
    if ( (e.type==_INT_) && (e.val<0) )
      return cos(-e,contextptr);
    if ((e==undef) || is_inf(e))
      return undef;
    int k;
    if (!approx_mode(contextptr)){ 
      if (is_multiple_of_pi_over_12(e,k,angle_radian(contextptr))) 
	return *table_cos[k];
      else {
	gen kk;
	if (angle_radian(contextptr)) 
	  kk=normal(rdiv(e,cst_pi));
	else
	  kk=normal(rdiv(e,180));
	if (is_assumed_integer(kk)){
	  if (is_assumed_integer(normal(rdiv(kk,plus_two))))
	    return plus_one;
	  else
	    return pow(minus_one,kk);
	}
	int n,d,q,r;
	if (is_rational(kk,n,d)){
	  q=n/d;
	  r=n%d;
	  if (r>d/2){
	    r -= d;
	    ++q;
	  }
	  if (q%2)
	    q=-1;
	  else
	    q=1;
	  if (r<0)
	    r=-r;
	  if (!(d%2) && d%4){ 
	    d=d/2; // cos(r/(2*d)*pi) = sin(pi/2(1-r/d))
	    if (angle_radian(contextptr)) 
	      return q*sin((r-d)/2*cst_pi/d,contextptr);
	    else 
	      return q*sin(rdiv((r-d)*90,d),contextptr);
	  }
	  if (angle_radian(contextptr)) 
	    return q*symb_cos(r*cst_pi/d);
	  else
	    return q*symb_cos(rdiv(r*180,d));
	}
      }
    }
    if (e.type==_SYMB) {
      unary_function_ptr u=e._SYMBptr->sommet;
      gen f=e._SYMBptr->feuille;
      if (u==at_neg)
	return cos(f,contextptr);
      if (u==at_acos)
	return f;
      if (u==at_asin)
	return sqrt(1-pow(f,2));
      if (u==at_atan)
	return sqrt(inv(pow(f,2)+1));
    }
    if (is_equal(e))
      return apply_to_equal(e,cos,contextptr);
    return symb_cos(e);
  }
  gen d_cos(const gen & e ,GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return -(sin(e,contextptr));
    else
      return -deg2rad_e*sin(e,contextptr);
  }
  partial_derivative_onearg D_at_cos(d_cos);
  const string _cos_s("cos");
  unary_function_eval __cos(&giac::cos,&D_at_cos,_cos_s);
  unary_function_ptr at_cos (&__cos);

  symbolic symb_sin(const gen & e){
    return symbolic(at_sin,e);
  }
  gen sin(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
      double d;
      if (angle_radian(contextptr)) 
	d=e._DOUBLE_val;
      else
	d=e._DOUBLE_val*deg2rad_d;
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_sin(d);
#else
      return std::sin(d);
#endif
    }	
    if (e.type==_REAL)
      return e._REALptr->sin();
    if (e.type==_CPLX){ 
      if (e.subtype){
	complex<double> d;
	if (angle_radian(contextptr)) 
	  d=gen2complex_d(e);
	else
	  d=gen2complex_d(e)*deg2rad_d;
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_sin(d);
#else
	return std::sin(d);
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen e1=e;
	if (!angle_radian(contextptr)) 
	  e1=e*deg2rad_g;
	gen e2=im(e1,contextptr);
	e1=re(e1,contextptr);
	bool tmp=angle_radian(contextptr);
	angle_radian(true,contextptr);
	gen res=sin(e1,contextptr)*cosh(e2)+cst_i*sinh(e2)*cos(e1,contextptr);
	angle_radian(tmp,contextptr);
	return res;
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_sin,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,giac::sin,contextptr);
    if (is_zero(e))
      return e;
    if ( (e.type==_INT_) && (e.val<0) )
      return -sin(-e,contextptr);
    if ((e==undef) || is_inf(e))
      return undef;
    int k;
    if (!approx_mode(contextptr)){ 
      if (is_multiple_of_pi_over_12(e,k,angle_radian(contextptr)))
	return *table_cos[(k+18)%24];
      else {
	gen kk;
	if (angle_radian(contextptr))
	  kk=normal(rdiv(e,cst_pi));
	else
	  kk=normal(rdiv(e,180));
	if (is_assumed_integer(kk))
	  return zero;
	int n,d,q,r;
	if (is_rational(kk,n,d)){
	  q=n/d;
	  r=n%d;
	  if (r>d/2){
	    r -= d;
	    ++q;
	  }
	  if (q%2)
	    q=-1;
	  else
	    q=1;
	  if (r<0){
	    r=-r;
	    q=-q;
	  }
	  if (!(d%2) && d%4){ 
	    d=d/2; // sin(r/(2*d)*pi) = cos(pi/2(1-r/d))
	    if (angle_radian(contextptr))
	      return q*cos((r-d)/2*cst_pi/d,contextptr);
	    else
	      return q*cos(rdiv((r-d)*90,d),contextptr);
	  }
	  if (angle_radian(contextptr)) 
	    return q*symb_sin(r*cst_pi/d);
	  else
	    return q*symb_sin(rdiv(r*180,d));
	}
      }
    }
    if (e.type==_SYMB) {
      unary_function_ptr u=e._SYMBptr->sommet;
      gen f=e._SYMBptr->feuille;
      if (u==at_neg)
	return -sin(f,contextptr);
      if (u==at_asin)
	return f;
      if (u==at_acos)
	return sqrt(1-pow(f,2));
      if (u==at_atan)
	return rdiv(f,sqrt(pow(f,2)+1));
    }
    if (is_equal(e))
      return apply_to_equal(e,sin,contextptr);
    return symb_sin(e);
  }
  gen d_sin(const gen & g,GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return cos(g,contextptr);
    else
      return deg2rad_e*cos(g,contextptr);
  }
  const string _sin_s("sin");
  partial_derivative_onearg D_at_sin(&d_sin);
  unary_function_eval __sin(&giac::sin,&D_at_sin,_sin_s);
  unary_function_ptr at_sin (&__sin);

  symbolic symb_tan(const gen & e){
    return symbolic(at_tan,e);
  }
  gen tan(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
      double d;
      if (angle_radian(contextptr)) 
	d=e._DOUBLE_val;
      else
	d=e._DOUBLE_val*deg2rad_d;
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_tan(d);
#else
      return std::tan(d);
#endif
    }	
    if (e.type==_REAL)
      return e._REALptr->tan();
    if (e.type==_CPLX){ 
      if (e.subtype){
	complex<double> c(gen2complex_d(e));
	if (!angle_radian(contextptr)) 
	  c *= deg2rad_d;
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_tan(c);
#else
	return std::sin(c)/std::cos(c);
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen e1=e;
	if (!angle_radian(contextptr)) 
	  e1=e*deg2rad_g;
	gen e2=im(e1,contextptr);
	e1=re(e1,contextptr);
	bool tmp=angle_radian(contextptr);
	angle_radian(true,contextptr);
	e1=tan(e1,contextptr);
	angle_radian(tmp,contextptr);
	e2=cst_i*tanh(e2);
	return (e1+e2)/(1-e1*e2);
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_tan,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,contextptr,giac::tan);
    if (is_zero(e))
      return e;
    if ((e==undef) || is_inf(e))
      return undef;
    if ( (e.type==_INT_) && (e.val<0) )
      return -tan(-e,contextptr);
    int k;
    if (!approx_mode(contextptr)){ 
      if (is_multiple_of_pi_over_12(e,k,angle_radian(contextptr))) 
	return *table_tan[(k%12)];
      else {
	gen kk;
	if (angle_radian(contextptr)) 
	  kk=normal(rdiv(e,cst_pi));
	else
	  kk=normal(rdiv(e,180));
	if (is_assumed_integer(kk))
	  return zero;
	int n,d;
	if (is_rational(kk,n,d)){
	  if (angle_radian(contextptr)) 
	    return symb_tan((n%d)*inv(d)*cst_pi);
	  else
	    return symb_tan(rdiv((n%d)*180,d));
	}
      }
    }
    if (e.type==_SYMB) {
      unary_function_ptr u=e._SYMBptr->sommet;
      gen f=e._SYMBptr->feuille;
      if (u==at_neg)
	return -tan(f,contextptr);
      if (u==at_atan)
	return f;
      if (u==at_acos)
	return rdiv(sqrt(1-pow(f,2)),f);
      if (u==at_asin)
	return rdiv(f,sqrt(1-pow(f,2)));
    }
    if (is_equal(e))
      return apply_to_equal(e,tan,contextptr);
    return symb_tan(e);
  }
  gen d_tan(const gen & e,GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return 1+pow(tan(e,contextptr),2);
    else
      return deg2rad_e*(1+pow(tan(e,contextptr),2));
  }
  partial_derivative_onearg D_at_tan(&d_tan);
  const string _tan_s("tan");
  unary_function_eval __tan(&giac::tan,&D_at_tan,_tan_s);
  unary_function_ptr at_tan (&__tan);

  symbolic symb_asin(const gen & e){
    return symbolic(at_asin,e);
  }
  gen asinasln(const gen & x){
    return cst_i*ln(sqrt(x*x-1)+x)+cst_pi_over_2;
  }
  gen asin(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      double d= std::giac_gnuwince_asin(e._DOUBLE_val);
#else
      double d=std::asin(e._DOUBLE_val);
#endif
      if (angle_radian(contextptr)) 
	return d;
      else
	return d*rad2deg_d;
    }
    if (e.type==_REAL)
      return e._REALptr->asin();
    if ( (e.type==_CPLX) && (e.subtype || e._CPLXptr->type==_REAL)){
      if (angle_radian(contextptr)) 
	return no_context_evalf(asinasln(e));
      else
	return no_context_evalf(asinasln(e))*gen(rad2deg_d);
    }
    if (is_squarematrix(e))
      return analytic_apply(at_asin,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,giac::asin,contextptr);
    if (is_zero(e))
      return e;
    if (is_one(e)){
      if (angle_radian(contextptr))
	return cst_pi_over_2;
      return 90;
    }
    if (e==plus_sqrt3_2){
      if (angle_radian(contextptr))
	return rdiv(cst_pi,3);
      return 60;
    }
    if (e==plus_sqrt2_2){
      if (angle_radian(contextptr)) 
	return rdiv(cst_pi,4);
      return 45;
    }
    if (e==plus_one_half){
      if (angle_radian(contextptr)) 
	return rdiv(cst_pi,6);
      return 30;
    }
    if (e==undef)
      return e;
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg))
      return -asin(e._SYMBptr->feuille,contextptr);
    if ( (e.type==_INT_) && (e.val<0) )
      return -asin(-e,contextptr);
    if (is_equal(e))
      return apply_to_equal(e,asin,contextptr);
    return symb_asin(e);
  }
  gen d_asin(const gen & args,GIAC_CONTEXT){
    gen g=inv(sqrt(1-pow(args,2)));
    if (angle_radian(contextptr)) 
      return g;
    else
      return g*deg2rad_e;
  }
  gen taylor_asin (const gen & lim_point,const int ordre,const unary_function_ptr & f, int direction,gen & shift_coeff,GIAC_CONTEXT){
    if (ordre<0)
      return 0; // no symbolic preprocessing
    if (is_one(lim_point)){
      shift_coeff=plus_one_half;
      identificateur x(" "); vecteur v;
      taylor(pow(2+x,minus_one_half),x,0,ordre,v,contextptr);
      // integration with shift 
      v=integrate(v,shift_coeff);
      if (!direction)
	direction=1;
      return normal((gen(-direction)*cst_i)*gen(v));
    }
    if (is_minus_one(lim_point)){
      shift_coeff=plus_one_half;
      identificateur x(" "); vecteur v;
      taylor(pow(2-x,minus_one_half),x,0,ordre,v,contextptr);
      // integration with shift 
      v=integrate(v,shift_coeff);
      return v;
    }
    return taylor(lim_point,ordre,f,direction,shift_coeff,contextptr);
  }
  partial_derivative_onearg D_at_asin(&d_asin);
  const string _asin_s("asin");
  unary_function_eval __asin(&giac::asin,&D_at_asin,&taylor_asin,_asin_s);
  unary_function_ptr at_asin (&__asin);

  symbolic symb_acos(const gen & e){
    return symbolic(at_acos,e);
  }
  gen acos(const gen & e,GIAC_CONTEXT){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      double d= std::giac_gnuwince_acos(e._DOUBLE_val);
#else
      double d=std::acos(e._DOUBLE_val);
#endif
      if (angle_radian(contextptr)) 
	return d;
      else
	return d*rad2deg_d;
    }
    if (e.type==_REAL)
      return e._REALptr->acos();
    if ( (e.type==_CPLX) && (e.subtype || e._CPLXptr->type==_REAL)){
      gen res=-cst_i*no_context_evalf(ln(sqrt(e*e-1)+e));
      if (angle_radian(contextptr)) 
	return res;
      else
	return res*gen(rad2deg_d);
    }
    if (is_squarematrix(e))
      return analytic_apply(at_acos,*e._VECTptr,contextptr);
    if (e.type==_VECT)
      return apply(e,giac::acos,contextptr);
    if (is_equal(e))
      return apply_to_equal(e,acos,contextptr);
    gen g=asin(e,contextptr);
    if ( (g.type==_SYMB) && (g._SYMBptr->sommet==at_asin) )
      return symb_acos(e);
    if (angle_radian(contextptr)) 
      return normal(cst_pi_over_2-asin(e,contextptr));
    else
      return 90-asin(e,contextptr);
  }
  gen d_acos(const gen & args,GIAC_CONTEXT){
    gen g= -inv(sqrt(1-pow(args,2)));
    if (angle_radian(contextptr)) 
      return g;
    else
      return g*deg2rad_e;
  }
  partial_derivative_onearg D_at_acos(&d_acos);
  const string _acos_s("acos");
  unary_function_eval __acos(&giac::acos,&D_at_acos,_acos_s);
  unary_function_ptr at_acos (&__acos);

  symbolic symb_sinh(const gen & e){
    return symbolic(at_sinh,e);
  }
  gen sinh(const gen & e){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_sinh(e._DOUBLE_val);
#else
      return std::sinh(e._DOUBLE_val);
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->sinh();
    if (e.type==_CPLX){
      if (e.subtype){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_sinh(gen2complex_d(e));
#else
      return std::sinh(gen2complex_d(e));
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen g=exp(e);
	return (g-inv(g))/2;
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_sinh,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::sinh);
    if ( is_zero(e) || (e==undef) || (is_inf(e)))
      return e;
    if (is_equal(e))
      return apply_to_equal(e,sinh);
    return symb_sinh(e);
  }
  gen d_at_sinh(const gen & e){
    return cosh(e);
  }
  partial_derivative_onearg D_at_sinh(&d_at_sinh);
  const string _sinh_s("sinh");
  unary_function_unary __sinh(&giac::sinh,&D_at_sinh,_sinh_s);
  unary_function_ptr at_sinh (&__sinh);

  symbolic symb_cosh(const gen & e){
    return symbolic(at_cosh,e);
  }
  gen cosh(const gen & e){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_cosh(e._DOUBLE_val);
#else
      return std::cosh(e._DOUBLE_val);
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->cosh();
    if (e.type==_CPLX){
      if (e.subtype){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_cosh(gen2complex_d(e));
#else
      return std::cosh(gen2complex_d(e));
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen g=exp(e);
	return (g+inv(g))/2;
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_cosh,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::cosh);
    if (is_zero(e))
      return 1;
    if (e==undef)
      return e;
    if (is_inf(e))
      return plus_inf;
    if (is_equal(e))
      return apply_to_equal(e,cosh);
    return symb_cosh(e);
  }
  partial_derivative_onearg D_at_cosh(at_sinh);
  const string _cosh_s("cosh");
  unary_function_unary __cosh(&giac::cosh,&D_at_cosh,_cosh_s);
  unary_function_ptr at_cosh (&__cosh);

  symbolic symb_tanh(const gen & e){
    return symbolic(at_tanh,e);
  }
  gen tanh(const gen & e){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_tanh(e._DOUBLE_val);
#else
      return std::tanh(e._DOUBLE_val);
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->tanh();
    if (e.type==_CPLX){
      if (e.subtype){
	complex<double> c(gen2complex_d(e));
#ifdef _SOFTMATH_H
	return std::giac_gnuwince_tanh(c);
#else
	return std::sinh(c)/std::cosh(c);
#endif
      }
      if (e._CPLXptr->type==_REAL){
	gen g=exp(2*e);
	return (g+1)/(g-1);
      }
    }
    if (is_squarematrix(e))
      return analytic_apply(at_tanh,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::tanh);
    if (is_zero(e) || (e==undef) || (e==unsigned_inf))
      return e;
    if (e==plus_inf)
      return 1;
    if (e==minus_inf)
      return -1;
    if (is_equal(e))
      return apply_to_equal(e,tanh);
    return symbolic(at_tanh,e);
  }
  gen d_tanh(const gen & e){
    return 1-pow(tanh(e),2);
  }
  partial_derivative_onearg D_at_tanh(&d_tanh);
  const string _tanh_s("tanh");
  unary_function_unary __tanh(&giac::tanh,&D_at_tanh,_tanh_s);
  unary_function_ptr at_tanh (&__tanh);

  symbolic symb_asinh(const gen & e){
    return symbolic(at_asinh,e);
  }
  gen asinhasln(const gen & x){
    return ln(x+sqrt(x*x+1));
  }
  gen asinh(const gen & e){
    if (e.type==_DOUBLE_)
      return asinhasln(e);
    if (e.type==_REAL)
      return e._REALptr->asinh();
    if ( (e.type==_CPLX) && (e.subtype || e._CPLXptr->type==_REAL))
      return no_context_evalf(asinhasln(e));
    if (is_squarematrix(e)){
      context tmp;
      return analytic_apply(at_asinh,*e._VECTptr,&tmp); 
    }
    if (e.type==_VECT)
      return apply(e,giac::asinh);
    if (is_zero(e) || is_inf(e))
      return e;
    if (e==undef)
      return e;
    if (is_equal(e))
      return apply_to_equal(e,asinh);
    return ln(e+sqrt(pow(e,2)+1));
    // return symbolic(at_asinh,e);
  }
  gen d_asinh(const gen & args){
    return inv(sqrt(pow(args,2)+1));
  }
  partial_derivative_onearg D_at_asinh(&d_asinh);
  const string _asinh_s("asinh");
  unary_function_unary __asinh(&giac::asinh,&D_at_asinh,_asinh_s);
  unary_function_ptr at_asinh (&__asinh);

  symbolic symb_acosh(const gen & e){
    return symbolic(at_cosh,e);
  }
  gen acoshasln(const gen & x){
    return ln(x+sqrt(x+1)*sqrt(x-1));
  }
  gen acosh(const gen & e){
    if (e.type==_DOUBLE_)
      return acoshasln(e);
    if (e.type==_REAL)
      return e._REALptr->acosh();
    if ( (e.type==_CPLX) && (e.subtype|| e._CPLXptr->type==_REAL))
      return no_context_evalf(acoshasln(e));
    if (is_squarematrix(e))
      return analytic_apply(at_acosh,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::acosh);
    if (is_one(e))
      return 0;
    if (e==plus_inf)
      return plus_inf;
    if (e==undef)
      return e;
    if (is_equal(e))
      return apply_to_equal(e,acosh);
    return ln(e+sqrt(pow(e,2)-1));
    // return symbolic(at_acosh,e);
  }
  gen d_acosh(const gen & args){
    return inv(sqrt(pow(args,2)-1));
  }
  partial_derivative_onearg D_at_acosh(&d_acosh);
  const string _acosh_s("acosh");
  unary_function_unary __acosh(&giac::acosh,&D_at_acosh,_acosh_s);
  unary_function_ptr at_acosh (&__acosh);

  symbolic symb_atanh(const gen & e){
    return symbolic(at_atanh,e);
  }
  gen atanh(const gen & e){
    if (e.type==_DOUBLE_){
#ifdef _SOFTMATH_H
      return std::giac_gnuwince_log((1+e._DOUBLE_val)/(1-e._DOUBLE_val))/2;
#else
      return std::log((1+e._DOUBLE_val)/(1-e._DOUBLE_val))/2;
#endif
    }
    if (e.type==_REAL)
      return e._REALptr->atanh();
    if ( (e.type==_CPLX) && (e.subtype || e._CPLXptr->type==_REAL))
      return no_context_evalf(rdiv(ln(rdiv(1+e,1-e)),plus_two));
    if (is_squarematrix(e))
      return analytic_apply(at_atanh,*e._VECTptr,0);
    if (e.type==_VECT)
      return apply(e,giac::atanh);
    if (is_zero(e))
      return e;
    if (is_one(e))
      return plus_inf;
    if (is_minus_one(e))
      return minus_inf;
    if (e==undef)
      return e;
    if (is_equal(e))
      return apply_to_equal(e,atanh);
    return rdiv(ln(rdiv(1+e,1-e)),plus_two);
    // return symbolic(at_atanh,e);
  }
  gen d_atanh(const gen & args){
    return inv(1-pow(args,2));
  }
  partial_derivative_onearg D_at_atanh(&d_atanh);
  const string _atanh_s("atanh");
  unary_function_unary __atanh(&giac::atanh,&D_at_atanh,_atanh_s);
  unary_function_ptr at_atanh (&__atanh);

  symbolic symb_quote(const gen & arg){
    return symbolic(at_quote,arg);
  }
  gen quote(const gen & arg){
    return symb_quote(arg);
  }
  partial_derivative_onearg D_at_quote(&quote);
  const string _quote_s("quote");
  unary_function_unary __quote(&giac::quote,&D_at_quote,_quote_s);
  unary_function_ptr at_quote (&__quote);

  symbolic symb_unquote(const gen & arg){
    return symbolic(at_unquote,arg);
  }
  gen unquote(const gen & arg,GIAC_CONTEXT){
    return eval(arg,contextptr);
  }
  partial_derivative_onearg D_at_unquote(&unquote);
  const string _unquote_s("unquote");
  unary_function_eval __unquote(&giac::unquote,&D_at_unquote,_unquote_s);
  unary_function_ptr at_unquote (&__unquote);

  symbolic symb_order_size(const gen & e){
    return symbolic(at_order_size,e);
  }
  gen order_size(const gen & arg){
    return symb_order_size(arg);
  }
  partial_derivative_onearg D_at_order_size(&order_size);
  const string _order_size_s("order_size");
  unary_function_unary __order_size(&giac::order_size,&D_at_order_size,_order_size_s);
  unary_function_ptr at_order_size (&__order_size);

  gen re(const gen & a,GIAC_CONTEXT){
    if (is_equal(a))
      return apply_to_equal(a,re,contextptr);
    return a.re(contextptr);
  }
  const string _re_s("re");
  string texprintasre(const gen & g,const string & s){
    return "\\Re("+gen2tex(g)+")";
  }
  unary_function_eval __re(&giac::re,_re_s,0,&texprintasre);
  unary_function_ptr at_re (&__re);

  gen im(const gen & a,GIAC_CONTEXT){
    if (is_equal(a))
      return apply_to_equal(a,im,contextptr);
    return a.im(contextptr);
  }
  const string _im_s("im");
  string texprintasim(const gen & g,const string & s){
    return "\\Im("+gen2tex(g)+")";
  }
  unary_function_eval __im(&giac::im,_im_s,0,&texprintasim);
  unary_function_ptr at_im (&__im);

  symbolic symb_conj(const gen & e){
    return symbolic(at_conj,e);
  }
  gen conj(const gen & a,GIAC_CONTEXT){
    if (is_equal(a))
      return apply_to_equal(a,conj,contextptr);
    return a.conj(contextptr);
  }
  const string _conj_s("conj");
  string texprintasconj(const gen & g,const string & s){
    return "\\overline{"+gen2tex(g)+"}";
  }
  unary_function_eval __conj(&giac::conj,_conj_s,0,&texprintasconj);
  unary_function_ptr at_conj (&__conj);

  gen _sign(const gen & g,GIAC_CONTEXT){
    return apply(g,contextptr,giac::sign);
  }
  symbolic symb_sign(const gen & e){
    return symbolic(at_sign,e);
  }
  const string _sign_s("sign");
  unary_function_eval __sign(_sign,_sign_s);
  unary_function_ptr at_sign (&__sign);

  gen _abs(const gen & args,GIAC_CONTEXT){
    return apply(args,contextptr,giac::abs);
  }
  symbolic symb_abs(const gen & e){
    return symbolic(at_abs,e);
  }
  gen taylor_abs (const gen & lim_point,const int ordre,const unary_function_ptr & f, int direction,gen & shift_coeff,GIAC_CONTEXT){
    if (ordre<0)
      return 0; // no symbolic preprocessing
    shift_coeff=0;
    if (is_positive(lim_point,contextptr) || (is_zero(lim_point) && direction==1))
      return makevecteur(lim_point,1);
    if (is_positive(-lim_point,contextptr) || (is_zero(lim_point) && direction==-1))
      return makevecteur(-lim_point,-1);
    setsizeerr("Taylor abs with unsigned limit");
  }
  const string _abs_s("abs");
  partial_derivative_onearg D_at_abs(at_sign);
  unary_function_eval __abs(&_abs,&D_at_abs,&taylor_abs,_abs_s);
  unary_function_ptr at_abs (&__abs);

  symbolic symb_arg(const gen & e){
    return symbolic(at_arg,e);
  }
  const string _arg_s("arg");
  unary_function_eval __arg(&giac::arg,_arg_s);
  unary_function_ptr at_arg (&__arg);

  symbolic symb_cyclotomic(const gen & e){
    return symbolic(at_cyclotomic,e);
  }
  gen _cyclotomic(const gen & a){
    if (a.type!=_INT_)
      return symb_cyclotomic(a);
    return cyclotomic(a.val);
  }
  const string _cyclotomic_s("cyclotomic");
  unary_function_unary __cyclotomic(&giac::_cyclotomic,_cyclotomic_s);
  unary_function_ptr at_cyclotomic (&__cyclotomic);

  string printassto(const gen & feuille,const string & sommetstr){
    if ( (feuille.type!=_VECT) || (feuille._VECTptr->size()!=2) )
      return sommetstr+'('+feuille.print()+')';
    vecteur & v=*feuille._VECTptr;
    if (maple_mode==3){
      if ( (v.front().type==_SYMB) && (v.front()._SYMBptr->sommet==at_program)){
	gen & b=v.front()._SYMBptr->feuille._VECTptr->back();
	if  (b.type==_VECT || (b.type==_SYMB && (b._SYMBptr->sommet==at_local || b._SYMBptr->sommet==at_bloc))){
	  string s(v.front().print());
	  s=s.substr(10,s.size()-10);
	  return ":"+v.back().print()+s;
	}
	else {
	  vecteur & tmpv = *v.front()._SYMBptr->feuille._VECTptr;
	  if (tmpv[0].type==_VECT && tmpv[0].subtype==_SEQ__VECT && tmpv[0]._VECTptr->size()==1)
	    return tmpv[2].print()+" => "+v.back().print()+"("+tmpv[0]._VECTptr->front().print()+")";
	  else
	    return tmpv[2].print()+" => "+v.back().print()+"("+tmpv[0].print()+")";
	}
      }
      else 
	return v.front().print()+" => "+v.back().print();
    }
    string s(v.back().print()+":=");
    if (v.front().type==_SEQ__VECT)
      return s+"("+v.front().print()+")";
    else
      return s+v.front().print();
  }
  string texprintassto(const gen & g,const string & sommetstr){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return sommetstr+'('+gen2tex(g)+')';
    string s(gen2tex(g._VECTptr->back())+":=");
    if (g._VECTptr->front().type==_SEQ__VECT)
      return s+"("+gen2tex(g._VECTptr->front())+")";
    else
      return s+gen2tex(g._VECTptr->front());
  }

  // store a in b
  bool signal_store=true;
  gen sto(const gen & a,const gen & b,const context * contextptr){
    // cerr << "Sto " << "->" << b << endl;
    if (b.type==_SYMB){ 
      if (b._SYMBptr->sommet==at_hash && b._SYMBptr->feuille.type==_STRNG)
	return sto(a,gen(*b._SYMBptr->feuille._STRNGptr),contextptr);
      if (b._SYMBptr->sommet==at_double_deux_points){
	gen a1,bb;
	check_binary(b._SYMBptr->feuille,a1,bb);
	gen ab=eval(a1,contextptr);
	if (ab.type==_VECT){
	  vecteur v=*ab._VECTptr;
	  iterateur it=v.begin(),itend=v.end();
	  for (;it!=itend;++it){
	    if (it->type!=_VECT || it->_VECTptr->size()!=2)
	      continue;
	    vecteur & w=*it->_VECTptr;
	    if (w[0]==bb)
	      w[1]=a;
	  }
	  if (it==itend)
	    v.push_back(makevecteur(bb,a));
	  return sto(gen(v,_FOLDER__VECT),a1,contextptr);
	}
	if (a1.type==_IDNT)
	  return sto(gen(vecteur(1,makevecteur(bb,a)),_FOLDER__VECT),a1,contextptr);
      }
    }
    if (b.type==_IDNT){
      eval(b); // This has the effect of removing stale local assignements
      if (*b._IDNTptr==e__IDNT_e || *b._IDNTptr==_IDNT_pi || *b._IDNTptr==_IDNT_infinity)
	setsizeerr(b.print()+": reserved word");
      gen aa(a),ans(aa);
      if (a==b)
	return _purge(b,contextptr);
      if (contains(lidnt(a),b))
	// replaced setsizeerr by cerr, to permit recursive function def.
	cerr << b.print()+": recursive definition" << endl;
      if ( (a.type==_SYMB) && (a._SYMBptr->sommet==at_parameter)){
	gen inter=a._SYMBptr->feuille,debut,fin;
	bool calc_aa=false;
	if ( (inter.type==_VECT) && (inter._VECTptr->size()==2) ){
	  aa=inter._VECTptr->back();
	  inter=inter._VECTptr->front();
	}
	else
	  calc_aa=true;
	if ( (inter.type==_SYMB) && (inter._SYMBptr->sommet==at_interval) ){
	  debut=inter._SYMBptr->feuille._VECTptr->front();
	  fin=inter._SYMBptr->feuille._VECTptr->back();
	}
	if (calc_aa)
	  aa=rdiv(debut+fin,plus_two);
	ans=symbolic(at_parameter,makevecteur(b,debut,fin,aa));
      }
      if (contextptr){
	if (!contextptr->globalcontextptr) // context is a global table
	  (*contextptr->tabptr)[*b._IDNTptr->name]=aa;
	else {
	  sym_tab::const_iterator it=contextptr->tabptr->find(*b._IDNTptr->name),itend=contextptr->tabptr->end();
	  if (it!=itend) // found in current local context
	    (*contextptr->tabptr)[*b._IDNTptr->name]=aa;
	  else // store b here
	    (*contextptr->globalcontextptr->tabptr)[*b._IDNTptr->name]=aa;
	}
	if (!child_id && signal_store)
	  _signal(symb_quote(symbolic(at_sto,makevecteur(aa,b))));
	return aa;
      }
      if (!b._IDNTptr->localvalue->empty() && (b.subtype!=_GLOBAL__EVAL))
	b._IDNTptr->localvalue->back()=aa;
      else {
	if (current_folder_name.type==_IDNT && current_folder_name._IDNTptr->value && current_folder_name._IDNTptr->value->type==_VECT){
	  vecteur v=*current_folder_name._IDNTptr->value->_VECTptr;
	  iterateur it=v.begin(),itend=v.end();
	  for (;it!=itend;++it){
	    if (it->type!=_VECT || it->_VECTptr->size()!=2)
	      continue;
	    vecteur & w=*it->_VECTptr;
	    if (w[0]==b){
	      w[1]=aa;
	      break;
	    }
	  }
	  if (it==itend)
	    v.push_back(makevecteur(b,aa));
	  gen gv(v,_FOLDER__VECT);
	  *current_folder_name._IDNTptr->value=gv;
	  if (!child_id && signal_store)
	    _signal(symb_quote(symbolic(at_sto,makevecteur(gv,current_folder_name))));
	  return aa;
	}
	else {
	  if (b._IDNTptr->value)
	    delete b._IDNTptr->value;
	  b._IDNTptr->value = new gen(aa);
	  if (!child_id && signal_store)
	    _signal(symb_quote(symbolic(at_sto,makevecteur(aa,b))));
	  if (!secure_run && variables_are_files(contextptr)){
	    ofstream a_out((*b._IDNTptr->name+cas_suffixe).c_str());
	    a_out << aa << endl;
	  }
	}
      }
      return ans;
    }
    if (b.type==_VECT){
      if (a.type!=_VECT)
	settypeerr();
      return apply(a,b,contextptr,giac::sto);
    }
    if ( (b.type==_SYMB) && (b._SYMBptr->sommet==at_at) ){
      // Store a in a vector or array or map
      gen destination=b._SYMBptr->feuille._VECTptr->front();
      gen valeur=destination.eval(DEFAULT_EVAL_LEVEL,contextptr);
      if ( valeur.type==_INT_ && valeur.val==0 && destination.type==_IDNT && !destination._IDNTptr->localvalue->empty() )
	valeur=destination; // non (0) initialized local var
      gen indice=b._SYMBptr->feuille._VECTptr->back().eval(DEFAULT_EVAL_LEVEL,contextptr);
      if ( (destination.type!=_IDNT) || (valeur.type!=_VECT && valeur.type!=_MAP && valeur.type!=_IDNT) )
	settypeerr("sto "+b.print()+ "="+valeur.print()+" not allowed!");
      if (valeur.type==_IDNT){
	gen g;
	if (maple_mode==3 && indice.type==_INT_ && indice.val>=0 ){
	  vecteur v(indice.val+1,zero);
	  v[indice.val]=a;
	  g=gen(v,destination.subtype);
	}
	else {
	  g=makemap();
	  (*g._MAPptr)[indice]=a;
	}
	return sto(g,destination,contextptr);
      }
      if (valeur.type==_MAP){
	gen_map m(*valeur._MAPptr);
	m[indice]=a;
	return sto(m,destination,contextptr);
      }
      vecteur v(*valeur._VECTptr);
      if (indice.type!=_VECT){
	if (indice.type!=_INT_ || indice.val<0 )
	  settypeerr("Bad index "+indice.print());
	// check size
	int is=v.size();
	for (;is<=indice.val;++is)
	  v.push_back(zero);
	// change indice's value
	v[indice.val]=a;
	return sto(gen(v,valeur.subtype),destination,contextptr);
      }
      vecteur empile;
      iterateur it=indice._VECTptr->begin(),itend=indice._VECTptr->end();
      for (;;){
	if (it->type!=_INT_)
	  settypeerr("Bad index "+indice.print());
	empile.push_back(v);
	gen tmp=v[it->val];
	++it;
	if (it==itend)
	  break;
	if (tmp.type!=_VECT)
	  settypeerr("Bad index "+indice.print());
	v=*tmp._VECTptr;
      }
      --itend;
      v[itend->val]=a;
      vecteur oldv;
      it=indice._VECTptr->begin();
      for (;;){
	if (itend==it)
	  break;
	--itend;
	empile.pop_back();
	oldv=*(empile.back()._VECTptr);
	oldv[itend->val]=v;
	v=oldv;
      }
      return sto(v,destination,contextptr);
    }
    if (b.type==_FUNC)
      return makevecteur(string2gen(b.print()+ " is a reserved word, sto not allowed:",false),a);
    settypeerr("sto "+b.print()+ " not allowed!");
  }
  symbolic symb_sto(const gen & a,gen & b){
    return symbolic(at_sto,makevecteur(a,b));
  }
  symbolic symb_sto(const gen & e){
    return symbolic(at_sto,e);
  }
  gen _sto(const gen & a,const context * contextptr){
    if (a.type!=_VECT)
      return symb_sto(a);
    if (rpn_mode){
      if (a._VECTptr->size()<2)
	toofewargs("STO");
      gen c=a._VECTptr->back();
      a._VECTptr->pop_back();
      gen b=a._VECTptr->back();
      a._VECTptr->pop_back();
      sto(b,c,contextptr);
      return gen(*a._VECTptr,_RPN_STACK__VECT);
    }
    if (a._VECTptr->size()!=2)
      settypeerr();
    return sto(a._VECTptr->front(),a._VECTptr->back(),contextptr);
  }
  const string _sto_s("sto");
  unary_function_eval __sto(&giac::_sto,_sto_s,&printassto,&texprintassto);
  unary_function_ptr at_sto (&__sto);

  bool is_assumed_integer(const gen & g){
    if (is_integer(g))
      return true;
    if (g.type==_IDNT)
      return g.subtype==_ZINT;
    if (g.type!=_SYMB)
      return false;
    unary_function_ptr & u=g._SYMBptr->sommet;
    gen & f=g._SYMBptr->feuille;
    if ( (u==at_neg) || (u==at_abs) )
      return is_assumed_integer(f);
    if ( (u==at_plus) || (u==at_prod) ){
      if (f.type!=_VECT)
	return is_assumed_integer(f);
      const_iterateur it=f._VECTptr->begin(),itend=f._VECTptr->end();
      for (;it!=itend;++it)
	if (!is_assumed_integer(*it))
	  return false;
      return true;
    }
    return false;
  }
  // v = previous assumptions, a=the real value, direction
  // is positive for [a,+inf[, negative for ]-inf,a]
  // |direction| = 1 (large) or 2 (strict) 
  gen doubleassume_and(const vecteur & v,const gen & a,int direction,bool or_assumption,GIAC_CONTEXT){
    vecteur v_intervalle,v_excluded;
    if ( (v.size()==3) && (v[1].type==_VECT) && (v[2].type==_VECT) ){
      v_intervalle=*v[1]._VECTptr;
      v_excluded=*v.back()._VECTptr;
    }
    if (!(direction %2) && !equalposcomp(v_excluded,a))
      v_excluded.push_back(a);
    if (or_assumption){ 
      // remove excluded values if they are in the interval we add
      vecteur old_v(v_excluded);
      v_excluded.clear();
      const_iterateur it=old_v.begin(),itend=old_v.end();
      for (;it!=itend;++it){
	bool a_greater_sup=ck_is_greater(a,*it,contextptr);
	if (a_greater_sup && (direction<0) )
	  continue;
	if (!a_greater_sup && (direction>0) )
	  continue;
	v_excluded.push_back(*it);
      }
    }
    if (v_intervalle.empty() || or_assumption){
      if (direction>0)
	v_intervalle.push_back(gen(makevecteur(a,plus_inf),_LINE__VECT));
      else
	v_intervalle.push_back(gen(makevecteur(minus_inf,a),_LINE__VECT));
      if (or_assumption)
	return gen(makevecteur(gen(_DOUBLE_).change_subtype(1),v_intervalle,v_excluded),_ASSUME__VECT);
    }
    else { // intersection of [a.+inf[ with every interval from v_intervalle
      vecteur old_v(v_intervalle);
      v_intervalle.clear();
      const_iterateur it=old_v.begin(),itend=old_v.end();
      for (;it!=itend;++it){
	if ( (it->type!=_VECT) || (it->subtype!=_LINE__VECT) || (it->_VECTptr->size()!= 2) )
	  setsizeerr();
	gen i_inf(it->_VECTptr->front()),i_sup(it->_VECTptr->back());
	bool a_greater_sup=ck_is_greater(a,i_sup,contextptr);
	if (a_greater_sup){
	  if (direction<0)
	    v_intervalle.push_back(*it);
	  continue;
	}
	bool a_greater_inf=ck_is_greater(a,i_inf,contextptr);
	if (!a_greater_inf){
	  if (direction>0)
	    v_intervalle.push_back(*it);
	  continue;
	}
	if (direction>0)
	  v_intervalle.push_back(gen(makevecteur(a,i_sup),_LINE__VECT));
	else
	  v_intervalle.push_back(gen(makevecteur(i_inf,a),_LINE__VECT));
      }
    }
    return gen(makevecteur(gen(_DOUBLE_).change_subtype(1),v_intervalle,v_excluded),_ASSUME__VECT);
  }
  // returns the assumed idnt name
  // used if assumptions are in OR conjonction
  gen assumesymbolic(const gen & a,gen idnt_must_be,GIAC_CONTEXT){
    if (a.type==_IDNT)
      return a._IDNTptr->eval(DEFAULT_EVAL_LEVEL,contextptr);
    if ( (a.type!=_SYMB) || (a._SYMBptr->feuille.type!=_VECT) )
      setsizeerr();
    while (idnt_must_be.type==_SYMB){
      idnt_must_be=idnt_must_be._SYMBptr->feuille;
      if ( (idnt_must_be.type==_VECT) && !(idnt_must_be._VECTptr->empty()) )
	idnt_must_be=idnt_must_be._VECTptr->front();
    }
    unary_function_ptr s(a._SYMBptr->sommet);
    vecteur v(*a._SYMBptr->feuille._VECTptr);
    int l=v.size();
    if (!l)
      setsizeerr();
    gen arg0(v.front()),arg1(v.back()),hyp(undef);
    if (s==at_sto){
      gen tmp(arg0);
      arg0=arg1;
      arg1=tmp;
    }
    if (s==at_and){
      assumesymbolic(arg0,0,contextptr);
      return assumesymbolic(arg1,0,contextptr);
    }
    if (s==at_ou){
      gen a0(assumesymbolic(arg0,0,contextptr));
      return assumesymbolic(arg1,a0,contextptr);
    }
    if (arg0.type!=_IDNT)
      arg0=arg0.eval(DEFAULT_EVAL_LEVEL,contextptr);
    if ( (arg0.type!=_IDNT) || (!is_zero(idnt_must_be) && (arg0!=idnt_must_be) ) )
      setsizeerr();
    bool or_assumption= !is_zero(idnt_must_be) && (arg0==idnt_must_be);
    vecteur last_hyp;
    arg1=arg0._IDNTptr->eval(DEFAULT_EVAL_LEVEL,contextptr);
    if ( (arg1.type!=_VECT) || (arg1.subtype!=_ASSUME__VECT) )
      last_hyp=makevecteur(vecteur(0),vecteur(0));
    else
      last_hyp=*arg1._VECTptr;
    if (l==2){
      if (s==at_sto)
	arg1=v[0].eval(DEFAULT_EVAL_LEVEL,contextptr);
      else
	arg1=v[1].eval(DEFAULT_EVAL_LEVEL,contextptr);
      gen borne_inf(gnuplot_xmin),borne_sup(gnuplot_xmax);
      if ( (s==at_equal || s==at_same) || (s==at_sto) ){     
	// ex: assume(a=[1.7,1.1,2.3])
	if (arg1.type==_VECT && arg1._VECTptr->size()==3){
	  vecteur vtmp=*arg1._VECTptr;
	  borne_inf=evalf_double(vtmp[1],DEFAULT_EVAL_LEVEL,contextptr);
	  borne_sup=evalf_double(vtmp[2],DEFAULT_EVAL_LEVEL,contextptr);
	  arg1=evalf_double(vtmp[0],DEFAULT_EVAL_LEVEL,contextptr);
	}
	gen tmp=arg1.type;
	tmp.subtype=1;
	hyp=gen(makevecteur(tmp,arg1),_ASSUME__VECT);
      }
      if (s==at_inferieur_strict) // ex: assume(a<1.7)
	hyp=doubleassume_and(last_hyp,arg1,-2,or_assumption,contextptr);
      if (s==at_inferieur_egal) 
	hyp=doubleassume_and(last_hyp,arg1,-1,or_assumption,contextptr);
      if (s==at_superieur_strict)
	hyp=doubleassume_and(last_hyp,arg1,2,or_assumption,contextptr);
      if (s==at_superieur_egal) 
	hyp=doubleassume_and(last_hyp,arg1,1,or_assumption,contextptr);
      if (!is_undef(hyp)){
	sto(hyp,arg0,contextptr); 
	if ( (s==at_equal || s==at_same) || (s==at_sto) )
	  return _parameter(makevecteur(arg0,borne_inf,borne_sup,arg1));
	return arg0;
      }
    }
    setsizeerr();
  }
  gen _assume(const gen & a,GIAC_CONTEXT){
    if ( (a.type==_VECT) && (a._VECTptr->size()==2) ){
      gen a1(a._VECTptr->front()),a2(a._VECTptr->back());
      if (a2.type==_INT_){
	// assume(a,real) for example
	a2.subtype=1;
	sto(gen(makevecteur(a2),_ASSUME__VECT),a1,contextptr);
	return a2;
      }
      if (a2==at_real){
	a2=_DOUBLE_;
	a2.subtype=1;
	sto(gen(makevecteur(a2),_ASSUME__VECT),a1,contextptr);
	return a2;
      }
      if ( (a2.type==_FUNC) && (*a2._FUNCptr==at_ou) ){
	return assumesymbolic(a1,a1,contextptr);
      }
    }
    return assumesymbolic(a,0,contextptr);
  }
  const string _assume_s("assume");
  unary_function_eval __assume(&giac::_assume,_assume_s);
  unary_function_ptr at_assume (&__assume,_QUOTE_ARGUMENTS);

  // multiargs
  symbolic symb_plus(const gen & a,const gen & b){
    if (a.is_symb_of_sommet(at_plus)){
      if (b.is_symb_of_sommet(at_plus))
	  return symbolic(at_plus,mergevecteur(*(a._SYMBptr->feuille._VECTptr),*(b._SYMBptr->feuille._VECTptr)));
	else
	  return symbolic(*a._SYMBptr,b);
    }
    return symbolic(at_plus,makevecteur(a,b));
  }
  gen _plus(const gen & args){
    if (args.type!=_VECT){
      if ((args.type==_IDNT) && (*args._IDNTptr==_IDNT_infinity))
	return plus_inf;
      return args;
    }
    iterateur it=args._VECTptr->begin(), itend=args._VECTptr->end();
    if (it==itend)
      return zero;
    gen sum(*it);
    ++it;
    for (;it!=itend;++it){
      if (sum.is_symb_of_sommet(at_plus) && sum._SYMBptr->feuille.type==_VECT && sum._SYMBptr->feuille._VECTptr->size()>1){
	// Add remaining elements to the symbolic sum, check float/inf/undef
	vecteur v=*sum._SYMBptr->feuille._VECTptr;
	for (;it!=itend;++it){
	  if (it->type==_USER && v.front().type==_USER){
	    v.front()=v.front()+*it;
	    continue;
	  }
	  if (it->type<=_POLY && v.back().type<=_POLY)
	    v.back()=v.back()+*it;
	  else {
	    if (is_inf(*it) || is_undef(*it) )
	      break;
	    if (!is_zero(*it))
	      v.push_back(*it);
	  }
	}
	if (is_zero(v.back()))
	  v.pop_back();
	if (v.size()==1)
	  sum=v.front();
	else
	  sum=symbolic(at_plus,v);
	if (it==itend)
	  break;
      }
      sum = sum + (*it);
    }
    return sum;
  }

  unary_function_ptr _D_at_plus (int i) {
    return at_one;
  }
  partial_derivative_multiargs D_at_plus(&_D_at_plus);
  const string _plus_s("+");
  unary_function_unary __plus(&_plus,&D_at_plus,_plus_s,&printsommetasoperator);
  unary_function_ptr at_plus (&__plus);

  symbolic symb_prod(const gen & a,const gen & b){
      return symbolic(at_prod,makevecteur(a,b));
  }
  gen _prod(const gen & args){
    if (args.type!=_VECT)
      return args;
    iterateur it=args._VECTptr->begin(), itend=args._VECTptr->end();
    gen prod(1);
    for (;it!=itend;++it){
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_inv) && (it->_SYMBptr->feuille.type!=_VECT) )
	prod = rdiv(prod,it->_SYMBptr->feuille);
      else
	prod = prod * (*it);
    }
    return prod;
  }

  unary_function_ptr _D_at_prod (int i) {
    vector<int> v;
    v.push_back(i);
    vector<unary_function_ptr> w;
    w.push_back(at_prod);
    w.push_back(unary_function_innerprod(v));
    return unary_function_compose(w);
  }
  partial_derivative_multiargs D_at_prod(&_D_at_prod);


  const string _prod_s("*");
  unary_function_unary __prod(&_prod,&D_at_prod,_prod_s,&printsommetasoperator);
  unary_function_ptr at_prod (&__prod);

  symbolic symb_pow(const gen & a,const gen & b){
      return symbolic(at_pow,makevecteur(a,b));
  }
  gen _pow(const gen & args){
    if (args.type!=_VECT)
      return args;
    vecteur & v = *args._VECTptr;
    if (v.size()!=2)
      setsizeerr("bad pow "+args.print());
    return pow( v[0],v[1]);
  }
  gen d1_pow(const gen & args){
    if (args.type!=_VECT)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      setsizeerr("bad pow "+args.print());
    if (v[1].type<=_REAL)
      return v[1]*pow(v[0],v[1]-1);
    else
      return v[1]/v[0]*symbolic(at_pow,v);
  }
  gen d2_pow(const gen & args){
    if (args.type!=_VECT)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      setsizeerr("bad pow "+args.print());
    return ln(v[0])*pow(v[0],v[1]);
  }
  unary_function_ptr D1_pow(unary_function_unary(&d1_pow,""));
  unary_function_ptr D2_pow(unary_function_unary(&d2_pow,""));
  unary_function_ptr d_pow(int i){
    if (i==1)
      return D1_pow;
    if (i==2)
      return D2_pow;
    setsizeerr();
  }
  partial_derivative_multiargs D_pow(&d_pow);
  const string _pow_s("^");
  unary_function_unary __pow(&_pow,&D_pow,_pow_s,&printsommetasoperator,0);
  unary_function_ptr at_pow (&__pow);

  symbolic symb_powmod(const gen & a,const gen & b,const gen & n){
    return symbolic(at_powmod,makevecteur(a,b,n));
  }
  symbolic symb_powmod(const gen & a){
    return symbolic(at_powmod,a);
  }
  gen _powmod(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3) )
      return symb_powmod(args);
    return powmod(args._VECTptr->front(),(*(args._VECTptr))[1],args._VECTptr->back());
  }
  const string _powmod_s("powmod");
  unary_function_unary __powmod(&giac::_powmod,_powmod_s);
  unary_function_ptr at_powmod (&__powmod);

  symbolic symb_inferieur_strict(const gen & a,const gen & b){
    return symbolic(at_inferieur_strict,makevecteur(a,b));
  }
  symbolic symb_inferieur_strict(const gen & a){
    return symbolic(at_inferieur_strict,a);
  }
  gen _inferieur_strict(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symb_inferieur_strict(args);
    return inferieur_strict(args._VECTptr->front(),args._VECTptr->back(),contextptr);
  }
  const string _inferieur_strict_s("<");
  unary_function_eval __inferieur_strict(&giac::_inferieur_strict,_inferieur_strict_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_inferieur_strict (&__inferieur_strict);

  symbolic symb_inferieur_egal(const gen & a,const gen & b){
    return symbolic(at_inferieur_egal,makevecteur(a,b));
  }
  symbolic symb_inferieur_egal(const gen & a){
    return symbolic(at_inferieur_egal,a);
  }
  gen _inferieur_egal(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symb_inferieur_egal(args);
    return inferieur_egal(args._VECTptr->front(), args._VECTptr->back(),contextptr);
  }
  const string _inferieur_egal_s("<=");
  string texprintasinferieur_egal(const gen & g,const string & s){
    return texprintsommetasoperator(g,"\\leq ");
  }
  unary_function_eval __inferieur_egal(&giac::_inferieur_egal,_inferieur_egal_s,&printsommetasoperator,&texprintasinferieur_egal);
  unary_function_ptr at_inferieur_egal (&__inferieur_egal);

  symbolic symb_superieur_strict(const gen & a,const gen & b){
    return symbolic(at_superieur_strict,makevecteur(a,b));
  }
  symbolic symb_superieur_strict(const gen & a){
    return symbolic(at_superieur_strict,a);
  }
  gen _superieur_strict(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symb_superieur_strict(args);
    return superieur_strict(args._VECTptr->front(),args._VECTptr->back(),contextptr);
  }
  const string _superieur_strict_s(">");
  unary_function_eval __superieur_strict(&giac::_superieur_strict,_superieur_strict_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_superieur_strict (&__superieur_strict);

  symbolic symb_superieur_egal(const gen & a,const gen & b){
    return symbolic(at_superieur_egal,makevecteur(a,b));
  }
  symbolic symb_superieur_egal(const gen & a){
    return symbolic(at_superieur_egal,a);
  }
  gen _superieur_egal(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symb_superieur_egal(args);
    return superieur_egal(args._VECTptr->front(), args._VECTptr->back(),contextptr);
  }
  const string _superieur_egal_s(">=");
  string texprintassuperieur_egal(const gen & g,const string & s){
    return texprintsommetasoperator(g,"\\geq ");
  }
  unary_function_eval __superieur_egal(&giac::_superieur_egal,_superieur_egal_s,&printsommetasoperator,&texprintassuperieur_egal);
  unary_function_ptr at_superieur_egal (&__superieur_egal);

  string printasdifferent(const gen & feuille,const string & sommetstr){
    if (maple_mode > 0)
      return printsommetasoperator(feuille,"<>");
    else
      return printsommetasoperator(feuille,sommetstr);
  }
  symbolic symb_different(const gen & a,const gen & b){
    return symbolic(at_different,makevecteur(a,b));
  }
  symbolic symb_different(const gen & a){
    return symbolic(at_different,a);
  }
  gen _different(const gen & args){
    if (args.type!=_VECT)
      return symb_different(args);
    return args._VECTptr->front() != args._VECTptr->back();
  }
  const string _different_s("!=");
  unary_function_unary __different(&giac::_different,_different_s,&printasdifferent);
  unary_function_ptr at_different (&__different);

  string printasof_(const gen & feuille,const string & sommetstr,int format){
    if ( (feuille.type!=_VECT) || (feuille._VECTptr->size()!=2) )
      return sommetstr+'('+gen2string(feuille,format)+')';
    string s=print_with_parenthesis_if_required(feuille._VECTptr->front(),format)+'(';
    gen & g=feuille._VECTptr->back();
    if (format==0 && g.type==_VECT && g.subtype==_SEQ__VECT)
      return s+printinner_VECT(*g._VECTptr,_SEQ__VECT)+')';
    else
      return s+gen2string(g,format)+')';
  }
  string texprintasof(const gen & feuille,const string & sommetstr){
    return printasof_(feuille,sommetstr,1);
  }
  string printasof(const gen & feuille,const string & sommetstr){
    return printasof_(feuille,sommetstr,0);
  }
  symbolic symb_of(const gen & a,const gen & b){
    if (b.type==_VECT && b.subtype==_SEQ__VECT && b._VECTptr->size()==1)
      return symbolic(at_of,makevecteur(a,b._VECTptr->front()));
    return symbolic(at_of,makevecteur(a,b));
  }
  symbolic symb_of(const gen & a){
    return symbolic(at_of,a);
  }
  gen _of(const gen & args,const context * contextptr){
    if (args.type!=_VECT)
      return symb_of(args);
    gen qf=args._VECTptr->front();
    gen b=args._VECTptr->back().eval(DEFAULT_EVAL_LEVEL,contextptr);
    gen f=qf.eval(DEFAULT_EVAL_LEVEL,contextptr);
    if ( (f.type==_SYMB) && (f._SYMBptr->sommet==at_program) && (qf.type==_IDNT)){
      gen tmp=f._SYMBptr->feuille;
      if (tmp.type!=_VECT)
	setsizeerr();
      (*tmp._VECTptr)[1]=b;
      return _program(tmp,qf,contextptr);
    }
    return f(b,contextptr);
  }
  const string _of_s("of");
  unary_function_eval __of(&giac::_of,_of_s,&printasof,&texprintasof);
  unary_function_ptr at_of (&__of,_QUOTE_ARGUMENTS);

  string gen2string(const gen & g,int format){
    if (format==1) 
      return gen2tex(g); 
    else 
      return g.print();
  }

  string print_with_parenthesis_if_required(const gen & g,int format){
    if (g.type==_SYMB || g.type==_FRAC || g.type==_CPLX || (g.type==_VECT && g.subtype==_SEQ__VECT) )
      return '('+gen2string(g,format)+')';
    else
      return gen2string(g,format);
  }
  
  string printasat_(const gen & feuille,const string & sommetstr,int format){
    if ( (feuille.type!=_VECT) || (feuille._VECTptr->size()!=2) )
      return sommetstr+'('+gen2string(feuille,format)+')';
    vecteur & v=*feuille._VECTptr;
    if (maple_mode > 0){
      gen indice;
      if (v.back().type==_VECT)
	indice=v.back()+vecteur(v.size(),plus_one);
      else
	indice=v.back()+plus_one;
      string s;
      return print_with_parenthesis_if_required(v.front(),format)+'['+gen2string(indice,format)+']';
    }
    else
      return print_with_parenthesis_if_required(feuille._VECTptr->front(),format)+'['+gen2string(feuille._VECTptr->back(),format)+']';
  }

  string printasat(const gen & feuille,const string & sommetstr){
    return printasat_(feuille,sommetstr,0);
  }
  string texprintasat(const gen & feuille,const string & sommetstr){
    return printasat_(feuille,sommetstr,1);
  }
  symbolic symb_at(const gen & a,const gen & b){
    if (maple_mode){
      gen bb;
      if (b.type==_VECT)
	bb=b-vecteur(b._VECTptr->size(),plus_one);
      else
	bb=b-plus_one;
      return symbolic(at_at,makevecteur(a,bb));
    }
    else
      return symbolic(at_at,makevecteur(a,b));
  }
  symbolic symb_at(const gen & a){
    return symbolic(at_at,a);
  }
  gen _at(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symb_at(args);
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      setsizeerr();
    gen a=eval(v.front(),contextptr);
    gen b=eval(v.back(),contextptr);
    if (a.type==_MAP){
      gen_map::const_iterator it=a._MAPptr->find(b),itend=a._MAPptr->end();
      if (it!=itend)
	return it->second;
      return symb_at(makevecteur(v.front(),b));
    }
    return a[b];
  }
  const string _at_s("at");
  unary_function_eval __at(&giac::_at,_at_s,&printasat,&texprintasat);
  unary_function_ptr at_at (&__at,_QUOTE_ARGUMENTS);

  gen _table(const gen & arg){
    vecteur v(gen2vecteur(arg));
    const_iterateur it=v.begin(),itend=v.end();
    gen_map m(ptr_fun(islesscomplexthanf));
    for (;it!=itend;++it){
      if (it->is_symb_of_sommet(at_equal)){
	gen & f =it->_SYMBptr->feuille;
	if (f.type==_VECT && f._VECTptr->size()==2){
	  vecteur & w=*f._VECTptr;
	  m[w.front()]=w.back();
	}
      }
    }
    return m;
  }
  const string _table_s("table");
  unary_function_unary __table(&giac::_table,_table_s);
  unary_function_ptr at_table (&__table);

  string printasand(const gen & feuille,const string & sommetstr){
    if (maple_mode > 0)
      return printsommetasoperator(feuille," and ");
    else
      return printsommetasoperator(feuille,sommetstr);
  }
  string texprintasand(const gen & g,const string & s){
    return texprintsommetasoperator(g,"\\mbox{ and }");
  }
  symbolic symb_and(const gen & a,const gen & b){
    return symbolic(at_and,makevecteur(a,b));
  }
  gen _and(const gen & arg,GIAC_CONTEXT){
    gen args=equaltosame(arg);
    if (args.type!=_VECT || args._VECTptr->empty())
      return eval(args,contextptr);
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    gen res=eval(*it,contextptr);
    ++it;
    for (;it!=itend;++it){
      if (res.type==_INT_ && res.val==0)
	return res;
      res = res && eval(*it,contextptr);
    }
    return res;
  }
  const string _and_s(" && ");
  unary_function_eval __and(&giac::_and,_and_s,&printasand,&texprintasand);
  unary_function_ptr at_and (&__and,_QUOTE_ARGUMENTS);

  string texprintasor(const gen & g,const string & s){
    return texprintsommetasoperator(g,"\\mbox{ or }");
  }
  string printasor(const gen & feuille,const string & sommetstr){
    if (maple_mode > 0)
      return printsommetasoperator(feuille," or ");
    else
      return printsommetasoperator(feuille,sommetstr);
  }
  symbolic symb_ou(const gen & a,const gen & b){
    return symbolic(at_ou,makevecteur(a,b));
  }
  gen _ou(const gen & arg,GIAC_CONTEXT){
    gen args=equaltosame(arg);
    if (args.type!=_VECT || args._VECTptr->empty())
      return eval(args,contextptr);
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    gen res=eval(*it,contextptr);
    ++it;
    for (;it!=itend;++it){
      if (res.type==_INT_ && res.val)
	return res;
      res = res || eval(*it,contextptr);
    }
    return res;
  }
  const string _ou_s(" || ");
  unary_function_eval __ou(&giac::_ou,_ou_s,&printasor,&texprintasor);
  unary_function_ptr at_ou (&__ou,_QUOTE_ARGUMENTS);

  gen _xor(const gen & arg,GIAC_CONTEXT){
    gen args=eval(equaltosame(arg),contextptr);
    if (args.type!=_VECT)
      return args;
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    gen res=*it;
    ++it;
    for (;it!=itend;++it){
      if (is_zero(res))
	res=*it;
      else
	res = !(*it);
    }
    return res;
  }
  const string _xor_s(" xor ");
  unary_function_eval __xor(&giac::_xor,_xor_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_xor (&__xor);

  symbolic symb_min(const gen & a,const gen & b){
    return symbolic(at_min,makevecteur(a,b));
  }
  gen _min(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return args;
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    if (ckmatrix(args))
      return apply(*it,*(it+1),contextptr,min);
    if (itend-it==2 && it->type==_VECT && (it+1)->type==_VECT )
      return matrix_apply(*it,*(it+1),contextptr,min);
    gen res=*it;
    ++it;
    for (;it!=itend;++it)
      res = min(res,*it,contextptr);
    return res;
  }
  const string _min_s("min");
  unary_function_eval __min(&giac::_min,_min_s);
  unary_function_ptr at_min (&__min);

  symbolic symb_max(const gen & a,const gen & b){
    return symbolic(at_max,makevecteur(a,b));
  }
  gen _max(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return args;
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    if (itend==it)
      setdimerr();
    if (itend-it==1)
      return _max(*it,contextptr);
    if (itend-it==2 && ckmatrix(args))
      return apply(*it,*(it+1),contextptr,max);
    if (itend-it==2 && it->type==_VECT && (it+1)->type==_VECT )
      return matrix_apply(*it,*(it+1),contextptr,max);
    gen res=*it;
    ++it;
    for (;it!=itend;++it)
      res = max(res,*it,contextptr);
    return res;
  }
  const string _max_s("max");
  unary_function_eval __max(&giac::_max,_max_s);
  unary_function_ptr at_max (&__max);

  symbolic symb_gcd(const gen & a,const gen & b){
    return symbolic(at_gcd,makevecteur(a,b));
  }
  gen _gcd(const gen & args){
    if (args.type!=_VECT)
      return args;
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    if (ckmatrix(args) && itend-it==2)
      return apply(*it,*(it+1),gcd);
    gen res(0);
    for (;it!=itend;++it)
      res=gcd(res,*it);
    return res;
  }
  const string _gcd_s("gcd");
  unary_function_unary __gcd(&giac::_gcd,_gcd_s);
  unary_function_ptr at_gcd (&__gcd);

  symbolic symb_lcm(const gen & a,const gen & b){
    return symbolic(at_lcm,makevecteur(a,b));
  }
  gen _lcm(const gen & args){
    if (args.type!=_VECT)
      return args;
    vecteur::const_iterator it=args._VECTptr->begin(),itend=args._VECTptr->end();
    if (ckmatrix(args) && itend-it==2)
      return apply(*it,*(it+1),lcm);
    gen res(1);
    for (;it!=itend;++it)
      res=lcm(res,*it);
    return res;
  }
  const string _lcm_s("lcm");
  unary_function_unary __lcm(&giac::_lcm,_lcm_s);
  unary_function_ptr at_lcm (&__lcm);

  symbolic symb_egcd(const gen & a,const gen & b){
    return symbolic(at_egcd,makevecteur(a,b));
  }
  gen _egcd(const gen & args){
    if ( (args.type!=_VECT) || args._VECTptr->empty() )
      setsizeerr();
    vecteur & a = *args._VECTptr;
    if ( (a.front().type==_VECT) && (a.back().type==_VECT) ){
      vecteur u,v,d;
      egcd(*a.front()._VECTptr,*a.back()._VECTptr,0,u,v,d);
      return gen(makevecteur(u,v,d),_POLY1__VECT);
    }
    vecteur lv;
    if (a.size()==3)
      lv=vecteur(1,a[2]);
    else
      lv=vecteur(1,vx_var);
    lvar(args,lv);
    gen aa=e2r(a[0],lv),aan,aad,bb=e2r(a[1],lv),bbn,bbd;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    fxnd(bb,bbn,bbd);
    if ( (bbd.type==_POLY) && (bbd._POLYptr->lexsorted_degree() ) )
      setsizeerr(); 
    gen u,v,d;
    if ( (aan.type==_POLY) && (bbn.type==_POLY) ){
      polynome un(aan._POLYptr->dim),vn(aan._POLYptr->dim),dn(aan._POLYptr->dim);
      egcd(*aan._POLYptr,*bbn._POLYptr,un,vn,dn);
      u=un;
      v=vn;
      d=dn;
    }
    else {
      if (aan.type==_POLY){
	u=zero;
	v=plus_one;
	d=bbn;
      }
      else {
	u=plus_one;
	v=zero;
	d=aan;
      }
    }
    u=r2e(u*aad,lv);
    v=r2e(v*bbd,lv);
    d=r2e(d,lv);
    return makevecteur(u,v,d);
  }
  const string _egcd_s("egcd");
  unary_function_unary __egcd(&giac::_egcd,_egcd_s);
  unary_function_ptr at_egcd (&__egcd);

  symbolic symb_iegcd(const gen & a,const gen & b){
    return symbolic(at_iegcd,makevecteur(a,b));
  }
  gen _iegcd(const gen & args){
    check_2d_vecteur(args);
    gen u,v,d;
    egcd(args._VECTptr->front(),args._VECTptr->back(),u,v,d);
    return makevecteur(u,v,d);
  }
  const string _iegcd_s("iegcd");
  unary_function_unary __iegcd(&giac::_iegcd,_iegcd_s);
  unary_function_ptr at_iegcd (&__iegcd);

  gen _equal(const gen & a){
    if ((a.type!=_VECT) || (a._VECTptr->size()!=2))
      settypeerr();
    return equal( (*(a._VECTptr))[0],(*(a._VECTptr))[1] );
  }
  const string _equal_s("=");
  unary_function_unary __equal(&giac::_equal,_equal_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_equal (&__equal);

  string printassame(const gen & feuille,const string & sommetstr){
    if (maple_mode > 0)
      return printsommetasoperator(feuille,"=");
    else
      return printsommetasoperator(feuille,sommetstr);
  }
  symbolic symb_same(const gen & a,const gen & b){
    return symbolic(at_same,makevecteur(a,b));
  }
  gen symb_same(const gen & a){
    return symbolic(at_same,a);
  }
  gen _same(const gen & a){
    if ((a.type!=_VECT) || (a._VECTptr->size()!=2))
      return symb_same(a);
    return a._VECTptr->front()==a._VECTptr->back();
  }
  const string _same_s("==");
  unary_function_unary __same(&giac::_same,_same_s,&printassame,&texprintsommetasoperator);
  unary_function_ptr at_same (&__same);

  // ******************
  // Arithmetic functions
  // *****************

  symbolic symb_smod(const gen & a,const gen & b){
    return symbolic(at_smod,makevecteur(a,b));
  }
  gen _smod(const gen & args){
    check_2d_vecteur(args);
    vecteur & v=*args._VECTptr;
    if (ckmatrix(v))
      return apply(v[0],v[1],smod);
    if (!is_cinteger(v.front()) || !is_cinteger(v.back()) )
      return symbolic(at_smod,args);
    return smod(args._VECTptr->front(),args._VECTptr->back());
  }
  const string _smod_s("smod");
  unary_function_unary __smod(&giac::_smod,_smod_s);
  unary_function_ptr at_smod (&__smod);

  symbolic symb_rdiv(const gen & a,const gen & b){
    return symbolic(at_rdiv,makevecteur(a,b));
  }
  gen _rdiv(const gen & args){
    check_2d_vecteur(args);
    return rdiv(args._VECTptr->front(),args._VECTptr->back());
  }
  const string _rdiv_s("rdiv");
  unary_function_unary __rdiv(&giac::_rdiv,_rdiv_s);
  unary_function_ptr at_rdiv (&__rdiv);

  gen unmod(const gen & g){
    if (g.type==_MOD)
      return *g._MODptr;
    if (g.type==_VECT)
      return apply(g,unmod);
    if (g.type==_SYMB)
      return symbolic(g._SYMBptr->sommet,unmod(g._SYMBptr->feuille));
    return g;
  }
  gen irem(const gen & a,const gen & b){
    gen q;
    return irem(a,b,q);
  }
  symbolic symb_irem(const gen & a,const gen & b){
    return symbolic(at_irem,makevecteur(a,b));
  }
  gen _irem(const gen & args,GIAC_CONTEXT){
    check_2d_vecteur(args);
    if (ckmatrix(args))
      return apply(args._VECTptr->front(),args._VECTptr->back(),irem);
    gen q;
    vecteur & v=*args._VECTptr;
    if (v.front().type==_SYMB){
      gen arg=v.front()._SYMBptr->feuille;
      if (v.front()._SYMBptr->sommet==at_pow && arg.type==_VECT && arg._VECTptr->size()==2 ){
	if (is_integer(arg._VECTptr->front()) && is_integer(arg._VECTptr->back()) )
	  return powmod(_irem(gen(makevecteur(arg._VECTptr->front(),v.back()),_SEQ__VECT),contextptr),arg._VECTptr->back(),v.back());
	return pow(_irem(gen(makevecteur(arg._VECTptr->front(),v.back()),_SEQ__VECT),contextptr),arg._VECTptr->back());
      }
      if (v.front()._SYMBptr->sommet==at_prod || v.front()._SYMBptr->sommet==at_plus){
	return v.front()._SYMBptr->sommet(_irem(gen(makevecteur(v.front()._SYMBptr->feuille,v.back()),_SEQ__VECT),contextptr),contextptr);
      }
      if (v.front()._SYMBptr->sommet==at_inv){
	gen g=invmod(arg,v.back());
	if (is_positive(g,contextptr))
	  return g;
	else
	  return g+v.back();
      }
      arg=makemod(arg,v.back());
      return unmod(v.front()._SYMBptr->sommet(arg,contextptr));
    }
    if (v.front().type==_FRAC){
      gen g=invmod(v.front()._FRACptr->den,v.back());
      if (!is_positive(g,contextptr))
	g= g+v.back();
      return _irem(gen(makevecteur(v.front()._FRACptr->num*g,v.back()),_SEQ__VECT),contextptr);
    }
    if (v.front().type==_VECT){
      const_iterateur it=v.front()._VECTptr->begin(),itend=v.front()._VECTptr->end();
      vecteur res;
      for (;it!=itend;++it)
	res.push_back(_irem(gen(makevecteur(*it,v.back()),_SEQ__VECT),contextptr));
      return gen(res,v.front().subtype);
    }
    if (v.front().type==_IDNT)
      return v.front();
    if (!is_cinteger(v.front()) || !is_cinteger(v.back()) )
      return symbolic(at_irem,args);
    gen r=irem(v.front(),v.back(),q);
    return r;
  }
  const string _irem_s("irem");
  unary_function_eval __irem(&giac::_irem,_irem_s);
  unary_function_ptr at_irem (&__irem);

  const string _mods_s("mods");
  unary_function_unary __mods(&giac::_smod,_mods_s);
  unary_function_ptr at_mods (&__mods);

  gen _quote_pow(const gen & args){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      settypeerr();
    vecteur & v = *args._VECTptr;
    if (ckmatrix(v.front()))
      return pow(v.front(),v.back());
    return symbolic(at_pow,args);
  }
  const string _quote_pow_s("&^");
  unary_function_unary __quote_pow(&giac::_quote_pow,_quote_pow_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_quote_pow (&__quote_pow);

  symbolic symb_iquo(const gen & a,const gen & b){
    return symbolic(at_iquo,makevecteur(a,b));
  }
  gen _iquo(const gen & args){
    check_2d_vecteur(args);
    if (ckmatrix(args))
      return apply(args._VECTptr->front(),args._VECTptr->back(),iquo);
    vecteur & v=*args._VECTptr;
    if (!is_cinteger(v.front()) || !is_cinteger(v.back()) )
      return symbolic(at_iquo,args);
    return iquo(args._VECTptr->front(),args._VECTptr->back());
  }
  const string _iquo_s("iquo");
  unary_function_unary __iquo(&giac::_iquo,_iquo_s);
  unary_function_ptr at_iquo (&__iquo);

  vecteur iquorem(const gen & a,const gen & b){
    gen q,r;
    r=irem(a,b,q);
    return makevecteur(q,r);
  }
  symbolic symb_iquorem(const gen & a,const gen & b){
    return symbolic(at_iquorem,makevecteur(a,b));
  }
  gen _iquorem(const gen & args){
    check_2d_vecteur(args);
    vecteur & v=*args._VECTptr;
    if (!is_cinteger(v.front()) || !is_cinteger(v.back()) )
      return symbolic(at_iquorem,args);
    return iquorem(args._VECTptr->front(),args._VECTptr->back());
  }
  const string _iquorem_s("iquorem");
  unary_function_unary __iquorem(&giac::_iquorem,_iquorem_s);
  unary_function_ptr at_iquorem (&__iquorem);

  gen quorem(const gen & a,const gen & b){
    if ((a.type!=_VECT) || (b.type!=_VECT))
      return symb_quorem(a,b);
    vecteur q,r;
    environment * env=new environment;
    bool res=DivRem(*a._VECTptr,*b._VECTptr,env,q,r,true);
    delete env;
    return makevecteur(gen(q,_POLY1__VECT),gen(r,_POLY1__VECT));
  }
  symbolic symb_quorem(const gen & a,const gen & b){
    return symbolic(at_quorem,makevecteur(a,b));
  }
  gen _quorem(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      setsizeerr();
    vecteur & a =*args._VECTptr;
    if ( (a.front().type==_VECT) && (a.back().type==_VECT))
      return quorem(a.front(),a.back());
    vecteur lv;
    if (a.size()==3){
      if (a[2].type==_MOD)
	lv=vecteur(1,*a[2]._MODptr);
      else
	lv=vecteur(1,a[2]);
    }
    else
      lv=vecteur(1,vx_var);
    lvar(args,lv);
    gen aa=e2r(a[0],lv),aan,aad,bb=e2r(a[1],lv),bbn,bbd;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    fxnd(bb,bbn,bbd);
    if ( (bbd.type==_POLY) && (bbd._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    gen u,v;
    gen ad(r2e(aad,lv));
    if ( (aan.type==_POLY) && (bbn.type==_POLY) ){
      vecteur aav(polynome2poly1(*aan._POLYptr,1)),bbv(polynome2poly1(*bbn._POLYptr,1)),un,vn;
      environment * env=new environment;
      DivRem(aav,bbv,env,un,vn);
      delete env;
      vecteur lvprime(lv.begin()+1,lv.end());
      u=rdiv(r2e(bbd,lv),ad)*symb_horner(*r2e(un,lvprime)._VECTptr,lv.front());
      v=inv(ad)*symb_horner(*r2e(vn,lvprime)._VECTptr,lv.front());
      return makevecteur(u,v);
    }
    else {
      if ( (bbn.type!=_POLY) || !bbn._POLYptr->lexsorted_degree() ){
	u=rdiv(aan,bbn);
	v=zero;
      }
      else {
	u=zero;
	v=aan;
      }
    }
    // aan=u*bbn+v -> aan/aad=u*bbd/aad * bbn/bbd +v/aad
    u=r2e(u*bbd,lv);
    v=r2e(v,lv);
    return makevecteur(rdiv(u,ad),rdiv(v,ad));
  }
  const string _quorem_s("quorem");
  unary_function_unary __quorem(&giac::_quorem,_quorem_s);
  unary_function_ptr at_quorem (&__quorem);

  symbolic symb_quo(const gen & a,const gen & b){
    return symbolic(at_quo,makevecteur(a,b));
  }
  gen _quo(const gen & args){
    return _quorem(args)[0];
  }
  const string _quo_s("quo");
  unary_function_unary __quo(&giac::_quo,_quo_s);
  unary_function_ptr at_quo (&__quo);

  symbolic symb_rem(const gen & a,const gen & b){
    return symbolic(at_rem,makevecteur(a,b));
  }
  gen _rem(const gen & args){
    return _quorem(args)[1];
  }
  const string _rem_s("rem");
  unary_function_unary __rem(&giac::_rem,_rem_s);
  unary_function_ptr at_rem (&__rem);

  gen double2gen(double d){
    mpz_t * m=new mpz_t[1];
    mpz_init_set_d(*m,d);
    return m;
  }
  symbolic symb_floor(const gen & a){
    return symbolic(at_floor,a);
  }
  gen _floor(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_floor);
    if (args.type==_CPLX)
      return _floor(*args._CPLXptr,contextptr)+cst_i*_floor(*(args._CPLXptr+1),contextptr);
    if ( (args.type==_INT_) || (args.type==_ZINT))
      return args;
    if (args.type==_FRAC){
      gen n=args._FRACptr->num,d=args._FRACptr->den;
      if ( ((n.type==_INT_) || (n.type==_ZINT)) && ( (d.type==_INT_) || (d.type==_ZINT)) ){
	if (is_positive(args,contextptr))
	  return iquo(n,d);
	else
	  return iquo(n,d)-1;
      }
    }
    gen tmp=args.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    if (tmp.type==_REAL){
      gen res=real2int(tmp);
      if (is_strictly_positive(-tmp,contextptr) && !is_zero(res-tmp))
	return res-1;
      return res;
    }
    if (tmp.type!=_DOUBLE_)
      return symb_floor(args);
    return double2gen(giac_floor(tmp._DOUBLE_val));
  }
  const string _floor_s("floor");
  unary_function_eval __floor(&giac::_floor,_floor_s);
  unary_function_ptr at_floor (&__floor);

  symbolic symb_ceil(const gen & a){
    return symbolic(at_ceil,a);
  }
  gen _ceil(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_ceil);
    if (args.type==_CPLX)
      return _ceil(*args._CPLXptr,contextptr)+cst_i*_ceil(*(args._CPLXptr+1),contextptr);
    if ( (args.type==_INT_) || (args.type==_ZINT))
      return args;
    if (args.type==_FRAC){
      gen n=args._FRACptr->num,d=args._FRACptr->den;
      if ( ((n.type==_INT_) || (n.type==_ZINT)) && ( (d.type==_INT_) || (d.type==_ZINT)) )
	return iquo(n,d)+1;
    }
    gen tmp=args.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    if (tmp.type==_REAL)
      return -_floor(-tmp,contextptr);
    if (tmp.type!=_DOUBLE_)
      return symb_ceil(args);
    return double2gen(giac_ceil(tmp._DOUBLE_val));
  }
  const string _ceil_s("ceil");
  unary_function_eval __ceil(&giac::_ceil,_ceil_s);
  unary_function_ptr at_ceil (&__ceil);

  symbolic symb_round(const gen & a){
    return symbolic(at_round,a);
  }
  gen _round(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()!=2)
      return apply(args,contextptr,_round);
    if (args.type==_VECT && args.subtype==_SEQ__VECT && args._VECTptr->back().type==_INT_){
#ifdef _SOFTMATH_H
      double d=std::giac_gnuwince_pow(10.0,double(args._VECTptr->back().val));
#else
      double d=std::pow(10.0,double(args._VECTptr->back().val));
#endif
      return _round(d*args._VECTptr->front(),contextptr)/d;
    }
    if (args.type==_CPLX)
      return _round(*args._CPLXptr,contextptr)+cst_i*_round(*(args._CPLXptr+1),contextptr);
    return _floor(args+plus_one_half,contextptr);
  }
  const string _round_s("round");
  unary_function_eval __round(&giac::_round,_round_s);
  unary_function_ptr at_round (&__round);

  string printasprint(const gen & feuille,const string & sommetstr){
    if (maple_mode!=3)
      return "print("+feuille.print()+")";
    else
      return "Disp "+feuille.print();
  }
  symbolic symb_print(const gen & a){
    return symbolic(at_print,a);
  }
  gen _print(const gen & args,GIAC_CONTEXT){
    if ( debug_infolevel && (args.type==_IDNT) && (!args._IDNTptr->localvalue->empty()))
      cerr << "Local var protected " << (*args._IDNTptr->localvalue)[args._IDNTptr->localvalue->size()-2].val << endl;
    gen tmp=args.eval(DEFAULT_EVAL_LEVEL,contextptr);
    // If giac used inside a console don't add to messages, since we print
    if (!child_id){
      if (args.type==_IDNT)
        messages_to_print += args.print() + ":"; 
      messages_to_print += tmp.print() +'\n';
      // cerr << "Child " << messages_to_print << endl;
    }
    if (args.type==_IDNT)
      cerr << args << ":";
    cerr << tmp << endl;
    return __interactive.op(symbolic(at_print,tmp));
  }
  const string _print_s("print");
  unary_function_eval __print(&giac::_print,_print_s,&printasprint);
  unary_function_ptr at_print (&__print,_QUOTE_ARGUMENTS);

  symbolic symb_is_prime(const gen & a){
    return symbolic(at_is_prime,a);
  }
  gen _is_prime(const gen & args){
#ifdef HAVE_LIBPARI
    string s=pari_isprime(args);
    if (s.size()==1)
      return s;
    return string2gen(s,false);
#else
    return is_probab_prime_p(args);
#endif
  }
  const string _is_prime_s("is_prime");
  unary_function_unary __is_prime(&giac::_is_prime,_is_prime_s);
  unary_function_ptr at_is_prime (&__is_prime);

  gen _is_pseudoprime(const gen & args){
    return is_probab_prime_p(args);
  }
  const string _is_pseudoprime_s("is_pseudoprime");
  unary_function_unary __is_pseudoprime(&giac::_is_pseudoprime,_is_pseudoprime_s);
  unary_function_ptr at_is_pseudoprime (&__is_pseudoprime,0,true);

  gen nextprime1(const gen & a){
    return nextprime(a+1);
  }
  const string _nextprime_s("nextprime");
  unary_function_unary __nextprime(&giac::nextprime1,_nextprime_s);
  unary_function_ptr at_nextprime (&__nextprime);

  gen prevprime1(const gen & a){
    return prevprime(a-1);
  }
  const string _prevprime_s("prevprime");
  unary_function_unary __prevprime(&giac::prevprime1,_prevprime_s);
  unary_function_ptr at_prevprime (&__prevprime);

  symbolic symb_jacobi_symbol(const gen & a,const gen & b){
    return symbolic(at_jacobi_symbol,makevecteur(a,b));
  }
  gen _jacobi_symbol(const gen & args){
    check_2d_vecteur(args);
    return jacobi(args._VECTptr->front(),args._VECTptr->back());    
  }
  const string _jacobi_symbol_s("jacobi_symbol");
  unary_function_unary __jacobi_symbol(&giac::_jacobi_symbol,_jacobi_symbol_s);
  unary_function_ptr at_jacobi_symbol (&__jacobi_symbol);

  symbolic symb_legendre_symbol(const gen & a,const gen & b){
    return symbolic(at_legendre_symbol,makevecteur(a,b));
  }
  gen _legendre_symbol(const gen & args){
    check_2d_vecteur(args);
    return legendre(args._VECTptr->front(),args._VECTptr->back());    
  }
  const string _legendre_symbol_s("legendre_symbol");
  unary_function_unary __legendre_symbol(&giac::_legendre_symbol,_legendre_symbol_s);
  unary_function_ptr at_legendre_symbol (&__legendre_symbol);

  symbolic symb_ichinrem(const gen & a,const gen & b){
    return symbolic(at_ichinrem,makevecteur(a,b));
  }

  gen ichinrem2(const gen  & a_orig,const gen & b_orig){
    gen a=a_orig;
    gen b=b_orig;
    if (a.type==_MOD)
      a=makevecteur(*a._MODptr,*(a._MODptr+1));
    if (b.type==_MOD)
      b=makevecteur(*b._MODptr,*(b._MODptr+1));
    vecteur l(lvar(a)); lvar(b,l);
    if (l.empty()){
      check_2d_vecteur(a);
      check_2d_vecteur(b);
      vecteur & av=*a._VECTptr;
      vecteur & bv=*b._VECTptr;
      gen &ab=av.back();
      gen &bb=bv.back();
      gen &aa=av.front();
      gen &ba=bv.front();
      gen res=ichinrem(aa,ba,ab,bb);
      if (a_orig.type==_MOD)
	return makemod(res,ab*bb);
      return makevecteur(res,ab*bb);
    }
    gen x=l.front();
    if (a.type!=_VECT || b.type!=_VECT ){
      // a and b are polynomial, must have the same degrees
      // build a new polynomial calling ichinrem2 on each element
      gen ax=_e2r(makevecteur(a_orig,x)),bx=_e2r(makevecteur(b_orig,x));
      if (ax.type!=_VECT || bx.type!=_VECT )
	setsizeerr();
      int as=ax._VECTptr->size(),bs=bx._VECTptr->size();
      if (as!=bs)
	setsizeerr("Polynomial degree differ "+a.print()+" "+b.print());
      const_iterateur it=ax._VECTptr->begin(),itend=ax._VECTptr->end(),jt=bx._VECTptr->begin();
      vecteur res;
      for (;it!=itend;++it,++jt)
	res.push_back(ichinrem2(*it,*jt));
      return _r2e(makevecteur(res,x));
    }
    if (a.type==_VECT && a._VECTptr->size()==2 && b.type==_VECT && b._VECTptr->size()==2 ){
      // ax and bx are the polynomials, 
      gen ax=_e2r(makevecteur(a._VECTptr->front(),x)),bx=_e2r(makevecteur(b._VECTptr->front(),x));
      if (ax.type!=_VECT || bx.type!=_VECT )
	setsizeerr();
      gen m=a._VECTptr->back(),n=b._VECTptr->back();
      int as=ax._VECTptr->size(),bs=bx._VECTptr->size();
      if (as!=bs)
	setsizeerr("Polynomial degree differ "+a.print()+" "+b.print());
      const_iterateur it=ax._VECTptr->begin(),itend=ax._VECTptr->end(),jt=bx._VECTptr->begin();
      vecteur res;
      for (;it!=itend;++it,++jt){
	gen tmp=ichinrem2(makevecteur(*it,m),makevecteur(*jt,n));
	if (tmp.type!=_VECT)
	  setsizeerr();
	res.push_back(tmp._VECTptr->front());
      }
      if (a_orig.type==_MOD)
	return makemod(_r2e(makevecteur(res,x)),m*n);
      return makevecteur(_r2e(makevecteur(res,x)),m*n);
    }
    setsizeerr();
  }
  gen _ichinrem(const gen & args){
    if (args.type!=_VECT)
      settypeerr("[a % p, b % q,...]");
    vecteur & v = *args._VECTptr;
    int s=v.size();
    if (s<2)
      setdimerr();
    gen res=ichinrem2(v[0],v[1]);
    for (int i=2;i<s;++i)
      res=ichinrem2(res,v[i]);
    return res;
  }
  const string _ichinrem_s("ichinrem");
  unary_function_unary __ichinrem(&giac::_ichinrem,_ichinrem_s);
  unary_function_ptr at_ichinrem (&__ichinrem);
  
  gen _fracmod(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
      return symbolic(at_fracmod,args);
    vecteur & v=*args._VECTptr;
    return fracmod(v[0],v[1]);
  }
  const string _fracmod_s("fracmod");
  unary_function_unary __fracmod(&giac::_fracmod,_fracmod_s);
  unary_function_ptr at_fracmod (&__fracmod);
  
  symbolic symb_chinrem(const gen & a,const gen & b){
    return symbolic(at_chinrem,makevecteur(a,b));
  }
  vecteur polyvect(const gen & a,const vecteur & v){
    if (a.type==_POLY)
      return polynome2poly1(*a._POLYptr,1);
    return vecteur(1,a);
  }
  gen _chinrem(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      setsizeerr();
    gen a=args._VECTptr->front();
    gen b=(*args._VECTptr)[1];
    check_2d_vecteur(a);
    check_2d_vecteur(b);
    if ((a._VECTptr->front().type!=_VECT) || (a._VECTptr->back().type!=_VECT) || (b._VECTptr->front().type!=_VECT) || (b._VECTptr->back().type!=_VECT) ){
      vecteur lv;
      if (args._VECTptr->size()==3)
	lv=vecteur(1,(*args._VECTptr)[2]);
      else
	lv=vecteur(1,vx_var);
      lvar(args,lv);
      vecteur lvprime(lv.begin()+1,lv.end());
      gen aa=e2r(a,lv),bb=e2r(b,lv),aan,aad,bbn,bbd;
      fxnd(aa,aan,aad);
      if (aad.type==_POLY){
	if (aad._POLYptr->lexsorted_degree() )
	  setsizeerr();
	else
	  aad=aad._POLYptr->trunc1();
      }
      fxnd(bb,bbn,bbd);
      if (bbd.type==_POLY){
	if (bbd._POLYptr->lexsorted_degree() )
	  setsizeerr();
	else
	  bbd=bbd._POLYptr->trunc1();
      }
      vecteur & aanv=*aan._VECTptr;
      vecteur & bbnv=*bbn._VECTptr;
      aanv[0]=polyvect(aanv[0],lv)/aad;
      aanv[1]=polyvect(aanv[1],lv);
      bbnv[0]=polyvect(bbnv[0],lv)/bbd;
      bbnv[1]=polyvect(bbnv[1],lv);
      vecteur res=*_chinrem(makevecteur(aanv,bbnv))._VECTptr;
      // convert back
      res[0]=symb_horner(*r2e(res[0],lvprime)._VECTptr,lv.front());
      res[1]=symb_horner(*r2e(res[1],lvprime)._VECTptr,lv.front());
      return res;
    }
    modpoly produit=(*a._VECTptr->back()._VECTptr)**b._VECTptr->back()._VECTptr;
    return makevecteur(gen(chinrem(*a._VECTptr->front()._VECTptr,*b._VECTptr->front()._VECTptr,*a._VECTptr->back()._VECTptr,*b._VECTptr->back()._VECTptr,0),_POLY1__VECT),gen(produit,_POLY1__VECT));    
  }
  const string _chinrem_s("chinrem");
  unary_function_unary __chinrem(&giac::_chinrem,_chinrem_s);
  unary_function_ptr at_chinrem (&__chinrem);

  gen _factorial(const gen & args){
    if (args.type==_VECT)
      return apply(args,_factorial);
    if (args.type!=_INT_)
      return symbolic(at_factorial,args);
    if (args.val<0)
      return plus_inf;
    return factorial((unsigned long int) args.val);
  }
  const string _factorial_s("factorial");
  unary_function_unary __factorial(&giac::_factorial,_factorial_s);
  unary_function_ptr at_factorial (&__factorial);

  gen double_is_int(const gen & g,GIAC_CONTEXT){
    gen f=_floor(g,contextptr);
    gen f1=evalf(g-f,DEFAULT_EVAL_LEVEL,contextptr);
    if (f1.type==_DOUBLE_ && fabs(f1._DOUBLE_val)<epsilon(contextptr))
      return f;
    else
      return g;
  }
  gen comb(const gen & a_orig,const gen &b_orig,GIAC_CONTEXT){
    gen a=double_is_int(a_orig,contextptr);
    gen b=double_is_int(b_orig,contextptr);
    if (a.type!=_INT_ || b.type!=_INT_)
      return gamma(a+1)/gamma(b+1)/gamma(a-b+1);
    return comb((unsigned long int) a.val,(unsigned long int) b.val);
  }
  gen _comb(const gen & args,GIAC_CONTEXT){
    if (ckmatrix(args))
      return apply(args._VECTptr->front(),args._VECTptr->back(),contextptr,comb);
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
      return symbolic(at_comb,args);
    vecteur & v=*args._VECTptr;
    if (v.front().type!=_INT_ || v.back().type!=_INT_)
      return comb(v.front(),v.back(),contextptr); 
    if (v.front().val<v.back().val)
      return zero;
    if (v.front().val<0)
      return undef;
    return comb((unsigned long int) v.front().val,(unsigned long int) v.back().val);
  }
  const string _comb_s("comb");
  unary_function_eval __comb(&giac::_comb,_comb_s);
  unary_function_ptr at_comb (&__comb);

  gen perm(const gen & a,const gen &b){
    if (a.type!=_INT_ || b.type!=_INT_)
      return symbolic(at_perm,makevecteur(a,b));
    return perm((unsigned long int) a.val,(unsigned long int) b.val);
  }
  gen _perm(const gen & args){
    if (ckmatrix(args))
      return apply(args._VECTptr->front(),args._VECTptr->back(),perm);
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) || (args._VECTptr->front().type!=_INT_) || (args._VECTptr->back().type!=_INT_) )
      return symbolic(at_perm,args);
    if (args._VECTptr->front().val<args._VECTptr->back().val)
      return zero;
    if (args._VECTptr->front().val<0)
      return undef;
    return perm((unsigned long int) args._VECTptr->front().val,(unsigned long int) args._VECTptr->back().val);
  }
  const string _perm_s("perm");
  unary_function_unary __perm(&giac::_perm,_perm_s);
  unary_function_ptr at_perm (&__perm);

  // ******************
  // Matrix functions
  // *****************

  symbolic symb_tran(const gen & a){
    return symbolic(at_tran,a);
  }
  symbolic symb_trace(const gen & a){
    return symbolic(at_trace,a);
  }
  symbolic symb_rref(const gen & a){
    return symbolic(at_rref,a);
  }
  symbolic symb_idn(const gen & e){
    return symbolic(at_idn,e);
  }
  symbolic symb_ranm(const gen & e){
    return symbolic(at_ranm,e);
  }
  symbolic symb_det(const gen & a){
    return symbolic(at_det,a);
  }
  symbolic symb_pcar(const gen & a){
    return symbolic(at_pcar,a);
  }
  symbolic symb_ker(const gen & a){
    return symbolic(at_ker,a);
  }  
  symbolic symb_image(const gen & a){
    return symbolic(at_image,a);
  }
  symbolic symb_moyal(const gen & a,const gen & b, const gen &vars,const gen & order){
    return symbolic(at_moyal,makevecteur(a,b,vars,order));
  }

  gen _evalf(const gen & a,GIAC_CONTEXT){
    if (a.is_symb_of_sommet(at_equal)&&a._SYMBptr->feuille.type==_VECT && a._SYMBptr->feuille._VECTptr->size()==2){
      vecteur & v(*a._SYMBptr->feuille._VECTptr);
      return symbolic(at_equal,makevecteur(evalf(v.front(),DEFAULT_EVAL_LEVEL,contextptr),evalf(v.back(),DEFAULT_EVAL_LEVEL,contextptr)));
    }
    if (a.type==_VECT && a.subtype==_SEQ__VECT && a._VECTptr->size()==2 && a._VECTptr->back().type==_INT_){
      int save_decimal_digits=decimal_digits;
      set_decimal_digits(a._VECTptr->back().val);
      gen res=a._VECTptr->front().evalf(DEFAULT_EVAL_LEVEL,contextptr);
      if (res.type==_REAL || res.type==_CPLX)
	res=accurate_evalf(res,digits2bits(a._VECTptr->back().val));
      set_decimal_digits(save_decimal_digits);      
      return res;
    }
    return a.evalf(DEFAULT_EVAL_LEVEL,contextptr);
  }
  const string _evalf_s("evalf");
  unary_function_eval __evalf(&giac::_evalf,_evalf_s);
  unary_function_ptr at_evalf (&__evalf);
  symbolic symb_evalf(const gen & a){
    return symbolic(at_evalf,a);
  }

  gen _eval(const gen & a,GIAC_CONTEXT){
    if (a.is_symb_of_sommet(at_equal)&&a._SYMBptr->feuille.type==_VECT && a._SYMBptr->feuille._VECTptr->size()==2){
      vecteur & v(*a._SYMBptr->feuille._VECTptr);
      return symbolic(at_equal,makevecteur(eval(v.front(),contextptr),eval(v.back(),contextptr)));
    }
    return a.eval(DEFAULT_EVAL_LEVEL,contextptr);
  }
  const string _eval_s("eval");
  unary_function_eval __eval(&giac::_eval,_eval_s);
  unary_function_ptr at_eval (&__eval);
  symbolic symb_eval(const gen & a){
    return symbolic(at_eval,a);
  }
  
  const string _evalm_s("evalm");
  unary_function_eval __evalm(&giac::_eval,_evalm_s);
  unary_function_ptr at_evalm (&__evalm);
  
  gen _ampersand_times(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    return g._VECTptr->front()*g._VECTptr->back();
  }
  const string _ampersand_times_s("&*");
  unary_function_unary __ampersand_times(&giac::_ampersand_times,_ampersand_times_s,&printsommetasoperator,&texprintsommetasoperator);
  unary_function_ptr at_ampersand_times (&__ampersand_times);
  
  const string _subst_s("subst");
  gen _subst(const gen & args){
    if (args.type!=_VECT)
      settypeerr();
    vecteur & v = *args._VECTptr;
    int s=v.size();
    if (s==2){
      gen e=v.back();
      if (e.type==_VECT){
	vecteur & w = *e._VECTptr;
	vecteur vin,vout;
	const_iterateur it=w.begin(),itend=w.end();
	for (;it!=itend;++it){
	  if (it->type!=_SYMB)
	    continue;
	  if (it->_SYMBptr->sommet!=at_equal && it->_SYMBptr->sommet!=at_same)
	    continue;
	  vin.push_back(it->_SYMBptr->feuille._VECTptr->front());
	  vout.push_back(it->_SYMBptr->feuille._VECTptr->back());
	}
	gen res=subst(v.front(),vin,vout);
	return res;
      }
      if (e.type!=_SYMB)
	settypeerr();
      if (e._SYMBptr->sommet!=at_equal && e._SYMBptr->sommet!=at_same)
	setsizeerr();
      return subst(v.front(),e._SYMBptr->feuille._VECTptr->front(),e._SYMBptr->feuille._VECTptr->back());
    }
    if (s<3)
      toofewargs(_subst_s);
    if (s>3)
      toomanyargs(_subst_s);
    if (v[1].is_symb_of_sommet(at_equal))
      return _subst(makevecteur(v.front(),vecteur(v.begin()+1,v.end())));
    return subst(v.front(),v[1],v.back());
  }
  unary_function_unary __subst(&giac::_subst,_subst_s);
  unary_function_ptr at_subst (&__subst);
  symbolic symb_subst(const gen & a){
    return symbolic(at_subst,a);
  }

  string printassubs(const gen & feuille,const string & sommetstr){
    if (maple_mode!=1 || feuille.type!=_VECT || feuille._VECTptr->size()!=2)
      return sommetstr+"("+feuille.print()+")";
    vecteur & v=*feuille._VECTptr;
    vecteur w=mergevecteur(vecteur(1,v.back()),vecteur(v.begin(),v.end()-1));
    return sommetstr+"("+gen(w,_SEQ__VECT).print()+")";
  }  
  gen _subs(const gen & g){
    return _subst(g);
  }
  const string _subs_s("subs");
  unary_function_unary __subs(&_subs,_subs_s,&printassubs);
  unary_function_ptr at_subs (&__subs);

  string printasmaple_subs(const gen & feuille,const string & sommetstr){
    if (maple_mode==1 || feuille.type!=_VECT || feuille._VECTptr->size()<2)
      return sommetstr+"("+feuille.print()+")";
    vecteur & v=*feuille._VECTptr;
    vecteur w=mergevecteur(vecteur(1,v.back()),vecteur(v.begin(),v.end()-1));
    return sommetstr+"("+gen(w,_SEQ__VECT).print()+")";
  }  
  gen _maple_subs(const gen & g){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      return _subst(g);
    vecteur & v=*g._VECTptr;
    return _subst(makevecteur(v.back(),vecteur(v.begin(),v.end()-1)));
  }
  const string _maple_subs_s("subs");
  unary_function_unary __maple_subs(&_maple_subs,_maple_subs_s,&printasmaple_subs);
  unary_function_ptr at_maple_subs (&__maple_subs);


  string version(){
    return string("giac ")+VERSION;
  }
  gen _version(const gen & a){
    return string2gen(version(),false);
  }
  const string _version_s("version");
  unary_function_unary __version(&giac::_version,_version_s);
  unary_function_ptr at_version (&__version);

  void prod2frac(const gen & g,vecteur & num,vecteur & den){
    num.clear();
    den.clear();
    if ( (g.type!=_SYMB) || (g._SYMBptr->sommet!=at_prod) || (g._SYMBptr->feuille.type!=_VECT)){
      if (g.is_symb_of_sommet(at_division)){
	vecteur num2,den2;
	prod2frac(g._SYMBptr->feuille._VECTptr->front(),num,den);
	prod2frac(g._SYMBptr->feuille._VECTptr->back(),den2,num2);
	num=mergevecteur(num,num2);
	den=mergevecteur(den,den2);
	return;
      }
      if (g.is_symb_of_sommet(at_inv))
	prod2frac(g._SYMBptr->feuille,den,num);
      else
	num=vecteur(1,g);
      return;
    }
    vecteur & v=*g._SYMBptr->feuille._VECTptr;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_inv) )
	den.push_back(it->_SYMBptr->feuille);
      else
	num.push_back(*it);
    }
  }

  bool need_parenthesis(const gen & g){
    if (g.type==_INT_ || g.type==_ZINT)
      return is_strictly_positive(-g,0); // FIXME GIAC_CONTEXT
    if (g.type==_CPLX){
      if (is_zero(re(g,0)) || is_zero(im(g,0)) )
	return false;
      else
	return true;
    }
    if (g.type!=_FUNC)
      return false;
    unary_function_ptr & u=*g._FUNCptr;
    if (u==at_pow)
      return false;
    if (u==at_neg || u==at_minus)
      return true;
    if (!u.ptr->printsommet)
      return false;
    return true;
  }

  gen _multistring(const gen & args){
    string res;
    if (args.type==_VECT){
      const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
      for (;it!=itend;){
	if (it->type!=_STRNG)
	  break;
	res += *it->_STRNGptr;
	++it;
	if (it==itend)
	  return string2gen(res,false);;
	res += '\n';
      }
    }
    else {// newline added, otherwise Eqw_compute_size would fail
      if (args.type==_STRNG)
	res=*args._STRNGptr;
      else
	res=args.print();
      res += '\n'; 
    }
    return string2gen(res,false);
  }
  const string _multistring_s("multistring");
  unary_function_unary __multistring(&giac::_multistring,_multistring_s);
  unary_function_ptr at_multistring (&__multistring);

  // gamma function
  gen gamma(const gen & x){
    if (x.type==_INT_){
      if (x.val<=0)
	return plus_inf;
      return factorial(x.val-1);
    }
#ifdef HAVE_LIBGSL
    if (x.type==_DOUBLE_)
      return gsl_sf_gamma(x._DOUBLE_val);
#endif
    return symbolic(at_gamma,x);
  }
  gen _gamma(const gen & args) {
    return gamma(args);
  }
  const string _gamma_s("gamma");
  unary_function_unary __gamma(&_gamma,_gamma_s);
  unary_function_ptr at_gamma (&__gamma);
  // digamma function
  gen psi(const gen & x,GIAC_CONTEXT){
    if (is_positive(-x,contextptr))
      return unsigned_inf;
    if (x==plus_inf)
      return x;
    if (is_inf(x) || (is_undef(x)) )
      return undef;
    if ( (x.type==_INT_) && (x.val<10000) && (x.val>1)){
      identificateur tt(" t");
      return symbolic(at_psi,plus_one)+sum_loop(inv(tt),tt,1,x.val-1,contextptr);
    }
#ifdef HAVE_LIBGSL
    if (x.type==_DOUBLE_)
      return gsl_sf_psi(x._DOUBLE_val);
#endif
    return symbolic(at_psi,x);
  }
  // n-th derivative of digamma function
  gen psi(const gen & x,int n,GIAC_CONTEXT){
    if (n<-1)
      setsizeerr();
    if (n==-1)
      return gamma(x);
    if (is_positive(-x,contextptr))
      return unsigned_inf;
    if (is_one(x)){
      if (n%2)
	return zeta(n+1,contextptr)*factorial(n);
      else
	return -zeta(n+1,contextptr)*factorial(n);
    }
    if (x==plus_inf)
      return zero;
    if (is_inf(x) || (is_undef(x)) )
      return undef;
    if (!n)
      return psi(x,contextptr);
    if ( (x.type==_INT_) && (x.val<10000) ){
      identificateur tt(" t");
      if (n%2)
	return factorial(n)*(zeta(n+1,contextptr)-sum_loop(pow(tt,-n-1),tt,1,x.val-1,contextptr));
      else
	return -factorial(n)*(zeta(n+1,contextptr)-sum_loop(pow(tt,-n-1),tt,1,x.val-1,contextptr));
    }
#ifdef HAVE_LIBGSL
    if (x.type==_DOUBLE_)
      return gsl_sf_psi_n(n,x._DOUBLE_val);
#endif 
    return symbolic(at_psi,makevecteur(x,n));
  }
  gen _psi(const gen & args,GIAC_CONTEXT) {
    if (args.type!=_VECT)
      return psi(args,contextptr);
    if ( args._VECTptr->size()!=2 )
      return symbolic(at_psi,args);
    gen x(args._VECTptr->front()),n(args._VECTptr->back());
    if (n.type!=_INT_)
      setsizeerr();
    return psi(x,n.val,contextptr);
  }
  const string _psi_s("psi");
  unary_function_eval __psi(&_psi,_psi_s);
  unary_function_ptr at_psi (&__psi);

  gen zeta(const gen & x,GIAC_CONTEXT){
    if ( (x.type==_INT_)){
      int n=x.val;
      if (!n)
	return minus_one_half;
      if (n==1)
	return plus_inf;
      if (n<0){
	if (n%2)
	  return -rdiv(bernoulli(1-n),(1-n)) ;
	else
	  return zero;
      }
      if (n%2)
	return symbolic(at_zeta,x);
      else
	return pow(cst_pi,n)*abs(bernoulli(x),contextptr)*rdiv(pow(plus_two,n-1),factorial(n));
    }
#ifdef HAVE_LIBGSL
    if (x.type==_DOUBLE_)
      return gsl_sf_zeta(x._DOUBLE_val);
#endif // HAVE_LIBGSL
    return symbolic(at_zeta,x);
  }
  gen _zeta(const gen & args,GIAC_CONTEXT) {
    return apply(args,contextptr,zeta);
  }
  const string _zeta_s("zeta");
  unary_function_eval __zeta(&_zeta,_zeta_s);
  unary_function_ptr at_zeta (&__zeta);

  // if you add functions to solve_fcns, modify the second argument of solve_fcns_v to reflect the number of functions in the array
  unary_function_ptr solve_fcns[]={ at_exp,at_ln,at_sin,at_cos,at_tan,at_asin,at_acos,at_atan,at_sinh,at_cosh,at_tanh,at_asinh,at_acosh,at_atanh};
  vector<unary_function_ptr > solve_fcns_v(solve_fcns,solve_fcns+sizeof(solve_fcns)/sizeof(unary_function_ptr));

  // #ifndef GNUWINCE
  // #ifndef WIN32
  // #endif
  gen cst_two_pi(symbolic(at_prod,makevecteur(2,cst_pi)));
  gen cst_pi_over_2(_FRAC2_SYMB(cst_pi,2));
  gen plus_inf(symbolic(at_plus,_IDNT_infinity));
  gen minus_inf(symbolic(at_neg,_IDNT_infinity));
  // #ifndef GNUWINCE
  // #endif
  // #endif
  gen plus_one_half(fraction(1,2));
  gen minus_one_half(-symb_inv(2));
  gen plus_sqrt3(sqrt(3));
  gen plus_sqrt2(sqrt(2));
  gen minus_sqrt3_2(-_FRAC2_SYMB(plus_sqrt3,2));
  gen minus_sqrt2_2(-_FRAC2_SYMB(plus_sqrt2,2));
  gen minus_sqrt3_3(-_FRAC2_SYMB(plus_sqrt3,3));
  gen minus_sqrt3(-plus_sqrt3);
  gen minus_sqrt2(-plus_sqrt2);
  gen plus_sqrt3_2(_FRAC2_SYMB(plus_sqrt3,2));
  gen plus_sqrt2_2(_FRAC2_SYMB(plus_sqrt2,2));
  gen plus_sqrt3_3(_FRAC2_SYMB(plus_sqrt3,3));
  gen rad2deg_e(_FRAC2_SYMB(180,cst_pi));
  gen deg2rad_e(_FRAC2_SYMB(cst_pi,180));
  unary_function_ptr reim_op[]={at_inv,at_exp,at_cos,at_sin,at_tan,at_cosh,at_sinh,at_tanh,at_atan};

  // for subst.cc
  unary_function_ptr sincostan_tab[]={at_sin,at_cos,at_tan};
  unary_function_ptr asinacosatan_tab[]={at_asin,at_acos,at_atan};
  unary_function_ptr sinhcoshtanh_tab[]={at_sinh,at_cosh,at_tanh};
  vector<unary_function_ptr> sincostan_v(sincostan_tab,sincostan_tab+3);
  vector<unary_function_ptr> asinacosatan_v(asinacosatan_tab,asinacosatan_tab+3);
  vector<unary_function_ptr> sinhcoshtanh_v(sinhcoshtanh_tab,sinhcoshtanh_tab+3);
  vector <unary_function_ptr> sincostansinhcoshtanh_v(merge(sincostan_v,sinhcoshtanh_v));
  vector<unary_function_ptr> exp_v(1,at_exp);
  vector<unary_function_ptr> tan_v(1,at_tan);
  vector<unary_function_ptr> asin_v(1,at_asin);
  vector<unary_function_ptr> acos_v(1,at_acos);
  vector<unary_function_ptr> atan_v(1,at_atan);
  vector<unary_function_ptr> pow_v(1,at_pow);
  gen_op halftan_tab[]={sin2tan2,cos2tan2,tan2tan2};
  gen_op hyp2exp_tab[]={sinh2exp,cosh2exp,tanh2exp};
  gen_op trig2exp_tab[]={sin2exp,cos2exp,tan2exp};
  gen_op atrig2ln_tab[]={asin2ln,acos2ln,atan2ln};
  vector< gen_op > halftan_v(halftan_tab,halftan_tab+3);
  vector< gen_op > hyp2exp_v(hyp2exp_tab,hyp2exp_tab+3);
  vector< gen_op > trig2exp_v(trig2exp_tab,trig2exp_tab+3);
  vector< gen_op > halftan_hyp2exp_v(merge(halftan_v,hyp2exp_v));
  vector< gen_op > exp2sincos_v(1,exp2sincos);
  vector< gen_op > tan2sincos_v(1,tantosincos);
  vector< gen_op > tan2sincos2_v(1,tantosincos2);
  vector< gen_op > tan2cossin2_v(1,tantocossin2);
  vector< gen_op > asin2acos_v(1,asintoacos);
  vector< gen_op > asin2atan_v(1,asintoatan);
  vector< gen_op > acos2asin_v(1,acostoasin);
  vector< gen_op > acos2atan_v(1,acostoatan);
  vector< gen_op > atan2asin_v(1,atantoasin);
  vector< gen_op > atan2acos_v(1,atantoacos);
  vector< gen_op > atrig2ln_v(atrig2ln_tab,atrig2ln_tab+3);
  vector< gen_op > trigcos_v(1,trigcospow);
  vector< gen_op > trigsin_v(1,trigsinpow);
  vector< gen_op > trigtan_v(1,trigtanpow);
  vector< gen_op > powexpand_v(1,powtopowexpand);
  vector< gen_op > exp2power_v(1,exptopower);
  vector< unary_function_ptr > gamma_v(1,at_gamma);
  vector< gen_op > gamma2factorial_v(1,gammatofactorial);

  // for integration
  unary_function_ptr primitive_tab_op[]={at_sin,at_cos,at_tan,at_exp,at_sinh,at_cosh,at_tanh,at_asin,at_acos,at_atan,at_ln,0};
  unary_function_ptr inverse_tab_op[]={at_asin,at_acos,at_atan,at_ln,at_asinh,at_acos,at_atanh,0};

  unary_function_ptr analytic_sommets[]={at_plus,at_prod,at_neg,at_inv,at_pow,at_sin,at_cos,at_tan,at_exp,at_sinh,at_cosh,at_tanh,at_asin,at_acos,at_atan,at_asinh,at_atanh,at_acosh,at_ln,at_sqrt,0};

  // test if g is < > <= >=, 
  unary_function_ptr inequality_tab[]={at_equal,at_inferieur_strict,at_inferieur_egal,at_different,at_superieur_strict,at_superieur_egal};
  vector<unary_function_ptr> inequality_sommets(inequality_tab,inequality_tab+sizeof(inequality_tab)/sizeof(unary_function_ptr));
  int is_inequality(const gen & g){
    if (g.type!=_SYMB)
      return false;
    return equalposcomp(inequality_sommets,g._SYMBptr->sommet);
  }


  string unquote(const string & s){
    int l=s.size();
    if (l>2 && s[0]=='"' && s[l-1]=='"')
      return s.substr(1,l-2);
    else
      return s;
  }


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
