/* -*- mode:C++ ; compile-command: "g++ -I.. -I../include -g -c -DGNUPLOT_IO plot.cc" -*- */
// NB: Using gnuplot optimally requires patching and recompiling gnuplot
// If you use the -DGNUPLOT_IO compile flag, you
// MUST compile gnuplot with interactive mode enabled, file src/plot.c
// line 448
/*
diff plot.c plot.c~
448c448
<     interactive = TRUE; // isatty(fileno(stdin));
---
>     interactive = isatty(fileno(stdin));
*/

/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "first.h"
using namespace std;
#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>
#include <cmath>

// C headers
#include <stdio.h>
#include <time.h> // for nanosleep
#ifndef __VISUALC__
#include <fcntl.h>
#endif // __VISUALC__

// Giac headers
#include "gen.h"
#include "usual.h"
#include "plot.h"
#include "prog.h"
#include "rpn.h"
#include "identificateur.h"
#include "subst.h"
#include "symbolic.h"
#include "derive.h"
#include "solve.h"
#include "intg.h"
#include "path.h"
#include "sym2poly.h"
#include "input_parser.h"
#include "input_lexer.h"
#include "ti89.h"
#include "isom.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

#ifdef IPAQ
  int LEGENDE_SIZE=40;
  int COORD_SIZE=26;
  int PARAM_STEP=18;
#else // IPAQ
  int LEGENDE_SIZE=90;
  int COORD_SIZE=30;
  int PARAM_STEP=20;
#endif // IPAQ

#ifdef GNUWINCE
  ofstream * outptr=0;
#endif
  logo_turtle turtle;
  std::vector<logo_turtle> turtle_stack(vector<logo_turtle>(1,turtle));
  vecteur plot_instructions,graph_instructions;
  int plot_instructionsh=300,plot_instructionsw=300;
  bool shift_on=false;
  bool run_modif=false;
#ifdef WIN32
  bool win9x=true; // For win95/98/Me we can not pipe to gnuplot
#else
  bool win9x=false;
#endif
  int run_modif_pos;
  pid_t gnuplot_pid=0;
  string gnuplot_name(giac_gnuplot_location);
  int gnuplot_fileno=0;
  bool gnuplot_do_splot=true;
  int gnuplot_in_pipe[2]={-1,-1},gnuplot_out_pipe[2]={-1,-1};
  FILE * gnuplot_out_readstream=0;

#ifdef IPAQ
  bool gnuplot_hidden3d=false;
#else
  bool gnuplot_hidden3d=true;
#endif
  bool gnuplot_pm3d=false;
  int gnuplot_wait_times=5;

  string gnuplot_filename("session3d");
  int _GROUP__VECT_subtype[]={_GROUP__VECT,_LINE__VECT,_HALFLINE__VECT,_VECTOR__VECT,_POLYEDRE__VECT,_POINT__VECT,0};
#ifdef IPAQ
  double gnuplot_xmin=-4,gnuplot_xmax=4,gnuplot_ymin=-5,gnuplot_ymax=2,gnuplot_zmin=-10,gnuplot_zmax=10,gnuplot_tmin=-6,gnuplot_tmax=6,gnuplot_tstep=0.07;
#else
  double gnuplot_xmin=-5,gnuplot_xmax=5,gnuplot_ymin=-5,gnuplot_ymax=2,gnuplot_zmin=-10,gnuplot_zmax=10,gnuplot_tmin=-6,gnuplot_tmax=6,gnuplot_tstep=0.07;
#endif
  double global_window_xmin(gnuplot_xmin),global_window_xmax(gnuplot_xmax),global_window_ymin(-2),global_window_ymax(gnuplot_ymax);
  double x_tick(1.0),y_tick(1.0);
  double class_minimum(0.0),class_size(1.0);
  int gnuplot_pixels_per_eval=2;
  bool autoscale=true;
  bool show_axes=true;
  bool has_gnuplot=true;
  bool as_function_of_mode=false;
  bool redraw_mouse_position_only=false;
  bool fastcurveprint=false;
  string PICTautoname("A");

  gen do_point3d(const gen & g){
    gen tmp(g);
    if (tmp.type==_VECT)
      tmp.subtype=_POINT__VECT;
    return tmp;
  }

  vecteur rand_3d(){
    int i=rand(),j=rand(),k=rand();
    i=i/(RAND_MAX/10)-5;
    j=j/(RAND_MAX/10)-5;
    k=k/(RAND_MAX/10)-5;
    return makevecteur(i,j,k);
  }
  gen rand_complex(){
    int i=rand(),j=rand();
    i=i/(RAND_MAX/10)-5;
    j=j/(RAND_MAX/10)-5;
    return i+cst_i*j;
  }

  gen abs_norm(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return sqrt(dotvecteur(*g._VECTptr,*g._VECTptr));
    else
      return abs(g,contextptr);
  }

  gen abs_norm2(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return dotvecteur(*g._VECTptr,*g._VECTptr);
    else
      return g*conj(g,contextptr);
  }

  void set3derr(){
    setsizeerr("2-d instruction");
  }

  void PICTautoname_plus_plus(){
    int labs=PICTautoname.size();
    if (!labs){
      PICTautoname="A";
      labs=1;
    }
    for (;labs>0;--labs){
      char c=PICTautoname[labs-1];
      if (c!='Z'){ // skip D, I=sqrt(-1), i, e
	switch (c){
	case 'C': case 'H': case 'h': case 'd':
	  ++c;
	default:
	  PICTautoname[labs-1]=char(c+1);
	}
	break;
      }
      PICTautoname[labs-1]='A';
    }
    if (!labs)
      PICTautoname += 'A';
  }

  // find last at_erase in history, return next position (or 0 if not found)
  int erase_pos(){
    int s=history_out.size();
    if (!s)
      return s;
    gen e;
    for (int i=s-1;i>=0;--i){
      e=history_out[i];
      if ( (e.type==_SYMB && (e._SYMBptr->sommet==at_erase 
			      // || equalposcomp(implicittex_plot_sommets,e._SYMBptr->sommet)
			      )) ||
	   (e.type==_FUNC && (*e._FUNCptr==at_erase 
			      // || equalposcomp(implicittex_plot_sommets,e._SYMBptr->sommet)
			      ) ) )
	return i+1;
    }
    return 0;
    // int i=history_out.find_last_of(at_erase);
  }

  int erase_pos(int current){
    int s=history_out.size();
    if (current>=s)
      current=s-1;
    if (!s)
      return s;
    gen e;
    for (int i=current;i>=0;--i){
      e=history_out[i];
      if ( (e.type==_SYMB && (e._SYMBptr->sommet==at_erase 
			      // || equalposcomp(implicittex_plot_sommets,e._SYMBptr->sommet)
			      )) ||
	   (e.type==_FUNC && (*e._FUNCptr==at_erase 
			      // || equalposcomp(implicittex_plot_sommets,e._SYMBptr->sommet)
			      ) ) )
	return i+1;
    }
    return 0;
    // int i=history_out.find_last_of(at_erase);
  }


  gen remove_at_pnt(const gen & e){
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt))
      return e._SYMBptr->feuille._VECTptr->front();
    else
      return e;
  }

  gen remove_sto(const gen & e){
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_sto))
      return e._SYMBptr->feuille._VECTptr->back();
    else
      return e;
  }

  vecteur selection2vecteur(const vector<int> & selected){
    int i=erase_pos();
    vecteur res;
    vector<int>::const_iterator it=selected.begin(),itend=selected.end();
    for (;it!=itend;++it){
      gen e=history_in[i+*it];
      if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_sto))
	res.push_back(e._SYMBptr->feuille._VECTptr->back());
      else
	res.push_back(e);
    }
    return res;
  }

  vecteur selection2vecteureval(const vector<int> & selected){
    int i=erase_pos();
    vecteur res;
    vector<int>::const_iterator it=selected.begin(),itend=selected.end();
    for (;it!=itend;++it){
      gen e=history_out[i+*it];
      res.push_back(e);
    }
    return res;
  }

  bool is_segment(const gen & e){
    gen f=remove_at_pnt(e);
    if ( (f.type!=_VECT) || (f._VECTptr->size()!=2))
      return false;
    return true;
  }

  int findfirstpoint(const vecteur & v){
    const_iterateur it=v.begin(),itend=v.end();
    gen p;
    for (;it!=itend;++it){
      p=*it;
      if ( (p.type==_SYMB) && (p._SYMBptr->sommet==at_pnt)){
	p=p._SYMBptr->feuille._VECTptr->front();
	if ( (p.type!=_VECT) && ( (p.type!=_SYMB) || (p._SYMBptr->sommet!=at_cercle)))
	  return it-v.begin();
      }
    }
    return -1;
  }

  int findfirstcercle(const vecteur & v){
    const_iterateur it=v.begin(),itend=v.end();
    gen p;
    for (;it!=itend;++it){
      p=*it;
      if ( (p.type==_SYMB) && (p._SYMBptr->sommet==at_pnt)){
	p=p._SYMBptr->feuille._VECTptr->front();
	if ( (p.type==_SYMB) && (p._SYMBptr->sommet==at_cercle) )
	  return it-v.begin();
      }
    }
    return -1;
  }

  // find best int in selected (and modify selected)
  // p is the pointed mouse point, eps the precision
  // try_perp=-1 if no try of perp line, =an history_position otherwise
  bool find_best(vector<int> & selected,const gen & p,double eps,int try_perp_history_pos,int & pnt_pos,int & history_position,gen & res){
    vecteur v=selection2vecteur(selected),w=selection2vecteureval(selected);
    pnt_pos=findfirstpoint(w);
    if (pnt_pos<0){ // no point nearby, try to make one
      // try intersection or midpoint
      int nsegments=0,n1=-1,n2=-1;
      const_iterateur it=w.begin(),itend=w.end();
      for (;it!=itend;++it){
	if (is_segment(*it)){
	  ++nsegments;
	  n2=it-w.begin();
	  if (n1<0)
	    n1=n2;
	}
      }
      if (nsegments>=2){
	history_position=erase_pos()+selected[n1];
	gen l1=remove_sto(history_in[history_position]);
	history_position=erase_pos()+selected[n2];
	gen l2=remove_sto(history_in[history_position]);
	res = symbolic(at_head,symbolic(at_inter,makevecteur(l1,l2)));
	selected=vector<int>(2);
	selected[0]=n1;
	selected[1]=n2;
	return true;
      }
      if (nsegments){ // near middle point?
	gen m=remove_at_pnt(_milieu(w[n1],0));
	if (is_greater(eps,abs(p-m,0),0)){
	  history_position=erase_pos()+selected[n1];
	  gen l1=remove_sto(history_in[history_position]);
	  res=symb_milieu(l1);
	  selected=vector<int>(1,n1);
	  return true;
	}
	// near perp?
	if (try_perp_history_pos>=0){
	  m=remove_at_pnt(history_out[try_perp_history_pos]);
	  gen pm=p-m;
	  gen wv=remove_at_pnt(w[n1]);
	  wv=wv._VECTptr->front()-wv._VECTptr->back();
	  if (is_greater(eps*abs(pm,0)*abs(wv,0),scalar_product(pm,wv,0),0)){
	    history_position=erase_pos()+selected[n1];
	    gen l1=remove_sto(history_in[history_position]);
	    res=symb_perpendiculaire(makevecteur(remove_sto(history_in[try_perp_history_pos]),l1));
	    res=symbolic(at_head,symbolic(at_inter,makevecteur(l1,res)));
	    selected=vector<int>(1,n1);
	    return true;
	  }
	}
      }
      // try a cercle
      pnt_pos=findfirstcercle(w);
      if (pnt_pos<0)
	return false;
      history_position=erase_pos()+selected[pnt_pos];
      res=remove_sto(history_in[history_position]);
      return true;
    }
    else {
      history_position=erase_pos()+selected[pnt_pos];
      res=remove_sto(history_in[history_position]);
      return true;
    }
  }

  int equalposcomp(const vector<int> v,int i){
    vector<int>::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it)
      if (*it==i)
	return it-v.begin()+1;
    return 0;
  }

  int equalposcomp(int tab[],int f){
    for (int i=1;*tab!=0;++tab,++i){
      if (*tab==f)
	return i;
    }
    return 0;
  }

  void streamcopy(FILE * source,FILE * target){
    char c;
    for (;!feof(source);){
      c=fgetc(source);
      if (!feof(source))
	fputc(c,target);
    }
    fclose(source);
  }

#ifndef __VISUALC__

  int set_cloexec_flag (int desc, int value){
    int oldflags = fcntl (desc, F_GETFD, 0);
    /* If reading the flags failed, return error indication now.
       if (oldflags < 0)
       return oldflags;
       /* Set just the flag we want to set. */
    if (value != 0)
      oldflags |= FD_CLOEXEC;
    else
      oldflags &= ~FD_CLOEXEC;
    /* Store modified flag word in the descriptor. */
    return fcntl (desc, F_SETFD, oldflags);
  }

  bool ck_gnuplot(string & s){
    if (!access(s.c_str(),R_OK))
      return true;
    if (!access("/usr/local/bin/gnuplot",R_OK)){
      s="/usr/local/bin/gnuplot";
      return true;
    }
    if (!access("/usr/bin/gnuplot",R_OK)){
      s="/usr/bin/gnuplot";
      return true;
    }
    if (!access((xcasroot+"gnuplot").c_str(),R_OK)){
      s=xcasroot+"gnuplot";
      return true;
    }
    return false;
  }

  int run_gnuplot(int & r){
#ifdef WITH_GNUPLOT
#ifdef WIN32
#ifdef GNUWINCE
    has_gnuplot=false;
    return -1;
#endif
    gnuplot_name=xcasroot+"pgnuplot.exe";
#endif //WIN32
    if (!gnuplot_pid){ // start a gnuplot process
      if (!ck_gnuplot(gnuplot_name))
	{
	  has_gnuplot=false;
	  return -1;
	}
      if (pipe(gnuplot_in_pipe) || pipe(gnuplot_out_pipe))
	throw(std::runtime_error("Plot error: Unable to create pipe"));
      gnuplot_pid=fork();
      if (gnuplot_pid<(pid_t) 0)
	throw(std::runtime_error("Plot error: Unable to fork"));
      if (!gnuplot_pid){ // child process, redirect input/output
#ifndef GNUPLOT_IO
	int gnuplot_out=open("gnuplot.out",O_WRONLY | O_CREAT); 
	dup2(gnuplot_out,STDERR_FILENO);
	close(gnuplot_out);
#else
	dup2(gnuplot_out_pipe[1],STDERR_FILENO);
	close(gnuplot_out_pipe[1]);
	set_cloexec_flag(STDERR_FILENO,0);
#endif
	dup2(gnuplot_in_pipe[0],STDIN_FILENO);
	close(gnuplot_in_pipe[0]);
	// cout << errno << endl;
#ifdef IPAQ
	execlp("gnuplot","gnuplot -geometry 235x300 -noraise",0);
#else
	execlp("gnuplot","gnuplot -geometry 500x700 -noraise",0); // start gnuplot
#endif
	exit(1); // in case gnuplot is not found
      }
      else {
	usleep(10000);
	bool clrplot=false;
	FILE * stream = open_gnuplot(clrplot,gnuplot_out_readstream,r);
	gnuplot_out_readstream=fdopen(r,"r");
	gnuplot_wait(r,gnuplot_out_readstream,3);
	// close(gnuplot_out_pipe[1]);
	// close(gnuplot_in_pipe[0]);
      }
    }
    r=gnuplot_out_pipe[0];
    return gnuplot_in_pipe[1];
#endif // WITH_GNUPLOT
    return 0;
  }

  FILE * open_gnuplot(bool & clrplot,FILE * & gnuplot_out_r,int & r){
    if (win9x){
      r=-1;
#ifdef WIN32
      gnuplot_name="'"+xcasroot+"wgnuplot.exe'"; // was xcasroot+"wgnuplot.exe", 
#ifdef GNUWINCE 
      gnuplot_name="/gnuplot.exe";
#endif
#endif
      FILE * stream= fopen("gnuplot.txt","a");
      if (!stream)
	stream=fopen("gnuplot.txt","w");
      if (!stream)
	setsizeerr("Gnuplot write error");
      return stream;
    }
    FILE * res=fdopen(run_gnuplot(r),"w");
    if (!res)
      setsizeerr("Gnuplot write error");
#ifdef __APPLE__
    if (debug_infolevel)
      printf("set terminal aqua\n");
    fprintf(res,"set terminal aqua\n");
#endif
    // cerr << errno << endl;
    gnuplot_out_r = gnuplot_out_readstream;
    // cerr << r << " " << errno << endl;
    return res;
  }

  void terminal_stream_replot(const char * terminal,FILE * stream,const char * filename){
#ifdef IPAQ
    if (string(terminal)=="png")
      return;
#endif
#ifdef __APPLE__
    if (string(terminal)=="png")
      return;
#endif
    fprintf(stream,"set terminal %s\nset output \"%s\"\nreplot\nset output\n",terminal,filename);
    if (debug_infolevel)
      printf("set terminal %s\nset output \"%s\"\nreplot\nset output\n",terminal,filename);
#ifdef WIN32
    if (debug_infolevel)
      printf("set terminal windows\n"); 
    fprintf(stream,"set terminal windows\n");
#else
#ifdef __APPLE__
    if (debug_infolevel)
      printf("set terminal aqua\n");
    fprintf(stream,"set terminal aqua\n");
#else
    if (debug_infolevel)
      printf("set terminal x11\n"); 
    fprintf(stream,"set terminal x11\n");
#endif
#endif
    fflush(stream);
  }

  void terminal_stream_replot(const char * terminal,FILE * stream,int i,const char * file_extension){
    terminal_stream_replot(terminal,stream,(gnuplot_filename+print_INT_(i)+"."+string(file_extension)).c_str());
  }

  int set_nonblock_flag (int desc, int value){
    int oldflags = fcntl (desc, F_GETFL, 0);
    /* If reading the flags failed, return error indication now. */
    if (oldflags == -1)
      return -1;
    /* Set just the flag we want to set. */
    if (value != 0)
      oldflags |= O_NONBLOCK;
    else
      oldflags &= ~O_NONBLOCK;
    /* Store modified flag word in the descriptor. */
    return fcntl (desc, F_SETFL, oldflags);
  }

  void gnuplot_wait(int handle,FILE * gnuplot_out_readstream,int ngwait){
#ifndef GNUPLOT_IO
    usleep(200000);
    return;
#endif
    // cerr << "gnuplot_wait " << handle << " " << gnuplot_out_readstream << " " << ngwait << endl;
    // wait for gnuplot to be quiet
    if (handle>0 && gnuplot_out_readstream){
      int i,ntry=500;
      for (;ntry;){
	usleep(10000);
	set_nonblock_flag(handle,1); 
	i=fgetc(gnuplot_out_readstream);
	// cerr << char(i) ;
	if (char(i)=='>')
	  --ngwait;
	set_nonblock_flag(handle,0); 
	if (i==EOF){
	  if (ngwait<=0)
	    break;
	  usleep(10000);
	  --ntry;
	}
	set_nonblock_flag(handle,1); 
	while (i!=EOF){
	  i=fgetc(gnuplot_out_readstream);
	  // cerr << char(i);
	  if (char(i)=='>')
	    --ngwait;
	}
	set_nonblock_flag(handle,0); 
      }
      // fclose(gnuplot_out_readstream);
      usleep(10000);
      // cerr << "gnuplot_wait end " << getpid() << " " <<clock() << endl;
    }
    else
      ;// cerr << "gnuplot wait no input" << endl;
  }

  bool terminal_replot(const char * terminal,int i,const char * file_extension){
    if (!has_gnuplot)
      return 0;
#ifdef WITH_GNUPLOT
    if (win9x){
      FILE * stream2 =fopen("gnuplot.gp","w");
      FILE * stream =fopen("gnuplot.txt","r");
      streamcopy(stream,stream2);
      terminal_stream_replot(terminal,stream2,i,file_extension);
      system((gnuplot_name+" gnuplot.gp").c_str());
    }
    else {
      bool clrplot;
      int r;
      FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,r);
      terminal_stream_replot(terminal,stream,i,file_extension);
      gnuplot_wait(r,gnuplot_out_readstream,gnuplot_wait_times);
    }
#endif
    return true;
  }

  bool terminal_replot(const char * terminal,const string & s){
    if (!has_gnuplot)
      return 0;
#ifdef IPAQ
    if (string(terminal)=="png")
      return 0;
#endif
#ifdef __APPLE__
    if (string(terminal)=="png")
      return 0;
#endif
#ifdef WITH_GNUPLOT
    if (win9x){
      FILE * stream = fopen("gnuplot.txt","r");
      FILE * stream2 =fopen("gnuplot.gp","w");
      streamcopy(stream,stream2);
      terminal_stream_replot(terminal,stream2,s.c_str());
      system((gnuplot_name+" gnuplot.gp").c_str());
    }
    else {
      bool clrplot;
      int r;
      FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,r);
      terminal_stream_replot(terminal,stream,s.c_str());
      gnuplot_wait(r,gnuplot_out_readstream,gnuplot_wait_times);
    }
#endif
    return true;
}

  void win9x_gnuplot(FILE * stream){
    if (!win9x){
#ifndef IPAQ
#ifndef __APPLE__
      latex_replot(stream,(gnuplot_filename+print_INT_(gnuplot_fileno)+".tex").c_str());
      terminal_stream_replot("png",stream,gnuplot_fileno,"png");
#endif
#endif
      fflush(stream);
      return;
    }
    fflush(stream);
    fclose(stream);
#ifdef GNUWINCE
    (*outptr) << "// <plot> </plot>" << endl;
    return ;
#endif
    // Copy to a temporary gnuplot
    unlink("gnuplot.gp");
    FILE * stream2 = fopen("gnuplot.gp","w");
    stream = fopen("gnuplot.txt","r");
    streamcopy(stream,stream2);
    fprintf(stream2,"pause -1 \"Press RETURN when OK\"\n");
    latex_replot(stream2,(gnuplot_filename+print_INT_(gnuplot_fileno)+".tex").c_str());
    terminal_stream_replot("png",stream2,gnuplot_fileno,"png");
    fflush(stream2);
    fclose(stream2);
    system((gnuplot_name+" gnuplot.gp").c_str());
  }

  void kill_gnuplot(){
#ifdef WITH_GNUPLOT
    if (gnuplot_pid){
      FILE *stream;
      stream = fdopen (gnuplot_in_pipe[1], "w");
      fprintf(stream,"\n\nq\n");
      if (fclose(stream))
	cerr << "Error closing gnuplot" << endl;
    }
#endif
  }

#else // __VISUALC__

  int run_gnuplot(int & r){
    has_gnuplot=false;
    return -1;
  }

  void kill_gnuplot(){
  }

  bool ck_gnuplot(string & s){
    return false;
  }

  FILE * open_gnuplot(bool & clrplot,FILE * & gnuplot_out_r,int & r){
    FILE * f=fopen("gnuplot.in","w");
    r=-1;
    return f;
  }
  
  void terminal_stream_replot(const char * terminal,FILE * stream,const char * filename){
  }

  void gnuplot_wait(int handle,FILE * gnuplot_out_readstream,int ngwait){
  }

  bool terminal_replot(const char * terminal,int i,const char * file_extension){
    return false;
  }

  bool terminal_replot(const char * terminal,const string & s){
    return false;
  }

  void win9x_gnuplot(FILE * stream){
  }

  int set_nonblock_flag (int desc, int value){
    return value;
  }
#endif // __VISUALC__

  bool png_replot(int i){
    return terminal_replot("png",i,"png");
  }

  bool png_replot(const string & s){
    return terminal_replot("png",s);
  }

  string gnuplot_traduit(const gen & g){
    string s(evalf(g).print());
    string f_s(s),ff_s;
    string::iterator it=f_s.begin(),itend=f_s.end();
    for (;it!=itend;++it){
      if (*it=='^')
	ff_s += "**";
      else
	ff_s += *it;
    }
    return  ff_s;
  }

  void plotpreprocess(gen & g,vecteur & quoted,GIAC_CONTEXT){
    gen tmp=eval(g,contextptr);
    if (tmp.type==_IDNT){
      g=tmp;
      quoted=vecteur(1,tmp);
      return;
    }
    if (tmp.type==_VECT){
      bool done=true;
      const_iterateur it=tmp._VECTptr->begin(),itend=tmp._VECTptr->end();
      if (it!=itend){
	for (;it!=itend;++it){
	  if (it->type!=_IDNT && !it->is_symb_of_sommet(at_at))
	    break;
	}
	if (it==itend){
	  g=tmp;
	  quoted=*tmp._VECTptr;
	}
        else
	  done=false;
      }
      else
	done=false;
      if (!done){
	if (g.type==_VECT)
	  quoted=*g._VECTptr;
	else
	  quoted=vecteur(1,g);
      }
    }
    else {
      quoted=vecteur(1,g);
    }
  }

  // args -> vector
  // add vx_var if args is not a seq
  // evaluate v[1], if it's not an idnt or a vector of idnt keep v[1]
  // evaluate v with v[1] quoted
  vecteur plotpreprocess(const gen & args,GIAC_CONTEXT){
    vecteur v;
    if ((args.type!=_VECT) || (args.subtype!=_SEQ__VECT) )
      v=makevecteur(args,vx_var);
    else {
      v=*args._VECTptr;
      if (v.empty())
	setsizeerr();
      if (v.size()==1)
	v.push_back(vx_var);
    }
    vecteur quoted;
    if ( v[1].type==_SYMB && (v[1]._SYMBptr->sommet==at_equal ||v[1]._SYMBptr->sommet==at_same ))
      plotpreprocess(v[1]._SYMBptr->feuille._VECTptr->front(),quoted,contextptr);
    else
      plotpreprocess(v[1],quoted,contextptr);
    const_iterateur it=quoted.begin(),itend=quoted.end();
    vector<int> save;
    for (;it!=itend;++it){
      if (it->type!=_IDNT)
	save.push_back(-1);
      else {
	if (contextptr && contextptr->quoted_global_vars){
	  contextptr->quoted_global_vars->push_back(*it);
	  save.push_back(0);
	}
	else {
	  save.push_back(*it->_IDNTptr->quoted);
	  *it->_IDNTptr->quoted=true;
	}
      }
    }
    try {
      v=*eval(v)._VECTptr;
    } catch (std::runtime_error & e){
      cerr << e.what() << endl;
    }
    it=quoted.begin();
    for (int i=0;it!=itend;++it,++i){
      if (save[i]>=0){
	if (contextptr && contextptr->quoted_global_vars)
	  contextptr->quoted_global_vars->pop_back();
	else
	  *it->_IDNTptr->quoted=save[i];
      }
    }
    return v;
  }

  void reset_gnuplot_hidden3d(FILE * stream){
#ifndef GNUWICE
    if (gnuplot_hidden3d){
      if (debug_infolevel)
	printf("\nset hidden3d nooffset\nset isosamples 16\n");
      fprintf(stream,"\nset hidden3d nooffset\nset isosamples 16\n");
    }
    else {
      if (debug_infolevel)
	printf("\nunset hidden3d nooffset\nset isosamples 10\n"); 
      fprintf(stream,"\nunset hidden3d nooffset\nset isosamples 10\n");
    }
#endif
    if (debug_infolevel)
      printf("set parametric\n");
    fprintf(stream,"set parametric\n");
    // Problems with history inclusion, see if it's a FLTK problem
#ifndef GNUWINCE // GNUWINCE gnuplot is 3.7, no pm3d
    if (gnuplot_pm3d){
      if (debug_infolevel)
	printf("set pm3d\n");
      fprintf(stream,"set pm3d\n");
    }
    else {
      if (debug_infolevel)
	printf("unset pm3d\n");
      fprintf(stream,"unset pm3d\n");
    }
#endif // GNUWINCE
  }

  gen plotfunc(const gen & f,const gen & vars,const gen & attribut,bool clrplot,double function_xmin,double function_xmax,double function_ymin,double function_ymax,double function_zmin, double function_zmax,int function_pixels_per_eval,const context * contextptr){
    vecteur res;
    int color=default_color(contextptr);
    if (attribut.type==_INT_)
      color=attribut.val;
#ifndef GNUWINCE
    if (vars.type==_IDNT){ // function plot
      if (f.type==_VECT){ // multi-plot
	vecteur & vf=*f._VECTptr;
	int s=vf.size();
	vecteur vattribut;
	if (attribut.type==_VECT)
	  vattribut=gen2vecteur(attribut);
	for (int j=0;j<s;++color,++j)
	  vattribut.push_back(color);
	vecteur res;
	for (int i=0;i<s;++i){
	  gen tmp=plotfunc(vf[i],vars,vattribut[i],false,function_xmin,function_xmax,function_ymin,function_ymax,function_zmin,function_zmax,function_pixels_per_eval,contextptr);
	  if (tmp.type==_VECT)
	    res=mergevecteur(res,*tmp._VECTptr);
	}
	return gen(res,_SEQ__VECT);
      }
      _Pictsize(0);
      int horizontal_pixels=plot_instructionsw-LEGENDE_SIZE,vertical_pixels=plot_instructionsh;
      double step=function_pixels_per_eval*(function_xmax-function_xmin)/(horizontal_pixels-1);
      gen locvar(vars);
      locvar.subtype=0;
      gen y=quotesubst(f,vars,locvar),yy;
      // gen y=f.evalf_double(),yy;
      double j,entrej,oldj,xmin=function_xmin,xmax=function_xmax;
      bool joindre;
      vecteur localvar(1,vars);
      context * newcontextptr= (context *) contextptr;
      int protect=bind(vecteur(1,xmin),localvar,newcontextptr);
      vecteur chemin;
      for (double i=xmin;i<xmax;i+= step,vars._IDNTptr->localvalue->back()._DOUBLE_val =i){
	yy=y.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
	if (yy.type!=_DOUBLE_)
	  continue;
	j=yy._DOUBLE_val;
	if (i!=xmin){
	  if (fabs(oldj-j)>(function_ymax-function_ymin)/5){ // try middle-pnt
	    vars._IDNTptr->localvalue->back()._DOUBLE_val -= step/2;
	    yy=y.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
	    if (yy.type!=_DOUBLE_)
	      joindre=false;
	    else {
	      entrej=yy._DOUBLE_val;
	      if (j>oldj)
		joindre=(j>=entrej) && (entrej>=oldj);
	      else
		joindre=(j<=entrej) && (entrej<=oldj);
	    }
	    vars._IDNTptr->localvalue->back()._DOUBLE_val += step/2;
	  }
	  else
	    joindre=true;
	}
	else
	  joindre=false;
	if (joindre)
	  chemin.push_back(gen(i,j));
	else {
	  if (!chemin.empty())
	    res.push_back(symb_pnt(symb_curve(gen(makevecteur(vars+cst_i*f,vars,xmin,i),_PNT__VECT),gen(chemin,_GROUP__VECT)),color,contextptr));
	  xmin=i;
	  chemin=vecteur(1,gen(i,j));
	}
	oldj=j;
      }
      if (!chemin.empty())
	res.push_back(symb_pnt(symb_curve(gen(makevecteur(vars+cst_i*f,vars,xmin,xmax),_PNT__VECT),gen(chemin,_GROUP__VECT)),color,contextptr));
      leave(protect,localvar,newcontextptr);
#ifndef WIN32
      //      if (child_id)
      //	plot_instructions.push_back(res);
#endif // WIN32
      gen e(res,_SEQ__VECT);
      return e;
    } // end 1-var function plot
#endif
    if (!has_gnuplot)
      return symb_plotfunc(f,vars);
    ck_parameter_x();
    ck_parameter_y();
    int s=0;
    gen var1,var2;
    if (vars.type==_VECT){
      s=vars._VECTptr->size();
      if (s>2)
	s=0;
      if (s)
	var1=vars._VECTptr->front();
      if (s==2)
	var2=vars._VECTptr->back();
      if ((var1.type!=_IDNT) || (var2.type!=_IDNT))
	s=0;
    }
    if (vars.type==_IDNT){
      s=1;
      var1=vars;
    }
    if (!s)
      settypeerr("Plotfunc 2nd arg must be var or [var1,var2]");
    gen ff=f.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    if (s==1)
      ff=makevecteur(t__IDNT_e,subst(ff,*var1._IDNTptr,t__IDNT_e));
    if (s==2){
      ff=subst(ff,*var1._IDNTptr,u__IDNT_e);
      ff=subst(ff,*var2._IDNTptr,v__IDNT_e);
      ff=makevecteur(u__IDNT_e,v__IDNT_e,ff);
    }
#ifdef WITH_GNUPLOT
    int out_handle;
    FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
    int ng=0;
#ifdef IPAQ
    fprintf(stream,"set samples 10\n");
    ++ng;
    //    fprintf(stream,"show samples\n");
#endif
    gen r;
    if (s==2)
      r=symb_pnt(hypersurface(gen(makevecteur(makevecteur(var1,var2,f),makevecteur(var1,var2),makevecteur(function_xmin,function_ymin),makevecteur(function_xmax,function_ymax)),_PNT__VECT),z__IDNT_e-f,makevecteur(var1,var2,z__IDNT_e)),color,contextptr);
    r.subtype=gnuplot_fileno;
    reset_gnuplot_hidden3d(stream);
    ++ng;
    if (debug_infolevel)
      printf("set xrange [%g:%g]\n",function_xmin,function_xmax);
    fprintf(stream,"set xrange [%g:%g]\n",function_xmin,function_xmax);
    ++ng;
    if (s==2){
      if (debug_infolevel)
	printf("set urange [%g:%g]\n",function_xmin,function_xmax);
      fprintf(stream,"set urange [%g:%g]\n",function_xmin,function_xmax);
      ++ng;
      if (debug_infolevel)
	printf("set yrange [%g:%g]\n",function_ymin,function_ymax);
      fprintf(stream,"set yrange [%g:%g]\n",function_ymin,function_ymax);
      ++ng;
    }
    if ((s==2) || (!autoscale)){
      if (debug_infolevel)
	printf("set vrange [%g:%g]\n",function_ymin,function_ymax);
      fprintf(stream,"set vrange [%g:%g]\n",function_ymin,function_ymax);
      ++ng;
    }
    if (autoscale){
      if (debug_infolevel)
	printf("%s","set autoscale\n");
      fprintf(stream,"%s","set autoscale\n");
      ++ng;
    }
    if (clrplot || gnuplot_do_splot){
      gnuplot_do_splot=false;
      if (s==1){
	if (debug_infolevel)
	  printf("%s","plot ");	
	fprintf(stream,"%s","plot ");
      }
      else {
	if (!autoscale){
	  if (debug_infolevel)
	    printf("set zrange [%g:%g]\n",function_zmin,function_zmax);
	  fprintf(stream,"set zrange [%g:%g]\n",function_zmin,function_zmax);
	  ++ng;
	}
	if (debug_infolevel)
	  printf("%s","set view\n");
	fprintf(stream,"%s","set view\n");
	++ng;
	if (debug_infolevel)
	  printf("%s","splot ");
	fprintf(stream,"%s","splot ");
      }
    }
    else {
      if (debug_infolevel)
	printf("%s","replot ");
      fprintf(stream,"%s","replot ");
    }
    if (ff.type!=_VECT){
      if (debug_infolevel)
	printf("%s\n",gnuplot_traduit(ff).c_str());
      fprintf(stream,"%s\n",gnuplot_traduit(ff).c_str());
      ++ng;
    }
    else {
      string tmp(gnuplot_traduit(ff));
      // cout << tmp.substr(1,tmp.size()-2) << endl;
      if (debug_infolevel)
	printf("%s\n",tmp.substr(1,tmp.size()-2).c_str());
      fprintf(stream,"%s\n",tmp.substr(1,tmp.size()-2).c_str());
      ++ng;
    }
    win9x_gnuplot(stream);
    gnuplot_wait(out_handle,gnuplot_out_readstream,ng+gnuplot_wait_times);
    // usleep(50000);
    r.subtype=gnuplot_fileno,
    ++gnuplot_fileno;
    return r;
    //return gnuplot_fileno-1;
#endif
    return symb_plotfunc(f,vars);
  }

  // Note gnuplot 3.7.2 seems to have a bug
  // The unit is set to 0.1bp at the beginning and scale to 0.05
  // gnuplot 3.7.1 seems to work correctly
  bool latex_replot(FILE * stream,const string & s){
    if (!has_gnuplot)
      return 0;
#ifdef WITH_GNUPLOT
    // printf("%s\n%s\"%s\"\n%s\n%s\n","set terminal pslatex","set output ",s.c_str(),"replot","set output");
    if (debug_infolevel){
      printf("%s\n","set terminal pslatex");
      printf("%s","set output ");
      printf("\"%s",s.c_str());
      printf("\"\n%s\n%s\n","replot","set output");
    }
    fprintf(stream,"%s\n","set terminal pslatex");
    fprintf(stream,"%s","set output ");
    fprintf(stream,"\"%s",s.c_str());
    fprintf(stream,"\"\n%s\n%s\n","replot","set output");
#ifdef WIN32
    if (debug_infolevel)
      printf("%s\n","set terminal windows"); 
    fprintf(stream,"%s\n","set terminal windows");
#else
#ifdef __APPLE__
    if (debug_infolevel)
      printf("set terminal aqua\n");
    fprintf(stream,"set terminal aqua\n");
#else
    if (debug_infolevel)
      printf("%s\n","set terminal x11"); 
    fprintf(stream,"%s\n","set terminal x11");
#endif
#endif
    fflush(stream);
#endif
    return true;
  }


  symbolic symb_plotfunc(const gen & a,const gen & b){
    return symbolic(at_plotfunc,makevecteur(a,b));
  }
  symbolic symb_plotfunc(const gen & a){
    return symbolic(at_plotfunc,a);
  }
  void setplotfuncerr(){
    setsizeerr("Plotfunc: bad variable name");
  }
  gen funcplotfunc(const gen & args,bool clrplot,const context * contextptr){
    double xmin=gnuplot_xmin,xmax=gnuplot_xmax,ymin=gnuplot_ymin,ymax=gnuplot_ymax,zmin=gnuplot_zmin,zmax=gnuplot_zmax;
    int pixels=gnuplot_pixels_per_eval;
    gen attribut=default_color(contextptr);
    vecteur vargs(plotpreprocess(args,contextptr));
    int s=vargs.size();
    gen e1=vargs[1];
    bool newsyntax;
    if (e1.type!=_VECT){
      newsyntax=readrange(e1,gnuplot_xmin,gnuplot_xmax,e1,xmin,xmax) && (e1.is_symb_of_sommet(at_equal) || s<=3);
    }
    else {
      if (e1._VECTptr->size()!=2)
	setplotfuncerr();
      vecteur v(*e1._VECTptr);
      newsyntax=readrange(v[0],gnuplot_xmin,gnuplot_xmax,v[0],xmin,xmax) && readrange(v[1],gnuplot_ymin,gnuplot_ymax,v[1],ymin,ymax);
      if (newsyntax)
	e1=v;
    }
    if (!newsyntax){ // plotfunc(fonction,var,min,max[,zminmax,attribut])
      if (s<=3)
	setplotfuncerr();
      gen e2=vargs[2];
      gen e3=vargs[3];
      if (e1.type==_VECT){
	if ((e2.type!=_VECT) || (e3.type!=_VECT) || (e2._VECTptr->size()!=2) || (e3._VECTptr->size()!=2))
	  settypeerr("Plotfunc: Range must be [xmin,ymin] or [xmax,ymax]");
	gen e21=evalf_double(e2._VECTptr->front(),DEFAULT_EVAL_LEVEL,contextptr);
	gen e22=evalf_double(e2._VECTptr->back(),DEFAULT_EVAL_LEVEL,contextptr);
	gen e31=evalf_double(e3._VECTptr->front(),DEFAULT_EVAL_LEVEL,contextptr);
	gen e32=evalf_double(e3._VECTptr->back(),DEFAULT_EVAL_LEVEL,contextptr);
	if ((e21.type!=_DOUBLE_) || (e22.type!=_DOUBLE_) || (e31.type!=_DOUBLE_) || (e32.type!=_DOUBLE_))
	  settypeerr("Plotfunc: bad range value!");
	xmin=e21._DOUBLE_val;
	ymin=e22._DOUBLE_val;
	xmax=e31._DOUBLE_val;
	ymax=e32._DOUBLE_val; 
	if (s>4){
	  gen e4=vargs[4];
	  if (e4.type==_VECT && e4._VECTptr->size()==2){
	    gen e41=evalf_double(e4._VECTptr->front(),DEFAULT_EVAL_LEVEL,contextptr);
	    gen e42=evalf_double(e4._VECTptr->back(),DEFAULT_EVAL_LEVEL,contextptr);
	    if (e41.type!=_DOUBLE_ || e42.type!=_DOUBLE_)
	      settypeerr("Plotfunc: bad range value!");
	    zmin=e41._DOUBLE_val;
	    zmax=e42._DOUBLE_val;
	  }
	  if (s>5)
	    attribut=vargs[5];
	}
      }
      else {
	e2=e2.evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
	e3=e3.evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
	if ((e2.type!=_DOUBLE_) || (e3.type!=_DOUBLE_))
	  settypeerr("Plotfunc: bad range value!");
	xmin=e2._DOUBLE_val;
	xmax=e3._DOUBLE_val;
	if (s>4)
	  attribut=vargs[4];
      }
      return plotfunc(vargs.front(),e1,attribut,clrplot,xmin,xmax,ymin,ymax,zmin,zmax,pixels,contextptr);
    }
    // plotfunc(func,x=xmin..xmax[,zminmax,attribut])
    if (e1.type==_VECT){
      if (s>2){
	if (readrange(vargs[2],gnuplot_zmin,gnuplot_zmax,vargs[2],zmin,zmax)){
	  if (s>3)
	    attribut=vargs[3];
	}
	else
	  attribut=vargs[2];
      }
    }
    else {
      if (s>2)
	attribut=vargs[2];
    }
    return plotfunc(vargs[0],e1,attribut,clrplot,xmin,xmax,ymin,ymax,zmin,zmax,pixels,contextptr);
  }
  gen _plotfunc(const gen & args,const context * contextptr){
    return funcplotfunc(args,true,contextptr);
  }
  const string _plotfunc_s("plotfunc");
  unary_function_eval __plotfunc(&giac::_plotfunc,_plotfunc_s);
  unary_function_ptr at_plotfunc (&__plotfunc,_QUOTE_ARGUMENTS);

  gen _funcplot(const gen & args,const context * contextptr=0){
    return funcplotfunc(args,false,contextptr);
  }
  const string _funcplot_s("funcplot"); // Same as plotfunc but with tex print
  unary_function_eval __funcplot(&giac::_funcplot,_funcplot_s);
  unary_function_ptr at_funcplot (&__funcplot,_QUOTE_ARGUMENTS);

  bool chk_double_interval(const gen & g,double & inf,double & sup){
    gen h(g);
    if (!h.is_symb_of_sommet(at_interval))
      return false;
    h=h._SYMBptr->feuille;
    if (h.type!=_VECT || h._VECTptr->size()!=2)
      return false;
    gen h1=evalf_double(h._VECTptr->front());
    gen h2=evalf_double(h._VECTptr->back());
    if (h1.type!=_DOUBLE_  || h2.type!=_DOUBLE_ )
      return false;
    inf=h1._DOUBLE_val;
    sup=h2._DOUBLE_val;
    return true;
  }

  gen readvar(const gen & g){
    if (g.type==_IDNT)
      return g;
    if (!g.is_symb_of_sommet(at_equal))
      return undef;
    gen & f=g._SYMBptr->feuille;
    if (f.type!=_VECT || f._VECTptr->size()!=2)
      return undef;
    return f._VECTptr->front();
  }

  bool readrange(const gen & g,double defaultxmin,double defaultxmax,gen & x, double & xmin, double & xmax){
    xmin=defaultxmin;
    xmax=defaultxmax;
    if (g.type==_IDNT){
      x=g;
      return true;
    }
    if (g.is_symb_of_sommet(at_equal)){
      gen & f=g._SYMBptr->feuille;
      if (f.type!=_VECT)
	return false;
      vecteur & v=*f._VECTptr;
      if (v.size()!=2 || v[0].type!=_IDNT)
	return false;
      bool res= chk_double_interval(v[1],xmin,xmax);
      x=v[0];
      return res;
    }
    return false;
  }
  symbolic symb_erase(const gen & a){
    return symbolic(at_erase,a);
  }
  gen _erase(const gen & args){
    plot_instructions.clear();
    __interactive.op(symbolic(at_erase,0));
    return symb_erase(args);
  }
  const string _erase_s("erase");
  unary_function_unary __erase(&giac::_erase,_erase_s,&printasconstant);
  unary_function_ptr at_erase (&__erase);

  int erase3d(){
#ifdef WITH_GNUPLOT
    if (gnuplot_do_splot)
      return gnuplot_fileno-1;
    int out_handle;
    bool clrplot;
    if (win9x){
      FILE * stream = fopen("gnuplot.txt","w");
      fputc(' ',stream);
      fflush(stream);
      fclose(stream);
    }
    else {
      FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
      if (show_axes)
	fprintf(stream,"unset arrow 1\nunset arrow 2\nunset arrow 3\n");
      fprintf(stream,"\nclear\n");
      fflush(stream);
      gnuplot_wait(out_handle,gnuplot_out_readstream);
      // WARNING never close the pipe! fclose(stream);
    }
    ++gnuplot_fileno;
    gnuplot_do_splot=true;
#endif
    return gnuplot_fileno-1;
  }

  gen _erase3d(const gen & args){
    return erase3d();
  }
  const string _erase3d_s("erase3d");
  unary_function_unary __erase3d(&giac::_erase3d,_erase3d_s);
  unary_function_ptr at_erase3d (&__erase3d,0,true);

  // arg=real/complex or (real/complex,label) or same+color
  gen pointonoff(const gen & args,int color,GIAC_CONTEXT){
    gen e,a(args.evalf(DEFAULT_EVAL_LEVEL,contextptr)); 
    if ( (a.is_real(contextptr)) || (a.type==_CPLX) ){
      e=symb_pnt(args.eval(DEFAULT_EVAL_LEVEL,contextptr),color,contextptr);
    }
    else { 
      if (args.type!=_VECT)
	return symb_pnt(args,color,contextptr);
      int s=args._VECTptr->size();
      if ( (s!=2) && (s!=3) )
	setsizeerr("pointon");
      gen x=args._VECTptr->front(),y=(*args._VECTptr)[1],c;
      if ( (s==3) || (y.type==_STRNG))
	e=symbolic(at_pnt,args);
      else 
	e=symb_pnt_name(x,color,y,contextptr);
    }
#ifndef WIN32
    if (child_id)
      plot_instructions.push_back(e);
#endif // WIN32
    return e;
  }

  // pixel (i,j,[color,xmin,xmax,ymin,ymax])
  gen _pixon(const gen & args,GIAC_CONTEXT){
    int s;
    if (args.type!=_VECT || (s=args._VECTptr->size())<2)
      setsizeerr();
    vecteur v(*args._VECTptr);
    if (s<3)
      v.push_back(default_color(contextptr));
    return symb_pnt(symbolic(at_pixon,gen(v,_SEQ__VECT)),0,contextptr);
  }
  const string _pixon_s("pixon");
  unary_function_eval __pixon(&giac::_pixon,_pixon_s);
  unary_function_ptr at_pixon (&__pixon);

  gen _pixoff(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur v(*args._VECTptr);
    v.push_back(FL_WHITE);
    return _pixon(gen(v,_SEQ__VECT),contextptr);
  }
  const string _pixoff_s("pixoff");
  unary_function_eval __pixoff(&giac::_pixoff,_pixoff_s);
  unary_function_ptr at_pixoff (&__pixoff);

  gen _droite_segment(const gen & args,int subtype,GIAC_CONTEXT){
    gen e;
    vecteur res(*args._VECTptr);
    iterateur it=res.begin(),itend=res.end();
    for (;it!=itend;++it){
      *it=remove_at_pnt(*it);
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_cercle)){
	gen centre,rayon;
	centre_rayon(*it,centre,rayon,false,0); // we don't care about rayon
	*it=centre;
      }
    }
    e=symb_pnt(gen(res,subtype),default_color(contextptr),contextptr);
    // ofstream pict("PICT",ios::app);
    // pict << " ," << endl << e ;
    // pict.close();
#ifndef WIN32
    if (child_id)
      plot_instructions.push_back(e);
#endif // WIN32
    return e;
  }

  gen droite_by_equation(const vecteur & v,bool est_plan,GIAC_CONTEXT){
    int s=v.size();
    if (!s)
      setdimerr();
    gen eq(v[0]._SYMBptr->feuille._VECTptr->front()-v[0]._SYMBptr->feuille._VECTptr->back()),x(x__IDNT_e),y(y__IDNT_e),z;
    if (s>=2 && v[1].is_symb_of_sommet(at_equal)){
      if (est_plan) // Only 1 cartesian eq for a 3-d plan
	setsizeerr();
      if (s<3) ck_parameter_x(); else x=v[2];
      if (s<4) ck_parameter_y(); else y=v[3];
      if (s<5){
	z=z__IDNT_e;
	ck_parameter_z();
      }
      else
	z=v[4];
      if (x.type!=_IDNT || y.type!=_IDNT || z.type!=_IDNT)
	setsizeerr();
      vecteur vxyz(makevecteur(x,y,z));
      gen eq1(v[1]._SYMBptr->feuille._VECTptr->front()-v[1]._SYMBptr->feuille._VECTptr->back());
      gen v0(derive(eq,vxyz)),v1(derive(eq1,vxyz));
      gen A;
      if (!is_zero(derive(v0,vxyz)) || !is_zero(derive(v1,vxyz)) )
	setsizeerr("Non linear equations");
      vecteur directeur(cross(*v0._VECTptr,*v1._VECTptr));
      if (is_zero(directeur))
	setsizeerr("Parallel plans");
      if (is_zero(directeur[2])){ // z is constant on the line
	if (is_zero(directeur[1]))
	  A=gen(vecteur(3),_POINT__VECT);
	else {
	  // fix y=0 for A, solve
	  gen sol=solve(makevecteur(subst(eq,y,0),subst(eq1,y,0)),makevecteur(x,z),0,contextptr);
	  if (sol.type!=_VECT || sol._VECTptr->size()!=1)
	    setsizeerr();
	  sol=sol._VECTptr->front();
	  A=gen(makevecteur(sol._VECTptr->front(),0,sol._VECTptr->back()),_POINT__VECT);
	}
      }
      else { // Fix z=0
	gen sol=solve(makevecteur(subst(eq,z,0),subst(eq1,z,0)),makevecteur(x,y),0,contextptr);
	if (sol.type!=_VECT || sol._VECTptr->size()!=1)
	  setsizeerr();
	sol=sol._VECTptr->front();
	A=gen(makevecteur(sol._VECTptr->front(),sol._VECTptr->back(),0),_POINT__VECT);
      }
      gen B(A+directeur);
      return symb_pnt(gen(makevecteur(A,B),_LINE__VECT),default_color(contextptr),contextptr);
    }
    if (s<2) ck_parameter_x(); else x=v[1];
    if (s<3) ck_parameter_y(); else y=v[2];
    if (est_plan){
      if (s<4){
	z=z__IDNT_e;
	ck_parameter_z();
      }
      else
	z=v[3];
    }
    if ( x.type!=_IDNT || y.type!=_IDNT || (est_plan && z.type!=_IDNT) )
      setsizeerr();
    gen eqx=derive(eq,*x._IDNTptr);
    gen eqy=derive(eq,*y._IDNTptr);
    gen eqz;
    if (est_plan)
      eqz=derive(eq,*z._IDNTptr);
    // FIXME The test should be done with all derivatives
    if (!is_zero(derive(eqx,*x._IDNTptr)) || !is_zero(derive(eqy,*y._IDNTptr)) || (est_plan && !is_zero(derive(eqz,*z._IDNTptr))) )
      setsizeerr();
    if (is_zero(eqx) && is_zero(eqy) && is_zero(eqz) )
      setsizeerr();
    // equation: eqx*x+eqy*y+eqz*z+cte=0
    gen cte=subst(eq,makevecteur(x,y,z),vecteur(3,zero)),A,B;
    if (est_plan){
      B=makevecteur(eqx,eqy,eqz);
      if (is_zero(eqz)){
	if (is_zero(eqy))
	  A=gen(makevecteur(-cte/eqx,0,0),_POINT__VECT);
	else
	  A=gen(makevecteur(0,-cte/eqy,0),_POINT__VECT);
      }
      else
	A=gen(makevecteur(0,0,-cte/eqz),_POINT__VECT);
      A=ratnormal(A);
      return _plan(makevecteur(B,A),contextptr);
    }
    // 2-d line
    if (is_zero(eqy))
      A=rdiv(-cte,eqx);
    else
      A=cst_i*rdiv(-cte,eqy);
    A=ratnormal(A);
    B=A - eqy + eqx*cst_i;
    B=ratnormal(B);
    gen e=symb_pnt(gen(makevecteur(A,B),_LINE__VECT),default_color(contextptr),contextptr);
    return e;
  }

  gen mkrand2d3d(int dim,int nargs,gen (* f)(const gen &,const context *),GIAC_CONTEXT){
    vecteur v;
    switch (dim){
    case 2:
      for (int i=0;i<nargs;++i)
	v.push_back(rand_complex());
      break;
    case 3:
      for (int i=0;i<nargs;++i)
	v.push_back(rand_3d());
      break;
    default:
      setdimerr();
    }
    return f(gen(v,_SEQ__VECT),contextptr);
  }
  gen _droite(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_)
      return mkrand2d3d(args.val,2,_droite,contextptr);
    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_equal) )
      return droite_by_equation(vecteur(1,args),false,contextptr);
    if (args.type!=_VECT)
      setsizeerr();
    if ((args._VECTptr->size()>=2) && (args._VECTptr->front().type==_SYMB) && (args._VECTptr->front()._SYMBptr->sommet==at_equal))
      return droite_by_equation(*args._VECTptr,false,contextptr);
    return _droite_segment(args,_LINE__VECT,contextptr);
  }
  const string _droite_s("droite");
  unary_function_eval __droite(&giac::_droite,_droite_s);
  unary_function_ptr at_droite (&__droite);

  vecteur hyperplan_normal(const gen & g){
    vecteur n,P;
    hyperplan_normal_point(g,n,P);
    return n;
  }
  void hyperplan_normal_point(const gen & g,vecteur & n,vecteur & P){
    gen h=remove_at_pnt(g);
    if (h.is_symb_of_sommet(at_hyperplan))
      h=h._SYMBptr->feuille;
    if (h.type!=_VECT || h._VECTptr->size()!=2 || h._VECTptr->front().type!=_VECT || h._VECTptr->back().type!=_VECT)
      setsizeerr();
    n=*h._VECTptr->front()._VECTptr;
    P=*h._VECTptr->back()._VECTptr;
  }
  // args=normal,point
  gen _hyperplan(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    return symbolic(at_hyperplan,args);
  }
  const string _hyperplan_s("hyperplan");
  unary_function_eval __hyperplan(&giac::_hyperplan,_hyperplan_s);
  unary_function_ptr at_hyperplan (&__hyperplan,0,true);

  gen _plan(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_ || (args.type==_VECT && args._VECTptr->empty()) )
      return mkrand2d3d(3,3,_plan,contextptr);
    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_equal) )
      return droite_by_equation(vecteur(1,args),true,contextptr);
    if (args.type!=_VECT || args._VECTptr->size()<2)
      setsizeerr();
    if ((args._VECTptr->size()>=2) && (args._VECTptr->front().type==_SYMB) && (args._VECTptr->front()._SYMBptr->sommet==at_equal))
      return droite_by_equation(*args._VECTptr,true,contextptr);
    vecteur v = *args._VECTptr;
    v[0]=remove_at_pnt(v[0]);
    int s=v.size();
    if (s==2)
      return symb_pnt(symbolic(at_hyperplan,gen(v,args.subtype)),default_color(contextptr),contextptr);
    if (s==3){ 
      v[1]=remove_at_pnt(v[1]);
      v[2]=remove_at_pnt(v[2]);
      if (v[0].type==_VECT && v[0]._VECTptr->size()==3 && v[1].type==_VECT && v[1]._VECTptr->size()==3 && v[2].type==_VECT && v[2]._VECTptr->size()==3){
      // given by 3 points, compute normal vector
	gen v1=v[1]-v[0];
	gen v2=v[2]-v[0];
	gen n=cross(*v1._VECTptr,*v2._VECTptr);
	return  symb_pnt(symbolic(at_hyperplan,gen(makevecteur(n,v[0]),_SEQ__VECT)),default_color(contextptr),contextptr);
      }
    }
    setsizeerr();
  }
  const string _plan_s("plan");
  unary_function_eval __plan(&giac::_plan,_plan_s);
  unary_function_ptr at_plan (&__plan,0,true);

  // args=center,radius
  gen _hypersphere(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    return symbolic(at_hypersphere,args);
  }
  const string _hypersphere_s("hypersphere");
  unary_function_eval __hypersphere(&giac::_hypersphere,_hypersphere_s);
  unary_function_ptr at_hypersphere (&__hypersphere,0,true);

  // args=center,radius
  gen _sphere(const gen & args,GIAC_CONTEXT){
    //    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_equal) )
    //  return sphere_by_equation(vecteur(1,args),contextptr);
    if (args.type!=_VECT || args._VECTptr->size()<2)
      setsizeerr();
    checkanglemode(contextptr);
    // if ((args._VECTptr->size()>=2) && (args._VECTptr->front().type==_SYMB) && (args._VECTptr->front()._SYMBptr->sommet==at_equal))
    //  return sphere_by_equation(*args._VECTptr,contextptr);
    vecteur v = *args._VECTptr;
    v[0]=remove_at_pnt(v[0]);
    v[1]=remove_at_pnt(v[1]);
    if (v[1].type==_VECT){
      gen tmp=v[1]-v[0];
      if (tmp.type==_VECT){
	v[0]=(v[0]+v[1])/2;
	v[1]=l2norm(*tmp._VECTptr,contextptr)/2;
      }
    }
    return symb_pnt(symbolic(at_hypersphere,gen(v,args.subtype)),default_color(contextptr),contextptr);
  }
  const string _sphere_s("sphere");
  unary_function_eval __sphere(&giac::_sphere,_sphere_s);
  unary_function_ptr at_sphere (&__sphere,0,true);

  gen cone(const gen & args,bool cone_complet,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    checkanglemode(contextptr);
    ck_parameter_x();
    ck_parameter_y();
    ck_parameter_z();
    vecteur v = *args._VECTptr;
    gen P=remove_at_pnt(v[0]),theta=v[2];
    if (v[1].type!=_VECT || P.type!=_VECT)
      setsizeerr();
    vecteur xyz(makevecteur(x__IDNT_e,y__IDNT_e,z__IDNT_e));
    vecteur xyzP=subvecteur(xyz,*P._VECTptr);
    vecteur n=*v[1]._VECTptr,n1,n2;
    normal3d(v[1],n1,n2);
    n=divvecteur(n,abs_norm(n,contextptr));
    n1=divvecteur(n1,abs_norm(n1,contextptr));
    n2=divvecteur(n2,abs_norm(n2,contextptr));
    gen eq=normal(pow(dotvecteur(xyzP,n),2)*pow(cos(theta,contextptr),2)-(pow(dotvecteur(xyzP,n1),2)+pow(dotvecteur(xyzP,n2),2))*pow(sin(theta,contextptr),2));
    gen uu;
    if (cone_complet)
      uu=u__IDNT_e-(gnuplot_tmin+gnuplot_tmax)/2;
    else
      uu=u__IDNT_e-gnuplot_tmin;
    uu=uu*((gnuplot_xmax-gnuplot_xmin+gnuplot_ymax-gnuplot_ymin+gnuplot_zmax-gnuplot_zmin)/(gnuplot_tmax-gnuplot_tmin)/2);
    gen vv=(2*M_PI/(gnuplot_tmax-gnuplot_tmin))*v__IDNT_e;
    P=P+uu*(cos(theta,contextptr)*n+sin(theta,contextptr)*(cos(vv,contextptr)*n1+sin(vv,contextptr)*n2));
    gen res= plotparam3d(P,makevecteur(u__IDNT_e,v__IDNT_e),gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,gnuplot_tmin,gnuplot_tmax,false,false,default_color(contextptr),eq,xyz,contextptr);
    return res;
  }
  // args=point, direction, angle 
  gen _cone(const gen & args,GIAC_CONTEXT){
    return cone(args,true,contextptr);
  }
  const string _cone_s("cone");
  unary_function_eval __cone(&giac::_cone,_cone_s);
  unary_function_ptr at_cone (&__cone,0,true);

  // args=point, direction, angle 
  gen _demi_cone(const gen & args,GIAC_CONTEXT){
    return cone(args,false,contextptr);
  }
  const string _demi_cone_s("demi_cone");
  unary_function_eval __demi_cone(&giac::_demi_cone,_demi_cone_s);
  unary_function_ptr at_demi_cone (&__demi_cone,0,true);

  // args=point, direction, radius 
  gen _cylindre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    vecteur v = *args._VECTptr;
    gen P=remove_at_pnt(v[0]),r=v[2];
    if (v[1].type!=_VECT || P.type!=_VECT)
      setsizeerr();
    vecteur xyz(makevecteur(x__IDNT_e,y__IDNT_e,z__IDNT_e));
    vecteur xyzP=subvecteur(xyz,*P._VECTptr);
    vecteur n=*v[1]._VECTptr,n1,n2;
    normal3d(v[1],n1,n2);
    n=divvecteur(n,abs_norm(n,contextptr));
    n1=divvecteur(n1,abs_norm(n1,contextptr));
    n2=divvecteur(n2,abs_norm(n2,contextptr));
    gen vv=(2*M_PI/(gnuplot_tmax-gnuplot_tmin))*v__IDNT_e;
    P=P+u__IDNT_e*n+r*(cos(vv,contextptr)*n1+sin(vv,contextptr)*n2);
    gen eq=normal(pow(r,2)-(pow(dotvecteur(xyzP,n1),2)+pow(dotvecteur(xyzP,n2),2)));
    return plotparam3d(P,makevecteur(u__IDNT_e,v__IDNT_e),gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,gnuplot_tmin,gnuplot_tmax,false,false,default_color(contextptr),eq,xyz,contextptr);
  }
  const string _cylindre_s("cylindre");
  unary_function_eval __cylindre(&giac::_cylindre,_cylindre_s);
  unary_function_ptr at_cylindre (&__cylindre,0,true);

  gen _demi_droite(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_)
      return mkrand2d3d(args.val,2,_demi_droite,contextptr);
    if (args.type!=_VECT)
      return symbolic(at_demi_droite,args);
    if (args._VECTptr->size()==3){
      vecteur v = *args._VECTptr;
      v[0]=remove_at_pnt(v[0]);
      v[1]=remove_at_pnt(v[1]);
      vecteur w;
      w.push_back(eval(symb_sto(_point(v[0],contextptr),v[2]),contextptr));
      w.push_back(_droite_segment(gen(makevecteur(v[0],v[1]),_SEQ__VECT),_HALFLINE__VECT,contextptr));
      return gen(w,_GROUP__VECT);
    }
    return _droite_segment(args,_HALFLINE__VECT,contextptr);
  }
  const string _demi_droite_s("demi_droite");
  unary_function_eval __demi_droite(&giac::_demi_droite,_demi_droite_s);
  unary_function_ptr at_demi_droite (&__demi_droite);

  gen _segment(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_)
      return mkrand2d3d(args.val,2,_segment,contextptr);
    if (args.type!=_VECT)
      return symbolic(at_segment,args);
    if (args._VECTptr->size()==4){
      vecteur v = *args._VECTptr;
      v[0]=remove_at_pnt(v[0]);
      v[1]=remove_at_pnt(v[1]);
      gen name;
      try {
	name=gen(v[2].print()+v[3].print());
      }
      catch (std::runtime_error & e){
	name=undef;
      }
      vecteur w;
      w.push_back(eval(symb_sto(_point(v[0],contextptr),v[2]),contextptr));
      w.push_back(eval(symb_sto(_point(v[1],contextptr),v[3]),contextptr));
      if (is_undef(name))
	w.push_back(_droite_segment(gen(makevecteur(v[0],v[1]),_SEQ__VECT),_GROUP__VECT,contextptr));
      else
	w.push_back(eval(symb_sto(_droite_segment(gen(makevecteur(v[0],v[1]),_SEQ__VECT),_GROUP__VECT,contextptr),name)));
      return gen(w,_GROUP__VECT);
      // return _droite_segment(gen(makevecteur(v[0]+cst_i*v[1],v[2]+cst_i*v[3]),_SEQ__VECT),_GROUP__VECT);
    }
    return _droite_segment(args,_GROUP__VECT,contextptr);
  }
  const string _segment_s("segment");
  unary_function_eval __segment(&giac::_segment,_segment_s);
  unary_function_ptr at_segment (&__segment);

  void gnuplot_set_hidden3d(bool hidden){
    gnuplot_hidden3d=hidden;
  }

  void gnuplot_set_pm3d(bool b){
    gnuplot_pm3d=b;
  }

  void show_3d_axes(FILE * stream){
    if (!show_axes)
      return;
    if (debug_infolevel)
      printf("set arrow 1 to %g,0,0 lt 1\nset arrow 2 to 0,%g,0 lt 2\nset arrow 3 to 0,0,%g lt 3\n",gnuplot_xmax,gnuplot_ymax,gnuplot_zmax);
    fprintf(stream,"set arrow 1 to %g,0,0 lt 1\nset arrow 2 to 0,%g,0 lt 2\nset arrow 3 to 0,0,%g lt 3\n",gnuplot_xmax,gnuplot_ymax,gnuplot_zmax);
  }

  int gnuplot_show_pnt(const symbolic & e,GIAC_CONTEXT){
    if (!show_point(contextptr))
      return -1;
    gen f=e.feuille;
    vecteur fv(lidnt(f));
    vecteur fvs(*evalf(fv)._VECTptr);
    if (!lidnt(fvs).empty())
      return -1;
    f = subst(f,fv,fvs);
    if (f.type==_VECT && !f._VECTptr->empty()){
      string legende;
      if (f._VECTptr->size()>=3){
	gen & g=(*f._VECTptr)[2];
	if (g.type==_STRNG)
	  legende=g.print();
	else
	  legende='"'+g.print()+'"';
	legende=" title "+legende;
      }
      // Plottable: 3-d points vector of length 3 with real coord. [,,] 
      // 3-d poly-line vector of arbitrary length with 3-d point or 2-d points
      // 3-d hyperplan (symb of sommet at_hyperplan, feuille=3-d pt + normal )
      // 3-d sphere (symb of sommet at_cercle and diameter = 3-d points)
      double les3=gnuplot_xmax-gnuplot_xmin+gnuplot_ymax-gnuplot_ymin+gnuplot_zmax-gnuplot_zmin,deltat=gnuplot_tmax-gnuplot_tmin,milieut=(gnuplot_tmax+gnuplot_tmin)/2;
      const gen & p=e.feuille._VECTptr->front();
      string tmpfilename("#xcas"+print_INT_(gnuplot_fileno)+".gnu");
      bool doplot=false,clrplot=false;
      int with_point;
      string splot;
      // splot if gnuplot_do_splot && !clrplot, replot otherwise
      identificateur _u("u"),_v("v");
      if (p.type==_VECT && p.subtype==_POINT__VECT && p._VECTptr->size()==3){
	vecteur & v=*p._VECTptr;
	splot=evalf(gen(v,_SEQ__VECT)).print() +" with point";
	with_point=2;
	doplot=true;
      }
      if (p.type==_VECT && p.subtype!=_POINT__VECT){
	vecteur & v=*p._VECTptr;
	if (p.subtype==_POLYEDRE__VECT){
	  // each element of v is a face with 3 or 4 vertices
	  ofstream tmpfile(tmpfilename.c_str());
	  const_iterateur it=v.begin(),itend=v.end();
	  for (;it!=itend;++it){
	    if (it->type==_VECT){ 
	      vecteur w = * it->_VECTptr;
	      int s=w.size();
	      if (s%2){
		w.insert(w.begin()+s/2,w[s/2]);
		++s;
	      }
	      if (s>=4){
		for (int j=0;j<s/2;++j){
		  if (w[j].type!=_VECT || w[j]._VECTptr->size()!=3)
		    tmpfile << "0 0 0" <<endl;
		  else {
		    const vecteur & ww=*w[j]._VECTptr;
		    tmpfile << evalf(ww[0]) << " " << evalf(ww[1]) << " " << evalf(ww[2]) << endl;
		  }
		}
		tmpfile << endl;
		for (int j=s-1;j>=s/2;--j){
		  if (w[j].type!=_VECT || w[j]._VECTptr->size()!=3)
		    tmpfile << "0 0 0" <<endl;
		  else {
		    const vecteur & ww=*w[j]._VECTptr;
		    tmpfile << evalf(ww[0]) << " " << evalf(ww[1]) << " " << evalf(ww[2]) << endl;
		  }
		}
		doplot=true;
	      }
	    }
	    tmpfile << endl << endl; // next face
	  } // end for
	  with_point=0;
	  tmpfile.close();	  
	}
	else {
	  if (v.size()==2){ 
	    // Using arrow
	    // parametric plot of the line
	    gen coeff=evalf(v.back()-v.front());
	    if (coeff.type!=_VECT)
	      return -1;
	    gen d;
	    if (p.subtype==_GROUP__VECT || p.subtype==_VECTOR__VECT)
	      d=1.0;
	    else
	      d=2*les3/abs_norm(coeff,0);
	    coeff=(d/deltat)*coeff;
	    gen tmp_eq;
	    if (p.subtype==_GROUP__VECT || p.subtype==_VECTOR__VECT)
	      tmp_eq=v.front()+coeff*(_u-gnuplot_tmin);
	    else
	      tmp_eq=v.front()+coeff*(_u-milieut);
	    // add an epsilon to see lines on plans
	    tmp_eq=tmp_eq+1e-3*makevecteur(gnuplot_xmax-gnuplot_xmin,gnuplot_ymax-gnuplot_ymin,gnuplot_zmax-gnuplot_zmin);
	    if (tmp_eq.type==_VECT){
	      tmp_eq.subtype=_SEQ__VECT;
	      splot=evalf(tmp_eq).print();
	      doplot=true;
	      with_point=2;
	    }
	  }
	  else {
	    ofstream tmpfile(tmpfilename.c_str());
	    const_iterateur it=v.begin(),itend=v.end();
	    for (;it!=itend;++it){
	      if (it->type==_VECT && it->_VECTptr->size()==3){
		const vecteur & w = * it->_VECTptr;
		tmpfile << evalf(w[0]) << " " << evalf(w[1]) << " " << evalf(w[2]) << endl;
		doplot=true;
	      }
	    }
	    with_point=0;
	    tmpfile.close();
	  }
	} // end else polyedre
      } // end if (p.type==_VECT)
      if (p.is_symb_of_sommet(at_hyperplan)){
	vecteur P,n;
	hyperplan_normal_point(p,n,P);
	with_point=2;
	vecteur v1,v2;
	normal3d(n,v1,v2);
	gen v1c=evalf(v1);
	v1c=(les3/2/deltat)*v1c/abs_norm(v1c,0);
	gen v2c=evalf(v2);
	v2c=(les3/2/deltat)*v2c/abs_norm(v2c,0);
	gen par_eq=P+(_u-milieut)*v1c+(_v-milieut)*v2c;
	par_eq.subtype=_SEQ__VECT;
	splot=evalf(par_eq).print();
	doplot=true;
      }
      if (p.is_symb_of_sommet(at_hypersphere)){
	// The optional 3rd argument of hypersphere is used for a "directional"
	// plot. A fourth arg might be used later for half-sphere...
	gen & f=p._SYMBptr->feuille;
	if (f.type==_VECT && f._VECTptr->size()>=2){
	  vecteur & v=*f._VECTptr;
	  if (v.front().type==_VECT && v.front()._VECTptr->size()==3){
	    with_point=2;
	    gen r=v[1];
	    if (r.type==_VECT && r._VECTptr->size()==3)
	      r=l2norm(*r._VECTptr,contextptr);
	    gen uu=2*M_PI/deltat*_u;
	    gen vv=2*M_PI/deltat*_v;
	    vecteur dir1(makevecteur(1,0,0)),dir2(makevecteur(0,1,0)),dir3(makevecteur(0,0,1));
	    if (v.size()>=3 && v[2].type==_VECT && v[2]._VECTptr->size()==3){
	      dir3=*v[2]._VECTptr;
	      dir3=divvecteur(dir3,sqrt(dotvecteur(dir3,dir3)));
	      if (!is_zero(dir3[0]) || !is_zero(dir3[1]) ){
		dir1=makevecteur(-dir3[1],dir3[0],0);
		dir1=divvecteur(dir1,sqrt(dotvecteur(dir1,dir1)));
		dir2=cross(dir3,dir1);
	      }
	    }
	    gen par_eq=v.front()+r*(cos(uu,contextptr)*(cos(vv,contextptr)*dir1+sin(vv,contextptr)*dir2)+sin(uu,contextptr)*dir3);
	    par_eq.subtype=_SEQ__VECT;
	    splot=evalf(par_eq).print()+" notitle";
	    doplot=true;
	  }
	}
      }
      if (doplot){
#ifdef WITH_GNUPLOT
	int out_handle;
	FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
	reset_gnuplot_hidden3d(stream);
	if (debug_infolevel)
	  printf("set xrange [%g:%g]\n",gnuplot_xmin,gnuplot_xmax);
	fprintf(stream,"set xrange [%g:%g]\n",gnuplot_xmin,gnuplot_xmax);
	if (debug_infolevel)
	  printf("set yrange [%g:%g]\n",gnuplot_ymin,gnuplot_ymax);
	fprintf(stream,"set yrange [%g:%g]\n",gnuplot_ymin,gnuplot_ymax);
	if (debug_infolevel)
	  printf("set zrange [%g:%g]\n",gnuplot_zmin,gnuplot_zmax);
	fprintf(stream,"set zrange [%g:%g]\n",gnuplot_zmin,gnuplot_zmax);
	if (debug_infolevel)
	  printf("set urange [%g:%g]\n",gnuplot_tmin,gnuplot_tmax);
	fprintf(stream,"set urange [%g:%g]\n",gnuplot_tmin,gnuplot_tmax);
	if (debug_infolevel)
	  printf("set vrange [%g:%g]\n",gnuplot_tmin,gnuplot_tmax);
	fprintf(stream,"set vrange [%g:%g]\n",gnuplot_tmin,gnuplot_tmax);
	gnuplot_wait(out_handle,gnuplot_out_readstream);
	gnuplot_set_hidden3d(gnuplot_hidden3d);
	if (clrplot || gnuplot_do_splot){
	  show_3d_axes(stream);
	  if (show_axes)
	    gnuplot_wait(out_handle,gnuplot_out_readstream);
	}
	if (clrplot || gnuplot_do_splot){
	  if (debug_infolevel)
	    printf("splot");
	  fprintf(stream,"splot");
	}
	else {
	  if (debug_infolevel)
	    printf("replot");
	  fprintf(stream,"replot");
	}
	if (with_point ==2 ){
	  if (debug_infolevel)
	    printf((" "+splot+legende +'\n').c_str());
	  fprintf(stream,(" "+splot+legende +'\n').c_str());
	}
	else {
	  if (debug_infolevel)
	    printf(" \"%s\" with ",tmpfilename.c_str());
	  fprintf(stream," \"%s\" with ",tmpfilename.c_str());
	  if (with_point){
	    if (debug_infolevel)
	      printf("point\n");
	    fprintf(stream,"point\n");
	  }
	  else {
	    if (debug_infolevel)
	      printf("line\n");
	    fprintf(stream,"line\n");
	  }
	}
	gnuplot_wait(out_handle,gnuplot_out_readstream);
	gnuplot_do_splot=false;
	win9x_gnuplot(stream);
	gnuplot_wait(out_handle,gnuplot_out_readstream,gnuplot_wait_times);
	++gnuplot_fileno;
	return gnuplot_fileno-1;
#endif
      }
    }
    return -1;
  }
  gen symb_pnt_name(const gen & x,const gen & c,const gen & nom,GIAC_CONTEXT){
    symbolic e=symbolic(at_pnt,gen(makevecteur(x,c,nom),_PNT__VECT));
    gen ee(e);
    ee.subtype=gnuplot_show_pnt(e,contextptr);
    if (io_graph(contextptr))
      __interactive.op(ee);
    return ee;
  }
  gen symb_segment(const gen & x,const gen & y,const gen & c,GIAC_CONTEXT){
    symbolic e=symbolic(at_pnt,gen(makevecteur(gen(makevecteur(x,y),_LINE__VECT),c),_PNT__VECT));
    gen ee(e);
    ee.subtype=gnuplot_show_pnt(e,contextptr);
    if (io_graph(contextptr))
      __interactive.op(ee);
    return ee;
  }
  gen symb_pnt(const gen & x,const gen & c,GIAC_CONTEXT){
    symbolic e= symbolic(at_pnt,gen(makevecteur(x,c),_PNT__VECT));
    gen ee(e);
    ee.subtype=gnuplot_show_pnt(e,contextptr);
    if (io_graph(contextptr))
      __interactive.op(ee);
    return ee;
  }
  gen symb_pnt(const gen & x,GIAC_CONTEXT){
    return symb_pnt(x,gen(FL_BLACK),contextptr);
  }
  gen _pnt(const gen & args){
    if ((args.type==_SYMB) && (args._SYMBptr->sommet==at_pnt))
      return args;
    if ( (args.type==_VECT) && (args._VECTptr->size()) ){
      vecteur v(*args._VECTptr);
      gen e=v.front();
      if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt))
	return e;
      if (v.size()==3)
	v.pop_back();
      return symbolic(at_pnt,gen(v,_PNT__VECT));
    }
    return symbolic(at_pnt,args);
  }
  const string _pnt_s("pnt");
  unary_function_unary __pnt(&giac::_pnt,_pnt_s);
  unary_function_ptr at_pnt (&__pnt);

  symbolic symb_point(const gen & x,const gen & y){
    return symbolic(at_point,makevecteur(x,y));
  }
  gen _point(const gen & args,GIAC_CONTEXT){
    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_pnt))
      return args;
    if (args.type==_VECT) {
      if (args._VECTptr->size()==2)
	return pointonoff(args._VECTptr->front()+cst_i*args._VECTptr->back(),default_color(contextptr),contextptr);
      return symb_pnt(gen(*args._VECTptr,_POINT__VECT),default_color(contextptr),contextptr);
    }
    return symb_pnt(args,default_color(contextptr),contextptr);
  }
  const string _point_s("point");
  unary_function_eval __point(&giac::_point,_point_s);
  unary_function_ptr at_point (&__point);

  gen rand_2d3d(bool espace){
    if (espace)
      return do_point3d(rand_3d());
    else
      return rand_complex();
  }
  gen point2d3d(bool espace,GIAC_CONTEXT){
    return symb_pnt(rand_2d3d(espace),default_color(contextptr),contextptr);
  }
  gen point2d3d(const gen & args,bool espace,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    if (v.empty())
      return point2d3d(espace,contextptr);
    vecteur w;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      w.push_back(symbolic(at_sto,makevecteur(symbolic(at_pnt,gen(makevecteur(rand_2d3d(espace),default_color(contextptr)),_PNT__VECT)),*it)));
    }
    return eval(w);
  }
  gen _point3d(const gen & args,GIAC_CONTEXT){
    return point2d3d(args,true,contextptr);
  }
  const string _point3d_s("point3d");
  unary_function_eval __point3d(&giac::_point3d,_point3d_s);
  unary_function_ptr at_point3d (&__point3d,_QUOTE_ARGUMENTS,true);

  gen _point2d(const gen & args,GIAC_CONTEXT){
    return point2d3d(args,false,contextptr);
  }
  const string _point2d_s("point2d");
  unary_function_eval __point2d(&giac::_point2d,_point2d_s);
  unary_function_ptr at_point2d (&__point2d,_QUOTE_ARGUMENTS,true);

  gen _affixe(const gen & args){
    return remove_at_pnt(args);
  }
  const string _affixe_s("affixe");
  unary_function_unary __affixe(&giac::_affixe,_affixe_s);
  unary_function_ptr at_affixe (&__affixe);

  gen _abscisse(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_abscisse);
    gen g=remove_at_pnt(args);
    if (g.type==_VECT && g._VECTptr->size()>=3)
      return (*g._VECTptr)[0];
    return re(g,contextptr);
  }
  const string _abscisse_s("abscisse");
  unary_function_eval __abscisse(&giac::_abscisse,_abscisse_s);
  unary_function_ptr at_abscisse (&__abscisse);

  gen _ordonnee(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_ordonnee);
    gen g=remove_at_pnt(args);
    if (g.type==_VECT && g._VECTptr->size()>=3)
      return (*g._VECTptr)[1];
    return im(g,contextptr);
  }
  const string _ordonnee_s("ordonnee");
  unary_function_eval __ordonnee(&giac::_ordonnee,_ordonnee_s);
  unary_function_ptr at_ordonnee (&__ordonnee);

  gen _cote(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_cote);
    gen g=remove_at_pnt(args);
    if (g.type==_VECT && g._VECTptr->size()>=3)
      return (*g._VECTptr)[2];
    setsizeerr("3-d instruction");
  }
  const string _cote_s("cote");
  unary_function_eval __cote(&giac::_cote,_cote_s);
  unary_function_ptr at_cote (&__cote,0,true);

  gen _coordonnees(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,contextptr,_coordonnees);
    gen P=remove_at_pnt(args);
    if (P.type==_VECT)
      return P;
    return makevecteur(re(P,contextptr),im(P,contextptr));
  }
  const string _coordonnees_s("coordonnees");
  unary_function_eval __coordonnees(&giac::_coordonnees,_coordonnees_s);
  unary_function_ptr at_coordonnees (&__coordonnees,0,true);

  // point + rayon or line
  gen _cercle(const gen & args,GIAC_CONTEXT){
    vecteur v(1,args);
    if (args.type==_VECT)
      v=(*args._VECTptr);
    if (v.empty())
      setsizeerr("cercle");
    // find diametre
    gen e=eval(v.front(),contextptr),diametre;
    int narg=0;
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt)){
      gen f=e._SYMBptr->feuille._VECTptr->front();
      if (f.type==_VECT && f._VECTptr->size()==2){
	diametre=f;
	narg=1;
      }
      else e=remove_at_pnt(e);
    }
    else {
      if ((e.type==_VECT) && (e._VECTptr->size()==2)){
	diametre=e;
	narg=1;
      }
    }
    if (!narg){
      if (v.size()<2)
	setsizeerr("cercle");
      gen f=eval(v[1],contextptr);
      if ((f.type==_SYMB) && (f._SYMBptr->sommet==at_pnt)){
	gen g=f._SYMBptr->feuille._VECTptr->front();
	diametre=gen(makevecteur(e,g),_GROUP__VECT);
      }
      else {
	gen ee=e-f;
	gen ff=e+f;
	diametre=gen(makevecteur(ee,ff),_GROUP__VECT);
      }
      narg=2;
    }
    if (diametre.type!=_VECT || diametre._VECTptr->size()!=2 )
      setsizeerr();
    gen d0=diametre._VECTptr->front();
    gen d1=(*diametre._VECTptr)[1];
    gen ce=normal(rdiv(d0+d1,plus_two));
    gen ra=normal(rdiv(d1-d0,plus_two));
    if (d0.type==_VECT){     
      // 3-d circle, angles not allowed yet
      if (v.size()!=1+narg)
	set3derr();
      // 3rd point defines the plan
      gen d2=remove_at_pnt(eval(v[narg],contextptr));
      gen d02=d2-d0;
      d02=cross(cross(ra,d02),ra); // normal in the same plan
      d02=sqrt(dotvecteur(ra,ra)/dotvecteur(d02,d02))*d02; // normalized
      // Make a parametric plot
      identificateur t("t");
      gen theta=(2*M_PI/(gnuplot_tmax-gnuplot_tmin))*t;
      return paramplotparam(gen(makevecteur(ce+cos(theta,contextptr)*ra+sin(theta,contextptr)*d02,t),_SEQ__VECT),false,contextptr);
    }
    // find angles
    gen a1(zero),a2(cst_two_pi);
    if (v.size()>1+narg){
      a1=eval(v[narg],contextptr);
      a2=eval(v[narg+1],contextptr);
    }
    gen res=symb_pnt(symbolic(at_cercle,gen(makevecteur(diametre,a1,a2),_PNT__VECT)),default_color(contextptr),contextptr);
    if (v.size()<3+narg)
      return res;
    vecteur w(1,res);
    gen ga1=ce+ra*exp(cst_i*a1);
    gen ga2=ce+ra*exp(cst_i*a2);
    w.push_back(eval(symb_sto(_point(ga1,contextptr),v[narg+2]),contextptr));
    if (v.size()>3+narg)
      w.push_back(eval(symb_sto(_point(ga2,contextptr),v[narg+3]),contextptr));
    return  gen(w,_GROUP__VECT);
  }
  const string _cercle_s("cercle");
  unary_function_eval __cercle(&giac::_cercle,_cercle_s);
  unary_function_ptr at_cercle (&__cercle,_QUOTE_ARGUMENTS,true);

  bool centre_rayon(const gen & cercle,gen & centre,gen & rayon,bool absrayon,GIAC_CONTEXT){
    gen c=remove_at_pnt(cercle);
    if (c.is_symb_of_sommet(at_hypersphere)){
      gen & f=c._SYMBptr->feuille;
      if (f.type!=_VECT || f._VECTptr->size()!=2)
	setsizeerr();
      centre=f._VECTptr->front();
      rayon=f._VECTptr->back();
      return true;
    }
    if ( (c.type!=_SYMB) || (c._SYMBptr->sommet!=at_cercle))
      return false;
    gen diam=remove_at_pnt(c._SYMBptr->feuille._VECTptr->front());
    if (diam.type!=_VECT)
      return false;
    gen a=remove_at_pnt(diam._VECTptr->front());
    gen b=remove_at_pnt(diam._VECTptr->back());
    centre=normal(rdiv(a+b,plus_two));
    rayon=rdiv(a-b,plus_two);
    if (absrayon)
      rayon=abs(normal(rayon),contextptr);
    return true;
  }

  symbolic symb_centre(const gen & args){
    return symbolic(at_centre,args);
  }
  gen _centre(const gen & args,GIAC_CONTEXT){
    gen a=remove_at_pnt(args);
    if ( (a.type!=_SYMB) || (a._SYMBptr->sommet!=at_cercle))
      return symb_centre(args);
    gen centre,rayon;
    centre_rayon(a,centre,rayon,false,contextptr);
    return symb_pnt(centre,default_color(contextptr),contextptr);
  }
  const string _centre_s("centre");
  unary_function_eval __centre(&giac::_centre,_centre_s);
  unary_function_ptr at_centre (&__centre);

  symbolic symb_rayon(const gen & args){
    return symbolic(at_rayon,args);
  }
  gen _rayon(const gen & args,GIAC_CONTEXT){
    gen a=remove_at_pnt(args);
    if ( (a.type!=_SYMB) || (a._SYMBptr->sommet!=at_cercle))
      return symb_rayon(args);
    gen centre,rayon;
    centre_rayon(a,centre,rayon,true,contextptr);
    return rayon;
  }
  const string _rayon_s("rayon");
  unary_function_eval __rayon(&giac::_rayon,_rayon_s);
  unary_function_ptr at_rayon (&__rayon);

  symbolic symb_milieu(const gen & x,const gen & y){
    return symbolic(at_milieu,makevecteur(x,y));
  }
  symbolic symb_milieu(const gen & x){
    return symbolic(at_milieu,x);
  }
  gen _milieu(const gen & args,GIAC_CONTEXT){
    gen e,f;
    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_pnt)){
      e=remove_at_pnt(args);
      if ((e.type!=_VECT) || (e._VECTptr->size()!=2))
	setsizeerr("milieu");
      f=e._VECTptr->back();
      e=e._VECTptr->front();
    }
    else {
      if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
	return symbolic(at_milieu,args);
      e=remove_at_pnt(args._VECTptr->front());
      f=remove_at_pnt(args._VECTptr->back());
    }
    return symb_pnt(rdiv(e+f,plus_two),default_color(contextptr),contextptr);
  }
  const string _milieu_s("milieu");
  unary_function_eval __milieu(&giac::_milieu,_milieu_s);
  unary_function_ptr at_milieu (&__milieu);

  symbolic symb_mediatrice(const gen & x,const gen & y){
    return symbolic(at_mediatrice,makevecteur(x,y));
  }
  gen _mediatrice(const gen & args,GIAC_CONTEXT){
    gen e,f;
    if ( (args.type==_SYMB) && (args._SYMBptr->sommet==at_pnt)){
      e=remove_at_pnt(args);
      if ((e.type!=_VECT) || (e._VECTptr->size()!=2))
	setsizeerr("milieu");
      f=e._VECTptr->back();
      e=e._VECTptr->front();
    }
    else {
      if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
	return symbolic(at_mediatrice,args);
      e=remove_at_pnt(args._VECTptr->front());
      f=remove_at_pnt(args._VECTptr->back());
    }
    if (e.type==_VECT && f.type==_VECT)
      return _perpendiculaire(makevecteur((e+f)/2,f-e),contextptr);
    gen direction=im(f-e,contextptr)-cst_i*re(f-e,contextptr);
    gen m=rdiv(e+f,plus_two);
    return symb_segment(m-direction,m+direction,default_color(contextptr),contextptr);
  }
  const string _mediatrice_s("mediatrice");
  unary_function_eval __mediatrice(&giac::_mediatrice,_mediatrice_s);
  unary_function_ptr at_mediatrice (&__mediatrice);

  symbolic symb_perpendiculaire(const gen & a){
    return symbolic(at_perpendiculaire,a);
  }
  gen _perpendiculaire(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symb_perpendiculaire(args);
    vecteur v(*args._VECTptr);
    gen e=v.front(),f=v[1];
    e=remove_at_pnt(e);
    if ( (f.type==_SYMB) && (f._SYMBptr->sommet==at_pnt) ){
      f=remove_at_pnt(f);
      if (f.is_symb_of_sommet(at_hyperplan)){ // args=point, hyperplan
	vecteur n,P;
	hyperplan_normal_point(f,n,P);
	return symb_pnt(gen(makevecteur(e,e+n),_LINE__VECT),default_color(contextptr),contextptr);
      }
      if (f.type==_VECT && f._VECTptr->size()==2) // args=point,line
	f=f._VECTptr->front()-f._VECTptr->back();
      else { // args=point,point,point
	if (v.size()!=3)
	  setsizeerr("perpendiculaire");
	f=f-remove_at_pnt(v[2]);
      }
    }
    // f is a complex or a vector normal to the perpendicular
    if (f.type==_VECT)
      return _plan(makevecteur(f,e),contextptr);
    f=cst_i*f;
    return symb_segment(e,e+f,default_color(contextptr),contextptr);
  }
  const string _perpendiculaire_s("perpendiculaire");
  unary_function_eval __perpendiculaire(&giac::_perpendiculaire,_perpendiculaire_s);
  unary_function_ptr at_perpendiculaire (&__perpendiculaire);

  bool check3dpoint(const gen & g){
    return g.type==_VECT && g._VECTptr->size()==3;
  }
  // find the 2 points of d1 and d2 and a common normal vector to d1 d2
  bool perpendiculaire_commune(const gen & d1,const gen & d2,gen & M, gen & N,vecteur & n,GIAC_CONTEXT){
    gen D1=remove_at_pnt(d1);
    gen D2=remove_at_pnt(d2);
    if (D1.type!=_VECT || D1._VECTptr->size()!=2 || D2.type!=_VECT || D2._VECTptr->size()!=2)
      setsizeerr();
    gen & A=D1._VECTptr->front();
    check3dpoint(A);
    gen & B=D1._VECTptr->back();
    check3dpoint(B);
    gen & C=D2._VECTptr->front();
    check3dpoint(C);
    gen & D=D2._VECTptr->back();
    check3dpoint(D);
    vecteur v1(subvecteur(*B._VECTptr,*A._VECTptr));
    vecteur v2(subvecteur(*D._VECTptr,*C._VECTptr));
    n=cross(v1,v2);
    if (is_zero(n))
      return false;
    // M=A+u*v1, N=C-v*v2, find u and v such that 
    // NM.v1=0 and NM.v2=0
    // where NM= -(C-A)+ u*v1+v*v2
    // Hence solve the system of matrix
    // v1.v1*u + v2.v1*v = (C-A).v1
    // v1.v2*u + v2.v2*v = (C-A).v2
    vecteur AC(subvecteur(*C._VECTptr,*A._VECTptr));
    gen v11(dotvecteur(v1,v1)),v22(dotvecteur(v2,v2)),v12(dotvecteur(v1,v2));
    gen AC1(dotvecteur(v1,AC)),AC2(dotvecteur(v2,AC));
    gen det(v11*v22-v12*v12);
    gen u= (v22*AC1-v12*AC2)/det,v=(v11*AC2-v12*AC1)/det;
    M=A+u*v1;
    N=C-v*v2;
    M.subtype=_POINT__VECT;
    N.subtype=_POINT__VECT;
    return true;
  }
  gen _perpendiculaire_commune(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      setsizeerr();
    gen M,N;
    vecteur n;
    if (!perpendiculaire_commune(args._VECTptr->front(),args._VECTptr->back(),M,N,n,contextptr))
      setsizeerr("Parellel lines");
    return symb_pnt(gen(makevecteur(M,N),_LINE__VECT),default_color(contextptr),contextptr);
  }
  const string _perpendiculaire_commune_s("perpendiculaire_commune");
  unary_function_eval __perpendiculaire_commune(&giac::_perpendiculaire_commune,_perpendiculaire_commune_s);
  unary_function_ptr at_perpendiculaire_commune (&__perpendiculaire_commune,0,true);

  gen _parallele(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) )
      return symbolic(at_parallele,args);
    gen a=remove_at_pnt(args._VECTptr->front()),b(args._VECTptr->back()),d;
    if ( (b.type==_SYMB) && (b._SYMBptr->sommet==at_pnt)){
      b=remove_at_pnt(b);
      if (b.type==_VECT){
	d=b;
	b=a;
      }
      else
	d=a;
    }
    else {
      d=makevecteur(zero,b);
    }
    if ( (d.type!=_VECT) || (d._VECTptr->size()!=2) || (a.type==_VECT) )
      setsizeerr();
    // parallel to d through a
    return symb_pnt(d+(a-d._VECTptr->front()),default_color(contextptr),contextptr);
  }
  const string _parallele_s("parallele");
  unary_function_eval __parallele(&giac::_parallele,_parallele_s);
  unary_function_ptr at_parallele (&__parallele);

  void gen23points(const gen & args,gen &e,gen &f,gen &g){
    assert(args.type==_VECT);
    vecteur v(*args._VECTptr);
    e=v.front(),f=v[1],g;
    e=remove_at_pnt(e);
    f=remove_at_pnt(f);
    if (f.type==_VECT){
      if (f._VECTptr->size()!=2)
	setsizeerr("gen23points");
      g=f._VECTptr->back();
      f=f._VECTptr->front();
    }
    else {
      if (v.size()!=3)
	setsizeerr("gen23points");
      g=remove_at_pnt(v[2]);
    }
  }

  gen _triangle(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3))
      return symbolic(at_triangle,args);
    vecteur v(*args._VECTptr);
    gen a=remove_at_pnt(v.front()),b=remove_at_pnt(v[1]),c=remove_at_pnt(v[2]);
    v=makevecteur(a,b,c,a);
    return symb_pnt(gen(v,_GROUP__VECT),default_color(contextptr),contextptr);
  }
  const string _triangle_s("triangle");
  unary_function_eval __triangle(&giac::_triangle,_triangle_s);
  unary_function_ptr at_triangle (&__triangle);

  gen _quadrilatere(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=4))
      return symbolic(at_quadrilatere,args);
    vecteur v(*args._VECTptr);
    gen a=remove_at_pnt(v.front()),b=remove_at_pnt(v[1]),c=remove_at_pnt(v[2]),d=remove_at_pnt(v[3]);
    return symb_pnt(gen(makevecteur(a,b,c,d,a),_GROUP__VECT),default_color(contextptr),contextptr);
  }
  const string _quadrilatere_s("quadrilatere");
  unary_function_eval __quadrilatere(&giac::_quadrilatere,_quadrilatere_s);
  unary_function_ptr at_quadrilatere (&__quadrilatere);

  void get_rectangle_args(const vecteur & v,gen & e,gen & f,gen & g,gen & tmp,GIAC_CONTEXT){
    e=remove_at_pnt(eval(v[0],contextptr));
    f=remove_at_pnt(eval(v[1],contextptr));
    gen d=remove_at_pnt(eval(v[2],contextptr));
    if (d.type==_VECT){
      gen eg,ef=f-e;
      if (d._VECTptr->size()==2){
	eg=remove_at_pnt(d._VECTptr->front())-e;
	tmp=d._VECTptr->back()*sqrt(dotvecteur(ef,ef));
      }
      else {
	eg=d-e;
	tmp=sqrt(dotvecteur(eg,eg));
      }
      eg=cross(cross(ef,eg),ef);
      tmp=tmp/sqrt(dotvecteur(eg,eg))*eg;
    }
    else
      tmp=(f-e)*d*cst_i;
    g=e+tmp;
  }

  gen _triangle_rectangle(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_triangle_rectangle,args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    gen e,f,g,tmp;
    get_rectangle_args(v,e,f,g,tmp,contextptr);
    gen res=symb_pnt(gen(makevecteur(e,f,g,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[3]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _triangle_rectangle_s("triangle_rectangle");
  unary_function_eval __triangle_rectangle(&giac::_triangle_rectangle,_triangle_rectangle_s);
  unary_function_ptr at_triangle_rectangle (&__triangle_rectangle,_QUOTE_ARGUMENTS);

  gen _rectangle(const gen & args, GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_rectangle,args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    gen e,f,g,tmp;
    get_rectangle_args(v,e,f,g,tmp,contextptr);
    gen h=f+tmp;
    gen res=symb_pnt(gen(makevecteur(e,f,h,g,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[3]),contextptr));
    if (s>4)
      w.push_back(eval(symb_sto(_point(h,contextptr),v[4]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _rectangle_s("rectangle");
  unary_function_eval __rectangle(&giac::_rectangle,_rectangle_s);
  unary_function_ptr at_rectangle (&__rectangle,_QUOTE_ARGUMENTS);

  gen _parallelogramme(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_parallelogramme,args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    gen e=remove_at_pnt(eval(v[0],contextptr)),f=remove_at_pnt(eval(v[1],contextptr)),d=remove_at_pnt(eval(v[2],contextptr));
    gen g=(f-e)+d;
    gen res=symb_pnt(gen(makevecteur(e,f,g,d,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[3]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _parallelogramme_s("parallelogramme");
  unary_function_eval __parallelogramme(&giac::_parallelogramme,_parallelogramme_s);
  unary_function_ptr at_parallelogramme (&__parallelogramme,_QUOTE_ARGUMENTS);

  void get_losange_args(const vecteur &v,gen &e,gen &f,gen & g,GIAC_CONTEXT){
    e=remove_at_pnt(eval(v[0],contextptr));
    f=remove_at_pnt(eval(v[1],contextptr));
    gen angle=remove_at_pnt(eval(v[2],contextptr));
    if (angle.type==_VECT){ // In 3-d angle is not the angle!
      gen ef=f-e;
      if (angle._VECTptr->size()==2){
	gen eg=remove_at_pnt(angle._VECTptr->front())-e;
	angle=angle._VECTptr->back(); 
	eg=cross(cross(ef,eg),ef);
	eg=sqrt(dotvecteur(ef,ef)/dotvecteur(eg,eg))*eg;
	g=e+ef*cos(angle,contextptr)+eg*sin(angle,contextptr);
      }
      else {
	gen eg=angle-e;
	g=e+sqrt(dotvecteur(ef,ef)/dotvecteur(eg,eg))*eg;
      }
    }
    else
      g=e+(f-e)*exp(cst_i*angle);
  }

  gen _triangle_isocele(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_triangle_isocele,args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    gen e,f,g;
    get_losange_args(v,e,f,g,contextptr);
    gen res=symb_pnt(gen(makevecteur(e,f,g,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[3]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _triangle_isocele_s("triangle_isocele");
  unary_function_eval __triangle_isocele(&giac::_triangle_isocele,_triangle_isocele_s);
  unary_function_ptr at_triangle_isocele (&__triangle_isocele,_QUOTE_ARGUMENTS);

  gen _losange(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_losange,args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    gen e,f,g;
    get_losange_args(v,e,f,g,contextptr);
    gen h=g;
    g=h-e+f;
    gen res=symb_pnt(gen(makevecteur(e,f,g,h,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[3]),contextptr));
    if (s>4)
      w.push_back(eval(symb_sto(_point(h,contextptr),v[4]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _losange_s("losange");
  unary_function_eval __losange(&giac::_losange,_losange_s);
  unary_function_ptr at_losange (&__losange,_QUOTE_ARGUMENTS);

  gen _triangle_equilateral(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_triangle_equilateral,args);
    vecteur & v=*args._VECTptr;
    int s=v.size(),dim=2;
    gen e=remove_at_pnt(eval(v[0],contextptr)),f=remove_at_pnt(eval(v[1],contextptr)),g;
    if (e.type==_VECT){
      dim=3;
      if (s==2)
	set3derr();
      g=remove_at_pnt(eval(v[2],contextptr));
      gen ef=f-e,eg=g-e;
      eg=cross(cross(ef,eg),ef);
      g=e+(ef+sqrt(3*dotvecteur(ef,ef)/dotvecteur(eg,eg))*eg)/2;
    }
    else
      g=e+(f-e)*rdiv((plus_sqrt3*cst_i+plus_one),plus_two);
    gen res=symb_pnt(gen(makevecteur(e,f,g,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==dim)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[dim]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _triangle_equilateral_s("triangle_equilateral");
  unary_function_eval __triangle_equilateral(&giac::_triangle_equilateral,_triangle_equilateral_s);
  unary_function_ptr at_triangle_equilateral (&__triangle_equilateral,_QUOTE_ARGUMENTS);

  // Args= Center A, point B, number of vertices
  extern unary_function_ptr at_isopolygone;
  gen _isopolygone(const gen & args,GIAC_CONTEXT){
    checkanglemode(contextptr);
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_isopolygone,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    gen e=remove_at_pnt(v[0]);
    gen f=remove_at_pnt(v[1]),g;
    gen ef=f-e,nd;
    if (s>3){
      g=remove_at_pnt(v[2]);
      gen eg=g-e;
      nd=cross(cross(ef,eg),ef);
      nd=sqrt(ratnormal(dotvecteur(ef,ef)/dotvecteur(nd,nd)))*nd;
    }
    else
      nd=cst_i*ef;
    gen gn=v[s-1];
    int n=gn.val;
    if (gn.type!=_INT_ || absint(n)<2)
      setsizeerr();  
    if (n>0){
      context tmp;
      gen c;
      c=(e+f)/2+nd/(2*tan(cst_pi/n,&tmp));
      f=e;
      e=c;
      ef=f-e;
      if (s>3){
	gen eg=g-e;
	nd=cross(cross(ef,eg),ef);
	nd=sqrt(ratnormal(dotvecteur(ef,ef)/dotvecteur(nd,nd)))*nd;
      }
      else
	nd=cst_i*ef;
    }
    else
      n=-n;
    vecteur w;
    w.push_back(f);
    for (int i=1;i<n;++i){
      w.push_back(e+ef*cos(2*cst_pi*i/n,contextptr)+nd*sin(2*cst_pi*i/n,contextptr));
    }
    w.push_back(f);
    gen res=symb_pnt(gen(w,_GROUP__VECT),default_color(contextptr),contextptr);
    return res;
  }
  const string _isopolygone_s("isopolygone");
  unary_function_eval __isopolygone(&giac::_isopolygone,_isopolygone_s);
  unary_function_ptr at_isopolygone (&__isopolygone,0,true);

  gen _carre(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_carre,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    gen e=remove_at_pnt(eval(v[0],contextptr));
    gen f=remove_at_pnt(eval(v[1],contextptr));
    gen ef=f-e,g;
    int dim=2;
    if (ef.type==_VECT){
      dim=3;
      if (s==2)
	set3derr();
      g=remove_at_pnt(eval(v[2],contextptr));
      gen eg=g-e;
      eg=cross(cross(ef,eg),ef);
      g=f+sqrt(dotvecteur(ef,ef)/dotvecteur(eg,eg))*eg;
    }
    else
      g=f+ef*cst_i;
    gen h=g-ef;
    gen res=symb_pnt(gen(makevecteur(e,f,g,h,e),_GROUP__VECT),default_color(contextptr),contextptr);
    if (s==dim)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(g,contextptr),v[dim]),contextptr));
    if (s>dim+1)
      w.push_back(eval(symb_sto(_point(h,contextptr),v[dim+1]),contextptr));
    return gen(w,_GROUP__VECT);
  }
  const string _carre_s("carre");
  unary_function_eval __carre(&giac::_carre,_carre_s);
  unary_function_ptr at_carre (&__carre,_QUOTE_ARGUMENTS);

  gen _polygone(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_polygone,args);
    vecteur v(*apply(args,remove_at_pnt)._VECTptr);
    v.push_back(v.front());
    return symb_pnt(gen(v,_GROUP__VECT),default_color(contextptr),contextptr);
  }
  const string _polygone_s("polygone");
  unary_function_eval __polygone(&giac::_polygone,_polygone_s);
  unary_function_ptr at_polygone (&__polygone);

  gen _polyedre(const gen & args,GIAC_CONTEXT);
  // Given a list of 3-d points, make a convex polyedre
  gen polyedre(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()<3)
      setsizeerr();
    vecteur v =*g._VECTptr;
    // Construct faces: easy algorithm
    // Make all possibles plans with 3 points, find equation
    // Add to the face any point that is in the plane (eq=0)
    // All other points must have the same sign
    // Otherwise break and try next triple of points
    iterateur i=v.begin(),ie=v.end();
    for (;i!=ie;++i){
      *i = remove_at_pnt(*i);
      if (i->type!=_VECT || i->_VECTptr->size()!=3)
	setdimerr();
    }
    vecteur faces;
    for (i=v.begin();i!=ie;++i){
      const_iterateur j=i+1;
      for (;j!=ie;++j){
	const_iterateur k=j+1;
	vecteur v1(subvecteur(*j->_VECTptr,*i->_VECTptr));
	for (;k!=ie;++k){
	  vecteur v2(subvecteur(*k->_VECTptr,*i->_VECTptr));
	  vecteur n(cross(v1,v2));
	  if (is_zero(n))
	    continue;
	  const_iterateur l=v.begin();
	  gen s; // sign
	  gen eq,eqs;
	  vecteur currentface;
	  for (;l!=ie;++l){
	    if (l==i || l==j || l==k){
	      currentface.push_back(*l);
	      continue;
	    }
	    eq=dotvecteur(n,subvecteur(*l->_VECTptr,*i->_VECTptr));
	    if (is_zero(eq)){
	      currentface.push_back(*l);
	      continue;
	    }
	    eqs=sign(eq,contextptr);
	    if (eq.type!=_INT_)
	      setsizeerr("Unable to check sign "+eq.print());
	    if (is_zero(s))
	      s=eqs;
	    if (eqs!=s)
	      break;
	  } // end for l
	  if (l==ie)
	    faces.push_back(currentface);
	}
      }
    }
    return _polyedre(faces,contextptr);
  }
  gen _polyedre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      setsizeerr();
    vecteur v(*args._VECTptr);
    int s=v.size();
    if (!s)
      setdimerr();
    if (s==2){
      // Base, sommet
      gen base=remove_at_pnt(v.front());
      if (base.type!=_VECT)
	setsizeerr();
      gen & sommet=v.back();
      vecteur w=*base._VECTptr;
      vecteur nv;
      if (w.front()!=w.back())
	w.push_back(w.front());
      int s=w.size();
      if (s<3)
	setdimerr();
      for (int i=0;i<s;++i){
	nv.push_back(makevecteur(w[i],w[(i+1)%s],sommet));
      }
      nv.push_back(base);
      return _polyedre(nv,contextptr);
    }
    gen g=remove_at_pnt(v.front());
    if (g.type==_VECT && g._VECTptr->size()==3 && remove_at_pnt(g._VECTptr->front()).type!=_VECT )
      return polyedre(v,contextptr);
    iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type!=_VECT)
	setsizeerr("Each element must be a face (vector of 3/4 points)");
      *it=apply(*it,remove_at_pnt);
      int s=it->_VECTptr->size();
      if (s<3)
	setsizeerr("at least 3 points by face");
    }
    return symb_pnt(gen(v,_POLYEDRE__VECT),default_color(contextptr),contextptr);
  }
  const string _polyedre_s("polyedre");
  unary_function_eval __polyedre(&giac::_polyedre,_polyedre_s);
  unary_function_ptr at_polyedre (&__polyedre,0,true);

  gen _prisme(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    gen base=remove_at_pnt(v[0]), sommet=remove_at_pnt(v[1]);
    if (base.type!=_VECT || base._VECTptr->size()<2)
      setsizeerr();
    vecteur w = *base._VECTptr;
    gen x=sommet-w[0];
    int s=w.size();
    vecteur faces;
    for (int i=0;i<s;++i){
      faces.push_back(makevecteur(w[i],w[(i+1)%s],w[(i+1)%s]+x,w[i]+x));
    }
    faces.push_back(base);
    for (int i=0;i<s;++i)
      w[i]=w[i]+x;
    faces.push_back(w);
    return _polyedre(faces,contextptr);
  }
  const string _prisme_s("prisme");
  unary_function_eval __prisme(&giac::_prisme,_prisme_s);
  unary_function_ptr at_prisme (&__prisme,0,true);

  gen _parallelepipede(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=4)
      setsizeerr();
    vecteur v(*args._VECTptr);
    gen & A=v[0];
    gen & B=v[1];
    gen & C=v[2];
    gen & D=v[3];
    gen AB=B-A,AC=C-A,AD=D-A;
    gen E=A+AB+AC,F=A+AC+AD,G=A+AB+AD,H=A+AB+AC+AD;
    vecteur res;
    // Face 1 A B // C E=A+AB+AC
    res.push_back(makevecteur(A,B,E,C));
    // Face 2 A C // D F=A+AC+AD
    res.push_back(makevecteur(A,C,F,D));
    // Face 3 A B // D G=A+AB+AD
    res.push_back(makevecteur(A,B,G,D));
    // Face 4 B E // G H
    res.push_back(makevecteur(B,E,H,G));
    // Face 5 C E // F H
    res.push_back(makevecteur(C,E,H,F));
    // Face 6 D G // F H
    res.push_back(makevecteur(D,G,H,F));
    return _polyedre(res,contextptr);
  }
  const string _parallelepipede_s("parallelepipede");
  unary_function_eval __parallelepipede(&giac::_parallelepipede,_parallelepipede_s);
  unary_function_ptr at_parallelepipede (&__parallelepipede,0,true);

  gen _pyramide(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()<3)
      setsizeerr();
    vecteur v(*args._VECTptr);
    gen & A=v[0];
    gen & B=v[1];
    gen C=v[2];
    if (v.size()==3){ // tetraedre
      gen AB=B-A,AC=C-A;
      if (AB.type!=_VECT || AB._VECTptr->size()!=3 || AC.type!=_VECT || AC._VECTptr->size()!=3)
	setsizeerr();
      vecteur v1(*AB._VECTptr),v2(*AC._VECTptr);
      vecteur n(cross(v1,v2));
      v2=cross(n,v1);
      // Normalize
      gen a(dotvecteur(v1,v1));
      v2=multvecteur(sqrt(3*a/dotvecteur(v2,v2)),v2);
      C = A + v1/2 + v2/2 ; 
      n=  multvecteur(sqrt(2*a/3/dotvecteur(n,n)),n);
      gen D = A + v1/2 + v2/6 + n;
      return _pyramide(makevecteur(A,B,C,D),contextptr);
    }
    gen & D=v[3];
    vecteur res;
    // Face 1 A B C
    res.push_back(makevecteur(A,B,C));
    // Face 2 A C D
    res.push_back(makevecteur(A,C,D));
    // Face 3 A B D
    res.push_back(makevecteur(A,B,D));
    // Face 4 B C D
    res.push_back(makevecteur(B,C,D));
    return _polyedre(res,contextptr);
  }
  const string _pyramide_s("pyramide");
  unary_function_eval __pyramide(&giac::_pyramide,_pyramide_s);
  unary_function_ptr at_pyramide (&__pyramide,0,true);

  const string _tetraedre_s("tetraedre");
  unary_function_eval __tetraedre(&giac::_pyramide,_tetraedre_s);
  unary_function_ptr at_tetraedre (&__tetraedre,0,true);
  
  // Find A,B,C,D such that AB=AC=AD and all are orthogonal
  void cube_octaedre(const gen & args,gen & A,gen & B,gen & C,gen & D,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    vecteur v(*args._VECTptr);
    A=v[0];
    B=v[1];
    C=v[2];
    gen AB(B-A),AC(C-A);
    if (AB.type!=_VECT || AB._VECTptr->size()!=3 || AC.type!=_VECT || AC._VECTptr->size()!=3)
      setsizeerr();
    // AB cross AC -> normal direction to ABC plan
    gen AB2(normal(scalar_product(AB,AB,contextptr)));
    vecteur AD=*normal(cross(*AB._VECTptr,*AC._VECTptr))._VECTptr;
    D=A+AD*sqrt(normal(AB2/dotvecteur(AD,AD)));
    // binormal direction gives the 2nd direction AB, AE, AD
    vecteur AE=*normal(cross(AD,*AB._VECTptr))._VECTptr;
    C=A+AE*sqrt(normal(AB2/dotvecteur(AE,AE)));
  }

  gen _tetraedre_centre(const gen & args,GIAC_CONTEXT){
    gen O,b,c,d;
    cube_octaedre(args,O,b,c,d,contextptr);
    gen v1(normal(b-O)),v2(normal(c-O)),v3(normal(d-O));
    gen A = b; // O + v1
    gen B = normal(O - v1/3 - sqrt(2)*v2/3 - sqrt(6)*v3/3);
    gen C = normal(O - v1/3 - sqrt(2)*v2/3 + sqrt(6)*v3/3);
    gen D = normal(O - v1/3 + 2*sqrt(2)*v2/3);
    return _pyramide(makevecteur(A,B,C,D),contextptr);
  }
  const string _tetraedre_centre_s("tetraedre_centre");
  unary_function_eval __tetraedre_centre(&giac::_tetraedre_centre,_tetraedre_centre_s);
  unary_function_ptr at_tetraedre_centre (&__tetraedre_centre,0,true);
  
  // args= 3 points A, B, C
  gen _cube(const gen & args,GIAC_CONTEXT){
    gen A,B,C,D;
    cube_octaedre(args,A,B,C,D,contextptr);
    return _parallelepipede(makevecteur(A,B,C,D),contextptr);
  }
  const string _cube_s("cube");
  unary_function_eval __cube(&giac::_cube,_cube_s);
  unary_function_ptr at_cube (&__cube,0,true);

  // args= center A, vertex B, point C such that ABC is a symmetry plan
  gen _cube_centre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    gen A,B,C,D;
    cube_octaedre(args,A,B,C,D,contextptr);
    gen x = (B-A)/3;
    // Take C1=A+cos(theta)*(B-A)+sin(theta)*(C-A), cos(theta)=1/3
    gen C1 = normal(A+x+2*sqrt(2)/3*(C-A));
    gen D1 = normal(A+x-sqrt(2)/3*(C-A)+sqrt(6)/3*(D-A));
    return _cube(makevecteur(B,D1,C1),contextptr);
  }
  const string _cube_centre_s("cube_centre");
  unary_function_eval __cube_centre(&giac::_cube_centre,_cube_centre_s);
  unary_function_ptr at_cube_centre (&__cube_centre,0,true);

  // args= center A, point B, C such that ABC = symmetry plan with 4 vertices
  gen _octaedre(const gen & args,GIAC_CONTEXT){
    gen A,B,C,D;
    cube_octaedre(args,A,B,C,D,contextptr);
    // B, C, D are 3 vertices
    gen E,F,G;
    E = A - (B-A);
    F = A - (C-A);
    G = A - (D-A);
    vecteur faces;
    faces.push_back(makevecteur(B,C,D));
    faces.push_back(makevecteur(B,C,G));
    faces.push_back(makevecteur(B,F,D));
    faces.push_back(makevecteur(B,F,G));
    faces.push_back(makevecteur(E,C,D));
    faces.push_back(makevecteur(E,C,G));
    faces.push_back(makevecteur(E,F,D));
    faces.push_back(makevecteur(E,F,G));
    return _polyedre(faces,contextptr);
  }
  const string _octaedre_s("octaedre");
  unary_function_eval __octaedre(&giac::_octaedre,_octaedre_s);
  unary_function_ptr at_octaedre (&__octaedre,0,true);

  void res_push(vecteur & res,gen * s, int i,int j,int k){
    res.push_back(makevecteur(s[i],s[j],s[k]));
  }
  // args= centre, sommet1, sommet2 (direction of)
  gen _icosaedre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    checkanglemode(contextptr);
    vecteur & v = *args._VECTptr;
    gen s[12];
    gen centre=v[0],s1=v[1],s2=v[2];
    gen v1g(s1-centre),v2g(s2-centre);
    // Icosaedre=s1+symetric of s1 with respect to center
    // + 2* 5 points as a pentagon on 2 plans perpendicular to centre->s1
    // If the distance of the 2 // plans is 1 to the center
    // Then the 5 vertices are at distance 2 to the intersection axe/plan
    // (abscisse=+/-1, sqrt(y^2+z^2)=2)
    // and |s1-centre|=sqrt(5)
    if (v1g.type!=_VECT|| v2g.type!=_VECT)
      setsizeerr();
    vecteur v1(*v1g._VECTptr),v2(*v2g._VECTptr);
    vecteur n(cross(v1,v2));
    v2=divvecteur(cross(n,v1),sqrt(dotvecteur(n,n))); 
    // norm=distance(centre,sommet)
    n=multvecteur(sqrt(dotvecteur(v1,v1)/dotvecteur(n,n)),n);
    // centre +/- (v1/sqrt(5) + 2/sqrt(5)*(cos(2*k*pi/5)*v2 +sin(2*k*pi/5)*n))
    s[0]=s1;
    s[11]=s1-2*v1;
    for (int i=0;i<5;++i){
      context ctmp;
      gen tmp = 1/sqrt(5)*(v1 + 2*(cos(2*i*cst_pi/5,&ctmp)*v2+sin(2*i*cst_pi/5,&ctmp)*n));
      s[1+i] = centre + tmp;
      s[10-i] = centre - tmp;
    }
    // Make 5 faces s[0] with s[1+i], s[2+i] for i in 1..4 and with 5,1
    // 5 with s[11] with s[11-i], s[10-i] for i in 1..4 and with 
    // 1,7,8 + 1,7,2 + 2,6,7 + 2,6,3 + 3,10,6 + 3,10,4 + 4,9,10 + 4,9,5 + 5,8,9+ 5,8,1
    vecteur res;
    for (int i=1;i<5;++i){
      res_push(res,s,0,i,1+i); res_push(res,s,11,11-i,10-i);
    }
    res_push(res,s,0,5,1); res_push(res,s,11,6,10);
    res_push(res,s,1,7,8); res_push(res,s,1,7,2);
    res_push(res,s,2,6,7); res_push(res,s,2,6,3);
    res_push(res,s,3,10,6); res_push(res,s,3,10,4);
    res_push(res,s,4,9,10); res_push(res,s,4,9,5);
    res_push(res,s,5,8,9); res_push(res,s,5,8,1);
    return _polyedre(res,contextptr);
  }
  const string _icosaedre_s("icosaedre");
  unary_function_eval __icosaedre(&giac::_icosaedre,_icosaedre_s);
  unary_function_ptr at_icosaedre (&__icosaedre,0,true);

  void res_push(vecteur & res,gen * s, int i,int j,int k,int l,int m){
    res.push_back(makevecteur(s[i],s[j],s[k],s[l],s[m]));
  }
  // args= centre, sommet1, 3rd point defining a plan containing the axis
  // Example dodecaedre([0,0,0],[0,2,sqrt(5)/2+3/2],[0,0,1])
  gen _dodecaedre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      setsizeerr();
    checkanglemode(contextptr);
    vecteur v = *evalf(args)._VECTptr;
    gen centre=v[0],s1=v[1],s2=v[2];
    gen v1g(s1-centre),v2g(s2-centre);
    if (v1g.type!=_VECT|| v2g.type!=_VECT)
      setsizeerr();
    vecteur v1(*v1g._VECTptr),v2(*v2g._VECTptr); 
    gen phi=evalf((sqrt(5)+1)/2);
    gen r2(dotvecteur(v1,v1)); // r = |v1| = sqrt(6+3phi)*unit of length
    vecteur w2(cross(v1,v2)); // y direction
    vecteur v3(cross(w2,v1)); // v1,v3 contains the axis v1.v3=0
    // Now normalize w2 to 1 unit of length
    w2=multvecteur(sqrt(r2/dotvecteur(w2,w2)/(6+3*phi)),w2);
    v3=multvecteur(sqrt(r2/dotvecteur(v3,v3)),v3); // |v3|=|v1|=r
    gen w1=(2*v1-(phi+1)*v3)/(6+3*phi); // |w1|=sqrt(6+3phi)*norm(v1)/(6+3phi)
    gen w3=((phi+1)*v1+2*v3)/(6+3*phi); // = |w2|=1 unit of length
    // Dodecaedre at center 0. Edge length=sqrt(10-2*sqrt(5))
    // Golden ratio phi=(sqrt(5)+1)/2 (phi^2=phi+1)
    // Vertices at +/-(2*cos(2*pi/5),2*sin(2*pi/5),phi+1)
    // and +/-(2*phi*cos(2*pi/5),2*phi*sin(2*pi/5),phi-1)
    // Sphere radius ^2 = 4 + (phi+1)^2 = 6+3*phi=(15+3*sqrt(5))/2
    gen s[20];
    context ctmp;
    for (int i=0;i<5;++i){
      s[i]=centre+evalf(2*cos(2*i*cst_pi/5,&ctmp)*w1+2*sin(2*i*cst_pi/5,&ctmp)*w2+(phi+1)*w3);
      s[15+i]=centre-s[i];
      s[5+i]=centre+evalf(2*phi*(cos(2*i*cst_pi/5,&ctmp)*w1+sin(2*i*cst_pi/5,&ctmp)*w2)+(phi-1)*w3);
      s[10+i]=centre-s[5+i];
    }
    vecteur res;
    res_push(res,s,0,1,2,3,4); res_push(res,s,15,16,17,18,19);
    for (int i=0;i<5;++i){
      res_push(res,s,(i+1)%5,5+(i+1)%5,10+(3+i)%5,5+i,i);
      res_push(res,s,15+(i+1)%5,10+(i+1)%5,5+(3+i)%5,10+i,15+i);
    }
    return _polyedre(res,contextptr);
  }
  const string _dodecaedre_s("dodecaedre");
  unary_function_eval __dodecaedre(&giac::_dodecaedre,_dodecaedre_s);
  unary_function_ptr at_dodecaedre (&__dodecaedre,0,true);

  gen bissectrice(const gen & args,bool interieur,GIAC_CONTEXT){
    gen e,f,g;
    gen23points(args,e,f,g);
    // direction (f-e)+(g-e)*||f-e||/||g-e||
    if (interieur)
      return symb_segment(e,f+(g-e)*rdiv(abs_norm(f-e,contextptr),abs_norm(g-e,contextptr)),default_color(contextptr),contextptr);
    else
      return symb_segment(e,f-(g-e)*rdiv(abs_norm(f-e,contextptr),abs_norm(g-e,contextptr)),default_color(contextptr),contextptr);
  }

  gen _bissectrice(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_bissectrice,args);
    return bissectrice(args,true,contextptr);
  }
  const string _bissectrice_s("bissectrice");
  unary_function_eval __bissectrice(&giac::_bissectrice,_bissectrice_s);
  unary_function_ptr at_bissectrice (&__bissectrice);

  gen _exbissectrice(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_exbissectrice,args);
    return bissectrice(args,false,contextptr);
  }
  const string _exbissectrice_s("exbissectrice");
  unary_function_eval __exbissectrice(&giac::_exbissectrice,_exbissectrice_s);
  unary_function_ptr at_exbissectrice (&__exbissectrice);

  gen _mediane(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_mediane,args);
    gen e,f,g;
    gen23points(args,e,f,g);
    // direction (f-e)+(g-e)*||f-e||/||g-e||
    return symb_segment(e,rdiv(f+g,plus_two),default_color(contextptr),contextptr);
  }
  const string _mediane_s("mediane");
  unary_function_eval __mediane(&giac::_mediane,_mediane_s);
  unary_function_ptr at_mediane (&__mediane);

  // chk if g is a pnt with 3 pnts as first argument
  gen chk_3pnt(const gen & g){
    if (!g.is_symb_of_sommet(at_pnt))
      return zero;
    gen & f=g._SYMBptr->feuille;
    if (f.type!=_VECT || f._VECTptr->empty())
      return zero;
    gen & g0=f._VECTptr->front();
    if (g0.type==_VECT && g0._VECTptr->size()>2)
      return vecteur(g0._VECTptr->begin(),g0._VECTptr->begin()+3);
    else
      return zero;
  }
  gen _circonscrit(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3)){
      gen tmp=chk_3pnt(args);
      if (is_zero(tmp))
	return symbolic(at_circonscrit,args);
      else
	return _circonscrit(tmp,contextptr);
    }
    vecteur v(*args._VECTptr);
    gen e,f,g,h;
    e=remove_at_pnt(v[0]);
    f=remove_at_pnt(v[1]);
    g=remove_at_pnt(v[2]);
    if (e.type==_VECT || f.type==_VECT || g.type==_VECT)
      set3derr();
    vecteur w(inter(_mediatrice(makevecteur(e,f),contextptr),_mediatrice(makevecteur(f,g),contextptr),contextptr)); 
    if (w.empty())
      setsizeerr();
    h=w.front();
    return _cercle(makevecteur(h,e-remove_at_pnt(h)),contextptr);
  }
  const string _circonscrit_s("circonscrit");
  unary_function_eval __circonscrit(&giac::_circonscrit,_circonscrit_s);
  unary_function_ptr at_circonscrit (&__circonscrit);

  // given 2 points e and f return equation of line e,f as coeffs a,b,c
  void point2abc(const gen & e,const gen & f,gen & a,gen & b,gen & c,GIAC_CONTEXT){
    gen tmp=f-e;
    if (tmp.type==_VECT)
      set3derr();
    a=im(tmp,contextptr);
    b=-re(tmp,contextptr);
    c=-a*re(e,contextptr)-b*im(e,contextptr);
    vecteur v(makevecteur(a,b,c));
    lcmdeno(v,c);
    a=v[0];
    b=v[1];
    c=v[2];
  }

  gen inscrit(const gen & args,bool exinscrit,GIAC_CONTEXT){
    vecteur v(*args._VECTptr),w;
    gen e,f,g;
    e=remove_at_pnt(v[0]);
    f=remove_at_pnt(v[1]);
    g=remove_at_pnt(v[2]);
    // find equations of droite(e,f) droite(e,g) droite(f,g)
    gen a_ef,b_ef,c_ef,a_fg,b_fg,c_fg,a_ge,b_ge,c_ge;
    point2abc(e,f,a_ef,b_ef,c_ef,contextptr);
    point2abc(f,g,a_fg,b_fg,c_fg,contextptr);
    point2abc(e,g,a_ge,b_ge,c_ge,contextptr);
    // value of equations droite(e,f) at g, etc.
    gen efg,fge,gef;
    efg=a_ef*re(g,contextptr)+b_ef*im(g,contextptr)+c_ef;
    fge=a_fg*re(e,contextptr)+b_fg*im(e,contextptr)+c_fg;
    gef=a_ge*re(f,contextptr)+b_ge*im(f,contextptr)+c_ge;
    gen eq_ef,eq_fg,eq_ge;
    gen sqrtg(sign(efg,contextptr)*sqrt(a_ef*a_ef+b_ef*b_ef)),sqrte(sign(fge,contextptr)*sqrt(a_fg*a_fg+b_fg*b_fg)),sqrtf(sign(gef,contextptr)*sqrt(a_ge*a_ge+b_ge*b_ge));
    vecteur coeff_eq_ef(makevecteur(a_ef/sqrtg,b_ef/sqrtg,c_ef/sqrtg));
    vecteur coeff_eq_fg(makevecteur(a_fg/sqrte,b_fg/sqrte,c_fg/sqrte));
    vecteur coeff_eq_ge(makevecteur(a_ge/sqrtf,b_ge/sqrtf,c_ge/sqrtf));
    matrice m(makevecteur(coeff_eq_fg,coeff_eq_ge,coeff_eq_ef));
    vecteur lv(alg_lvar(m));
    m=*e2r(m,lv)._VECTptr;
    // system to solve is A*x+B*y+C=0 D*x+E*y+F=0
    gen A,B,C,D,E,F;
    if (exinscrit){
      A=m[0][0]+m[1][0];
      B=m[0][1]+m[1][1];
      C=m[0][2]+m[1][2];
    }
    else {
      A=m[0][0]-m[1][0];
      B=m[0][1]-m[1][1];
      C=m[0][2]-m[1][2];
    }
    D=m[1][0]-m[2][0];
    E=m[1][1]-m[2][1];
    F=m[1][2]-m[2][2];
    // Solution is :
    gen x=(B*F-E*C)/(-D*B+A*E),y=(A*F-D*C)/(-A*E+B*D);
    // Compute r
    gen r=r2sym(dotvecteur(*m[0]._VECTptr,makevecteur(x,y,1)),lv);
    gen xy=r2sym(x+cst_i*y,lv);
    return _cercle(makevecteur(xy,r),contextptr);
    /*
    if (exinscrit)
      w=inter(bissectrice(makevecteur(e,f,g),false),bissectrice(makevecteur(f,g,e)));
    else
      w=inter(bissectrice(makevecteur(e,f,g)),bissectrice(makevecteur(f,g,e)));
    if (w.empty())
      setsizeerr();
    h=w.front();
    ef=_droite(makevecteur(e,f));
    w= inter(ef,_perpendiculaire(makevecteur(h,ef))); // point on circle
    if (w.empty())
      setsizeerr();
    d=w.front();
    return _cercle(makevecteur(h,d-h));
    */
  }

  gen _inscrit(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3)){
      gen tmp=chk_3pnt(args);
      if (is_zero(tmp))
	return symbolic(at_inscrit,args);
      else
	return inscrit(tmp,false,contextptr);
    }
    return inscrit(args,false,contextptr);
  }
  const string _inscrit_s("inscrit");
  unary_function_eval __inscrit(&giac::_inscrit,_inscrit_s);
  unary_function_ptr at_inscrit (&__inscrit);

  gen _exinscrit(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3)){
      gen tmp=chk_3pnt(args);
      if (is_zero(tmp))
	return symbolic(at_exinscrit,args);
      else
	return inscrit(tmp,true,contextptr);
    }
    return inscrit(args,true,contextptr);
  }
  const string _exinscrit_s("exinscrit");
  unary_function_eval __exinscrit(&giac::_exinscrit,_exinscrit_s);
  unary_function_ptr at_exinscrit (&__exinscrit);

  gen _isobarycentre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT){
      if (args.is_symb_of_sommet(at_pnt)){
	gen & f=args._SYMBptr->feuille;
	if (f.type!=_VECT)
	  return args;
	vecteur & v=*f._VECTptr;
	int s=v.size();
	if (!s)
	  return args;
	gen &g=v[0];
	if (g.type!=_VECT || (s=g._VECTptr->size())<2)
	  return args;
	gen sum;
	const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
	--itend;
	--s;
	for (;it!=itend;++it)
	  sum=sum+*it;
	return _point(sum/s,contextptr);
      }
      return _point(args,contextptr);
    }
    const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
    int i=0;
    gen sum;
    for (;it!=itend;++it,++i){
      sum = sum+remove_at_pnt(*it);
    }
    return symb_pnt(rdiv(sum,i),default_color(contextptr),contextptr);
  }
  const string _isobarycentre_s("isobarycentre");
  unary_function_eval __isobarycentre(&giac::_isobarycentre,_isobarycentre_s);
  unary_function_ptr at_isobarycentre (&__isobarycentre);

  gen inbarycentre(const gen & args,GIAC_CONTEXT){
    const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
    int i=0;
    gen sum,n;
    for (;it!=itend;++it,++i){
      if (it->type!=_VECT){
	sum = sum+remove_at_pnt(*it);
	n = n + 1;
      }
      else {
	vecteur & v = *it->_VECTptr;
	if (v.size()!=2)
	  setsizeerr();
	sum = sum + v.back()*remove_at_pnt(v.front());
	n = n + v.back();
      }
    }
    if (is_zero(n))
      setsizeerr("Sum of coeff is 0");
    return symb_pnt(rdiv(sum,n),default_color(contextptr),contextptr);
  }
  gen _barycentre(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->empty())
      setsizeerr();
    const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
    if (itend-it==2 && ckmatrix(args)){
      vecteur & v=*(it+1)->_VECTptr;
      if (!v.front().is_symb_of_sommet(at_pnt))
	return inbarycentre(_tran(args),contextptr);
    }
    return inbarycentre(args,contextptr);
  }

  const string _barycentre_s("barycentre");
  unary_function_eval __barycentre(&giac::_barycentre,_barycentre_s);
  unary_function_ptr at_barycentre (&__barycentre,0,true);

  gen scalar_product(const gen & a,const gen & b,GIAC_CONTEXT){
    if (a.type==_VECT && b.type==_VECT)
      return dotvecteur(*a._VECTptr,*b._VECTptr);
    gen res=re(a,contextptr)*re(b,contextptr)+im(a,contextptr)*im(b,contextptr);
    return res;
  }
  gen normalized_scalar_product(const gen & a,const gen & b,GIAC_CONTEXT){
    if (a.type==_VECT && b.type==_VECT)
      return dotvecteur(*a._VECTptr,*b._VECTptr)/sqrt(dotvecteur(*a._VECTptr,*a._VECTptr)*dotvecteur(*b._VECTptr,*b._VECTptr));
    gen res1=re(a,contextptr)*re(b,contextptr)+im(a,contextptr)*im(b,contextptr);
    gen res2=sqrt(a.squarenorm(contextptr)*b.squarenorm(contextptr));
    return rdiv(res1,res2);
  }
  // return t such that ta+(1-t)b is the projection of c on [a,b] 
  gen projection(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen num=scalar_product(b-c,b-a,contextptr),den=scalar_product(b-a,b-a,contextptr);
    return rdiv(num,den);
  }

  void rewrite_with_t_real(gen & eq,const gen & t,GIAC_CONTEXT){
    eq=subst(eq,im(t,contextptr),zero);
    eq=subst(eq,re(t,contextptr),t);
  }

  // test if a point f is on a parametric curve e
  // compute t if true
  bool on(const gen & e_orig,const gen & f,gen & t,GIAC_CONTEXT){
    gen e;
    if ( (e_orig.type==_SYMB) && (e_orig._SYMBptr->sommet==at_curve))
      e=e_orig._SYMBptr->feuille._VECTptr->front();
    else
      e=e_orig;
    if ((e.type!=_VECT) || (e._VECTptr->size()!=4) )
      settypeerr("on");
    vecteur ee(*e._VECTptr);
    gen tt=ee[1];
    gen tmin=ee[2],tmax=ee[3];
    gen eq=re(ee[0],contextptr)-re(f,contextptr);
    rewrite_with_t_real(eq,tt,contextptr); 
    vecteur v(solve(eq,tt,0,contextptr));
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if ( is_zero(subst(im(ee[0],contextptr),tt,*it)-im(f,contextptr)) && 
	   ck_is_greater(*it,tmin,contextptr) && ck_is_greater(tmax,*it,contextptr) ){
	t=*it;
	return true;
      }
    }
    return false;
  }
  // projection of a (true) point pp on a parametric curve
  // e=symb_cercle or line, returns t
  gen projection(const gen & e,const gen & pp,GIAC_CONTEXT){
    gen p=remove_at_pnt(pp);
    if (p.type==_SYMB){
      if ( (p._SYMBptr->sommet==at_cercle) || (p._SYMBptr->sommet==at_curve))
	setsizeerr();
    }
    if (p.type==_VECT)
      setsizeerr();
    gen sur=remove_at_pnt(e);
    if ( (sur.type==_VECT) && (sur._VECTptr->size()>=2)){
      gen a=sur._VECTptr->front();
      gen b=(*sur._VECTptr)[1];
      return projection(a,b,p,contextptr);
    }
    if ( (sur.type==_SYMB) && (sur._SYMBptr->sommet==at_cercle) ){
      gen centre,rayon;
      centre_rayon(sur,centre,rayon,false,0); // don't care about radius
      return arg(p-centre,contextptr); 
    }
    // if p is on e return 0
    if ( (sur.type==_SYMB) && (sur._SYMBptr->sommet==at_curve) ){
      sur = sur._SYMBptr->feuille._VECTptr->front();
      gen t;
      // if (on(sur,p,t))
      // return zero;
      // find all orthogonalities tangeant/rayon 
      vecteur v(*sur._VECTptr);
      identificateur tt=*v[1]._IDNTptr;
      gen tangeant(derive(v[0],tt));
      gen t_found=v[2];
      try {
	gen eq=scalar_product(tangeant,p-v[0],contextptr);
	// should expand and assume that tt is real
	rewrite_with_t_real(eq,v[1],contextptr);
	vecteur sol(solve(eq,v[1],0,contextptr));
	sol.push_back(v[3]);
	// find smallest 
	const_iterateur it=sol.begin(),itend=sol.end();
	gen distance2_found=distance2pp(subst(v[0],v[1],v[2]),p,contextptr),cur_distance2;
	for (;it!=itend;++it){
	  if (!is_zero(it->im(contextptr)))
	    continue;
	  cur_distance2=distance2pp(subst(v[0],v[1],*it),p,contextptr);
	  if (ck_is_greater(distance2_found,cur_distance2,contextptr)){
	    t_found=*it;
	    distance2_found=cur_distance2;
	  }
	} // end for
      } // end try
      catch(std::runtime_error & error ){ // could not solve
	// if curve is a function (plotfunc) return re(p)
	gen eq=re(v[0],contextptr);
	rewrite_with_t_real(eq,v[1],contextptr);
	vecteur sol(solve(eq-re(p,contextptr),v[1],0,contextptr));
	if (sol.empty())
	  setsizeerr();
	t_found=sol.front();
      }
      return t_found;
    }
    // unknown
    return undef;
  }

  // cercle to parametric curve
  gen cercle2curve(const gen & f){
    ck_parameter_t();
    gen centre,rayon;
    centre_rayon(f,centre,rayon,false,0); // don't care about radius sign
    return symb_curve(gen(makevecteur(centre+normal(rayon)*exp(cst_i*t__IDNT_e),t__IDNT_e,zero,cst_two_pi),_PNT__VECT),f);
  }

  // line to parametric curve
  gen line2curve(const gen & f){
    ck_parameter_t();
    if ( (f.type!=_VECT) || (f._VECTptr->size()!=2))
      setsizeerr("line2curve");
    gen A(f._VECTptr->front());
    gen B(f._VECTptr->back());
    gen tmin,tmax;
    if (f.subtype==_LINE__VECT){
      tmin=minus_inf;
      tmax=plus_inf;
    }
    else {
      tmin=zero;
      tmax=plus_one;
    }
    return symb_curve(gen(makevecteur(ratnormal(t__IDNT_e*A+(1-t__IDNT_e)*B),t__IDNT_e,tmin,tmax),_PNT__VECT),f);
  }

  // square distance curve/curve
  gen distance2cc(const gen & e,const gen & f,GIAC_CONTEXT){
    setsizeerr("Distance curve/curve not implemented");
    return zero;
  }
  // square distance curve/point or curve/curve
  // ee is a curve
  gen distance2cp(const gen & ee,const gen & f,GIAC_CONTEXT){
    gen e=ee._SYMBptr->feuille._VECTptr->front();
    if ((f.type==_SYMB) && (f._SYMBptr->sommet==at_curve))
      return distance2cc(e,f._SYMBptr->feuille._VECTptr->front(),contextptr);
    if ((f.type==_SYMB) && (f._SYMBptr->sommet==at_cercle))
      return distance2cp(ee,cercle2curve(f),contextptr);
    vecteur v(*e._VECTptr);
    gen projete=subst(v[0],v[1],projection(ee,f,contextptr));
    return distance2pp(projete,f,contextptr);
  }

  // square distance point/point
  gen distance2pp(const gen & ee,const gen & ff,GIAC_CONTEXT){
    gen e(remove_at_pnt(ee));
    gen f(remove_at_pnt(ff));
    if (e.type==_VECT){
      if (e.subtype==_POINT__VECT){ // n-d point
	vecteur & ev = *e._VECTptr;
	if (f.type==_VECT && f.subtype==_POINT__VECT){ // square distance between 2 n-d points
	  vecteur ef(subvecteur(*f._VECTptr,ev));
	  return dotvecteur(ef,ef);
	}
	if (f.is_symb_of_sommet(at_hyperplan)){ // point to hyperplan
	  // get normal vector n and base point P of hyperplan
	  gen & ff=f._SYMBptr->feuille;
	  if (ff.type==_VECT && ff._VECTptr->size()==2){
	    gen & P = ff._VECTptr->front();
	    gen & n = ff._VECTptr->back();
	    // -> ((P-e).n)^2/n.n
	    if (P.type==_VECT && n.type==_VECT){
	      vecteur & nv = *n._VECTptr;
	      vecteur & Pv = *P._VECTptr;
	      return dotvecteur(subvecteur(Pv,ev),nv)/dotvecteur(nv,nv);
	    }
	  }
	}
	if (f.is_symb_of_sommet(at_hypersphere)){
	  // point to hypersphere: (abs_norm(point-center)-radius)^2
	  gen centre,rayon;
	  centre_rayon(e,centre,rayon,true,contextptr); 
	  gen delta=abs_norm(f-centre,contextptr)-rayon; 
	  return delta*delta;
	}
	if (f.type==_VECT && f._VECTptr->size()==2){ // point to line
	  gen & A=f._VECTptr->front();
	  gen & B=f._VECTptr->back();
	  if (A.type==_VECT && B.type==_VECT){
	    // M=A+t(B-A), find t s.t. B-A is orthogonal to e-M
	    // (B-A).e=(B-A).M=(B-A).A+t(B-A).(B-A)
	    // t=(B-A).(e-A)/(B-A).(B-A)
	    vecteur & vA=*A._VECTptr;
	    vecteur & vB=*B._VECTptr;
	    vecteur vAB(subvecteur(vB,vA));
	    vecteur veA(subvecteur(ev,vA));
	    gen t=dotvecteur(veA,vAB)/dotvecteur(vAB,vAB);
	    // distance2 = (M-e).(M-e)
	    vecteur veM(addvecteur(veA,multvecteur(t,vAB)));
	    return dotvecteur(veM,veM);
	  }
	}
      } // end e of type n-d point
      if (e._VECTptr->size()==2){ // e of type line
	if (f.type==_VECT && f.subtype==_POINT__VECT)
	  return distance2pp(f,e,contextptr);
	if (f.type==_VECT && f._VECTptr->size()==2){ // line to line
	  gen M,N;
	  vecteur n;
	  if (perpendiculaire_commune(e,f,M,N,n,contextptr)){
	    gen MN(M-N);
	    if (MN.type==_VECT){
	      vecteur & vMN=*MN._VECTptr;
	      return dotvecteur(vMN,vMN);
	    }
	  }
	}
      }
      setsizeerr();
    }
    if (f.type==_VECT) // f is a line or a point
      return distance2pp(f,e,contextptr);
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_curve))
      return distance2cp(e,f,contextptr);
    if ((f.type==_SYMB) && (f._SYMBptr->sommet==at_curve))
      return distance2cp(f,e,contextptr);
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_cercle)){
      if ( (f.type==_SYMB) && (f._SYMBptr->sommet==at_cercle)){
	gen centre1,rayon1,centre2,rayon2;
	centre_rayon(e,centre1,rayon1,true,contextptr); 
	centre_rayon(f,centre2,rayon2,true,contextptr); 
	gen r=abs_norm(centre2-centre1,contextptr)-rayon1-rayon2;
	if (ck_is_positive(r,contextptr)) 
	  return r*r;
	else
	  return zero;
      }
      // cercle <-> point: (abs_norm(point-center)-radius)^2
      gen centre,rayon;
      centre_rayon(e,centre,rayon,true,contextptr); 
      gen delta=abs_norm(f-centre,contextptr)-rayon; 
      return delta*delta;
    }
    if ( (f.type==_SYMB) && (f._SYMBptr->sommet==at_cercle))
      return distance2pp(f,e,contextptr);
    return pow(re(e-f,contextptr),2)+pow(im(e-f,contextptr),2);
  }

  // square distance line/point
  gen distance2sp(const_iterateur it,const const_iterateur itend,const gen & p,GIAC_CONTEXT){
    gen res,newres,a,b,t,c,r;
    bool is_cercle=centre_rayon(p,c,r,false,0);
    a=*it;
    res=distance2pp(a,p,contextptr);
    ++it;
    for (;;){
      b=*it;
      if (is_cercle)
	t=projection(a,b,c,contextptr);
      else
	t=projection(a,b,p,contextptr);
      if (ck_is_positive(t,contextptr) && (ck_is_greater(1,t,contextptr)))
	newres=distance2pp(t*a+(1-t)*b,p,contextptr);
      else
	newres=distance2pp(b,p,contextptr);
      if (ck_is_greater(res,newres,contextptr))
	res=newres;
      ++it;
      if (it==itend)
	break;
      a=b;
    }
    return res;
  }

  gen distance2(const gen & f1,const gen & f2,GIAC_CONTEXT){
    gen e1(f1),e2(f2);
    vecteur v1,v2;
    if ((e1.type==_SYMB) && (e1._SYMBptr->sommet==at_pnt))
      e1=e1._SYMBptr->feuille._VECTptr->front();
    if (e1.type!=_VECT || e1.subtype==_POINT__VECT)
      v1=makevecteur(e1);
    else
      v1=*e1._VECTptr;
    if ((e2.type==_SYMB) && (e2._SYMBptr->sommet==at_pnt))
      e2=e2._SYMBptr->feuille._VECTptr->front();
    if (e2.type!=_VECT || e2.subtype==_POINT__VECT)
      v2=makevecteur(e2);
    else
      v2=*e2._VECTptr;
    const_iterateur it=v1.begin(),itend=v1.end(),jt=v2.begin(),jtend=v2.end();
    if ( (itend==it+1) && (jtend==jt+1) )
      return distance2pp(*it,*jt,contextptr);
    if (itend==it+1) 
      return distance2sp(jt,jtend,*it,contextptr);
    if (jtend==jt+1)
      return distance2sp(it,itend,*jt,contextptr);
    gen res=plus_inf,newres;
    for (;it!=itend;++it){
      for (jt=v2.begin();jt!=jtend;++jt){
	newres=distance2pp(*it,*jt,contextptr);
	if (ck_is_strictly_greater(res,newres,contextptr));
	  res=newres;
      }
    }
    return res;
  }
  gen _longueur2(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
      return symbolic(at_longueur2,args);
    gen e1=args._VECTptr->front(),e2=args._VECTptr->back();
    return distance2(e1,e2,contextptr);
  }
  const string _longueur2_s("longueur2");
  unary_function_eval __longueur2(&giac::_longueur2,_longueur2_s);
  unary_function_ptr at_longueur2 (&__longueur2);

  gen _longueur(const gen & args,GIAC_CONTEXT){
    return sqrt(_longueur2(args,contextptr));
  }
  const string _longueur_s("longueur");
  unary_function_eval __longueur(&giac::_longueur,_longueur_s);
  unary_function_ptr at_longueur (&__longueur);

  gen _aire(const gen & args,GIAC_CONTEXT){
    gen g=remove_at_pnt(args);
    if (g.is_symb_of_sommet(at_cercle)){
      gen centre,rayon;
      centre_rayon(g,centre,rayon,false,contextptr);
      return cst_pi*pow(rayon,2);
    }
    if (g.type!=_VECT)
      return undef;
    vecteur v=*g._VECTptr;
    int s=v.size();
    if (s<3)
      return undef;
    if (v.front()!=v.back()){
      ++s;
      v.push_back(v.front());
    }
    gen res;
    for (int i=3;i<s;++i){
      gen cote1(v[i-2]-v[0]),cote2(v[i-1]-v[0]);
      res = res +im(cote2,contextptr)*re(cote1,contextptr)-re(cote2,contextptr)*im(cote1,contextptr);
    }
    return res/2;
  }
  const string _aire_s("aire");
  unary_function_eval __aire(&giac::_aire,_aire_s);
  unary_function_ptr at_aire (&__aire,0,true);

  // angle accepts an optionnal last argument as a string
  // for legends, if the string has a terminal =, the value of
  // the angle is added to the legend
  gen _angle(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || args.subtype!=_SEQ__VECT || (args._VECTptr->size()<2))
      return arg(args,contextptr);
    vecteur v(*args._VECTptr);
    bool montrer=false;
    string legende;
    if (v.back().type==_STRNG){
      montrer=true;
      legende=*v.back()._STRNGptr;
      v.pop_back();
      if (v.size()<2)
	setsizeerr();
    }
    gen e=v.front(),f=v[1],g;
    e=remove_at_pnt(e);
    f=remove_at_pnt(f);
    if (e.type==_VECT){
      if ((e._VECTptr->size()!=2) || (f._VECTptr->size()!=2))
	setsizeerr("angle");
      vecteur w=inter(v.front(),v[1],0); // context does not apply for lines
      if (w.empty())
	setsizeerr();
      g=remove_at_pnt(w.front());
      f=f._VECTptr->back();
      e=e._VECTptr->back();
    }
    else {
      if (f.type==_VECT){
	if (f._VECTptr->size()!=2)
	  setsizeerr("angle");
	g=f._VECTptr->back();
	f=f._VECTptr->front();
      }
      else {
	if (v.size()!=3)
	  setsizeerr("angle");
	g=remove_at_pnt(v[2]);
      }
    }
    bool tmp=angle_radian(contextptr);
    angle_radian(true,contextptr);
    gen res=arg(conj(f-e,contextptr)*(g-e),contextptr);
    gen c;
    montrer = montrer && f.type!=_VECT;
    if (montrer) 
      // Show the angle on the figure, using an arc of circle
      c=_cercle(makevecteur(e,(f-e)/5,0,res),contextptr);
    if (!tmp){
      angle_radian(tmp,contextptr);
      if (res.type==_DOUBLE_)
	res= res*rad2deg_d;
      else
	res=res*rad2deg_e;
    }
    if (montrer && c.is_symb_of_sommet(at_pnt) && c._SYMBptr->feuille.type==_VECT && c._SYMBptr->feuille._VECTptr->size()==2){
      vecteur v=*c._SYMBptr->feuille._VECTptr;
      if (!legende.empty() && legende[legende.size()-1]=='=')
	v.push_back(string2gen(legende+res.print(),false));
      else
	v.push_back(string2gen(legende,false));
      c=symbolic(at_pnt,v);
      return gen(makevecteur(res,c),_SEQ__VECT);
    }
    return res;
  }
  const string _angle_s("angle");
  unary_function_eval __angle(&giac::_angle,_angle_s);
  unary_function_ptr at_angle (&__angle);

  // added protection
  // find neighboor points of p in a history vector v (distance < eps)
  vector<int> nearest_point(const vecteur & v,const gen & p,double eps,GIAC_CONTEXT){
    vector<int> res;
    gen d;
    const_iterateur it=v.begin(),itend=v.end();
    for (int i=0;it!=itend;++it,++i){
      if ( (it->type==_SYMB) && equalposcomp(plot_sommets,it->_SYMBptr->sommet)){
	try {
	  d=distance2(*it,p,contextptr).evalf_double(DEFAULT_EVAL_LEVEL,contextptr); 
	}
	catch (std::runtime_error & error ){
	  cerr << error.what() << endl;
	  d=zero;
	}
	if ((d.type==_DOUBLE_) && (d._DOUBLE_val<eps*eps))
	  res.push_back(i);
      }
    }
    return res;
  }

      // used to signal an intermediate answer to the parent process
  gen _signal(const gen & args){
#ifndef WIN32
    if (child_id) // must be a child process!
      return symbolic(at_signal,args);
#endif
    gen args_evaled;
    try {
      args_evaled=args.eval();
    }
    catch (std::runtime_error & error ){
      args_evaled = string2gen('"'+string(error.what())+'"');
    }
#ifdef WIN32
    history_in.push_back(symbolic(at_signal,args));
    history_out.push_back(args_evaled);
    plot_instructions.push_back(args_evaled);    
    return args_evaled;
#else // WIN32
    ofstream child_out(cas_sortie_name().c_str());
    archive(child_out,symbolic(at_signal,args));
    archive(child_out,args_evaled);
    child_out << messages_to_print << '' ;
    child_out.close();
    // cerr << "Signal reads " << res << endl;
    return wait_parent();
#endif // WIN32
  }
  const string _signal_s("signal");
  unary_function_unary __signal(&giac::_signal,_signal_s);
  unary_function_ptr at_signal (&__signal,_QUOTE_ARGUMENTS);

  vecteur gen2vecteur(const gen & args){
    if (args.type==_VECT)
      return *args._VECTptr;
    else
      return vecteur(1,args);
  }
  // user input sent back to the parent process
  // Use a vector of format [message,default_value,variable,convert_to_string]
  gen _click(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    int vs=v.size();
    if (vs>1)
      v[1]=eval(v[1],contextptr);
    gen res;
#ifdef WIN32
    cout << "// " << args ;
    string s;
    cin >> s;
    if (vs==4)
      res=string2gen(s,false);
    else
      res=gen(s);
#else
    if (child_id){ 
      cout << "// " << args;
      string s;
      cin >> s;
      if (vs==4)
	res=string2gen(s,false);
      else
	res=gen(s);
    }
    else {
      ofstream child_out(cas_sortie_name().c_str());
      gen e(symbolic(at_click,args));
      // cerr << "Archiving " << e << endl;
      archive(child_out,e);
      archive(child_out,e);
      if ( (args.type==_VECT) && (args._VECTptr->empty()) )
	child_out << "User input requested\n" << '' ;
      else
	child_out << args << '' ;
      child_out.close();
      kill_and_wait_sigusr2();
      ifstream child_in(cas_entree_name().c_str());
      res= unarchive(child_in,list_one_letter__IDNT);
      child_in.close();
      // cerr << "Click reads " << res << endl;
    }
#endif //WIN32
    if (vs>2 && !is_zero(v[2])){
      if (res.type==_VECT){
	if (vs==4 && res._VECTptr->size()==2){
	  vecteur tmp=*res._VECTptr;
	  if (tmp[1].is_symb_of_sommet(at_sto)){
	    gen f=tmp[1]._SYMBptr->feuille;
	    if (f.type==_VECT && f._VECTptr->size()==2){
	      tmp[1]=symb_sto(string2gen(f._VECTptr->front().print(),false),f._VECTptr->back());
	      cerr << tmp[1] << endl;
	      res=tmp;
	    }
	  }
	}
	return eval(res,contextptr);
      }
      else
	return sto(res,v[2],contextptr); 
    }
    else
      return res;
  }
  const string _click_s("click");
  unary_function_eval __click(&giac::_click,_click_s);
  unary_function_ptr at_click (&__click,_QUOTE_ARGUMENTS);

  vecteur make_VECTifnot_VECT(const gen & e){
      if ((e.type==_VECT) && !e._VECTptr->empty())
          return *e._VECTptr;
      return makevecteur(e);
  }

  gen parameter2point(const vecteur & v,GIAC_CONTEXT){
    // convert geometric object, parameter value to
    // a point on the geometric object
    gen res;
    assert(v.size()>=2);
    gen t=v[1];
    gen geo_obj=remove_at_pnt(v[0]);
    // geo_obj.type = _VECT (ligne brisee), _SYMB (at_cercle), symb_curve
    if (geo_obj.type==_VECT){
      vecteur ligne=*geo_obj._VECTptr;
      assert(ligne.size()>=2);
      if (t.type!=_VECT)
	return symb_pnt(t*ligne[0]+(1-t)*ligne[1],default_color(contextptr),contextptr);
      vecteur param=*t._VECTptr;
      assert(param.size()>=2);
      assert(param[0].type==_INT_);
      int n=param[0].val;
      if (ligne.size()<=n)
	n=ligne.size()-1;
      return symb_pnt((plus_one-param[1])*ligne[n]+param[1]*ligne[n+1],default_color(contextptr),contextptr);
    }
    if ( (geo_obj.type==_SYMB) && (geo_obj._SYMBptr->sommet==at_cercle)){
      gen centre,rayon;
      centre_rayon(geo_obj,centre,rayon,true,contextptr);
      return symb_pnt(centre+normal(rayon)*exp(cst_i*t),default_color(contextptr),contextptr);
    }
    if ( (geo_obj.type==_SYMB) && (geo_obj._SYMBptr->sommet==at_curve)){
      vecteur v(*geo_obj._SYMBptr->feuille._VECTptr->front()._VECTptr);
      return subst(v[0],v[1],t);
    }
    return res;
  }
  
  gen _element(const gen & args,GIAC_CONTEXT){
    if ( ((args.type==_SYMB) && (args._SYMBptr->sommet==at_interval)) || ((args.type==_VECT) && (args._VECTptr->size()==2) && (args._VECTptr->front().type==_SYMB) && (args._VECTptr->front()._SYMBptr->sommet==at_interval)) )
      return symbolic(at_parameter,args);
    gen a(args);
    vecteur v(make_VECTifnot_VECT(a));
    if (args.type==_VECT){
      v=(*a._VECTptr);
      if (v.empty())
	setsizeerr();
      if ( (v.front().type==_VECT) && (v.front()._VECTptr->size()) )
	v.front()=v.front()._VECTptr->front();
      if ( (v.front().type!=_SYMB) || (v.front()._SYMBptr->sommet!=at_pnt))
	v=make_VECTifnot_VECT(v.front());
    }
    if (v.size()==1)
      v.push_back(plus_one_half);
    gen s=remove_at_pnt(parameter2point(v,contextptr));
    return symbolic(at_pnt,gen(makevecteur(s,makevecteur(FL_BLACK,v)),_PNT__VECT));
  }
  const string _element_s("element");
  unary_function_eval __element(&giac::_element,_element_s);
  unary_function_ptr at_element (&__element);

  // returns res as a local bloc
  // def_x is the definition of x as read from history
  void reeval_with_1arg_quoted(const gen & x,gen & res,gen & def_x){
    // cerr << x << " " << res << " " << history_in << endl;
    // find first occurence of storing something in x in the history
    def_x=undef;
    const_iterateur it0=history_in.begin()-1,itend=history_in.end(),itpos,it;
    it=itend-1;
    vecteur localvars;
    for (;;--it){
      if (it==it0)
	setsizeerr();
      // cerr << *it << " " << x << endl;
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_sto) ){
	if (it->_SYMBptr->feuille._VECTptr->back()==x){
	  def_x=it->_SYMBptr->feuille._VECTptr->front();
	  break;
	}
      }
    }
    ++it;
    itpos=it;
    // cerr << "Position " << itpos-history_in.begin() << endl;
    for (;it!=itend;++it){
      // cerr << *it << " " << x << endl;
      if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_sto) ){
	if (it->_SYMBptr->feuille._VECTptr->back()==res)
	  break;
	if (it->_SYMBptr->feuille._VECTptr->back().type==_IDNT)
	  localvars.push_back(it->_SYMBptr->feuille._VECTptr->back());
      }
    }
    if (it==itend)
      setsizeerr();
    ++it;
    vecteur prog(itpos,it);
    prog.back()=symbolic(at_return,(it-1)->_SYMBptr->feuille._VECTptr->front());
    res=symb_local(gen(localvars,_SEQ__VECT),prog);
  }
  
  gen _as_function_of(const gen & args){
    if ( rpn_mode || as_function_of_mode || (args.type!=_VECT) || (args._VECTptr->size()!=2) || (args._VECTptr->back().type!=_IDNT) )
      return symbolic(at_as_function_of,args);
    gen res=args._VECTptr->front();
    gen x=args._VECTptr->back(),def_x;
    reeval_with_1arg_quoted(x,res,def_x);
    return symb_program(x,zero,res);
  }
  const string _as_function_of_s("as_function_of");
  unary_function_unary __as_function_of(&giac::_as_function_of,_as_function_of_s);
  unary_function_ptr at_as_function_of (&__as_function_of,_QUOTE_ARGUMENTS);

  gen _lieu(const gen & args,GIAC_CONTEXT){
    if ( rpn_mode || as_function_of_mode || (args.type!=_VECT) || (args._VECTptr->size()!=2) || (args._VECTptr->back().type!=_IDNT) )
      return symbolic(at_lieu,args);
    bool old_signal_store=signal_store;
    signal_store=false;
    bool old_io_graph=io_graph(contextptr);
    io_graph(false,contextptr);
    try {
      gen res=args._VECTptr->front();
      gen x=args._VECTptr->back(),def_x;
      reeval_with_1arg_quoted(x,res,def_x);
      gen prog=symb_program(x,zero,res);
      if ( (def_x.type==_SYMB) && (def_x._SYMBptr->sommet==at_element) ){
	gen e;
	if (def_x._SYMBptr->feuille.type==_VECT) {
	  vecteur & v=*def_x._SYMBptr->feuille._VECTptr;
	  if ( !v.empty() )
	    e=v.front();
	}
	else
	  e=def_x._SYMBptr->feuille;
	if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_interval)){
	  gen tmin=e._SYMBptr->feuille._VECTptr->front().evalf_double();
	  gen tmax=e._SYMBptr->feuille._VECTptr->back().evalf_double();
	  if ( (tmin.type==_DOUBLE_) && (tmax.type==_DOUBLE_) ){
	    vecteur p;
	    double tmin_d=tmin._DOUBLE_val,tmax_d=tmax._DOUBLE_val;
	    for (double t=tmin_d;t<tmax_d;t+=gnuplot_tstep){
	      p.push_back(remove_at_pnt(prog(t,contextptr)));
	    }
	    signal_store=old_signal_store;
	    io_graph(old_io_graph,contextptr);
	    return symb_pnt(gen(p,_GROUP__VECT),default_color(contextptr),contextptr);
	  }
	}
      }
      // x must eval to a parametric element of an object
      gen tt=x.eval(2);
      // type checking
      if ( (tt.type!=_SYMB) || (tt._SYMBptr->sommet!=at_pnt) || (tt._SYMBptr->feuille.type!=_VECT) )
	setsizeerr();
      vecteur v=*tt._SYMBptr->feuille._VECTptr;
      gen t0=v[1];
      if (t0.type!=_VECT)
	setsizeerr();
      vecteur w=*t0._VECTptr;
      t0=w[1];
      if (t0.type!=_VECT)
	setsizeerr();
      w=*t0._VECTptr;
      w[0]=w[0].evalf();
      bool is_ligne_brisee=(w[1].type==_VECT);
      vecteur p;
      for (double t=gnuplot_tmin;t<gnuplot_tmax;t+=gnuplot_tstep){
	if (is_ligne_brisee)
	  w[1]._VECTptr->back()=t;
	else
	  w[1]=t;
	gen tmp=prog(parameter2point(w,contextptr),contextptr);
	// makes lieu easier if the image is a line intersection
	if (tmp.type==_VECT && tmp._VECTptr->size()==1)
	  tmp=tmp._VECTptr->front();
	p.push_back(remove_at_pnt(tmp));
      }
      signal_store=old_signal_store;
      io_graph(old_io_graph,contextptr);
      return symb_pnt(gen(p,_GROUP__VECT),default_color(contextptr),contextptr);
    }
    catch (std::runtime_error & e){
      signal_store=old_signal_store;
      io_graph(old_io_graph,contextptr);
      throw(std::runtime_error(e.what()));
    }
  }
  const string _lieu_s("lieu");
  unary_function_eval __lieu(&giac::_lieu,_lieu_s);
  unary_function_ptr at_lieu (&__lieu,_QUOTE_ARGUMENTS);

  gen _head(const gen & args){
    if (args.type==_STRNG){
      string & s =*args._STRNGptr;
      int l=s.size();
      if (!l)
	return args;
      return string2gen(s.substr(0,1),false);
    }
    if (args.type!=_VECT)
      return args;
    if (args._VECTptr->size())
      return args._VECTptr->front();
    else
      return args;
  }
  const string _head_s("head");
  unary_function_unary __head(&giac::_head,_head_s);
  unary_function_ptr at_head (&__head);

  gen _tail(const gen & args){
    if (args.type==_STRNG){
      string & s =*args._STRNGptr;
      int l=s.size();
      if (!l)
	return args;
      return string2gen(s.substr(1,l-1),false);
    }
    if (args.type!=_VECT)
      return vecteur(0);
    if (args._VECTptr->size()){
      const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
      return gen(vecteur(it+1,itend),args.subtype);
    }
    else
      return args;
  }
  const string _tail_s("tail");
  unary_function_unary __tail(&giac::_tail,_tail_s);
  unary_function_ptr at_tail (&__tail);

  // return the vector of affixes of vertices of a polygonal line
  // If popback is true and the polygonal line is closed
  // the last vertex is removed from the returned vector
  vecteur sommet(const gen & args,bool popback=true){
    if (args.type==_VECT && !args._VECTptr->empty()){
      vecteur v(*args._VECTptr);
      gen v0=remove_at_pnt(v[0]);
      if (v0.type==_VECT)
	return sommet(v0,popback);
    }
    gen g=remove_at_pnt(args);
    vecteur v(gen2vecteur(g));
    if (ckmatrix(v.front())){ // polyedre
      vecteur res;
      const_iterateur it=v.begin(),itend=v.end();
      for (;it!=itend;++it){
	if (!ckmatrix(*it))
	  setsizeerr();
	const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
	for (;jt!=jtend;++jt){
	  if (!equalposcomp(res,*jt))
	    res.push_back(*jt);
	}
      }
      return res;
    }
    if (popback && v.size()>1 && v.back()==v.front())
      v.pop_back();
    return v;
  }

  gen _sommets(const gen & args,GIAC_CONTEXT){
    gen g=sommet(args,true);
    bool tmp=show_point(contextptr);
    show_point(false,contextptr);
    g=apply(g,_point,contextptr);
    show_point(tmp,contextptr);
    return g;
  }
  const string _sommets_s("sommets");
  unary_function_eval __sommets(&giac::_sommets,_sommets_s);
  unary_function_ptr at_sommets (&__sommets,0,true);

  gen _sommets_abca(const gen & args,GIAC_CONTEXT){
    gen g=sommet(args,false);
    bool tmp=show_point(contextptr);
    show_point(false,contextptr);
    g=apply(g,_point,contextptr);
    show_point(tmp,contextptr);
    return g;
  }
  const string _sommets_abca_s("sommets_abca");
  unary_function_eval __sommets_abca(&giac::_sommets_abca,_sommets_abca_s);
  unary_function_ptr at_sommets_abca (&__sommets_abca,0,true);

  const string _sommets_abc_s("sommets_abc");
  unary_function_eval __sommets_abc(&giac::_sommets,_sommets_abc_s);
  unary_function_ptr at_sommets_abc (&__sommets_abc,0,true);

  gen _aretes(const gen & args,GIAC_CONTEXT){
    bool tmp=show_point(contextptr);
    show_point(false,contextptr);
    gen g=remove_at_pnt(args);
    vecteur v(gen2vecteur(g));
    vecteur res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (!ckmatrix(*it))
	setsizeerr();
      const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
      for (;jt+1!=jtend;++jt){
	res.push_back(_segment(makevecteur(*jt,*(jt+1)),contextptr));
      }
      res.push_back(_segment(makevecteur(*jt,it->_VECTptr->front()),contextptr));
    }
    show_point(tmp,contextptr);
    return res;
  }
  const string _aretes_s("aretes");
  unary_function_eval __aretes(&giac::_aretes,_aretes_s);
  unary_function_ptr at_aretes (&__aretes,0,true);

  gen _faces(const gen & args,GIAC_CONTEXT){
    return remove_at_pnt(args);
  }
  const string _faces_s("faces");
  unary_function_eval __faces(&giac::_faces,_faces_s);
  unary_function_ptr at_faces (&__faces,0,true);

  gen symetrie(const gen & aa,const gen & bb,GIAC_CONTEXT){
    if (bb.type==_VECT)
      return apply2nd(aa,bb,contextptr,symetrie);
    gen a=remove_at_pnt(aa);
    if ( (a.type==_SYMB) && (a._SYMBptr->sommet==at_cercle) )
      setsizeerr();
    gen b=remove_at_pnt(bb);
    if (a.is_symb_of_sommet(at_hyperplan)){ // plan symmetry
      vecteur n,P;
      hyperplan_normal_point(a,n,P);
      gen n2(dotvecteur(n,n));
      if (b.type==_VECT){
	vecteur & bv=*b._VECTptr;
	if (b.subtype && b.subtype!=_POINT__VECT){ // line
	  vecteur res;
	  const_iterateur it=bv.begin(),itend=bv.end();
	  for (;it!=itend;++it){
	    if (b.subtype==_POLYEDRE__VECT){
	      if (it->type==_VECT){
		const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
		vecteur rest;
		for (;jt!=jtend;++jt){
		  rest.push_back(*jt-scalar_product(n,*jt-P,contextptr)/n2*n);
		}
		res.push_back(rest);
	      }
	    }
	    else
	      res.push_back(*it-scalar_product(n,*it-P,contextptr)/n2*n);
	  }
	  return symb_pnt(gen(res,b.subtype),default_color(contextptr),contextptr);
	}
	// point
	return symb_pnt(b-dotvecteur(n,subvecteur(bv,P))/n2*n,default_color(contextptr),contextptr);
      }
      if (b.is_symb_of_sommet(at_hyperplan)){ // hyperplan
	vecteur nb,Pb;
	hyperplan_normal_point(b,nb,Pb);
	gen nbi=nb-dotvecteur(nb,n)/n2*n;
	gen Pbi=Pb-scalar_product(n,Pb-P,contextptr)/n2*n;
	return _plan(makevecteur(nbi,Pbi),contextptr);
      }
      if (b.is_symb_of_sommet(at_hypersphere)){ // hypersphere
	gen c,r;
	centre_rayon(b,c,r,0,false);
	return _sphere(makevecteur(c-scalar_product(n,c-P,contextptr)/n2*n,r),contextptr);
      }
    }
    if (a.type!=_VECT)
      return symb_pnt(plus_two*a-b,default_color(contextptr),contextptr);
    // symetrie / droite
    vecteur v=*a._VECTptr;
    if (v.size()!=2)
      setsizeerr();
    if (v[0].type==_VECT){ // 3-d symmetry -> rotation of angle pi
      return _rotation(makevecteur(aa,cst_pi,bb),contextptr);
    }
    gen w=cst_i*(v[1]-v[0]);
    gen AM=b-v[0];
    return symb_pnt(b-plus_two*w*rdiv(scalar_product(AM,w,contextptr),w.squarenorm(contextptr)),default_color(contextptr),contextptr);
  }

  gen _symetrie(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==2))
      return symetrie(args._VECTptr->front(),args._VECTptr->back(),contextptr);
    if (args.type!=_VECT){
      return symb_program(x__IDNT_e,zero,symbolic(at_symetrie,makevecteur(args,x__IDNT_e)));
    }
    return symbolic(at_symetrie,args);
  }
  const string _symetrie_s("symetrie");
  unary_function_eval __symetrie(&giac::_symetrie,_symetrie_s);
  unary_function_ptr at_symetrie (&__symetrie);

  gen similitude3d(const vecteur & centrev,const gen & angle,const gen & rapport,const gen & b,int symrot,GIAC_CONTEXT){
    if (centrev.size()!=2 || centrev.front().type!=_VECT || centrev.back().type!=_VECT)
      setsizeerr();
    vecteur A(*centrev.front()._VECTptr),B(*centrev.back()._VECTptr);
    vecteur AB(subvecteur(B,A));
    if (AB.size()!=3)
      setdimerr();
    // Find rotation matrix from isom.cc/h
    gen M=rapport*mkisom(makevecteur(AB,angle),symrot);
    if (b.type==_VECT){
      if (b.subtype && b.subtype!=_POINT__VECT){
	vecteur & bv=*b._VECTptr;
	vecteur res;
	const_iterateur it=bv.begin(),itend=bv.end();
	for (;it!=itend;++it){
	  if (b.subtype==_POLYEDRE__VECT){
	    if (it->type==_VECT){
	      const_iterateur jt=it->_VECTptr->begin(),jtend=jt->_VECTptr->end();
	      vecteur rest;
	      for (;jt!=jtend;++jt)
		rest.push_back(A+M*(*jt-A));
	      res.push_back(rest);
	    }
	  }
	  else
	    res.push_back(A+M*(*it-A));
	}
	return symb_pnt(gen(res,b.subtype),default_color(contextptr),contextptr);
      }
      else {
	gen res=A+M*(b-A);
	return _point(res,contextptr);
      }
    }
    if (b.is_symb_of_sommet(at_hypersphere)){
      gen c,r;
      centre_rayon(b,c,r,0,false);
      c=A+M*(c-A);
      return _sphere(makevecteur(c,r),contextptr);
    }
    if (b.is_symb_of_sommet(at_hyperplan)){
      vecteur n,P;
      hyperplan_normal_point(b,n,P);
      gen Pr=A+M*(P-A);
      gen nr=M*n;
      return _plan(makevecteur(nr,Pr),contextptr);
    }
    setsizeerr();
  }

  // 2-d r(centre,angle,M) or 3-d r(line,angle,M)
  gen _rotation(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==3)){
      gen centre=remove_at_pnt(args._VECTptr->front());
      gen angle=(*args._VECTptr)[1];
      gen b=args._VECTptr->back();
      if (b.type==_VECT){
	const_iterateur it=b._VECTptr->begin(),itend=b._VECTptr->end();
	vecteur res;
	res.reserve(itend-it);
	for (;it!=itend;++it){
	  res.push_back(_rotation(makevecteur(centre,angle,*it),contextptr));
	}
	return gen(res,_GROUP__VECT);
      }
      b=remove_at_pnt(b);
      if ( centre.type==_SYMB && (centre._SYMBptr->sommet==at_cercle) )
	setsizeerr();
      if (centre.is_symb_of_sommet(at_hyperplan)){
	vecteur n,P;
	hyperplan_normal_point(centre,n,P);
	return similitude3d(makevecteur(P,P+n),angle,1,b,-1,contextptr);
      }
      if (centre.type!=_VECT) // 2-d rotation
	return symb_pnt(centre+exp(cst_i*degtorad(angle))*(b-centre),default_color(contextptr),contextptr);
      return similitude3d(*centre._VECTptr,angle,1,b,1,contextptr);
    }
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      vecteur v=*args._VECTptr;
      v.push_back(x__IDNT_e);
      return symb_program(x__IDNT_e,zero,symbolic(at_rotation,v));      
    }
    return symbolic(at_rotation,args);
  }
  const string _rotation_s("rotation");
  unary_function_eval __rotation(&giac::_rotation,_rotation_s);
  unary_function_ptr at_rotation (&__rotation);

  gen projectionpoint(const gen & aa,const gen & bb,GIAC_CONTEXT){
    gen b=remove_at_pnt(bb);
    if (aa.is_symb_of_sommet(at_hyperplan)){
      vecteur n,P;
      hyperplan_normal_point(aa,n,P);
      gen n2(dotvecteur(n,n));
      // projection of point and line on hyperplan
      if (b.type==_VECT && b._VECTptr->size()==2){ // line
	// project both points and return the line
	gen & A=b._VECTptr->front();
	gen & B=b._VECTptr->back();
	gen AP=A+scalar_product(P-A,n,contextptr)/n2;
	gen BP=B+scalar_product(P-B,n,contextptr)/n2;
	return _droite_segment(makevecteur(AP,BP),b.subtype,contextptr);
      }
      // point: b -> M=b+tn such that MP is orthogonal to n
      // hence (b-P+tn).n=0 -> t=(P-b).n/n.n
      if (b.type!=_VECT)
	setsizeerr();
      gen tmp=b+scalar_product(P-b,n,contextptr)/n2;
      return symb_pnt(do_point3d(tmp),contextptr);
    }
    if (aa.type==_VECT && aa._VECTptr->size()==2){ 
      // projection of a point b over a line AB
      gen & A=aa._VECTptr->front();
      gen & B=aa._VECTptr->back();
      gen v(B-A);
      // Find M such that M=A+t*(B-A) and (M-b).(B-A)=0
      // Hence (A-b).(B-A)+t(B-A)^2=0 -> t=(b-A).(B-A)/(B-A)^2
      gen t=scalar_product(b-A,v,contextptr)/scalar_product(v,v,contextptr);
      return symb_pnt(A+t*v,default_color(contextptr),contextptr);
    }
    vecteur v;
    v.push_back(aa);
    v.push_back(projection(aa,b,contextptr));
    return parameter2point(v,contextptr);
  }  
  
  gen _projection(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      gen a=remove_at_pnt(args._VECTptr->front());
      gen b=args._VECTptr->back();
      if (b.type==_VECT)
	return apply2nd(a,b,contextptr,projectionpoint);
      else 
	return projectionpoint(a,b,contextptr);
    }
    if (args.type!=_VECT){
      return symb_program(x__IDNT_e,zero,symbolic(at_projection,makevecteur(args,x__IDNT_e)));
    }
    return symbolic(at_projection,args);
  }
  const string _projection_s("projection");
  unary_function_eval __projection(&giac::_projection,_projection_s);
  unary_function_ptr at_projection (&__projection);

  // h(centre,rapport,M)
  gen _homothetie(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==3)){
      gen centre=remove_at_pnt(args._VECTptr->front());
      if ( (centre.type==_VECT) || ( (centre.type==_SYMB) && (centre._SYMBptr->sommet==at_cercle) ) )
	setsizeerr();
      gen rapport=(*args._VECTptr)[1];
      gen b=args._VECTptr->back();
      if (b.type==_VECT){
	const_iterateur it=b._VECTptr->begin(),itend=b._VECTptr->end();
	vecteur res;
	res.reserve(itend-it);
	for (;it!=itend;++it)
	  res.push_back(_homothetie(makevecteur(centre,rapport,*it),contextptr));
	return gen(res,_GROUP__VECT);
      }
      b=remove_at_pnt(b);
      if (b.is_symb_of_sommet(at_hyperplan)){
	vecteur n,P;
	hyperplan_normal_point(b,n,P);
	return _plan(makevecteur(n,centre+rapport*(P-centre)),contextptr);
      }
      if (b.is_symb_of_sommet(at_hypersphere)){
	gen c,r;
	centre_rayon(b,c,r,0,false);
	return _sphere(makevecteur(centre+rapport*(c-centre),r),contextptr);
      }
      return symb_pnt(centre+rapport*(b-centre),default_color(contextptr),contextptr);
    }
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      vecteur v=*args._VECTptr;
      v.push_back(x__IDNT_e);
      return symb_program(x__IDNT_e,zero,symbolic(at_homothetie,v));      
    }
    return symbolic(at_homothetie,args);
  }
  const string _homothetie_s("homothetie");
  unary_function_eval __homothetie(&giac::_homothetie,_homothetie_s);
  unary_function_ptr at_homothetie (&__homothetie);

  // True if a=coeff*b
  bool est_parallele_vecteur(const vecteur & a,const vecteur &b,gen & coeff){
    int s=a.size(),i,j;
    if (s!=b.size())
      setsizeerr();
    for (i=0;i<s;++i){
      if (!is_zero(b[i]))
	break;
    }
    if (i==s){ // b==0
      coeff=unsigned_inf;
      return true;
    }
    coeff=a[i]/b[i];
    j=i;
    for (++i;i<s;++i){
      if (!is_zero(ratnormal(a[i]*b[j]-a[j]*b[i])))
	return false;
    }
    return true;
  }
  bool est_parallele_vecteur(const vecteur & a,const vecteur &b){
    gen coeff;
    return est_parallele_vecteur(a,b,coeff);
  }
  bool est_parallele(const gen & a,const gen & b,GIAC_CONTEXT){
    if (a.type==_VECT && b.type==_VECT){
      gen coeff;
      return est_parallele_vecteur(*a._VECTptr,*b._VECTptr,coeff);
    }
    gen d(im(a*conj(b,contextptr),contextptr));
    return is_zero(d);
  }
  int est_aligne(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    return est_parallele(b-a,c-a,contextptr);
    // gen d(im((b-a)*conj(c-a,contextptr),contextptr));
    // return is_zero(d);
  }
  // renvoie 2 si tous les points sont confondus
  // renvoie 1 si tous les points sont alignes et 0 sinon 
  gen _est_aligne(const gen & args,GIAC_CONTEXT){
    gen a;
    if (args.type!=_VECT) {
      a=remove_at_pnt(args);
      if (a.type==_VECT) setsizeerr();
      return 2;
    }
    int s=args._VECTptr->size();
    vecteur v(*args._VECTptr);
    if (s==1) {
      a=remove_at_pnt(v[0]);
      if (a.type==_VECT)
	setsizeerr();
      return 2;
    }
    if (s>=2){
      a=remove_at_pnt(v[0]);
      gen b=remove_at_pnt(v[1]);
      if ((a.type==_VECT) || (b.type==_VECT))
	setsizeerr(); 
      bool res;
      int i=2;
      while (a==b && i<s){
	b=remove_at_pnt(v[i]);
	if (b.type==_VECT) setsizeerr(); 
	i=i+1;
      }
      //a==b tous les points sonts confondus sinon on a a!=b
      if (i==s){
	if (a==b) return(2);
	return(1);
      }
      //a!=b
      for (int j=i;j<s;j++){
	gen c=remove_at_pnt(v[j]);
	if (c.type==_VECT) 
	  setsizeerr();
	res=est_aligne(a,b,c,contextptr);
	if (res==0) return 0; 
      }
      return res;
    }
    return symbolic(at_est_aligne,args);
  }
  const string _est_aligne_s("est_aligne");
  unary_function_eval __est_aligne(&giac::_est_aligne,_est_aligne_s);
  unary_function_ptr at_est_aligne (&__est_aligne);

  bool est_coplanaire(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    if (a.type!=_VECT)
      setsizeerr("3-d instruction");    
    gen n1(b-a),n2(c-a),n3(c-a);
    return is_zero(mdet(makevecteur(n1,n2,n3)));
  }
  gen _est_coplanaire(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=4)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    return est_coplanaire(v[0],v[1],v[2],v[3],contextptr);
  }
  const string _est_coplanaire_s("est_coplanaire");
  unary_function_eval __est_coplanaire(&giac::_est_coplanaire,_est_coplanaire_s);
  unary_function_ptr at_est_coplanaire (&__est_coplanaire,0,true);

  bool est_cocyclique(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    if (a.type==_VECT)
      set3derr();
    gen e(im((b-a)*(c-d)*conj(c-a,contextptr)*conj(b-d,contextptr),contextptr));
    return is_zero(normal(e));
  }
  // renvoie 3 si tous les points sont confondus
  // renvoie 2 si on a 2 points
  // renvoie 1 si tous les points sont cocycliques et 0 sinon
  gen _est_cocyclique(const gen & args,GIAC_CONTEXT){
    gen a;
    if (args.type!=_VECT){
      a=remove_at_pnt(args);
      if (a.type==_VECT) setsizeerr();
      return 3;
    }
    int s=args._VECTptr->size();
    vecteur v(*args._VECTptr);
    if (s==1) {
      a=remove_at_pnt(v[0]);
      if (a.type==_VECT)
	setsizeerr();
      return 3;
    }
    if (s>=2){
      vecteur v(*args._VECTptr);
      a=remove_at_pnt(v[0]);
      gen b=remove_at_pnt(v[1]);
      if ((a.type==_VECT) || (b.type==_VECT))
	setsizeerr();
      int i=2; 
      while (a==b && i<s){
	b=remove_at_pnt(v[i]);
	if (b.type==_VECT) setsizeerr(); 
	i=i+1;
      }
      //a==b tous les points sonts confondus valeur= 3 sinon on a a!=b
      if (i==s){
	if (a==b) return 3;
	return 2;
      }
      //a!=b
      gen c=remove_at_pnt(v[i]);
      if (c.type==_VECT)
	setsizeerr();
      i=i+1; 
      while ((a==c ||c==b) && i<s){
	c=remove_at_pnt(v[i]);
	if (c.type==_VECT) setsizeerr(); 
	i=i+1;
      }
      //a==c ou b==c on a 2 points sinon on a 3 points distincts
      if (i==s){
	if (a==b || b==c) return 2;
	if (est_aligne(a,b,c,contextptr)) 
	  return 0; 
	else 
	  return 1;
      }	 
      
      gen res;
      for (int j=i;i<s;i++){
	gen d=remove_at_pnt(v[j]);
	if (d.type==_VECT) 
	  setsizeerr();
	res=est_cocyclique(a,b,c,d,contextptr);
	if (res==0) return 0; 
      }
      return res;
    }
    return symbolic(at_est_cocyclique,args);
  }
  const string _est_cocyclique_s("est_cocyclique");
  unary_function_eval __est_cocyclique(&giac::_est_cocyclique,_est_cocyclique_s);
  unary_function_ptr at_est_cocyclique (&__est_cocyclique);

  gen _est_parallele(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      vecteur v(*args._VECTptr);
      gen a=remove_at_pnt(v[0]),b=remove_at_pnt(v[1]);
      bool av=a.type==_VECT && a._VECTptr->size()==2;
      bool bv=b.type==_VECT && b._VECTptr->size()==2;
      if (a.is_symb_of_sommet(at_hyperplan)){
	vecteur n(hyperplan_normal(a));
	if (b.is_symb_of_sommet(at_hyperplan))
	  return est_parallele_vecteur(n,hyperplan_normal(b));
	if (bv)
	  return is_zero(scalar_product(n,b[0]-b[1],contextptr));
      }
      if (b.is_symb_of_sommet(at_hyperplan) && av)
	return is_zero(scalar_product(hyperplan_normal(b),a[0]-a[1],contextptr));
      if ( !av || !bv)
	setsizeerr();
      return est_parallele(a[0]-a[1],b[0]-b[1],contextptr);
    }
    return symbolic(at_est_parallele,args);
  }
  const string _est_parallele_s("est_parallele");
  unary_function_eval __est_parallele(&giac::_est_parallele,_est_parallele_s);
  unary_function_ptr at_est_parallele (&__est_parallele);

  bool est_perpendiculaire(const gen & a,const gen & b,GIAC_CONTEXT){
    gen d;
    if (a.type==_VECT && b.type==_VECT)
      d=dotvecteur(*a._VECTptr,*b._VECTptr);
    else
      d=re(a*conj(b,contextptr),contextptr);
    return is_zero(d);
  }

  // check if a belongs to b, a must be a complex, b a line or circle or curve
  bool est_element(const gen & a_orig,const gen & b_orig,GIAC_CONTEXT){
    gen a=remove_at_pnt(a_orig),b=remove_at_pnt(b_orig);
    if (b.is_symb_of_sommet(at_cercle)){
      // check orthogonality with diameter
      gen diam=remove_at_pnt(b._SYMBptr->feuille._VECTptr->front());
      if (diam.type!=_VECT)
	return false;
      gen c1=remove_at_pnt(diam._VECTptr->front());
      gen c2=remove_at_pnt(diam._VECTptr->back());
      return est_perpendiculaire(a-c1,a-c2,contextptr);
    }
    if (b.is_symb_of_sommet(at_hypersphere)){
      gen c,r;
      centre_rayon(b,c,r,0,false);
      return is_zero(ratnormal(pow(a-c,2)-pow(r,2)));
    }
    if (b.is_symb_of_sommet(at_hyperplan)){
      vecteur n,P;
      hyperplan_normal_point(b,n,P);
      return is_zero(scalar_product(a-P,n,contextptr));
    }
    if (b.is_symb_of_sommet(at_curve)){
      gen t;
      return on(b,a,t,contextptr);
    }
    if (b.type==_VECT && b._VECTptr->size()==2){
      gen c1=remove_at_pnt(b._VECTptr->front());
      gen c2=remove_at_pnt(b._VECTptr->back());      
      return est_aligne(c1,c2,a,contextptr);
    }
    return false; // FIXME implement est_element of a polygon 
  }
  gen _est_element(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      vecteur v(*args._VECTptr);
      gen a=v[0],b=v[1];
      return est_element(a,b,contextptr);
    }
    return symbolic(at_est_element,args);
  }
  const string _est_element_s("est_element");
  unary_function_eval __est_element(&giac::_est_element,_est_element_s);
  unary_function_ptr at_est_element (&__est_element);

  gen inversion(const gen & centre,const gen & rapport,const gen & arg,GIAC_CONTEXT){
    gen b=remove_at_pnt(arg);
    // special care must be taken for circles and segments
    if (b.type==_VECT){
      if (b._VECTptr->size()!=2)
	setsizeerr("not implemented");
      // inversion of a line
      gen c=b._VECTptr->front(),d=b._VECTptr->back();
      if (est_aligne(centre,c,d,contextptr))
	return b;
      // find the image of the projection of the center of inversion
      // the image of the line is the circle of diameter centre,image
      gen t=projection(c,d,centre,contextptr);
      gen p=d+t*(c-d);
      return remove_at_pnt(_cercle(_droite(makevecteur(centre,inversion(centre,rapport,p,contextptr)),contextptr),contextptr));
    }
    if ( (b.type!=_SYMB) || (!equalposcomp(plot_sommets,b._SYMBptr->sommet)) )
      return centre+rapport*inv(conj(b-centre,contextptr));
    if (b._SYMBptr->sommet!=at_cercle)
      setsizeerr("not implemented");
    // inversion of a circle
    // if it contains the center of inversion it's a line, otherw. a circle
    gen c,r;
    centre_rayon(b,c,r,true,contextptr);
    if (is_zero(evalf_double(distance2pp(c,centre,contextptr)-r*r))){
      // c-centre is the normal direction
      gen n(c-centre);
      n=im(n,contextptr)-cst_i*re(n,contextptr);
      // the line passes at the image of 2*centre-c
      gen pied(inversion(centre,rapport,plus_two*c-centre,contextptr));
      return remove_at_pnt(_droite(makevecteur(pied,pied+n),contextptr));
    }
    else {
      gen d(c-centre);
      r=rdiv(r,abs_norm(d,contextptr))*d;
      gen d1(inversion(centre,rapport,c+r,contextptr));
      gen d2(inversion(centre,rapport,c-r,contextptr));
      return remove_at_pnt(_cercle(_droite(makevecteur(d1,d2),contextptr),contextptr));
    }
  }

  gen _inversion(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==3)){
      gen centre=remove_at_pnt(args._VECTptr->front());
      gen rapport=(*args._VECTptr)[1];
      gen b=args._VECTptr->back();
      if ( centre.type==_VECT || (centre.type==_SYMB && centre._SYMBptr->sommet==at_cercle) )
	setsizeerr();
      if (b.type==_VECT){
	const_iterateur it=b._VECTptr->begin(),itend=b._VECTptr->end();
	vecteur res;
	res.reserve(itend-it);
	for (;it!=itend;++it){
	  res.push_back(inversion(centre,rapport,*it,contextptr));
	}
	return gen(res,_GROUP__VECT);
      }
      return symb_pnt(inversion(centre,rapport,b,contextptr),default_color(contextptr),contextptr);
    }
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      vecteur v=*args._VECTptr;
      v.push_back(x__IDNT_e);
      return symb_program(x__IDNT_e,zero,symbolic(at_inversion,v));      
    }
    return symbolic(at_inversion,args);
  }
  const string _inversion_s("inversion");
  unary_function_eval __inversion(&giac::_inversion,_inversion_s);
  unary_function_ptr at_inversion (&__inversion);

  // s(centre,rapport,angle,M)
  gen _similitude(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==4)){
      gen centre=remove_at_pnt(args._VECTptr->front());
      if ( (centre.type==_VECT) || ( (centre.type==_SYMB) && (centre._SYMBptr->sommet==at_cercle) ) )
	setsizeerr();
      gen rapport=(*args._VECTptr)[1];
      gen angle=(*args._VECTptr)[2];
      gen b=args._VECTptr->back();
      if (b.type==_VECT){
	const_iterateur it=b._VECTptr->begin(),itend=b._VECTptr->end();
	vecteur res;
	res.reserve(itend-it);
	for (;it!=itend;++it){
	  res.push_back(_similitude(makevecteur(centre,rapport,angle,b),contextptr));
	}
	return gen(res,_GROUP__VECT);
      }
      if (centre.is_symb_of_sommet(at_hyperplan)){
	vecteur n,P;
	hyperplan_normal_point(centre,n,P);
	return similitude3d(makevecteur(P,P+n),angle,rapport,b,-1,contextptr);
      }
      b=remove_at_pnt(b);
      if (centre.type!=_VECT)
	return symb_pnt(centre+rapport*exp(cst_i*degtorad(angle))*(b-centre),default_color(contextptr),contextptr);
      return similitude3d(*centre._VECTptr,angle,rapport,b,1,contextptr);
    }
    if ((args.type==_VECT) && (args._VECTptr->size()==3)){
      vecteur v=*args._VECTptr;
      v.push_back(x__IDNT_e);
      return symb_program(x__IDNT_e,zero,symbolic(at_similitude,v));      
    }
    return symbolic(at_similitude,args);
  }
  const string _similitude_s("similitude");
  unary_function_eval __similitude(&giac::_similitude,_similitude_s);
  unary_function_ptr at_similitude (&__similitude);

  gen translation(const gen & a,const gen & bb,GIAC_CONTEXT){
    gen b=remove_at_pnt(bb);
    if (b.is_symb_of_sommet(at_hyperplan)){
      vecteur n,P;
      hyperplan_normal_point(b,n,P);
      return _plan(makevecteur(n,translation(a,P,contextptr)),contextptr);
    }
    if (b.is_symb_of_sommet(at_hypersphere)){
      gen c,r;
      centre_rayon(b,c,r,0,false);
      return _sphere(makevecteur(translation(a,c,contextptr),r),contextptr);
    }
    if (a.type!=_VECT)
      return symb_pnt(a+b,default_color(contextptr),contextptr);
    // translation / segment
    vecteur v=*a._VECTptr;
    if (v.size()!=2)
      setsizeerr();
    gen w=v[1]-v[0];
    return symb_pnt(b+w,default_color(contextptr),contextptr);
  }
  gen _translation(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==2)){
      gen a=remove_at_pnt(args._VECTptr->front());
      if ( (a.type==_SYMB) && (a._SYMBptr->sommet==at_cercle) )
	setsizeerr();
      gen b=args._VECTptr->back();
      return apply2nd(a,b,contextptr,translation);
    }
    if (args.type!=_VECT){
      return symb_program(x__IDNT_e,zero,symbolic(at_translation,makevecteur(args,x__IDNT_e)));
    }
    return symbolic(at_translation,args);
  }
  const string _translation_s("translation");
  unary_function_eval __translation(&giac::_translation,_translation_s);
  unary_function_ptr at_translation (&__translation);

  // curve is the innert form of a parametric curve
  // source=[pnt,var], plot=ligne brisee
  string printascurve(const gen & feuille,const string & sommetstr){
    if ( !fastcurveprint || (feuille.type!=_VECT) || (feuille._VECTptr->size()!=2) )
      return sommetstr+'('+feuille.print()+')';
    return sommetstr+'('+feuille._VECTptr->front().print()+",undef)";
  }
  symbolic symb_curve(const gen & source,const gen & plot){
    return symbolic(at_curve,gen(makevecteur(source,plot),_GROUP__VECT));
  }
  gen _curve(const gen & args){
    return symbolic(at_curve,args);
  }
  const string _curve_s("curve");
  unary_function_unary __curve(&giac::_curve,_curve_s,&printascurve);
  unary_function_ptr at_curve (&__curve);

  gen hypersurface(const gen & args,const gen & equation,const gen & vars){
    return _hypersurface(gen(makevecteur(args,equation,vars),_SEQ__VECT));
  }
  gen _hypersurface(const gen & args){
    return symbolic(at_hypersurface,args);
  }
  const string _hypersurface_s("hypersurface");
  unary_function_unary __hypersurface(&giac::_hypersurface,_hypersurface_s);
  unary_function_ptr at_hypersurface (&__hypersurface,0,true);

  gen plotparam(const gen & f,const gen & vars,const gen & attribut,bool clrplot,double function_xmin,double function_xmax,double function_ymin,double function_ymax,double function_tmin, double function_tmax,double function_tstep,const context * contextptr){
    bool save_approx_mode=approx_mode(contextptr);
    approx_mode(true,contextptr);
    gen locvar(vars);
    locvar.subtype=0;
    gen xy=quotesubst(f,vars,locvar),xy_,x_,y_;
    bool joindre;
    vecteur localvar(1,vars),res;
    context * newcontextptr=(context *) contextptr;
    int protect=bind(vecteur(1,function_tmin),localvar,newcontextptr);
    vecteur chemin;
    double i,j,oldi,oldj,entrei,entrej;
    for (double t=function_tmin;t<function_tmax;t+= function_tstep,vars._IDNTptr->localvalue->back()._DOUBLE_val =t){
      xy_=xy.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
      x_=re(xy_,newcontextptr);
      y_=im(xy_,newcontextptr);
      if ( (x_.type!=_DOUBLE_) || (y_.type!=_DOUBLE_) )
	continue;
      i=x_._DOUBLE_val;
      j=y_._DOUBLE_val;
      if (t!=function_tmin){
	if ( (fabs(oldj-j)>(function_ymax-function_ymin)/5) ||
	     (fabs(oldi-i)>(function_xmax-function_xmin)/5) ){
	  vars._IDNTptr->localvalue->back()._DOUBLE_val -= function_tstep/2;
	  xy_=xy.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
	  x_=re(xy_,newcontextptr);
	  y_=im(xy_,newcontextptr);
	  if ( (x_.type!=_DOUBLE_) || (y_.type!=_DOUBLE_) )
	    joindre=false;
	  else {
	    entrei=x_._DOUBLE_val;
	    entrej=y_._DOUBLE_val;
	    if (j>oldj)
	      joindre=(j>=entrej) && (entrej>=oldj);
	    else
	      joindre=(j<=entrej) && (entrej<=oldj);
	    if (i>oldi)
	      joindre=joindre && (i>=entrei) && (entrei>=oldi);
	    else
	      joindre=joindre && (i<=entrei) && (entrei<=oldi);
	  }
	  vars._IDNTptr->localvalue->back()._DOUBLE_val -= function_tstep/2;
	}
	else
	  joindre=true;
      } // if t!=function_tmin
      else
	joindre=true;
      if (joindre)
	chemin.push_back(gen(i,j));
      else {
	if (!chemin.empty())
	  res.push_back(symb_pnt(symb_curve(gen(makevecteur(f,vars,function_tmin,t),_PNT__VECT),gen(chemin,_GROUP__VECT)),attribut,contextptr));
	function_tmin=t;
	chemin=vecteur(1,gen(i,j));
      }
      oldi=i;
      oldj=j;
    }
    if (!chemin.empty())
      res.push_back(symb_pnt(symb_curve(gen(makevecteur(f,vars,function_tmin,function_tmax),_PNT__VECT),gen(chemin,_GROUP__VECT)),attribut,contextptr));
    leave(protect,localvar,newcontextptr);
#ifndef WIN32
    if (child_id)
      plot_instructions.push_back(res);
#endif // WIN32
    approx_mode(save_approx_mode,contextptr);
    gen e(res,_SEQ__VECT);
    return e;
  }

  gen plotparam3d(const gen & f,const vecteur & vars,double function_xmin,double function_xmax, double function_ymin, double function_ymax,double function_zmin,double function_zmax,double function_umin,double function_umax,double function_vmin,double function_vmax,bool clrplot,bool f_autoscale,int color,const gen & eq,const vecteur & eqvars,GIAC_CONTEXT){
    identificateur u("u"),v("v");
    vecteur vsub(1,u);
    vsub.push_back(v);
    gen ff=subst(f,vars,vsub);
#ifdef WITH_GNUPLOT
    int out_handle;
    FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
#ifdef IPAQ
    fprintf(stream,"set samples 10\n");
    //    fprintf(stream,"show samples\n");
#endif
    gen r;
    if (is_zero(derive(ff,v)))
      r=symb_pnt(symb_curve(gen(makevecteur(f,vars[0],function_xmin,function_xmax),_PNT__VECT),undef),color,contextptr);
    else
      r=symb_pnt(hypersurface(gen(makevecteur(f,vars,makevecteur(function_umin,function_vmin),makevecteur(function_umax,function_vmax)),_PNT__VECT),eq,eqvars),color,contextptr);
    r.subtype=gnuplot_fileno;
    reset_gnuplot_hidden3d(stream);
    if (debug_infolevel)
      printf("set urange [%g:%g]\n",function_umin,function_umax);
    fprintf(stream,"set urange [%g:%g]\n",function_umin,function_umax);
    if (debug_infolevel)
      printf("set vrange [%g:%g]\n",function_vmin,function_vmax);
    fprintf(stream,"set vrange [%g:%g]\n",function_vmin,function_vmax);
    if (!f_autoscale){
      if (debug_infolevel)
	printf("set xrange [%g:%g]\n",function_xmin,function_xmax);
      fprintf(stream,"set xrange [%g:%g]\n",function_xmin,function_xmax);
      if (debug_infolevel)
	printf("set yrange [%g:%g]\n",function_ymin,function_ymax);
      fprintf(stream,"set yrange [%g:%g]\n",function_ymin,function_ymax);
      if (debug_infolevel)
	printf("set zrange [%g:%g]\n",function_zmin,function_zmax);
      fprintf(stream,"set zrange [%g:%g]\n",function_zmin,function_zmax);
    }
    if (clrplot || gnuplot_do_splot){
      if (debug_infolevel)
	printf("%s","splot ");	
      fprintf(stream,"%s","splot ");
    }
    else {
      if (debug_infolevel)
	printf("%s","replot ");
      fprintf(stream,"%s","replot ");
    }
    gnuplot_do_splot=false;
    if (ff.type!=_VECT){
      if (debug_infolevel)
	printf("%s notitle\n",gnuplot_traduit(ff).c_str());
      fprintf(stream,"%s notitle\n",gnuplot_traduit(ff).c_str());
    }
    else {
      string tmp(gnuplot_traduit(ff));
      // cout << tmp.substr(1,tmp.size()-2) << endl;
      if (debug_infolevel)
	printf("%s notitle\n",tmp.substr(1,tmp.size()-2).c_str());
      fprintf(stream,"%s notitle\n",tmp.substr(1,tmp.size()-2).c_str());
    }
    win9x_gnuplot(stream);
    gnuplot_wait(out_handle,gnuplot_out_readstream,gnuplot_wait_times);
    ++gnuplot_fileno;
    // return gnuplot_fileno-1;
    return r;
#endif // GNUPLOT    
    return symbolic(at_plotparam,makevecteur(f,vars));
  }
  gen paramplotparam(const gen & args,bool clrplot,const context * contextptr){
    // args= [x(t)+i*y(t),t] should add a t interval
    bool f_autoscale=autoscale;
    if (args.type!=_VECT)
      return paramplotparam(gen(makevecteur(args,t__IDNT_e),_SEQ__VECT),clrplot,contextptr);
    vecteur vargs(plotpreprocess(args,contextptr));
    if (vargs.size()<2)
      return symbolic(at_plotparam,args);
    gen f=vargs.front();
    gen vars=vargs[1];
    if (f.type==_VECT && f._VECTptr->size()==2)
      f=f._VECTptr->front()+cst_i*f._VECTptr->back();
    if (f.type!=_VECT){
      if (vars.type==_VECT){
	if (vars._VECTptr->size()!=1)
	  setsizeerr();
	vars=vars._VECTptr->front();
      }
    }
    int s=vargs.size();
    double umin,vmin,umax,vmax,tmin,tmax,xmin=gnuplot_xmin,xmax=gnuplot_xmax,ymin=gnuplot_ymin,ymax=gnuplot_ymax,zmin=gnuplot_zmin,zmax=gnuplot_zmax;
    for (int i=2;i<s;++i){
      if (readvar(vargs[i])==x__IDNT_e){
	f_autoscale=false;
	readrange(vargs[i],gnuplot_xmin,gnuplot_xmax,vargs[i],xmin,xmax);
      }
      if (readvar(vargs[i])==y__IDNT_e){
	f_autoscale=false;
	readrange(vargs[i],gnuplot_ymin,gnuplot_ymax,vargs[i],ymin,ymax);
      }
      if (readvar(vargs[i])==z__IDNT_e){
	f_autoscale=false;
	readrange(vargs[i],gnuplot_zmin,gnuplot_zmax,vargs[i],zmin,zmax); 
      }
    }
    if (f._VECTptr->size()==3 && vars.type!=_VECT){ // 3-d curve
      if (s==4)
	vars=symbolic(at_equal,makevecteur(vars,symbolic(at_interval,makevecteur(vargs[2],vargs[3]))));
      vars=makevecteur(vars,v__IDNT_e);
    }
    if (vars.type==_VECT && vars._VECTptr->size()>=2){
      vecteur v=*vars._VECTptr;
      if (v.size()!=2)
	setsizeerr();
      if (readrange(v[0],gnuplot_tmin,gnuplot_tmax,v[0],umin,umax) && readrange(v[1],gnuplot_tmin,gnuplot_tmax,v[1],vmin,vmax))
	return plotparam3d(f,makevecteur(v[0],v[1]),xmin,xmax,ymin,ymax,zmin,zmax,umin,umax,vmin,vmax,clrplot,f_autoscale,default_color(contextptr),undef,vecteur(0),contextptr);
      setsizeerr();
    }
    if (!readrange(vars,gnuplot_tmin,gnuplot_tmax,vars,tmin,tmax))
      setsizeerr("2nd arg must be a free variable");
    if (vargs.size()==3){
      gen trange=vargs[2].evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
      if ( (trange.type==_SYMB) && (trange._SYMBptr->sommet==at_interval)){
	gen tmin=trange._SYMBptr->feuille._VECTptr->front();
	gen tmax=trange._SYMBptr->feuille._VECTptr->back();
	if ( (tmin.type==_DOUBLE_) && (tmax.type==_DOUBLE_) )
	  return plotparam(f,vars,default_color(contextptr),clrplot,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,tmin._DOUBLE_val,tmax._DOUBLE_val,gnuplot_tstep,contextptr);
      }
    }
    if (vargs.size()==4){
      gen tmin=vargs[2].evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
      gen tmax=vargs[3].evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
      if ( (tmin.type==_DOUBLE_) && (tmax.type==_DOUBLE_) )
	return plotparam(f,vars,default_color(contextptr),clrplot,xmin,xmax,ymin,ymax,tmin._DOUBLE_val,tmax._DOUBLE_val,gnuplot_tstep,contextptr);
    }
    return plotparam(f,vars,default_color(contextptr),clrplot,xmin,xmax,ymin,ymax,tmin,tmax,gnuplot_tstep,contextptr);
  }
  gen _plotparam(const gen & args,const context * contextptr){
    return paramplotparam(args,true,contextptr);
  }
  const string _plotparam_s("plotparam");
  unary_function_eval __plotparam(&giac::_plotparam,_plotparam_s);
  unary_function_ptr at_plotparam (&__plotparam,_QUOTE_ARGUMENTS);

  gen _paramplot(const gen & args,const context * contextptr){
    return paramplotparam(args,false,contextptr);
  }
  const string _paramplot_s("paramplot");
  unary_function_eval __paramplot(&giac::_paramplot,_paramplot_s);
  unary_function_ptr at_paramplot (&__paramplot,_QUOTE_ARGUMENTS);

  gen plotpoints(const vecteur & v,const gen & attribut,GIAC_CONTEXT){
    vecteur w(v);
    iterateur it=w.begin(),itend=w.end();
    for (;it!=itend;++it){
      if (it->type!=_VECT || it->_VECTptr->size()!=2)
	setsizeerr();
      *it=it->_VECTptr->front()+cst_i*it->_VECTptr->back();
    }
    return symb_pnt(gen(w,_GROUP__VECT),attribut.val,contextptr); 
    // should change symb_pnt so that attribut is more generic than color
  }
  gen _plot(const gen & g,const context * contextptr){
    if (g.type!=_VECT)
      return _plotfunc(g,contextptr);
    vecteur & v=*g._VECTptr;
    int s=v.size();
    if (g.subtype!=_SEQ__VECT && s==3 ){
      if (v[2].type==_IDNT)
	return plotparam(v[0]+cst_i*v[1],v[2],default_color(contextptr),true,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_tmin,gnuplot_tmax,gnuplot_tstep,contextptr);
      if (v[2].is_symb_of_sommet(at_equal)){ // parametric plot
	gen & gw=v[2]._SYMBptr->feuille;
	if (gw.type==_VECT && gw._VECTptr->size()==2){
	  gen gx=gw._VECTptr->front();
	  double inf,sup;
	  if (gx.type==_IDNT && chk_double_interval(gw._VECTptr->back(),inf,sup))
	    return plotparam(v[0]+cst_i*v[1],gx,default_color(contextptr),true,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,inf,sup,gnuplot_tstep,contextptr);
	}
      }
    }
    if (s<1)
      return _plotfunc(g,contextptr);
    if (g.subtype!=_SEQ__VECT)
      return plotpoints(v,0,contextptr);
    double xmin=gnuplot_xmin,xmax=gnuplot_xmax,ymin=gnuplot_ymin,ymax=gnuplot_ymax,zmin=gnuplot_zmin,zmax=gnuplot_zmax;
    gen xvar=vx_var,yvar=y__IDNT_e;
    int pixels=gnuplot_pixels_per_eval;
    gen attribut=default_color(contextptr);
    bool clrplot=true;
    // parse options
    for (int i=1;i<s;++i){
      if (v[i].type==_IDNT){
	if (i==1)
	  xvar=v[1];
	if (i==2)
	  yvar=v[2];
      }
      if (i==1 && v[i].is_symb_of_sommet(at_interval)){
	identificateur tmp(" x");
	vecteur w(v);
	w[0]=v[0](tmp,contextptr);
	w[1]=symbolic(at_equal,makevecteur(tmp,v[1]));
	return _plot(gen(w,_SEQ__VECT),contextptr);
      }
      if (!v[i].is_symb_of_sommet(at_equal))
	continue;
      gen & opt=v[i]._SYMBptr->feuille;
      if (opt.type!=_VECT || opt._VECTptr->size()!=2)
	continue;
      gen opt1=opt._VECTptr->front(),opt2=opt._VECTptr->back();
      double inf,sup;
      if ( opt1.type==_IDNT &&chk_double_interval(opt2,inf,sup)){	
	if (i==1){
	  xvar=opt1;
	  xmin=inf;
	  xmax=sup;
	}
	if (i==2){
	  yvar=opt1;
	  ymin=inf;
	  ymax=sup;
	}
      }
      if ( opt1.type!=_INT_)
	continue;
      switch (opt1.val){
      case _COLOR:
	attribut=opt2;
      }
    }
    if (v[0].type==_VECT && !v[0]._VECTptr->empty() && v[0]._VECTptr->front().type==_VECT)
      return plotpoints(*v[0]._VECTptr,attribut,contextptr);
    else
      return plotfunc(v[0],xvar,attribut,clrplot,xmin,xmax,ymin,ymax,zmin,zmax,pixels,contextptr);
  }
  const string _plot_s("plot"); // FIXME use maple arguments
  unary_function_eval __plot(&giac::_plot,_plot_s);
  unary_function_ptr at_plot (&__plot);

  gen _plotpolar(const gen & args,const context * contextptr){
    // args= [rho(theta),theta] should add a theta interval
    vecteur vargs(plotpreprocess(args,contextptr));
    gen rho=vargs.front();
    gen theta=vargs[1];
    if (theta.type!=_IDNT)
      setsizeerr("2nd arg must be a free variable");
    if (vargs.size()==3)
      return _plotparam(gen(makevecteur(rho*exp(cst_i*degtorad(theta)),theta,vargs.back()),_SEQ__VECT),contextptr);
    if (vargs.size()==4)
      return _plotparam(gen(makevecteur(rho*exp(cst_i*degtorad(theta)),theta,vargs[2],vargs.back()),_SEQ__VECT),contextptr);
    return _plotparam(gen(makevecteur(rho*exp(cst_i*degtorad(theta)),theta),_SEQ__VECT),contextptr);
  }
  const string _plotpolar_s("plotpolar");
  unary_function_eval __plotpolar(&giac::_plotpolar,_plotpolar_s);
  unary_function_ptr at_plotpolar (&__plotpolar,_QUOTE_ARGUMENTS);

  const string _polarplot_s("polarplot");
  unary_function_eval __polarplot(&giac::_plotpolar,_polarplot_s);
  unary_function_ptr at_polarplot (&__polarplot,_QUOTE_ARGUMENTS);

  void ck_parameter_x(){
    if (x__IDNT_e.eval()!=x__IDNT_e)
      setsizeerr("Variable x should be purged");
  }

  void ck_parameter_y(){
    if (y__IDNT_e.eval()!=y__IDNT_e)
      setsizeerr("Variable y should be purged");
  }

  void ck_parameter_z(){
    if (z__IDNT_e.eval()!=z__IDNT_e)
      setsizeerr("Variable z should be purged");
  }

  void ck_parameter(const gen & g){
    if ( (g.type==_IDNT) && (g.eval()!=g) )
      setsizeerr("Variable "+g.print()+" should be purged");
  }

  void ck_parameter_t(){
    if (t__IDNT_e.eval()!=t__IDNT_e)
      setsizeerr("Variable t should be purged");
  }

  void ck_parameter_u(){
    if (u__IDNT_e.eval()!=u__IDNT_e)
      setsizeerr("Variable u should be purged");
  }

  void ck_parameter_v(){
    if (v__IDNT_e.eval()!=v__IDNT_e)
      setsizeerr("Variable v should be purged");
  }

  void normal3d(const gen & nn,vecteur & v1,vecteur & v2){
    if (nn.type!=_VECT || nn._VECTptr->size()!=3)
      setdimerr();
    vecteur & n = *nn._VECTptr;
    if (is_zero(n[0]))
      v1=makevecteur(1,0,0);
    else
      v1=makevecteur(n[1],-n[0],0);
    v2=cross(n,v1);
  }

  // Parametric equation
  gen _parameq(const gen & args){
    vecteur v;
    int s=1;
    if (args.type!=_VECT){
      v=vecteur(1,args);
      v.push_back(t__IDNT_e);
    }
    else {
      s=args._VECTptr->size();
      if (args.subtype==_SEQ__VECT)
	return _parameq(args._VECTptr->front());
      if (s<2) 
	setsizeerr();
      v=*args._VECTptr;
    }
    // e= symb_curve or line or cercle
    gen e=remove_at_pnt(v.front());
    gen t=v[1],u;
    if (e.is_symb_of_sommet(at_hyperplan)){
      if (s<2){
	ck_parameter_u();
	t=u__IDNT_e;
      }
      if (s<3){
	ck_parameter_v();
	u=v__IDNT_e;
      }
      else
	u=v[2];
      vecteur P,n;
      hyperplan_normal_point(e,n,P);
      vecteur v1,v2;
      normal3d(n,v1,v2);
      gen par_eq=t*v1+u*v2+P;
      return par_eq;
    }
    else {
      if (s==1)
	ck_parameter_t();
    }
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_cercle))
      e=cercle2curve(e);
    if (e.type==_VECT)
      e=line2curve(e);
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_curve)){
      vecteur v=*e._SYMBptr->feuille._VECTptr->front()._VECTptr;
      return subst(v[0],v[1],t);
    }
    else
      return e;
  }
  const string _parameq_s("parameq");
  unary_function_unary __parameq(&giac::_parameq,_parameq_s);
  unary_function_ptr at_parameq (&__parameq);

  gen hypersphere_equation(const gen & g,const vecteur & xyz){
    gen centre,rayon;
    if (!centre_rayon(g,centre,rayon,false,0) ||centre.type!=_VECT)
      setsizeerr();
    vecteur & v=*centre._VECTptr;
    if (v.size()!=3)
      setdimerr();
    vecteur xyzc(subvecteur(xyz,v));
    gen eq=ratnormal(dotvecteur(xyzc,xyzc)-pow(rayon,2));
    return eq;
  }

  gen hypersurface_equation(const gen & g,const vecteur & xyz){
    if (!g.is_symb_of_sommet(at_hypersurface))
      setsizeerr();
    gen & f=g._SYMBptr->feuille;
    if (f.type!=_VECT) setsizeerr();
    vecteur & fv=*f._VECTptr;
    if (fv.size()==3 && fv[1].type!=_VECT && fv[2].type==_VECT){
      return subst(fv[1],*fv[2]._VECTptr,xyz);
    }
    setsizeerr("Hypersurface w/o equation");
  }

  gen _equation(const gen & args,GIAC_CONTEXT){
    vecteur v;
    if (args.type!=_VECT){
      ck_parameter_x();
      ck_parameter_y();
      ck_parameter_z();
      v=vecteur(1,args);
      v.push_back(makevecteur(x__IDNT_e,y__IDNT_e,z__IDNT_e));
    }
    else {
      if (args._VECTptr->size()!=2) 
	return symbolic(at_equation,args);
      else
	v=*args._VECTptr;
    }
    // e= symb_curve or line or cercle
    gen e=remove_at_pnt(v.front());
    gen xy=v.back();
    if ( (xy.type!=_VECT) || (xy._VECTptr->size()<2))
      setsizeerr();
    vecteur & vxyz=*xy._VECTptr;
    gen x(vxyz[0]),y(vxyz[1]),z;
    if (vxyz.size()==3)
      z=vxyz[2];
    if (e.is_symb_of_sommet(at_hyperplan)){
      gen & f=e._SYMBptr->feuille;
      if (ckmatrix(f) && f._VECTptr->size()==2){
	return symbolic(at_equal,makevecteur(dotvecteur(*f._VECTptr->front()._VECTptr,subvecteur(vxyz,*f._VECTptr->back()._VECTptr)),zero));
      }
    }
    if (e.is_symb_of_sommet(at_hypersphere)){
      gen f=hypersphere_equation(e,vxyz);
      return symbolic(at_equal,makevecteur(f,zero));
    }
    if (e.is_symb_of_sommet(at_hypersurface)){
      gen f=hypersurface_equation(e,vxyz);
      return symbolic(at_equal,makevecteur(f,zero));
    }
    if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_cercle)){
      gen centre,rayon;
      centre_rayon(e,centre,rayon,false,contextptr);
      rayon=normal(rayon);
      return symbolic(at_equal,makevecteur(pow(x-re(centre,contextptr),2)+pow(y-im(centre,contextptr),2),rayon*conj(rayon,contextptr)));
    }
    if ( (e.type==_VECT) && (e._VECTptr->size()==2) ){
      gen A=e._VECTptr->front();
      gen B=e._VECTptr->back();
      gen v=B-A;
      if (v.type==_VECT){ // 3-d line?
	vecteur & vv=*v._VECTptr;
	if (vv.size()!=3 || A.type!=_VECT || A._VECTptr->size()!=3)
	  setsizeerr();
	vecteur & vA=*A._VECTptr;
	if (is_zero(vv[0])&&is_zero(vv[1]))
	  return gen(makevecteur(symbolic(at_equal,makevecteur(x,vA[0]),symbolic(at_equal,makevecteur(y,vA[1])))),_SEQ__VECT);
	vecteur v1(makevecteur(vv[1],-vv[0],0));
	vecteur v2(cross(vv,v1));
	vecteur xyz(subvecteur(vxyz,vA));
	return gen(makevecteur(symbolic(at_equal,makevecteur(dotvecteur(v1,xyz),zero)),symbolic(at_equal,makevecteur(dotvecteur(v2,vxyz),zero))),_SEQ__VECT);
      }
      gen a=im(v,contextptr);
      gen b=-re(v,contextptr);
      gen d=gcd(a,b);
      a=normal(a/d);
      b=normal(b/d);
      gen c=a*re(A,contextptr)+b*im(A,contextptr);
      if (!is_zero(b))
	return symbolic(at_equal,makevecteur(y,normal(-a/b)*x+normal(c/b)));
      return symbolic(at_equal,makevecteur(a*x+b*y,c));
    }
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_curve)){
      vecteur v=*e._SYMBptr->feuille._VECTptr->front()._VECTptr;
      gen xt=re(v[0],contextptr);
      rewrite_with_t_real(xt,v[1],contextptr);
      gen yt=im(v[0],contextptr);
      rewrite_with_t_real(yt,v[1],contextptr);
      vecteur w(solve(xt-x,v[1],0,contextptr));
      if (w.empty())
	setsizeerr("Can't isolate");
      yt=subst(yt,v[1],w.front());
      return symbolic(at_equal,makevecteur(y,yt));
    }
    else
      return e;
  }
  const string _equation_s("equation");
  unary_function_eval __equation(&giac::_equation,_equation_s);
  unary_function_ptr at_equation (&__equation);

  // If subtype is segment/vector or halfline remove
  // the element of v that are not in the segment/halfline/... ab
  vecteur remove_not_in_segment(const gen & a,const gen & b,int subtype,const vecteur & v,GIAC_CONTEXT){
    if (subtype==_LINE__VECT)
      return v;
    gen ab(b-a);
    const_iterateur it=v.begin(),itend=v.end();
    vecteur res;
    for (;it!=itend;++it){
      gen t=scalar_product(remove_at_pnt(*it)-a,ab,contextptr)/scalar_product(ab,ab,contextptr);
      if (subtype==_HALFLINE__VECT){
	if (!is_strictly_positive(-t,contextptr))
	  res.push_back(*it);
      }
      else {
	if (!is_strictly_positive(-t,contextptr) && !is_strictly_positive(t-1,contextptr))
	  res.push_back(*it);
      }
    }
    return res;
  }

  vecteur interdroitecercle(const gen & a,const gen &b,GIAC_CONTEXT){
    // D inter C
    gen centre,rayon;
    centre_rayon(b,centre,rayon,true,contextptr);
    // projection of the center of C over D
    gen a1(a._VECTptr->front()),a2(a._VECTptr->back()); // D=(a1a2)
    if (a1.type==_VECT && b.is_symb_of_sommet(at_cercle))
      setsizeerr("3-d line/2-d circle");
    if (a1.type!=_VECT && b.is_symb_of_sommet(at_hypersphere))
      setsizeerr("2-d line/sphere");
    gen t(projection(a1,a2,centre,contextptr));
    gen pr(t*a1+(1-t)*a2);
    gen a_pr(pr-centre);
    gen delta(abs_norm(a_pr,contextptr));
    if (ck_is_strictly_greater(delta,rayon,contextptr))
      return vecteur(0);
    gen a_pr_perp=a2-a1;
    a_pr_perp=rdiv(a_pr_perp,abs_norm(a_pr_perp,contextptr));
    gen d_a(sqrt(pow(rayon,plus_two)-pow(delta,plus_two)));
    return remove_not_in_segment(a1,a2,a.subtype,makevecteur(symb_pnt(ratnormal(pr+d_a*a_pr_perp),default_color(contextptr),contextptr),symb_pnt(ratnormal(pr-d_a*a_pr_perp),default_color(contextptr),contextptr)),contextptr);
  }  

  // a must be a vector of length 2, b a symbolic
  vecteur interdroitehyperplan(const gen & a,const gen &b,GIAC_CONTEXT){ 
    if (a.type!=_VECT || b.type!=_SYMB || a._VECTptr->size()!=2)
      setsizeerr();
    // D inter H
    gen A=a._VECTptr->front(),B=a._VECTptr->back(); // D=(AB)
    gen & f=b._SYMBptr->feuille;
    gen AB=B-A;
    if (f.type!=_VECT || f._VECTptr->size()!=2 )
      setsizeerr();
    gen C=f._VECTptr->back(),Hn=f._VECTptr->front(); // H= C normal is n
    gen AC=C-A;
    if (Hn.type!=_VECT || AB.type!=_VECT || AC.type!=_VECT)
      setsizeerr();
    vecteur v(*AB._VECTptr),n(*Hn._VECTptr);
    gen vn(dotvecteur(v,n));
    if (is_zero(vn)){ // D is parallel to H
      return vecteur(0); // FIXME should be D if D is in H
    }
    // H inter D = A + t*v, with t such that A+t*v-C is normal to n
    // Hence t=(C-A).n/(v.n)
    gen t=dotvecteur(*AC._VECTptr,n)/vn;
    gen M(_point(A+t*v,contextptr));
    return remove_not_in_segment(A,B,a.subtype,vecteur(1,M),contextptr);
  }

  // a=hypersurface, b=curve
  vecteur interhypersurfacecurve(const gen & a,const gen &b,GIAC_CONTEXT){ 
    gen & af=a._SYMBptr->feuille;
    gen & bf=b._SYMBptr->feuille;
    if (af.type!=_VECT || bf.type!=_VECT || af._VECTptr->empty() || bf._VECTptr->empty() )
      setsizeerr();
    vecteur av=*af._VECTptr; 
    // av[0]=point on hypersurface, av[1]=parameters
    gen & bf0=bf._VECTptr->front();
    if (bf0.type!=_VECT || bf0._VECTptr->size()<2 || bf0._VECTptr->front().type!=_VECT)
      setsizeerr();
    vecteur & bv=*bf0._VECTptr; // bv[0]=point on curve, bv[1]=parameter
    if (av.size()==3 && av[1].type!=_VECT && av[2].type==_VECT){
      // Hypersurface with an equation
      // av[1]=equation, av[2]=variables
      vecteur & vars=*av[2]._VECTptr;
      gen eq(subst(av[1],vars,*bv[0]._VECTptr));
      vecteur sol;
      try { 
	sol=solve(eq,bv[1],0,contextptr);
      }
      catch (std::runtime_error & e){
      }
      vecteur res;
      iterateur it=sol.begin(),itend=sol.end();
      for (;it!=itend;++it){
	res.push_back(_point(subst(bv[0],bv[1],*it),contextptr));
      }
      return res;
    }
    // Hypersurface without equation (parametrized only)
    if (av.size()<2 || av[1].type!=_VECT)
      setsizeerr();
    gen eq(av[0]-bv[0]);
    vecteur vars(*av[1]._VECTptr);
    vars.push_back(bv[1]);
    vecteur sol;
    try {
      sol=solve(eq,vars,0,contextptr);
    }
    catch (std::runtime_error & e){
      setsizeerr();
    }
    vecteur res;
    iterateur it=sol.begin(),itend=sol.end();
    for (;it!=itend;++it){
      res.push_back(_point(subst(bv[0],bv[1],it->_VECTptr->back()),contextptr));
    }
    return res;
  }

  gen hyperplan2hypersurface(const gen & g){
    if (!g.is_symb_of_sommet(at_hyperplan))
      setsizeerr();
    vecteur n,P;
    hyperplan_normal_point(g,n,P);
    if (n.size()!=3)
      setdimerr();
    vecteur xyz(makevecteur(x__IDNT_e,y__IDNT_e,z__IDNT_e));
    gen eq=dotvecteur(subvecteur(xyz,P),n);
    return hypersurface(undef,eq,xyz);
  }

  gen hypersphere2hypersurface(const gen & g){
    if (!g.is_symb_of_sommet(at_hypersphere))
      setsizeerr();
    vecteur xyz(makevecteur(x__IDNT_e,y__IDNT_e,z__IDNT_e));
    return hypersurface(undef,hypersphere_equation(g,xyz),xyz);
  }

  vecteur interpolygone(const vecteur & p,const gen & bb,GIAC_CONTEXT){
    vecteur res;
    const_iterateur it=p.begin(),itend=p.end();
    if (itend-it<2)
      return res;
    for (++it;it!=itend;++it){
      gen tmp=symbolic(at_pnt,makevecteur(gen(makevecteur(*(it-1),*it),_GROUP__VECT),0));
      vecteur add(inter(tmp,bb,contextptr));
      const_iterateur jt=add.begin(),jtend=add.end();
      for (;jt!=jtend;++jt){
	if (!equalposcomp(res,*jt))
	  res.push_back(*jt);
      }
    }
    return res;
  }

  // Currently works only if v is made of lines
  // For each line, get the intersections of the polygone ABCD with the line
  // If there are 2 intersections return a segment, else return a point or void
  vecteur remove_face(const vecteur & face,const vecteur & v,GIAC_CONTEXT){
    vecteur ABCD;
    // if the face has 4 vertices, reorder it for convexity
    if (face.size()==4)
      ABCD=makevecteur(face[0],face[1],face[3],face[2]);
    else
      ABCD=face;
    vecteur res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      gen tmp=remove_at_pnt(*it);
      if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	res.push_back(*it); // FIXME, arc of circles etc.
      vecteur v(interpolygone(ABCD,*it,contextptr));
      int s=v.size();
      if (!s)
	continue;
      if (s==1)
	res.push_back(v.front());
      if (s==2)
	res.push_back(symb_pnt(gen(makevecteur(v.front(),v.back()),_GROUP__VECT),contextptr));
    }
    return res;
  }

  // Intersection polyedre with something
  // Find intersection of each face with something
  // For each face, find intersection of hyperplan with something
  vecteur interpolyedre(const vecteur & p,const gen & bb,GIAC_CONTEXT){
    vecteur res;
    const_iterateur it=p.begin(),itend=p.end();
    for (;it!=itend;++it){
      if (it->type!=_VECT)
	continue;
      vecteur v=*it->_VECTptr;
      int s=v.size();
      if (s==4){
	v.pop_back();
	--s;
      }
      if (s!=3)
	continue;
      gen AB(v[1]-v[0]),AC(v[2]-v[0]);
      if (AB.type!=_VECT || AB._VECTptr->size()!=3 || AC.type!=_VECT || AC._VECTptr->size()!=3)
	continue;
      vecteur n=cross(*AB._VECTptr,*AC._VECTptr);
      gen tmp=symbolic(at_hyperplan,makevecteur(n,v[0]));
      bool b=show_point(contextptr);
      show_point(false,contextptr);
      vecteur w(inter(tmp,bb,contextptr));
      show_point(b,contextptr);
      res=mergevecteur(res,remove_face(*it->_VECTptr,w,contextptr));
    }
    return res;
  }

  vecteur interhyperplan(const gen & p1,const gen & p2,GIAC_CONTEXT){
    vecteur P1,n1,P2,n2;
    hyperplan_normal_point(p1,n1,P1);
    hyperplan_normal_point(p2,n2,P2);
    vecteur n=cross(n1,n2); // direction of intersection
    vecteur n3=cross(n,n1); // perpendicular to n1 hence in P1 and not // inter
    // Find a point on intersection: P1+t n3 -P2 perpendicular to n2
    // hence (P2-P1).n2=t n3.n2
    gen P=do_point3d(P1-scalar_product(P1-P2,n2,contextptr)/dotvecteur(n3,n2)*n3);
    gen Q=do_point3d(P+n);
    return makevecteur(symb_pnt(gen(makevecteur(P,Q),_LINE__VECT),contextptr));
  }

  vecteur inter(const gen & aa,const gen & bb,GIAC_CONTEXT){
    if (aa.type==_VECT){
      if (bb.type==_VECT){
	vecteur res;
	const_iterateur it=aa._VECTptr->begin(),itend=aa._VECTptr->end();
	for (;it!=itend;++it){
	  res=mergevecteur(res,inter(*it,bb,contextptr));
	}
	return res;
      }
      return inter(bb,aa,contextptr);
    }
    if (bb.type==_VECT){
      vecteur res;
      const_iterateur it=bb._VECTptr->begin(),itend=bb._VECTptr->end();
      for (;it!=itend;++it){
	res=mergevecteur(res,inter(aa,*it,contextptr));
      }
      return res;
    }
    gen a=remove_at_pnt(aa);
    gen b=remove_at_pnt(bb);
    bool asp=a.is_symb_of_sommet(at_hypersphere),bsp=b.is_symb_of_sommet(at_hypersphere);
    bool ac=a.is_symb_of_sommet(at_cercle),bc=b.is_symb_of_sommet(at_cercle);
    if (a.type==_VECT){
      if (a.subtype==_POLYEDRE__VECT)
	return interpolyedre(*a._VECTptr,bb,contextptr);
      int as=a._VECTptr->size();
      if (as>2)
	return interpolygone(*a._VECTptr,bb,contextptr);
      if (as==2){ 
	if (b.type==_VECT){
	  int bs=b._VECTptr->size();
	  if (bs>2)
	    return interpolygone(*b._VECTptr,aa,contextptr);
	  if (bs==2){ // D inter D'
	    gen a1=a._VECTptr->front(),a2=a._VECTptr->back(),b1=b._VECTptr->front(),b2=b._VECTptr->back();
	    gen v(a2-a1),w(b2-b1);
	    if (v.type==_VECT){
	      if (w.type!=_VECT || v._VECTptr->size()!=3 || w._VECTptr->size()!=3)
		setsizeerr();
	      // 3-d line intersection
	      gen M,N;
	      vecteur n;
	      if (perpendiculaire_commune(a,b,M,N,n,contextptr)){
		if (is_zero(M-N))
		  return remove_not_in_segment(b1,b2,b.subtype,remove_not_in_segment(a1,a2,a.subtype,makevecteur(M),contextptr),contextptr);
	      }
	      return vecteur(0);
	    }
	    // 2-d line intersection
	    gen x=(b1-a1).re(contextptr),y=(b1-a1).im(contextptr),vx=v.re(contextptr),vy=v.im(contextptr);
	    gen wx=w.re(contextptr),wy=w.im(contextptr);
	    gen alpha=rdiv(wy*x-wx*y,wy*vx-wx*vy);
	    return remove_not_in_segment(b1,b2,b.subtype,remove_not_in_segment(a1,a2,a.subtype,makevecteur(symb_pnt(normal(a1+alpha*v),default_color(contextptr),contextptr)),contextptr),contextptr);
	  } // end bs==2
	} // end b.type==_VECT
	if (bc || bsp)
	  return interdroitecercle(a,b,contextptr);
	if (b.is_symb_of_sommet(at_hyperplan)) // n-d line inter hyperplan
	  return interdroitehyperplan(a,b,contextptr);
      } // end droite inter something (as==2)
    }
    if (b.type==_VECT) {
      if (b.subtype==_POLYEDRE__VECT)
	return interpolyedre(*b._VECTptr,aa,contextptr);
      int bs=b._VECTptr->size();
      if (bs>2)
	return interpolygone(*b._VECTptr,aa,contextptr);
      if (bs==2){
	if (a.is_symb_of_sommet(at_hyperplan))
	  return interdroitehyperplan(b,a,contextptr);      
	if (ac || asp)
	  return interdroitecercle(b,a,contextptr);
      }
    }
    if ( (ac && bc) || (asp && bsp) ){
      gen centre_a,rayon_a,centre_b,rayon_b;
      centre_rayon(a,centre_a,rayon_a,true,contextptr);
      centre_rayon(b,centre_b,rayon_b,true,contextptr);
      gen ab(centre_b-centre_a);
      gen ab2(abs_norm(ab,contextptr));
      if (ck_is_strictly_greater(ab2,rayon_a+rayon_b,contextptr))
	return vecteur(0); // empty
      if (asp && (centre_a.type!=_VECT || centre_a._VECTptr->size()!=3 ))
	setsizeerr();
      // delta=(ra^2-rb^2)/ab^2
      gen delta=(pow(rayon_a,plus_two)-pow(rayon_b,plus_two))/ab2;
      gen d_a(rdiv(delta+ab2,plus_two));
      gen ab4(centre_a+rdiv(d_a,ab2)*ab);
      gen d_perp(sqrt(pow(rayon_a,plus_two)-pow(d_a,plus_two)));
      if (ac){ // circle inter circle = 2 points (or 1)
	gen ab_perp(im(ab,contextptr)-cst_i*re(ab,contextptr));
	ab_perp=rdiv(ab_perp,abs_norm(ab_perp,contextptr));
	return makevecteur(symb_pnt(normal(ab4+d_perp*ab_perp),default_color(contextptr),contextptr),symb_pnt(normal(ab4-d_perp*ab_perp),default_color(contextptr),contextptr));
      }
      else { // 3-d sphere inter sphere = 2-d circle
	// we will draw a parametric curve in the plan perpendicular to ab
	// at ab4, with radius d_perp
	// Find 2 vectors normal to ab
	vecteur v1,v2;
	normal3d(ab,v1,v2);
	v1=divvecteur(v1,abs_norm(v1,contextptr));
	v2=divvecteur(v2,abs_norm(v2,contextptr));
	gen eq=ab4+symb_cos(vx_var)*v1+symb_sin(vx_var)*v2;
	return makevecteur(_plotparam(eq,contextptr));
      }
    }
    // Now replace line and circles by curve
    if ((a.type==_VECT) && (a._VECTptr->size()==2))
      a=line2curve(a);
    if ((b.type==_VECT) && (b._VECTptr->size()==2))
      b=line2curve(b);
    if (ac)
      a=cercle2curve(a);
    if (bc)
      b=cercle2curve(b);
    if (a.is_symb_of_sommet(at_hyperplan)){
      if (b.is_symb_of_sommet(at_hyperplan))
	return interhyperplan(a,b,contextptr);
      a=hyperplan2hypersurface(a);
    }
    if (b.is_symb_of_sommet(at_hyperplan))
      b=hyperplan2hypersurface(b);
    if (asp)
      a=hypersphere2hypersurface(a);
    if (bsp)
      b=hypersphere2hypersurface(b);
    if (a.is_symb_of_sommet(at_hypersurface) && b.is_symb_of_sommet(at_curve))
      return interhypersurfacecurve(a,b,contextptr);
    if (b.is_symb_of_sommet(at_hypersurface) && a.is_symb_of_sommet(at_curve))
      return interhypersurfacecurve(b,a,contextptr);
    if ( (a.type==_SYMB) && (a._SYMBptr->sommet==at_curve) && (b.type==_SYMB) && (b._SYMBptr->sommet==at_curve) ){ // curve inter curve
      gen fa,fb;
      if ((fa=a._SYMBptr->feuille).type!=_VECT || (fb=b._SYMBptr->feuille).type!=_VECT || fa._VECTptr->empty() || fb._VECTptr->empty() || fa._VECTptr->front().type!=_VECT || fb._VECTptr->front().type!=_VECT)
	setsizeerr();
      vecteur va=*fa._VECTptr->front()._VECTptr;
      vecteur vb=*fb._VECTptr->front()._VECTptr;
      if (va.size()<2 || vb.size()<2 || va[1].type!=_IDNT || vb[1].type!=_IDNT)
	setsizeerr();
      gen xa=re(va[0],contextptr),ya=im(va[0],contextptr);
      rewrite_with_t_real(xa,va[1],contextptr);
      rewrite_with_t_real(ya,va[1],contextptr);
      gen xb=re(vb[0],contextptr),yb=im(vb[0],contextptr);
      rewrite_with_t_real(xb,va[1],contextptr);
      rewrite_with_t_real(yb,va[1],contextptr);
      gen eq=xa-xb;
      vecteur sol,res;
      try {
	sol=solve(eq,*va[1]._IDNTptr,0,contextptr); 
	const_iterateur it=sol.begin(),itend=sol.end();
	for (;it!=itend;++it){
	  eq=subst(yb-ya,va[1],*it);
	  res=mergevecteur(res,solve(eq,*vb[1]._IDNTptr,0,contextptr)); 
	}
      }
      catch (std::runtime_error & error){
	return makevecteur(symbolic(at_inter,makevecteur(a,b)));
      }
      sol.clear();
      const_iterateur it=res.begin(),itend=res.end();
      for (;it!=itend;++it){
	sol.push_back(symb_pnt(subst(vb[0],vb[1],*it),default_color(contextptr),contextptr));
      }
      return sol;
    }
    return makevecteur(symbolic(at_inter,makevecteur(a,b)));
  }

  // args=[curve, parameter] or element(curve)
  gen _tangent(const gen & args,GIAC_CONTEXT){
    gen arg=args;
    if (arg.type!=_VECT){
      // check if arg is an element of a curve
      if ( (arg.type==_SYMB) && (arg._SYMBptr->sommet==at_pnt)){
	arg=(*arg._SYMBptr->feuille._VECTptr)[1];
	if (arg.type==_VECT)
	  arg=(*arg._VECTptr)[1];
      }
    }
    if ( (arg.type==_VECT) && (arg._VECTptr->size()>2)){
      vecteur v(args._VECTptr->end()-2,args._VECTptr->end());
      return _tangent(v,contextptr);
    }
    if ( (arg.type==_VECT) && (arg._VECTptr->size()==2)){
      gen curve=remove_at_pnt(arg._VECTptr->front());
      if (curve.type==_VECT)
	return symb_pnt(curve,contextptr);
      gen t=arg._VECTptr->back();
      if ( (curve.type==_SYMB) && (curve._SYMBptr->sommet==at_cercle || curve._SYMBptr->sommet==at_hypersphere)){
	gen centre,rayon;
	centre_rayon(curve,centre,rayon,false,contextptr);
	rayon=ratnormal(rayon);
	if ( t.type==_VECT || t.is_symb_of_sommet(at_pnt) || !is_zero(im(t,contextptr)) ){
	  // tangent to cercle via point
	  t=remove_at_pnt(t);
	  gen OA=t-centre;
	  gen rayon2(pow(abs_norm(OA,contextptr),2)-rayon*conj(rayon,contextptr));
	  if (is_zero(rayon2)){
	    if (OA.type==_VECT)
	      return _plan(makevecteur(OA,t),contextptr);
	    return _droite(makevecteur(t,t+im(OA,contextptr)-cst_i*re(OA,contextptr)),contextptr);
	  }
	  if (is_positive(-rayon2,contextptr) || OA.type==_VECT)
	    setsizeerr();
	  vecteur v=inter(_cercle(makevecteur(symb_pnt(t,contextptr),sqrt(rayon2)),contextptr),curve,contextptr);
	  if (v.empty())
	    setsizeerr();
	  return makevecteur(_droite(makevecteur(t,v.front()),contextptr),_droite(makevecteur(t,v.back()),contextptr));
	}
	else {
	  t=centre+rayon*exp(cst_i*t);
	  return _perpendiculaire(makevecteur(t,t,centre),contextptr);
	}
      }
      if (curve.is_symb_of_sommet(at_hyperplan))
	return curve;
      if (curve.is_symb_of_sommet(at_hypersurface)){
	gen & curvef=curve._SYMBptr->feuille;
	if (curvef.type!=_VECT || curvef._VECTptr->size()<3)
	  setsizeerr();
	vecteur curvev=*curvef._VECTptr;
	gen eq(undef),vars;
	if (curvev[1].type!=_VECT){
	  eq=curvev[1];
	  vars=curvev[2];
	  curvev=*curvev[0]._VECTptr;
	}
	if (t.type==_VECT){ // parameters values
	  gen f=curvev[0];
	  vars=curvev[1];
	  gen fprime=derive(f,vars);
	  fprime=ratnormal(subst(fprime,vars,t));
	  t=ratnormal(subst(f,vars,t));
	  // make hyperplan by t with normal orthogonal to fprime
	  if (!ckmatrix(fprime) || fprime._VECTptr->size()!=2)
	    setsizeerr();
	  vecteur n(cross(*fprime._VECTptr->front()._VECTptr,*fprime._VECTptr->back()._VECTptr));
	  return _plan(makevecteur(n,t),contextptr);
	}
	if (is_undef(eq))
	  setsizeerr("Hypersurface w/o equation not implemented");
	t=remove_at_pnt(t);
	// Check that the point is on the hypersurface (with the equation)
	gen res=simplify(subst(eq,vars,t));
	if (!is_zero(res))
	  setsizeerr("f(t)!=0:"+res.print());
	gen fprime(derive(eq,vars));
	fprime=ratnormal(subst(fprime,vars,t));
	fprime=fprime/abs_norm(fprime,contextptr);
	return _plan(makevecteur(fprime,t),contextptr);
      }
      if ( (curve.type!=_SYMB) || (curve._SYMBptr->sommet!=at_curve))
	setsizeerr();
      vecteur v=*curve._SYMBptr->feuille._VECTptr->front()._VECTptr;
      gen direction(derive(v[0],*v[1]._IDNTptr));
      if ( (t.type==_SYMB) && (t._SYMBptr->sommet==at_pnt)){
	t=remove_at_pnt(t);
	gen x(re(v[0],contextptr)),y(im(v[0],contextptr)),xp(re(direction,contextptr)),yp(im(direction,contextptr));
	rewrite_with_t_real(x,v[1],contextptr);
	rewrite_with_t_real(y,v[1],contextptr);
	rewrite_with_t_real(xp,v[1],contextptr);
	rewrite_with_t_real(yp,v[1],contextptr);
	// for plotfunc, try v[1]=re(t)
	if (is_zero(normal(subst(y,v[1],re(t,contextptr))-im(t,contextptr)))){
	  direction=subst(direction,v[1],re(t,contextptr));
	  return _droite(makevecteur(t,t+direction),contextptr);
	}
	gen lambda=rdiv(re(t,contextptr)-x,xp);
	vecteur sol;
	try {
	  sol=solve(y+lambda*yp-im(t,contextptr),*v[1]._IDNTptr,0,contextptr);
	}
	catch (std::runtime_error & error ){
	  cerr << error.what() << endl;
	  sol.clear();
	}
	iterateur it=sol.begin(),itend=sol.end();
	for (;it!=itend;++it){
	  *it=subst(v[0],v[1],*it);
	}
	return gen(sol,_GROUP__VECT);
      }
      else {
	direction=subst(direction,v[1],t);
	gen point(subst(v[0],v[1],t));
	return _droite(makevecteur(point,point+direction),contextptr);
      }
    }
    return symbolic(at_tangent,args);
  }
  const string _tangent_s("tangent");
  unary_function_eval __tangent(&giac::_tangent,_tangent_s);
  unary_function_ptr at_tangent (&__tangent);

  void foyers_a(const gen & args,gen & F1,gen & F2, gen & a){
    gen FF=remove_at_pnt(args._VECTptr->front());
    if (FF.type==_VECT && FF._VECTptr->size()!=3){
      if (FF._VECTptr->size()!=2)
	setsizeerr();
      F1=FF._VECTptr->front();
      F2=FF._VECTptr->back();
    }
    else {
      if (args._VECTptr->size()!=3)
	setsizeerr();
      F1=FF;
      F2=remove_at_pnt((*args._VECTptr)[1]);
    }
    a=args._VECTptr->back();
  }

  gen xy2eitheta(const gen & x,const gen & y,GIAC_CONTEXT){
    if (is_zero(x))
      return cst_i;
    gen t=rdiv(y,x);
    t=rdiv(plus_one+cst_i*t,sqrt(plus_one+t*t));
    if (!is_positive(x,contextptr)) 
      return -t;
    else
      return t;
  }

  gen _ellipse(const gen & args,GIAC_CONTEXT){
    ck_parameter_t();
    if ((args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_ellipse,args);
    // args=[F,F',M] or [F,F',a] / FM+F'M=2*a
    gen F1,F2,a,n;
    foyers_a(args,F1,F2,a);
    if ( a.is_symb_of_sommet(at_pnt) || !is_zero(im(a,contextptr)) ){
      a=remove_at_pnt(a);
      if (a.type==_VECT)
	n=a;
      a=rdiv(abs_norm(a-F1,contextptr)+abs_norm(a-F2,contextptr),plus_two);
    }
    gen F1F2=F2-F1;
    gen O=rdiv(F1+F2,plus_two);
    gen c2=rdiv(F1F2.squarenorm(contextptr),gen(4));
    gen b=sqrt(a*a-c2),res;
    gen theta=t__IDNT_e*cst_two_pi/(gnuplot_tmax-gnuplot_tmin);
    if (n.type==_VECT){ // 3-d
      n=n-O;
      n=cross(cross(F1F2,n),F1F2);
      res=O+a*symb_cos(theta)*F1F2/abs_norm(F1F2,contextptr)+b*symb_sin(theta)*n/abs_norm(n,contextptr);
    }
    else { // 2-d
      gen xF1F2=re(F1F2,contextptr),yF1F2=im(F1F2,contextptr);
      gen eitheta(xy2eitheta(xF1F2,yF1F2,contextptr));
      res=eitheta*(a*symb_cos(theta)+b*cst_i*symb_sin(theta))+O;
    }
    return _paramplot(gen(makevecteur(res,t__IDNT_e,gnuplot_tmin,gnuplot_tmax),_SEQ__VECT),contextptr);
  }
  const string _ellipse_s("ellipse");
  unary_function_eval __ellipse(&giac::_ellipse,_ellipse_s);
  unary_function_ptr at_ellipse (&__ellipse);

  gen _hyperbole(const gen & args,GIAC_CONTEXT){
    ck_parameter_t();
    if ((args.type!=_VECT) || (args._VECTptr->size()<2))
      return symbolic(at_hyperbole,args);
    // args=[F,F',M] or [F,F',a] / FM+F'M=2*a
    gen F1,F2,a,n;
    foyers_a(args,F1,F2,a);
    if ( a.is_symb_of_sommet(at_pnt) || !is_zero(im(a,contextptr)) ){
      a=remove_at_pnt(a);
      if (a.type==_VECT)
	n=a;
      a=rdiv(abs_norm(a-F1,contextptr)-abs_norm(a-F2,contextptr),plus_two);
    }
    gen F1F2=F2-F1;
    gen O=rdiv(F1+F2,plus_two);
    gen c2=rdiv(F1F2.squarenorm(contextptr),gen(4));
    gen b=sqrt(c2-a*a),res;
    gen theta=(t__IDNT_e-(gnuplot_tmax+gnuplot_tmin)/2)*6/(gnuplot_tmax-gnuplot_tmin);
    if (n.type==_VECT){
      n=n-O;
      n=cross(cross(F1F2,n),F1F2);
      res=a*cosh(theta)*F1F2/abs_norm(F1F2,contextptr)+b*sinh(theta)*n/abs_norm(n,contextptr);
    }
    else {
      gen xF1F2=re(F1F2,contextptr),yF1F2=im(F1F2,contextptr);
      gen eitheta(xy2eitheta(xF1F2,yF1F2,contextptr));
      res=eitheta*(a*cosh(theta)+b*cst_i*sinh(theta));
    }
    return makesuite(_paramplot(gen(makevecteur(res+O,t__IDNT_e,gnuplot_tmin,gnuplot_tmax),_SEQ__VECT),contextptr),_paramplot(gen(makevecteur(-res+O,t__IDNT_e,gnuplot_tmin,gnuplot_tmax),_SEQ__VECT),contextptr)); // should be -inf..inf
  }
  const string _hyperbole_s("hyperbole");
  unary_function_eval __hyperbole(&giac::_hyperbole,_hyperbole_s);
  unary_function_ptr at_hyperbole (&__hyperbole);

  gen _parabole(const gen & args,GIAC_CONTEXT){
    ck_parameter_t();
    // args=[O,F] symmetry axe=OF (t+i*t^2)/OF
    // or args=[0,c] symmetry axe=Oy y=c*x^2 (t+i*t^2)/c or args=c
    gen O,F,c,eitheta=plus_one;
    if (args.type!=_VECT)
      c=args;
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if (s<2)
	setsizeerr();
      F=remove_at_pnt(v.front());
      O=v[1];
      if ( O.is_symb_of_sommet(at_pnt) || !is_zero(im(O,contextptr)) ){
	O=remove_at_pnt(O);
	gen OF=F-O;
	c=rdiv(abs_norm(OF,contextptr),plus_two);
	if (OF.type!=_VECT)
	  eitheta=xy2eitheta(im(OF,contextptr),-re(OF,contextptr),contextptr);
	else {
	  eitheta=remove_at_pnt(v[2])-O;
	  eitheta=cross(cross(OF,eitheta),OF);
	  eitheta=eitheta/abs_norm(eitheta,contextptr);
	}
      }
      else {
	c=O;
	O=F;
      }
    }
    gen res;
    gen cc=c.evalf_double();
    double ccc=fabs(cc._DOUBLE_val)*5;
    gen theta=(t__IDNT_e-(gnuplot_tmax+gnuplot_tmin)/2)*(ccc/(gnuplot_tmax-gnuplot_tmin));
    if (eitheta.type==_VECT){
      res=O+(F-O)/(c*abs_norm(F-O,contextptr))*pow(t__IDNT_e,2)+eitheta*t__IDNT_e/c;
    }
    else 
      res=O+rdiv(eitheta*t__IDNT_e*(1+cst_i*t__IDNT_e),c);
    if (cc.type!=_DOUBLE_)
      setsizeerr();
    double save_tstep=gnuplot_tstep;
    gnuplot_tstep=ccc/50;
    res= _paramplot(gen(makevecteur(res,t__IDNT_e,gnuplot_tmin,gnuplot_tmax),_SEQ__VECT),contextptr);
    gnuplot_tstep=save_tstep;
    return res;
  }
  const string _parabole_s("parabole");
  unary_function_eval __parabole(&giac::_parabole,_parabole_s);
  unary_function_ptr at_parabole (&__parabole);

  gen _legende(const gen & a,GIAC_CONTEXT){
    gen args=a.eval();
    if ( (a.type==_SYMB) && (args.type!=_VECT)){
      vecteur v;
      gen e(a._SYMBptr->feuille);
      if ( (e.type==_VECT) && (!e._VECTptr->empty()) )
	e=e._VECTptr->front();
      v.push_back(e.eval());
      v.push_back(args);
      args=v;
    }
    else {
      if ((args.type!=_VECT) || (args._VECTptr->size()<2))
	return symbolic(at_legende,args);
    }
    // gen decal=gen(5*(gnuplot_xmax-gnuplot_xmin)/plot_instructionsh+5*(gnuplot_ymax-gnuplot_ymin)/(plot_instructionsw-LEGENDE_SIZE))*cst_i;
    gen decal=0;
    int save_digits=decimal_digits;
    int couleur=FL_GREEN;
    decimal_digits=3;
    string s;
    vecteur & v(*args._VECTptr);
    for (int i=1;i<v.size();++i){
      if (v[i].type==_STRNG)
	s += *v[i]._STRNGptr;
      else {
	if (v[i].type==_INT_ && v[i].subtype==_INT_COLOR)
	  couleur=v[i].val;
	else
	  s += v[i].print();
      }
    }
    decimal_digits=save_digits;
    gen position=remove_at_pnt(args._VECTptr->front());
    if (position.type==_SYMB) {
      if (position._SYMBptr->sommet==at_cercle)
	position=position._SYMBptr->feuille;
      else 
	if (position._SYMBptr->sommet==at_curve){
	  vecteur v(*position._SYMBptr->feuille._VECTptr->front()._VECTptr);
	  position=subst(v[0],v[1],rdiv(v[2]+v[3],plus_two));
	}
    }
    if ( (position.type==_VECT) && (position._VECTptr->size()==2) ){
      position=rdiv(position._VECTptr->front()+position._VECTptr->back(),plus_two);
    }
    return symb_pnt_name(position+decal,couleur,string2gen(s,false),contextptr);
  }
  const string _legende_s("legende");
  unary_function_eval __legende(&giac::_legende,_legende_s);
  unary_function_ptr at_legende (&__legende,_QUOTE_ARGUMENTS);

  gen _couleur(const gen & a,GIAC_CONTEXT){
    if (a.type==_INT_){
      int i=default_color(contextptr);
      default_color(a.val,contextptr);
      return i;
    }
    if ( (a.type!=_VECT) || (a._VECTptr->size()<2))
      return default_color(contextptr);
    gen c=a._VECTptr->back(),b;
    if (a._VECTptr->size()>2)
      b=vecteur(a._VECTptr->begin(),a._VECTptr->end()-1);
    else
      b=a._VECTptr->front();
    if (b.type==_VECT){
      const_iterateur it=b._VECTptr->begin(),itend=b._VECTptr->end();
      vecteur res;
      res.reserve(itend-it);
      for (;it!=itend;++it)
	res.push_back(_couleur(makevecteur(*it,c),contextptr));
      return gen(res,b.subtype);
    }
    if ( (b.type!=_SYMB) || (b._SYMBptr->sommet!=at_pnt))
      return symbolic(at_couleur,a);      
    vecteur v(*b._SYMBptr->feuille._VECTptr);
    v[1]=c;
    gen e=symbolic(at_pnt,gen(v,_PNT__VECT));
    if (io_graph(contextptr))
      __interactive.op(e);    
    return e;
  }
  const string _couleur_s("affichage");
  unary_function_eval __couleur(&giac::_couleur,_couleur_s);
  unary_function_ptr at_couleur (&__couleur);

  // innert instruction, used e.g. for a:=element(3..5)
  // returns parameter(a,3..5,4) and stores 4 in a
  gen _parameter(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=4))
      setsizeerr();
    return symbolic(at_parameter,args);
  }
  const string _parameter_s("parameter");
  unary_function_unary __parameter(&giac::_parameter,_parameter_s);
  unary_function_ptr at_parameter (&__parameter);

  gen _hauteur(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_hauteur,args);
    gen e,f,g;
    gen23points(args,e,f,g);
    f=f-g;
    f=im(f,contextptr)-cst_i*re(f,contextptr);
    return symb_segment(e,e+f,default_color(contextptr),contextptr);
  }
  const string _hauteur_s("hauteur");
  unary_function_eval __hauteur(&giac::_hauteur,_hauteur_s);
  unary_function_ptr at_hauteur (&__hauteur);

  gen plotfield(const gen & xp,const gen & yp,const gen & x,const gen & y,const context * contextptr){
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    _Pictsize(0);
    int horizontal_pixels=plot_instructionsw-LEGENDE_SIZE,vertical_pixels=plot_instructionsh;
    int scaling=3;
    double xstep=3*gnuplot_pixels_per_eval*(gnuplot_xmax-gnuplot_xmin)/(horizontal_pixels-1);
    double ystep=3*gnuplot_pixels_per_eval*(gnuplot_ymax-gnuplot_ymin)/(vertical_pixels-1);
    vecteur xy_v;
    xy_v.push_back(x);
    xy_v.push_back(y);
    gen xp_eval,yp_eval,xy(xy_v),origine;
    vecteur curxcury(2);
    vecteur res;
    double echelle;
    if (xstep<ystep) echelle=xstep; else echelle=ystep;
    for (double curx=gnuplot_xmin;curx<=gnuplot_xmax;curx+=scaling*xstep){
      curxcury[0]=curx;
      for (double cury=gnuplot_ymin;cury<=gnuplot_ymax;cury+=scaling*ystep){
	curxcury[1]=cury;
	xp_eval=subst(xp,xy,curxcury).evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
	yp_eval=subst(yp,xy,curxcury).evalf_double(DEFAULT_EVAL_LEVEL,contextptr);
	if ((xp_eval.type==_DOUBLE_) && (yp_eval.type==_DOUBLE_)){
	  origine=gen(curx)+cst_i*gen(cury);
	  res.push_back(symb_pnt(gen(makevecteur(origine,origine+xp_eval*echelle+cst_i*yp_eval*echelle),_GROUP__VECT),FL_GREEN,contextptr));
	}
      }
    }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
    return gen(res,_GROUP__VECT);
  }
  // args=[dx/dt,dy/dt,x,y] or [dy/dx,x,y]
  // or [ [dx/dt,dy/dt], [x,y] ] or [ dy/dx, [x,y]]
  gen _plotfield(const gen & args,const context * contextptr){
    if (args.type!=_VECT)
      return symbolic(at_plotfield,args);
    if (args._VECTptr->size()<2)
      setsizeerr();
    vecteur v(*args._VECTptr);
    gen xp,yp,x,y;
    switch (v.size()){
    case 2:
      if ( (v.back().type!=_VECT) || (v.back()._VECTptr->size()!=2) )
	setsizeerr();
      x=v.back()._VECTptr->front();
      y=v.back()._VECTptr->back();
      yp=v.front();
      if (yp.type==_VECT){
	xp=yp._VECTptr->front();
	yp=yp._VECTptr->back();
      }
      else
	xp=plus_one;
      break;
    case 3:
      xp=plus_one;
      yp=v[0];
      x=v[1];
      y=v[2];
      break;
    default:
      xp=v[0];
      yp=v[1];
      x=v[2];
      y=v[3];
    }
    return plotfield(xp,yp,x,y,contextptr);
  }
  const string _plotfield_s("plotfield");
  unary_function_eval __plotfield(&giac::_plotfield,_plotfield_s);
  unary_function_ptr at_plotfield (&__plotfield);

  const string _fieldplot_s("fieldplot");
  unary_function_eval __fieldplot(&giac::_plotfield,_fieldplot_s);
  unary_function_ptr at_fieldplot (&__fieldplot);

  // plot solution of y'=f(x,y) [x0,y0] in current plot range
  // OR [x,y]'=f(t,x,y) t0 x0 y0 in current plot range
  // args = dy/dx, [x, y], [x0, y0]
  // OR [dx/dt, dy/dt], [t, x, y], [t0, x0, y0]
  gen _plotode(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=3) )
      return symbolic(at_plotode,args);
    vecteur v(*args._VECTptr);
    gen fp=v[0];
    gen vars=v[1];
    gen init=v[2];
    if (fp.type!=_VECT){ // y'=f(x,y)
      if ( (vars.type!=_VECT) || (vars._VECTptr->size()!=2) )
	setsizeerr();
      vecteur f=makevecteur(makevecteur(plus_one,fp),identificateur(" t"),vars);
      if ( (init.type!=_VECT) || (init._VECTptr->size()!=2) )
	setsizeerr();
      gen t0=init._VECTptr->front();
      gen y0=makevecteur(t0,init._VECTptr->back());
      double ymin[2];
      ymin[0]=gnuplot_xmin;
      ymin[1]=gnuplot_ymin;
      double ymax[2];
      ymax[0]=gnuplot_xmax;
      ymax[1]=gnuplot_ymax;
      gen res1=odesolve(t0,-1e499,f,y0,true,ymin,ymax,contextptr);
      vecteur res1v=*res1._VECTptr;
      std::reverse(res1v.begin(),res1v.end());
      res1v.push_back(makevecteur(t0,makevecteur(t0,y0)));
      gen res2=odesolve(t0,1e499,f,y0,true,ymin,ymax,contextptr);
      vecteur resv=mergevecteur(res1v,*res2._VECTptr);
      // make the curve
      const_iterateur it=resv.begin(),itend=resv.end();
      vecteur res;
      res.reserve(itend-it);
      for (;it!=itend;++it){
	res.push_back(it->_VECTptr->back()._VECTptr->front()+cst_i*it->_VECTptr->back()._VECTptr->back());
      }
      return symb_pnt(gen(res,_GROUP__VECT),contextptr);
    }
  }
  const string _plotode_s("plotode");
  unary_function_eval __plotode(&giac::_plotode,_plotode_s);
  unary_function_ptr at_plotode (&__plotode);

  const string _odeplot_s("odeplot");
  unary_function_eval __odeplot(&giac::_plotode,_odeplot_s);
  unary_function_ptr at_odeplot (&__odeplot);

  // like plotode but the initial(s) condition(s) will be specified
  // by the user
  gen _interactive_plotode(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) )
      return symbolic(at_plotode,args);
    vecteur v(*args._VECTptr);
    gen fp=v[0];
    gen vars=v[1];
    if (fp.type!=_VECT){
      if ( (vars.type!=_VECT) || (vars._VECTptr->size()!=2) )
	setsizeerr();
#ifdef WIN32
      _plotfield(gen(makevecteur(fp,vars),_SEQ__VECT),contextptr);
#else
      _signal(_plotfield(gen(makevecteur(fp,vars),_SEQ__VECT),contextptr));
#endif
      vecteur f=makevecteur(makevecteur(plus_one,fp),identificateur(" t"),vars);
      for (;;){
	gen localisation=__click.op(vecteur(0),contextptr).evalf_double();
	double x0,y0;
	if (localisation.type==_DOUBLE_){
	  x0=localisation._DOUBLE_val;
	  y0=0;
	}
	else {
	  if ((localisation.type==_CPLX) && (localisation.subtype==3) ) {
	    x0=localisation._CPLXptr->_DOUBLE_val;
	    y0=(localisation._CPLXptr+1)->_DOUBLE_val;
	  }
	  else {
	    cerr << "End interactive_plotode" << endl;
	    break;
	  }
	}
#ifdef WIN32
	_plotode(makevecteur(fp,vars,makevecteur(x0,y0)),contextptr);
#else
	_signal(_plotode(makevecteur(fp,vars,makevecteur(x0,y0)),contextptr));
#endif
      }
    }
    return symbolic(at_interactive_plotode,args);
  }
  const string _interactive_plotode_s("interactive_plotode");
  unary_function_eval __interactive_plotode(&giac::_interactive_plotode,_interactive_plotode_s);
  unary_function_ptr at_interactive_plotode (&__interactive_plotode);

  const string _interactive_odeplot_s("interactive_odeplot");
  unary_function_eval __interactive_odeplot(&giac::_interactive_plotode,_interactive_odeplot_s);
  unary_function_ptr at_interactive_odeplot (&__interactive_odeplot);

  vecteur unarchive_VECT(istream & is,const vecteur & l){
    vecteur v;
    int taille;
    is >> taille;
    v.reserve(taille);
    for (int i=0;i<taille;++i)
      v.push_back(unarchive(is,l));
    return v;
  }

  gen_map unarchive_MAP(istream & is,const vecteur & l){
    gen_map m(ptr_fun(islesscomplexthanf));
    int taille;
    is >> taille;
    for (int i=0;i<taille;++i){
      gen f=unarchive(is,l);
      gen s=unarchive(is,l);
      m[f]=s;
    }
    return m;
  }

  gen unarchive_FUNC(istream & is,const vecteur & l){
    int op;
    is >> op;
    unary_function_ptr u(at_plus);
    if (op){
      if (op<0){
	string s;
	is >> s;
	// read function from lexer_functions
	sym_tab::const_iterator i = lexer_functions.find(s);
	if (i==lexer_functions.end()) // should be error
	  return undef;
	return i->second;
      }
      u=archive_function_tab[op-1];
    }
    else {
      // read using the parser
      string s;
      is >> s;
      gen e;
      try {
	e=gen(s);
      }
      catch (std::runtime_error & err){
	e=string2gen(s,false);
      }
      //cerr << s << " " <<e.type << endl;
      if (e.type!=_FUNC){
	if ( (e.type!=_SYMB) ){
	  cerr << "Unarchive error: "+e.print() << endl;
	  return e;
	}
	if (e._SYMBptr->sommet.ptr->printsommet==printastifunction)
	  return e._SYMBptr->sommet;
	return e._SYMBptr->feuille;
      }
      u=*e._FUNCptr;
    }
    return u;
  }

  symbolic unarchive_SYMB(istream & is,const vecteur & l){
    gen e=unarchive_FUNC(is,l);
    gen f=unarchive(is,l);
    if (e.type==_FUNC)
      return symbolic(*e._FUNCptr,f);
    return symb_of(e,f);
  }

  gen unarchivedefault(istream & is,const vecteur & l){
    string s;
    is >> s;
    return gen(s,l);
  }

  gen unarchiveidnt(istream & is,const vecteur & l,int t){
    char c;
    is >> c;
    string s;
    s +=c;
    for (int i=1;i<t;++i){
      is.get(c);
      s +=c;
    }
    gen res=gen(s,l);
    if (res.type!=_IDNT && s[0]=='_')
      res=find_or_make_symbol(s);
    if (res.type!=_IDNT){
      string s1 ="~"+s;
      res=gen(s1,l);
      if (res.type!=_IDNT)
	res=find_or_make_symbol(s);
    }
    return res;
  }

  gen unarchivestring(istream & is){
    int l;
    is >> l;
    string s;
    char c;
    for (;;){
      is.get(c);
      if (c=='|')
	break;
    }
    for (int i=0;i<l;++i){
      is.get(c);
      s +=c;
    }
    return string2gen(s,false);
  }

  gen unarchivereal(istream & is){
    unsigned int prec;
    is >> prec;
    string s;
    is >> s;
    if (!prec)
      return s;
    return read_binary(s,prec);
  }

  gen unarchive(istream & is,const vecteur & l){
    if (is.eof())
      settypeerr("End of stream");
    int type,val;
    double d;
    char ch;
    gen a,b;
    is >> type;
    switch (type){
    case _INT_:
      is >> val >> type;
      a=val;
      a.subtype=type;
      return a;
    case _DOUBLE_:
      is.get(ch);
      is.read((char *)&d,sizeof(double));
      return d;
    case _CPLX:
      a=unarchive(is,l);
      b=unarchive(is,l);
      return a+cst_i*b;
    case _REAL:
      a=unarchivereal(is);
      return a;
    case _FRAC:
      a=unarchive(is,l);
      b=unarchive(is,l);
      return fraction(a,b);
    case _VECT:
      is >> type;
      return gen(unarchive_VECT(is,l),type);
    case _SYMB:
      is >> type;
      a=unarchive_SYMB(is,l);
      a.subtype=type;
      return a;
    case _IDNT:
      is >> type;
      return unarchiveidnt(is,l,type);
    case _FUNC:
      is >> type;
      a=unarchive_FUNC(is,l);
      if (a.type==_FUNC)
	return gen(*a._FUNCptr,type);
      return a;
    case _STRNG:
      return unarchivestring(is);
    case _MOD:
      a=unarchive(is,l);
      b=unarchive(is,l);
      return makemodquoted(a,b);
    case _MAP:
      return unarchive_MAP(is,l);
    case _POINTER_:
      is >> type;
      if (type==_FL_WIDGET_POINTER && fl_widget_unarchive_function)
	return fl_widget_unarchive_function(is,l);
    default:
      return unarchivedefault(is,l);
    }
  }

  ostream & archive_FUNC(ostream & os,const unary_function_ptr & u){
    if (has_special_syntax(u.ptr->s)){
      os << -1 << " " << u.ptr->s << " ";
      return os;
    }
    int i=equalposcomp(archive_function_tab,u);
    os << i << " ";
    if (!i){
      string s;
      if (u.ptr->printsommet) 
	s=(u.ptr->printsommet(zero,u.ptr->s));
      int l=s.size();
      if ( (l>4) && (s.substr(l-3,3)=="(0)") )
	os << s.substr(0,l-3) << " ";
      else
	os << gen(u) << " ";
    }
    return os;
  }

  ostream & archive_SYMB(ostream & os,const symbolic & s){
    unary_function_ptr u=s.sommet;
    gen f=s.feuille;
    archive_FUNC(os,u);
    archive(os,f);
    return os;
  }

  ostream & archive_VECT(ostream & os,const vecteur & v){
    const_iterateur it=v.begin(),itend=v.end();
    os << itend-it << " ";
    for (;it!=itend;++it)
      archive(os,*it);
    return os;
  }

  ostream & archive_MAP(ostream & os,const gen_map & v){
    gen_map::const_iterator it=v.begin(),itend=v.end();
    int i=0;
    for (;it!=itend;++it) 
      ++i;
    os << i << " ";
    for (it=v.begin();it!=itend;++it){
      archive(os,it->first);
      os << " ";
      archive(os,it->second);
    }
    return os;
  }

  ostream & archive_IDNT(ostream & os,const identificateur & i){
    string s=i.print();
    return os << s.size() << " " << s << " ";
  }

  ostream & archive(ostream & os,const gen & e){
    switch (e.type){
    case _INT_:
      return os << e.type << " " << e.val << " " << e.subtype << endl;
    case _DOUBLE_:
      if (my_isinf(e._DOUBLE_val) || my_isnan(e._DOUBLE_val) )
	return archive(os,gen(e.print()));
      os << e.type << " ";
      os.write((char *)&e._DOUBLE_val,sizeof(double));
      return os << endl;
    case _CPLX:
      os << e.type << " ";
      archive(os,*e._CPLXptr);
      return archive(os,*(e._CPLXptr+1));
    case _REAL:
      os << e.type << " ";
#ifdef HAVE_LIBMPFR
      os << mpfr_get_prec(e._REALptr->inf) << " ";
      return os << print_binary(*e._REALptr) << endl;
#else
      os << 0 << " ";
      return os << e.print() << endl ;
#endif
    case _VECT:
      os << e.type << " " << e.subtype << " " ;
      return archive_VECT(os,*e._VECTptr);
    case _SYMB:
      os << e.type << " "<< e.subtype << " ";
      return archive_SYMB(os,*e._SYMBptr);
    case _IDNT:
      os << e.type << " ";
      return archive_IDNT(os,*e._IDNTptr);
    case _FUNC:
      os << e.type << " " << e.subtype << " ";
      return archive_FUNC(os,*e._FUNCptr);
    case _FRAC:
      os << e.type << " ";
      archive(os,e._FRACptr->num);
      return archive(os,e._FRACptr->den);
    case _STRNG:
      return os << e.type << " " << e._STRNGptr->size() << " |" << *e._STRNGptr << " ";
    case _MOD:
      os << e.type << " ";
      archive(os,*e._MODptr); 
      os << " ";
      archive(os,*(e._MODptr+1));
      return os << " ";
    case _MAP:
      os << e.type << " ";
      archive_MAP(os,*e._MAPptr);
      return os << " ";
    case _POINTER_:
      if (e.subtype==_FL_WIDGET_POINTER && fl_widget_archive_function)
	return fl_widget_archive_function(os,e._POINTER_val);
      else
	return archive(os,string2gen("Done",false));
    default:
      return os << e.type << " " << e.print() << endl;
    }
  }

  vecteur giac_current_status(GIAC_CONTEXT){
    // cas and geo config
    vecteur res(1,symbolic(at_cas_setup,cas_setup(contextptr)));
    res.push_back(xyztrange(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,global_window_xmin,global_window_xmax,global_window_ymin,global_window_ymax,show_axes,class_minimum,class_size,gnuplot_hidden3d));
    // session
    res.push_back(history_in);
    res.push_back(history_out);
    // user variables
    if (contextptr && contextptr->tabptr){
      sym_tab::const_iterator jt=contextptr->tabptr->begin(),jtend=contextptr->tabptr->end();
      for (;jt!=jtend;++jt){
	gen a=jt->second;
	gen b=identificateur(jt->first);
	res.push_back(symb_sto(a,b));
      }
    }
    else {
      sym_tab::const_iterator it=syms.begin(),itend=syms.end();
      for (;it!=itend;++it){
	gen id=it->second;
	if (id.type==_IDNT && id._IDNTptr->value)
	  res.push_back(symb_sto(*id._IDNTptr->value,id));
      }
    }
    return res;
  }

  // Archive cas, geo setup, history_in, history_out, all variable values
  gen archive_session(const string & s,GIAC_CONTEXT){
    ofstream os(s.c_str());
    os << "giac archive"<< endl;
    gen g(giac_current_status(contextptr));
    archive(os,g);
    return g;
  }

  // Unarchive a session from archive named s
  // Replace one level of history by replace
  // Return 0 if not successfull, or a vector of remaining gen in the archive
  gen unarchive_session(const string & s,int level, const gen & replace,GIAC_CONTEXT){
    ifstream is(s.c_str());
    if (!is)
      return false;
    char buf[BUFFER_SIZE+1];
    is.getline(buf,BUFFER_SIZE,'\n');
    if (string(buf)!="giac archive")
      return false;
    gen g=unarchive(is,list_one_letter__IDNT);
    if (!is)
      return false;
    int l;
    if (g.type!=_VECT || (l=g._VECTptr->size())<4)
      return false;
    vecteur & v=*g._VECTptr;
    if (v[2].type!=_VECT || v[3].type!=_VECT || v[2]._VECTptr->size()!=v[3]._VECTptr->size())
      return false;
    history_in=*v[2]._VECTptr;
    history_out=*v[3]._VECTptr;
#ifndef GNUWINCE
    protecteval(v[0],DEFAULT_EVAL_LEVEL,contextptr); 
    protecteval(v[1],DEFAULT_EVAL_LEVEL,contextptr); 
#endif
    // restore variables
    for (int i=4;i<l;++i)
      protecteval(v[i],DEFAULT_EVAL_LEVEL,contextptr); 
    // eval replace if level>=0
    if (level<0 || level>=l){
      history_in.push_back(replace);
      history_out.push_back(protecteval(replace,DEFAULT_EVAL_LEVEL,contextptr)); 
    }
    else {
      history_in[level]=replace;
      for (int i=level;i<l;++i)
	history_out[i]=protecteval(history_in[i],DEFAULT_EVAL_LEVEL,contextptr); 
    }
    vecteur res;
    while (!is.eof()){
      res.push_back(unarchive(is,list_one_letter__IDNT));
    }
    return res;
  }

  gen _archive(const gen & args,GIAC_CONTEXT){
    check_secure();
    if (args.type==_STRNG){ // archive session state
      return archive_session(*args._STRNGptr,contextptr);
    }
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
      setsizeerr();
    gen a=args._VECTptr->front();
    gen b=args._VECTptr->back();
    if (a.type!=_STRNG)
      setsizeerr();
    ofstream os(a._STRNGptr->c_str());
    archive(os,b);
    return b;
  }
  const string _archive_s("archive");
  unary_function_eval __archive(&giac::_archive,_archive_s);
  unary_function_ptr at_archive (&__archive);

  gen _unarchive(const gen & args){
    if (args.type!=_STRNG)
      setsizeerr();
    ifstream is(args._STRNGptr->c_str());
    return unarchive(is,list_one_letter__IDNT);
  }
  const string _unarchive_s("unarchive");
  unary_function_unary __unarchive(&giac::_unarchive,_unarchive_s);
  unary_function_ptr at_unarchive (&__unarchive);

  void geo_setup(const vecteur & w){
    if (w.size()<12)
      setsizeerr();
    if (w.size()>12) {
      gen g=w[12];
      g=_floor(g,0);
      if (g.type!=_INT_)
	setsizeerr();
      show_axes=g.val;
    }
    if (w.size()>15) {
      gen g=w[15];
      g=_floor(g,0);
      if (g.type!=_INT_)
	setsizeerr();
      int i=g.val;
      gnuplot_hidden3d=i %2;
      i =i/2;
      gnuplot_pm3d=i%2;
    }
    vecteur v=*evalf_double(w)._VECTptr;
    if (v.size()>14) {
      if (v[13].type!=_DOUBLE_ || v[14].type!=_DOUBLE_)
	setsizeerr();
      class_minimum=v[13]._DOUBLE_val;
      class_size=v[14]._DOUBLE_val;
    }
    for (int i=0;i<12;++i){
      if (v[i].type!=_DOUBLE_)
	setsizeerr();
    }
    gnuplot_xmin=v[0]._DOUBLE_val;
    gnuplot_xmax=v[1]._DOUBLE_val;
    gnuplot_ymin=v[2]._DOUBLE_val;
    gnuplot_ymax=v[3]._DOUBLE_val;
    gnuplot_zmin=v[4]._DOUBLE_val;
    gnuplot_zmax=v[5]._DOUBLE_val;
    gnuplot_tmin=v[6]._DOUBLE_val;
    gnuplot_tmax=v[7]._DOUBLE_val;
    global_window_xmin=v[8]._DOUBLE_val;
    global_window_xmax=v[9]._DOUBLE_val;
    global_window_ymin=v[10]._DOUBLE_val;
    global_window_ymax=v[11]._DOUBLE_val;
  }

  gen xyztrange(double xmin,double xmax,double ymin,double ymax,double zmin,double zmax,double tmin,double tmax,double wxmin,double wxmax,double wymin, double wymax, int axes,double class_min,double class_size,bool gnuplot_hidden3d){
    vecteur v;
    v.push_back(xmin);
    v.push_back(xmax);
    v.push_back(ymin);
    v.push_back(ymax);
    v.push_back(zmin);
    v.push_back(zmax);
    v.push_back(tmin);
    v.push_back(tmax);
    v.push_back(wxmin);
    v.push_back(wxmax);
    v.push_back(wymin);
    v.push_back(wymax);
    v.push_back(axes);
    v.push_back(class_min);
    v.push_back(class_size);
    v.push_back(int(gnuplot_hidden3d+2*gnuplot_pm3d));
    return symbolic(at_xyztrange,v);
  }
  gen _xyztrange(const gen & args){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<12) )
      setsizeerr();
    vecteur & w=*args._VECTptr;
    geo_setup(w);
    if (!child_id){
      _signal(symbolic(at_quote,symbolic(at_xyztrange,w)));
    }
    return args;
  }
  const string _xyztrange_s("xyztrange");
  unary_function_unary __xyztrange(&giac::_xyztrange,_xyztrange_s);
  unary_function_ptr at_xyztrange (&__xyztrange);

  gen _switch_axes(const gen & args){
    if (args.type==_INT_)
      show_axes=args.val;
    else {
      if (show_axes)
	show_axes=false;
      else
	show_axes=true;
    }
    return eval(xyztrange(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_tmin,gnuplot_tmax,global_window_xmin,global_window_xmax,global_window_ymin,global_window_ymax,show_axes,class_minimum,class_size,gnuplot_hidden3d+2*gnuplot_pm3d));
  }
  const string _switch_axes_s("switch_axes");
  unary_function_unary __switch_axes(&giac::_switch_axes,_switch_axes_s);
  unary_function_ptr at_switch_axes (&__switch_axes);

  gen plotseq(const gen& f,const gen&x,double x0,int niter,const context * contextptr){
    vecteur res(2*niter+1);
    res[0]=x0;
    int j=1;
    gen newx0;
    double x1;
    for (int i=0;i<niter;++i){
      newx0=evalf_double(subst(f,x,x0),DEFAULT_EVAL_LEVEL,contextptr);
      if (newx0.type!=_DOUBLE_)
	setsizeerr("Bad iteration");
      x1=newx0._DOUBLE_val;
      res[j]=gen(x0,x1);
      ++j;
      x0=x1;
      res[j]=gen(x0,x0);
      ++j;
    }
    gen g=_plotfunc(gen(makevecteur(f,x),_SEQ__VECT),contextptr);
    g._VECTptr->push_back(symb_pnt(gen(makevecteur(gen(gnuplot_xmin,gnuplot_xmin),gen(gnuplot_xmax,gnuplot_xmax)),_LINE__VECT),default_color(contextptr),contextptr));
    g._VECTptr->push_back(symb_pnt(gen(res,_LINE__VECT),FL_GREEN,contextptr));
    return g;
  }
  // args=[expr,[var=]x0[,niter]]
  gen _plotseq(const gen & args,const context * contextptr){
    if (args.type!=_VECT)
      return symbolic(at_plotseq,args);
    vecteur & v=*args._VECTptr;
    int l=v.size();
    int niter=30;
    double x0d;
    gen x,x0;
    if (l>3){ // expr,var,x0,niter
      x=v[1];
      x0=v[2];
      if (v[3].type==_INT_)
	niter=absint(v[3].val);
      else
	setsizeerr("Bad iteration number");
    }
    else {
      if (l>2){
	if (v[2].type==_INT_)
	  niter=absint(v[2].val);
	else
	  setsizeerr("Bad iteration number");
      }
      if ( (v[1].type==_SYMB) && (v[1]._SYMBptr->sommet==at_equal) ){
	vecteur & w=*v[1]._SYMBptr->feuille._VECTptr;
	x=w[0];
	x0=w[1];
      }
      else {
	x=vx_var;
	x0=v[1];
      }
    }
    x0=evalf_double(x0,DEFAULT_EVAL_LEVEL,contextptr);
    if (x0.type!=_DOUBLE_)
      setsizeerr("Non numeric initial value");
    x0d=x0._DOUBLE_val;
    return plotseq(v[0],x,x0d,niter,contextptr);
  }
  const string _plotseq_s("plotseq");
  unary_function_eval __plotseq(&giac::_plotseq,_plotseq_s);
  unary_function_ptr at_plotseq (&__plotseq);

  const string _seqplot_s("seqplot");
  unary_function_eval __seqplot(&giac::_plotseq,_seqplot_s);
  unary_function_ptr at_seqplot (&__seqplot);

  double l2norm(double x,double y){
    return std::sqrt(x*x+y*y);
  }

  gen plotimplicit(const gen& f_orig,const gen&x_orig,const gen & y_orig,double eps,const context * contextptr){
    gen x,y;
    double xmin,xmax,ymin,ymax;
    readrange(x_orig,gnuplot_xmin,gnuplot_xmax,x,xmin,xmax);
    readrange(y_orig,gnuplot_ymin,gnuplot_ymax,y,ymin,ymax);
    if ( (x.type!=_IDNT) || (y.type!=_IDNT) )
      setsizeerr("Variables must be free");
    // make a lattice between gnuplot_xmin/gnuplot_xmax and ymin/ymax
    // find zeros of f inside each square and follow the branches
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    _Pictsize(0);
    int horizontal_pixels=plot_instructionsw-LEGENDE_SIZE,vertical_pixels=plot_instructionsh;
    int implicit_pixels=gnuplot_pixels_per_eval*2;
    int nxstep=(horizontal_pixels-1)/implicit_pixels;
    int nystep=(vertical_pixels-1)/implicit_pixels;
    double xstep=implicit_pixels*(xmax-xmin)/(horizontal_pixels-1);
    double ystep=implicit_pixels*(ymax-ymin)/(vertical_pixels-1);
    identificateur xloc(" xloc"),yloc(" yloc");
    vecteur xy(makevecteur(x,y)),locvar(makevecteur(xloc,yloc));
    gen f=quotesubst(f_orig,xy,locvar).evalf();
    gen dfx=derive(f_orig,x),dfy=derive(f_orig,y);
    vecteur localvar(makevecteur(xloc,yloc));
    context * newcontextptr=(context *) contextptr;
    int protect=bind(makevecteur(xmin,ymin),localvar,newcontextptr);
    bool visited[nystep+2][nxstep+2];
    double fxy[nystep+1][nxstep+1],dfxorig[nystep+1][nxstep+1],dfyorig[nystep+1][nxstep+1],dfxyorig_abs[nystep+1][nxstep+1];
    double xorig[nystep+1][nxstep+1],yorig[nystep+1][nxstep+1];
    gen gtmp;
    // initialize each cell to non visited
    yloc.localvalue->back()._DOUBLE_val = ymin ;
    for (int i=0;i<=nystep+1;++i){
      for (int j=0;j<=nxstep+1;++j)
	visited[i][j]=false;
    }
    for (int i=0;i<=nystep;++i){
      xloc.localvalue->back()._DOUBLE_val = xmin;
      for (int j=0;j<=nxstep;++j){
	gtmp=f.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
	if (gtmp.type==_DOUBLE_)
	  fxy[i][j]=gtmp._DOUBLE_val;
	else
	  fxy[i][j]=0;
	xloc.localvalue->back()._DOUBLE_val += xstep;
      }
      yloc.localvalue->back()._DOUBLE_val += ystep;
    }
    leave(protect,localvar,newcontextptr);
    double xx,yy=ymin,tmp,xcurrent,ycurrent;
    bool pathfound;
    vecteur res;
    for (int i=1;i<=nystep;++i){
      if (debug_infolevel)
	cout << "// Implicitplot row " << i << endl;
      yy += ystep;
      xx = xmin;
      for (int j=1;j<=nxstep;++j){
	xx+=xstep;
	// If cell has been visited -> done
	if (!j || visited[i][j] || ((visited[i][j-1] || visited[i][j+1]) && (visited[i+1][j]||visited[i-1][j]) ))
	  continue;
	// now look for annulation in the cell
	tmp=fxy[i][j-1]*fxy[i][j];
	pathfound=false;
	if ( tmp<0) {
	  pathfound=true;
	  // find an horizontal solution
	  xcurrent=xx;
	  ycurrent=yy;
	  gen fx=subst(f_orig,y,ycurrent);
	  gen sol=_fsolve(gen(makevecteur(fx,x,makevecteur(xcurrent-xstep,xcurrent),_BISECTION_SOLVER,100*eps),_SEQ__VECT),contextptr);
	  xcurrent=(sol[0]+sol[1])._DOUBLE_val/2;
	}
	else {
	  tmp=fxy[i-1][j]*fxy[i][j];
	  if (tmp<0) {
	    pathfound=true;
	    // find a vertical solution
	    xcurrent=xx;
	    ycurrent=yy;
	    gen fy=subst(f_orig,x,xcurrent);
	    gen sol=_fsolve(gen(makevecteur(fy,y,makevecteur(ycurrent-ystep,ycurrent),_BISECTION_SOLVER,100*eps),_SEQ__VECT),contextptr);
	    ycurrent=(sol[0]+sol[1])._DOUBLE_val/2;
	  }
	}
	if (!pathfound)
	  continue;
	// Annulation found, let's add a path
	int iorig=int((ycurrent-ymin)/ystep);
	int jorig=int((xcurrent-xmin)/xstep);
	if (!iorig || !jorig || visited[iorig][jorig] 
	    || ((visited[iorig][jorig-1] || visited[iorig][jorig+1]) && (visited[iorig+1][jorig]||visited[iorig-1][jorig]) )
	    )
	  continue;
	vecteur chemin(1,gen(xcurrent,ycurrent));
	visited[iorig][jorig]=true;
	int icur,jcur,oldi=iorig,oldj=jorig;
	xorig[iorig][jorig]=xcurrent;
	yorig[iorig][jorig]=ycurrent;
	gtmp=evalf_double(subst(dfx,xy,makevecteur(xcurrent,ycurrent)),DEFAULT_EVAL_LEVEL,contextptr);
	if (gtmp.type==_DOUBLE_)
	  dfxorig[iorig][jorig]=gtmp._DOUBLE_val;
	else
	  dfxorig[iorig][jorig]=(fxy[oldi][oldj]-fxy[oldi][oldj-1])/xstep;
	gtmp=evalf_double(subst(dfy,xy,makevecteur(xcurrent,ycurrent)),DEFAULT_EVAL_LEVEL,contextptr);
	if (gtmp.type==_DOUBLE_)
	  dfyorig[iorig][jorig]=gtmp._DOUBLE_val;
	else
	  dfyorig[iorig][jorig]=(fxy[oldi][oldj]-fxy[oldi-1][oldj])/ystep;
	dfxyorig_abs[iorig][jorig]=l2norm(dfxorig[iorig][jorig],dfyorig[iorig][jorig]);
	int sign;
	if ( (dfyorig[iorig][jorig]<-100*eps*dfxyorig_abs[iorig][jorig]) ||
	     ((dfyorig[iorig][jorig]<100*eps*dfxyorig_abs[iorig][jorig])&&
	      (dfxorig[iorig][jorig]>0)) )
	  sign=-1;
	else
	  sign=1;
	for (int count=0;count<nxstep*nystep;){
	  vecteur xycurrent(makevecteur(xcurrent,ycurrent));
	  double dfxcurrent,dfycurrent;
	  gtmp=evalf_double(subst(dfx,xy,xycurrent),DEFAULT_EVAL_LEVEL,contextptr);
	  bool use_newton=true;
	  // bool use_newton=false; // FIXME?? Newton does not seem to work
	  if (gtmp.type==_DOUBLE_)
	    dfxcurrent=gtmp._DOUBLE_val;
	  else {
	    use_newton=false;
	    dfxcurrent=(fxy[oldi][oldj]-fxy[oldi][oldj-1])/xstep;
	  }
	  gtmp=evalf_double(subst(dfy,xy,xycurrent),DEFAULT_EVAL_LEVEL,contextptr);
	  if (gtmp.type==_DOUBLE_)
	    dfycurrent=gtmp._DOUBLE_val;
	  else {
	    dfycurrent=(fxy[oldi][oldj]-fxy[oldi-1][oldj])/ystep;
	    use_newton=false;
	  }
	  if (sign==-1){
	    dfxcurrent=-dfxcurrent;
	    dfycurrent=-dfycurrent;
	  }
	  // (dfxcurrent,dfycurrent) is normal to the path
	  // If it's near 0 we are near a singular point
	  // Otherwise go to the next cell, and end if at original i,j
	  // or at the border
	  gen direction(dfycurrent,-dfxcurrent);
	  double dfxycurrent_abs=l2norm(dfxcurrent,dfycurrent);
	  if (dfxycurrent_abs<0.1)
	    break;
	  direction=direction/dfxycurrent_abs;
	  double deltax=evalf_double(re(direction,contextptr),DEFAULT_EVAL_LEVEL,contextptr)._DOUBLE_val,deltay=evalf_double(im(direction,contextptr),DEFAULT_EVAL_LEVEL,contextptr)._DOUBLE_val;
	  double thestep=ystep/2;
	  if (xstep<ystep)
	    thestep=xstep/2;
	  xcurrent += thestep*deltax;
	  ycurrent += thestep*deltay;
	  // Test for mostly horizontal tangeant
	  if (fabs(deltay)<fabs(deltax)){
	    gen fx=subst(f_orig,x,xcurrent);
	    if (use_newton){
	      ycurrent=_fsolve(gen(makevecteur(fx,y,ycurrent,_NEWTON_SOLVER),_SEQ__VECT),contextptr)._DOUBLE_val;
	    }
	    else {
	      gen sol=_fsolve(gen(makevecteur(fx,y,makevecteur(ycurrent-ystep,ycurrent+ystep),_BISECTION_SOLVER,100*eps),_SEQ__VECT),contextptr);
	      ycurrent=(sol[0]+sol[1])._DOUBLE_val/2;
	    }
	  }
	  else {
	    // recompute solution
	    gen fy=subst(f_orig,y,ycurrent);
	    if (use_newton){
	      xcurrent=_fsolve(gen(makevecteur(fy,x,xcurrent,_NEWTON_SOLVER),_SEQ__VECT),contextptr)._DOUBLE_val;
	    }
	    else {
	      gen sol=_fsolve(gen(makevecteur(fy,x,makevecteur(xcurrent-xstep,xcurrent+xstep),_BISECTION_SOLVER,100*eps),_SEQ__VECT),contextptr);
	      xcurrent=(sol[0]+sol[1])._DOUBLE_val/2;
	    }
	  }
	  chemin.push_back(gen(xcurrent,ycurrent));
	  // check cell
	  icur=int((ycurrent-ymin)/ystep);
	  jcur=int((xcurrent-xmin)/xstep);
	  if ((icur<=0) || (icur>nystep) || (jcur<=0) || (jcur>nxstep) )
	    break;
	  if ( (icur==oldi) && (jcur==oldj) ){
	    ++count;
	    continue;
	  }
	  if (visited[icur][jcur]){  
	    if (0.1*dfxyorig_abs[icur][jcur]*dfxycurrent_abs>fabs(dfxcurrent*dfyorig[icur][jcur]-dfycurrent*dfxorig[icur][jcur])){
	      // join to this point
	      chemin.push_back(gen(xorig[icur][jcur],yorig[icur][jcur]));
	      break;
	    }
	  }
	  else {
	    visited[icur][jcur]=true;
	    dfxorig[icur][jcur]=dfxcurrent;
	    dfyorig[icur][jcur]=dfycurrent;
	    xorig[icur][jcur]=xcurrent;
	    yorig[icur][jcur]=ycurrent;
	    dfxyorig_abs[icur][jcur]=dfxycurrent_abs;
	  }
	  if (debug_infolevel)
	    cout << "Implicitplot " << icur << " " << jcur << endl;	  
	  oldi=icur;
	  oldj=jcur;
	}
	res.push_back(symb_pnt(gen(chemin,_GROUP__VECT),default_color(contextptr),contextptr));
      }
    }
#ifndef WIN32
    if (child_id)
      plot_instructions.push_back(res);
    io_graph(old_iograph,contextptr);
#endif // WIN32
    return gen(res,_SEQ__VECT);
    return zero;
  }
  gen _plotimplicit(const gen & args,const context * contextptr){
    if (args.type!=_VECT)
      return plotimplicit(remove_equal(args),vx_var,y__IDNT_e,epsilon(contextptr),contextptr);
    // vecteur v(plotpreprocess(args));
    vecteur v(*args._VECTptr);
    if ( (v.size()==2) && (v[1].type==_VECT) && (v[1]._VECTptr->size()==2) )
      return plotimplicit(remove_equal(v[0]),v[1]._VECTptr->front(),v[1]._VECTptr->back(),epsilon(contextptr),contextptr);
    if (v.size()!=3)
      setsizeerr();
    return plotimplicit(remove_equal(v[0]),v[1],v[2],epsilon(contextptr),contextptr);
  }
  const string _plotimplicit_s("plotimplicit");
  unary_function_eval __plotimplicit(&giac::_plotimplicit,_plotimplicit_s);
  unary_function_ptr at_plotimplicit (&__plotimplicit);

  const string _implicitplot_s("implicitplot");
  unary_function_eval __implicitplot(&giac::_plotimplicit,_implicitplot_s);
  unary_function_ptr at_implicitplot (&__implicitplot);

  gen plotcontour(const gen& f_orig,const gen&x_orig,const gen & y_orig,const vecteur & v_orig,const context * contextptr){
    gen x,y;
    double xmin,xmax,ymin,ymax;
    readrange(x_orig,gnuplot_xmin,gnuplot_xmax,x,xmin,xmax);
    readrange(y_orig,gnuplot_ymin,gnuplot_ymax,y,ymin,ymax);
    if (x.type!=_IDNT || y.type!=_IDNT)
      setsizeerr("Variables must be free");
    gen tmp =evalf(v_orig);
    if (tmp.type!=_VECT)
      setsizeerr();
    vecteur v=*tmp._VECTptr;
    _Pictsize(0);
    int horizontal_pixels=plot_instructionsw-LEGENDE_SIZE,vertical_pixels=plot_instructionsh;
    int nxstep=(horizontal_pixels-1);
    int nystep=(vertical_pixels-1);
    double xstep=(xmax-xmin)/(horizontal_pixels-1);
    double ystep=(ymax-ymin)/(vertical_pixels-1);
    identificateur xloc(" xloc"),yloc(" yloc");
    vecteur xy(makevecteur(x,y)),locvar(makevecteur(xloc,yloc));
    gen f=quotesubst(f_orig,xy,locvar).evalf();
    vecteur localvar(makevecteur(xloc,yloc));
    context * newcontextptr = (context *) contextptr;
    int protect=bind(makevecteur(xmin,ymin),localvar,newcontextptr);
    int fxy[nystep+1][nxstep+1];
    gen gtmp;
    yloc.localvalue->back()._DOUBLE_val = ymax ;
    int vs=v.size();
    for (int i=0;i<=nystep;++i){
      xloc.localvalue->back()._DOUBLE_val = xmax;
      for (int j=0;j<=nxstep;++j){
	gtmp=f.evalf_double(DEFAULT_EVAL_LEVEL,newcontextptr);
	if (gtmp.type==_DOUBLE_){
	  double d=gtmp._DOUBLE_val;
	  int k=0;
	  for (;k<vs;++k){
	    if (d<v[k]._DOUBLE_val)
	      break;
	  }
	  fxy[i][j]=k;
	}
	else
	  fxy[i][j]=-1;
	xloc.localvalue->back()._DOUBLE_val -= xstep;
      }
      yloc.localvalue->back()._DOUBLE_val -= ystep;
    }
    leave(protect,localvar,newcontextptr);
    // now make the bitmap
    vecteur res(nystep+1);
    for (int i=0;i<=nystep;++i){
      gen gres;
      int prec=fxy[i][0],tmpf;
      for (int j=1;j<=nxstep;++j,gres=2*gres){
	tmpf=fxy[i][j];
	if ( tmpf!=prec || (i && tmpf!=fxy[i-1][j]))
	  gres=gres+1;
	prec=fxy[i][j];
      }
      res[i]=gres;
    }
    vecteur param(makevecteur(res,0,0,gnuplot_xmin,gnuplot_xmax));
    param.push_back(gnuplot_ymin);
    param.push_back(gnuplot_ymax);
    param.push_back(plot_instructionsw-LEGENDE_SIZE);
    param.push_back(plot_instructionsh);
    return symb_pnt(symbolic(at_bitmap,param),contextptr);
  }
  gen _plotcontour(const gen & args,const context * contextptr){
    if (args.type!=_VECT){
      vecteur sequence(11);
      for (int i=0;i<11;++i)
	sequence[i]=gnuplot_zmin+i*(gnuplot_zmax-gnuplot_zmin)/10;
      return plotcontour(args,vx_var,y__IDNT_e,sequence,contextptr);
    }  
    vecteur v(*args._VECTptr);
    if (v.size()==2 && (v[1].type==_VECT) && (v[1]._VECTptr->size()==2)){
      vecteur sequence(11);
      for (int i=0;i<11;++i)
	sequence[i]=gnuplot_zmin+i*(gnuplot_zmax-gnuplot_zmin)/10;
      v.push_back(sequence);
    } 
    if ( (v.size()==3) && (v[1].type==_VECT) && (v[1]._VECTptr->size()==2) && v[2].type==_VECT)
      return plotcontour(remove_equal(v[0]),v[1]._VECTptr->front(),v[1]._VECTptr->back(),*v[2]._VECTptr,contextptr);
    if (v.size()!=4 || v[3].type!=_VECT)
      setsizeerr();
    return plotcontour(remove_equal(v[0]),v[1],v[2],*v[3]._VECTptr,contextptr);
  }
  const string _plotcontour_s("plotcontour");
  unary_function_eval __plotcontour(&giac::_plotcontour,_plotcontour_s);
  unary_function_ptr at_plotcontour (&__plotcontour);

  const string _contourplot_s("contourplot");
  unary_function_eval __contourplot(&giac::_plotcontour,_contourplot_s);
  unary_function_ptr at_contourplot (&__contourplot);

  gen _bitmap(const gen & args,GIAC_CONTEXT){
    return symb_pnt(symbolic(at_bitmap,args),0,contextptr);
  }
  const string _bitmap_s("bitmap");
  unary_function_eval __bitmap(&giac::_bitmap,_bitmap_s);
  unary_function_ptr at_bitmap (&__bitmap);

  gen _papier_pointe(const gen & args,GIAC_CONTEXT){
    double deltax=1.0,deltay=1.0,angle=evalf_double(cst_pi/2)._DOUBLE_val;
    if (args.type==_VECT){
      vecteur & w=*args._VECTptr;
      int s=w.size();
      if (s>0){
	gen tmp=evalf_double(w[0]);
	if (tmp.type==_DOUBLE_)
	  deltax=fabs(tmp._DOUBLE_val);
      }
      if (s>1){
	gen tmp=evalf_double(w[1]);
	if (tmp.type==_DOUBLE_)
	  deltay=fabs(tmp._DOUBLE_val);
      }
      if (s>2){
	gen tmp=evalf_double(w[2]);
	if (tmp.type==_DOUBLE_){
	  angle=tmp._DOUBLE_val;
	  if (fabs(angle)<epsilon(contextptr))
	    setsizeerr();
	}
      }
    }
    vecteur res;
    double y=deltay*std::sin(angle);
    int jmin(int(gnuplot_ymin/y)),jmax(int(gnuplot_ymax/y));
    for (int j=jmin;j<=jmax+1;++j){
      double decal=j*std::cos(angle);
      decal=decal-deltax*std::floor(decal/deltax);
      for (int i=int(gnuplot_xmin/deltax)-1;i<gnuplot_xmax/deltax;++i){
	res.push_back(_point(makevecteur(i*deltax+decal,j*y),contextptr));
      }
    }
    return gen(res,_SEQ__VECT);
  }
  const string _papier_pointe_s("papier_pointe");
  unary_function_eval __papier_pointe(&giac::_papier_pointe,_papier_pointe_s);
  unary_function_ptr at_papier_pointe (&__papier_pointe,0,true);

  // inert function used to keep the attribute of a graphical object
  gen _plot_style(const gen & args){
    return symbolic(at_plot_style,args);
  }
  const string _plot_style_s("plot_style");
  unary_function_unary __plot_style(&giac::_plot_style,_plot_style_s);
  unary_function_ptr at_plot_style (&__plot_style);

  // Returns the current dimensions of the picture 
  // and adjust plot_instructionsw/h
  gen _Pictsize(const gen & args){
    gen res=__interactive.op(symbolic(at_Pictsize,args));
    if (res.type==_VECT && res._VECTptr->size()==2){
      plot_instructionsw=res._VECTptr->front().val;
      plot_instructionsh=res._VECTptr->back().val;
    }
    return res;
  }
  const string _Pictsize_s("Pictsize");
  unary_function_unary __Pictsize(&giac::_Pictsize,_Pictsize_s);
  unary_function_ptr at_Pictsize (&__Pictsize);

  gen _plot3d(const gen & g,const context * contextptr){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      return symbolic(at_plot3d,g);
    vecteur v=*g._VECTptr;
    if (v.size()<3)
      v.push_back(v__IDNT_e);
    if (v[0].type==_VECT){
      if (v[0]._VECTptr->size()!=3)
	setdimerr();
      double tmin,tmax,smin,smax;
      if (v[1].is_symb_of_sommet(at_interval)){
	if (!chk_double_interval(v[1],tmin,tmax) || !chk_double_interval(v[2],smin,smax))
	  setsizeerr();
	vecteur vars(makevecteur(s__IDNT_e,t__IDNT_e));
	return plotparam3d(v[0](vars,contextptr),vars,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_xmax,tmin,tmax,smin,smax,true,autoscale,default_color(contextptr),undef,vecteur(0),contextptr);
      }
      else {
	if (!readrange(v[1],gnuplot_tmin,gnuplot_tmax,v[1],tmin,tmax) || !readrange(v[2],gnuplot_tmin,gnuplot_tmax,v[2],smin,smax))
	  setsizeerr();
	return plotparam3d(v[0],makevecteur(v[1],v[2]),gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,gnuplot_zmin,gnuplot_xmax,tmin,tmax,smin,smax,true,autoscale,default_color(contextptr),undef,vecteur(0),contextptr);
      }
    }
    else {
      double xmin,xmax,ymin,ymax;
      if (v[1].is_symb_of_sommet(at_interval)){
	if (!chk_double_interval(v[1],xmin,xmax) || !chk_double_interval(v[2],ymin,ymax))
	  setsizeerr();
	vecteur vars(makevecteur(x__IDNT_e,y__IDNT_e));
	return plotfunc(v[0](vars,contextptr),vars,default_color(contextptr),true,xmin,xmax,ymin,ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_pixels_per_eval,contextptr);
      }
      else {
	if (!readrange(v[1],gnuplot_xmin,gnuplot_xmax,v[1],xmin,xmax) || !readrange(v[2],gnuplot_ymin,gnuplot_ymax,v[2],ymin,ymax))
	  setsizeerr();
	return plotfunc(v[0],makevecteur(v[1],v[2]),default_color(contextptr),true,xmin,xmax,ymin,ymax,gnuplot_zmin,gnuplot_zmax,gnuplot_pixels_per_eval,contextptr);
      }
    }
  }
  const string _plot3d_s("plot3d");
  unary_function_eval __plot3d(&_plot3d,_plot3d_s);
  unary_function_ptr at_plot3d (&__plot3d);

  // FIXME 394 should be T_LOGO, 340 T_RETURN
  //#define T_LOGO 394
  //#define T_RETURN 340
  gen _DrawInv(const gen & g,const context * contextptr){
    // return _symetrie(makevecteur(_droite(makevecteur(0,1+cst_i)),_plotfunc(g)));
    gen y(g),x(vx_var);
    if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()==2 ){
      vecteur & v=*g._VECTptr;
      y=v[0];
      x=v[1];
    }
    return _plotparam(gen(makevecteur(y+cst_i*x,x,gnuplot_xmin,gnuplot_xmax),_SEQ__VECT),contextptr);
  }
  const string _DrawInv_s("DrawInv");
  unary_function_eval __DrawInv(&_DrawInv,_DrawInv_s,&printastifunction);
  unary_function_ptr at_DrawInv (&__DrawInv,0,T_RETURN); 

  gen _Graph(const gen & g,GIAC_CONTEXT){
    return _plotfunc(g,contextptr);
    // FIXME: add a mode
    // vecteur v(gen2vecteur(g));
    
  }
  const string _Graph_s("Graph");
  unary_function_eval __Graph(&_Graph,_Graph_s,&printastifunction);
  unary_function_ptr at_Graph (&__Graph,0,T_RETURN);

  const string _DrawFunc_s("DrawFunc");
  unary_function_eval __DrawFunc(&_plotfunc,_DrawFunc_s,&printastifunction);
  unary_function_ptr at_DrawFunc (&__DrawFunc,0,T_RETURN);

  const string _DrawPol_s("DrawPol");
  unary_function_eval __DrawPol(&_plotpolar,_DrawPol_s,&printastifunction);
  unary_function_ptr at_DrawPol (&__DrawPol,0,T_RETURN);

  const string _DrawParm_s("DrawParm");
  unary_function_eval __DrawParm(&_plotparam,_DrawParm_s,&printastifunction);
  unary_function_ptr at_DrawParm (&__DrawParm,0,T_RETURN);

  gen _DrwCtour(const gen & g){
    // FIXME
    return undef;
  }
  const string _DrwCtour_s("DrwCtour");
  unary_function_eval __DrwCtour(&_plotcontour,_DrwCtour_s,&printastifunction);
  unary_function_ptr at_DrwCtour (&__DrwCtour,0,T_RETURN);

  logo_turtle vecteur2turtle(const vecteur & v){
    int s=v.size();
    if (s>=5 && v[0].type==_DOUBLE_ && v[1].type==_DOUBLE_ && v[2].type==_DOUBLE_ && v[3].type==_INT_ && v[4].type==_INT_ ){
      logo_turtle t;
      t.x=v[0]._DOUBLE_val;
      t.y=v[1]._DOUBLE_val;
      t.theta=v[2]._DOUBLE_val;
      int i=v[3].val;
      t.mark=i%2;
      i=i >> 1;
      t.visible=i%2;
      i=i >> 1;
      t.direct = i%2;
      i=i >> 1;
      t.turtle_length = i & 0xff;
      i=i >> 8;
      t.color = i;
      t.radius = v[4].val;
      if (s>5 && v[5].type==_STRNG)
	t.s=*v[5]._STRNGptr;
      else
	t.s="";
      return t;
    }
    setsizeerr();
  }

  int turtle_status(const logo_turtle & turtle){
    int status= (turtle.color << 11) | ( (turtle.turtle_length & 0xff) << 3) ;
    if (turtle.direct)
      status += 4;
    if (turtle.visible)
      status += 2;
    if (turtle.mark)
      status += 1;
    return status;
  }

  bool set_turtle_state(const vecteur & v){
    if (v.size()>=2 && v[0].type==_DOUBLE_ && v[1].type==_DOUBLE_){
      vecteur w(v);
      int s=w.size();
      if (s==2)
	w.push_back(turtle.theta);
      if (s<4)
	w.push_back(turtle_status(turtle));
      if (s<5)
	w.push_back(0);
      if (w[2].type==_DOUBLE_ && w[3].type==_INT_ && w[4].type==_INT_){
	turtle=vecteur2turtle(w);
	turtle_stack.push_back(turtle);
	return true;
      }
    }
    return false;
  }

  gen turtle2gen(const logo_turtle & turtle){
    return makevecteur(turtle.x,turtle.y,turtle.theta,turtle_status(turtle),turtle.radius,string2gen(turtle.s,false));
  }

  vecteur turtlevect2vecteur(const std::vector<logo_turtle> & v){
    vecteur res;
    vector<logo_turtle>::const_iterator it=v.begin(),itend=v.end();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(turtle2gen(*it));
    return res;
  }

  std::vector<logo_turtle> vecteur2turtlevect(const vecteur & v){
    std::vector<logo_turtle> res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (it->type==_VECT)
	res.push_back(vecteur2turtle(*it->_VECTptr));
    }
    return res;
  }

  gen turtle_state(){
    return turtle2gen(turtle);
  }

  gen update_turtle_state(bool clrstring){
    if (clrstring)
      turtle.s="";
    turtle.theta = turtle.theta - floor(turtle.theta/360)*360;
    turtle_stack.push_back(turtle);
    gen res=turtle_state();
    // update parent turtle state
    if (turtle_stack.size()==1)
      __interactive.op(symbolic(at_pnt,-1)); // clear parent stack
    else { // code turtle
      __interactive.op(symbolic(at_pnt,res));
    }
    return res;
  }

  gen _avance(const gen & g){
    // logo instruction
    double i;
    if (g.type!=_INT_){
      if (g.type==_VECT)
	i=turtle.turtle_length;
      else {
	gen g1=evalf_double(g);
	if (g1.type==_DOUBLE_)
	  i=g1._DOUBLE_val;
	else
	  setsizeerr();
      }
    }
    else
      i=g.val;
    turtle.x += i * std::cos(turtle.theta*deg2rad_d);
    turtle.y += i * std::sin(turtle.theta*deg2rad_d) ;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _avance_s("avance");
  unary_function_unary __avance(&_avance,_avance_s,&printastifunction);
  unary_function_ptr at_avance (&__avance,0,T_LOGO);

  std::string gen2string(const gen & g){
    if (g.type==_STRNG)
      return *g._STRNGptr;
    else
      return g.print();
  }
  gen _recule(const gen & g){
    // logo instruction
    if (g.type==_VECT)
      return _avance(-turtle.turtle_length);
    return _avance(-g);
  }
  const string _recule_s("recule");
  unary_function_unary __recule(&_recule,_recule_s,&printastifunction);
  unary_function_ptr at_recule (&__recule,0,T_LOGO);

  gen _position(const gen & g){
    // logo instruction
    if (g.type!=_VECT)
      return turtle_state();
    vecteur v = *g._VECTptr;
    int s=v.size();
    if (!s)
      return makevecteur(turtle.x,turtle.y);
    v[0]=evalf_double(v[0]);
    if (s>1)
      v[1]=evalf_double(v[1]);
    if (s>2)
      v[2]=evalf_double(v[2]);      
    if (set_turtle_state(v))
      return update_turtle_state(true);
    return zero;
  }
  const string _position_s("position");
  unary_function_unary __position(&_position,_position_s,&printastifunction);
  unary_function_ptr at_position (&__position,0,T_RETURN);

  gen _cap(const gen & g){
    // logo instruction
    gen gg=evalf_double(g);
    if (gg.type!=_DOUBLE_)
      return turtle.theta;
    turtle.theta=gg._DOUBLE_val;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _cap_s("cap");
  unary_function_unary __cap(&_cap,_cap_s,&printastifunction);
  unary_function_ptr at_cap (&__cap,0,T_LOGO);

  gen _tourne_droite(const gen & g){
    // logo instruction
    if (g.type!=_INT_){
      if (g.type==_VECT)
	turtle.theta -= 90;
      else {
	gen g1=evalf_double(g);
	if (g1.type==_DOUBLE_)
	  turtle.theta -= g1._DOUBLE_val;
	else
	  setsizeerr();
      }
    }
    else
      turtle.theta -= g.val;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _tourne_droite_s("tourne_droite");
  unary_function_unary __tourne_droite(&_tourne_droite,_tourne_droite_s,&printastifunction);
  unary_function_ptr at_tourne_droite (&__tourne_droite,0,T_LOGO);

  gen _tourne_gauche(const gen & g){
    // logo instruction
    if (g.type==_VECT){
      turtle.theta += 90;
      turtle.radius = 0;
      return update_turtle_state(true);
    }
    return _tourne_droite(-g);
  }
  const string _tourne_gauche_s("tourne_gauche");
  unary_function_unary __tourne_gauche(&_tourne_gauche,_tourne_gauche_s,&printastifunction);
  unary_function_ptr at_tourne_gauche (&__tourne_gauche,0,T_LOGO);

  gen _leve_crayon(const gen & g){
    // logo instruction
    turtle.mark = false;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _leve_crayon_s("leve_crayon");
  unary_function_unary __leve_crayon(&_leve_crayon,_leve_crayon_s,&printastifunction);
  unary_function_ptr at_leve_crayon (&__leve_crayon,0,T_LOGO);

  gen _baisse_crayon(const gen & g){
    // logo instruction
    turtle.mark = true;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _baisse_crayon_s("baisse_crayon");
  unary_function_unary __baisse_crayon(&_baisse_crayon,_baisse_crayon_s,&printastifunction);
  unary_function_ptr at_baisse_crayon (&__baisse_crayon,0,T_LOGO);

  gen _ecris(const gen & g){
    // logo instruction
    turtle.radius=14;
    if (g.type==_VECT){ 
      vecteur & v =*g._VECTptr;
      int s=v.size();
      if (s==2 && v[1].type==_INT_){
	turtle.radius=absint(v[1].val);
	turtle.s=gen2string(v.front());
	return update_turtle_state(false);
      }
      if (s==4 && v[1].type==_INT_ && v[2].type==_INT_ && v[3].type==_INT_){
	logo_turtle t=turtle;
	_leve_crayon(0);
	_position(makevecteur(v[2],v[3]));
	turtle.radius=absint(v[1].val);
	turtle.s=gen2string(v.front());
	update_turtle_state(false);
	turtle=t;
	return update_turtle_state(true);
      }
    }
    turtle.s=gen2string(g);
    return update_turtle_state(false);
  }
  const string _ecris_s("ecris");
  unary_function_unary __ecris(&_ecris,_ecris_s,&printastifunction);
  unary_function_ptr at_ecris (&__ecris,0,T_LOGO);

  gen _signe(const gen & g){
    // logo instruction
    return _ecris(makevecteur(g,20,10,10));
  }
  const string _signe_s("signe");
  unary_function_unary __signe(&_signe,_signe_s,&printastifunction);
  unary_function_ptr at_signe (&__signe,0,T_LOGO);

  const string _ramene_s("ramene");
  unary_function_eval __ramene(&_read,_ramene_s,&printastifunction);
  unary_function_ptr at_ramene (&__ramene,0,T_LOGO);

  const string _sauve_s("sauve");
  unary_function_unary __sauve(&_write,_sauve_s,&printastifunction);
  unary_function_ptr at_sauve (&__sauve,0,T_LOGO);

  gen _saute(const gen & g){
    _leve_crayon(0);
    _avance(g);
    return _baisse_crayon(0);
  }
  const string _saute_s("saute");
  unary_function_unary __saute(&_saute,_saute_s,&printastifunction);
  unary_function_ptr at_saute (&__saute,0,T_LOGO);

  gen _pas_de_cote(const gen & g){
    _leve_crayon(0);
    _tourne_droite(90);
    _avance(g);
    _tourne_droite(-90);
    return _baisse_crayon(0);
  }
  const string _pas_de_cote_s("pas_de_cote");
  unary_function_unary __pas_de_cote(&_pas_de_cote,_pas_de_cote_s,&printastifunction);
  unary_function_ptr at_pas_de_cote (&__pas_de_cote,0,T_LOGO);

  gen _cache_tortue(const gen & g){
    // logo instruction
    turtle.visible=false;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _cache_tortue_s("cache_tortue");
  unary_function_unary __cache_tortue(&_cache_tortue,_cache_tortue_s,&printastifunction);
  unary_function_ptr at_cache_tortue (&__cache_tortue,0,T_LOGO);

  gen _montre_tortue(const gen & g){
    // logo instruction
    turtle.visible=true;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _montre_tortue_s("montre_tortue");
  unary_function_unary __montre_tortue(&_montre_tortue,_montre_tortue_s,&printastifunction);
  unary_function_ptr at_montre_tortue (&__montre_tortue,0,T_LOGO);

  gen _debut_enregistrement(const gen & g){
    // logo instruction
    if (g.type!=_SYMB && g.type!=_IDNT)
      setsizeerr("Donner le nom de la procedure sane guillements");
    return g;
  }
  const string _debut_enregistrement_s("debut_enregistrement");
  unary_function_unary __debut_enregistrement(&_debut_enregistrement,_debut_enregistrement_s,&printastifunction);
  unary_function_ptr at_debut_enregistrement (&__debut_enregistrement,0,T_LOGO);

  gen _fin_enregistrement(const gen & g,GIAC_CONTEXT){
    // logo instruction
    if (g.type!=_STRNG)
      setsizeerr("Donner le nom de fichier");
    // Search for debut_enregistrement in history_out
    int s=history_in.size(),i;
    for (i=s-1;i>=0;--i){
      if (history_in[i].is_symb_of_sommet(at_debut_enregistrement))
	break;
    }
    if (i<0)
      setsizeerr("Instruction debut_enregistrement non trouvee");
    ofstream of(g._STRNGptr->c_str());
    of << history_in[i]._SYMBptr->feuille << "():={" << endl;
    for (++i;i<s-1;++i)
      of << " " << history_in[i] << ";" << endl;
    of << "}" << endl;
    return _read(g,contextptr);
  }
  const string _fin_enregistrement_s("fin_enregistrement");
  unary_function_eval __fin_enregistrement(&_fin_enregistrement,_fin_enregistrement_s,&printastifunction);
  unary_function_ptr at_fin_enregistrement (&__fin_enregistrement,0,T_LOGO);

  gen _repete(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      setsizeerr();
    // logo instruction
    vecteur v = *g._VECTptr;
    v[0]=eval(v[0],contextptr);
    if (v.front().type!=_INT_)
      settypeerr();
    gen prog=vecteur(v.begin()+1,v.end());
    int i=absint(v.front().val);
    gen res;
    for (int j=0;j<i;++j){
      res=eval(prog,contextptr);
    }
    return res;
  }
  const string _repete_s("repete");
  unary_function_eval __repete(&_repete,_repete_s,&printastifunction);
  unary_function_ptr at_repete (&__repete,_QUOTE_ARGUMENTS,T_RETURN);

  gen _crayon(const gen & g){
    // logo instruction
    if (g.type!=_INT_){
      gen res=turtle.color;
      res.subtype=_INT_COLOR;
      return res;
    }
    turtle.color=g.val;
    turtle.radius = 0;
    return update_turtle_state(true);
  }
  const string _crayon_s("crayon");
  unary_function_unary __crayon(&_crayon,_crayon_s,&printastifunction);
  unary_function_ptr at_crayon (&__crayon,0,T_LOGO);

  gen _efface(const gen & g){
    if (g.type==_INT_){
      _crayon(49);
      _recule(g);
      return _crayon(0);
    }
    // logo instruction
    turtle = logo_turtle();
    turtle_stack.clear();
    return update_turtle_state(true);
  }
  const string _efface_s("efface");
  unary_function_unary __efface(&_efface,_efface_s,&printastifunction);
  unary_function_ptr at_efface (&__efface,0,T_LOGO);

  gen _vers(const gen & g){
    // logo instruction
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      setsizeerr();
    gen x=evalf_double(g._VECTptr->front()),y=evalf_double(g._VECTptr->back());
    if (x.type!=_DOUBLE_ || y.type!=_DOUBLE_)
      setsizeerr();
    double xv=x._DOUBLE_val,yv=y._DOUBLE_val,xt=turtle.x,yt=turtle.y;
    double theta=atan2(yv-yt,xv-xt);
    return _cap(theta*180/M_PI);
  }
  const string _vers_s("vers");
  unary_function_unary __vers(&_vers,_vers_s,&printastifunction);
  unary_function_ptr at_vers (&__vers,0,T_LOGO);


  int find_radius(const gen & g,int & r,int & theta2,bool & direct){
    int radius;
    direct=true;
    theta2 = 360 ;
    // logo instruction
    if (g.type==_VECT && !g._VECTptr->empty()){
      vecteur & v = *g._VECTptr;
      if (v.size()<2)
	setdimerr();
      if (v[0].type==_DOUBLE_)
	r=int(v[0]._DOUBLE_val+0.5);
      else { 
	if (v[0].type==_INT_)
	  r = v[0].val;
	else setsizeerr();
      }
      if (r<0){
	r=-r;
	direct=false;
      }
      int theta1;
      if (v[1].type==_DOUBLE_)
	theta1=int(v[1]._DOUBLE_val+0.5);
      else { 
	if (v[1].type==_INT_)
	  theta1=v[1].val;
	else setsizeerr();
      }
      if (v.size()>=3){
	if (v[2].type==_DOUBLE_)
	  theta2 = int(v[2]._DOUBLE_val+0.5);
	else {
	  if (v[2].type==_INT_)
	    theta2 = v[2].val;
	  else setsizeerr();
	}
	if (theta2<0)
	  theta2 += 360;
	radius = min(r,512) | (min(theta1,360) << 9) | (min(theta2,360) << 18 );
      }
      else {// angle 1=0
	theta2 = theta1;
	if (theta2<0)
	  theta2 += 360;
	radius = min(r,512) | (min(theta2,360) << 18 );
      }
      return radius;
    }
    radius = 10;
    if (g.type==_INT_)
      radius= (r=g.val);
    if (g.type==_DOUBLE_)
      radius= (r=int(g._DOUBLE_val));
    if (radius<=0){
      radius = -radius;
      direct=false;
    }
    radius = min(radius,512 )+(360 << 18) ; // 2nd angle = 360 degrees
    return radius;
  }

  void turtle_move(int r,int theta2){
    double theta0;
    if (turtle.direct)
      theta0=turtle.theta-90;
    else {
      theta0=turtle.theta+90;
      theta2=-theta2;
    }
    turtle.x += r*(std::cos(M_PI/180*(theta2+theta0))-std::cos(M_PI/180*theta0));
    turtle.y += r*(std::sin(M_PI/180*(theta2+theta0))-std::sin(M_PI/180*theta0));;
    turtle.theta += theta2 ;
  }

  gen _rond(const gen & g){
    int r,theta2;
    turtle.radius=find_radius(g,r,theta2,turtle.direct);
    turtle_move(r,theta2);
    return update_turtle_state(true);
  }
  const string _rond_s("rond");
  unary_function_unary __rond(&_rond,_rond_s,&printastifunction);
  unary_function_ptr at_rond (&__rond,0,T_LOGO);

  gen _disque(const gen & g){
    int r,theta2;
    turtle.radius=find_radius(g,r,theta2,turtle.direct);
    turtle_move(r,theta2);
    turtle.radius += 1 << 27;
    return update_turtle_state(true);
  }
  const string _disque_s("disque");
  unary_function_unary __disque(&_disque,_disque_s,&printastifunction);
  unary_function_ptr at_disque (&__disque,0,T_LOGO);

  gen _polygone_rempli(const gen & g){
    if (g.type==_INT_){
      turtle.radius=-absint(g.val);
      if (turtle.radius<-1)
	return update_turtle_state(true);
    }
    setsizeerr("Argument entier >= 2");
  }
  const string _polygone_rempli_s("polygone_rempli");
  unary_function_unary __polygone_rempli(&_polygone_rempli,_polygone_rempli_s,&printastifunction);
  unary_function_ptr at_polygone_rempli (&__polygone_rempli,0,T_LOGO);

  gen _rectangle_plein(const gen & g){
    gen gx=g,gy=g;
    if (g.type==_VECT && g._VECTptr->size()==2){
      gx=g._VECTptr->front();
      gy=g._VECTptr->back();
    }
    for (int i=0;i<2;++i){
      _avance(gx);
      _tourne_droite(-90);
      _avance(gy);
      _tourne_droite(-90);
    }
    return _polygone_rempli(-8);
  }
  const string _rectangle_plein_s("rectangle_plein");
  unary_function_unary __rectangle_plein(&_rectangle_plein,_rectangle_plein_s,&printastifunction);
  unary_function_ptr at_rectangle_plein (&__rectangle_plein,0,T_LOGO);

  gen _triangle_plein(const gen & g){
    gen gx=g,gy=g,gtheta=60;
    if (g.type==_VECT && g._VECTptr->size()>=2){
      vecteur & v=*g._VECTptr;
      gx=v.front();
      gy=v[1];
      gtheta=90;
      if (v.size()>2)
	gtheta=v[2];
    }
    logo_turtle t=turtle;
    _avance(gx);
    double save_x=turtle.x,save_y=turtle.y;
    _recule(gx);
    _tourne_gauche(gtheta);
    _avance(gy);
    turtle.x=save_x;
    turtle.y=save_y;
    update_turtle_state(true);
    turtle=t;
    turtle.radius=0;
    update_turtle_state(true);
    return _polygone_rempli(-3);
  }
  const string _triangle_plein_s("triangle_plein");
  unary_function_unary __triangle_plein(&_triangle_plein,_triangle_plein_s,&printastifunction);
  unary_function_ptr at_triangle_plein (&__triangle_plein,0,T_LOGO);

  gen _dessine_tortue(const gen & g){
    // logo instruction
    /*
    _triangle_plein(makevecteur(17,5));
    _tourne_droite(90);
    _triangle_plein(makevecteur(5,17));
    return _tourne_droite(-90);
    */
    double save_x=turtle.x,save_y=turtle.y;
    _tourne_droite(90);
    _avance(5);
    _tourne_gauche(106);
    _avance(18);
    _tourne_gauche(148);
    _avance(18);
    _tourne_gauche(106);
    _avance(5);
    turtle.x=save_x; turtle.y=save_y;
    gen res=_tourne_gauche(90);
    if (is_one(g))
      return res;
    return _polygone_rempli(-9);
  }
  const string _dessine_tortue_s("dessine_tortue");
  unary_function_unary __dessine_tortue(&_dessine_tortue,_dessine_tortue_s,&printastifunction);
  unary_function_ptr at_dessine_tortue (&__dessine_tortue,0,T_LOGO);

  const string _hasard_s("hasard");
  unary_function_unary __hasard(&_rand,_hasard_s,&printastifunction);
  unary_function_ptr at_hasard (&__hasard,0,T_LOGO);

  /* A FAIRE:
     traduction latex
   */


  extern unary_function_ptr at_arc;
  gen _arc(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_arc,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    gen e=remove_at_pnt(eval(v[0],contextptr));
    gen f=remove_at_pnt(eval(v[1],contextptr));
    gen g=eval(v[2],contextptr);
    if (evalf_double(v[2],DEFAULT_EVAL_LEVEL,contextptr).type!=_DOUBLE_)
      setsizeerr();
    gen c=(e+f)/2+cst_i*(f-e)/(2*tan(g/2,contextptr));
    gen diametre=gen(makevecteur(2*c-e,e),_GROUP__VECT);
    gen res=symb_pnt(symbolic(at_cercle,gen(makevecteur(diametre,zero,g),_PNT__VECT)),default_color(contextptr),contextptr);
    gen h=abs_norm(c-e,contextptr);
    if (s==3)
      return res;
    vecteur w(1,res);
    w.push_back(eval(symb_sto(_point(c,contextptr),v[3]),contextptr));
    if (s>4)
      eval(symb_sto(h,v[4]),contextptr);
    return gen(w,_GROUP__VECT);
   }
  const string _arc_s("arc");
  unary_function_eval __arc(&giac::_arc,_arc_s);
  unary_function_ptr at_arc (&__arc,_QUOTE_ARGUMENTS,true);

  typedef gen (* propriete)(const gen & g,GIAC_CONTEXT);
  gen _est(const gen & args,const propriete & f,GIAC_CONTEXT){
    if (args.type!=_SYMB)
      return zero;
    gen g=args._SYMBptr->feuille;
    if (g.type==_VECT){
      vecteur v=*g._VECTptr;
      if (!v.empty()){
	g=v.front();
	if (g.type==_VECT){
	  v=*g._VECTptr;
	  if (!v.empty() && v.front()==v.back()){
	    v.pop_back();
	    return f(gen(v,_SEQ__VECT),contextptr);
	  }
	}
      }
    }
    return zero;
  }

  int est_rect(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    gen e(a-b+c-d);
    gen f(im((d-a)*conj(cst_i*(b-a),contextptr),contextptr));
    if (is_zero(normal(e)) && is_zero(normal(f))) {
      gen g((a-b)*conj(a-b,contextptr));
      gen gg((a-d)*conj(a-d,contextptr));
      if (is_zero(normal(g-gg))) 
	return 2;
      return 1;
    }
    return 0;
  }

  int est_losange(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    gen e(a-b+c-d);
    gen f(im((d-b)*conj(cst_i*(c-a),contextptr),contextptr));
    if (is_zero(normal(e)) && is_zero(normal(f))) {
      gen g(cst_i*a-a-cst_i*b+d);
      gen gg(-cst_i*a-a+cst_i*b+d);
      if (is_zero(normal(g)) || is_zero(normal(g))) 
	return 2;
      return 1;
    }
    return 0;
  }

  int est_parallelogramme(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    gen e(a-b+c-d);
    if (is_zero(normal(e))) {
      gen f(cst_i*a-a-cst_i*b+d);
      gen ff(-cst_i*a-a+cst_i*b+d);
      if (is_zero(normal(f)) || is_zero(normal(ff))) return 4;
      gen g(im((d-b)*conj(cst_i*(c-a),contextptr),contextptr));
      if (is_zero(normal(g))) return 2;
      gen h(im((d-a)*conj(cst_i*(b-a),contextptr),contextptr));
      if (is_zero(normal(h))) return 3;
      return 1;
    }
    return 0;
  }

  bool est_carre(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    gen e(a-b+c-d);
    gen f(cst_i*a-a-cst_i*b+d);
    gen g(-cst_i*a-a+cst_i*b+d);
    return is_zero(normal(e)) && (is_zero(normal(f)) || is_zero(normal(g)));
  }

  int est_isocele(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen d((b-a)*conj(b-a,contextptr));
    gen e((c-a)*conj(c-a,contextptr));
    gen f((b-c)*conj(b-c,contextptr));
    if (is_zero(normal(d-e)))
      return 1;
    if (is_zero(normal(e-f)))
      return 3;
    if (is_zero(normal(f-d)))
      return 2;
    if (is_zero(normal(f-d)) && is_zero(normal(e-f)))
      return 4;
    return 0;
  }

  bool est_equilateral(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen d((b-a)*conj(b-a,contextptr));
    gen f((c-a)*conj(c-a,contextptr));
    gen e((b-c)*conj(b-c,contextptr));
    return is_zero(normal(d-f)) && is_zero(normal(e-f));
  }

  int est_trianglerect(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen d(im((c-a)*conj(cst_i*(b-a),contextptr),contextptr));
    gen e(im((a-c)*conj(cst_i*(b-c),contextptr),contextptr));
    gen f(im((a-b)*conj(cst_i*(c-b),contextptr),contextptr));
    if (is_zero(normal(d)))
      return 1;
    if (is_zero(normal(e)))
      return 3;
    if (is_zero(normal(f)))
      return 2;
    return 0;
  }
  
  gen _est_isocele(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_isocele,contextptr);
    vecteur v=sommet(args);
    if (v.size()==3){
      return est_isocele(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),contextptr);
    }
    return symbolic(at_est_isocele,args);
  }
  const string _est_isocele_s("est_isocele");
  unary_function_eval __est_isocele(&giac::_est_isocele,_est_isocele_s);
  unary_function_ptr at_est_isocele (&__est_isocele,0,true);
  
  gen _est_equilateral(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_equilateral,contextptr);
    vecteur v=sommet(args);
    if (v.size()==3){
      return est_equilateral(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),contextptr);
    }
    return symbolic(at_est_equilateral,args);
  }
  const string _est_equilateral_s("est_equilateral");
  unary_function_eval __est_equilateral(&giac::_est_equilateral,_est_equilateral_s);
  unary_function_ptr at_est_equilateral (&__est_equilateral,0,true);
  
  gen _est_parallelogramme(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_parallelogramme,contextptr);
    vecteur v=sommet(args);
    if (v.size()==4){
      return est_parallelogramme(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),remove_at_pnt(v[3]),contextptr);
    }
    return symbolic(at_est_parallelogramme,args);
  }
  const string _est_parallelogramme_s("est_parallelogramme");
  unary_function_eval __est_parallelogramme(&giac::_est_parallelogramme,_est_parallelogramme_s);
  unary_function_ptr at_est_parallelogramme (&__est_parallelogramme,0,true);

  gen _est_carre(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_carre,contextptr);
    vecteur v=sommet(args);
    if (v.size()==4){
      return est_carre(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),remove_at_pnt(v[3]),contextptr);
    }
    return symbolic(at_est_carre,args);
  }
  const string _est_carre_s("est_carre");
  unary_function_eval __est_carre(&giac::_est_carre,_est_carre_s);
  unary_function_ptr at_est_carre (&__est_carre,0,true);

  gen _est_losange(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_losange,contextptr);
    vecteur v=sommet(args);
    if (v.size()==4){
      return est_losange(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),remove_at_pnt(v[3]),contextptr);
    }
    return symbolic(at_est_losange,args);
  }
  const string _est_losange_s("est_losange");
  unary_function_eval __est_losange(&giac::_est_losange,_est_losange_s);
  unary_function_ptr at_est_losange (&__est_losange,0,true);

  gen _est_rectangle(const gen & args,GIAC_CONTEXT){
    if (args.is_symb_of_sommet(at_pnt))
      return _est(args,_est_rectangle,contextptr);
    vecteur v=sommet(args);
    if (v.size()==3){
	return est_trianglerect(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),contextptr);
    }
    if (v.size()==4){
      return est_rect(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),remove_at_pnt(v[3]),contextptr);
    }
    return symbolic(at_est_rectangle,args);
  }
  const string _est_rectangle_s("est_rectangle");
  unary_function_eval __est_rectangle(&giac::_est_rectangle,_est_rectangle_s);
  unary_function_ptr at_est_rectangle (&__est_rectangle,0,true);

  //=(c-a)*(d-b)/((c-b)*(d-a))= birapport de 4 complexes ou points a,b,c,d
  gen _birapport(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==4)){
      vecteur v(*args._VECTptr);
      gen a(remove_at_pnt(v[0])),b(remove_at_pnt(v[1])),c(remove_at_pnt(v[2])),d(remove_at_pnt(v[3]));
      gen res =normal((c-a)*(d-b)/((c-b)*(d-a)));
      return normal(res);
    }
    return symbolic(at_birapport,args);
  }
  const string _birapport_s("birapport");
  unary_function_eval __birapport(&giac::_birapport,_birapport_s);
  unary_function_ptr at_birapport (&__birapport,0,true);
  //puissance d'1 point A/ cercle C=d2-R2 (d=distance de A au centre(C))
  //parametre C,point ou para du cercle et point
  gen _puissance(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT) {
      vecteur v(*args._VECTptr);
      gen a,C;
      if (args._VECTptr->size()==3){
	C=_cercle(makevecteur(v[0],v[1]),contextptr);
	a=remove_at_pnt(v[2]);
      }
      else {
	if (args._VECTptr->size()==2){
	  C=v[0];
	  a=remove_at_pnt(v[1]);
	}
	else
	  setsizeerr();
      }
      gen c,R;
      if (!centre_rayon(C,c,R,false,contextptr))
	setsizeerr();
      gen res =ratnormal((c-a)*conj(c-a,contextptr)-R*conj(R,contextptr));
      return normal(res);
    }
    return symbolic(at_puissance,args);
    //setsizeerr();
  }
  const string _puissance_s("puissance");
  unary_function_eval __puissance(&giac::_puissance,_puissance_s);
  unary_function_ptr at_puissance (&__puissance,0,true);

  //axe radical de 2 cercles
  //parametres 2 cercles ou 4 parametres=param des 2 cercles
  gen _axe_radical(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT) {
      vecteur v(*args._VECTptr);
      gen C1,C2;
      if (args._VECTptr->size()==4){
	C1=_cercle(makevecteur(v[0],v[1]),contextptr);
	C2=_cercle(makevecteur(v[2],v[3]),contextptr);
      }
      else {
	if (args._VECTptr->size()==2){
	  C1=v[0];
	  C2=v[1];
	}
	else
	  setsizeerr();
      }
      gen c1,c2,R1,R2;
      if (!centre_rayon(C1,c1,R1,false,contextptr) ||
	  !centre_rayon(C2,c2,R2,false,contextptr) )
	setsizeerr();
      if (is_zero(c1-c2))
	setsizeerr("Circle centers are identical");
      gen k =ratnormal((R1*conj(R1,contextptr)-R2*conj(R2,contextptr))/((c1-c2)*conj(c1-c2,contextptr)));
      gen H=ratnormal((c1+c2+k*(c2-c1))/2);
      gen K=ratnormal(H+cst_i*(c2-c1));
      return _droite(makevecteur(normal(H),normal(K)),contextptr);
    }
    return symbolic(at_axe_radical,args);
    //setsizeerr();
  }
  const string _axe_radical_s("axe_radical");
  unary_function_eval __axe_radical(&giac::_axe_radical,_axe_radical_s);
  unary_function_ptr at_axe_radical (&__axe_radical,0,true);

  //renvoie une droite=la polaire d'un point A/ cercle C
  //parametres C,A ou param du cercle et A
  gen _polaire(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT) {
      vecteur v(*args._VECTptr);
      gen C,a;
      if (args._VECTptr->size()==3){
	gen b0=v[0], b1=v[1];
	if  ((b0.type!=_VECT)&& (b1.type!=_VECT)){
	  C=_cercle(makevecteur(b0,b1),contextptr);
	  a=remove_at_pnt(v[2]);
	} else setsizeerr();}
      else {
	if (args._VECTptr->size()==2){
	  C=v[0];
	  a=remove_at_pnt(v[1]);
	}
	else
	  setsizeerr();
      }
      gen c,R,a1,a2,p1,p2;
      if (!centre_rayon(C,c,R,false,contextptr))
	setsizeerr();
      a1=re(a-c,contextptr);
      a2=im(a-c,contextptr);
      if (a1==0 && a2==0) setsizeerr();
      if (a1==0){
	p1=c+cst_i*(R*conj(R,contextptr))/a2;
	p2=1+c+cst_i*(R*conj(R,contextptr))/a2;
	return _droite(makevecteur(normal(p1),normal(p2)),contextptr);
      }
      if (a2==0){
	p1=c+(R*conj(R,contextptr))/a1;
	p2=c+cst_i+(R*conj(R,contextptr))/a1;
	return _droite(makevecteur(normal(p1),normal(p2)),contextptr);
      }
      p1=c+(R*conj(R,contextptr))/a1;
      p2=c+cst_i*(R*conj(R,contextptr))/a2;
      return _droite(makevecteur(normal(p1),normal(p2)),contextptr);
    }
    return symbolic(at_polaire,args);
    //setsizeerr();
  }

  const string _polaire_s("polaire");
  unary_function_eval __polaire(&giac::_polaire,_polaire_s);
  unary_function_ptr at_polaire (&__polaire,0,true);

  //renvoie un point= le pole d'une droite D par rapport a un cercle C
  //parametres C,D  
  gen _pole(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT) {
      vecteur v(*args._VECTptr);
      gen D,C;
      if (args._VECTptr->size()==3){
	C=_cercle(makevecteur(v[0],v[1]),contextptr);
	D=remove_at_pnt(v[2]);
      }
      else {
	if (args._VECTptr->size()==2){
	  C=v[0];
	  D=remove_at_pnt(v[1]);
	}
	else
	  setsizeerr();
      }
      gen c,R,A1,A2,a1,a2,k;
      if (!centre_rayon(C,c,R,false,contextptr))
	setsizeerr();
      A1=D[0]-c;
      A2=D[1]-c;
      a1=im(A1-A2,contextptr);
      a2=re(A2-A1,contextptr);
      k=im(A1*conj(A2,contextptr),contextptr);
      gen res =ratnormal(c+R*conj(R,contextptr)*(a1+cst_i*a2)/k);
      return symb_pnt(res,contextptr);
    }
    return  symbolic(at_pole,args);
    //setsizeerr();
  }
  const string _pole_s("pole");
  unary_function_eval __pole(&giac::_pole,_pole_s);
  unary_function_ptr at_pole (&__pole,0,true);

  //renvoie un point (resp une droite) pole (resp polaire) de D/ cercle C
  //parametre C (ou param du cercle C), droite D (resp point D)   
  gen _polaire_reciproque(const gen & args,GIAC_CONTEXT){
    gen a;
    if (args.type==_VECT && !args._VECTptr->empty()) {
      a=args._VECTptr->back();
      if (a.type==_VECT){
	const vecteur & v=*a._VECTptr;
	const_iterateur it=v.begin(),itend=v.end();
	vecteur argsbegin(*args._VECTptr),res;
	int s=argsbegin.size();
	res.reserve(s);
	for (--s;it!=itend;++it){
	  argsbegin[s]=*it;
	  res.push_back(_polaire_reciproque(argsbegin,contextptr));
	}
	return gen(res,a.subtype);
      }
      a=remove_at_pnt(a);
      if (a.type==_VECT) 
	return _pole(args,contextptr);
      return _polaire(args,contextptr);
    }
    return symbolic(at_polaire_reciproque,args);
  }  
  const string _polaire_reciproque_s("polaire_reciproque");
  unary_function_eval __polaire_reciproque(&giac::_polaire_reciproque,_polaire_reciproque_s);
  unary_function_ptr at_polaire_reciproque (&__polaire_reciproque,0,true);

  //teste si deux cercles C1 centre c1 rayon R1 et C2  centre c2 rayon R2
  //sont orthogonaux
  bool est_orthogonal(const gen & c1,const gen & R1,const gen & c2,const gen & R2,GIAC_CONTEXT){
    gen res =ratnormal(-R1*conj(R1,contextptr)-R2*conj(R2,contextptr)+abs_norm2(c1-c2,contextptr));
    return is_zero(res);
  }

  //teste si deux cercles C1 et C2 sont orthogonaux ou 
  //si 2 droites D1,D2 sont perpendiculaires
  //parametres C1, C2 (ou D1,D2 ou 2 vecteurs de 2 points),
  gen _est_orthogonal(const gen & args,GIAC_CONTEXT){
    if ( args.type==_VECT && args._VECTptr->size()==2){
      vecteur v(*args._VECTptr);
      gen a=remove_at_pnt(v[0]),b=remove_at_pnt(v[1]);
      if (a.is_symb_of_sommet(at_hyperplan)){ 
	if (b.type==_VECT && b._VECTptr->size()==3)
	  return est_parallele_vecteur(hyperplan_normal(a),*b._VECTptr);
	if (ckmatrix(b) && b._VECTptr->size()==2)
	  return est_parallele_vecteur(hyperplan_normal(a),*(b._VECTptr->back()-b._VECTptr->front())._VECTptr);
	if (b.is_symb_of_sommet(at_hyperplan))
	  return is_zero(dotvecteur(hyperplan_normal(a),hyperplan_normal(b)));
      }
      if (b.is_symb_of_sommet(at_hyperplan) && a.type==_VECT && a._VECTptr->size()==2)
	return est_parallele_vecteur(hyperplan_normal(b),*a._VECTptr);
      if ( a.type!=_VECT && b.type!=_VECT){
	//on a 2 cercles ou 2 spheres a et b
	gen c1,c2,R1,R2;
	if (!centre_rayon(a,c1,R1,false,contextptr) ||
	    !centre_rayon(b,c2,R2,false,contextptr))
	  setsizeerr();
	return est_orthogonal(c1,R1,c2,R2,contextptr);
      }
      if ( (a.type!=_VECT) || (a._VECTptr->size()!=2) || (b.type!=_VECT) || (b._VECTptr->size()!=2) )
	setsizeerr();     
      //on a 2 droites
      return est_perpendiculaire(a[0]-a[1],b[0]-b[1],contextptr);
    }
    return symbolic(at_est_orthogonal,args);
  }
  
  const string _est_orthogonal_s("est_orthogonal");
  unary_function_eval __est_orthogonal(&giac::_est_orthogonal,_est_orthogonal_s);
  unary_function_ptr at_est_orthogonal (&__est_orthogonal,0,true);
  
  const string _est_perpendiculaire_s("est_perpendiculaire");
  unary_function_eval __est_perpendiculaire(&giac::_est_orthogonal,_est_perpendiculaire_s);
  unary_function_ptr at_est_perpendiculaire (&__est_perpendiculaire);


  //teste si 2 points (resp 2 droites) sont conj /cercle C
  //teste si 2 points (resp droites) sont conjugues /2 droites D1,D2
  //teste si 2 points sont conjugues /2 points
  //est_conjugue(C,D,A) ou est_conjugue(C,A,D)(C=cercle,D=droite,A=point)
  //est_conjugue(C,B,A) (C=cercle,B,A= 2 points)
  //est_conjugue(C,D3,D4) (C=cercle, D3,D4= 2 droites)
  //est_conjugue(D1,D2,B,A) (B,A 2 points)
  //est_conjugue(D1,D2,D3,A) ou est_conjugue(D1,D2,A,D3)(D3=droite,A= point)
  //est_conjugue(D1,D2,D3,D4) (D3,D4= 2 droites)
  //est_conjugue(P1,P2,B,A) (P1,P2,B,A= 4 points)
   gen _est_conjugue(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_est_conjugue,args);
    gen a,b,c,d;
    vecteur v(*args._VECTptr);
    a=remove_at_pnt(v[0]);
    b=remove_at_pnt(v[1]);
    int s=v.size();
    if (s==3) {
      c=remove_at_pnt(v[2]);
      //par rapport a un cercle
      if ((c.type!=_VECT) && (b.type!=_VECT)) // on a 2 points
	return _est_orthogonal(makevecteur(v[0],_cercle(makevecteur(v[1],v[2]),contextptr)),contextptr);
      if (b.type==_VECT){
	gen p=_pole(makevecteur(v[0],v[1]),contextptr);
	if (c.type!=_VECT)
	  return is_zero(c-remove_at_pnt(p));
	return est_element(remove_at_pnt(p),c,contextptr);       
      }
      if ((c.type==_VECT) && (b.type!=_VECT)){
	gen p=_pole(makevecteur(v[0],c),contextptr);
	return is_zero(b-remove_at_pnt(p));
      }
      setsizeerr();
    }
      if (s==4 && a.type==_VECT && b.type==_VECT ){
	gen dd=_conj_harmonique(makevecteur(v[0],v[1],v[2]),contextptr);
	d= remove_at_pnt(v[3]);
	dd=remove_at_pnt(dd);
	if (d.type!=_VECT)
	  return est_element(d,dd,contextptr);
	return est_element(d[0],dd,contextptr) && est_element(d[1],dd,contextptr);    }
      if (s==4 && (b.type!=_VECT) && (a.type!=_VECT)){ // on a /2 points
	if ((c.type!=_VECT)&&(d.type!=_VECT))
	  return _est_harmonique(args,contextptr);
	setsizeerr();
      }
  }

  const string _est_conjugue_s("est_conjugue");
  unary_function_eval __est_conjugue(&giac::_est_conjugue,_est_conjugue_s);
  unary_function_ptr at_est_conjugue (&__est_conjugue,0,true);

  //teste si 4 points forment une division harmonique
  bool est_harmonique(const gen & a,const gen & b,const gen & c,const gen & d,GIAC_CONTEXT){
    if (est_aligne(a,b,c,contextptr) && est_aligne(a,b,d,contextptr)){
      gen e((c-a)/(c-b)+(d-a)/(d-b));
      return is_zero(normal(e));
    }else setsizeerr();
  }
  gen _est_harmonique(const gen & args,GIAC_CONTEXT){
    if ((args.type==_VECT) && (args._VECTptr->size()==4)){
      vecteur v(*args._VECTptr);
      return est_harmonique(remove_at_pnt(v[0]),remove_at_pnt(v[1]),remove_at_pnt(v[2]),remove_at_pnt(v[3]),contextptr);
    }
    return symbolic(at_est_harmonique,args);
  }
  const string _est_harmonique_s("est_harmonique");
  unary_function_eval __est_harmonique(&giac::_est_harmonique,_est_harmonique_s);
  unary_function_ptr at_est_harmonique (&__est_harmonique,0,true);

  //conj_harmonique(D1,D2,A)= la droite des conjugues de A par rapport a D1,D2
  //conj_harmonique(D1,D2,D3)=D4 (D1,D2,D3,D4)=-1 (D1,D2,D3 concourantes ou //)
  //point D=conj_harmonique(A,B,C) est le point tel que (A,B,C,D)=-1
  gen _conj_harmonique(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_conj_harmonique,args);
    //setsizeerr();
    vecteur v(*args._VECTptr);
    gen d1,d2,a,D3,p1,p2,b;;
    if (args._VECTptr->size()==3){
      bool est_point=false;
      d1=remove_at_pnt(v[0]);
      if (d1.type==_VECT){
	if (d1._VECTptr->size()!=2)
	  setsizeerr();
	d1=d1._VECTptr->back()-d1._VECTptr->front();
      }
      else
	est_point=true;
      d2=remove_at_pnt(v[1]);
      if (d2.type==_VECT){
	if (est_point || d2._VECTptr->size()!=2)
	  setsizeerr();
	d2=d2._VECTptr->front()-d2._VECTptr->back();
      }
      else {
	if (!est_point)
	  setsizeerr();
      }
      a=remove_at_pnt(v[2]);
      if (est_point){
	if (a.type==_VECT)
	  setsizeerr();
	gen e(im((a-d1)*conj(a-d2,contextptr),contextptr));
	if (! is_zero(normal(e)))
	  //les points ne sont pas alignes
	  setsizeerr();
	gen d=normal((d1*a+d2*a-2*d1*d2)/(2*a-d2-d1));
	return symb_pnt(d,contextptr);
      }
      if (est_parallele(d1,d2,contextptr)) {
	if (a.type==_VECT){
	  if (a._VECTptr->size()!=2)
	    setsizeerr("conj_harmonique");
	  gen aa=a._VECTptr->front()-a._VECTptr->back();
	  if (est_parallele(d1,aa,contextptr))
	    a=a._VECTptr->front();
	  else setsizeerr("conj_harmonique");
	}
	D3=_perpendiculaire(makevecteur(a,v[0]),contextptr);
	vecteur w1(inter(D3,v[0],contextptr));
	p1=remove_at_pnt(w1.front());
	vecteur w2(inter(D3,v[1],contextptr));
	p2=remove_at_pnt(w2.front());
	b=_conj_harmonique(makevecteur(p1,p2,a),contextptr);
	return _parallele(makevecteur(b,v[0]),contextptr);
      }
      else  {
	vecteur w1(inter(v[0],v[1],0));
	p1=remove_at_pnt(w1.front());
	if (a.type==_VECT){
	  if (a._VECTptr->size()!=2)
	    setsizeerr("conj_harmonique");
	  a=a._VECTptr->front()-a._VECTptr->back();
	  if (est_element(p1,v[2],contextptr))
	    a=p1+a; else  setsizeerr();
	}
	D3=_parallele(makevecteur(a,v[0]),contextptr);
	vecteur w2(inter(D3,v[1],0));
	p2=remove_at_pnt(w2.front());
	b=normal(2*p2-a);
	gen res=_droite(makevecteur(normal(p1),b),contextptr);
	return res;
      }
    }
    else setsizeerr();
  }
  const string _conj_harmonique_s("conj_harmonique");
  unary_function_eval __conj_harmonique(&giac::_conj_harmonique,_conj_harmonique_s);
  unary_function_ptr at_conj_harmonique (&__conj_harmonique,0,true);

  //renvoie M point qui divise AB ds un rapport k(reel ou complexe)
  //mes alg(MA=k*MB) (z*a=k*(z-b)), parametres: A,B,k
  gen _point_div(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3))
      return symbolic(at_point_div,args);
    vecteur v(*args._VECTptr);
    gen a=remove_at_pnt(eval(v[0],contextptr)),b=remove_at_pnt(eval(v[1],contextptr)),k=eval(v[2]);
    //if (! is_zero(im(k,contextptr))) setsizeerr();
    gen c;
    k=normal(1-k);
    if (is_zero(k)) setsizeerr();
      //{c=unsigned_inf;}
    c=normal((a+(k-1)*b)/k);
    return symb_pnt(c,contextptr);
  }
  const string _point_div_s("point_div");
  unary_function_eval __point_div(&giac::_point_div,_point_div_s);
  unary_function_ptr at_point_div (&__point_div,0,true);

  //=1 si 3 cercles ont meme axe radical,2 si concentriques et 0 sinon
  int est_faisceau_cercle(const gen & c1,const gen & R1,const gen & c2,const gen & R2,const gen & c3,const gen & R3,GIAC_CONTEXT){
    if (is_equal(makevecteur(c1,c2))){
      if (is_equal(makevecteur(c1,c3))) 
	return 2;
      else 
	return 0;
    } 
      if (is_equal(makevecteur(c1,c3)))
	return 0;
    //les centres sont distincts
    if (!est_aligne(c1,c2,c3,contextptr)) return 0;
    //les centres sont alignes
    gen v=_axe_radical(makevecteur(_cercle(makevecteur(c1,R1),contextptr),_cercle(makevecteur(c2,R2),contextptr)),contextptr);
    gen w=_axe_radical(makevecteur(_cercle(makevecteur(c1,R1),contextptr),_cercle(makevecteur(c3,R3),contextptr)),contextptr);
    v=remove_at_pnt(v);
    return est_element(v[0],w,contextptr) && est_element(v[1],w,contextptr);
  }

  //renvoie 3 si tous les cercles sont confondus
  //renvoie 2 si concentriques 
  //renvoie 1 si les cercles ont meme axe radical et 0 sinon
  gen _est_faisceau_cercle(const gen & args,GIAC_CONTEXT){
    gen c1,R1;
    if (args.type!=_VECT){
      if (!centre_rayon(args,c1,R1,false,contextptr))
	setsizeerr();
      return 3;
    }
    int s=args._VECTptr->size();
    vecteur v(*args._VECTptr);
    if (s==1) {
      if (!centre_rayon(v[0],c1,R1,false,contextptr))
	setsizeerr();
      return 3;
    }
    if (s>=2){
      gen c2,c3,R2,R3;
      if (!centre_rayon(v[0],c1,R1,false,contextptr) ||
	  !centre_rayon(v[1],c2,R2,false,contextptr))
	setsizeerr();
      c1=remove_at_pnt(c1);
      c2=remove_at_pnt(c2);
      int res;
      int i=2;
      while (c1==c2 && R1==R2 && i<s){
	if (!centre_rayon(v[i],c2,R2,false,contextptr))
	  setsizeerr();
	c2=remove_at_pnt(c2);
	i=i+1;
      }
      if (i==s){
	if (c1==c2 && R1==R2) return 3;
	if (c1==c2) return 2;
	return 1;
      }
      for (int j=i;j<s;j++){
	if (!centre_rayon(v[j],c3,R3,false,contextptr))
	  setsizeerr();
	res=est_faisceau_cercle(c1,R1,c2,R2,remove_at_pnt(c3),R3,contextptr);
	if (res==0) return 0; 
      }
      return res;
    }
    return symbolic(at_est_faisceau_cercle,args);
  }
  const string _est_faisceau_cercle_s("est_faisceau_cercle");
  unary_function_eval __est_faisceau_cercle(&giac::_est_faisceau_cercle,_est_faisceau_cercle_s);
  unary_function_ptr at_est_faisceau_cercle (&__est_faisceau_cercle,0,true);
  
  //renvoie 1 si les 2 droites sont confondues,2 si elles sont// 
  //et 0 sinon
  int est_confondu_droite(const gen & a,const gen & b,GIAC_CONTEXT){
    gen d(im((a[0]-a[1])*conj(b[0]-b[1],contextptr),contextptr));
    if (is_zero(d)) {
      if (est_element(b[0],_droite(a,contextptr),contextptr))
	return 1;
      else return 2;
    }
    return 0;
  }
  //renvoie 1 si les 3 droites sont concourantes,2 si elles sont// 
  //et 0 sinon qd a et b sont 2 droites differentes
  int est_faisceau_droite(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen d(im((a[0]-a[1])*conj(b[0]-b[1],contextptr),contextptr));
    gen e(im((a[0]-a[1])*conj(c[0]-c[1],contextptr),contextptr));
    if (is_zero(d)) {
      if (is_zero(e))
	return 2;
      else return 0;
    }
    if (is_zero(e))
      return 0;
    //les 3 droites ne sont pas paralleles
    gen v=inter(_droite(a,contextptr),_droite(b,contextptr),0);
    //gen w =inter(_droite(a,contextptr),_droite(c,contextptr),0);
    if (est_element(v[0],_droite(c,contextptr),contextptr))
      return 1;
    else return 0;
  }

  //args contient au moins 3 droites
  //renvoie 3 si toutes les droites sont confondues
  //renvoie 2 si les droites sont //
  //renvoie 1 si les droites sont concourantes en 1 meme point
  //renvoie 0 sinon
  gen _est_faisceau_droite(const gen & args,GIAC_CONTEXT){
    gen a;
    if (args.type!=_VECT){
      a=remove_at_pnt(args);
      if ((a.type!=_VECT) || (a._VECTptr->size()!=2))
	setsizeerr();
      return 3; 
    }
     int s=args._VECTptr->size();
     vecteur v(*args._VECTptr);
     if (s==1) {
	a=remove_at_pnt(v[0]);
	if (a.type==_VECT)
	  setsizeerr();
	return 3;
      } 
     if (s>=2){
      a=remove_at_pnt(v[0]);
      gen b=remove_at_pnt(v[1]);
      if ((a.type!=_VECT) || (a._VECTptr->size()!=2) || (b.type!=_VECT) || (b._VECTptr->size()!=2))
	setsizeerr(); 
      int i=2;
      int res;
      while (est_confondu_droite(a,b,contextptr) && i<s){
	b=remove_at_pnt(v[i]);
	if ((b.type!=_VECT) || (b._VECTptr->size()!=2))
	  setsizeerr(); 
	i=i+1;
      }
      if (i==s){
	res=est_confondu_droite(a,b,contextptr);
	if (res==1) return 3;
	if (res==0) return 1;
	return 2;
      }
      for (int j=i;j<s;j++){
	gen c=remove_at_pnt(v[j]);
	if ((c.type!=_VECT) || (c._VECTptr->size()!=2)) 
	  setsizeerr();
       res=est_faisceau_droite(a,b,c,contextptr);
       if (res==0) return 0; 
      }
	return res;
      }
      return symbolic(at_est_faisceau_droite,args);
  }
  const string _est_faisceau_droite_s("est_faisceau_droite");
  unary_function_eval __est_faisceau_droite(&giac::_est_faisceau_droite,_est_faisceau_droite_s);
  unary_function_ptr at_est_faisceau_droite (&__est_faisceau_droite,0,true);

  //div_harmonique(A,B,C,D) remplit D tel que (A,B,C,D)=-1 
  //div_harmonique(D1,D2,A,D) remplit D tel que (D1,D2,A,D)=-1 
  //div_harmonique(D1,D2,D3,D) remplit D tel que (D1,D2,D3,D)=-1
  //et dessine les 4 points ou les 3 droites + le point ou les 4 droites
  gen _div_harmonique(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()<4))
      return symbolic(at_div_harmonique,args);
    vecteur v(*args._VECTptr);
    gen d;
    gen v0=eval(v[0],contextptr),v1=eval(v[1],contextptr),v2=eval(v[2],contextptr);
      d=_conj_harmonique(makevecteur(v0,v1,v2),contextptr);
      vecteur w=makevecteur(v0,v1,v2);
      w.push_back(eval(symb_sto(d,v[3]),contextptr));
      return gen(w,_GROUP__VECT);
    setsizeerr("div-harmonique)");
  }
  const string _div_harmonique_s("div_harmonique");
  unary_function_eval __div_harmonique(&giac::_div_harmonique,_div_harmonique_s);
  unary_function_ptr at_div_harmonique (&__div_harmonique,_QUOTE_ARGUMENTS,true);

    //enveloppe(y+x*tan(t)-2*sin(t),t)
    //enveloppe(y+x*tan(t)-2*sin(t),[t])
    //enveloppe(y+x*tan(t)-2*sin(t),[x,y,t])
 
  gen _enveloppe(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2))
      return symbolic(at_enveloppe,args);
    vecteur v(*args._VECTptr);
    gen equ=equal2diff(v[0]),var=v[1];
    //identificateur gen xvar=vx_var,yvar=y__IDNT_e;
    gen tvar,xvar,yvar;
    if (var.type!=_VECT){
      tvar=var;
      if (tvar.type!=_IDNT)
      settypeerr();
      xvar=vx_var;
      yvar=y__IDNT_e;;
    } else
      if (var._VECTptr->size()==1) {
	tvar=var[0];
	if (tvar.type!=_IDNT)
	  settypeerr();
	xvar=vx_var;
	yvar=y__IDNT_e;;
      } else
	if (var._VECTptr->size()==3){
	  xvar=var[0];
	  if (xvar.type!=_IDNT)
	    settypeerr();
	  yvar=var[1];
	  if (yvar.type!=_IDNT)
	    settypeerr();
	  tvar=var[2];
	  if (tvar.type!=_IDNT)
	    settypeerr();
	} else {
	  setsizeerr("enveloppe");
	}
    gen equder=derive(equ,tvar);
    gen sol;
    sol=solve(makevecteur(equ,equder),makevecteur(xvar,yvar),0,contextptr);
    if (sol.type==_VECT){
      vecteur & v = *sol._VECTptr;
      vecteur res;
      int s=v.size();
      for (int i=0;i<s;++i){
	gen tmp=v[i];
	if (tmp.type==_VECT && tmp._VECTptr->size()==2){
	  tmp=tmp._VECTptr->front()+cst_i*tmp._VECTptr->back();
	  tmp=gen(makevecteur(tmp,tvar),_SEQ__VECT);
	  tmp=_paramplot(tmp,contextptr);
	  if (tmp.type==_VECT)
	    res=mergevecteur(res,*tmp._VECTptr);
	  else
	    res.push_back(tmp);
	}
      }
      return gen(res,_SEQ__VECT);
    }
    return sol;
 }
    
  const string _enveloppe_s("enveloppe");
  unary_function_eval __enveloppe(&giac::_enveloppe,_enveloppe_s);
  unary_function_ptr at_enveloppe (&__enveloppe,0,true);

  gen _gnuplot(const gen & args,GIAC_CONTEXT){
#ifdef WITH_GNUPLOT
    vecteur v(gen2vecteur(args));
    int s=v.size();
    bool clrplot;
    int out_handle;
    FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
    for (int i=0;i<s;++i){
      if (v[i].type==_STRNG)
	fprintf(stream,"%s\n",v[i]._STRNGptr->c_str());
    }
    win9x_gnuplot(stream);
    return 1;
#else
    return zero;
#endif
  }
  const string _gnuplot_s("gnuplot");
  unary_function_eval __gnuplot(&giac::_gnuplot,_gnuplot_s);
  unary_function_ptr at_gnuplot (&__gnuplot,0,true);


  unary_function_ptr plot_sommets[]={at_pnt,at_parameter,at_cercle,at_curve,0};
  unary_function_ptr notexprint_plot_sommets[]={at_funcplot,at_paramplot,at_polarplot,at_implicitplot,at_contourplot,at_odeplot,at_interactive_odeplot,at_fieldplot,at_seqplot,at_ellipse,at_hyperbole,at_parabole,0};
  unary_function_ptr implicittex_plot_sommets[]={at_plot,at_plot3d,at_plotfunc,at_plotparam,at_plotpolar,at_plotimplicit,at_plotcontour,at_DrawInv,at_DrawFunc,at_DrawParm,at_DrawPol,at_DrwCtour,at_plotode,at_plotfield,at_interactive_plotode,at_plotseq,at_Graph,0};

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
